-- Version: 2022.1 2022.1.0.10
-- File used only for Simulation

library ieee;
use ieee.std_logic_1164.all;
library polarfire;
use polarfire.all;

entity ctu_can_fd_libero_top is

    port( clk_sys   : in    std_logic;
          res_n     : in    std_logic;
          res_n_out : out   std_logic;
          data_in   : in    std_logic_vector(31 downto 0);
          data_out  : out   std_logic_vector(31 downto 0);
          adress    : in    std_logic_vector(15 downto 0);
          scs       : in    std_logic;
          srd       : in    std_logic;
          swr       : in    std_logic;
          sbe       : in    std_logic_vector(3 downto 0);
          int_Z     : out   std_logic;
          can_tx    : out   std_logic;
          can_rx    : in    std_logic;
          timestamp : in    std_logic_vector(63 downto 0)
        );

end ctu_can_fd_libero_top;

architecture DEF_ARCH of ctu_can_fd_libero_top is 

  component CFG4A
    generic (INIT:std_logic_vector(15 downto 0) := x"0000");

    port( A : in    std_logic := 'U';
          B : in    std_logic := 'U';
          C : in    std_logic := 'U';
          D : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component SLE
    port( D   : in    std_logic := 'U';
          CLK : in    std_logic := 'U';
          EN  : in    std_logic := 'U';
          ALn : in    std_logic := 'U';
          ADn : in    std_logic := 'U';
          SLn : in    std_logic := 'U';
          SD  : in    std_logic := 'U';
          LAT : in    std_logic := 'U';
          Q   : out   std_logic
        );
  end component;

  component CFG4
    generic (INIT:std_logic_vector(15 downto 0) := x"0000");

    port( A : in    std_logic := 'U';
          B : in    std_logic := 'U';
          C : in    std_logic := 'U';
          D : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component CFG3
    generic (INIT:std_logic_vector(7 downto 0) := x"00");

    port( A : in    std_logic := 'U';
          B : in    std_logic := 'U';
          C : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component CFG2
    generic (INIT:std_logic_vector(3 downto 0) := x"0");

    port( A : in    std_logic := 'U';
          B : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component ARI1_CC
    generic (INIT:std_logic_vector(19 downto 0) := x"00000");

    port( A   : in    std_logic := 'U';
          B   : in    std_logic := 'U';
          C   : in    std_logic := 'U';
          D   : in    std_logic := 'U';
          FCI : in    std_logic := 'U';
          S   : out   std_logic;
          Y   : out   std_logic;
          FCO : out   std_logic;
          CC  : in    std_logic := 'U';
          P   : out   std_logic;
          Y3  : out   std_logic;
          Y3A : out   std_logic
        );
  end component;

  component IOIN_IB_E
    generic (TX_MODE:std_logic_vector(6 downto 0) := "000" & x"0"; 
        RX_MODE:std_logic_vector(3 downto 0) := x"0"; 
        TX_OE_MODE:std_logic_vector(2 downto 0) := "000"; 
        INPUT_DELAY_SEL:std_logic_vector(1 downto 0) := "00"; 
        DELAY_LINE_MODE:std_logic_vector(1 downto 0) := "00"; 
        RX_DELAY_VAL:std_logic_vector(6 downto 0) := "000" & x"0"; 
        RX_DELAY_VAL_X2:std_logic_vector(0 downto 0) := "0"; 
        TX_DELAY_VAL:std_logic_vector(6 downto 0) := "000" & x"0"
        );

    port( Y   : out   std_logic;
          E   : in    std_logic := 'U';
          YIN : in    std_logic := 'U'
        );
  end component;

  component INV_BA
    port( A : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component CC_CONFIG
    port( CI  : in    std_logic := 'U';
          CO  : out   std_logic;
          P   : in    std_logic_vector(0 to 11) := (others => 'U');
          Y3  : in    std_logic_vector(0 to 11) := (others => 'U');
          Y3A : in    std_logic_vector(0 to 11) := (others => 'U');
          CC  : out   std_logic_vector(0 to 11)
        );
  end component;

  component IOPAD_IN
    port( PAD : in    std_logic := 'U';
          Y   : out   std_logic
        );
  end component;

  component IOPAD_TRI
    port( PAD : out   std_logic;
          D   : in    std_logic := 'U';
          E   : in    std_logic := 'U'
        );
  end component;

  component IOTRI_OB_EB
    generic (TX_MODE:std_logic_vector(6 downto 0) := "000" & x"0"; 
        RX_MODE:std_logic_vector(3 downto 0) := x"0"; 
        TX_OE_MODE:std_logic_vector(2 downto 0) := "000"; 
        INPUT_DELAY_SEL:std_logic_vector(1 downto 0) := "00"; 
        DELAY_LINE_MODE:std_logic_vector(1 downto 0) := "00"; 
        RX_DELAY_VAL:std_logic_vector(6 downto 0) := "000" & x"0"; 
        RX_DELAY_VAL_X2:std_logic_vector(0 downto 0) := "0"; 
        TX_DELAY_VAL:std_logic_vector(6 downto 0) := "000" & x"0"
        );

    port( D    : in    std_logic := 'U';
          E    : in    std_logic := 'U';
          DOUT : out   std_logic;
          EOUT : out   std_logic
        );
  end component;

  component RGB
    port( A  : in    std_logic := 'U';
          EN : in    std_logic := 'U';
          Y  : out   std_logic
        );
  end component;

  component FCEND_BUFF_CC
    port( FCI : in    std_logic := 'U';
          CO  : out   std_logic;
          CC  : in    std_logic := 'U';
          P   : out   std_logic;
          Y3  : out   std_logic;
          Y3A : out   std_logic
        );
  end component;

  component CFG1
    generic (INIT:std_logic_vector(1 downto 0) := "00");

    port( A : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component GB
    port( A  : in    std_logic := 'U';
          EN : in    std_logic := 'U';
          Y  : out   std_logic
        );
  end component;

  component ICB_CLKINT
    port( A : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component GND
    port(Y : out std_logic); 
  end component;

  component VCC
    port(Y : out std_logic); 
  end component;

    signal \data_in_c[31]\, \data_in_c[30]\, \data_in_c[29]\, 
        \data_in_c[28]\, \data_in_c[27]\, \data_in_c[26]\, 
        \data_in_c[25]\, \data_in_c[24]\, \data_in_c[23]\, 
        \data_in_c[22]\, \data_in_c[21]\, \data_in_c[20]\, 
        \data_in_c[19]\, \data_in_c[18]\, \data_in_c[17]\, 
        \data_in_c[16]\, \data_in_c[15]\, \data_in_c[14]\, 
        \data_in_c[13]\, \data_in_c[12]\, \data_in_c[11]\, 
        \data_in_c[10]\, \data_in_c[9]\, \data_in_c[8]\, 
        \data_in_c[7]\, \data_in_c[6]\, \data_in_c[5]\, 
        \data_in_c[4]\, \data_in_c[3]\, \data_in_c[2]\, 
        \data_in_c[1]\, \data_in_c[0]\, \adress_c[11]\, 
        \adress_c[10]\, \adress_c[9]\, \adress_c[8]\, 
        \adress_c[7]\, \adress_c[6]\, \adress_c[5]\, 
        \adress_c[4]\, \adress_c[3]\, \adress_c[2]\, \sbe_c[3]\, 
        \sbe_c[2]\, \sbe_c[1]\, \sbe_c[0]\, \timestamp_c[63]\, 
        \timestamp_c[62]\, \timestamp_c[61]\, \timestamp_c[60]\, 
        \timestamp_c[59]\, \timestamp_c[58]\, \timestamp_c[57]\, 
        \timestamp_c[56]\, \timestamp_c[55]\, \timestamp_c[54]\, 
        \timestamp_c[53]\, \timestamp_c[52]\, \timestamp_c[51]\, 
        \timestamp_c[50]\, \timestamp_c[49]\, \timestamp_c[48]\, 
        \timestamp_c[47]\, \timestamp_c[46]\, \timestamp_c[45]\, 
        \timestamp_c[44]\, \timestamp_c[43]\, \timestamp_c[42]\, 
        \timestamp_c[41]\, \timestamp_c[40]\, \timestamp_c[39]\, 
        \timestamp_c[38]\, \timestamp_c[37]\, \timestamp_c[36]\, 
        \timestamp_c[35]\, \timestamp_c[34]\, \timestamp_c[33]\, 
        \timestamp_c[32]\, \timestamp_c[31]\, \timestamp_c[30]\, 
        \timestamp_c[29]\, \timestamp_c[28]\, \timestamp_c[27]\, 
        \timestamp_c[26]\, \timestamp_c[25]\, \timestamp_c[24]\, 
        \timestamp_c[23]\, \timestamp_c[22]\, \timestamp_c[21]\, 
        \timestamp_c[20]\, \timestamp_c[19]\, \timestamp_c[18]\, 
        \timestamp_c[17]\, \timestamp_c[16]\, \timestamp_c[15]\, 
        \timestamp_c[14]\, \timestamp_c[13]\, \timestamp_c[12]\, 
        \timestamp_c[11]\, \timestamp_c[10]\, \timestamp_c[9]\, 
        \timestamp_c[8]\, \timestamp_c[7]\, \timestamp_c[6]\, 
        \timestamp_c[5]\, \timestamp_c[4]\, \timestamp_c[3]\, 
        \timestamp_c[2]\, \timestamp_c[1]\, \timestamp_c[0]\, 
        \data_out_c[31]\, \data_out_c[30]\, \data_out_c[29]\, 
        \data_out_c[28]\, \data_out_c[27]\, \data_out_c[26]\, 
        \data_out_c[25]\, \data_out_c[24]\, \data_out_c[23]\, 
        \data_out_c[22]\, \data_out_c[21]\, \data_out_c[20]\, 
        \data_out_c[19]\, \data_out_c[18]\, \data_out_c[17]\, 
        \data_out_c[16]\, \data_out_c[15]\, \data_out_c[14]\, 
        \data_out_c[13]\, \data_out_c[12]\, \data_out_c[11]\, 
        \data_out_c[10]\, \data_out_c[9]\, \data_out_c[8]\, 
        \data_out_c[7]\, \data_out_c[6]\, \data_out_c[5]\, 
        \data_out_c[4]\, \data_out_c[3]\, \data_out_c[2]\, 
        \data_out_c[1]\, \data_out_c[0]\, \I_1/U0_Y\, 
        clk_sys_ibuf_Z, res_n_c, scs_c, srd_c, swr_c, can_rx_c, 
        res_n_out_c, int_c, can_tx_c, 
        \can_top_level_inst/stat_bus[385]\, 
        \can_top_level_inst/stat_bus[384]\, 
        \can_top_level_inst/stat_bus[381]\, 
        \can_top_level_inst/stat_bus[380]\, 
        \can_top_level_inst/stat_bus[379]\, 
        \can_top_level_inst/stat_bus[378]\, 
        \can_top_level_inst/stat_bus[377]\, 
        \can_top_level_inst/stat_bus[376]\, 
        \can_top_level_inst/stat_bus[375]\, 
        \can_top_level_inst/stat_bus[374]\, 
        \can_top_level_inst/stat_bus[373]\, 
        \can_top_level_inst/stat_bus[372]\, 
        \can_top_level_inst/stat_bus[371]\, 
        \can_top_level_inst/stat_bus[370]\, 
        \can_top_level_inst/stat_bus[305]\, 
        \can_top_level_inst/stat_bus[304]\, 
        \can_top_level_inst/stat_bus[303]\, 
        \can_top_level_inst/stat_bus[302]\, 
        \can_top_level_inst/stat_bus[301]\, 
        \can_top_level_inst/stat_bus[300]\, 
        \can_top_level_inst/stat_bus[296]\, 
        \can_top_level_inst/stat_bus[295]\, 
        \can_top_level_inst/stat_bus[294]\, 
        \can_top_level_inst/stat_bus[293]\, 
        \can_top_level_inst/stat_bus[292]\, 
        \can_top_level_inst/stat_bus[291]\, 
        \can_top_level_inst/stat_bus[290]\, 
        \can_top_level_inst/stat_bus[289]\, 
        \can_top_level_inst/stat_bus[288]\, 
        \can_top_level_inst/stat_bus[287]\, 
        \can_top_level_inst/stat_bus[286]\, 
        \can_top_level_inst/stat_bus[285]\, 
        \can_top_level_inst/stat_bus[284]\, 
        \can_top_level_inst/stat_bus[283]\, 
        \can_top_level_inst/stat_bus[282]\, 
        \can_top_level_inst/stat_bus[281]\, 
        \can_top_level_inst/stat_bus[280]\, 
        \can_top_level_inst/stat_bus[279]\, 
        \can_top_level_inst/stat_bus[278]\, 
        \can_top_level_inst/stat_bus[277]\, 
        \can_top_level_inst/stat_bus[276]\, 
        \can_top_level_inst/stat_bus[275]\, 
        \can_top_level_inst/stat_bus[274]\, 
        \can_top_level_inst/stat_bus[273]\, 
        \can_top_level_inst/stat_bus[272]\, 
        \can_top_level_inst/stat_bus[271]\, 
        \can_top_level_inst/stat_bus[270]\, 
        \can_top_level_inst/stat_bus[269]\, 
        \can_top_level_inst/stat_bus[268]\, 
        \can_top_level_inst/stat_bus[267]\, 
        \can_top_level_inst/stat_bus[266]\, 
        \can_top_level_inst/stat_bus[265]\, 
        \can_top_level_inst/stat_bus[264]\, 
        \can_top_level_inst/stat_bus[263]\, 
        \can_top_level_inst/stat_bus[262]\, 
        \can_top_level_inst/stat_bus[261]\, 
        \can_top_level_inst/stat_bus[260]\, 
        \can_top_level_inst/stat_bus[259]\, 
        \can_top_level_inst/stat_bus[258]\, 
        \can_top_level_inst/stat_bus[257]\, 
        \can_top_level_inst/stat_bus[253]\, 
        \can_top_level_inst/stat_bus[251]\, 
        \can_top_level_inst/stat_bus[250]\, 
        \can_top_level_inst/stat_bus[249]\, 
        \can_top_level_inst/stat_bus[248]\, 
        \can_top_level_inst/stat_bus[247]\, 
        \can_top_level_inst/stat_bus[246]\, 
        \can_top_level_inst/stat_bus[245]\, 
        \can_top_level_inst/stat_bus[244]\, 
        \can_top_level_inst/stat_bus[243]\, 
        \can_top_level_inst/stat_bus[242]\, 
        \can_top_level_inst/stat_bus[241]\, 
        \can_top_level_inst/stat_bus[240]\, 
        \can_top_level_inst/stat_bus[239]\, 
        \can_top_level_inst/stat_bus[238]\, 
        \can_top_level_inst/stat_bus[237]\, 
        \can_top_level_inst/stat_bus[236]\, 
        \can_top_level_inst/stat_bus[235]\, 
        \can_top_level_inst/stat_bus[234]\, 
        \can_top_level_inst/stat_bus[233]\, 
        \can_top_level_inst/stat_bus[232]\, 
        \can_top_level_inst/stat_bus[231]\, 
        \can_top_level_inst/stat_bus[230]\, 
        \can_top_level_inst/stat_bus[229]\, 
        \can_top_level_inst/stat_bus[228]\, 
        \can_top_level_inst/stat_bus[227]\, 
        \can_top_level_inst/stat_bus[226]\, 
        \can_top_level_inst/stat_bus[225]\, 
        \can_top_level_inst/stat_bus[224]\, 
        \can_top_level_inst/stat_bus[223]\, 
        \can_top_level_inst/stat_bus[222]\, 
        \can_top_level_inst/stat_bus[221]\, 
        \can_top_level_inst/stat_bus[220]\, 
        \can_top_level_inst/stat_bus[219]\, 
        \can_top_level_inst/stat_bus[218]\, 
        \can_top_level_inst/stat_bus[217]\, 
        \can_top_level_inst/stat_bus[216]\, 
        \can_top_level_inst/stat_bus[215]\, 
        \can_top_level_inst/stat_bus[214]\, 
        \can_top_level_inst/stat_bus[213]\, 
        \can_top_level_inst/stat_bus[212]\, 
        \can_top_level_inst/stat_bus[211]\, 
        \can_top_level_inst/stat_bus[210]\, 
        \can_top_level_inst/stat_bus[209]\, 
        \can_top_level_inst/stat_bus[208]\, 
        \can_top_level_inst/stat_bus[207]\, 
        \can_top_level_inst/stat_bus[206]\, 
        \can_top_level_inst/stat_bus[205]\, 
        \can_top_level_inst/stat_bus[204]\, 
        \can_top_level_inst/stat_bus[203]\, 
        \can_top_level_inst/stat_bus[202]\, 
        \can_top_level_inst/stat_bus[201]\, 
        \can_top_level_inst/stat_bus[200]\, 
        \can_top_level_inst/stat_bus[199]\, 
        \can_top_level_inst/stat_bus[198]\, 
        \can_top_level_inst/stat_bus[197]\, 
        \can_top_level_inst/stat_bus[196]\, 
        \can_top_level_inst/stat_bus[195]\, 
        \can_top_level_inst/stat_bus[194]\, 
        \can_top_level_inst/stat_bus[193]\, 
        \can_top_level_inst/stat_bus[192]\, 
        \can_top_level_inst/stat_bus[191]\, 
        \can_top_level_inst/stat_bus[190]\, 
        \can_top_level_inst/stat_bus[189]\, 
        \can_top_level_inst/stat_bus[188]\, 
        \can_top_level_inst/stat_bus[184]\, 
        \can_top_level_inst/stat_bus[156]\, 
        \can_top_level_inst/stat_bus[155]\, 
        \can_top_level_inst/stat_bus[109]\, 
        \can_top_level_inst/stat_bus[108]\, 
        \can_top_level_inst/stat_bus[107]\, 
        \can_top_level_inst/stat_bus[106]\, 
        \can_top_level_inst/stat_bus[105]\, 
        \can_top_level_inst/stat_bus[104]\, 
        \can_top_level_inst/stat_bus[103]\, 
        \can_top_level_inst/stat_bus[102]\, 
        \can_top_level_inst/stat_bus[101]\, 
        \can_top_level_inst/stat_bus[100]\, 
        \can_top_level_inst/stat_bus[89]\, 
        \can_top_level_inst/stat_bus[88]\, 
        \can_top_level_inst/stat_bus[87]\, 
        \can_top_level_inst/stat_bus[86]\, 
        \can_top_level_inst/stat_bus[85]\, 
        \can_top_level_inst/stat_bus[84]\, 
        \can_top_level_inst/stat_bus[83]\, 
        \can_top_level_inst/stat_bus[82]\, 
        \can_top_level_inst/stat_bus[81]\, 
        \can_top_level_inst/stat_bus[79]\, 
        \can_top_level_inst/stat_bus[78]\, 
        \can_top_level_inst/stat_bus[77]\, 
        \can_top_level_inst/stat_bus[76]\, 
        \can_top_level_inst/stat_bus[75]\, 
        \can_top_level_inst/stat_bus[74]\, 
        \can_top_level_inst/stat_bus[73]\, 
        \can_top_level_inst/stat_bus[72]\, 
        \can_top_level_inst/stat_bus[71]\, 
        \can_top_level_inst/stat_bus[68]\, 
        \can_top_level_inst/stat_bus[67]\, 
        \can_top_level_inst/stat_bus[66]\, 
        \can_top_level_inst/stat_bus[15]\, 
        \can_top_level_inst/stat_bus[14]\, 
        \can_top_level_inst/stat_bus[13]\, 
        \can_top_level_inst/stat_bus[11]\, 
        \can_top_level_inst/stat_bus[10]\, 
        \can_top_level_inst/stat_bus[9]\, 
        \can_top_level_inst/stat_bus[6]\, 
        \can_top_level_inst/stat_bus[5]\, 
        \can_top_level_inst/stat_bus[4]\, 
        \can_top_level_inst/stat_bus[3]\, 
        \can_top_level_inst/stat_bus[2]\, 
        \can_top_level_inst/stat_bus[1]\, 
        \can_top_level_inst/stat_bus[0]\, 
        \can_top_level_inst/drv_bus[513]\, 
        \can_top_level_inst/drv_bus[512]\, 
        \can_top_level_inst/drv_bus[511]\, 
        \can_top_level_inst/drv_bus[510]\, 
        \can_top_level_inst/drv_bus[508]\, 
        \can_top_level_inst/drv_bus[507]\, 
        \can_top_level_inst/drv_bus[477]\, 
        \can_top_level_inst/drv_bus[476]\, 
        \can_top_level_inst/drv_bus[475]\, 
        \can_top_level_inst/drv_bus[474]\, 
        \can_top_level_inst/drv_bus[473]\, 
        \can_top_level_inst/drv_bus[472]\, 
        \can_top_level_inst/drv_bus[471]\, 
        \can_top_level_inst/drv_bus[470]\, 
        \can_top_level_inst/drv_bus[469]\, 
        \can_top_level_inst/drv_bus[468]\, 
        \can_top_level_inst/drv_bus[467]\, 
        \can_top_level_inst/drv_bus[466]\, 
        \can_top_level_inst/drv_bus[465]\, 
        \can_top_level_inst/drv_bus[460]\, 
        \can_top_level_inst/drv_bus[429]\, 
        \can_top_level_inst/drv_bus[426]\, 
        \can_top_level_inst/drv_bus[425]\, 
        \can_top_level_inst/drv_bus[424]\, 
        \can_top_level_inst/drv_bus[423]\, 
        \can_top_level_inst/drv_bus[422]\, 
        \can_top_level_inst/drv_bus[421]\, 
        \can_top_level_inst/drv_bus[420]\, 
        \can_top_level_inst/drv_bus[419]\, 
        \can_top_level_inst/drv_bus[418]\, 
        \can_top_level_inst/drv_bus[417]\, 
        \can_top_level_inst/drv_bus[416]\, 
        \can_top_level_inst/drv_bus[415]\, 
        \can_top_level_inst/drv_bus[414]\, 
        \can_top_level_inst/drv_bus[413]\, 
        \can_top_level_inst/drv_bus[412]\, 
        \can_top_level_inst/drv_bus[411]\, 
        \can_top_level_inst/drv_bus[410]\, 
        \can_top_level_inst/drv_bus[409]\, 
        \can_top_level_inst/drv_bus[408]\, 
        \can_top_level_inst/drv_bus[407]\, 
        \can_top_level_inst/drv_bus[406]\, 
        \can_top_level_inst/drv_bus[405]\, 
        \can_top_level_inst/drv_bus[404]\, 
        \can_top_level_inst/drv_bus[403]\, 
        \can_top_level_inst/drv_bus[402]\, 
        \can_top_level_inst/drv_bus[401]\, 
        \can_top_level_inst/drv_bus[400]\, 
        \can_top_level_inst/drv_bus[382]\, 
        \can_top_level_inst/drv_bus[381]\, 
        \can_top_level_inst/drv_bus[380]\, 
        \can_top_level_inst/drv_bus[379]\, 
        \can_top_level_inst/drv_bus[378]\, 
        \can_top_level_inst/drv_bus[377]\, 
        \can_top_level_inst/drv_bus[376]\, 
        \can_top_level_inst/drv_bus[375]\, 
        \can_top_level_inst/drv_bus[374]\, 
        \can_top_level_inst/drv_bus[352]\, 
        \can_top_level_inst/drv_bus[351]\, 
        \can_top_level_inst/drv_bus[330]\, 
        \can_top_level_inst/drv_bus[329]\, 
        \can_top_level_inst/drv_bus[328]\, 
        \can_top_level_inst/drv_bus[327]\, 
        \can_top_level_inst/drv_bus[326]\, 
        \can_top_level_inst/drv_bus[325]\, 
        \can_top_level_inst/drv_bus[324]\, 
        \can_top_level_inst/drv_bus[323]\, 
        \can_top_level_inst/drv_bus[322]\, 
        \can_top_level_inst/drv_bus[321]\, 
        \can_top_level_inst/drv_bus[320]\, 
        \can_top_level_inst/drv_bus[319]\, 
        \can_top_level_inst/drv_bus[318]\, 
        \can_top_level_inst/drv_bus[317]\, 
        \can_top_level_inst/drv_bus[316]\, 
        \can_top_level_inst/drv_bus[315]\, 
        \can_top_level_inst/drv_bus[314]\, 
        \can_top_level_inst/drv_bus[313]\, 
        \can_top_level_inst/drv_bus[312]\, 
        \can_top_level_inst/drv_bus[311]\, 
        \can_top_level_inst/drv_bus[310]\, 
        \can_top_level_inst/drv_bus[309]\, 
        \can_top_level_inst/drv_bus[308]\, 
        \can_top_level_inst/drv_bus[307]\, 
        \can_top_level_inst/drv_bus[306]\, 
        \can_top_level_inst/drv_bus[305]\, 
        \can_top_level_inst/drv_bus[304]\, 
        \can_top_level_inst/drv_bus[303]\, 
        \can_top_level_inst/drv_bus[302]\, 
        \can_top_level_inst/drv_bus[301]\, 
        \can_top_level_inst/drv_bus[300]\, 
        \can_top_level_inst/drv_bus[299]\, 
        \can_top_level_inst/drv_bus[298]\, 
        \can_top_level_inst/drv_bus[297]\, 
        \can_top_level_inst/drv_bus[296]\, 
        \can_top_level_inst/drv_bus[295]\, 
        \can_top_level_inst/drv_bus[294]\, 
        \can_top_level_inst/drv_bus[293]\, 
        \can_top_level_inst/drv_bus[292]\, 
        \can_top_level_inst/drv_bus[291]\, 
        \can_top_level_inst/drv_bus[290]\, 
        \can_top_level_inst/drv_bus[289]\, 
        \can_top_level_inst/drv_bus[288]\, 
        \can_top_level_inst/drv_bus[287]\, 
        \can_top_level_inst/drv_bus[286]\, 
        \can_top_level_inst/drv_bus[285]\, 
        \can_top_level_inst/drv_bus[284]\, 
        \can_top_level_inst/drv_bus[283]\, 
        \can_top_level_inst/drv_bus[282]\, 
        \can_top_level_inst/drv_bus[281]\, 
        \can_top_level_inst/drv_bus[280]\, 
        \can_top_level_inst/drv_bus[279]\, 
        \can_top_level_inst/drv_bus[278]\, 
        \can_top_level_inst/drv_bus[277]\, 
        \can_top_level_inst/drv_bus[276]\, 
        \can_top_level_inst/drv_bus[275]\, 
        \can_top_level_inst/drv_bus[274]\, 
        \can_top_level_inst/drv_bus[273]\, 
        \can_top_level_inst/drv_bus[272]\, 
        \can_top_level_inst/drv_bus[271]\, 
        \can_top_level_inst/drv_bus[270]\, 
        \can_top_level_inst/drv_bus[269]\, 
        \can_top_level_inst/drv_bus[268]\, 
        \can_top_level_inst/drv_bus[267]\, 
        \can_top_level_inst/drv_bus[266]\, 
        \can_top_level_inst/drv_bus[265]\, 
        \can_top_level_inst/drv_bus[264]\, 
        \can_top_level_inst/drv_bus[263]\, 
        \can_top_level_inst/drv_bus[262]\, 
        \can_top_level_inst/drv_bus[261]\, 
        \can_top_level_inst/drv_bus[260]\, 
        \can_top_level_inst/drv_bus[259]\, 
        \can_top_level_inst/drv_bus[258]\, 
        \can_top_level_inst/drv_bus[257]\, 
        \can_top_level_inst/drv_bus[256]\, 
        \can_top_level_inst/drv_bus[255]\, 
        \can_top_level_inst/drv_bus[254]\, 
        \can_top_level_inst/drv_bus[253]\, 
        \can_top_level_inst/drv_bus[252]\, 
        \can_top_level_inst/drv_bus[251]\, 
        \can_top_level_inst/drv_bus[250]\, 
        \can_top_level_inst/drv_bus[249]\, 
        \can_top_level_inst/drv_bus[248]\, 
        \can_top_level_inst/drv_bus[247]\, 
        \can_top_level_inst/drv_bus[246]\, 
        \can_top_level_inst/drv_bus[245]\, 
        \can_top_level_inst/drv_bus[244]\, 
        \can_top_level_inst/drv_bus[243]\, 
        \can_top_level_inst/drv_bus[242]\, 
        \can_top_level_inst/drv_bus[241]\, 
        \can_top_level_inst/drv_bus[240]\, 
        \can_top_level_inst/drv_bus[239]\, 
        \can_top_level_inst/drv_bus[238]\, 
        \can_top_level_inst/drv_bus[237]\, 
        \can_top_level_inst/drv_bus[236]\, 
        \can_top_level_inst/drv_bus[235]\, 
        \can_top_level_inst/drv_bus[234]\, 
        \can_top_level_inst/drv_bus[233]\, 
        \can_top_level_inst/drv_bus[232]\, 
        \can_top_level_inst/drv_bus[231]\, 
        \can_top_level_inst/drv_bus[230]\, 
        \can_top_level_inst/drv_bus[229]\, 
        \can_top_level_inst/drv_bus[228]\, 
        \can_top_level_inst/drv_bus[227]\, 
        \can_top_level_inst/drv_bus[226]\, 
        \can_top_level_inst/drv_bus[225]\, 
        \can_top_level_inst/drv_bus[224]\, 
        \can_top_level_inst/drv_bus[223]\, 
        \can_top_level_inst/drv_bus[222]\, 
        \can_top_level_inst/drv_bus[221]\, 
        \can_top_level_inst/drv_bus[220]\, 
        \can_top_level_inst/drv_bus[219]\, 
        \can_top_level_inst/drv_bus[218]\, 
        \can_top_level_inst/drv_bus[217]\, 
        \can_top_level_inst/drv_bus[216]\, 
        \can_top_level_inst/drv_bus[215]\, 
        \can_top_level_inst/drv_bus[214]\, 
        \can_top_level_inst/drv_bus[213]\, 
        \can_top_level_inst/drv_bus[212]\, 
        \can_top_level_inst/drv_bus[211]\, 
        \can_top_level_inst/drv_bus[210]\, 
        \can_top_level_inst/drv_bus[209]\, 
        \can_top_level_inst/drv_bus[208]\, 
        \can_top_level_inst/drv_bus[207]\, 
        \can_top_level_inst/drv_bus[206]\, 
        \can_top_level_inst/drv_bus[205]\, 
        \can_top_level_inst/drv_bus[204]\, 
        \can_top_level_inst/drv_bus[203]\, 
        \can_top_level_inst/drv_bus[202]\, 
        \can_top_level_inst/drv_bus[201]\, 
        \can_top_level_inst/drv_bus[200]\, 
        \can_top_level_inst/drv_bus[199]\, 
        \can_top_level_inst/drv_bus[198]\, 
        \can_top_level_inst/drv_bus[197]\, 
        \can_top_level_inst/drv_bus[196]\, 
        \can_top_level_inst/drv_bus[195]\, 
        \can_top_level_inst/drv_bus[194]\, 
        \can_top_level_inst/drv_bus[193]\, 
        \can_top_level_inst/drv_bus[192]\, 
        \can_top_level_inst/drv_bus[191]\, 
        \can_top_level_inst/drv_bus[190]\, 
        \can_top_level_inst/drv_bus[189]\, 
        \can_top_level_inst/drv_bus[188]\, 
        \can_top_level_inst/drv_bus[187]\, 
        \can_top_level_inst/drv_bus[186]\, 
        \can_top_level_inst/drv_bus[185]\, 
        \can_top_level_inst/drv_bus[184]\, 
        \can_top_level_inst/drv_bus[183]\, 
        \can_top_level_inst/drv_bus[182]\, 
        \can_top_level_inst/drv_bus[181]\, 
        \can_top_level_inst/drv_bus[180]\, 
        \can_top_level_inst/drv_bus[179]\, 
        \can_top_level_inst/drv_bus[178]\, 
        \can_top_level_inst/drv_bus[177]\, 
        \can_top_level_inst/drv_bus[176]\, 
        \can_top_level_inst/drv_bus[175]\, 
        \can_top_level_inst/drv_bus[174]\, 
        \can_top_level_inst/drv_bus[173]\, 
        \can_top_level_inst/drv_bus[172]\, 
        \can_top_level_inst/drv_bus[171]\, 
        \can_top_level_inst/drv_bus[170]\, 
        \can_top_level_inst/drv_bus[169]\, 
        \can_top_level_inst/drv_bus[168]\, 
        \can_top_level_inst/drv_bus[167]\, 
        \can_top_level_inst/drv_bus[166]\, 
        \can_top_level_inst/drv_bus[165]\, 
        \can_top_level_inst/drv_bus[164]\, 
        \can_top_level_inst/drv_bus[163]\, 
        \can_top_level_inst/drv_bus[162]\, 
        \can_top_level_inst/drv_bus[161]\, 
        \can_top_level_inst/drv_bus[160]\, 
        \can_top_level_inst/drv_bus[159]\, 
        \can_top_level_inst/drv_bus[158]\, 
        \can_top_level_inst/drv_bus[157]\, 
        \can_top_level_inst/drv_bus[156]\, 
        \can_top_level_inst/drv_bus[155]\, 
        \can_top_level_inst/drv_bus[154]\, 
        \can_top_level_inst/drv_bus[153]\, 
        \can_top_level_inst/drv_bus[152]\, 
        \can_top_level_inst/drv_bus[151]\, 
        \can_top_level_inst/drv_bus[150]\, 
        \can_top_level_inst/drv_bus[149]\, 
        \can_top_level_inst/drv_bus[148]\, 
        \can_top_level_inst/drv_bus[147]\, 
        \can_top_level_inst/drv_bus[146]\, 
        \can_top_level_inst/drv_bus[145]\, 
        \can_top_level_inst/drv_bus[144]\, 
        \can_top_level_inst/drv_bus[143]\, 
        \can_top_level_inst/drv_bus[142]\, 
        \can_top_level_inst/drv_bus[141]\, 
        \can_top_level_inst/drv_bus[140]\, 
        \can_top_level_inst/drv_bus[139]\, 
        \can_top_level_inst/drv_bus[138]\, 
        \can_top_level_inst/drv_bus[137]\, 
        \can_top_level_inst/drv_bus[136]\, 
        \can_top_level_inst/drv_bus[135]\, 
        \can_top_level_inst/drv_bus[134]\, 
        \can_top_level_inst/drv_bus[133]\, 
        \can_top_level_inst/drv_bus[132]\, 
        \can_top_level_inst/drv_bus[131]\, 
        \can_top_level_inst/drv_bus[130]\, 
        \can_top_level_inst/drv_bus[129]\, 
        \can_top_level_inst/drv_bus[128]\, 
        \can_top_level_inst/drv_bus[127]\, 
        \can_top_level_inst/drv_bus[126]\, 
        \can_top_level_inst/drv_bus[125]\, 
        \can_top_level_inst/drv_bus[124]\, 
        \can_top_level_inst/drv_bus[123]\, 
        \can_top_level_inst/drv_bus[122]\, 
        \can_top_level_inst/drv_bus[121]\, 
        \can_top_level_inst/drv_bus[120]\, 
        \can_top_level_inst/drv_bus[119]\, 
        \can_top_level_inst/drv_bus[118]\, 
        \can_top_level_inst/drv_bus[117]\, 
        \can_top_level_inst/drv_bus[116]\, 
        \can_top_level_inst/drv_bus[115]\, 
        \can_top_level_inst/drv_bus[114]\, 
        \can_top_level_inst/drv_bus[113]\, 
        \can_top_level_inst/drv_bus[112]\, 
        \can_top_level_inst/drv_bus[111]\, 
        \can_top_level_inst/drv_bus[110]\, 
        \can_top_level_inst/drv_bus[109]\, 
        \can_top_level_inst/drv_bus[108]\, 
        \can_top_level_inst/drv_bus[107]\, 
        \can_top_level_inst/drv_bus[106]\, 
        \can_top_level_inst/drv_bus[105]\, 
        \can_top_level_inst/drv_bus[104]\, 
        \can_top_level_inst/drv_bus[103]\, 
        \can_top_level_inst/drv_bus[102]\, 
        \can_top_level_inst/drv_bus[101]\, 
        \can_top_level_inst/drv_bus[100]\, 
        \can_top_level_inst/drv_bus[99]\, 
        \can_top_level_inst/drv_bus[98]\, 
        \can_top_level_inst/drv_bus[97]\, 
        \can_top_level_inst/drv_bus[96]\, 
        \can_top_level_inst/drv_bus[95]\, 
        \can_top_level_inst/drv_bus[94]\, 
        \can_top_level_inst/drv_bus[93]\, 
        \can_top_level_inst/drv_bus[92]\, 
        \can_top_level_inst/drv_bus[91]\, 
        \can_top_level_inst/drv_bus[90]\, 
        \can_top_level_inst/drv_bus[89]\, 
        \can_top_level_inst/drv_bus[88]\, 
        \can_top_level_inst/drv_bus[87]\, 
        \can_top_level_inst/drv_bus[86]\, 
        \can_top_level_inst/drv_bus[85]\, 
        \can_top_level_inst/drv_bus[84]\, 
        \can_top_level_inst/drv_bus[83]\, 
        \can_top_level_inst/drv_bus[82]\, 
        \can_top_level_inst/drv_bus[81]\, 
        \can_top_level_inst/drv_bus[60]\, 
        \can_top_level_inst/drv_bus[59]\, 
        \can_top_level_inst/drv_bus[58]\, 
        \can_top_level_inst/drv_bus[57]\, 
        \can_top_level_inst/drv_bus[56]\, 
        \can_top_level_inst/drv_bus[55]\, 
        \can_top_level_inst/drv_bus[54]\, 
        \can_top_level_inst/drv_bus[53]\, 
        \can_top_level_inst/drv_bus[52]\, 
        \can_top_level_inst/drv_bus[51]\, 
        \can_top_level_inst/drv_bus[50]\, 
        \can_top_level_inst/drv_bus[49]\, 
        \can_top_level_inst/drv_bus[48]\, 
        \can_top_level_inst/drv_bus[47]\, 
        \can_top_level_inst/drv_bus[46]\, 
        \can_top_level_inst/drv_bus[45]\, 
        \can_top_level_inst/drv_bus[44]\, 
        \can_top_level_inst/drv_bus[43]\, 
        \can_top_level_inst/drv_bus[42]\, 
        \can_top_level_inst/drv_bus[41]\, 
        \can_top_level_inst/drv_bus[40]\, 
        \can_top_level_inst/drv_bus[39]\, 
        \can_top_level_inst/drv_bus[38]\, 
        \can_top_level_inst/drv_bus[37]\, 
        \can_top_level_inst/drv_bus[36]\, 
        \can_top_level_inst/drv_bus[35]\, 
        \can_top_level_inst/drv_bus[34]\, 
        \can_top_level_inst/drv_bus[33]\, 
        \can_top_level_inst/drv_bus[32]\, 
        \can_top_level_inst/drv_bus[31]\, 
        \can_top_level_inst/drv_bus[30]\, 
        \can_top_level_inst/drv_bus[29]\, 
        \can_top_level_inst/drv_bus[28]\, 
        \can_top_level_inst/drv_bus[27]\, 
        \can_top_level_inst/drv_bus[26]\, 
        \can_top_level_inst/drv_bus[25]\, 
        \can_top_level_inst/drv_bus[24]\, 
        \can_top_level_inst/drv_bus[23]\, 
        \can_top_level_inst/drv_bus[22]\, 
        \can_top_level_inst/drv_bus[21]\, 
        \can_top_level_inst/drv_bus[20]\, 
        \can_top_level_inst/drv_bus[19]\, 
        \can_top_level_inst/drv_bus[18]\, 
        \can_top_level_inst/drv_bus[17]\, 
        \can_top_level_inst/drv_bus[16]\, 
        \can_top_level_inst/drv_bus[15]\, 
        \can_top_level_inst/drv_bus[14]\, 
        \can_top_level_inst/drv_bus[13]\, 
        \can_top_level_inst/drv_bus[12]\, 
        \can_top_level_inst/drv_bus[11]\, 
        \can_top_level_inst/drv_bus[10]\, 
        \can_top_level_inst/drv_bus[9]\, 
        \can_top_level_inst/drv_bus[8]\, 
        \can_top_level_inst/drv_bus[7]\, 
        \can_top_level_inst/drv_bus[6]\, 
        \can_top_level_inst/drv_bus[5]\, 
        \can_top_level_inst/drv_bus[4]\, 
        \can_top_level_inst/drv_bus[3]\, 
        \can_top_level_inst/drv_bus[2]\, 
        \can_top_level_inst/drv_bus[1]\, 
        \can_top_level_inst/drv_bus[0]\, 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\, 
        \can_top_level_inst/rec_ident[28]\, 
        \can_top_level_inst/rec_ident[27]\, 
        \can_top_level_inst/rec_ident[26]\, 
        \can_top_level_inst/rec_ident[25]\, 
        \can_top_level_inst/rec_ident[24]\, 
        \can_top_level_inst/rec_ident[23]\, 
        \can_top_level_inst/rec_ident[22]\, 
        \can_top_level_inst/rec_ident[21]\, 
        \can_top_level_inst/rec_ident[20]\, 
        \can_top_level_inst/rec_ident[19]\, 
        \can_top_level_inst/rec_ident[18]\, 
        \can_top_level_inst/rec_ident[17]\, 
        \can_top_level_inst/rec_ident[16]\, 
        \can_top_level_inst/rec_ident[15]\, 
        \can_top_level_inst/rec_ident[14]\, 
        \can_top_level_inst/rec_ident[13]\, 
        \can_top_level_inst/rec_ident[12]\, 
        \can_top_level_inst/rec_ident[11]\, 
        \can_top_level_inst/rec_ident[10]\, 
        \can_top_level_inst/rec_ident[9]\, 
        \can_top_level_inst/rec_ident[8]\, 
        \can_top_level_inst/rec_ident[7]\, 
        \can_top_level_inst/rec_ident[6]\, 
        \can_top_level_inst/rec_ident[5]\, 
        \can_top_level_inst/rec_ident[4]\, 
        \can_top_level_inst/rec_ident[3]\, 
        \can_top_level_inst/rec_ident[2]\, 
        \can_top_level_inst/rec_ident[1]\, 
        \can_top_level_inst/rec_ident[0]\, 
        \can_top_level_inst/rec_dlc[3]\, 
        \can_top_level_inst/rec_dlc[2]\, 
        \can_top_level_inst/rec_dlc[1]\, 
        \can_top_level_inst/rec_dlc[0]\, 
        \can_top_level_inst/test_registers_out.tst_control[1]\, 
        \can_top_level_inst/test_registers_out.tst_control[0]\, 
        \can_top_level_inst/test_registers_out.tst_dest[19]\, 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        \can_top_level_inst/test_registers_out.tst_dest[16]\, 
        \can_top_level_inst/test_registers_out.tst_dest[6]\, 
        \can_top_level_inst/test_registers_out.tst_dest[5]\, 
        \can_top_level_inst/test_registers_out.tst_dest[4]\, 
        \can_top_level_inst/test_registers_out.tst_dest[3]\, 
        \can_top_level_inst/test_registers_out.tst_dest[2]\, 
        \can_top_level_inst/test_registers_out.tst_dest[1]\, 
        \can_top_level_inst/test_registers_out.tst_dest[0]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[31]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[30]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[29]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[28]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[27]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[26]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[25]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[24]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[23]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[22]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[21]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[20]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[19]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[18]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[17]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[16]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[15]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[14]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[13]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[12]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[11]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[10]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[9]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[8]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[7]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[6]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[5]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[4]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[3]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[2]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[1]\, 
        \can_top_level_inst/test_registers_out.tst_wdata[0]\, 
        \can_top_level_inst/rx_frame_count[5]\, 
        \can_top_level_inst/rx_frame_count[4]\, 
        \can_top_level_inst/rx_frame_count[3]\, 
        \can_top_level_inst/rx_frame_count[2]\, 
        \can_top_level_inst/rx_frame_count[1]\, 
        \can_top_level_inst/rx_frame_count[0]\, 
        \can_top_level_inst/rx_mem_free[7]\, 
        \can_top_level_inst/rx_mem_free[6]\, 
        \can_top_level_inst/rx_mem_free[5]\, 
        \can_top_level_inst/rx_mem_free[4]\, 
        \can_top_level_inst/rx_mem_free[3]\, 
        \can_top_level_inst/rx_mem_free[2]\, 
        \can_top_level_inst/rx_mem_free[1]\, 
        \can_top_level_inst/rx_mem_free[0]\, 
        \can_top_level_inst/rx_read_pointer[6]\, 
        \can_top_level_inst/rx_read_pointer[5]\, 
        \can_top_level_inst/rx_read_pointer[4]\, 
        \can_top_level_inst/rx_read_pointer[3]\, 
        \can_top_level_inst/rx_read_pointer[2]\, 
        \can_top_level_inst/rx_read_pointer[1]\, 
        \can_top_level_inst/rx_read_pointer[0]\, 
        \can_top_level_inst/rx_write_pointer[6]\, 
        \can_top_level_inst/rx_write_pointer[5]\, 
        \can_top_level_inst/rx_write_pointer[4]\, 
        \can_top_level_inst/rx_write_pointer[3]\, 
        \can_top_level_inst/rx_write_pointer[2]\, 
        \can_top_level_inst/rx_write_pointer[1]\, 
        \can_top_level_inst/rx_write_pointer[0]\, 
        \can_top_level_inst/txtb_sw_cmd_index[6]\, 
        \can_top_level_inst/txtb_sw_cmd_index[5]\, 
        \can_top_level_inst/txtb_sw_cmd_index[4]\, 
        \can_top_level_inst/txtb_sw_cmd_index[3]\, 
        \can_top_level_inst/txtb_sw_cmd_index[2]\, 
        \can_top_level_inst/txtb_sw_cmd_index[1]\, 
        \can_top_level_inst/txtb_sw_cmd_index[0]\, 
        \can_top_level_inst/txtb_prorities_0[2]\, 
        \can_top_level_inst/txtb_prorities_0[1]\, 
        \can_top_level_inst/txtb_prorities_0[0]\, 
        \can_top_level_inst/txtb_prorities_1[2]\, 
        \can_top_level_inst/txtb_prorities_1[1]\, 
        \can_top_level_inst/txtb_prorities_1[0]\, 
        \can_top_level_inst/txtb_prorities_2[2]\, 
        \can_top_level_inst/txtb_prorities_2[1]\, 
        \can_top_level_inst/txtb_prorities_2[0]\, 
        \can_top_level_inst/txtb_prorities_3[2]\, 
        \can_top_level_inst/txtb_prorities_3[1]\, 
        \can_top_level_inst/txtb_prorities_3[0]\, 
        \can_top_level_inst/txtb_prorities_4[2]\, 
        \can_top_level_inst/txtb_prorities_4[1]\, 
        \can_top_level_inst/txtb_prorities_4[0]\, 
        \can_top_level_inst/txtb_prorities_5[2]\, 
        \can_top_level_inst/txtb_prorities_5[1]\, 
        \can_top_level_inst/txtb_prorities_5[0]\, 
        \can_top_level_inst/txtb_prorities_6[2]\, 
        \can_top_level_inst/txtb_prorities_6[1]\, 
        \can_top_level_inst/txtb_prorities_6[0]\, 
        \can_top_level_inst/txtb_prorities_7[2]\, 
        \can_top_level_inst/txtb_prorities_7[1]\, 
        \can_top_level_inst/txtb_prorities_7[0]\, 
        \can_top_level_inst/trv_delay[6]\, 
        \can_top_level_inst/trv_delay[5]\, 
        \can_top_level_inst/trv_delay[4]\, 
        \can_top_level_inst/trv_delay[3]\, 
        \can_top_level_inst/trv_delay[2]\, 
        \can_top_level_inst/trv_delay[1]\, 
        \can_top_level_inst/trv_delay[0]\, 
        \can_top_level_inst/int_vector[11]\, 
        \can_top_level_inst/int_vector[10]\, 
        \can_top_level_inst/int_vector[9]\, 
        \can_top_level_inst/int_vector[8]\, 
        \can_top_level_inst/int_vector[7]\, 
        \can_top_level_inst/int_vector[6]\, 
        \can_top_level_inst/int_vector[5]\, 
        \can_top_level_inst/int_vector[4]\, 
        \can_top_level_inst/int_vector[3]\, 
        \can_top_level_inst/int_vector[2]\, 
        \can_top_level_inst/int_vector[1]\, 
        \can_top_level_inst/int_vector[0]\, 
        \can_top_level_inst/int_ena[11]\, 
        \can_top_level_inst/int_ena[10]\, 
        \can_top_level_inst/int_ena[9]\, 
        \can_top_level_inst/int_ena[8]\, 
        \can_top_level_inst/int_ena[7]\, 
        \can_top_level_inst/int_ena[6]\, 
        \can_top_level_inst/int_ena[5]\, 
        \can_top_level_inst/int_ena[4]\, 
        \can_top_level_inst/int_ena[3]\, 
        \can_top_level_inst/int_ena[2]\, 
        \can_top_level_inst/int_ena[1]\, 
        \can_top_level_inst/int_ena[0]\, 
        \can_top_level_inst/int_mask[11]\, 
        \can_top_level_inst/int_mask[10]\, 
        \can_top_level_inst/int_mask[9]\, 
        \can_top_level_inst/int_mask[8]\, 
        \can_top_level_inst/int_mask[7]\, 
        \can_top_level_inst/int_mask[6]\, 
        \can_top_level_inst/int_mask[5]\, 
        \can_top_level_inst/int_mask[4]\, 
        \can_top_level_inst/int_mask[3]\, 
        \can_top_level_inst/int_mask[2]\, 
        \can_top_level_inst/int_mask[1]\, 
        \can_top_level_inst/int_mask[0]\, 
        \can_top_level_inst/store_data_word[31]\, 
        \can_top_level_inst/store_data_word[30]\, 
        \can_top_level_inst/store_data_word[29]\, 
        \can_top_level_inst/store_data_word[28]\, 
        \can_top_level_inst/store_data_word[27]\, 
        \can_top_level_inst/store_data_word[26]\, 
        \can_top_level_inst/store_data_word[25]\, 
        \can_top_level_inst/store_data_word[24]\, 
        \can_top_level_inst/store_data_word[23]\, 
        \can_top_level_inst/store_data_word[22]\, 
        \can_top_level_inst/store_data_word[21]\, 
        \can_top_level_inst/store_data_word[20]\, 
        \can_top_level_inst/store_data_word[19]\, 
        \can_top_level_inst/store_data_word[18]\, 
        \can_top_level_inst/store_data_word[17]\, 
        \can_top_level_inst/store_data_word[16]\, 
        \can_top_level_inst/store_data_word[15]\, 
        \can_top_level_inst/store_data_word[14]\, 
        \can_top_level_inst/store_data_word[13]\, 
        \can_top_level_inst/store_data_word[12]\, 
        \can_top_level_inst/store_data_word[11]\, 
        \can_top_level_inst/store_data_word[10]\, 
        \can_top_level_inst/store_data_word[9]\, 
        \can_top_level_inst/store_data_word[8]\, 
        \can_top_level_inst/store_data_word[7]\, 
        \can_top_level_inst/store_data_word[6]\, 
        \can_top_level_inst/store_data_word[5]\, 
        \can_top_level_inst/store_data_word[4]\, 
        \can_top_level_inst/store_data_word[3]\, 
        \can_top_level_inst/store_data_word[2]\, 
        \can_top_level_inst/store_data_word[1]\, 
        \can_top_level_inst/store_data_word[0]\, 
        \can_top_level_inst/tran_word[31]\, 
        \can_top_level_inst/tran_word[30]\, 
        \can_top_level_inst/tran_word[29]\, 
        \can_top_level_inst/tran_word[28]\, 
        \can_top_level_inst/tran_word[27]\, 
        \can_top_level_inst/tran_word[26]\, 
        \can_top_level_inst/tran_word[25]\, 
        \can_top_level_inst/tran_word[24]\, 
        \can_top_level_inst/tran_word[23]\, 
        \can_top_level_inst/tran_word[22]\, 
        \can_top_level_inst/tran_word[21]\, 
        \can_top_level_inst/tran_word[20]\, 
        \can_top_level_inst/tran_word[19]\, 
        \can_top_level_inst/tran_word[18]\, 
        \can_top_level_inst/tran_word[17]\, 
        \can_top_level_inst/tran_word[16]\, 
        \can_top_level_inst/tran_word[15]\, 
        \can_top_level_inst/tran_word[14]\, 
        \can_top_level_inst/tran_word[13]\, 
        \can_top_level_inst/tran_word[12]\, 
        \can_top_level_inst/tran_word[11]\, 
        \can_top_level_inst/tran_word[10]\, 
        \can_top_level_inst/tran_word[9]\, 
        \can_top_level_inst/tran_word[8]\, 
        \can_top_level_inst/tran_word[7]\, 
        \can_top_level_inst/tran_word[6]\, 
        \can_top_level_inst/tran_word[5]\, 
        \can_top_level_inst/tran_word[4]\, 
        \can_top_level_inst/tran_word[3]\, 
        \can_top_level_inst/tran_word[2]\, 
        \can_top_level_inst/tran_word[1]\, 
        \can_top_level_inst/tran_word[0]\, 
        \can_top_level_inst/tran_dlc[3]\, 
        \can_top_level_inst/tran_dlc[2]\, 
        \can_top_level_inst/tran_dlc[1]\, 
        \can_top_level_inst/tran_dlc[0]\, 
        \can_top_level_inst/tran_identifier[28]\, 
        \can_top_level_inst/tran_identifier[27]\, 
        \can_top_level_inst/tran_identifier[26]\, 
        \can_top_level_inst/tran_identifier[25]\, 
        \can_top_level_inst/tran_identifier[24]\, 
        \can_top_level_inst/tran_identifier[23]\, 
        \can_top_level_inst/tran_identifier[22]\, 
        \can_top_level_inst/tran_identifier[21]\, 
        \can_top_level_inst/tran_identifier[20]\, 
        \can_top_level_inst/tran_identifier[19]\, 
        \can_top_level_inst/tran_identifier[18]\, 
        \can_top_level_inst/tran_identifier[17]\, 
        \can_top_level_inst/tran_identifier[16]\, 
        \can_top_level_inst/tran_identifier[15]\, 
        \can_top_level_inst/tran_identifier[14]\, 
        \can_top_level_inst/tran_identifier[13]\, 
        \can_top_level_inst/tran_identifier[12]\, 
        \can_top_level_inst/tran_identifier[11]\, 
        \can_top_level_inst/tran_identifier[10]\, 
        \can_top_level_inst/tran_identifier[9]\, 
        \can_top_level_inst/tran_identifier[8]\, 
        \can_top_level_inst/tran_identifier[7]\, 
        \can_top_level_inst/tran_identifier[6]\, 
        \can_top_level_inst/tran_identifier[5]\, 
        \can_top_level_inst/tran_identifier[4]\, 
        \can_top_level_inst/tran_identifier[3]\, 
        \can_top_level_inst/tran_identifier[2]\, 
        \can_top_level_inst/tran_identifier[1]\, 
        \can_top_level_inst/tran_identifier[0]\, 
        \can_top_level_inst/tran_frame_test.tprm[4]\, 
        \can_top_level_inst/tran_frame_test.tprm[3]\, 
        \can_top_level_inst/tran_frame_test.tprm[2]\, 
        \can_top_level_inst/tran_frame_test.tprm[1]\, 
        \can_top_level_inst/tran_frame_test.tprm[0]\, 
        \can_top_level_inst/txtb_ptr[4]\, 
        \can_top_level_inst/txtb_ptr[3]\, 
        \can_top_level_inst/txtb_ptr[2]\, 
        \can_top_level_inst/txtb_ptr[1]\, 
        \can_top_level_inst/txtb_ptr[0]\, 
        \can_top_level_inst/rx_triggers[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[31]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[30]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[29]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[28]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[27]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[26]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[25]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[24]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[23]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[22]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[21]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[20]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[19]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[18]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[17]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[16]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[15]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[14]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[13]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[12]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[11]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[10]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[9]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[8]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[7]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[6]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[5]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[4]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[3]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[2]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[1]\, 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[0]\, 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[15]\, 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[13]\, 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[11]\, 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[9]\, 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_tst_control_reg_comp_wr_select[0]\, 
        \can_top_level_inst/txtb_port_b_address[4]\, 
        \can_top_level_inst/txtb_port_b_address[3]\, 
        \can_top_level_inst/txtb_port_b_address[2]\, 
        \can_top_level_inst/txtb_port_b_address[1]\, 
        \can_top_level_inst/txtb_port_b_address[0]\, 
        \can_top_level_inst/txtb_index_muxed[2]\, 
        \can_top_level_inst/txtb_index_muxed[1]\, 
        \can_top_level_inst/txtb_index_muxed[0]\, 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, 
        \can_top_level_inst/txtb_hw_cmd_index[1]\, 
        \can_top_level_inst/txtb_hw_cmd_index[0]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        \can_top_level_inst/reg_value_r_rep[17]\, 
        \can_top_level_inst/txtb_allow_bb[6]\, 
        \can_top_level_inst/txtb_allow_bb[4]\, 
        \can_top_level_inst/txtb_allow_bb[2]\, 
        \can_top_level_inst/txtb_allow_bb[0]\, 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[6]\, 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[3]\, 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[2]\, 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[1]\, 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[0]\, 
        \can_top_level_inst/txtb_parity_error_valid[7]\, 
        \can_top_level_inst/txtb_parity_error_valid[6]\, 
        \can_top_level_inst/txtb_parity_error_valid[5]\, 
        \can_top_level_inst/txtb_parity_error_valid[3]\, 
        \can_top_level_inst/txtb_parity_error_valid[2]\, 
        \can_top_level_inst/txtb_parity_error_valid[1]\, 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        \can_top_level_inst/txtb_state_2[3]\, 
        \can_top_level_inst/rx_read_buff[18]\, 
        \can_top_level_inst/rx_read_buff[16]\, 
        \can_top_level_inst/rx_read_buff[15]\, 
        \can_top_level_inst/rx_read_buff[13]\, 
        \can_top_level_inst/rx_read_buff[11]\, 
        \can_top_level_inst/rx_read_buff[10]\, 
        \can_top_level_inst/rx_read_buff[9]\, 
        \can_top_level_inst/rx_read_buff[8]\, 
        \can_top_level_inst/rx_read_buff[6]\, 
        \can_top_level_inst/rx_read_buff[3]\, 
        \can_top_level_inst/txtb_state_0[3]\, 
        \can_top_level_inst/txtb_port_b_data_1[31]\, 
        \can_top_level_inst/txtb_port_b_data_1[30]\, 
        \can_top_level_inst/txtb_port_b_data_1[29]\, 
        \can_top_level_inst/txtb_port_b_data_1[28]\, 
        \can_top_level_inst/txtb_port_b_data_1[27]\, 
        \can_top_level_inst/txtb_port_b_data_1[26]\, 
        \can_top_level_inst/txtb_port_b_data_1[25]\, 
        \can_top_level_inst/txtb_port_b_data_1[24]\, 
        \can_top_level_inst/txtb_port_b_data_1[23]\, 
        \can_top_level_inst/txtb_port_b_data_1[22]\, 
        \can_top_level_inst/txtb_port_b_data_1[21]\, 
        \can_top_level_inst/txtb_port_b_data_1[20]\, 
        \can_top_level_inst/txtb_port_b_data_1[19]\, 
        \can_top_level_inst/txtb_port_b_data_1[18]\, 
        \can_top_level_inst/txtb_port_b_data_1[17]\, 
        \can_top_level_inst/txtb_port_b_data_1[16]\, 
        \can_top_level_inst/txtb_port_b_data_1[15]\, 
        \can_top_level_inst/txtb_port_b_data_1[14]\, 
        \can_top_level_inst/txtb_port_b_data_1[13]\, 
        \can_top_level_inst/txtb_port_b_data_1[12]\, 
        \can_top_level_inst/txtb_port_b_data_1[11]\, 
        \can_top_level_inst/txtb_port_b_data_1[10]\, 
        \can_top_level_inst/txtb_port_b_data_1[9]\, 
        \can_top_level_inst/txtb_port_b_data_1[8]\, 
        \can_top_level_inst/txtb_port_b_data_1[7]\, 
        \can_top_level_inst/txtb_port_b_data_1[6]\, 
        \can_top_level_inst/txtb_port_b_data_1[5]\, 
        \can_top_level_inst/txtb_port_b_data_1[4]\, 
        \can_top_level_inst/txtb_port_b_data_1[3]\, 
        \can_top_level_inst/txtb_port_b_data_1[2]\, 
        \can_top_level_inst/txtb_port_b_data_1[1]\, 
        \can_top_level_inst/txtb_port_b_data_1[0]\, 
        \can_top_level_inst/txtb_port_b_data_5[31]\, 
        \can_top_level_inst/txtb_port_b_data_5[30]\, 
        \can_top_level_inst/txtb_port_b_data_5[29]\, 
        \can_top_level_inst/txtb_port_b_data_5[28]\, 
        \can_top_level_inst/txtb_port_b_data_5[27]\, 
        \can_top_level_inst/txtb_port_b_data_5[26]\, 
        \can_top_level_inst/txtb_port_b_data_5[25]\, 
        \can_top_level_inst/txtb_port_b_data_5[24]\, 
        \can_top_level_inst/txtb_port_b_data_5[23]\, 
        \can_top_level_inst/txtb_port_b_data_5[22]\, 
        \can_top_level_inst/txtb_port_b_data_5[21]\, 
        \can_top_level_inst/txtb_port_b_data_5[20]\, 
        \can_top_level_inst/txtb_port_b_data_5[19]\, 
        \can_top_level_inst/txtb_port_b_data_5[18]\, 
        \can_top_level_inst/txtb_port_b_data_5[17]\, 
        \can_top_level_inst/txtb_port_b_data_5[16]\, 
        \can_top_level_inst/txtb_port_b_data_5[15]\, 
        \can_top_level_inst/txtb_port_b_data_5[14]\, 
        \can_top_level_inst/txtb_port_b_data_5[13]\, 
        \can_top_level_inst/txtb_port_b_data_5[12]\, 
        \can_top_level_inst/txtb_port_b_data_5[11]\, 
        \can_top_level_inst/txtb_port_b_data_5[10]\, 
        \can_top_level_inst/txtb_port_b_data_5[9]\, 
        \can_top_level_inst/txtb_port_b_data_5[8]\, 
        \can_top_level_inst/txtb_port_b_data_5[7]\, 
        \can_top_level_inst/txtb_port_b_data_5[6]\, 
        \can_top_level_inst/txtb_port_b_data_5[5]\, 
        \can_top_level_inst/txtb_port_b_data_5[4]\, 
        \can_top_level_inst/txtb_port_b_data_5[3]\, 
        \can_top_level_inst/txtb_port_b_data_5[2]\, 
        \can_top_level_inst/txtb_port_b_data_5[1]\, 
        \can_top_level_inst/txtb_port_b_data_5[0]\, 
        \can_top_level_inst/txtb_port_b_data_2[31]\, 
        \can_top_level_inst/txtb_port_b_data_2[30]\, 
        \can_top_level_inst/txtb_port_b_data_2[29]\, 
        \can_top_level_inst/txtb_port_b_data_2[28]\, 
        \can_top_level_inst/txtb_port_b_data_2[27]\, 
        \can_top_level_inst/txtb_port_b_data_2[26]\, 
        \can_top_level_inst/txtb_port_b_data_2[25]\, 
        \can_top_level_inst/txtb_port_b_data_2[24]\, 
        \can_top_level_inst/txtb_port_b_data_2[23]\, 
        \can_top_level_inst/txtb_port_b_data_2[22]\, 
        \can_top_level_inst/txtb_port_b_data_2[21]\, 
        \can_top_level_inst/txtb_port_b_data_2[20]\, 
        \can_top_level_inst/txtb_port_b_data_2[19]\, 
        \can_top_level_inst/txtb_port_b_data_2[18]\, 
        \can_top_level_inst/txtb_port_b_data_2[17]\, 
        \can_top_level_inst/txtb_port_b_data_2[16]\, 
        \can_top_level_inst/txtb_port_b_data_2[15]\, 
        \can_top_level_inst/txtb_port_b_data_2[14]\, 
        \can_top_level_inst/txtb_port_b_data_2[13]\, 
        \can_top_level_inst/txtb_port_b_data_2[12]\, 
        \can_top_level_inst/txtb_port_b_data_2[11]\, 
        \can_top_level_inst/txtb_port_b_data_2[10]\, 
        \can_top_level_inst/txtb_port_b_data_2[9]\, 
        \can_top_level_inst/txtb_port_b_data_2[8]\, 
        \can_top_level_inst/txtb_port_b_data_2[7]\, 
        \can_top_level_inst/txtb_port_b_data_2[6]\, 
        \can_top_level_inst/txtb_port_b_data_2[5]\, 
        \can_top_level_inst/txtb_port_b_data_2[4]\, 
        \can_top_level_inst/txtb_port_b_data_2[3]\, 
        \can_top_level_inst/txtb_port_b_data_2[2]\, 
        \can_top_level_inst/txtb_port_b_data_2[1]\, 
        \can_top_level_inst/txtb_port_b_data_2[0]\, 
        \can_top_level_inst/txtb_port_b_data_6[31]\, 
        \can_top_level_inst/txtb_port_b_data_6[30]\, 
        \can_top_level_inst/txtb_port_b_data_6[29]\, 
        \can_top_level_inst/txtb_port_b_data_6[28]\, 
        \can_top_level_inst/txtb_port_b_data_6[27]\, 
        \can_top_level_inst/txtb_port_b_data_6[26]\, 
        \can_top_level_inst/txtb_port_b_data_6[25]\, 
        \can_top_level_inst/txtb_port_b_data_6[24]\, 
        \can_top_level_inst/txtb_port_b_data_6[23]\, 
        \can_top_level_inst/txtb_port_b_data_6[22]\, 
        \can_top_level_inst/txtb_port_b_data_6[21]\, 
        \can_top_level_inst/txtb_port_b_data_6[20]\, 
        \can_top_level_inst/txtb_port_b_data_6[19]\, 
        \can_top_level_inst/txtb_port_b_data_6[18]\, 
        \can_top_level_inst/txtb_port_b_data_6[17]\, 
        \can_top_level_inst/txtb_port_b_data_6[16]\, 
        \can_top_level_inst/txtb_port_b_data_6[15]\, 
        \can_top_level_inst/txtb_port_b_data_6[14]\, 
        \can_top_level_inst/txtb_port_b_data_6[13]\, 
        \can_top_level_inst/txtb_port_b_data_6[12]\, 
        \can_top_level_inst/txtb_port_b_data_6[11]\, 
        \can_top_level_inst/txtb_port_b_data_6[10]\, 
        \can_top_level_inst/txtb_port_b_data_6[9]\, 
        \can_top_level_inst/txtb_port_b_data_6[8]\, 
        \can_top_level_inst/txtb_port_b_data_6[7]\, 
        \can_top_level_inst/txtb_port_b_data_6[6]\, 
        \can_top_level_inst/txtb_port_b_data_6[5]\, 
        \can_top_level_inst/txtb_port_b_data_6[4]\, 
        \can_top_level_inst/txtb_port_b_data_6[3]\, 
        \can_top_level_inst/txtb_port_b_data_6[2]\, 
        \can_top_level_inst/txtb_port_b_data_6[1]\, 
        \can_top_level_inst/txtb_port_b_data_6[0]\, 
        \can_top_level_inst/txtb_port_b_data_3[31]\, 
        \can_top_level_inst/txtb_port_b_data_3[30]\, 
        \can_top_level_inst/txtb_port_b_data_3[29]\, 
        \can_top_level_inst/txtb_port_b_data_3[28]\, 
        \can_top_level_inst/txtb_port_b_data_3[27]\, 
        \can_top_level_inst/txtb_port_b_data_3[26]\, 
        \can_top_level_inst/txtb_port_b_data_3[25]\, 
        \can_top_level_inst/txtb_port_b_data_3[24]\, 
        \can_top_level_inst/txtb_port_b_data_3[23]\, 
        \can_top_level_inst/txtb_port_b_data_3[22]\, 
        \can_top_level_inst/txtb_port_b_data_3[21]\, 
        \can_top_level_inst/txtb_port_b_data_3[20]\, 
        \can_top_level_inst/txtb_port_b_data_3[19]\, 
        \can_top_level_inst/txtb_port_b_data_3[18]\, 
        \can_top_level_inst/txtb_port_b_data_3[17]\, 
        \can_top_level_inst/txtb_port_b_data_3[16]\, 
        \can_top_level_inst/txtb_port_b_data_3[15]\, 
        \can_top_level_inst/txtb_port_b_data_3[14]\, 
        \can_top_level_inst/txtb_port_b_data_3[13]\, 
        \can_top_level_inst/txtb_port_b_data_3[12]\, 
        \can_top_level_inst/txtb_port_b_data_3[11]\, 
        \can_top_level_inst/txtb_port_b_data_3[10]\, 
        \can_top_level_inst/txtb_port_b_data_3[9]\, 
        \can_top_level_inst/txtb_port_b_data_3[8]\, 
        \can_top_level_inst/txtb_port_b_data_3[7]\, 
        \can_top_level_inst/txtb_port_b_data_3[6]\, 
        \can_top_level_inst/txtb_port_b_data_3[5]\, 
        \can_top_level_inst/txtb_port_b_data_3[4]\, 
        \can_top_level_inst/txtb_port_b_data_3[3]\, 
        \can_top_level_inst/txtb_port_b_data_3[2]\, 
        \can_top_level_inst/txtb_port_b_data_3[1]\, 
        \can_top_level_inst/txtb_port_b_data_3[0]\, 
        \can_top_level_inst/txtb_port_b_data_7[31]\, 
        \can_top_level_inst/txtb_port_b_data_7[30]\, 
        \can_top_level_inst/txtb_port_b_data_7[29]\, 
        \can_top_level_inst/txtb_port_b_data_7[28]\, 
        \can_top_level_inst/txtb_port_b_data_7[27]\, 
        \can_top_level_inst/txtb_port_b_data_7[26]\, 
        \can_top_level_inst/txtb_port_b_data_7[25]\, 
        \can_top_level_inst/txtb_port_b_data_7[24]\, 
        \can_top_level_inst/txtb_port_b_data_7[23]\, 
        \can_top_level_inst/txtb_port_b_data_7[22]\, 
        \can_top_level_inst/txtb_port_b_data_7[21]\, 
        \can_top_level_inst/txtb_port_b_data_7[20]\, 
        \can_top_level_inst/txtb_port_b_data_7[19]\, 
        \can_top_level_inst/txtb_port_b_data_7[18]\, 
        \can_top_level_inst/txtb_port_b_data_7[17]\, 
        \can_top_level_inst/txtb_port_b_data_7[16]\, 
        \can_top_level_inst/txtb_port_b_data_7[15]\, 
        \can_top_level_inst/txtb_port_b_data_7[14]\, 
        \can_top_level_inst/txtb_port_b_data_7[13]\, 
        \can_top_level_inst/txtb_port_b_data_7[12]\, 
        \can_top_level_inst/txtb_port_b_data_7[11]\, 
        \can_top_level_inst/txtb_port_b_data_7[10]\, 
        \can_top_level_inst/txtb_port_b_data_7[9]\, 
        \can_top_level_inst/txtb_port_b_data_7[8]\, 
        \can_top_level_inst/txtb_port_b_data_7[7]\, 
        \can_top_level_inst/txtb_port_b_data_7[6]\, 
        \can_top_level_inst/txtb_port_b_data_7[5]\, 
        \can_top_level_inst/txtb_port_b_data_7[4]\, 
        \can_top_level_inst/txtb_port_b_data_7[3]\, 
        \can_top_level_inst/txtb_port_b_data_7[2]\, 
        \can_top_level_inst/txtb_port_b_data_7[1]\, 
        \can_top_level_inst/txtb_port_b_data_7[0]\, 
        \can_top_level_inst/txtb_port_b_data_0[31]\, 
        \can_top_level_inst/txtb_port_b_data_0[30]\, 
        \can_top_level_inst/txtb_port_b_data_0[29]\, 
        \can_top_level_inst/txtb_port_b_data_0[28]\, 
        \can_top_level_inst/txtb_port_b_data_0[27]\, 
        \can_top_level_inst/txtb_port_b_data_0[26]\, 
        \can_top_level_inst/txtb_port_b_data_0[25]\, 
        \can_top_level_inst/txtb_port_b_data_0[24]\, 
        \can_top_level_inst/txtb_port_b_data_0[23]\, 
        \can_top_level_inst/txtb_port_b_data_0[22]\, 
        \can_top_level_inst/txtb_port_b_data_0[21]\, 
        \can_top_level_inst/txtb_port_b_data_0[20]\, 
        \can_top_level_inst/txtb_port_b_data_0[19]\, 
        \can_top_level_inst/txtb_port_b_data_0[18]\, 
        \can_top_level_inst/txtb_port_b_data_0[17]\, 
        \can_top_level_inst/txtb_port_b_data_0[16]\, 
        \can_top_level_inst/txtb_port_b_data_0[15]\, 
        \can_top_level_inst/txtb_port_b_data_0[14]\, 
        \can_top_level_inst/txtb_port_b_data_0[13]\, 
        \can_top_level_inst/txtb_port_b_data_0[12]\, 
        \can_top_level_inst/txtb_port_b_data_0[11]\, 
        \can_top_level_inst/txtb_port_b_data_0[10]\, 
        \can_top_level_inst/txtb_port_b_data_0[9]\, 
        \can_top_level_inst/txtb_port_b_data_0[8]\, 
        \can_top_level_inst/txtb_port_b_data_0[7]\, 
        \can_top_level_inst/txtb_port_b_data_0[6]\, 
        \can_top_level_inst/txtb_port_b_data_0[5]\, 
        \can_top_level_inst/txtb_port_b_data_0[4]\, 
        \can_top_level_inst/txtb_port_b_data_0[3]\, 
        \can_top_level_inst/txtb_port_b_data_0[2]\, 
        \can_top_level_inst/txtb_port_b_data_0[1]\, 
        \can_top_level_inst/txtb_port_b_data_0[0]\, 
        \can_top_level_inst/txtb_port_b_data_4[31]\, 
        \can_top_level_inst/txtb_port_b_data_4[30]\, 
        \can_top_level_inst/txtb_port_b_data_4[29]\, 
        \can_top_level_inst/txtb_port_b_data_4[28]\, 
        \can_top_level_inst/txtb_port_b_data_4[27]\, 
        \can_top_level_inst/txtb_port_b_data_4[26]\, 
        \can_top_level_inst/txtb_port_b_data_4[25]\, 
        \can_top_level_inst/txtb_port_b_data_4[24]\, 
        \can_top_level_inst/txtb_port_b_data_4[23]\, 
        \can_top_level_inst/txtb_port_b_data_4[22]\, 
        \can_top_level_inst/txtb_port_b_data_4[21]\, 
        \can_top_level_inst/txtb_port_b_data_4[20]\, 
        \can_top_level_inst/txtb_port_b_data_4[19]\, 
        \can_top_level_inst/txtb_port_b_data_4[18]\, 
        \can_top_level_inst/txtb_port_b_data_4[17]\, 
        \can_top_level_inst/txtb_port_b_data_4[16]\, 
        \can_top_level_inst/txtb_port_b_data_4[15]\, 
        \can_top_level_inst/txtb_port_b_data_4[14]\, 
        \can_top_level_inst/txtb_port_b_data_4[13]\, 
        \can_top_level_inst/txtb_port_b_data_4[12]\, 
        \can_top_level_inst/txtb_port_b_data_4[11]\, 
        \can_top_level_inst/txtb_port_b_data_4[10]\, 
        \can_top_level_inst/txtb_port_b_data_4[9]\, 
        \can_top_level_inst/txtb_port_b_data_4[8]\, 
        \can_top_level_inst/txtb_port_b_data_4[7]\, 
        \can_top_level_inst/txtb_port_b_data_4[6]\, 
        \can_top_level_inst/txtb_port_b_data_4[5]\, 
        \can_top_level_inst/txtb_port_b_data_4[4]\, 
        \can_top_level_inst/txtb_port_b_data_4[3]\, 
        \can_top_level_inst/txtb_port_b_data_4[2]\, 
        \can_top_level_inst/txtb_port_b_data_4[1]\, 
        \can_top_level_inst/txtb_port_b_data_4[0]\, 
        \can_top_level_inst/txtb_state_4[3]\, 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_command_reg_comp_wr_select[1]\, 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_command_reg_comp_wr_select[0]\, 
        \can_top_level_inst/txtb_parity_error_valid_2[6]\, 
        \can_top_level_inst/txtb_state_1[3]\, 
        \can_top_level_inst/txtb_state_5[3]\, 
        \can_top_level_inst/txtb_state_3[3]\, 
        \can_top_level_inst/txtb_parity_error_valid_1[0]\, 
        \can_top_level_inst/drv_bus_i[353]\, 
        \can_top_level_inst/rx_triggers_i[1]\, 
        \can_top_level_inst/drv_bus_fast[475]\, 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_tx_command_reg_comp_reg_value_r_fast[14]\, 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        \can_top_level_inst/ssp_reset\, 
        \can_top_level_inst/br_shifted\, 
        \can_top_level_inst/is_bus_off\, 
        \can_top_level_inst/rec_valid\, 
        \can_top_level_inst/tran_valid\, 
        \can_top_level_inst/rec_brs\, 
        \can_top_level_inst/rec_esi\, 
        \can_top_level_inst/fcs_changed\, 
        \can_top_level_inst/err_detected\, 
        \can_top_level_inst/is_overload\, 
        \can_top_level_inst/sof_pulse\, 
        \can_top_level_inst/rx_data_overrun\, 
        \can_top_level_inst/rx_parity_error\, 
        \can_top_level_inst/txtb_sw_cmd.set_rdy\, 
        \can_top_level_inst/txtb_sw_cmd.set_ety\, 
        \can_top_level_inst/txtb_sw_cmd.set_abt\, 
        \can_top_level_inst/txt_buf_failed_bof\, 
        \can_top_level_inst/store_data_f\, 
        \can_top_level_inst/txtb_hw_cmd.unlock\, 
        \can_top_level_inst/txtb_hw_cmd.err\, 
        \can_top_level_inst/txtb_hw_cmd.arbl\, 
        \can_top_level_inst/txtb_hw_cmd.failed\, 
        \can_top_level_inst/tran_is_rtr\, 
        \can_top_level_inst/tran_brs\, 
        \can_top_level_inst/tran_frame_test.fstc\, 
        \can_top_level_inst/tran_frame_test.fcrc\, 
        \can_top_level_inst/tran_frame_test.sdlc\, 
        \can_top_level_inst/tran_frame_valid\, 
        \can_top_level_inst/tran_frame_parity_error\, 
        \can_top_level_inst/txtb_changed\, 
        \can_top_level_inst/txtb_clk_en\, 
        \can_top_level_inst/store_metadata\, 
        \can_top_level_inst/store_data\, 
        \can_top_level_inst/rec_abort\, 
        \can_top_level_inst/tx_trigger\, 
        \can_top_level_inst/dbt_ctrs_en\, 
        \can_top_level_inst/rx_data_wbs\, 
        \can_top_level_inst/bit_err\, 
        \can_top_level_inst/btmc_reset\, 
        \can_top_level_inst/gen_first_ssp\, 
        \can_top_level_inst/sync_edge\, 
        \can_top_level_inst/tq_edge\, 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_Y\, 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        \can_top_level_inst/memory_registers_inst_global_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        \can_top_level_inst/rec_is_rtr\, 
        \can_top_level_inst/bus_sampling_inst_prev_Sample\, 
        \can_top_level_inst/bus_sampling_inst_data_edge_detector_inst_rx_data_sync_prev\, 
        \can_top_level_inst/frame_filters_inst_ident_valid_q\, 
        \can_top_level_inst/int_manager_inst_int_module_gen_7_int_module_inst_int_stat_proc_un2_int_status_set\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_159\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_190\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_139\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        \can_top_level_inst/N_4606\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_183_1\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        \can_top_level_inst/N_138\, 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10_2\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_160\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_172\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_172\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_180\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_147\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_139\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        \can_top_level_inst/tx_arbitrator_inst_priority_decoder_inst_un1_m2_0_a2_0_1\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena_2\, 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_data_mux_test_registers_comp_un2_index_sat_1\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa3\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0_1\, 
        \can_top_level_inst/prescaler_inst_tq_edge_nbt\, 
        \can_top_level_inst/prescaler_inst_tq_edge_dbt\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_181\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_148\, 
        \can_top_level_inst/txtb_parity_check_valid\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena_3\, 
        \can_top_level_inst/rec_abort_f\, 
        \can_top_level_inst/rec_valid_f\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8_2\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_159\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        \can_top_level_inst/memory_registers_inst_test_registers_in_tst_rdata_sn_N_8\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        \can_top_level_inst/rx_mof\, 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa7\, 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_stat_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa19\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa18\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa16\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa15\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa13\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa9\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa4\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa1\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5_1\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6_1\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20_1\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_161_2\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa17\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa14\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa12\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa11\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_135\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_135\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_129\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_129\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena_2\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_139\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_190\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_159\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_147\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_148\, 
        \can_top_level_inst/memory_registers_inst_txnf_calc_proc_6_un6_txtb_state_2\, 
        \can_top_level_inst/memory_registers_inst_txnf_calc_proc_7_un7_txtb_state_2\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_148\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_147\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_148\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_147\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_159\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_190\, 
        \can_top_level_inst/bus_sampling_inst_data_edge_detector_inst_un7_sync_edge_1\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_un1_txtb_available_i_0_out_tz\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_un1_ram_write_0\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_un1_ram_write_0\, 
        \can_top_level_inst/prescaler_inst_un2_tq_edge\, 
        \can_top_level_inst/rx_buffer_inst_rx_full_1_4\, 
        \can_top_level_inst/rx_buffer_inst_rx_full_1_5\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_un15_txtb_hw_cmd_int_0_a3_1_0\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_un73_txtb_hw_cmd_int_0_a3_1_0\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_m1_e_1\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_m1_e_1\, 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_129_i\, 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_129_i\, 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_129_i\, 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_129_i\, 
        \can_top_level_inst/N_3902_i\, 
        \can_top_level_inst/N_3901_i\, 
        \can_top_level_inst/N_3904_i\, 
        \can_top_level_inst/N_3903_i\, 
        \can_top_level_inst/N_3900_i\, 
        \can_top_level_inst/N_3897_i\, 
        \can_top_level_inst/N_3899_i\, 
        \can_top_level_inst/N_3898_i\, 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_135_i\, 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_135_i\, 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_135_i\, 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_135_i\, 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena_i\, 
        \can_top_level_inst/N_141_mux_i\, 
        \can_top_level_inst/N_3896_i\, 
        \can_top_level_inst/txtb_sw_cmd.set_abt_fast\, 
        \can_top_level_inst/txtb_sw_cmd.set_abt_rep1\, 
        \can_top_level_inst/drv_bus_475_rep1\, 
        \can_top_level_inst/drv_bus_475_rep2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_2_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/N_3897\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un7_txtb_parity_error_valid_i_1_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_0_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_126_i\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_124_i\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_163\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_170_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_147\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_151\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_149\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_176\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_183\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_192\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_195\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_150\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/N_4117\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/N_4114\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/N_4118\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_23_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_22_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_21_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_20_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_19_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_18_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_17_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_16_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_29_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_28_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        \can_top_level_inst/memory_registers_inst/tx_command_rep1[2]\, 
        \can_top_level_inst/memory_registers_inst/tx_command_fast[2]\, 
        \can_top_level_inst/memory_registers_inst/tx_command[2]\, 
        \can_top_level_inst/memory_registers_inst/tx_command[1]\, 
        \can_top_level_inst/memory_registers_inst/tx_command[0]\, 
        \can_top_level_inst/memory_registers_inst/command[10]\, 
        \can_top_level_inst/memory_registers_inst/command_i[9]\, 
        \can_top_level_inst/memory_registers_inst/ctr_pres[10]\, 
        \can_top_level_inst/memory_registers_inst/ctr_pres[9]\, 
        \can_top_level_inst/memory_registers_inst/status_comb_2_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/status_comb_3_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/status_comb_4_1_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/status_comb_4_1_0_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/status_comb_4_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[31]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[30]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[29]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[28]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[27]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[26]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[25]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[24]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[23]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[22]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[21]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[20]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[19]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[18]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[17]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[16]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[15]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[14]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[13]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[12]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[11]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[10]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[9]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[8]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[7]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[6]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[5]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[4]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[3]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[2]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[1]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[0]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[31]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[30]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[29]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[28]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[27]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[26]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[25]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[24]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[23]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[22]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[21]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[20]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[19]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[18]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[17]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[16]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[15]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[14]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[13]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[12]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[11]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[10]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[9]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[8]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[7]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[6]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[5]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[4]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[3]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[2]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[1]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[0]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[31]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[30]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[29]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[28]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[27]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[26]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[25]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[24]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[23]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[22]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[21]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[20]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[19]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[18]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[17]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[16]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[15]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[14]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[13]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[12]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[11]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[10]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[9]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[8]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[7]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[6]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[5]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[4]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[3]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[2]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[1]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[0]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[31]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[30]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[29]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[28]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[27]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[26]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[25]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[24]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[23]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[22]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[21]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[20]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[19]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[18]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[17]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[16]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[15]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[14]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[13]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[12]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[11]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[10]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[9]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[8]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[7]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[6]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[5]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[4]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[3]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[2]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[1]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[31]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[30]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[29]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[25]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[24]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[18]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[17]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[15]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[11]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[10]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[7]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[0]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[19]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[18]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[17]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[16]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[15]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[14]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[13]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[12]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[11]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[10]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[9]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[8]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[7]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[6]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[5]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[4]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[3]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[2]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[1]\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata[0]\, 
        \can_top_level_inst/memory_registers_inst/mode[9]\, 
        \can_top_level_inst/memory_registers_inst/mode_i[0]\, 
        \can_top_level_inst/memory_registers_inst/addr_dec_3[1]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        \can_top_level_inst/memory_registers_inst/un3_test_registers_cs_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        \can_top_level_inst/memory_registers_inst/tx_double_parity_error_Z\, 
        \can_top_level_inst/memory_registers_inst/tx_double_parity_error_1_sqmuxa_i_Z\, 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_Z\, 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_i_Z\, 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_0\, 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_1\, 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_2\, 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_3\, 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_4\, 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_5\, 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_6\, 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_7\, 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_8\, 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_0\, 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_1\, 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_2\, 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_3\, 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_4\, 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_5\, 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_6\, 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_7\, 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_8\, 
        \can_top_level_inst/memory_registers_inst/N_2466\, 
        \can_top_level_inst/memory_registers_inst/N_2474\, 
        \can_top_level_inst/memory_registers_inst/N_2534\, 
        \can_top_level_inst/memory_registers_inst/N_2542\, 
        \can_top_level_inst/memory_registers_inst/N_2476\, 
        \can_top_level_inst/memory_registers_inst/N_2544\, 
        \can_top_level_inst/memory_registers_inst/N_2489\, 
        \can_top_level_inst/memory_registers_inst/N_2557\, 
        \can_top_level_inst/memory_registers_inst/N_2485\, 
        \can_top_level_inst/memory_registers_inst/N_2553\, 
        \can_top_level_inst/memory_registers_inst/N_2481\, 
        \can_top_level_inst/memory_registers_inst/N_2549\, 
        \can_top_level_inst/memory_registers_inst/N_2486\, 
        \can_top_level_inst/memory_registers_inst/N_2554\, 
        \can_top_level_inst/memory_registers_inst/N_2480\, 
        \can_top_level_inst/memory_registers_inst/N_2548\, 
        \can_top_level_inst/memory_registers_inst/N_2479\, 
        \can_top_level_inst/memory_registers_inst/N_2547\, 
        \can_top_level_inst/memory_registers_inst/N_2527\, 
        \can_top_level_inst/memory_registers_inst/N_2459\, 
        \can_top_level_inst/memory_registers_inst/N_2477\, 
        \can_top_level_inst/memory_registers_inst/N_2545\, 
        \can_top_level_inst/memory_registers_inst/N_2469\, 
        \can_top_level_inst/memory_registers_inst/N_2533\, 
        \can_top_level_inst/memory_registers_inst/N_2465\, 
        \can_top_level_inst/memory_registers_inst/N_2463\, 
        \can_top_level_inst/memory_registers_inst/N_2535\, 
        \can_top_level_inst/memory_registers_inst/N_2467\, 
        \can_top_level_inst/memory_registers_inst/N_2552\, 
        \can_top_level_inst/memory_registers_inst/N_2550\, 
        \can_top_level_inst/memory_registers_inst/N_2546\, 
        \can_top_level_inst/memory_registers_inst/N_2543\, 
        \can_top_level_inst/memory_registers_inst/N_2538\, 
        \can_top_level_inst/memory_registers_inst/N_2536\, 
        \can_top_level_inst/memory_registers_inst/N_2532\, 
        \can_top_level_inst/memory_registers_inst/N_2531\, 
        \can_top_level_inst/memory_registers_inst/N_2530\, 
        \can_top_level_inst/memory_registers_inst/N_2529\, 
        \can_top_level_inst/memory_registers_inst/N_2528\, 
        \can_top_level_inst/memory_registers_inst/N_2484\, 
        \can_top_level_inst/memory_registers_inst/N_2482\, 
        \can_top_level_inst/memory_registers_inst/N_2478\, 
        \can_top_level_inst/memory_registers_inst/N_2475\, 
        \can_top_level_inst/memory_registers_inst/N_2470\, 
        \can_top_level_inst/memory_registers_inst/N_2468\, 
        \can_top_level_inst/memory_registers_inst/N_2464\, 
        \can_top_level_inst/memory_registers_inst/N_2462\, 
        \can_top_level_inst/memory_registers_inst/N_2461\, 
        \can_top_level_inst/memory_registers_inst/N_2460\, 
        \can_top_level_inst/memory_registers_inst/N_2555\, 
        \can_top_level_inst/memory_registers_inst/N_2551\, 
        \can_top_level_inst/memory_registers_inst/N_2540\, 
        \can_top_level_inst/memory_registers_inst/N_2537\, 
        \can_top_level_inst/memory_registers_inst/N_2487\, 
        \can_top_level_inst/memory_registers_inst/N_2483\, 
        \can_top_level_inst/memory_registers_inst/N_2472\, 
        \can_top_level_inst/memory_registers_inst/N_2556\, 
        \can_top_level_inst/memory_registers_inst/N_2539\, 
        \can_top_level_inst/memory_registers_inst/N_2526\, 
        \can_top_level_inst/memory_registers_inst/N_2488\, 
        \can_top_level_inst/memory_registers_inst/N_2471\, 
        \can_top_level_inst/memory_registers_inst/N_2458\, 
        \can_top_level_inst/memory_registers_inst/N_2541\, 
        \can_top_level_inst/memory_registers_inst/N_2473\, 
        \can_top_level_inst/memory_registers_inst/N_2629\, 
        \can_top_level_inst/memory_registers_inst/N_2632\, 
        \can_top_level_inst/memory_registers_inst/N_2643\, 
        \can_top_level_inst/memory_registers_inst/N_2647\, 
        \can_top_level_inst/memory_registers_inst/N_2644\, 
        \can_top_level_inst/memory_registers_inst/N_2634\, 
        \can_top_level_inst/memory_registers_inst/N_2630\, 
        \can_top_level_inst/memory_registers_inst/N_2654\, 
        \can_top_level_inst/memory_registers_inst/N_2652\, 
        \can_top_level_inst/memory_registers_inst/N_2655\, 
        \can_top_level_inst/memory_registers_inst/N_2624\, 
        \can_top_level_inst/memory_registers_inst/N_2648\, 
        \can_top_level_inst/memory_registers_inst/N_2645\, 
        \can_top_level_inst/memory_registers_inst/N_2626\, 
        \can_top_level_inst/memory_registers_inst/N_2631\, 
        \can_top_level_inst/memory_registers_inst/N_2637\, 
        \can_top_level_inst/memory_registers_inst/N_2642\, 
        \can_top_level_inst/memory_registers_inst/N_2639\, 
        \can_top_level_inst/memory_registers_inst/N_2650\, 
        \can_top_level_inst/memory_registers_inst/N_2633\, 
        \can_top_level_inst/memory_registers_inst/N_2638\, 
        \can_top_level_inst/memory_registers_inst/N_2641\, 
        \can_top_level_inst/memory_registers_inst/N_2653\, 
        \can_top_level_inst/memory_registers_inst/N_2628\, 
        \can_top_level_inst/memory_registers_inst/N_2625\, 
        \can_top_level_inst/memory_registers_inst/N_2649\, 
        \can_top_level_inst/memory_registers_inst/N_2635\, 
        \can_top_level_inst/memory_registers_inst/N_2627\, 
        \can_top_level_inst/memory_registers_inst/N_2651\, 
        \can_top_level_inst/memory_registers_inst/N_2636\, 
        \can_top_level_inst/memory_registers_inst/N_2640\, 
        \can_top_level_inst/memory_registers_inst/N_2646\, 
        \can_top_level_inst/memory_registers_inst/N_4605\, 
        \can_top_level_inst/memory_registers_inst/un3_test_registers_cs_2_Z\, 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_2_Z\, 
        \can_top_level_inst/memory_registers_inst/un9_status_comb_Z\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        \can_top_level_inst/memory_registers_inst/N_2704\, 
        \can_top_level_inst/memory_registers_inst/N_2689\, 
        \can_top_level_inst/memory_registers_inst/N_2702\, 
        \can_top_level_inst/memory_registers_inst/N_2719\, 
        \can_top_level_inst/memory_registers_inst/N_2703\, 
        \can_top_level_inst/memory_registers_inst/N_2714\, 
        \can_top_level_inst/memory_registers_inst/N_2718\, 
        \can_top_level_inst/memory_registers_inst/N_2691\, 
        \can_top_level_inst/memory_registers_inst/N_2692\, 
        \can_top_level_inst/memory_registers_inst/N_2694\, 
        \can_top_level_inst/memory_registers_inst/N_2695\, 
        \can_top_level_inst/memory_registers_inst/N_2699\, 
        \can_top_level_inst/memory_registers_inst/N_2706\, 
        \can_top_level_inst/memory_registers_inst/N_2709\, 
        \can_top_level_inst/memory_registers_inst/N_2713\, 
        \can_top_level_inst/memory_registers_inst/N_2715\, 
        \can_top_level_inst/memory_registers_inst/N_2698\, 
        \can_top_level_inst/memory_registers_inst/N_2696\, 
        \can_top_level_inst/memory_registers_inst/N_2701\, 
        \can_top_level_inst/memory_registers_inst/N_2693\, 
        \can_top_level_inst/memory_registers_inst/N_2716\, 
        \can_top_level_inst/memory_registers_inst/N_2700\, 
        \can_top_level_inst/memory_registers_inst/N_2717\, 
        \can_top_level_inst/memory_registers_inst/N_2712\, 
        \can_top_level_inst/memory_registers_inst/N_2708\, 
        \can_top_level_inst/memory_registers_inst/N_2690\, 
        \can_top_level_inst/memory_registers_inst/N_2710\, 
        \can_top_level_inst/memory_registers_inst/N_2711\, 
        \can_top_level_inst/memory_registers_inst/N_2720\, 
        \can_top_level_inst/memory_registers_inst/N_2707\, 
        \can_top_level_inst/memory_registers_inst/N_2705\, 
        \can_top_level_inst/memory_registers_inst/N_2697\, 
        \can_top_level_inst/memory_registers_inst/N_2671\, 
        \can_top_level_inst/memory_registers_inst/N_2656\, 
        \can_top_level_inst/memory_registers_inst/N_2669\, 
        \can_top_level_inst/memory_registers_inst/N_2686\, 
        \can_top_level_inst/memory_registers_inst/N_2670\, 
        \can_top_level_inst/memory_registers_inst/N_2681\, 
        \can_top_level_inst/memory_registers_inst/N_2685\, 
        \can_top_level_inst/memory_registers_inst/N_2658\, 
        \can_top_level_inst/memory_registers_inst/N_2659\, 
        \can_top_level_inst/memory_registers_inst/N_2662\, 
        \can_top_level_inst/memory_registers_inst/N_2666\, 
        \can_top_level_inst/memory_registers_inst/N_2673\, 
        \can_top_level_inst/memory_registers_inst/N_2676\, 
        \can_top_level_inst/memory_registers_inst/N_2680\, 
        \can_top_level_inst/memory_registers_inst/N_2682\, 
        \can_top_level_inst/memory_registers_inst/N_2665\, 
        \can_top_level_inst/memory_registers_inst/N_2661\, 
        \can_top_level_inst/memory_registers_inst/N_2663\, 
        \can_top_level_inst/memory_registers_inst/N_2668\, 
        \can_top_level_inst/memory_registers_inst/N_2660\, 
        \can_top_level_inst/memory_registers_inst/N_2683\, 
        \can_top_level_inst/memory_registers_inst/N_2667\, 
        \can_top_level_inst/memory_registers_inst/N_2684\, 
        \can_top_level_inst/memory_registers_inst/N_2679\, 
        \can_top_level_inst/memory_registers_inst/N_2675\, 
        \can_top_level_inst/memory_registers_inst/N_2657\, 
        \can_top_level_inst/memory_registers_inst/N_2677\, 
        \can_top_level_inst/memory_registers_inst/N_2678\, 
        \can_top_level_inst/memory_registers_inst/N_2687\, 
        \can_top_level_inst/memory_registers_inst/N_2674\, 
        \can_top_level_inst/memory_registers_inst/N_2672\, 
        \can_top_level_inst/memory_registers_inst/N_2664\, 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        \can_top_level_inst/memory_registers_inst/rx_data_read\, 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_5_Z\, 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_0_Z\, 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_3_Z\, 
        \can_top_level_inst/memory_registers_inst/tx_double_parity_error_1_sqmuxa_1_0_Z\, 
        \can_top_level_inst/memory_registers_inst/un2_res_core_d_n_i\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2_0_0[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/reg_sel_2[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode[11]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/access_active_1[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/masked_data[7]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_0[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_1[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_50_0_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_53_0_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_104\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_113\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_128\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_2_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/NN_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_settings_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_settings_reg_comp/wr_select_0_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_1_0_0_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_0_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_data_access_signaller_comp/access_active_0_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_data_access_signaller_comp/be_active_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[31]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[30]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[29]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[25]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[24]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[18]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[17]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[15]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[11]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[10]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_wmux_1_Y[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_1_y0[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_wmux_0_Y[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_2_y0[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_wmux_0_Y[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_2_y0[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_1_wmux_0_Y[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_1_1_y0[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_wmux_0_Y[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_2_y0[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_37_1_wmux_0_Y[18]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_37_1_0_y0[18]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_10_1_0_y0[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_34_1_0_y0[17]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_28_1_0_y0[11]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_6_1_0_y0[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_6_1_0_y0[18]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_21_1_0_y0[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_34_1_0_y0[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_15_1_0_y0[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_38_1_0_y0[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_30_1_0_y0[17]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_38_1_0_y0[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_15_1_0_y0[10]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_38_1_0_y0[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_38_1_0_y0[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_30_1_0_y0[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_15_1_0_y0[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_38_1_0_y0[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_38_1_0_y0[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_38_1_0_y0[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_38_1_0_y0[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_38_1_0_y0[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_38_1_0_y0[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_38_1_0_y0[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_38_1_0_y0[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_38_1_0_y0[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11[10]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_38_1_0_y0[10]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_38_1_0_y0[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18[17]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_38_1_0_y0[17]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_0_y1[11]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_0_y3[11]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_y0_0[11]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_0_y0[11]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_0_y1[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_0_y3[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_y0_0[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_0_y0[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_0_y1[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_0_y3[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_y0_0[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_0_y0[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_7_2_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_7_2_0_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_0_y1[25]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_0_y3[25]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_y0_0[25]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_0_y0[25]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_0_y1[24]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_0_y3[24]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_y0_0[24]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_0_y0[24]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_0_y1[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_0_y3[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_y0_0[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_0_y0[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_0_y1[18]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_0_y3[18]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_y0_0[18]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_0_y0[18]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_0_y1[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_0_y3[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_y0_0[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_0_y0[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_0_y1[15]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_0_y3[15]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_y0_0[15]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_0_y0[15]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[24]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[1214]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[1211]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[1210]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[926]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[922]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[895]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[894]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[892]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[668]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[667]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[666]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[634]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[446]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[443]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_37_2_0_Z[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_37_2_0_Z[18]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_37_2_0_Z[11]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_37_1_Z[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_1_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_37_1_Z[11]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_0_1_Z[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_2_0_1_Z[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_2_1_1[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_2_1[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_2_1_1[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_1_0_Z[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_2_0[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_1[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_1_1_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_0_1_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_0_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_1_0_1_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_1_0_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_1_0_1_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_0_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_1_Z[29]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_1_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_15_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_31_1_Z[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_2_0_Z[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_2_Z[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_7_Z[29]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_12_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_17_1_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_8_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_17_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_2_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_3_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_10_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_11_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_0_Z[29]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_4_1_Z[29]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_4_Z[29]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_31_1_1_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_2_0_1_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_31_1_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_2_1[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_0_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_1_1_Z[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_1_Z[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_2_1_Z[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_1_1_Z[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_1_Z[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_7_2_0_1_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_1_1_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_1_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_1[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_1_0_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_0_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_1_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_2_1[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_2_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_0_1_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_0_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_1_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_2_1[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_2_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_0_1_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_0_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_2_1[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_1_0_1_Z[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_37_2_1_1_0[15]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_37_2_1_1[15]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_37_1_0_Z[15]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_37_2_1_1[25]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_37_2_1_2[25]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_37_2_1_1[24]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_37_2_1_2[24]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_37_2_1_1_0[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_37_2_1_1[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_37_1_0_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_2_1_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_1[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_0_1_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_1_1_Z[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_1_Z[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_1[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_Z[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_1_0_Z[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_0_Z[5]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_1_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_2_1[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_2_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_0_1_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_0_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_1_1_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_1_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_2_0_1_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_1_0_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_2_0_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1_Z[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_2_1[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_2_Z[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1_0_1_Z[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1_0_Z[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_1_Z[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_Z[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_0_1_Z[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_0_Z[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_2_0_1_Z[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_2_0_Z[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_1_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_0_1_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_0_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_2_0_1_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_2_0_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_1_1_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_1_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_2_0_1_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_1_0_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_2_0_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1_1_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_1[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1_0_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_0_1_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_0_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_1_1_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_1_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_1[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_1_Z[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_Z[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_2_1[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_0_1_Z[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_0_Z[8]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_1_Z[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_Z[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_2_1[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_2_Z[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_0_1_Z[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_0_Z[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_1_Z[17]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_Z[17]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_0_1_Z[17]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_0_Z[17]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_1_Z[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_Z[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_1[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_Z[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_0_1_Z[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_0_Z[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_0_1_Z[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_0_Z[19]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_1_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_2_1[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_1_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_2_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_1_Z[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_Z[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1_1_Z[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1_Z[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_1[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_Z[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1_0_Z[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_0_1_Z[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_0_Z[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1_Z[10]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_2_1[10]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_2_Z[10]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1_0_1_Z[10]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1_0_Z[10]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_1_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_1[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_0_1_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_0_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_0_1_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_0_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_2_1[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_2_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1_0_1_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1_0_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_1_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_2_1[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_2_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_0_1_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_0_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_1_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_0_1_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_0_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_2_0_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_2_0_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_2_0_Z[10]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_2_Z[4]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_2_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_2_0_Z[17]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_2_Z[17]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_2_0_Z[9]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_2_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_2_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_2_Z[16]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_2_0_Z[6]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_2_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_2_0_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_37_2_Z[11]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_2_0_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_2_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_1_0_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_2_0_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_2_0_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_37_2_Z[12]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_6_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_0_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_3_Z[30]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_0_Z[31]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m_0[890]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_6_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_5_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_0_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_12_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_7_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_6_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_4_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_2_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_11_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_10_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_4_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_0_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_5_Z[30]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_1_Z[30]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_3_Z[29]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_5_Z[31]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_3_Z[31]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_11_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_8_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_8_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_5_Z[29]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_2_Z[31]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_10_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_9_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_9_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_2_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_4_Z[30]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_16_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_15_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_12_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_7_Z[30]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_7_Z[31]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1304\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1305\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1301\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1302\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1213\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1214\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1210\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1211\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1228\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1229\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1225\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1226\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1478\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_816\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1572\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1360\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_826\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1261\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_4991\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1483\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1481\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1471\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1474\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1198\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1200\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1179\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1194\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_812\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_807\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_810\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_800\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_803\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1160\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1162\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1141\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1156\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1521\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1516\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1519\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1509\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1512\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1312\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1314\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1293\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1308\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1236\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1238\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1217\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1232\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_774\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_769\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_772\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_762\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_765\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1369\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1364\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1367\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1357\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_930\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_932\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_911\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_926\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1006\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1008\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_987\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1002\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_892\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_894\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_873\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_888\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_702\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_704\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_683\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_698\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_968\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_970\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_949\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_964\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1044\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1046\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1025\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1040\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1350\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1352\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1331\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1346\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1502\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1504\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1498\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_778\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_780\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_759\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1540\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1542\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1536\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1390\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1384\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_818\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_797\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1575\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1569\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1562\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1565\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1554\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1557\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1547\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1550\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1152\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1153\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1149\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1150\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1145\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1146\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1142\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1143\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1271\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1265\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1268\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1258\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1117\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1111\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1114\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1104\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1107\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1096\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1099\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1089\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1092\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1079\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1073\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1076\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1066\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1069\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1058\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1061\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1051\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1054\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_606\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_851\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_845\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_848\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_838\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_841\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_830\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_833\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_823\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1452\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_737\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_731\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_734\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_724\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_727\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_716\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_719\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_709\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_712\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac19_0_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac1_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac37_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac21_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7_2_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_617\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_622\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_886\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_694\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac11_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1460_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1461\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac8_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_661\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_619\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_743\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1123\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1119\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1085\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1081\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_667\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_648\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_655\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1175\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1176\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1190\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1191\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1181\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1180\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1533\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1532\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1523\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1522\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1287\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1286\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1297\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1298\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1204\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1203\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1221\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1222\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_793\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_794\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_787\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_786\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1495\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1494\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1441\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1442\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1135\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1134\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_981\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_980\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_977\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_976\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_901\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_900\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_749\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_748\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1381\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1380\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1371\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1370\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_943\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_942\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_939\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_938\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_962\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1581\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1277\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_924\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1000\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_857\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_629\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac22_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac19_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_0_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac1_0_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac20_1_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5_2_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac21_1_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1463\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac4_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_0_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac2_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_623\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_778_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1236_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1238_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1006_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1554_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1092_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1162_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1200_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_780_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1044_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1540_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1160_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1466_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1198_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1474_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_704_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_930_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_762_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1512_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1350_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1562_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1550_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1455_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_968_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1502_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1516_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1352_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1314_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1054_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_892_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1312_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1565_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac16_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac17_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac20_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac28_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac23_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac37_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac29_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac25_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1466\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1364_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1106\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1068\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_800_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_838_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1058_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1467\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1458\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1268_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_833_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_655_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_727_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1557_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1519_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1455\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_765_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1481_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1367_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_1_2[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_1_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_1_1_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_1_2_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_20_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i79_mux\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i81_mux\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m19_1_0_y0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i69_mux\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i71_mux\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_37_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_26_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_35_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m36_1_0_y0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_17_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m112_2_1_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m103_2_1_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_139_mux\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m34_1_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m25_1_1_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m25_1_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m127_2_1_0_Z\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i77_mux\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i75_mux\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_13_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_16_0_2\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_16_0_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_65_1\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_140\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_39_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_65\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_42_0\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_1_0_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_Z\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[15]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[14]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[13]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[12]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[11]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[10]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[9]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[8]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[7]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_1_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_0_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_control_reg_comp/wr_select_0_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_control_reg_comp/wr_select_4_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[31]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[30]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[29]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[28]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[27]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[26]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[25]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[24]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[23]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[22]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[21]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[20]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[19]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[18]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[17]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[16]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[15]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[12]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[11]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[10]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[9]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[8]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[7]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[6]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[5]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[4]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_1[0]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_1_3_1_0_y0[0]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_2[1]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_2_3_1_0_y0[1]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[8]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[19]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[18]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[17]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[16]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[15]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[14]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[13]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[12]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[11]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[10]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[9]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[8]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[7]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[6]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[5]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[4]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[3]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[24]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[16]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[31]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[30]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[29]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[28]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[27]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[26]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[25]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[24]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[23]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[22]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[21]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[20]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[2]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[1]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_1_2_Z[0]\, 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_1_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/N_3899\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un7_txtb_parity_error_valid_i_1_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_2_1_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_0_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_126_i\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_124_i\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_193\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_147\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_195\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_148\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_149\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_151\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_176_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_172\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_177\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_173\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_170\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_150\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_168\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/N_4157\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/N_4154\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/N_4158\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_23_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_22_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_21_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_20_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_19_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_18_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_17_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_16_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_29_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_28_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        \can_top_level_inst/rst_sync_inst/rff_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[7]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[6]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[5]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[4]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[3]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[2]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[1]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[0]\, 
        \can_top_level_inst/bus_sampling_inst/tx_trigger_q\, 
        \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp_Z\, 
        \can_top_level_inst/bus_sampling_inst/un1_ssp_enable_Z\, 
        \can_top_level_inst/bus_sampling_inst/edge_tx_valid\, 
        \can_top_level_inst/bus_sampling_inst/rx_data_prev\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_prev\, 
        \can_top_level_inst/bus_sampling_inst/sample_sec\, 
        \can_top_level_inst/bus_sampling_inst/data_tx_delayed\, 
        \can_top_level_inst/bus_sampling_inst/reg_q\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[14]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[13]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[12]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[11]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[10]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[9]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[8]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[7]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[6]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[5]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[4]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[3]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[2]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[1]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[0]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[7]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[6]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[5]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[4]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[3]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[2]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[1]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[0]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[14]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[13]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[12]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[11]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[10]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[9]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[8]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[7]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[6]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[5]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[4]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[3]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[2]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[1]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[0]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[14]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[13]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[12]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[11]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[10]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[9]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[8]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[7]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[6]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[5]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[4]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[3]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[2]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[1]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[0]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[13]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[12]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[11]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[10]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[9]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[8]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[7]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[6]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[5]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[4]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[3]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[2]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[1]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[0]\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_11_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_10_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_9_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_8_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_7_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_6_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_5_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_4_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_3_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_2_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_1_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_0_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_meas_running_q_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un7_btmc_meas_running_d_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un12_btmc_meas_running_d_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_ena_q_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_first_ssp_d_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un6_sspc_ena_d_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un6_first_ssp_d_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_s_14_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_13_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_12_S\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry_cy\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_0_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_1_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_2_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_3_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_4_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_5_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_6_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_7_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_8_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_9_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_10_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_11_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_12_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_13_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_14_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_s_0_2645_FCO\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_0_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_1_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_2_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_3_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_4_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_5_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_6_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_7_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_8_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_9_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_10_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_11_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_12_Z\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_13_Z\, 
        \can_top_level_inst/bus_sampling_inst/sample_mux_inst/prev_sample_q_2\, 
        \can_top_level_inst/bus_sampling_inst/shift_regs_rst_reg_inst/rx_shift_res_reg_inst/ssp_reset_i\, 
        \can_top_level_inst/bus_sampling_inst/can_rx_sig_sync_inst/rff_Z\, 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_d_Z\, 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_ssp_capt_q_Z\, 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/un4_bit_err_ssp_capt_d_Z\, 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_d_1_Z\, 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/un6_bit_err_ssp_valid_Z\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[2]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[1]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[0]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_2[2]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_2[1]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_2[0]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[2]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[1]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_3[2]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_3[1]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_3[0]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_0_y1[0]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_0_y3[0]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_y0_0[0]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_0_y0[0]\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram7_0\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe7\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram6_0\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe6\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram5_0\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe5\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram4_0\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe4\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram3_0\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe3\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram2_0\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe2\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram1_0\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe1\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram0_0\, 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe0\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[7]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[6]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[5]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[4]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[3]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[2]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[1]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[0]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[6]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[5]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[4]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[3]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[2]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[1]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[0]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNO_S[6]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI2E6D1_S[5]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIA6981_S[4]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIJVB31_S[3]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNITPEU_S[2]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI8LHP_S[1]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIKHKK_S[0]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNID7LB_Y[0]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNISGAN_S[1]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIDSV21_S[2]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI0ALE1_S[3]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNILPAQ1_S[4]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNICB062_S[5]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI5VLH2_S[6]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI6BEO2_S[6]\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_38\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_39\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_40\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_41\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_42\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_43\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_36\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_37\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_meas_progress_del_Z\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_meas_progress_q_Z\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/reg_q\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_rst_d_Z\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un10_trv_meas_progress_d_Z\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_1_i\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_0_cy\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_d_0_Z\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_d_4_Z\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_0\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_1\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_2\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_3\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_4\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_5\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_0\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_1\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_2\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_3\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_4\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_5\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_6\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_7\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_rst_d_i\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/N_3901\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_1_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_141\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_162\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_167\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_139\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_0_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_182\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_164\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_168\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_154\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/N_4197\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/N_4194\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/N_4198\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_23_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_22_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_21_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_20_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_19_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_18_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_17_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_16_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_29_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_28_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[7]\, 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[6]\, 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[5]\, 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[4]\, 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[3]\, 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[2]\, 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[1]\, 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[0]\, 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[6]\, 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[5]\, 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[4]\, 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[3]\, 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[2]\, 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[1]\, 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[0]\, 
        \can_top_level_inst/prescaler_inst/current_state[1]\, 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_a_4[7]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[8]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[7]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[6]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[5]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[4]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[3]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[2]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[1]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[0]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[8]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[7]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[6]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[5]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[4]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[3]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[2]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[1]\, 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[0]\, 
        \can_top_level_inst/prescaler_inst/start_edge\, 
        \can_top_level_inst/prescaler_inst/is_tseg1\, 
        \can_top_level_inst/prescaler_inst/N_5058\, 
        \can_top_level_inst/prescaler_inst/g0_0\, 
        \can_top_level_inst/prescaler_inst/is_tseg2\, 
        \can_top_level_inst/prescaler_inst/resync_edge_valid\, 
        \can_top_level_inst/prescaler_inst/g0_0_3\, 
        \can_top_level_inst/prescaler_inst/g0_0_2\, 
        \can_top_level_inst/prescaler_inst/g1\, 
        \can_top_level_inst/prescaler_inst/g1_0\, 
        \can_top_level_inst/prescaler_inst/g0_0_0\, 
        \can_top_level_inst/prescaler_inst/sync_flag\, 
        \can_top_level_inst/prescaler_inst/un18_sel_tseg1_2230\, 
        \can_top_level_inst/prescaler_inst/segment_end\, 
        \can_top_level_inst/prescaler_inst/g0_0_5\, 
        \can_top_level_inst/prescaler_inst/un8_exit_segm_regular_tseg1\, 
        \can_top_level_inst/prescaler_inst/exit_segm_req_nbt_1\, 
        \can_top_level_inst/prescaler_inst/N_7\, 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE_5\, 
        \can_top_level_inst/prescaler_inst/N_8\, 
        \can_top_level_inst/prescaler_inst/g0_0_7\, 
        \can_top_level_inst/prescaler_inst/un6_phase_err_sjw_by_one_NE\, 
        \can_top_level_inst/prescaler_inst/g1_0_0\, 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, 
        \can_top_level_inst/prescaler_inst/is_tseg2_fast\, 
        \can_top_level_inst/prescaler_inst/un7_exit_segm_regular_cry_9\, 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        \can_top_level_inst/prescaler_inst/exit_segm_req_dbt\, 
        \can_top_level_inst/prescaler_inst/un5_exit_ph2_immediate_0\, 
        \can_top_level_inst/prescaler_inst/un8_exit_segm_regular_tseg1_0\, 
        \can_top_level_inst/prescaler_inst/un7_exit_segm_regular_cry_9_0\, 
        \can_top_level_inst/prescaler_inst/bt_ctr_clear\, 
        \can_top_level_inst/prescaler_inst/N_27\, 
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/tx_trig_req_flag_q_Z\, 
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/un9_tx_trig_req_flag_d_Z\, 
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/tx_trigger_0_1_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[8]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[0]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[0]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[0]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s_Z[8]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s_Z[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_RNIG6M52_Z[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_segm_counter_ce_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s_2641_FCO\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s_2642_FCO\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un2_tq_counter_allowlto7_4_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_m1_0_a2_1\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c4_i\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_NE_0\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_NE_1\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c5_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_NE_2\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/N_3910\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_NE_4\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/N_3913\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[8]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[0]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[0]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[0]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s_Z[8]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[6]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[5]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[4]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[3]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[2]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[1]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s_Z[7]\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_segm_counter_ce_0\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un1_tq_counter_ce_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s_2639_FCO\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s_2640_FCO\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un10_tq_counter_d_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/g1_0_0\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/g1_4_0\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/g0_i_a3_4\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un2_tq_counter_allow\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c4_i\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_NE_5_0\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_NE_5_1\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c5_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_NE_5_2\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/N_3920\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/N_3918\, 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/N_3921\, 
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/un1_h_or_re_sync_edge_Z\, 
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/un6_sync_flag_ce_Z\, 
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/g0_0_0_0_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[9]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[8]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[7]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[6]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[5]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[4]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[3]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[2]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[1]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_i[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[9]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[8]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[7]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[6]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[5]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[4]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[3]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[2]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[1]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[9]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[8]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[7]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[6]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[5]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[4]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[3]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[2]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[1]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[9]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[8]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[7]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[6]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[5]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[4]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[3]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[2]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[1]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[8]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[7]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[6]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[5]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[4]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[3]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[2]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[1]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[4]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[3]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[2]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[1]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_Z[6]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_Z[5]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_49\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_50\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_2\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_51\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_3\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_52\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_4\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_53\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_5\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_h_sync_valid_2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_6\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_7\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_8\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_0_cy_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_2\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_3\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_4\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_5\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_6\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_7\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_8\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_2\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_2\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_3\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_3\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_4\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_4\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_5\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_5\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_6\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_6\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_7\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_7\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_9\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_8\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_8\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_0_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_1_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_3_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_4_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_5_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_6_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_7_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_0_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_1_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_3_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_4_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_5_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_6_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_7_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_8_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_s_0_2636_FCO\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_2\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_3\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_4\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_5_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_6_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_7_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_4_1_sx_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_4_1_0_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g1_2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_2_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_sjw_mt_zero_0_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/CO2\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_5_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_1_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE_4_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE_0_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_4_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_shorten_tseg1_after_tseg2_i\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_1_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_2_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segm_end_req_capt_q_Z[2]\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segm_end_req_capt_q_Z[1]\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segment_end_i\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_req_capt_ce\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un13_segm_end_req_capt_ce\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g1_0\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_nbt_tq_active\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_0_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_12_1_sx\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_12_1_0_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_8_x_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_sx_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_6_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_1\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_N_3L4_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_10_N_3L3_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_1_0\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_10_N_2L1_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g2_0_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_RNIOH58E_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/N_4\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_8_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_1\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g1_0_0_0\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un3_segment_end_i_0_1_N_2L1_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segment_end_1\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_3_Z\, 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_4_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[9]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[8]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[7]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[6]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[5]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[4]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[3]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[2]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[1]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_i[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_Z[9]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_Z[8]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_Z[7]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_Z[6]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[5]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[4]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[3]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[2]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[1]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[9]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[8]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[7]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[6]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[5]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[4]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[3]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[2]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[1]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[9]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[8]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[7]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[6]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[5]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[4]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[3]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[2]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[1]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[8]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[7]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[6]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[5]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[4]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[3]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[2]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[1]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[5]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[4]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[3]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[2]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[1]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[0]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_Z[7]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_Z[6]\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_47\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_48\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_2\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_49\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_3\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_50\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_4\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_51\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_5\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_6\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_7\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_8\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_0_cy_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_2\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_3\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_4\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_5\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_6\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_7\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_8\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_2\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_2\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_3\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_3\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_4\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_4\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_5\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_5\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_6\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_6\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_7\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_7\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_9\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_8\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_8\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_0_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_1_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_3_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_4_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_5_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_6_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_7_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_0_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_1_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_3_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_4_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_5_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_6_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_7_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_8_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_s_0_2637_FCO\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_2\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_3\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_4\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_5\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_6_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_7_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_shorten_tseg1_after_tseg2_i\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g1_0_0_a3_0_4_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_0_1_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_sjw_mt_zero_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_0_3_0\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_4_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_3_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_1_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un4_use_basic_segm_length_i\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_3906\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/CO2\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_4_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/N_33_i_rep1\, 
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/N_33_i_fast\, 
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/N_33_i\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/drv_ena_reg_2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/drv_ena_reg_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_2_S\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_1_S\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_axb_0_i\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_5_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_5_S\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_4_S\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_3_S\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_2_S\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_1_S\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_axb_0_i\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_6_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_6_S\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_5_S\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_4_S\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_3_S\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_0_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_1_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_3_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_4_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_5_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_0_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_1_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_2_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_3_Z\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_4_Z\, 
        \can_top_level_inst/int_manager_inst/int_input_active_18_1_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_15_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_4_0_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_18_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_0_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_9_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_15_1_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_1_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_24_tz_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_11_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_5_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_24_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_16_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_13_0_Z[11]\, 
        \can_top_level_inst/int_manager_inst/int_input_active_17_0_Z[11]\, 
        \can_top_level_inst/int_manager_inst/un1_int_i_5_Z\, 
        \can_top_level_inst/int_manager_inst/un1_int_i_4_Z\, 
        \can_top_level_inst/int_manager_inst/un1_int_i_3_Z\, 
        \can_top_level_inst/int_manager_inst/un1_int_i_2_Z\, 
        \can_top_level_inst/int_manager_inst/un1_int_i_1_Z\, 
        \can_top_level_inst/int_manager_inst/un1_int_i_0_Z\, 
        \can_top_level_inst/int_manager_inst/un1_int_i_9_Z\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/int_mask_load_Z\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/un1_int_ena_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/un2_int_status_set\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/un1_int_status_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/int_mask_load_Z\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/un1_int_ena_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/un2_int_status_set\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/un1_int_status_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/int_mask_load_Z\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/un1_int_ena_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/un2_int_status_set\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/un1_int_status_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/int_mask_load_Z\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/un1_int_ena_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/un2_int_status_set\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/un1_int_status_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/int_mask_load_Z\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/un1_int_ena_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/un2_int_status_set\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/un1_int_status_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/int_mask_load_Z\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/un1_int_ena_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/un2_int_status_set\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/un1_int_status_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/int_mask_load_Z\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/un1_int_ena_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/un2_int_status_set\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/un1_int_status_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/int_mask_load_Z\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/un1_int_ena_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/un2_int_status_set\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/un1_int_status_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/un1_int_ena_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/un2_int_status_set\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/un1_int_status_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/int_mask_load_Z\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/int_mask_load_Z\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/un1_int_ena_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/un2_int_status_set\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/un1_int_status_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/int_mask_load_Z\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/un1_int_ena_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/un2_int_status_set\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/un1_int_status_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.7.int_module_inst/un1_int_ena_clear\, 
        \can_top_level_inst/int_manager_inst/int_module_gen.7.int_module_inst/int_mask_load_Z\, 
        \can_top_level_inst/int_manager_inst/dff_int_output_reg/un1_int_i_i\, 
        \can_top_level_inst/frame_filters_inst/ident_valid_d_Z\, 
        \can_top_level_inst/frame_filters_inst/un1_filter_b_enable_1_0_Z\, 
        \can_top_level_inst/frame_filters_inst/un1_filter_b_enable_Z\, 
        \can_top_level_inst/frame_filters_inst/un1_filter_range_enable_1_Z\, 
        \can_top_level_inst/frame_filters_inst/un1_filter_range_enable_0_Z\, 
        \can_top_level_inst/frame_filters_inst/un2_filter_a_enable_1_Z\, 
        \can_top_level_inst/frame_filters_inst/un2_filter_a_enable_0_Z\, 
        \can_top_level_inst/frame_filters_inst/un1_filter_c_enable_1_Z\, 
        \can_top_level_inst/frame_filters_inst/un1_filter_c_enable_0_Z\, 
        \can_top_level_inst/frame_filters_inst/int_filter_ran_valid\, 
        \can_top_level_inst/frame_filters_inst/valid_0_I_87_FCO\, 
        \can_top_level_inst/frame_filters_inst/un6_filter_result_0\, 
        \can_top_level_inst/frame_filters_inst/int_filter_C_valid\, 
        \can_top_level_inst/frame_filters_inst/valid_0_I_87_FCO_0\, 
        \can_top_level_inst/frame_filters_inst/un6_filter_result\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_0\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_1\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_2\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_3\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_4\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_5\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_6\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_7\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_8\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_9\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_10\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_11\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_12\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_13\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_14\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_15\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_16\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_17\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_18\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_19\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_20\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_21\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_22\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_23\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_24\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_25\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_26\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_27\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_0\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_1\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_2\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_3\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_4\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_5\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_6\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_7\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_8\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_9\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_10\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_11\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_12\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_13\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_14\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_15\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_16\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_17\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_18\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_19\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_20\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_21\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_22\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_23\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_24\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_25\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_26\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_27\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[13]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[12]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[11]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[10]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[9]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[8]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[7]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[6]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[5]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[4]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[3]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[2]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[1]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[0]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_72\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_32\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_61\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_56\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_11\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_51\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_41\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_36\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_7\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_67\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_21\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_16\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_26\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_46\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[13]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[12]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[11]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[10]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[9]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[8]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[7]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[6]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[5]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[4]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[3]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[2]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[1]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[0]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_72\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_32\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_61\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_56\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_11\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_51\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_42\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_36\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_6\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_67\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_21\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_16\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_26\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_46\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[13]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[12]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[11]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[10]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[9]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[8]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[7]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[6]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[5]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[4]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[3]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[2]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[1]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[0]\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_71\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_32\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_61\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_56\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_11\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_51\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_42\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_36\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_6\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_67\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_21\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_16\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_26\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_46\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_I_87_FCO_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/N_3900\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un7_txtb_parity_error_valid_i_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_1_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_126_i\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_124_i\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_147\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_148\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_163\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_149\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_151\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_183\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_170_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_192\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_195\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_174\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_170\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_150\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_168\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/N_4177\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/N_4174\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/N_4178\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_23_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_22_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_21_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_20_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_19_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_18_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_17_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_16_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_29_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_28_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/N_3902\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/txtb_parity_error_valid[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_141\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_182\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_154\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_178\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/N_4784\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/N_4214\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/wa4_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/N_4218\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_23_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_22_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_21_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_20_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_19_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_18_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_17_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_16_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_29_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_28_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/un41_txtb_pointer_meta_d_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/un41_txtb_pointer_meta_d_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf_Z[3]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[4]\, 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[3]\, 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[31]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[30]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[29]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[28]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[27]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[26]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[25]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[24]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[23]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[22]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[21]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[20]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[19]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[18]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[17]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[16]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[15]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[14]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[13]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[12]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[11]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[10]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[9]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[8]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[7]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[6]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[5]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[4]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[3]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/last_txtb_index_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/last_txtb_index_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/last_txtb_index_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[15]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[14]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[13]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[12]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[11]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[10]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[9]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[8]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[7]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[6]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[5]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[4]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[3]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[10]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[9]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[8]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[7]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[6]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[5]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[4]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[3]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[10]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[9]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[8]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[7]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[6]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[5]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[4]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[3]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[10]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[9]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[8]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[7]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[6]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[5]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[4]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[3]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[31]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[30]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[29]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[28]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[27]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[26]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[25]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[24]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[23]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[22]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[21]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[20]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[19]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[18]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[17]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[16]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[15]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[14]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[13]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[12]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[11]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[31]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[30]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[29]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[28]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[27]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[26]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[25]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[24]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[23]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[22]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[21]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[20]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[19]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[18]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[17]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[16]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[15]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[14]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[13]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[12]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[11]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[31]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[30]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[29]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[28]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[27]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[26]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[25]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[24]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[23]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[22]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[21]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[20]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[19]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[18]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[17]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[16]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[15]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[14]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[13]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[12]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[11]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[31]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[30]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[29]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[28]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[27]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[26]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[25]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[24]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[23]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[22]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[21]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[20]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[19]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[18]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[17]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[16]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[15]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[14]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[13]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[12]\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[11]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[7]\, 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_clk_en_q_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_ident_type_dbl_buf_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/buffer_md_w\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_is_rtr_dbl_buf_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/fcrc\, 
        \can_top_level_inst/tx_arbitrator_inst/load_ffmt_w_addr\, 
        \can_top_level_inst/tx_arbitrator_inst/fstc\, 
        \can_top_level_inst/tx_arbitrator_inst/sdlc\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_2_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_brs_dbl_buf_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_type_dbl_buf_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_valid_com_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_frame_valid_com_clear\, 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, 
        \can_top_level_inst/tx_arbitrator_inst/store_last_txtb_index\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_30\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_axb_31_i\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid\, 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_I_50\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_0\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_1\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_2\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_3\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_4\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_5\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_6\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_7\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_8\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_9\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_10\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_11\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_12\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_13\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_14\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_15\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_16\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_17\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_18\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_19\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_20\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_21\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_22\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_23\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_24\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_25\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_26\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_27\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_28\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_29\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_30\, 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_0\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_1\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_2\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_3\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_4\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_5\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_6\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_7\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_8\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_9\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_10\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_11\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_12\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_13\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_14\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_15\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_16\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_17\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_18\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_19\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_20\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_21\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_22\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_23\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_24\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_25\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_26\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_27\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_28\, 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_29\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_0_y1\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_0_y3\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_y0_0\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_0_y0\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_0_y1\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_0_y3\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_y0_0\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_0_y0\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_0_y1\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_0_y3\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_y0_0\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_0_y0\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i_1_0\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_m3_0_a3_0\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_N_7_mux_i_0_0\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i_1_1\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i_1\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_N_3_mux\, 
        \can_top_level_inst/tx_arbitrator_inst/r_N_5_mux\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_parity_check_valid_0_RNO_2_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/un2_select_index_changed_2_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/txtb_m1_e_2\, 
        \can_top_level_inst/tx_arbitrator_inst/un4_txtb_changed_NE_0_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/un2_select_index_changed_0\, 
        \can_top_level_inst/tx_arbitrator_inst/N_109\, 
        \can_top_level_inst/tx_arbitrator_inst/N_104\, 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        \can_top_level_inst/tx_arbitrator_inst/N_120\, 
        \can_top_level_inst/tx_arbitrator_inst/N_158\, 
        \can_top_level_inst/tx_arbitrator_inst/N_144\, 
        \can_top_level_inst/tx_arbitrator_inst/N_94\, 
        \can_top_level_inst/tx_arbitrator_inst/un1_l3_valid\, 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_09\, 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_19\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/select_buf_index_i_0_1[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3_Z[0]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/select_buf_index_i_0_i_1[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_i\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_i\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner1\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner3\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_m_3_1_0_y0\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner0\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner2\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_i\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_12\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_26_i\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_121\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_123\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_12_m_3_1_0_y0\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_120\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_122\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_36_i\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_02\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06_i\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_021\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_023\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_02_m_3_1_0_y0\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_020\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_022\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_16_i\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_RNI791Q4_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_RNIU99F7_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO2_1\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO2_1_0\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO2_1_1\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_1_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_i1_mux_2\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0_1_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0_0_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_i1_mux_0\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_0\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_06\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_1_2\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_i1_mux\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_0_1_1_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_2_1_1_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_1_1_1\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_i1_mux_1\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/output_i3_mux\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_i3_mux\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_m2_0_a2_1_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_0\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_1\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_39\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_19\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_2\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_3\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_4\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_5\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_6\, 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_7\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[6]\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[5]\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[4]\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[3]\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_Z[1]\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_0_1_Z[5]\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a0_0_Z[5]\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_0_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_1_1_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_1_2_Z[2]\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_91_i\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_89_i\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_87_i\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_85_i\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_83_i\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_81_i\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_78_i\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_100_i\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_111\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/un1_N_7_mux_i_0_1\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/fsm_wait_state_d_i_1_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_100_i_1\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_87_i_1\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_155\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_146\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_122\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_114\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_112\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_109_0\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/commit_m5_1\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_132\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_128\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_72\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/fsm_wait_state_d_i_0_Z\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/tx_arb_parity_check_valid_1\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_130\, 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_121\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_1_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/N_3903\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_2_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_152\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_141\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_167\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_160\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_139\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_163\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_1_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_168\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_164\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_179\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_158\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_155\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_182\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_153\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_165\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/N_4795\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/N_4234\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/N_4796\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_23_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_22_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_21_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_20_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_19_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_18_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_17_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_16_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_29_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_28_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_13_Z\, 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_23_Z\, 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_21_Z\, 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_20_Z\, 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_19_Z\, 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_18_Z\, 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_17_Z\, 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_16_Z\, 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_27_Z\, 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_28_Z\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[63]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[62]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[61]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[60]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[59]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[58]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[57]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[56]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[55]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[54]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[53]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[52]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[51]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[50]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[49]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[48]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[47]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[46]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[45]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[44]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[43]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[42]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[41]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[40]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[39]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[38]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[37]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[36]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[35]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[34]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[33]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[32]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI62SU1_S[3]\, 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI305F1_S[2]\, 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI1VDV_S[1]\, 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI0VMF_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNO_S[5]\, 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNIA5JE2_S[4]\, 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[6]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[5]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[4]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[3]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[2]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[1]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[0]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_0_2_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[6]\, 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[5]\, 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[4]\, 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[3]\, 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[2]\, 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[1]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[6]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[5]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[4]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[3]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[2]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[1]\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[0]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_0_0_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rwcnt_com[2]\, 
        \can_top_level_inst/rx_buffer_inst/frame_form_w_9[12]\, 
        \can_top_level_inst/rx_buffer_inst/frame_form_w_9[11]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/data_selector[0]\, 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_i_0\, 
        \can_top_level_inst/rx_buffer_inst/reg_q\, 
        \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        \can_top_level_inst/rx_buffer_inst/commit_overrun_abort_1_sqmuxa_i\, 
        \can_top_level_inst/rx_buffer_inst/commit_overrun_abort_Z\, 
        \can_top_level_inst/rx_buffer_inst/commit_overrun_abort_0_sqmuxa_i_Z\, 
        \can_top_level_inst/rx_buffer_inst/un5_drv_read_start\, 
        \can_top_level_inst/rx_buffer_inst/un3_drv_clr_rxpe\, 
        \can_top_level_inst/rx_buffer_inst/un8_overrun_condition\, 
        \can_top_level_inst/rx_buffer_inst/data_overrun_i_Z\, 
        \can_top_level_inst/rx_buffer_inst/un12_overrun_condition\, 
        \can_top_level_inst/rx_buffer_inst/un1_drv_bus_2_i\, 
        \can_top_level_inst/rx_buffer_inst/N_3925_i\, 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        \can_top_level_inst/rx_buffer_inst/N_3924_i\, 
        \can_top_level_inst/rx_buffer_inst/N_3928_i\, 
        \can_top_level_inst/rx_buffer_inst/N_3927_i\, 
        \can_top_level_inst/rx_buffer_inst/N_3926_i\, 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_0\, 
        \can_top_level_inst/rx_buffer_inst/un10_read_increment\, 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_1\, 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_2\, 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_3\, 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_4\, 
        \can_top_level_inst/rx_buffer_inst/N_2308_1\, 
        \can_top_level_inst/rx_buffer_inst/N_2307_1\, 
        \can_top_level_inst/rx_buffer_inst/N_2309_1\, 
        \can_top_level_inst/rx_buffer_inst/is_free_word_1_Z\, 
        \can_top_level_inst/rx_buffer_inst/un40_frame_form_w_i\, 
        \can_top_level_inst/rx_buffer_inst/rx_mof_1\, 
        \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, 
        \can_top_level_inst/rx_buffer_inst/N_4810\, 
        \can_top_level_inst/rx_buffer_inst/N_4811\, 
        \can_top_level_inst/rx_buffer_inst/N_4812\, 
        \can_top_level_inst/rx_buffer_inst/N_4813\, 
        \can_top_level_inst/rx_buffer_inst/N_4814\, 
        \can_top_level_inst/rx_buffer_inst/N_4815\, 
        \can_top_level_inst/rx_buffer_inst/N_4817\, 
        \can_top_level_inst/rx_buffer_inst/N_4820\, 
        \can_top_level_inst/rx_buffer_inst/N_4821\, 
        \can_top_level_inst/rx_buffer_inst/N_4823\, 
        \can_top_level_inst/rx_buffer_inst/N_4824\, 
        \can_top_level_inst/rx_buffer_inst/N_4825\, 
        \can_top_level_inst/rx_buffer_inst/N_4826\, 
        \can_top_level_inst/rx_buffer_inst/N_4827\, 
        \can_top_level_inst/rx_buffer_inst/N_4829\, 
        \can_top_level_inst/rx_buffer_inst/N_4281\, 
        \can_top_level_inst/rx_buffer_inst/N_4279\, 
        \can_top_level_inst/rx_buffer_inst/N_4275\, 
        \can_top_level_inst/rx_buffer_inst/N_4274\, 
        \can_top_level_inst/rx_buffer_inst/N_4271\, 
        \can_top_level_inst/rx_buffer_inst/N_4267\, 
        \can_top_level_inst/rx_buffer_inst/N_4265\, 
        \can_top_level_inst/rx_buffer_inst/N_4261\, 
        \can_top_level_inst/rx_buffer_inst/N_2246\, 
        \can_top_level_inst/rx_buffer_inst/N_2250\, 
        \can_top_level_inst/rx_buffer_inst/N_2252\, 
        \can_top_level_inst/rx_buffer_inst/N_2256\, 
        \can_top_level_inst/rx_buffer_inst/N_2259\, 
        \can_top_level_inst/rx_buffer_inst/N_2260\, 
        \can_top_level_inst/rx_buffer_inst/N_2264\, 
        \can_top_level_inst/rx_buffer_inst/N_2266\, 
        \can_top_level_inst/rx_buffer_inst/N_2245\, 
        \can_top_level_inst/rx_buffer_inst/N_2247\, 
        \can_top_level_inst/rx_buffer_inst/N_2251\, 
        \can_top_level_inst/rx_buffer_inst/N_2253\, 
        \can_top_level_inst/rx_buffer_inst/N_2255\, 
        \can_top_level_inst/rx_buffer_inst/N_2261\, 
        \can_top_level_inst/rx_buffer_inst/N_2262\, 
        \can_top_level_inst/rx_buffer_inst/N_2265\, 
        \can_top_level_inst/rx_buffer_inst/N_2267\, 
        \can_top_level_inst/rx_buffer_inst/N_2269\, 
        \can_top_level_inst/rx_buffer_inst/N_2270\, 
        \can_top_level_inst/rx_buffer_inst/N_2249\, 
        \can_top_level_inst/rx_buffer_inst/N_2268\, 
        \can_top_level_inst/rx_buffer_inst/N_2273\, 
        \can_top_level_inst/rx_buffer_inst/N_2248\, 
        \can_top_level_inst/rx_buffer_inst/N_2258\, 
        \can_top_level_inst/rx_buffer_inst/N_2257\, 
        \can_top_level_inst/rx_buffer_inst/N_4819\, 
        \can_top_level_inst/rx_buffer_inst/N_4818\, 
        \can_top_level_inst/rx_buffer_inst/N_2271\, 
        \can_top_level_inst/rx_buffer_inst/N_2272\, 
        \can_top_level_inst/rx_buffer_inst/N_4287\, 
        \can_top_level_inst/rx_buffer_inst/N_4828\, 
        \can_top_level_inst/rx_buffer_inst/N_2263\, 
        \can_top_level_inst/rx_buffer_inst/N_4822\, 
        \can_top_level_inst/rx_buffer_inst/N_2254\, 
        \can_top_level_inst/rx_buffer_inst/N_4816\, 
        \can_top_level_inst/rx_buffer_inst/un10_read_increment_1\, 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_2_Z\, 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_0_Z\, 
        \can_top_level_inst/rx_buffer_inst/un3_is_free_wordlto5_3_Z\, 
        \can_top_level_inst/rx_buffer_inst/N_48\, 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_4_Z\, 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_3_Z\, 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        \can_top_level_inst/rx_buffer_inst/un5_ram_write_Z\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        \can_top_level_inst/rx_buffer_inst/N_2279\, 
        \can_top_level_inst/rx_buffer_inst/N_2283\, 
        \can_top_level_inst/rx_buffer_inst/N_2285\, 
        \can_top_level_inst/rx_buffer_inst/N_2289\, 
        \can_top_level_inst/rx_buffer_inst/N_2292\, 
        \can_top_level_inst/rx_buffer_inst/N_2293\, 
        \can_top_level_inst/rx_buffer_inst/N_2278\, 
        \can_top_level_inst/rx_buffer_inst/N_2280\, 
        \can_top_level_inst/rx_buffer_inst/N_2281\, 
        \can_top_level_inst/rx_buffer_inst/N_2284\, 
        \can_top_level_inst/rx_buffer_inst/N_2286\, 
        \can_top_level_inst/rx_buffer_inst/N_2288\, 
        \can_top_level_inst/rx_buffer_inst/N_2291\, 
        \can_top_level_inst/rx_buffer_inst/N_2290\, 
        \can_top_level_inst/rx_buffer_inst/N_2287\, 
        \can_top_level_inst/rx_buffer_inst/N_2308_2\, 
        \can_top_level_inst/rx_buffer_inst/N_2307_2\, 
        \can_top_level_inst/rx_buffer_inst/N_2309_2\, 
        \can_top_level_inst/rx_buffer_inst/N_2833\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        \can_top_level_inst/rx_buffer_inst/N_2207_2\, 
        \can_top_level_inst/rx_buffer_inst/N_5\, 
        \can_top_level_inst/rx_buffer_inst/N_2835\, 
        \can_top_level_inst/rx_buffer_inst/N_10\, 
        \can_top_level_inst/rx_buffer_inst/N_57\, 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_Z\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_10\, 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_13\, 
        \can_top_level_inst/rx_buffer_inst/N_2206\, 
        \can_top_level_inst/rx_buffer_inst/reset_overrun_flag\, 
        \can_top_level_inst/rx_buffer_inst/parity_mismatch_1_29\, 
        \can_top_level_inst/rx_buffer_inst/parity_mismatch_1_30\, 
        \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_i[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_read_pointer_i[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_RNIJ6NH_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_write_pointer_ts_ce_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un8_read_increment_i\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un2_commit_rx_frame\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry_cy\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_1\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2373\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_sn_N_2_i\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_1_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_2\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2374\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_2_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_3\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2375\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_3_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_4\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2376\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_4_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_5\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2377\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_5_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2379\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_s_7_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_6\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2378\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_6_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s_2638_FCO\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_s_1_2643_FCO\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_1_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_2_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_3_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_4_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_5_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_6_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_s_1_2644_FCO\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_1_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_1_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_2_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_2_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_3_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_3_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_4_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_4_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_5_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_5_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_s_7_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_6_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_6_S\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_pointer_inc_1_i_c4\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_wmux_10_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_wmux_10_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_127_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_wmux_10_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_wmux_10_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y21[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y3_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y1_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_8[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y5_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y7_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_7[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_6[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_5[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_4[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y9[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y5[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y7[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_2[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y21[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y3_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y1_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_8[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y5_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y7_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_7[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_6[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_5[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_4[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y9[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y5[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y7[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_2[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y21[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y3_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y1_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_8[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y5_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y7_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_7[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_6[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_5[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_4[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y9[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y5[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y7[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_2[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y21[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y3_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y1_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_8[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y5_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y7_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_7[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_6[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_5[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_4[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y9[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y5[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y7[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_2[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_read_exp_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram14_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe14\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram13_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe13\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram12_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe12\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram11_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe11\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram10_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe10\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram9_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe9\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram8_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe8\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram7_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe7\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram6_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe6\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram5_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe5\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram4_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe4\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram3_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe3\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram2_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe2\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram1_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe1\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram0_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram29_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe29\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram28_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe28\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram27_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe27\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram26_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe26\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram25_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe25\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram24_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe24\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram23_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe23\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram22_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe22\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram21_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe21\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram20_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe20\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram19_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe19\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram18_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe18\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram17_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe17\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram16_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe16\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram15_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe15\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram44_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe44\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram43_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe43\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram42_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe42\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram41_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe41\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram40_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe40\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram39_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe39\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram38_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe38\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram37_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe37\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram36_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe36\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram35_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe35\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram34_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe34\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram33_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe33\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram32_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe32\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram31_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe31\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram30_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe30\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram59_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe59\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram58_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe58\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram57_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe57\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram56_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe56\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram55_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe55\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram54_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe54\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram53_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe53\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram52_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe52\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram51_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe51\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram50_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe50\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram49_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe49\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram48_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe48\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram47_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe47\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram46_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe46\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram45_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe45\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram74_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe74\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram73_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe73\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram72_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe72\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram71_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe71\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram70_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe70\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram69_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe69\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram68_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe68\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram67_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe67\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram66_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe66\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram65_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe65\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram64_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe64\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram63_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe63\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram62_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe62\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram61_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe61\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram60_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe60\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram89_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe89\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram88_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe88\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram87_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe87\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram86_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe86\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram85_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe85\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram84_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe84\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram83_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe83\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram82_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe82\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram81_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe81\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram80_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe80\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram79_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe79\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram78_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe78\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram77_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe77\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram76_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe76\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram75_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe75\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram104_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe104\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram103_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe103\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram102_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe102\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram101_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe101\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram100_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe100\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram99_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe99\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram98_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe98\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram97_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe97\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram96_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe96\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram95_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe95\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram94_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe94\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram93_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe93\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram92_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe92\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram91_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe91\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram90_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe90\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram119_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe119\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram118_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe118\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram117_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe117\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram116_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe116\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram115_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe115\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram114_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe114\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram113_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe113\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram112_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe112\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram111_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe111\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram110_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe110\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram109_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe109\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram108_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe108\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram107_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe107\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram106_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe106\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram105_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe105\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram127_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe127\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram126_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe126\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram125_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe125\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram124_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe124\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram123_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe123\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram122_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe122\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram121_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe121\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram120_0\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe120\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_write_i_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_0_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_23_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_22_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_21_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_20_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_19_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_18_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_17_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_24_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_340\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_341\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_342\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_343\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_344\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_345\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_346\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_347\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_348\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_349\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_350\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_351\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_352\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_353\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_354\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_355\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_183\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_184\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_185\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_186\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_189\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_3_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_14_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_11_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_8_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_18_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_17_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_15_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_21_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_24_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_26_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_27_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_25_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_3_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_3_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_RNO_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_RNO_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_wmux_3_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_0_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_Z[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_Z[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_1[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_1[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_1[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_0_1_Z[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y1[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y1[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y3[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y3[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_y0_0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_y0_0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_0_y1[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_0_y3[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_y0_0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[31]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[30]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[29]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[28]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[27]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[26]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[25]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[24]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[23]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[22]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[21]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[20]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[19]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[18]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[17]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[16]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[15]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[14]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[13]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[12]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[11]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[10]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[9]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[8]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[6]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[4]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1464_2\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1455_2\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1459_2\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1458_2\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1462_2\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1463_2\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1454_2\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1456_2\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1457_2\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns[5]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns[2]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns[1]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_Z[3]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_a3_2_Z[0]\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/write_raw_intent_i_0_Z\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/N_54\, 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/memory_write_data_sn_N_3\, 
        \can_top_level_inst/rx_buffer_inst/rst_reg_inst/rx_shift_res_reg_inst/un1_rx_buf_res_n_d_i\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_1_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/N_3898\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un7_txtb_parity_error_valid_i_0_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_2_1_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_1_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_126_i\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_124_i\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_147\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_195\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_148\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_139\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_163\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_149\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_151\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_192\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_170_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_170\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_174\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_150\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_168\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/N_4137\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/N_4134\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/N_4138\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_23_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_22_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_21_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_20_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_19_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_18_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_17_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_16_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_29_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_28_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/N_3904\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_141\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_163\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_139\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_1_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_164\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_168\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_161\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_165\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_156\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_178\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/N_4257\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/N_4254\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/N_4258\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_23_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_22_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_21_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_20_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_19_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_18_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_17_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_16_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_29_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_28_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[31]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[30]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[29]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[28]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[27]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[26]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[25]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[24]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[23]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[22]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[21]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[20]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[19]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[18]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[17]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[16]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[15]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[14]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[13]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[12]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[11]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[10]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[9]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[8]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[7]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[6]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[5]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[4]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[3]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[2]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[1]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[0]\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        \can_top_level_inst/can_core_inst/crc_15[14]\, 
        \can_top_level_inst/can_core_inst/crc_15[13]\, 
        \can_top_level_inst/can_core_inst/crc_15[12]\, 
        \can_top_level_inst/can_core_inst/crc_15[11]\, 
        \can_top_level_inst/can_core_inst/crc_15[10]\, 
        \can_top_level_inst/can_core_inst/crc_15[9]\, 
        \can_top_level_inst/can_core_inst/crc_15[8]\, 
        \can_top_level_inst/can_core_inst/crc_15[7]\, 
        \can_top_level_inst/can_core_inst/crc_15[6]\, 
        \can_top_level_inst/can_core_inst/crc_15[5]\, 
        \can_top_level_inst/can_core_inst/crc_15[4]\, 
        \can_top_level_inst/can_core_inst/crc_15[3]\, 
        \can_top_level_inst/can_core_inst/crc_15[2]\, 
        \can_top_level_inst/can_core_inst/crc_15[1]\, 
        \can_top_level_inst/can_core_inst/crc_15[0]\, 
        \can_top_level_inst/can_core_inst/crc_17[16]\, 
        \can_top_level_inst/can_core_inst/crc_17[15]\, 
        \can_top_level_inst/can_core_inst/crc_17[14]\, 
        \can_top_level_inst/can_core_inst/crc_17[13]\, 
        \can_top_level_inst/can_core_inst/crc_17[12]\, 
        \can_top_level_inst/can_core_inst/crc_17[11]\, 
        \can_top_level_inst/can_core_inst/crc_17[10]\, 
        \can_top_level_inst/can_core_inst/crc_17[9]\, 
        \can_top_level_inst/can_core_inst/crc_17[8]\, 
        \can_top_level_inst/can_core_inst/crc_17[7]\, 
        \can_top_level_inst/can_core_inst/crc_17[6]\, 
        \can_top_level_inst/can_core_inst/crc_17[5]\, 
        \can_top_level_inst/can_core_inst/crc_17[4]\, 
        \can_top_level_inst/can_core_inst/crc_17[3]\, 
        \can_top_level_inst/can_core_inst/crc_17[2]\, 
        \can_top_level_inst/can_core_inst/crc_17[1]\, 
        \can_top_level_inst/can_core_inst/crc_17[0]\, 
        \can_top_level_inst/can_core_inst/crc_21[20]\, 
        \can_top_level_inst/can_core_inst/crc_21[19]\, 
        \can_top_level_inst/can_core_inst/crc_21[18]\, 
        \can_top_level_inst/can_core_inst/crc_21[17]\, 
        \can_top_level_inst/can_core_inst/crc_21[16]\, 
        \can_top_level_inst/can_core_inst/crc_21[15]\, 
        \can_top_level_inst/can_core_inst/crc_21[14]\, 
        \can_top_level_inst/can_core_inst/crc_21[13]\, 
        \can_top_level_inst/can_core_inst/crc_21[12]\, 
        \can_top_level_inst/can_core_inst/crc_21[11]\, 
        \can_top_level_inst/can_core_inst/crc_21[10]\, 
        \can_top_level_inst/can_core_inst/crc_21[9]\, 
        \can_top_level_inst/can_core_inst/crc_21[8]\, 
        \can_top_level_inst/can_core_inst/crc_21[7]\, 
        \can_top_level_inst/can_core_inst/crc_21[6]\, 
        \can_top_level_inst/can_core_inst/crc_21[5]\, 
        \can_top_level_inst/can_core_inst/crc_21[4]\, 
        \can_top_level_inst/can_core_inst/crc_21[3]\, 
        \can_top_level_inst/can_core_inst/crc_21[2]\, 
        \can_top_level_inst/can_core_inst/crc_21[1]\, 
        \can_top_level_inst/can_core_inst/crc_21[0]\, 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state[6]\, 
        \can_top_level_inst/can_core_inst/stuff_length[0]\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\, 
        \can_top_level_inst/can_core_inst/data_halt\, 
        \can_top_level_inst/can_core_inst/stuff_err\, 
        \can_top_level_inst/can_core_inst/destuffed\, 
        \can_top_level_inst/can_core_inst/destuff_enable\, 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, 
        \can_top_level_inst/can_core_inst/set_idle\, 
        \can_top_level_inst/can_core_inst/set_err_active\, 
        \can_top_level_inst/can_core_inst/bds_data_out\, 
        \can_top_level_inst/can_core_inst/tx_frame_no_sof\, 
        \can_top_level_inst/can_core_inst/load_init_vect\, 
        \can_top_level_inst/can_core_inst/bst_data_out\, 
        \can_top_level_inst/can_core_inst/crc_data_rx_wbs\, 
        \can_top_level_inst/can_core_inst/trigger_mux_inst_tx_trigger_q\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_data_halt_q\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un6_same_bits_rst_trig_i\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un1_data_halt_d_i\, 
        \can_top_level_inst/can_core_inst/un2_bds_data_in_i\, 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst_tx_shift_reg_inst_tx_data_nbs_iv\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_first_err_delim_q\, 
        \can_top_level_inst/can_core_inst/bds_data_in\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state_proc_un1_is_receiver\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_detector_inst_un10_err_ctrs_unchanged\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst_ack_err\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_N_865\, 
        \can_top_level_inst/can_core_inst/crc_calc_from_rx\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_N_846\, 
        \can_top_level_inst/can_core_inst/bit_err_after_ack_err\, 
        \can_top_level_inst/can_core_inst/act_err_ovr_flag\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_decrement_rec_0_0\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_data_in\, 
        \can_top_level_inst/can_core_inst/can_crc_inst_crc_enable\, 
        \can_top_level_inst/can_core_inst/operation_control_inst_set_transmitter\, 
        \can_top_level_inst/can_core_inst/operation_control_inst_set_receiver\, 
        \can_top_level_inst/can_core_inst/can_crc_inst_crc_17_21_trigger_1_1\, 
        \can_top_level_inst/can_core_inst/crc_calc_from_rx_0\, 
        \can_top_level_inst/can_core_inst/can_crc_inst_crc_17_21_trigger_1_0\, 
        \can_top_level_inst/can_core_inst/crc_calc_from_rx_1_0\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/dec_one\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_eight\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_one\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un1_inc_eight_i_2_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un23_inc_eight_i_0_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un9_inc_eight_i_1_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un23_inc_eight_i_2_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/curr_state_ns[1]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/err_warning_limit_q_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un1_err_warning_limit_d_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/reg_q\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/N_27_i\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_0_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_1_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_2_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_3_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_4_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_5_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_6_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_7_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_8_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_0_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_1_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_2_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_3_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_4_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_5_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_6_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_7_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_8_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_0\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_1\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_2\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_3\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_4\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_5\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_6\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_7\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_8\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_0\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_1\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_2\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_3\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_4\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_5\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_6\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_7\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_8\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un12_tx_err_ctr_mt_erp\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/drv_bus_i[425]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[8]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[7]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[6]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[5]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[4]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[3]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[2]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[1]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNIERIU_Y[0]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[8]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[7]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[6]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[5]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[4]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[3]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[2]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[1]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[0]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIG63U9_S[13]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI16B69_S[12]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIK7JE8_S[11]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI9BRM7_S[10]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI0H3V6_S[9]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI79P86_S[8]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIG3FI5_S[7]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIRV4S4_S[6]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI8UQ54_S[5]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNINUGF3_S[4]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI817P2_S[3]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIR5T22_S[2]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIGCJC1_S[1]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_dat_sel_ctr_i[0]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIKDJDB_S[15]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI19RLA_S[14]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[5]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[4]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_RNO_S[6]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[8]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[7]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[6]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[5]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[4]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[3]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[2]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[1]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[6]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[5]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[4]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[3]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[2]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[1]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[0]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_m0_Z[8]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_m0_Z[7]\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_tx_err_ctr_ce_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_rx_err_ctr_ce_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_0\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_1\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_2\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_3\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_4\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_5\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_6\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_7\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_0\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_1\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_2\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_3\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_4\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_5\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_6\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_7\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_8\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_9\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_10\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_11\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_12\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_13\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_14\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_0\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNITG6E_Y\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_1\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNIR2DS_S\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_2\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNIQLJA1_S\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_3\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNIQ9QO1_S\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_4\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNIRU072_S\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_5\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNITK7L2_S\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_0\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_3_1_RNIMVGD1_Y\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_0_1\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_1\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_2\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_3\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_4\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_5\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_6\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_7\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_0\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_1\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_2\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_3\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c3\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_4\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_5\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c5\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_6\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_7\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c7\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_i\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_8\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_26\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_24_1_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_20_1_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_16_1_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_12_1_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_8_1_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_0_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_4_1_Z\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_5\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_4\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un5_tx_err_ctr_declto8_i_a2_5\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un5_tx_err_ctr_declto8_i_a2_4\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_238\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rst_reg_inst/rx_shift_res_reg_inst/un1_res_err_ctrs_d_i\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIUBEJ4_S[5]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4QRQ3_S[4]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNICA923_S[3]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIMSM92_S[2]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI2H4H1_S[1]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/sel_value_i[0]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNISMQGE_S[20]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI8QRSD_S[19]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4FU8D_S[18]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI261LC_S[17]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI2V31C_S[16]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4Q6DB_S[15]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI8N9PA_S[14]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIEMC5A_S[13]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIMNFH9_S[12]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0RIT8_S[11]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIC0M98_S[10]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIQ7PL7_S[9]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOD6T6_S[8]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOLJ46_S[7]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIQV0C5_S[6]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOOLCL_S[31]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0MKOK_S[30]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIALJ4K_S[29]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI46KGJ_S[28]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0PKSI_S[27]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIUDL8I_S[26]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIU4MKH_S[25]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0UM0H_S[24]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4PNCG_S[23]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIAMOOF_S[22]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIILP4F_S[21]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_0\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_1\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_2\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_3\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_4\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_5\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_6\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_7\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_8\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_9\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_10\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_11\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_12\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_13\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_14\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_15\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_16\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_17\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_18\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_19\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_20\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_21\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_22\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_23\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_24\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_25\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_26\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_27\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_28\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_29\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_30\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_reg_rst_inst/rx_shift_res_reg_inst/drv_bus_i[473]\, 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_reg_rst_inst/rx_shift_res_reg_inst/drv_bus_i[474]\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d_Z[2]\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d_Z[1]\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d_Z[0]\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[2]\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[1]\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[0]\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_3[2]\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_3[0]\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_3_iv_i[1]\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un4_data_out_d_Z\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/reg_q\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/CO0_1\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un9_stuff_lvl_reached_NE\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un3_same_bits_rst_i_0\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un1_bst_ctr_q3_Z\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/N_10\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un1_same_bits_rst_trig_Z\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un4_bst_ctr_d_Z\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/fixed_reg_q\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/fixed_reg_d\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/CO0_0\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/dff_data_out_reg/N_374_mux\, 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/dff_data_out_reg/data_out_d_sn_N_4_mux\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[2]\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[1]\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[0]\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_d_Z[2]\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_d_Z[1]\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_d_Z[0]\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d_Z[2]\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d_Z[1]\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d_Z[0]\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/CO0_1\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_stuff_lvl_reached_NE_Z\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un17_same_bits_erase_Z\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un10_destuffed_d_Z\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un4_dst_ctr_d_Z\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/fixed_prev_q\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_discard_stuff_bit_Z\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/prev_val_d_Z\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/prev_val_q\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un2_stuff_rule_violate_Z\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/reg_q_0\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/fixed_prev_d\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/CO0_0\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un17_same_bits_erase_1_Z\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_stuff_err_d_Z\, 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dff_destuffed_flag_reg/N_26_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_iv_4[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_0_iv_2[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_0_iv_2[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_cnst_m[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_0_iv_3[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_mem_index[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_cnst[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[10]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state_0_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_stuff_count[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_stuff_count[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_stuff_count[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_stuff_count[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_dominant\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un3_data_length_sub_c_axbxc5\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_length_i_m\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_909\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un3_data_length_sub_c_axbxc4\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_881\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/form_err\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_877\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1003\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retr_ctr_clear\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un3_data_length_sub_c_axbxc3\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/bit_err_disable_1_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un2_compl_ctr_div_32_plus_5_sat_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/CO0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_stuff_count\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_dlc\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_clear_match_flag\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un3_data_length_sub_c_axbxc2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/CO2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_980\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1310\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reinteg_ctr_expired\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_one\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/bit_err_arb\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_ena\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_761_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_err\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_763_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_765_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_860_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_715_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1215_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1077\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ide\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_esi\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_brs\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un22_rx_data_nbs_i_m\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un22_rx_data_nbs_m\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_match\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1324\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1246\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un2_bit_err_enable_i_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/next_state_16_sqmuxa\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_966\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_934\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_962_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1012\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retr_ctr_add\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_is_rtr_i_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un17_rx_shift_cmd\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un25_rx_shift_cmd\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un9_rx_shift_cmd\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_shift_res_reg_inst/rx_shift_res_reg_inst/N_1251_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/shift_reg_in_4[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/shift_reg_in_6[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/shift_reg_in_2[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy_Y[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un3_reinteg_ctr_ce_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un6_reinteg_ctr_expired_5\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un6_reinteg_ctr_expired_4\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[9]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un4_crc_match_d_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un24_err_type_d_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_1170_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_type_d_ss0_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_FCO\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_FCO\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_FCO\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_406\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un6_form_err_i_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_err_type_d_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_crc_err_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_stuff_count_ok_NE_0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_stuff_count_ok_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_stuff_count_ok_2_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_crc_match_c_0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un24_err_type_d_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un26_err_frm_req_i_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_1250\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[33]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[32]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[30]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[29]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[27]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[23]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[17]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[16]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[15]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[14]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_9[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/AXB[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/AXB[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/AXB[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_1_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_3_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_7_1_0_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_3_1_Z[35]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[35]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[31]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[30]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[28]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[15]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[13]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_3_Z[35]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ALTB_1[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ALTB[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ALTB[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_Z[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_a4_1_0_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_6_1_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a4_0_0_Z[23]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_3_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_iv_0_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_0_Z[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_0_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_0_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_2_1_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_1_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_1_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_1[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_1[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_4[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_4[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_4[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_3[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_2[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_2[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_2[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_14_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_12_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_11_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_10_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_9_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_0_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_1_0_Z[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_a4_0_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_0_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_0_2_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_18_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_15_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2_0_1_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2_0_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_o2_3_0_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_0_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_1_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_1_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_1_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_2_3_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[37]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[34]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[31]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[25]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_7[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_21_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_20_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_0_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_2_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_0_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_1_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_19_m_Z[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_iv_2_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_3_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_4_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_2_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_5_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_2_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_2_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_1_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1317_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_metadata_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid_d_0_sqmuxa_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/unlock_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/arbl_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/failed_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_retr_ctr_clear_i_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_771_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1369_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1367_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_802_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_800_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_798_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_796_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_794_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_792_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_790_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_788_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_786_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_784_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_782_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_780_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_778_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_776_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_774_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_832_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_830_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_828_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_826_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_824_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_822_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_820_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_818_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_816_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_814_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_812_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_810_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_808_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_806_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_804_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_844_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_842_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_840_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_838_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_836_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_834_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/drv_bus_off_reset_q_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_rx_trigger\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_869_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_875\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_drv_cpexs\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_flag_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1533_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_ack_err_flag_clr\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_918_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_enable_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/retr_ctr_add_block_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_retr_ctr_add_block_clr\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un12_sync_control_d_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un10_sync_control_d_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_data_nbs_prev_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_138\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_968\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_929\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_953\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1069\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_984\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_223\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/br_shifted_u_1_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ide_is_arbitration_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/g2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_gate_mem_read_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_txtb_clk_en_d_NE_0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_txtb_clk_en_d_NE_2_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_913_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1038\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_930\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_926\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un21_ctrl_signal_upd_i_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1221\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un21_ctrl_signal_upd_i_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_920_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_944\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_941\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_931\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1327\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1203\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_933\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_973\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_952\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_939\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lto3_0_0_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_data_len_can_2_0_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lto3_0_0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_clear_i_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_clear_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_936_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_988\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1116\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_2_0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_17_0_a4_0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_928_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/set_idle_i_5_sqmuxa\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1234\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1238\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_0_sqmuxa\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_9\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_is_transmitter\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/frame_type\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1013\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1151_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1050\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_932\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1103_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_err_frm_req_3\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1320\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_arb_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_949\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1314\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1462_3\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_4\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1372\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1421\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_27\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1465\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_3_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_s1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lt4_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_go_to_suspend_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1024\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1023\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_890_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_879_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un21_ctrl_signal_upd_i_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_0_2_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/perform_hsync_2_i_a2_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/is_control_2_i_a2_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un4_ctrl_counted_byte_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a2_0_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa_5\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa_4\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a2_1_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/pexs_set_i_0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1268\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_switch_to_ssp_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1420\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/drv_pex_li_m_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_ack_err_flag\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_2_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_24\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_717_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/unlock_sn_N_8_mux\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/no_data_receiver_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_no_data_transmitter_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_i_3_tz_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_905_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_892_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1035\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1045\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1047\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1051\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_switch_data\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_drv_rom_ena_m\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1196\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_957\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1239\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1408\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1085_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lt4\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1048\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_886_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_960\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1250\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_data_len_can_2_0_i_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_load_base_id_i_0_0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_frame_no_sof_d_i_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_set_receiver_i_a4_0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_2_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_4_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/is_control_2_i_a2_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1192\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1193\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_sync_control_d_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1180\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_sqmuxa_m\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1202\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1230\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_arbitration_lost_condition_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un26_is_receiver\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1124\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1142\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1236\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1304\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_sm0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1299_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1062\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_958\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1043\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1267_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_is_receiver\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1370\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1414\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_28_mux\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_13\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1138\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_flag_clr_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_22\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_736_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1186_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_4_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_0_4_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_3_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_2_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/crc_enable_i_a4_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_i_3_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1046\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_5_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1228\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_unaliged_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un109_ctrl_ctr_zero\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1457\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_no_data_field_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid_d_1_sqmuxa_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_clear_sm0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_ide_is_arbitration\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ssp_reset_i_4_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_971\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/next_state_17_sqmuxa\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_is_fd_frame_m\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/i5_mux\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_8_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_m2_2_0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_3_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_2_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_2_3_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa_2_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_5_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_854_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1189\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1244\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/unlock_5\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_no_data_field_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1255\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1269\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_951\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_996\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_956\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lt4\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_8\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_m2_1_0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_set_idle_0_a4_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_4_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1190\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1137\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_950\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data_d_0_sqmuxa_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data_d_1_sqmuxa_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1059\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_975\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_978\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1033\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1300\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_24\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_945\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_937\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1497_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_7_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_3_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1194\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_25_sn\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1213\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_955\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_form_err_i_0_sqmuxa_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1183\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_970\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_976\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1330\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21_tz\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1004\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1415\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_961\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un11_crc_use_21_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1497_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1197\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/retr_ctr_add_sn_N_5_mux\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_c4_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1131\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1113\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_sn_N_7_mux\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_900\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_sm0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_c5_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_8_to_64_integer[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/un2_data_len_can_2_0_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_can_fd_1_sqmuxa_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_len_8_to_64_integer[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_len_can_fd_1_sqmuxa_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_Z[12]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_Z[10]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_Z[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_i_o2_1_Z[16]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1_Z[18]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1_Z[13]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[18]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[9]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[18]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[17]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[16]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[15]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[14]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[13]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[12]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[11]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[10]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[9]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_1_Z[15]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[15]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[13]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[12]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[11]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[10]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1_0_Z[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_0[12]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_0[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[27]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[26]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[25]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[24]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[23]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[22]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[21]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[27]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[26]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[25]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[24]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[23]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[22]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[21]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[20]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[19]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[18]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[17]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[16]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[15]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[14]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[13]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[12]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[11]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[18]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[17]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[13]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[12]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[11]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[10]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[9]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_2_Z[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_0_Z[11]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[29]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[20]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[19]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[18]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[11]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[13]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[27]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[26]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[25]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[24]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[23]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[22]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[21]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1194\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1206\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1196\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc3\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1214\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/m50_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1192\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_29\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_19_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_19\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/m42_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1218\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_20_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/m51_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1219\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_c3\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_output_i_m_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1187\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_46_i\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1186\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_388\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_RNIH1071_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[30]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[29]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[28]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[27]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[26]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[25]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[24]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[23]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[22]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[21]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[20]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[19]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[18]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[17]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[16]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[15]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[14]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[13]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[12]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[11]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[10]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[9]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[31]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[30]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[29]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[28]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[27]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[26]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[25]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[24]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[23]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[22]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[21]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[20]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[19]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[18]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[17]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[16]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[15]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[14]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[13]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[12]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[11]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[10]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[9]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/tx_sr_pload_val[28]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[31]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[30]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_12_Z[31]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[20]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[19]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[18]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[17]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[16]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[15]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[14]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/tx_sr_output\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_o4_3_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_12_0_o4_0_3\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_8_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_77_mux\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_83_mux\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_o4_3\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/g0_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_22\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_a5_2_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_24\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_o4_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_21\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_19_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_17_i_x4_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_13_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_14_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_17_i_o4_0_4\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_17_i_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_10\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_17_i_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_15\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_18\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_12_0_x4_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_11_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_12_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_12_0_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_12_0_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_13_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_14_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_11\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_18_0_o4_0_3\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_12\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_18_0_0\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_8\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_18_0_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_13\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_14\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_82_mux\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_1225_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_1226_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_1225\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_1226\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_RNITOLC_Y[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNI3Q2C_FCO[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[8]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[7]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[6]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[5]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[4]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un6_ctrl_ctr_ce_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un4_compl_ctr_ce_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ANB2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_2_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_1_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_one_2\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un2_ctrl_ctr_one_1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_5_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_7_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d_Z[3]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d_Z[2]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d_Z[1]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d_Z[0]\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/un1_retr_ctr_add_Z\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/CO1\, 
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_ns[1]\, 
        \can_top_level_inst/can_core_inst/operation_control_inst/N_10_i\, 
        \can_top_level_inst/can_core_inst/operation_control_inst/m15_1\, 
        \can_top_level_inst/can_core_inst/operation_control_inst/un2_go_to_off_i_0\, 
        \can_top_level_inst/can_core_inst/operation_control_inst/N_20\, 
        \can_top_level_inst/can_core_inst/operation_control_inst/N_9\, 
        \can_top_level_inst/can_core_inst/operation_control_inst/N_21_mux\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_2_Z\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_1_0\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_Z\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_1_Z\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/un2_crc_ena_17_21_i\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/un2_crc_ena_15_i\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_15_trigger_Z\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[15]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[12]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[10]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[9]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[8]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[7]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[5]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[2]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[0]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[16]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[14]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[13]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[11]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[6]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[4]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[3]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[1]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g1_0_Z\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g1_1_Z\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_nxt_Z\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[14]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[10]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[8]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[7]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[4]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[3]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[13]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[12]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[11]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[9]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[6]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[5]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[2]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[1]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[0]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_nxt_Z\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[19]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[18]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[17]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[16]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[15]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[14]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[12]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[10]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[9]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[8]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[6]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[5]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[2]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[1]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[0]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[20]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[13]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[11]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[7]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[4]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[3]\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g1_0_Z\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g1_1_Z\, 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_nxt_Z\, 
        \timestamp_ibuf[63]/YIN\, \data_in_ibuf[29]/YIN\, 
        \data_out_obuf[13]/DOUT\, \data_out_obuf[13]/EOUT\, 
        \data_in_ibuf[27]/YIN\, \data_out_obuf[30]/DOUT\, 
        \data_out_obuf[30]/EOUT\, \data_out_obuf[7]/DOUT\, 
        \data_out_obuf[7]/EOUT\, \data_in_ibuf[0]/YIN\, 
        \can_tx_obuf/DOUT\, \can_tx_obuf/EOUT\, 
        \adress_ibuf[6]/YIN\, \data_in_ibuf[6]/YIN\, 
        \data_in_ibuf[19]/YIN\, \data_in_ibuf[17]/YIN\, 
        \data_out_obuf[26]/DOUT\, \data_out_obuf[26]/EOUT\, 
        \timestamp_ibuf[2]/YIN\, \timestamp_ibuf[53]/YIN\, 
        \timestamp_ibuf[23]/YIN\, \timestamp_ibuf[43]/YIN\, 
        \timestamp_ibuf[38]/YIN\, \adress_ibuf[9]/YIN\, 
        \data_in_ibuf[8]/YIN\, \data_in_ibuf[24]/YIN\, 
        \timestamp_ibuf[56]/YIN\, \timestamp_ibuf[26]/YIN\, 
        \timestamp_ibuf[46]/YIN\, \adress_ibuf[4]/YIN\, 
        \timestamp_ibuf[31]/YIN\, \timestamp_ibuf[59]/YIN\, 
        \data_out_obuf[16]/DOUT\, \data_out_obuf[16]/EOUT\, 
        \timestamp_ibuf[29]/YIN\, \data_out_obuf[1]/DOUT\, 
        \data_out_obuf[1]/EOUT\, \timestamp_ibuf[49]/YIN\, 
        \res_n_out_obuf/DOUT\, \res_n_out_obuf/EOUT\, 
        \data_out_obuf[21]/DOUT\, \data_out_obuf[21]/EOUT\, 
        \timestamp_ibuf[1]/YIN\, \data_in_ibuf[28]/YIN\, 
        \data_in_ibuf[7]/YIN\, \adress_ibuf[5]/YIN\, 
        \timestamp_ibuf[17]/YIN\, \data_in_ibuf[14]/YIN\, 
        \can_rx_ibuf/YIN\, \data_in_ibuf[3]/YIN\, \scs_ibuf/YIN\, 
        \timestamp_ibuf[7]/YIN\, \data_out_obuf[29]/DOUT\, 
        \data_out_obuf[29]/EOUT\, \data_in_ibuf[18]/YIN\, 
        \data_out_obuf[11]/DOUT\, \data_out_obuf[11]/EOUT\, 
        \srd_ibuf/YIN\, \sbe_ibuf[3]/YIN\, 
        \data_out_obuf[3]/DOUT\, \data_out_obuf[3]/EOUT\, 
        \timestamp_ibuf[10]/YIN\, \data_out_obuf[19]/DOUT\, 
        \data_out_obuf[19]/EOUT\, \data_in_ibuf[30]/YIN\, 
        \adress_ibuf[7]/YIN\, \data_out_obuf[5]/DOUT\, 
        \data_out_obuf[5]/EOUT\, \timestamp_ibuf[12]/YIN\, 
        \data_in_ibuf[9]/YIN\, \timestamp_ibuf[61]/YIN\, 
        \data_out_obuf[4]/DOUT\, \data_out_obuf[4]/EOUT\, 
        \data_in_ibuf[20]/YIN\, \adress_ibuf[10]/YIN\, 
        \timestamp_ibuf[37]/YIN\, \timestamp_ibuf[58]/YIN\, 
        \timestamp_ibuf[28]/YIN\, \timestamp_ibuf[48]/YIN\, 
        \adress_ibuf[2]/YIN\, \data_out_obuf[9]/DOUT\, 
        \data_out_obuf[9]/EOUT\, \data_out_obuf[22]/DOUT\, 
        \data_out_obuf[22]/EOUT\, \data_out_obuf[0]/DOUT\, 
        \data_out_obuf[0]/EOUT\, \timestamp_ibuf[14]/YIN\, 
        \timestamp_ibuf[15]/YIN\, \int_obuf/DOUT\, 
        \int_obuf/EOUT\, \data_in_ibuf[10]/YIN\, 
        \timestamp_ibuf[51]/YIN\, \timestamp_ibuf[21]/YIN\, 
        \timestamp_ibuf[41]/YIN\, \data_in_ibuf[23]/YIN\, 
        \sbe_ibuf[0]/YIN\, \res_n_ibuf/YIN\, 
        \data_out_obuf[2]/DOUT\, \data_out_obuf[2]/EOUT\, 
        \data_out_obuf[12]/DOUT\, \data_out_obuf[12]/EOUT\, 
        \timestamp_ibuf[30]/YIN\, \data_in_ibuf[13]/YIN\, 
        \data_out_obuf[31]/DOUT\, \data_out_obuf[31]/EOUT\, 
        \timestamp_ibuf[32]/YIN\, \timestamp_ibuf[13]/YIN\, 
        \data_in_ibuf[4]/YIN\, \adress_ibuf[11]/YIN\, 
        \data_out_obuf[25]/DOUT\, \data_out_obuf[25]/EOUT\, 
        \data_in_ibuf[26]/YIN\, \data_out_obuf[28]/DOUT\, 
        \data_out_obuf[28]/EOUT\, \data_out_obuf[20]/DOUT\, 
        \data_out_obuf[20]/EOUT\, \timestamp_ibuf[34]/YIN\, 
        \timestamp_ibuf[16]/YIN\, \timestamp_ibuf[35]/YIN\, 
        \timestamp_ibuf[19]/YIN\, \data_in_ibuf[22]/YIN\, 
        \timestamp_ibuf[6]/YIN\, \data_in_ibuf[1]/YIN\, 
        \data_in_ibuf[2]/YIN\, \data_in_ibuf[16]/YIN\, 
        \data_in_ibuf[5]/YIN\, \data_out_obuf[15]/DOUT\, 
        \data_out_obuf[15]/EOUT\, \timestamp_ibuf[57]/YIN\, 
        \timestamp_ibuf[27]/YIN\, \data_out_obuf[18]/DOUT\, 
        \data_out_obuf[18]/EOUT\, \timestamp_ibuf[47]/YIN\, 
        \clk_sys_ibuf/YIN\, \adress_ibuf[8]/YIN\, 
        \data_out_obuf[27]/DOUT\, \data_out_obuf[27]/EOUT\, 
        \data_out_obuf[10]/DOUT\, \data_out_obuf[10]/EOUT\, 
        \data_out_obuf[8]/DOUT\, \data_out_obuf[8]/EOUT\, 
        \data_in_ibuf[12]/YIN\, \timestamp_ibuf[8]/YIN\, 
        \timestamp_ibuf[60]/YIN\, \adress_ibuf[3]/YIN\, 
        \data_in_ibuf[25]/YIN\, \timestamp_ibuf[9]/YIN\, 
        \timestamp_ibuf[62]/YIN\, \data_in_ibuf[31]/YIN\, 
        \timestamp_ibuf[33]/YIN\, \data_out_obuf[24]/DOUT\, 
        \data_out_obuf[24]/EOUT\, \data_out_obuf[17]/DOUT\, 
        \data_out_obuf[17]/EOUT\, \sbe_ibuf[2]/YIN\, 
        \data_in_ibuf[21]/YIN\, \data_in_ibuf[15]/YIN\, 
        \sbe_ibuf[1]/YIN\, \timestamp_ibuf[50]/YIN\, 
        \timestamp_ibuf[20]/YIN\, \timestamp_ibuf[40]/YIN\, 
        \timestamp_ibuf[36]/YIN\, \timestamp_ibuf[52]/YIN\, 
        \timestamp_ibuf[39]/YIN\, \timestamp_ibuf[22]/YIN\, 
        \timestamp_ibuf[42]/YIN\, \data_in_ibuf[11]/YIN\, 
        \data_out_obuf[14]/DOUT\, \data_out_obuf[14]/EOUT\, 
        \swr_ibuf/YIN\, \timestamp_ibuf[18]/YIN\, 
        \data_out_obuf[6]/DOUT\, \data_out_obuf[6]/EOUT\, 
        \timestamp_ibuf[3]/YIN\, \data_out_obuf[23]/DOUT\, 
        \data_out_obuf[23]/EOUT\, \timestamp_ibuf[54]/YIN\, 
        \timestamp_ibuf[24]/YIN\, \timestamp_ibuf[44]/YIN\, 
        \timestamp_ibuf[5]/YIN\, \timestamp_ibuf[55]/YIN\, 
        \timestamp_ibuf[25]/YIN\, \timestamp_ibuf[45]/YIN\, 
        \timestamp_ibuf[4]/YIN\, \timestamp_ibuf[11]/YIN\, 
        \timestamp_ibuf[0]/YIN\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_0_FCNET1\, 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_14_FCNET1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8_FCNET1\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_87_FCNET1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_FCNET1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_FCNET1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_9_FCNET1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_8_FCNET1\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_8_FCNET1\, 
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_0_FCNET1\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_5_FCNET1\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_87_FCNET1\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_8_FCNET1\, 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_9_FCNET1\, 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_FCNET1\, 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_87_FCNET1\, 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI6BEO2[6]_FCNET1\, 
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_FCNET1\, 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_6_FCNET1\, 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIBGHO99[10]_FCNET1\, 
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_87_FCNET1\, 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_FCNET1\, 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIO8KOB[31]_FCNET1\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_28_FCNET1\, 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_28_FCNET1\, 
        NN_1, \I_1/U0_RGB1_RGB0_rgb_net_1\, 
        \I_1/U0_RGB1_RGB1_rgb_net_1\, 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, 
        \I_1/U0_RGB1_RGB6_rgb_net_1\, 
        \I_1/U0_RGB1_RGB7_rgb_net_1\, 
        \can_top_level_inst/reg_q_arst\, 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADLIB_GND, ADLIB_VCC, CI_TO_CO, NET_CC_CONFIG, 
        NET_CC_CONFIG0, NET_CC_CONFIG1, NET_CC_CONFIG2, 
        NET_CC_CONFIG3, NET_CC_CONFIG4, NET_CC_CONFIG5, 
        NET_CC_CONFIG6, NET_CC_CONFIG7, NET_CC_CONFIG8, 
        NET_CC_CONFIG9, NET_CC_CONFIG10, NET_CC_CONFIG11, 
        NET_CC_CONFIG12, NET_CC_CONFIG13, NET_CC_CONFIG14, 
        NET_CC_CONFIG15, NET_CC_CONFIG16, NET_CC_CONFIG17, 
        NET_CC_CONFIG18, NET_CC_CONFIG19, NET_CC_CONFIG20, 
        NET_CC_CONFIG21, NET_CC_CONFIG22, NET_CC_CONFIG23, 
        NET_CC_CONFIG24, NET_CC_CONFIG25, NET_CC_CONFIG26, 
        NET_CC_CONFIG27, NET_CC_CONFIG28, NET_CC_CONFIG29, 
        NET_CC_CONFIG30, NET_CC_CONFIG31, NET_CC_CONFIG32, 
        NET_CC_CONFIG33, NET_CC_CONFIG34, NET_CC_CONFIG35, 
        NET_CC_CONFIG36, NET_CC_CONFIG37, NET_CC_CONFIG38, 
        NET_CC_CONFIG39, NET_CC_CONFIG40, NET_CC_CONFIG41, 
        NET_CC_CONFIG42, NET_CC_CONFIG43, NET_CC_CONFIG44, 
        NET_CC_CONFIG45, NET_CC_CONFIG46, NET_CC_CONFIG47, 
        NET_CC_CONFIG48, NET_CC_CONFIG49, NET_CC_CONFIG50, 
        NET_CC_CONFIG51, NET_CC_CONFIG52, NET_CC_CONFIG53, 
        NET_CC_CONFIG54, NET_CC_CONFIG55, NET_CC_CONFIG56, 
        NET_CC_CONFIG57, NET_CC_CONFIG58, NET_CC_CONFIG59, 
        NET_CC_CONFIG60, NET_CC_CONFIG61, NET_CC_CONFIG62, 
        NET_CC_CONFIG63, NET_CC_CONFIG64, NET_CC_CONFIG65, 
        NET_CC_CONFIG66, NET_CC_CONFIG67, NET_CC_CONFIG68, 
        NET_CC_CONFIG69, NET_CC_CONFIG70, NET_CC_CONFIG71, 
        NET_CC_CONFIG72, NET_CC_CONFIG73, NET_CC_CONFIG74, 
        NET_CC_CONFIG75, NET_CC_CONFIG76, NET_CC_CONFIG77, 
        NET_CC_CONFIG78, NET_CC_CONFIG79, NET_CC_CONFIG80, 
        NET_CC_CONFIG81, NET_CC_CONFIG82, CI_TO_CO83, 
        NET_CC_CONFIG84, NET_CC_CONFIG85, NET_CC_CONFIG86, 
        NET_CC_CONFIG87, NET_CC_CONFIG88, NET_CC_CONFIG89, 
        NET_CC_CONFIG90, NET_CC_CONFIG91, NET_CC_CONFIG92, 
        NET_CC_CONFIG93, NET_CC_CONFIG94, NET_CC_CONFIG95, 
        NET_CC_CONFIG96, NET_CC_CONFIG97, NET_CC_CONFIG98, 
        NET_CC_CONFIG99, NET_CC_CONFIG100, NET_CC_CONFIG101, 
        NET_CC_CONFIG102, NET_CC_CONFIG103, NET_CC_CONFIG104, 
        NET_CC_CONFIG105, NET_CC_CONFIG106, NET_CC_CONFIG107, 
        NET_CC_CONFIG108, NET_CC_CONFIG109, NET_CC_CONFIG110, 
        NET_CC_CONFIG111, NET_CC_CONFIG112, NET_CC_CONFIG113, 
        NET_CC_CONFIG114, NET_CC_CONFIG115, NET_CC_CONFIG116, 
        NET_CC_CONFIG117, NET_CC_CONFIG118, NET_CC_CONFIG119, 
        NET_CC_CONFIG120, NET_CC_CONFIG121, NET_CC_CONFIG122, 
        NET_CC_CONFIG123, NET_CC_CONFIG124, NET_CC_CONFIG125, 
        NET_CC_CONFIG126, NET_CC_CONFIG127, NET_CC_CONFIG128, 
        NET_CC_CONFIG129, NET_CC_CONFIG130, NET_CC_CONFIG131, 
        NET_CC_CONFIG132, NET_CC_CONFIG133, NET_CC_CONFIG134, 
        NET_CC_CONFIG135, NET_CC_CONFIG136, NET_CC_CONFIG137, 
        NET_CC_CONFIG138, NET_CC_CONFIG139, NET_CC_CONFIG140, 
        NET_CC_CONFIG141, NET_CC_CONFIG142, NET_CC_CONFIG143, 
        NET_CC_CONFIG144, NET_CC_CONFIG145, NET_CC_CONFIG146, 
        NET_CC_CONFIG147, NET_CC_CONFIG148, NET_CC_CONFIG149, 
        NET_CC_CONFIG150, NET_CC_CONFIG151, NET_CC_CONFIG152, 
        NET_CC_CONFIG153, NET_CC_CONFIG154, NET_CC_CONFIG155, 
        NET_CC_CONFIG156, NET_CC_CONFIG157, NET_CC_CONFIG158, 
        NET_CC_CONFIG159, NET_CC_CONFIG160, NET_CC_CONFIG161, 
        NET_CC_CONFIG162, NET_CC_CONFIG163, NET_CC_CONFIG164, 
        NET_CC_CONFIG165, NET_CC_CONFIG166, NET_CC_CONFIG167, 
        NET_CC_CONFIG168, NET_CC_CONFIG169, NET_CC_CONFIG170, 
        NET_CC_CONFIG171, NET_CC_CONFIG172, NET_CC_CONFIG173, 
        NET_CC_CONFIG174, NET_CC_CONFIG175, NET_CC_CONFIG176, 
        NET_CC_CONFIG177, NET_CC_CONFIG178, NET_CC_CONFIG179, 
        NET_CC_CONFIG180, NET_CC_CONFIG181, NET_CC_CONFIG182, 
        NET_CC_CONFIG183, NET_CC_CONFIG184, NET_CC_CONFIG185, 
        NET_CC_CONFIG186, NET_CC_CONFIG187, NET_CC_CONFIG188, 
        NET_CC_CONFIG189, NET_CC_CONFIG190, NET_CC_CONFIG191, 
        NET_CC_CONFIG192, NET_CC_CONFIG193, NET_CC_CONFIG194, 
        NET_CC_CONFIG195, NET_CC_CONFIG196, NET_CC_CONFIG197, 
        NET_CC_CONFIG198, NET_CC_CONFIG199, NET_CC_CONFIG200, 
        NET_CC_CONFIG201, NET_CC_CONFIG202, NET_CC_CONFIG203, 
        NET_CC_CONFIG204, NET_CC_CONFIG205, NET_CC_CONFIG206, 
        NET_CC_CONFIG207, CI_TO_CO208, NET_CC_CONFIG209, 
        NET_CC_CONFIG210, NET_CC_CONFIG211, NET_CC_CONFIG212, 
        NET_CC_CONFIG213, NET_CC_CONFIG214, NET_CC_CONFIG215, 
        NET_CC_CONFIG216, NET_CC_CONFIG217, NET_CC_CONFIG218, 
        NET_CC_CONFIG219, NET_CC_CONFIG220, NET_CC_CONFIG221, 
        NET_CC_CONFIG222, NET_CC_CONFIG223, NET_CC_CONFIG224, 
        NET_CC_CONFIG225, NET_CC_CONFIG226, NET_CC_CONFIG227, 
        NET_CC_CONFIG228, NET_CC_CONFIG229, NET_CC_CONFIG230, 
        NET_CC_CONFIG231, NET_CC_CONFIG232, NET_CC_CONFIG233, 
        NET_CC_CONFIG234, NET_CC_CONFIG235, NET_CC_CONFIG236, 
        NET_CC_CONFIG237, NET_CC_CONFIG238, NET_CC_CONFIG239, 
        NET_CC_CONFIG240, NET_CC_CONFIG241, NET_CC_CONFIG242, 
        NET_CC_CONFIG243, NET_CC_CONFIG244, NET_CC_CONFIG245, 
        NET_CC_CONFIG246, NET_CC_CONFIG247, NET_CC_CONFIG248, 
        NET_CC_CONFIG249, NET_CC_CONFIG250, NET_CC_CONFIG251, 
        NET_CC_CONFIG252, NET_CC_CONFIG253, NET_CC_CONFIG254, 
        NET_CC_CONFIG255, NET_CC_CONFIG256, NET_CC_CONFIG257, 
        NET_CC_CONFIG258, NET_CC_CONFIG259, NET_CC_CONFIG260, 
        NET_CC_CONFIG261, NET_CC_CONFIG262, NET_CC_CONFIG263, 
        NET_CC_CONFIG264, NET_CC_CONFIG265, NET_CC_CONFIG266, 
        NET_CC_CONFIG267, NET_CC_CONFIG268, NET_CC_CONFIG269, 
        NET_CC_CONFIG270, NET_CC_CONFIG271, NET_CC_CONFIG272, 
        NET_CC_CONFIG273, NET_CC_CONFIG274, NET_CC_CONFIG275, 
        NET_CC_CONFIG276, NET_CC_CONFIG277, NET_CC_CONFIG278, 
        NET_CC_CONFIG279, NET_CC_CONFIG280, NET_CC_CONFIG281, 
        NET_CC_CONFIG282, NET_CC_CONFIG283, NET_CC_CONFIG284, 
        NET_CC_CONFIG285, NET_CC_CONFIG286, NET_CC_CONFIG287, 
        NET_CC_CONFIG288, NET_CC_CONFIG289, NET_CC_CONFIG290, 
        NET_CC_CONFIG291, NET_CC_CONFIG292, NET_CC_CONFIG293, 
        NET_CC_CONFIG294, NET_CC_CONFIG295, NET_CC_CONFIG296, 
        NET_CC_CONFIG297, NET_CC_CONFIG298, NET_CC_CONFIG299, 
        NET_CC_CONFIG300, NET_CC_CONFIG301, NET_CC_CONFIG302, 
        NET_CC_CONFIG303, NET_CC_CONFIG304, NET_CC_CONFIG305, 
        NET_CC_CONFIG306, NET_CC_CONFIG307, NET_CC_CONFIG308, 
        NET_CC_CONFIG309, NET_CC_CONFIG310, NET_CC_CONFIG311, 
        NET_CC_CONFIG312, NET_CC_CONFIG313, NET_CC_CONFIG314, 
        NET_CC_CONFIG315, NET_CC_CONFIG316, NET_CC_CONFIG317, 
        NET_CC_CONFIG318, NET_CC_CONFIG319, NET_CC_CONFIG320, 
        NET_CC_CONFIG321, NET_CC_CONFIG322, NET_CC_CONFIG323, 
        NET_CC_CONFIG324, CI_TO_CO325, NET_CC_CONFIG326, 
        NET_CC_CONFIG327, NET_CC_CONFIG328, NET_CC_CONFIG329, 
        NET_CC_CONFIG330, NET_CC_CONFIG331, NET_CC_CONFIG332, 
        NET_CC_CONFIG333, NET_CC_CONFIG334, NET_CC_CONFIG335, 
        NET_CC_CONFIG336, NET_CC_CONFIG337, NET_CC_CONFIG338, 
        NET_CC_CONFIG339, NET_CC_CONFIG340, NET_CC_CONFIG341, 
        NET_CC_CONFIG342, NET_CC_CONFIG343, NET_CC_CONFIG344, 
        NET_CC_CONFIG345, NET_CC_CONFIG346, NET_CC_CONFIG347, 
        NET_CC_CONFIG348, NET_CC_CONFIG349, NET_CC_CONFIG350, 
        NET_CC_CONFIG351, NET_CC_CONFIG352, NET_CC_CONFIG353, 
        NET_CC_CONFIG354, NET_CC_CONFIG355, NET_CC_CONFIG356, 
        NET_CC_CONFIG357, NET_CC_CONFIG358, NET_CC_CONFIG359, 
        NET_CC_CONFIG360, NET_CC_CONFIG361, NET_CC_CONFIG362, 
        NET_CC_CONFIG363, NET_CC_CONFIG364, NET_CC_CONFIG365, 
        NET_CC_CONFIG366, NET_CC_CONFIG367, NET_CC_CONFIG368, 
        NET_CC_CONFIG369, NET_CC_CONFIG370, NET_CC_CONFIG371, 
        NET_CC_CONFIG372, NET_CC_CONFIG373, NET_CC_CONFIG374, 
        NET_CC_CONFIG375, NET_CC_CONFIG376, NET_CC_CONFIG377, 
        NET_CC_CONFIG378, NET_CC_CONFIG379, NET_CC_CONFIG380, 
        NET_CC_CONFIG381, NET_CC_CONFIG382, NET_CC_CONFIG383, 
        NET_CC_CONFIG384, NET_CC_CONFIG385, NET_CC_CONFIG386, 
        NET_CC_CONFIG387, NET_CC_CONFIG388, NET_CC_CONFIG389, 
        NET_CC_CONFIG390, NET_CC_CONFIG391, NET_CC_CONFIG392, 
        NET_CC_CONFIG393, CI_TO_CO394, NET_CC_CONFIG395, 
        NET_CC_CONFIG396, NET_CC_CONFIG397, NET_CC_CONFIG398, 
        NET_CC_CONFIG399, NET_CC_CONFIG400, NET_CC_CONFIG401, 
        NET_CC_CONFIG402, NET_CC_CONFIG403, NET_CC_CONFIG404, 
        NET_CC_CONFIG405, NET_CC_CONFIG406, NET_CC_CONFIG407, 
        NET_CC_CONFIG408, NET_CC_CONFIG409, NET_CC_CONFIG410, 
        NET_CC_CONFIG411, NET_CC_CONFIG412, NET_CC_CONFIG413, 
        NET_CC_CONFIG414, NET_CC_CONFIG415, NET_CC_CONFIG416, 
        NET_CC_CONFIG417, NET_CC_CONFIG418, NET_CC_CONFIG419, 
        NET_CC_CONFIG420, NET_CC_CONFIG421, NET_CC_CONFIG422, 
        NET_CC_CONFIG423, NET_CC_CONFIG424, NET_CC_CONFIG425, 
        NET_CC_CONFIG426, NET_CC_CONFIG427, NET_CC_CONFIG428, 
        NET_CC_CONFIG429, NET_CC_CONFIG430, NET_CC_CONFIG431, 
        NET_CC_CONFIG432, NET_CC_CONFIG433, NET_CC_CONFIG434, 
        NET_CC_CONFIG435, NET_CC_CONFIG436, NET_CC_CONFIG437, 
        NET_CC_CONFIG438, NET_CC_CONFIG439, NET_CC_CONFIG440, 
        NET_CC_CONFIG441, NET_CC_CONFIG442, NET_CC_CONFIG443, 
        NET_CC_CONFIG444, NET_CC_CONFIG445, NET_CC_CONFIG446, 
        NET_CC_CONFIG447, NET_CC_CONFIG448, NET_CC_CONFIG449, 
        NET_CC_CONFIG450, NET_CC_CONFIG451, NET_CC_CONFIG452, 
        NET_CC_CONFIG453, NET_CC_CONFIG454, NET_CC_CONFIG455, 
        NET_CC_CONFIG456, NET_CC_CONFIG457, NET_CC_CONFIG458, 
        CI_TO_CO459, NET_CC_CONFIG460, NET_CC_CONFIG461, 
        NET_CC_CONFIG462, NET_CC_CONFIG463, NET_CC_CONFIG464, 
        NET_CC_CONFIG465, NET_CC_CONFIG466, NET_CC_CONFIG467, 
        NET_CC_CONFIG468, NET_CC_CONFIG469, NET_CC_CONFIG470, 
        NET_CC_CONFIG471, NET_CC_CONFIG472, NET_CC_CONFIG473, 
        NET_CC_CONFIG474, NET_CC_CONFIG475, NET_CC_CONFIG476, 
        NET_CC_CONFIG477, NET_CC_CONFIG478, NET_CC_CONFIG479, 
        NET_CC_CONFIG480, NET_CC_CONFIG481, NET_CC_CONFIG482, 
        NET_CC_CONFIG483, NET_CC_CONFIG484, NET_CC_CONFIG485, 
        NET_CC_CONFIG486, NET_CC_CONFIG487, NET_CC_CONFIG488, 
        NET_CC_CONFIG489, NET_CC_CONFIG490, NET_CC_CONFIG491, 
        NET_CC_CONFIG492, NET_CC_CONFIG493, NET_CC_CONFIG494, 
        NET_CC_CONFIG495, NET_CC_CONFIG496, NET_CC_CONFIG497, 
        NET_CC_CONFIG498, NET_CC_CONFIG499, NET_CC_CONFIG500, 
        NET_CC_CONFIG501, NET_CC_CONFIG502, NET_CC_CONFIG503, 
        NET_CC_CONFIG504, NET_CC_CONFIG505, NET_CC_CONFIG506, 
        NET_CC_CONFIG507, NET_CC_CONFIG508, NET_CC_CONFIG509, 
        NET_CC_CONFIG510, NET_CC_CONFIG511, NET_CC_CONFIG512, 
        NET_CC_CONFIG513, NET_CC_CONFIG514, NET_CC_CONFIG515, 
        NET_CC_CONFIG516, NET_CC_CONFIG517, NET_CC_CONFIG518, 
        NET_CC_CONFIG519, NET_CC_CONFIG520, NET_CC_CONFIG521, 
        NET_CC_CONFIG522, NET_CC_CONFIG523, CI_TO_CO524, 
        CI_TO_CO525, NET_CC_CONFIG526, NET_CC_CONFIG527, 
        NET_CC_CONFIG528, NET_CC_CONFIG529, NET_CC_CONFIG530, 
        NET_CC_CONFIG531, NET_CC_CONFIG532, NET_CC_CONFIG533, 
        NET_CC_CONFIG534, NET_CC_CONFIG535, NET_CC_CONFIG536, 
        NET_CC_CONFIG537, NET_CC_CONFIG538, NET_CC_CONFIG539, 
        NET_CC_CONFIG540, NET_CC_CONFIG541, NET_CC_CONFIG542, 
        NET_CC_CONFIG543, NET_CC_CONFIG544, NET_CC_CONFIG545, 
        NET_CC_CONFIG546, NET_CC_CONFIG547, NET_CC_CONFIG548, 
        NET_CC_CONFIG549, NET_CC_CONFIG550, NET_CC_CONFIG551, 
        NET_CC_CONFIG552, NET_CC_CONFIG553, NET_CC_CONFIG554, 
        NET_CC_CONFIG555, NET_CC_CONFIG556, NET_CC_CONFIG557, 
        NET_CC_CONFIG558, NET_CC_CONFIG559, NET_CC_CONFIG560, 
        NET_CC_CONFIG561, NET_CC_CONFIG562, NET_CC_CONFIG563, 
        NET_CC_CONFIG564, NET_CC_CONFIG565, NET_CC_CONFIG566, 
        NET_CC_CONFIG567, NET_CC_CONFIG568, NET_CC_CONFIG569, 
        NET_CC_CONFIG570, NET_CC_CONFIG571, NET_CC_CONFIG572, 
        NET_CC_CONFIG573, NET_CC_CONFIG574, NET_CC_CONFIG575, 
        NET_CC_CONFIG576, NET_CC_CONFIG577, NET_CC_CONFIG578, 
        NET_CC_CONFIG579, NET_CC_CONFIG580, NET_CC_CONFIG581, 
        NET_CC_CONFIG582, NET_CC_CONFIG583, NET_CC_CONFIG584, 
        NET_CC_CONFIG585, NET_CC_CONFIG586, NET_CC_CONFIG587, 
        NET_CC_CONFIG588, NET_CC_CONFIG589, NET_CC_CONFIG590, 
        NET_CC_CONFIG591, NET_CC_CONFIG592, NET_CC_CONFIG593, 
        NET_CC_CONFIG594, NET_CC_CONFIG595, NET_CC_CONFIG596, 
        NET_CC_CONFIG597, NET_CC_CONFIG598, NET_CC_CONFIG599, 
        NET_CC_CONFIG600, NET_CC_CONFIG601, NET_CC_CONFIG602, 
        NET_CC_CONFIG603, NET_CC_CONFIG604, NET_CC_CONFIG605, 
        NET_CC_CONFIG606, NET_CC_CONFIG607, NET_CC_CONFIG608, 
        NET_CC_CONFIG609, NET_CC_CONFIG610, NET_CC_CONFIG611, 
        NET_CC_CONFIG612, NET_CC_CONFIG613, NET_CC_CONFIG614, 
        NET_CC_CONFIG615, NET_CC_CONFIG616, NET_CC_CONFIG617, 
        NET_CC_CONFIG618, NET_CC_CONFIG619, NET_CC_CONFIG620, 
        NET_CC_CONFIG621, NET_CC_CONFIG622, NET_CC_CONFIG623, 
        NET_CC_CONFIG624, NET_CC_CONFIG625, NET_CC_CONFIG626, 
        NET_CC_CONFIG627, NET_CC_CONFIG628, NET_CC_CONFIG629, 
        NET_CC_CONFIG630, NET_CC_CONFIG631, NET_CC_CONFIG632, 
        NET_CC_CONFIG633, NET_CC_CONFIG634, NET_CC_CONFIG635, 
        NET_CC_CONFIG636, NET_CC_CONFIG637, NET_CC_CONFIG638, 
        NET_CC_CONFIG639, NET_CC_CONFIG640, NET_CC_CONFIG641, 
        NET_CC_CONFIG642, NET_CC_CONFIG643, NET_CC_CONFIG644, 
        NET_CC_CONFIG645, NET_CC_CONFIG646, NET_CC_CONFIG647, 
        NET_CC_CONFIG648, NET_CC_CONFIG649, NET_CC_CONFIG650, 
        NET_CC_CONFIG651, NET_CC_CONFIG652, NET_CC_CONFIG653, 
        NET_CC_CONFIG654, NET_CC_CONFIG655, NET_CC_CONFIG656, 
        NET_CC_CONFIG657, NET_CC_CONFIG658, NET_CC_CONFIG659, 
        NET_CC_CONFIG660, NET_CC_CONFIG661, NET_CC_CONFIG662, 
        NET_CC_CONFIG663, NET_CC_CONFIG664, NET_CC_CONFIG665, 
        NET_CC_CONFIG666, NET_CC_CONFIG667, NET_CC_CONFIG668, 
        NET_CC_CONFIG669, NET_CC_CONFIG670, NET_CC_CONFIG671, 
        NET_CC_CONFIG672, NET_CC_CONFIG673, NET_CC_CONFIG674, 
        NET_CC_CONFIG675, NET_CC_CONFIG676, NET_CC_CONFIG677, 
        NET_CC_CONFIG678, NET_CC_CONFIG679, NET_CC_CONFIG680, 
        NET_CC_CONFIG681, NET_CC_CONFIG682, NET_CC_CONFIG683, 
        NET_CC_CONFIG684, NET_CC_CONFIG685, CI_TO_CO686, 
        NET_CC_CONFIG687, NET_CC_CONFIG688, NET_CC_CONFIG689, 
        NET_CC_CONFIG690, NET_CC_CONFIG691, NET_CC_CONFIG692, 
        NET_CC_CONFIG693, NET_CC_CONFIG694, NET_CC_CONFIG695, 
        NET_CC_CONFIG696, NET_CC_CONFIG697, NET_CC_CONFIG698, 
        NET_CC_CONFIG699, NET_CC_CONFIG700, NET_CC_CONFIG701, 
        NET_CC_CONFIG702, NET_CC_CONFIG703, NET_CC_CONFIG704, 
        NET_CC_CONFIG705, NET_CC_CONFIG706, NET_CC_CONFIG707, 
        NET_CC_CONFIG708, NET_CC_CONFIG709, NET_CC_CONFIG710, 
        NET_CC_CONFIG711, NET_CC_CONFIG712, NET_CC_CONFIG713, 
        NET_CC_CONFIG714, NET_CC_CONFIG715, NET_CC_CONFIG716, 
        NET_CC_CONFIG717, NET_CC_CONFIG718, NET_CC_CONFIG719, 
        NET_CC_CONFIG720, NET_CC_CONFIG721, NET_CC_CONFIG722, 
        NET_CC_CONFIG723, NET_CC_CONFIG724, NET_CC_CONFIG725, 
        NET_CC_CONFIG726, NET_CC_CONFIG727, NET_CC_CONFIG728, 
        NET_CC_CONFIG729, NET_CC_CONFIG730, NET_CC_CONFIG731, 
        NET_CC_CONFIG732, NET_CC_CONFIG733, NET_CC_CONFIG734, 
        NET_CC_CONFIG735, NET_CC_CONFIG736, NET_CC_CONFIG737, 
        NET_CC_CONFIG738, NET_CC_CONFIG739, NET_CC_CONFIG740, 
        NET_CC_CONFIG741, NET_CC_CONFIG742, NET_CC_CONFIG743, 
        NET_CC_CONFIG744, NET_CC_CONFIG745, NET_CC_CONFIG746, 
        NET_CC_CONFIG747, NET_CC_CONFIG748, NET_CC_CONFIG749, 
        NET_CC_CONFIG750, CI_TO_CO751, CI_TO_CO752, 
        NET_CC_CONFIG753, NET_CC_CONFIG754, NET_CC_CONFIG755, 
        NET_CC_CONFIG756, NET_CC_CONFIG757, NET_CC_CONFIG758, 
        NET_CC_CONFIG759, NET_CC_CONFIG760, NET_CC_CONFIG761, 
        NET_CC_CONFIG762, NET_CC_CONFIG763, NET_CC_CONFIG764, 
        NET_CC_CONFIG765, NET_CC_CONFIG766, NET_CC_CONFIG767, 
        NET_CC_CONFIG768, NET_CC_CONFIG769, NET_CC_CONFIG770, 
        NET_CC_CONFIG771, NET_CC_CONFIG772, NET_CC_CONFIG773, 
        NET_CC_CONFIG774, NET_CC_CONFIG775, NET_CC_CONFIG776, 
        NET_CC_CONFIG777, NET_CC_CONFIG778, NET_CC_CONFIG779, 
        NET_CC_CONFIG780, NET_CC_CONFIG781, NET_CC_CONFIG782, 
        NET_CC_CONFIG783, NET_CC_CONFIG784, NET_CC_CONFIG785, 
        NET_CC_CONFIG786, NET_CC_CONFIG787, NET_CC_CONFIG788, 
        NET_CC_CONFIG789, NET_CC_CONFIG790, NET_CC_CONFIG791, 
        NET_CC_CONFIG792, NET_CC_CONFIG793, NET_CC_CONFIG794, 
        NET_CC_CONFIG795, NET_CC_CONFIG796, NET_CC_CONFIG797, 
        NET_CC_CONFIG798, NET_CC_CONFIG799, NET_CC_CONFIG800, 
        NET_CC_CONFIG801, NET_CC_CONFIG802, NET_CC_CONFIG803, 
        NET_CC_CONFIG804, NET_CC_CONFIG805, NET_CC_CONFIG806, 
        NET_CC_CONFIG807, NET_CC_CONFIG808, NET_CC_CONFIG809, 
        NET_CC_CONFIG810, NET_CC_CONFIG811, NET_CC_CONFIG812, 
        NET_CC_CONFIG813, NET_CC_CONFIG814, NET_CC_CONFIG815, 
        NET_CC_CONFIG816, NET_CC_CONFIG817, NET_CC_CONFIG818, 
        NET_CC_CONFIG819, NET_CC_CONFIG820, NET_CC_CONFIG821, 
        NET_CC_CONFIG822, NET_CC_CONFIG823, NET_CC_CONFIG824, 
        NET_CC_CONFIG825, NET_CC_CONFIG826, NET_CC_CONFIG827, 
        NET_CC_CONFIG828, NET_CC_CONFIG829, NET_CC_CONFIG830, 
        NET_CC_CONFIG831, NET_CC_CONFIG832, NET_CC_CONFIG833, 
        NET_CC_CONFIG834, NET_CC_CONFIG835, NET_CC_CONFIG836, 
        NET_CC_CONFIG837, NET_CC_CONFIG838, NET_CC_CONFIG839, 
        NET_CC_CONFIG840, NET_CC_CONFIG841, NET_CC_CONFIG842, 
        NET_CC_CONFIG843, NET_CC_CONFIG844, NET_CC_CONFIG845, 
        NET_CC_CONFIG846, NET_CC_CONFIG847, NET_CC_CONFIG848, 
        NET_CC_CONFIG849, NET_CC_CONFIG850, NET_CC_CONFIG851, 
        NET_CC_CONFIG852, NET_CC_CONFIG853, NET_CC_CONFIG854, 
        NET_CC_CONFIG855, NET_CC_CONFIG856, NET_CC_CONFIG857, 
        NET_CC_CONFIG858, NET_CC_CONFIG859, NET_CC_CONFIG860, 
        NET_CC_CONFIG861, NET_CC_CONFIG862, NET_CC_CONFIG863, 
        NET_CC_CONFIG864, NET_CC_CONFIG865, NET_CC_CONFIG866, 
        NET_CC_CONFIG867, NET_CC_CONFIG868, NET_CC_CONFIG869, 
        NET_CC_CONFIG870, NET_CC_CONFIG871, NET_CC_CONFIG872, 
        NET_CC_CONFIG873, NET_CC_CONFIG874, NET_CC_CONFIG875, 
        NET_CC_CONFIG876, NET_CC_CONFIG877, NET_CC_CONFIG878, 
        NET_CC_CONFIG879, NET_CC_CONFIG880, NET_CC_CONFIG881, 
        NET_CC_CONFIG882, NET_CC_CONFIG883, NET_CC_CONFIG884, 
        NET_CC_CONFIG885, NET_CC_CONFIG886, NET_CC_CONFIG887, 
        NET_CC_CONFIG888, NET_CC_CONFIG889, NET_CC_CONFIG890, 
        NET_CC_CONFIG891, NET_CC_CONFIG892, NET_CC_CONFIG893, 
        NET_CC_CONFIG894, NET_CC_CONFIG895, NET_CC_CONFIG896, 
        NET_CC_CONFIG897, NET_CC_CONFIG898, NET_CC_CONFIG899, 
        NET_CC_CONFIG900, NET_CC_CONFIG901, NET_CC_CONFIG902, 
        NET_CC_CONFIG903, NET_CC_CONFIG904, NET_CC_CONFIG905, 
        NET_CC_CONFIG906, NET_CC_CONFIG907, NET_CC_CONFIG908, 
        NET_CC_CONFIG909, NET_CC_CONFIG910, NET_CC_CONFIG911, 
        NET_CC_CONFIG912, NET_CC_CONFIG913, NET_CC_CONFIG914, 
        NET_CC_CONFIG915, NET_CC_CONFIG916, NET_CC_CONFIG917, 
        NET_CC_CONFIG918, NET_CC_CONFIG919, NET_CC_CONFIG920, 
        CI_TO_CO921, CI_TO_CO922, CI_TO_CO923, NET_CC_CONFIG924, 
        NET_CC_CONFIG925, NET_CC_CONFIG926, NET_CC_CONFIG927, 
        NET_CC_CONFIG928, NET_CC_CONFIG929, NET_CC_CONFIG930, 
        NET_CC_CONFIG931, NET_CC_CONFIG932, NET_CC_CONFIG933, 
        NET_CC_CONFIG934, NET_CC_CONFIG935, NET_CC_CONFIG936, 
        NET_CC_CONFIG937, NET_CC_CONFIG938, NET_CC_CONFIG939, 
        NET_CC_CONFIG940, NET_CC_CONFIG941, NET_CC_CONFIG942, 
        NET_CC_CONFIG943, NET_CC_CONFIG944, NET_CC_CONFIG945, 
        NET_CC_CONFIG946, NET_CC_CONFIG947, NET_CC_CONFIG948, 
        NET_CC_CONFIG949, NET_CC_CONFIG950, NET_CC_CONFIG951, 
        NET_CC_CONFIG952, NET_CC_CONFIG953, NET_CC_CONFIG954, 
        NET_CC_CONFIG955, NET_CC_CONFIG956, NET_CC_CONFIG957, 
        NET_CC_CONFIG958, NET_CC_CONFIG959, NET_CC_CONFIG960, 
        NET_CC_CONFIG961, NET_CC_CONFIG962, NET_CC_CONFIG963, 
        NET_CC_CONFIG964, NET_CC_CONFIG965, NET_CC_CONFIG966, 
        NET_CC_CONFIG967, NET_CC_CONFIG968, NET_CC_CONFIG969, 
        NET_CC_CONFIG970, NET_CC_CONFIG971, NET_CC_CONFIG972, 
        NET_CC_CONFIG973, NET_CC_CONFIG974, NET_CC_CONFIG975, 
        NET_CC_CONFIG976, NET_CC_CONFIG977, NET_CC_CONFIG978, 
        NET_CC_CONFIG979, NET_CC_CONFIG980, NET_CC_CONFIG981, 
        NET_CC_CONFIG982, NET_CC_CONFIG983, NET_CC_CONFIG984, 
        NET_CC_CONFIG985, NET_CC_CONFIG986, NET_CC_CONFIG987, 
        NET_CC_CONFIG988, NET_CC_CONFIG989, NET_CC_CONFIG990, 
        NET_CC_CONFIG991, NET_CC_CONFIG992, NET_CC_CONFIG993, 
        NET_CC_CONFIG994, NET_CC_CONFIG995, NET_CC_CONFIG996, 
        NET_CC_CONFIG997, NET_CC_CONFIG998, NET_CC_CONFIG999, 
        NET_CC_CONFIG1000, NET_CC_CONFIG1001, NET_CC_CONFIG1002, 
        NET_CC_CONFIG1003, NET_CC_CONFIG1004, NET_CC_CONFIG1005, 
        NET_CC_CONFIG1006, NET_CC_CONFIG1007, NET_CC_CONFIG1008, 
        NET_CC_CONFIG1009, NET_CC_CONFIG1010, NET_CC_CONFIG1011, 
        NET_CC_CONFIG1012, NET_CC_CONFIG1013, NET_CC_CONFIG1014, 
        NET_CC_CONFIG1015, NET_CC_CONFIG1016, NET_CC_CONFIG1017, 
        NET_CC_CONFIG1018, NET_CC_CONFIG1019, NET_CC_CONFIG1020, 
        NET_CC_CONFIG1021, NET_CC_CONFIG1022, NET_CC_CONFIG1023, 
        NET_CC_CONFIG1024, NET_CC_CONFIG1025, NET_CC_CONFIG1026, 
        NET_CC_CONFIG1027, NET_CC_CONFIG1028, NET_CC_CONFIG1029, 
        NET_CC_CONFIG1030, NET_CC_CONFIG1031, NET_CC_CONFIG1032, 
        NET_CC_CONFIG1033, NET_CC_CONFIG1034, NET_CC_CONFIG1035, 
        NET_CC_CONFIG1036, NET_CC_CONFIG1037, NET_CC_CONFIG1038, 
        NET_CC_CONFIG1039, NET_CC_CONFIG1040, NET_CC_CONFIG1041, 
        NET_CC_CONFIG1042, NET_CC_CONFIG1043, NET_CC_CONFIG1044, 
        NET_CC_CONFIG1045, NET_CC_CONFIG1046, NET_CC_CONFIG1047, 
        NET_CC_CONFIG1048, NET_CC_CONFIG1049, NET_CC_CONFIG1050, 
        NET_CC_CONFIG1051, NET_CC_CONFIG1052, NET_CC_CONFIG1053, 
        NET_CC_CONFIG1054, NET_CC_CONFIG1055, NET_CC_CONFIG1056, 
        NET_CC_CONFIG1057, NET_CC_CONFIG1058, NET_CC_CONFIG1059, 
        NET_CC_CONFIG1060, NET_CC_CONFIG1061, NET_CC_CONFIG1062, 
        NET_CC_CONFIG1063, NET_CC_CONFIG1064, NET_CC_CONFIG1065, 
        NET_CC_CONFIG1066, NET_CC_CONFIG1067, NET_CC_CONFIG1068, 
        NET_CC_CONFIG1069, NET_CC_CONFIG1070, NET_CC_CONFIG1071, 
        NET_CC_CONFIG1072, NET_CC_CONFIG1073, NET_CC_CONFIG1074, 
        NET_CC_CONFIG1075, NET_CC_CONFIG1076, NET_CC_CONFIG1077, 
        NET_CC_CONFIG1078, NET_CC_CONFIG1079, NET_CC_CONFIG1080, 
        NET_CC_CONFIG1081, NET_CC_CONFIG1082, NET_CC_CONFIG1083, 
        NET_CC_CONFIG1084, NET_CC_CONFIG1085, NET_CC_CONFIG1086, 
        NET_CC_CONFIG1087, NET_CC_CONFIG1088, NET_CC_CONFIG1089, 
        NET_CC_CONFIG1090, NET_CC_CONFIG1091, NET_CC_CONFIG1092, 
        NET_CC_CONFIG1093, NET_CC_CONFIG1094, NET_CC_CONFIG1095, 
        NET_CC_CONFIG1096, NET_CC_CONFIG1097, NET_CC_CONFIG1098, 
        NET_CC_CONFIG1099, NET_CC_CONFIG1100, NET_CC_CONFIG1101, 
        NET_CC_CONFIG1102, NET_CC_CONFIG1103, NET_CC_CONFIG1104, 
        NET_CC_CONFIG1105, NET_CC_CONFIG1106, NET_CC_CONFIG1107, 
        NET_CC_CONFIG1108, NET_CC_CONFIG1109, NET_CC_CONFIG1110, 
        NET_CC_CONFIG1111, NET_CC_CONFIG1112, NET_CC_CONFIG1113, 
        NET_CC_CONFIG1114, NET_CC_CONFIG1115, NET_CC_CONFIG1116, 
        NET_CC_CONFIG1117, NET_CC_CONFIG1118, NET_CC_CONFIG1119, 
        NET_CC_CONFIG1120, NET_CC_CONFIG1121, NET_CC_CONFIG1122, 
        NET_CC_CONFIG1123, NET_CC_CONFIG1124, NET_CC_CONFIG1125, 
        NET_CC_CONFIG1126, NET_CC_CONFIG1127, NET_CC_CONFIG1128, 
        NET_CC_CONFIG1129, NET_CC_CONFIG1130, NET_CC_CONFIG1131, 
        NET_CC_CONFIG1132, NET_CC_CONFIG1133, NET_CC_CONFIG1134, 
        NET_CC_CONFIG1135, NET_CC_CONFIG1136, NET_CC_CONFIG1137, 
        NET_CC_CONFIG1138, NET_CC_CONFIG1139, NET_CC_CONFIG1140, 
        NET_CC_CONFIG1141, NET_CC_CONFIG1142, NET_CC_CONFIG1143, 
        NET_CC_CONFIG1144, NET_CC_CONFIG1145, NET_CC_CONFIG1146, 
        NET_CC_CONFIG1147, NET_CC_CONFIG1148, NET_CC_CONFIG1149, 
        NET_CC_CONFIG1150, NET_CC_CONFIG1151, NET_CC_CONFIG1152, 
        NET_CC_CONFIG1153, NET_CC_CONFIG1154, NET_CC_CONFIG1155, 
        NET_CC_CONFIG1156, NET_CC_CONFIG1157, NET_CC_CONFIG1158, 
        NET_CC_CONFIG1159, NET_CC_CONFIG1160, NET_CC_CONFIG1161, 
        NET_CC_CONFIG1162, NET_CC_CONFIG1163, CI_TO_CO1164, 
        NET_CC_CONFIG1165, NET_CC_CONFIG1166, NET_CC_CONFIG1167, 
        NET_CC_CONFIG1168, NET_CC_CONFIG1169, NET_CC_CONFIG1170, 
        NET_CC_CONFIG1171, NET_CC_CONFIG1172, NET_CC_CONFIG1173, 
        NET_CC_CONFIG1174, NET_CC_CONFIG1175, NET_CC_CONFIG1176, 
        NET_CC_CONFIG1177, NET_CC_CONFIG1178, NET_CC_CONFIG1179, 
        NET_CC_CONFIG1180, NET_CC_CONFIG1181, NET_CC_CONFIG1182, 
        NET_CC_CONFIG1183, NET_CC_CONFIG1184, NET_CC_CONFIG1185, 
        NET_CC_CONFIG1186, NET_CC_CONFIG1187, NET_CC_CONFIG1188, 
        NET_CC_CONFIG1189, NET_CC_CONFIG1190, NET_CC_CONFIG1191, 
        NET_CC_CONFIG1192, NET_CC_CONFIG1193, NET_CC_CONFIG1194, 
        NET_CC_CONFIG1195, NET_CC_CONFIG1196, NET_CC_CONFIG1197, 
        NET_CC_CONFIG1198, NET_CC_CONFIG1199, NET_CC_CONFIG1200, 
        NET_CC_CONFIG1201, NET_CC_CONFIG1202, NET_CC_CONFIG1203, 
        NET_CC_CONFIG1204, CI_TO_CO1205, CI_TO_CO1206, 
        NET_CC_CONFIG1207, NET_CC_CONFIG1208, NET_CC_CONFIG1209, 
        NET_CC_CONFIG1210, NET_CC_CONFIG1211, NET_CC_CONFIG1212, 
        NET_CC_CONFIG1213, NET_CC_CONFIG1214, NET_CC_CONFIG1215, 
        NET_CC_CONFIG1216, NET_CC_CONFIG1217, NET_CC_CONFIG1218, 
        NET_CC_CONFIG1219, NET_CC_CONFIG1220, NET_CC_CONFIG1221, 
        NET_CC_CONFIG1222, NET_CC_CONFIG1223, NET_CC_CONFIG1224, 
        NET_CC_CONFIG1225, NET_CC_CONFIG1226, NET_CC_CONFIG1227, 
        NET_CC_CONFIG1228, NET_CC_CONFIG1229, NET_CC_CONFIG1230, 
        NET_CC_CONFIG1231, NET_CC_CONFIG1232, NET_CC_CONFIG1233, 
        NET_CC_CONFIG1234, NET_CC_CONFIG1235, NET_CC_CONFIG1236, 
        NET_CC_CONFIG1237, NET_CC_CONFIG1238, NET_CC_CONFIG1239, 
        NET_CC_CONFIG1240, NET_CC_CONFIG1241, NET_CC_CONFIG1242, 
        NET_CC_CONFIG1243, NET_CC_CONFIG1244, NET_CC_CONFIG1245, 
        NET_CC_CONFIG1246, NET_CC_CONFIG1247, NET_CC_CONFIG1248, 
        NET_CC_CONFIG1249, NET_CC_CONFIG1250, NET_CC_CONFIG1251, 
        NET_CC_CONFIG1252, NET_CC_CONFIG1253, NET_CC_CONFIG1254, 
        NET_CC_CONFIG1255, NET_CC_CONFIG1256, NET_CC_CONFIG1257, 
        NET_CC_CONFIG1258, NET_CC_CONFIG1259, NET_CC_CONFIG1260, 
        NET_CC_CONFIG1261, NET_CC_CONFIG1262, NET_CC_CONFIG1263, 
        NET_CC_CONFIG1264, NET_CC_CONFIG1265, NET_CC_CONFIG1266, 
        NET_CC_CONFIG1267, NET_CC_CONFIG1268, NET_CC_CONFIG1269, 
        NET_CC_CONFIG1270, CI_TO_CO1271, CI_TO_CO1272, 
        CI_TO_CO1273, NET_CC_CONFIG1274, NET_CC_CONFIG1275, 
        NET_CC_CONFIG1276, NET_CC_CONFIG1277, NET_CC_CONFIG1278, 
        NET_CC_CONFIG1279, NET_CC_CONFIG1280, NET_CC_CONFIG1281, 
        NET_CC_CONFIG1282, NET_CC_CONFIG1283, NET_CC_CONFIG1284, 
        NET_CC_CONFIG1285, NET_CC_CONFIG1286, NET_CC_CONFIG1287, 
        NET_CC_CONFIG1288, NET_CC_CONFIG1289, NET_CC_CONFIG1290, 
        NET_CC_CONFIG1291, NET_CC_CONFIG1292, NET_CC_CONFIG1293, 
        NET_CC_CONFIG1294, NET_CC_CONFIG1295, NET_CC_CONFIG1296, 
        NET_CC_CONFIG1297, NET_CC_CONFIG1298, NET_CC_CONFIG1299, 
        NET_CC_CONFIG1300, NET_CC_CONFIG1301, NET_CC_CONFIG1302, 
        NET_CC_CONFIG1303, NET_CC_CONFIG1304, NET_CC_CONFIG1305, 
        NET_CC_CONFIG1306, NET_CC_CONFIG1307, NET_CC_CONFIG1308, 
        NET_CC_CONFIG1309, NET_CC_CONFIG1310, NET_CC_CONFIG1311, 
        NET_CC_CONFIG1312, NET_CC_CONFIG1313, NET_CC_CONFIG1314, 
        NET_CC_CONFIG1315, NET_CC_CONFIG1316, NET_CC_CONFIG1317, 
        NET_CC_CONFIG1318, NET_CC_CONFIG1319, NET_CC_CONFIG1320, 
        NET_CC_CONFIG1321, NET_CC_CONFIG1322, NET_CC_CONFIG1323, 
        NET_CC_CONFIG1324, NET_CC_CONFIG1325, NET_CC_CONFIG1326, 
        NET_CC_CONFIG1327, NET_CC_CONFIG1328, NET_CC_CONFIG1329, 
        NET_CC_CONFIG1330, NET_CC_CONFIG1331, NET_CC_CONFIG1332, 
        NET_CC_CONFIG1333, NET_CC_CONFIG1334, NET_CC_CONFIG1335, 
        NET_CC_CONFIG1336, NET_CC_CONFIG1337, NET_CC_CONFIG1338, 
        NET_CC_CONFIG1339, NET_CC_CONFIG1340, NET_CC_CONFIG1341, 
        NET_CC_CONFIG1342, NET_CC_CONFIG1343, NET_CC_CONFIG1344, 
        NET_CC_CONFIG1345, NET_CC_CONFIG1346, NET_CC_CONFIG1347, 
        NET_CC_CONFIG1348, NET_CC_CONFIG1349, NET_CC_CONFIG1350, 
        NET_CC_CONFIG1351, NET_CC_CONFIG1352, NET_CC_CONFIG1353, 
        NET_CC_CONFIG1354, NET_CC_CONFIG1355, NET_CC_CONFIG1356, 
        NET_CC_CONFIG1357, NET_CC_CONFIG1358, NET_CC_CONFIG1359, 
        NET_CC_CONFIG1360, NET_CC_CONFIG1361, NET_CC_CONFIG1362, 
        NET_CC_CONFIG1363, NET_CC_CONFIG1364, NET_CC_CONFIG1365, 
        NET_CC_CONFIG1366, NET_CC_CONFIG1367, NET_CC_CONFIG1368, 
        NET_CC_CONFIG1369, NET_CC_CONFIG1370, NET_CC_CONFIG1371, 
        NET_CC_CONFIG1372, NET_CC_CONFIG1373, NET_CC_CONFIG1374, 
        NET_CC_CONFIG1375, NET_CC_CONFIG1376, NET_CC_CONFIG1377, 
        NET_CC_CONFIG1378, NET_CC_CONFIG1379, NET_CC_CONFIG1380, 
        NET_CC_CONFIG1381, NET_CC_CONFIG1382, NET_CC_CONFIG1383, 
        NET_CC_CONFIG1384, NET_CC_CONFIG1385, NET_CC_CONFIG1386, 
        NET_CC_CONFIG1387, NET_CC_CONFIG1388, NET_CC_CONFIG1389, 
        NET_CC_CONFIG1390, NET_CC_CONFIG1391, NET_CC_CONFIG1392, 
        NET_CC_CONFIG1393, NET_CC_CONFIG1394, NET_CC_CONFIG1395, 
        NET_CC_CONFIG1396, NET_CC_CONFIG1397, NET_CC_CONFIG1398, 
        NET_CC_CONFIG1399, NET_CC_CONFIG1400, NET_CC_CONFIG1401, 
        NET_CC_CONFIG1402, NET_CC_CONFIG1403, NET_CC_CONFIG1404, 
        NET_CC_CONFIG1405, NET_CC_CONFIG1406, NET_CC_CONFIG1407, 
        NET_CC_CONFIG1408, NET_CC_CONFIG1409, NET_CC_CONFIG1410, 
        NET_CC_CONFIG1411, NET_CC_CONFIG1412, NET_CC_CONFIG1413, 
        NET_CC_CONFIG1414, NET_CC_CONFIG1415, NET_CC_CONFIG1416, 
        NET_CC_CONFIG1417, NET_CC_CONFIG1418, NET_CC_CONFIG1419, 
        NET_CC_CONFIG1420, NET_CC_CONFIG1421, NET_CC_CONFIG1422, 
        NET_CC_CONFIG1423, NET_CC_CONFIG1424, NET_CC_CONFIG1425, 
        NET_CC_CONFIG1426, NET_CC_CONFIG1427, NET_CC_CONFIG1428, 
        NET_CC_CONFIG1429, NET_CC_CONFIG1430, NET_CC_CONFIG1431, 
        NET_CC_CONFIG1432, NET_CC_CONFIG1433, NET_CC_CONFIG1434, 
        NET_CC_CONFIG1435, NET_CC_CONFIG1436, NET_CC_CONFIG1437, 
        NET_CC_CONFIG1438, NET_CC_CONFIG1439, NET_CC_CONFIG1440, 
        NET_CC_CONFIG1441, NET_CC_CONFIG1442, NET_CC_CONFIG1443, 
        NET_CC_CONFIG1444, NET_CC_CONFIG1445, NET_CC_CONFIG1446, 
        NET_CC_CONFIG1447, NET_CC_CONFIG1448, NET_CC_CONFIG1449, 
        NET_CC_CONFIG1450, NET_CC_CONFIG1451, NET_CC_CONFIG1452, 
        NET_CC_CONFIG1453, NET_CC_CONFIG1454, NET_CC_CONFIG1455, 
        NET_CC_CONFIG1456, NET_CC_CONFIG1457, NET_CC_CONFIG1458, 
        NET_CC_CONFIG1459, NET_CC_CONFIG1460, NET_CC_CONFIG1461, 
        NET_CC_CONFIG1462, NET_CC_CONFIG1463, NET_CC_CONFIG1464, 
        NET_CC_CONFIG1465, NET_CC_CONFIG1466, NET_CC_CONFIG1467, 
        NET_CC_CONFIG1468, NET_CC_CONFIG1469, NET_CC_CONFIG1470, 
        NET_CC_CONFIG1471, NET_CC_CONFIG1472, NET_CC_CONFIG1473, 
        NET_CC_CONFIG1474, NET_CC_CONFIG1475, NET_CC_CONFIG1476, 
        NET_CC_CONFIG1477, NET_CC_CONFIG1478, NET_CC_CONFIG1479, 
        NET_CC_CONFIG1480, NET_CC_CONFIG1481, NET_CC_CONFIG1482, 
        NET_CC_CONFIG1483, NET_CC_CONFIG1484, NET_CC_CONFIG1485, 
        NET_CC_CONFIG1486, NET_CC_CONFIG1487, NET_CC_CONFIG1488, 
        NET_CC_CONFIG1489, NET_CC_CONFIG1490, NET_CC_CONFIG1491, 
        NET_CC_CONFIG1492, NET_CC_CONFIG1493, NET_CC_CONFIG1494, 
        NET_CC_CONFIG1495, NET_CC_CONFIG1496, NET_CC_CONFIG1497, 
        NET_CC_CONFIG1498, NET_CC_CONFIG1499, NET_CC_CONFIG1500, 
        NET_CC_CONFIG1501, NET_CC_CONFIG1502, NET_CC_CONFIG1503, 
        NET_CC_CONFIG1504, NET_CC_CONFIG1505, NET_CC_CONFIG1506, 
        NET_CC_CONFIG1507, NET_CC_CONFIG1508, NET_CC_CONFIG1509, 
        NET_CC_CONFIG1510, NET_CC_CONFIG1511, NET_CC_CONFIG1512, 
        NET_CC_CONFIG1513, NET_CC_CONFIG1514, NET_CC_CONFIG1515, 
        NET_CC_CONFIG1516, NET_CC_CONFIG1517, NET_CC_CONFIG1518, 
        NET_CC_CONFIG1519, NET_CC_CONFIG1520, NET_CC_CONFIG1521, 
        NET_CC_CONFIG1522, NET_CC_CONFIG1523, NET_CC_CONFIG1524, 
        NET_CC_CONFIG1525, NET_CC_CONFIG1526, NET_CC_CONFIG1527, 
        NET_CC_CONFIG1528, NET_CC_CONFIG1529, NET_CC_CONFIG1530, 
        NET_CC_CONFIG1531, NET_CC_CONFIG1532, NET_CC_CONFIG1533, 
        NET_CC_CONFIG1534, NET_CC_CONFIG1535, NET_CC_CONFIG1536, 
        NET_CC_CONFIG1537, NET_CC_CONFIG1538, NET_CC_CONFIG1539, 
        NET_CC_CONFIG1540, NET_CC_CONFIG1541, NET_CC_CONFIG1542, 
        NET_CC_CONFIG1543, NET_CC_CONFIG1544, NET_CC_CONFIG1545, 
        NET_CC_CONFIG1546, NET_CC_CONFIG1547, NET_CC_CONFIG1548, 
        NET_CC_CONFIG1549, NET_CC_CONFIG1550, NET_CC_CONFIG1551, 
        NET_CC_CONFIG1552, NET_CC_CONFIG1553, NET_CC_CONFIG1554, 
        NET_CC_CONFIG1555, NET_CC_CONFIG1556, NET_CC_CONFIG1557, 
        NET_CC_CONFIG1558, NET_CC_CONFIG1559, NET_CC_CONFIG1560, 
        NET_CC_CONFIG1561, NET_CC_CONFIG1562, NET_CC_CONFIG1563, 
        NET_CC_CONFIG1564, NET_CC_CONFIG1565, NET_CC_CONFIG1566, 
        NET_CC_CONFIG1567, NET_CC_CONFIG1568, NET_CC_CONFIG1569, 
        NET_CC_CONFIG1570, NET_CC_CONFIG1571, NET_CC_CONFIG1572, 
        NET_CC_CONFIG1573, NET_CC_CONFIG1574, NET_CC_CONFIG1575, 
        NET_CC_CONFIG1576, NET_CC_CONFIG1577, NET_CC_CONFIG1578, 
        NET_CC_CONFIG1579, NET_CC_CONFIG1580, NET_CC_CONFIG1581, 
        NET_CC_CONFIG1582, NET_CC_CONFIG1583, NET_CC_CONFIG1584, 
        NET_CC_CONFIG1585, NET_CC_CONFIG1586, NET_CC_CONFIG1587, 
        NET_CC_CONFIG1588, NET_CC_CONFIG1589, NET_CC_CONFIG1590, 
        NET_CC_CONFIG1591, NET_CC_CONFIG1592, NET_CC_CONFIG1593, 
        NET_CC_CONFIG1594, NET_CC_CONFIG1595, NET_CC_CONFIG1596, 
        NET_CC_CONFIG1597, NET_CC_CONFIG1598, NET_CC_CONFIG1599, 
        NET_CC_CONFIG1600, NET_CC_CONFIG1601, NET_CC_CONFIG1602, 
        NET_CC_CONFIG1603, NET_CC_CONFIG1604, NET_CC_CONFIG1605, 
        NET_CC_CONFIG1606, NET_CC_CONFIG1607, NET_CC_CONFIG1608, 
        NET_CC_CONFIG1609, NET_CC_CONFIG1610, NET_CC_CONFIG1611, 
        NET_CC_CONFIG1612, NET_CC_CONFIG1613, NET_CC_CONFIG1614, 
        NET_CC_CONFIG1615, NET_CC_CONFIG1616, NET_CC_CONFIG1617, 
        NET_CC_CONFIG1618, NET_CC_CONFIG1619, NET_CC_CONFIG1620, 
        NET_CC_CONFIG1621, NET_CC_CONFIG1622, NET_CC_CONFIG1623, 
        NET_CC_CONFIG1624, NET_CC_CONFIG1625, NET_CC_CONFIG1626, 
        NET_CC_CONFIG1627, NET_CC_CONFIG1628, NET_CC_CONFIG1629, 
        NET_CC_CONFIG1630, NET_CC_CONFIG1631, NET_CC_CONFIG1632, 
        NET_CC_CONFIG1633, NET_CC_CONFIG1634, NET_CC_CONFIG1635, 
        NET_CC_CONFIG1636, NET_CC_CONFIG1637, NET_CC_CONFIG1638, 
        NET_CC_CONFIG1639, NET_CC_CONFIG1640, NET_CC_CONFIG1641, 
        NET_CC_CONFIG1642, NET_CC_CONFIG1643, NET_CC_CONFIG1644, 
        NET_CC_CONFIG1645, NET_CC_CONFIG1646, NET_CC_CONFIG1647, 
        NET_CC_CONFIG1648, NET_CC_CONFIG1649, NET_CC_CONFIG1650, 
        NET_CC_CONFIG1651, NET_CC_CONFIG1652, NET_CC_CONFIG1653, 
        NET_CC_CONFIG1654, NET_CC_CONFIG1655, NET_CC_CONFIG1656, 
        NET_CC_CONFIG1657, NET_CC_CONFIG1658, NET_CC_CONFIG1659, 
        NET_CC_CONFIG1660, NET_CC_CONFIG1661, NET_CC_CONFIG1662, 
        NET_CC_CONFIG1663, NET_CC_CONFIG1664, NET_CC_CONFIG1665, 
        NET_CC_CONFIG1666, NET_CC_CONFIG1667, NET_CC_CONFIG1668, 
        NET_CC_CONFIG1669, CI_TO_CO1670, NET_CC_CONFIG1671, 
        NET_CC_CONFIG1672, NET_CC_CONFIG1673, NET_CC_CONFIG1674, 
        NET_CC_CONFIG1675, NET_CC_CONFIG1676, NET_CC_CONFIG1677, 
        NET_CC_CONFIG1678, NET_CC_CONFIG1679, NET_CC_CONFIG1680, 
        NET_CC_CONFIG1681, NET_CC_CONFIG1682, NET_CC_CONFIG1683, 
        NET_CC_CONFIG1684, NET_CC_CONFIG1685, NET_CC_CONFIG1686, 
        NET_CC_CONFIG1687, NET_CC_CONFIG1688, NET_CC_CONFIG1689, 
        NET_CC_CONFIG1690, NET_CC_CONFIG1691, NET_CC_CONFIG1692, 
        NET_CC_CONFIG1693, NET_CC_CONFIG1694, NET_CC_CONFIG1695, 
        NET_CC_CONFIG1696, NET_CC_CONFIG1697, NET_CC_CONFIG1698, 
        NET_CC_CONFIG1699, NET_CC_CONFIG1700, NET_CC_CONFIG1701, 
        NET_CC_CONFIG1702, NET_CC_CONFIG1703, NET_CC_CONFIG1704, 
        NET_CC_CONFIG1705, NET_CC_CONFIG1706, NET_CC_CONFIG1707, 
        NET_CC_CONFIG1708, NET_CC_CONFIG1709, NET_CC_CONFIG1710, 
        NET_CC_CONFIG1711, NET_CC_CONFIG1712, NET_CC_CONFIG1713, 
        NET_CC_CONFIG1714, NET_CC_CONFIG1715, NET_CC_CONFIG1716, 
        NET_CC_CONFIG1717, NET_CC_CONFIG1718, NET_CC_CONFIG1719, 
        NET_CC_CONFIG1720, NET_CC_CONFIG1721, NET_CC_CONFIG1722, 
        NET_CC_CONFIG1723, NET_CC_CONFIG1724, NET_CC_CONFIG1725, 
        NET_CC_CONFIG1726, NET_CC_CONFIG1727, NET_CC_CONFIG1728, 
        NET_CC_CONFIG1729, NET_CC_CONFIG1730, NET_CC_CONFIG1731, 
        NET_CC_CONFIG1732, NET_CC_CONFIG1733, NET_CC_CONFIG1734, 
        NET_CC_CONFIG1735, NET_CC_CONFIG1736, NET_CC_CONFIG1737, 
        NET_CC_CONFIG1738, NET_CC_CONFIG1739, NET_CC_CONFIG1740, 
        NET_CC_CONFIG1741, NET_CC_CONFIG1742, NET_CC_CONFIG1743, 
        NET_CC_CONFIG1744, NET_CC_CONFIG1745, NET_CC_CONFIG1746, 
        NET_CC_CONFIG1747, NET_CC_CONFIG1748, NET_CC_CONFIG1749, 
        NET_CC_CONFIG1750, NET_CC_CONFIG1751, NET_CC_CONFIG1752, 
        NET_CC_CONFIG1753, NET_CC_CONFIG1754, NET_CC_CONFIG1755, 
        NET_CC_CONFIG1756, NET_CC_CONFIG1757, NET_CC_CONFIG1758, 
        NET_CC_CONFIG1759, NET_CC_CONFIG1760, NET_CC_CONFIG1761, 
        NET_CC_CONFIG1762, CI_TO_CO1763, NET_CC_CONFIG1764, 
        NET_CC_CONFIG1765, NET_CC_CONFIG1766, NET_CC_CONFIG1767, 
        NET_CC_CONFIG1768, NET_CC_CONFIG1769, NET_CC_CONFIG1770, 
        NET_CC_CONFIG1771, NET_CC_CONFIG1772, NET_CC_CONFIG1773, 
        NET_CC_CONFIG1774, NET_CC_CONFIG1775, NET_CC_CONFIG1776, 
        NET_CC_CONFIG1777, NET_CC_CONFIG1778, NET_CC_CONFIG1779, 
        NET_CC_CONFIG1780, NET_CC_CONFIG1781, NET_CC_CONFIG1782, 
        NET_CC_CONFIG1783, NET_CC_CONFIG1784, NET_CC_CONFIG1785, 
        NET_CC_CONFIG1786, NET_CC_CONFIG1787, NET_CC_CONFIG1788, 
        NET_CC_CONFIG1789, NET_CC_CONFIG1790, NET_CC_CONFIG1791, 
        NET_CC_CONFIG1792, NET_CC_CONFIG1793, NET_CC_CONFIG1794, 
        NET_CC_CONFIG1795, NET_CC_CONFIG1796, NET_CC_CONFIG1797, 
        NET_CC_CONFIG1798, NET_CC_CONFIG1799, NET_CC_CONFIG1800, 
        NET_CC_CONFIG1801, NET_CC_CONFIG1802, NET_CC_CONFIG1803, 
        NET_CC_CONFIG1804, NET_CC_CONFIG1805, NET_CC_CONFIG1806, 
        NET_CC_CONFIG1807, NET_CC_CONFIG1808, NET_CC_CONFIG1809, 
        NET_CC_CONFIG1810, NET_CC_CONFIG1811, NET_CC_CONFIG1812, 
        NET_CC_CONFIG1813, NET_CC_CONFIG1814, NET_CC_CONFIG1815, 
        NET_CC_CONFIG1816, NET_CC_CONFIG1817, NET_CC_CONFIG1818, 
        NET_CC_CONFIG1819, NET_CC_CONFIG1820, NET_CC_CONFIG1821, 
        NET_CC_CONFIG1822, NET_CC_CONFIG1823, NET_CC_CONFIG1824, 
        NET_CC_CONFIG1825, NET_CC_CONFIG1826, NET_CC_CONFIG1827, 
        CI_TO_CO1828, CI_TO_CO1829, NET_CC_CONFIG1830, 
        NET_CC_CONFIG1831, NET_CC_CONFIG1832, NET_CC_CONFIG1833, 
        NET_CC_CONFIG1834, NET_CC_CONFIG1835, NET_CC_CONFIG1836, 
        NET_CC_CONFIG1837, NET_CC_CONFIG1838, NET_CC_CONFIG1839, 
        NET_CC_CONFIG1840, NET_CC_CONFIG1841, NET_CC_CONFIG1842, 
        NET_CC_CONFIG1843, NET_CC_CONFIG1844, NET_CC_CONFIG1845, 
        NET_CC_CONFIG1846, NET_CC_CONFIG1847, NET_CC_CONFIG1848, 
        NET_CC_CONFIG1849, NET_CC_CONFIG1850, NET_CC_CONFIG1851, 
        NET_CC_CONFIG1852, NET_CC_CONFIG1853, NET_CC_CONFIG1854, 
        NET_CC_CONFIG1855, NET_CC_CONFIG1856, NET_CC_CONFIG1857, 
        NET_CC_CONFIG1858, NET_CC_CONFIG1859, NET_CC_CONFIG1860, 
        NET_CC_CONFIG1861, NET_CC_CONFIG1862, NET_CC_CONFIG1863, 
        NET_CC_CONFIG1864, NET_CC_CONFIG1865, NET_CC_CONFIG1866, 
        NET_CC_CONFIG1867, NET_CC_CONFIG1868, NET_CC_CONFIG1869, 
        NET_CC_CONFIG1870, NET_CC_CONFIG1871, NET_CC_CONFIG1872, 
        NET_CC_CONFIG1873, NET_CC_CONFIG1874, NET_CC_CONFIG1875, 
        NET_CC_CONFIG1876, NET_CC_CONFIG1877, NET_CC_CONFIG1878, 
        NET_CC_CONFIG1879, NET_CC_CONFIG1880, NET_CC_CONFIG1881, 
        NET_CC_CONFIG1882, NET_CC_CONFIG1883, NET_CC_CONFIG1884, 
        NET_CC_CONFIG1885, NET_CC_CONFIG1886, NET_CC_CONFIG1887, 
        NET_CC_CONFIG1888, NET_CC_CONFIG1889, NET_CC_CONFIG1890, 
        NET_CC_CONFIG1891, NET_CC_CONFIG1892, NET_CC_CONFIG1893, 
        NET_CC_CONFIG1894, NET_CC_CONFIG1895, NET_CC_CONFIG1896, 
        NET_CC_CONFIG1897, NET_CC_CONFIG1898, NET_CC_CONFIG1899, 
        NET_CC_CONFIG1900, NET_CC_CONFIG1901, NET_CC_CONFIG1902, 
        NET_CC_CONFIG1903, NET_CC_CONFIG1904, NET_CC_CONFIG1905, 
        NET_CC_CONFIG1906, NET_CC_CONFIG1907, NET_CC_CONFIG1908, 
        NET_CC_CONFIG1909, NET_CC_CONFIG1910, NET_CC_CONFIG1911, 
        NET_CC_CONFIG1912, NET_CC_CONFIG1913, NET_CC_CONFIG1914, 
        NET_CC_CONFIG1915, NET_CC_CONFIG1916, NET_CC_CONFIG1917, 
        NET_CC_CONFIG1918, NET_CC_CONFIG1919, NET_CC_CONFIG1920, 
        NET_CC_CONFIG1921, NET_CC_CONFIG1922, NET_CC_CONFIG1923, 
        NET_CC_CONFIG1924, NET_CC_CONFIG1925, NET_CC_CONFIG1926, 
        NET_CC_CONFIG1927, NET_CC_CONFIG1928, NET_CC_CONFIG1929, 
        NET_CC_CONFIG1930, NET_CC_CONFIG1931, NET_CC_CONFIG1932, 
        NET_CC_CONFIG1933, NET_CC_CONFIG1934, NET_CC_CONFIG1935, 
        NET_CC_CONFIG1936, NET_CC_CONFIG1937, NET_CC_CONFIG1938, 
        NET_CC_CONFIG1939, NET_CC_CONFIG1940, NET_CC_CONFIG1941, 
        NET_CC_CONFIG1942, NET_CC_CONFIG1943, NET_CC_CONFIG1944, 
        NET_CC_CONFIG1945, NET_CC_CONFIG1946, NET_CC_CONFIG1947, 
        NET_CC_CONFIG1948, NET_CC_CONFIG1949, NET_CC_CONFIG1950, 
        NET_CC_CONFIG1951, NET_CC_CONFIG1952, NET_CC_CONFIG1953, 
        NET_CC_CONFIG1954, NET_CC_CONFIG1955, NET_CC_CONFIG1956, 
        NET_CC_CONFIG1957, NET_CC_CONFIG1958, NET_CC_CONFIG1959, 
        NET_CC_CONFIG1960, NET_CC_CONFIG1961, NET_CC_CONFIG1962, 
        NET_CC_CONFIG1963, NET_CC_CONFIG1964, NET_CC_CONFIG1965, 
        NET_CC_CONFIG1966, NET_CC_CONFIG1967, NET_CC_CONFIG1968, 
        NET_CC_CONFIG1969, NET_CC_CONFIG1970, NET_CC_CONFIG1971, 
        NET_CC_CONFIG1972, NET_CC_CONFIG1973, NET_CC_CONFIG1974, 
        NET_CC_CONFIG1975, NET_CC_CONFIG1976, NET_CC_CONFIG1977, 
        NET_CC_CONFIG1978, NET_CC_CONFIG1979, NET_CC_CONFIG1980, 
        NET_CC_CONFIG1981, NET_CC_CONFIG1982, NET_CC_CONFIG1983, 
        NET_CC_CONFIG1984, NET_CC_CONFIG1985, NET_CC_CONFIG1986, 
        NET_CC_CONFIG1987, NET_CC_CONFIG1988, NET_CC_CONFIG1989, 
        NET_CC_CONFIG1990, NET_CC_CONFIG1991, NET_CC_CONFIG1992, 
        NET_CC_CONFIG1993, NET_CC_CONFIG1994, NET_CC_CONFIG1995, 
        NET_CC_CONFIG1996, NET_CC_CONFIG1997, NET_CC_CONFIG1998, 
        NET_CC_CONFIG1999, NET_CC_CONFIG2000, NET_CC_CONFIG2001, 
        NET_CC_CONFIG2002, NET_CC_CONFIG2003, NET_CC_CONFIG2004, 
        NET_CC_CONFIG2005, NET_CC_CONFIG2006, NET_CC_CONFIG2007, 
        NET_CC_CONFIG2008, NET_CC_CONFIG2009, NET_CC_CONFIG2010, 
        NET_CC_CONFIG2011, NET_CC_CONFIG2012, NET_CC_CONFIG2013, 
        NET_CC_CONFIG2014, NET_CC_CONFIG2015, NET_CC_CONFIG2016, 
        NET_CC_CONFIG2017, NET_CC_CONFIG2018, NET_CC_CONFIG2019, 
        NET_CC_CONFIG2020, NET_CC_CONFIG2021, NET_CC_CONFIG2022, 
        NET_CC_CONFIG2023, NET_CC_CONFIG2024, NET_CC_CONFIG2025, 
        NET_CC_CONFIG2026, NET_CC_CONFIG2027, NET_CC_CONFIG2028, 
        NET_CC_CONFIG2029, NET_CC_CONFIG2030, NET_CC_CONFIG2031, 
        NET_CC_CONFIG2032, NET_CC_CONFIG2033, NET_CC_CONFIG2034, 
        NET_CC_CONFIG2035, NET_CC_CONFIG2036, NET_CC_CONFIG2037, 
        NET_CC_CONFIG2038, NET_CC_CONFIG2039, NET_CC_CONFIG2040, 
        NET_CC_CONFIG2041, NET_CC_CONFIG2042, NET_CC_CONFIG2043, 
        NET_CC_CONFIG2044, NET_CC_CONFIG2045, NET_CC_CONFIG2046, 
        NET_CC_CONFIG2047, NET_CC_CONFIG2048, NET_CC_CONFIG2049, 
        NET_CC_CONFIG2050, NET_CC_CONFIG2051, NET_CC_CONFIG2052, 
        NET_CC_CONFIG2053, NET_CC_CONFIG2054, NET_CC_CONFIG2055, 
        NET_CC_CONFIG2056, NET_CC_CONFIG2057, CI_TO_CO2058, 
        NET_CC_CONFIG2059, NET_CC_CONFIG2060, NET_CC_CONFIG2061, 
        NET_CC_CONFIG2062, NET_CC_CONFIG2063, NET_CC_CONFIG2064, 
        NET_CC_CONFIG2065, NET_CC_CONFIG2066, NET_CC_CONFIG2067, 
        NET_CC_CONFIG2068, NET_CC_CONFIG2069, NET_CC_CONFIG2070, 
        NET_CC_CONFIG2071, NET_CC_CONFIG2072, NET_CC_CONFIG2073, 
        NET_CC_CONFIG2074, NET_CC_CONFIG2075, NET_CC_CONFIG2076, 
        NET_CC_CONFIG2077, NET_CC_CONFIG2078, NET_CC_CONFIG2079, 
        NET_CC_CONFIG2080, NET_CC_CONFIG2081, NET_CC_CONFIG2082, 
        NET_CC_CONFIG2083, NET_CC_CONFIG2084, NET_CC_CONFIG2085, 
        NET_CC_CONFIG2086, NET_CC_CONFIG2087, NET_CC_CONFIG2088, 
        NET_CC_CONFIG2089, NET_CC_CONFIG2090, NET_CC_CONFIG2091, 
        NET_CC_CONFIG2092, NET_CC_CONFIG2093, NET_CC_CONFIG2094, 
        NET_CC_CONFIG2095, NET_CC_CONFIG2096, NET_CC_CONFIG2097, 
        NET_CC_CONFIG2098, CI_TO_CO2099, NET_CC_CONFIG2100, 
        NET_CC_CONFIG2101, NET_CC_CONFIG2102, NET_CC_CONFIG2103, 
        NET_CC_CONFIG2104, NET_CC_CONFIG2105, NET_CC_CONFIG2106, 
        NET_CC_CONFIG2107, NET_CC_CONFIG2108, NET_CC_CONFIG2109, 
        NET_CC_CONFIG2110, NET_CC_CONFIG2111, NET_CC_CONFIG2112, 
        NET_CC_CONFIG2113, NET_CC_CONFIG2114, NET_CC_CONFIG2115, 
        NET_CC_CONFIG2116, NET_CC_CONFIG2117, NET_CC_CONFIG2118, 
        NET_CC_CONFIG2119, NET_CC_CONFIG2120, NET_CC_CONFIG2121, 
        NET_CC_CONFIG2122, NET_CC_CONFIG2123, NET_CC_CONFIG2124, 
        NET_CC_CONFIG2125, NET_CC_CONFIG2126, NET_CC_CONFIG2127, 
        NET_CC_CONFIG2128, NET_CC_CONFIG2129, NET_CC_CONFIG2130, 
        NET_CC_CONFIG2131, NET_CC_CONFIG2132, NET_CC_CONFIG2133, 
        NET_CC_CONFIG2134, NET_CC_CONFIG2135, NET_CC_CONFIG2136, 
        NET_CC_CONFIG2137, NET_CC_CONFIG2138, NET_CC_CONFIG2139, 
        NET_CC_CONFIG2140, NET_CC_CONFIG2141, NET_CC_CONFIG2142, 
        NET_CC_CONFIG2143, NET_CC_CONFIG2144, NET_CC_CONFIG2145, 
        NET_CC_CONFIG2146, NET_CC_CONFIG2147, NET_CC_CONFIG2148, 
        NET_CC_CONFIG2149, NET_CC_CONFIG2150, NET_CC_CONFIG2151, 
        NET_CC_CONFIG2152, NET_CC_CONFIG2153, NET_CC_CONFIG2154, 
        NET_CC_CONFIG2155, NET_CC_CONFIG2156, NET_CC_CONFIG2157, 
        NET_CC_CONFIG2158, NET_CC_CONFIG2159, NET_CC_CONFIG2160, 
        NET_CC_CONFIG2161, NET_CC_CONFIG2162, NET_CC_CONFIG2163, 
        NET_CC_CONFIG2164, NET_CC_CONFIG2165, NET_CC_CONFIG2166, 
        NET_CC_CONFIG2167, NET_CC_CONFIG2168, NET_CC_CONFIG2169, 
        NET_CC_CONFIG2170, NET_CC_CONFIG2171, NET_CC_CONFIG2172, 
        NET_CC_CONFIG2173, NET_CC_CONFIG2174, NET_CC_CONFIG2175, 
        NET_CC_CONFIG2176, NET_CC_CONFIG2177, NET_CC_CONFIG2178, 
        NET_CC_CONFIG2179, NET_CC_CONFIG2180, NET_CC_CONFIG2181, 
        NET_CC_CONFIG2182, NET_CC_CONFIG2183, NET_CC_CONFIG2184, 
        NET_CC_CONFIG2185, NET_CC_CONFIG2186, NET_CC_CONFIG2187, 
        NET_CC_CONFIG2188, NET_CC_CONFIG2189, NET_CC_CONFIG2190, 
        NET_CC_CONFIG2191, NET_CC_CONFIG2192, NET_CC_CONFIG2193, 
        NET_CC_CONFIG2194, NET_CC_CONFIG2195, NET_CC_CONFIG2196, 
        NET_CC_CONFIG2197, NET_CC_CONFIG2198, NET_CC_CONFIG2199, 
        NET_CC_CONFIG2200, NET_CC_CONFIG2201, NET_CC_CONFIG2202, 
        NET_CC_CONFIG2203, NET_CC_CONFIG2204, NET_CC_CONFIG2205, 
        NET_CC_CONFIG2206, NET_CC_CONFIG2207, NET_CC_CONFIG2208, 
        NET_CC_CONFIG2209, NET_CC_CONFIG2210, NET_CC_CONFIG2211, 
        NET_CC_CONFIG2212, NET_CC_CONFIG2213, NET_CC_CONFIG2214, 
        NET_CC_CONFIG2215, NET_CC_CONFIG2216, NET_CC_CONFIG2217, 
        NET_CC_CONFIG2218, NET_CC_CONFIG2219, NET_CC_CONFIG2220, 
        NET_CC_CONFIG2221, NET_CC_CONFIG2222, NET_CC_CONFIG2223, 
        NET_CC_CONFIG2224, NET_CC_CONFIG2225, NET_CC_CONFIG2226, 
        NET_CC_CONFIG2227, NET_CC_CONFIG2228, NET_CC_CONFIG2229, 
        NET_CC_CONFIG2230, NET_CC_CONFIG2231, NET_CC_CONFIG2232, 
        NET_CC_CONFIG2233, NET_CC_CONFIG2234, NET_CC_CONFIG2235, 
        NET_CC_CONFIG2236, NET_CC_CONFIG2237, NET_CC_CONFIG2238, 
        NET_CC_CONFIG2239, NET_CC_CONFIG2240, NET_CC_CONFIG2241, 
        NET_CC_CONFIG2242, NET_CC_CONFIG2243, NET_CC_CONFIG2244, 
        NET_CC_CONFIG2245, NET_CC_CONFIG2246, NET_CC_CONFIG2247, 
        CI_TO_CO2248, CI_TO_CO2249, NET_CC_CONFIG2250, 
        NET_CC_CONFIG2251, NET_CC_CONFIG2252, NET_CC_CONFIG2253, 
        NET_CC_CONFIG2254, NET_CC_CONFIG2255, NET_CC_CONFIG2256, 
        NET_CC_CONFIG2257, NET_CC_CONFIG2258, NET_CC_CONFIG2259, 
        NET_CC_CONFIG2260, NET_CC_CONFIG2261, NET_CC_CONFIG2262, 
        NET_CC_CONFIG2263, NET_CC_CONFIG2264, NET_CC_CONFIG2265, 
        NET_CC_CONFIG2266, NET_CC_CONFIG2267, NET_CC_CONFIG2268, 
        NET_CC_CONFIG2269, NET_CC_CONFIG2270, NET_CC_CONFIG2271, 
        NET_CC_CONFIG2272, NET_CC_CONFIG2273, NET_CC_CONFIG2274, 
        NET_CC_CONFIG2275, NET_CC_CONFIG2276, NET_CC_CONFIG2277, 
        NET_CC_CONFIG2278, NET_CC_CONFIG2279, NET_CC_CONFIG2280, 
        NET_CC_CONFIG2281, NET_CC_CONFIG2282, NET_CC_CONFIG2283, 
        NET_CC_CONFIG2284, NET_CC_CONFIG2285, NET_CC_CONFIG2286, 
        NET_CC_CONFIG2287, NET_CC_CONFIG2288, NET_CC_CONFIG2289, 
        NET_CC_CONFIG2290, NET_CC_CONFIG2291, NET_CC_CONFIG2292, 
        NET_CC_CONFIG2293, NET_CC_CONFIG2294, NET_CC_CONFIG2295, 
        NET_CC_CONFIG2296, NET_CC_CONFIG2297, NET_CC_CONFIG2298, 
        NET_CC_CONFIG2299, NET_CC_CONFIG2300, NET_CC_CONFIG2301, 
        NET_CC_CONFIG2302, NET_CC_CONFIG2303, NET_CC_CONFIG2304, 
        NET_CC_CONFIG2305, NET_CC_CONFIG2306, NET_CC_CONFIG2307, 
        NET_CC_CONFIG2308, NET_CC_CONFIG2309, NET_CC_CONFIG2310, 
        NET_CC_CONFIG2311, NET_CC_CONFIG2312, NET_CC_CONFIG2313, 
        CI_TO_CO2314, NET_CC_CONFIG2315, NET_CC_CONFIG2316, 
        NET_CC_CONFIG2317, NET_CC_CONFIG2318, NET_CC_CONFIG2319, 
        NET_CC_CONFIG2320, NET_CC_CONFIG2321, NET_CC_CONFIG2322, 
        NET_CC_CONFIG2323, NET_CC_CONFIG2324, NET_CC_CONFIG2325, 
        NET_CC_CONFIG2326, NET_CC_CONFIG2327, NET_CC_CONFIG2328, 
        NET_CC_CONFIG2329, NET_CC_CONFIG2330, NET_CC_CONFIG2331, 
        NET_CC_CONFIG2332, NET_CC_CONFIG2333, NET_CC_CONFIG2334, 
        NET_CC_CONFIG2335, NET_CC_CONFIG2336, NET_CC_CONFIG2337, 
        NET_CC_CONFIG2338, NET_CC_CONFIG2339, NET_CC_CONFIG2340, 
        NET_CC_CONFIG2341, NET_CC_CONFIG2342, NET_CC_CONFIG2343, 
        NET_CC_CONFIG2344, NET_CC_CONFIG2345, NET_CC_CONFIG2346, 
        NET_CC_CONFIG2347, NET_CC_CONFIG2348, NET_CC_CONFIG2349, 
        NET_CC_CONFIG2350, NET_CC_CONFIG2351, NET_CC_CONFIG2352, 
        NET_CC_CONFIG2353, NET_CC_CONFIG2354, NET_CC_CONFIG2355, 
        NET_CC_CONFIG2356, NET_CC_CONFIG2357, NET_CC_CONFIG2358, 
        NET_CC_CONFIG2359, NET_CC_CONFIG2360, NET_CC_CONFIG2361, 
        NET_CC_CONFIG2362, NET_CC_CONFIG2363, NET_CC_CONFIG2364, 
        NET_CC_CONFIG2365, NET_CC_CONFIG2366, NET_CC_CONFIG2367, 
        NET_CC_CONFIG2368, NET_CC_CONFIG2369, NET_CC_CONFIG2370, 
        NET_CC_CONFIG2371, NET_CC_CONFIG2372, NET_CC_CONFIG2373, 
        NET_CC_CONFIG2374, NET_CC_CONFIG2375, NET_CC_CONFIG2376, 
        NET_CC_CONFIG2377, NET_CC_CONFIG2378, NET_CC_CONFIG2379, 
        NET_CC_CONFIG2380, NET_CC_CONFIG2381, NET_CC_CONFIG2382, 
        NET_CC_CONFIG2383, NET_CC_CONFIG2384, NET_CC_CONFIG2385, 
        NET_CC_CONFIG2386, NET_CC_CONFIG2387, NET_CC_CONFIG2388, 
        NET_CC_CONFIG2389, NET_CC_CONFIG2390, NET_CC_CONFIG2391, 
        NET_CC_CONFIG2392, NET_CC_CONFIG2393, NET_CC_CONFIG2394, 
        NET_CC_CONFIG2395, NET_CC_CONFIG2396, NET_CC_CONFIG2397, 
        NET_CC_CONFIG2398, NET_CC_CONFIG2399, NET_CC_CONFIG2400, 
        NET_CC_CONFIG2401, NET_CC_CONFIG2402, NET_CC_CONFIG2403, 
        NET_CC_CONFIG2404, NET_CC_CONFIG2405, NET_CC_CONFIG2406, 
        NET_CC_CONFIG2407, NET_CC_CONFIG2408, NET_CC_CONFIG2409, 
        NET_CC_CONFIG2410, NET_CC_CONFIG2411, NET_CC_CONFIG2412, 
        NET_CC_CONFIG2413, NET_CC_CONFIG2414, NET_CC_CONFIG2415, 
        NET_CC_CONFIG2416, NET_CC_CONFIG2417, NET_CC_CONFIG2418, 
        NET_CC_CONFIG2419, NET_CC_CONFIG2420, NET_CC_CONFIG2421, 
        NET_CC_CONFIG2422, NET_CC_CONFIG2423, NET_CC_CONFIG2424, 
        NET_CC_CONFIG2425, NET_CC_CONFIG2426, NET_CC_CONFIG2427, 
        NET_CC_CONFIG2428, NET_CC_CONFIG2429, NET_CC_CONFIG2430, 
        NET_CC_CONFIG2431, NET_CC_CONFIG2432, NET_CC_CONFIG2433, 
        NET_CC_CONFIG2434, NET_CC_CONFIG2435, NET_CC_CONFIG2436, 
        NET_CC_CONFIG2437, NET_CC_CONFIG2438, NET_CC_CONFIG2439, 
        NET_CC_CONFIG2440, NET_CC_CONFIG2441, NET_CC_CONFIG2442, 
        NET_CC_CONFIG2443, NET_CC_CONFIG2444, NET_CC_CONFIG2445, 
        NET_CC_CONFIG2446, NET_CC_CONFIG2447, NET_CC_CONFIG2448, 
        NET_CC_CONFIG2449, NET_CC_CONFIG2450, NET_CC_CONFIG2451, 
        NET_CC_CONFIG2452, NET_CC_CONFIG2453, NET_CC_CONFIG2454, 
        NET_CC_CONFIG2455, NET_CC_CONFIG2456, NET_CC_CONFIG2457, 
        NET_CC_CONFIG2458, NET_CC_CONFIG2459, NET_CC_CONFIG2460, 
        NET_CC_CONFIG2461, NET_CC_CONFIG2462, NET_CC_CONFIG2463, 
        NET_CC_CONFIG2464, NET_CC_CONFIG2465, NET_CC_CONFIG2466, 
        NET_CC_CONFIG2467, NET_CC_CONFIG2468, NET_CC_CONFIG2469, 
        NET_CC_CONFIG2470, NET_CC_CONFIG2471, NET_CC_CONFIG2472, 
        NET_CC_CONFIG2473, NET_CC_CONFIG2474, NET_CC_CONFIG2475, 
        NET_CC_CONFIG2476, NET_CC_CONFIG2477, NET_CC_CONFIG2478, 
        NET_CC_CONFIG2479, NET_CC_CONFIG2480, NET_CC_CONFIG2481, 
        NET_CC_CONFIG2482, NET_CC_CONFIG2483, NET_CC_CONFIG2484, 
        NET_CC_CONFIG2485, NET_CC_CONFIG2486, NET_CC_CONFIG2487, 
        NET_CC_CONFIG2488, NET_CC_CONFIG2489, NET_CC_CONFIG2490, 
        NET_CC_CONFIG2491, NET_CC_CONFIG2492, NET_CC_CONFIG2493, 
        NET_CC_CONFIG2494, NET_CC_CONFIG2495, NET_CC_CONFIG2496, 
        NET_CC_CONFIG2497, NET_CC_CONFIG2498, NET_CC_CONFIG2499, 
        NET_CC_CONFIG2500, NET_CC_CONFIG2501, NET_CC_CONFIG2502, 
        NET_CC_CONFIG2503, NET_CC_CONFIG2504, NET_CC_CONFIG2505, 
        NET_CC_CONFIG2506, NET_CC_CONFIG2507, NET_CC_CONFIG2508, 
        NET_CC_CONFIG2509, NET_CC_CONFIG2510, NET_CC_CONFIG2511, 
        NET_CC_CONFIG2512, NET_CC_CONFIG2513, NET_CC_CONFIG2514, 
        NET_CC_CONFIG2515, NET_CC_CONFIG2516, NET_CC_CONFIG2517, 
        NET_CC_CONFIG2518, NET_CC_CONFIG2519, NET_CC_CONFIG2520, 
        NET_CC_CONFIG2521, NET_CC_CONFIG2522, CI_TO_CO2523, 
        NET_CC_CONFIG2524, NET_CC_CONFIG2525, NET_CC_CONFIG2526, 
        NET_CC_CONFIG2527, NET_CC_CONFIG2528, NET_CC_CONFIG2529, 
        NET_CC_CONFIG2530, NET_CC_CONFIG2531, NET_CC_CONFIG2532, 
        NET_CC_CONFIG2533, NET_CC_CONFIG2534, NET_CC_CONFIG2535, 
        NET_CC_CONFIG2536, NET_CC_CONFIG2537, NET_CC_CONFIG2538, 
        NET_CC_CONFIG2539, NET_CC_CONFIG2540, NET_CC_CONFIG2541, 
        NET_CC_CONFIG2542, NET_CC_CONFIG2543, NET_CC_CONFIG2544, 
        NET_CC_CONFIG2545, NET_CC_CONFIG2546, NET_CC_CONFIG2547, 
        NET_CC_CONFIG2548, NET_CC_CONFIG2549, NET_CC_CONFIG2550, 
        NET_CC_CONFIG2551, NET_CC_CONFIG2552, NET_CC_CONFIG2553, 
        NET_CC_CONFIG2554, NET_CC_CONFIG2555, NET_CC_CONFIG2556, 
        NET_CC_CONFIG2557, NET_CC_CONFIG2558, NET_CC_CONFIG2559, 
        NET_CC_CONFIG2560, NET_CC_CONFIG2561, NET_CC_CONFIG2562, 
        NET_CC_CONFIG2563, CI_TO_CO2564, NET_CC_CONFIG2565, 
        NET_CC_CONFIG2566, NET_CC_CONFIG2567, NET_CC_CONFIG2568, 
        NET_CC_CONFIG2569, NET_CC_CONFIG2570, NET_CC_CONFIG2571, 
        NET_CC_CONFIG2572, NET_CC_CONFIG2573, NET_CC_CONFIG2574, 
        NET_CC_CONFIG2575, NET_CC_CONFIG2576, NET_CC_CONFIG2577, 
        NET_CC_CONFIG2578, NET_CC_CONFIG2579, NET_CC_CONFIG2580, 
        NET_CC_CONFIG2581, NET_CC_CONFIG2582, NET_CC_CONFIG2583, 
        NET_CC_CONFIG2584, NET_CC_CONFIG2585, NET_CC_CONFIG2586, 
        NET_CC_CONFIG2587, NET_CC_CONFIG2588, NET_CC_CONFIG2589, 
        NET_CC_CONFIG2590, NET_CC_CONFIG2591, NET_CC_CONFIG2592, 
        NET_CC_CONFIG2593, NET_CC_CONFIG2594, NET_CC_CONFIG2595, 
        NET_CC_CONFIG2596, NET_CC_CONFIG2597, NET_CC_CONFIG2598, 
        NET_CC_CONFIG2599, NET_CC_CONFIG2600, NET_CC_CONFIG2601, 
        NET_CC_CONFIG2602, NET_CC_CONFIG2603, NET_CC_CONFIG2604, 
        NET_CC_CONFIG2605, NET_CC_CONFIG2606, NET_CC_CONFIG2607, 
        NET_CC_CONFIG2608, NET_CC_CONFIG2609, NET_CC_CONFIG2610, 
        NET_CC_CONFIG2611, NET_CC_CONFIG2612, NET_CC_CONFIG2613, 
        NET_CC_CONFIG2614, NET_CC_CONFIG2615, NET_CC_CONFIG2616, 
        NET_CC_CONFIG2617, NET_CC_CONFIG2618, NET_CC_CONFIG2619, 
        NET_CC_CONFIG2620, NET_CC_CONFIG2621, NET_CC_CONFIG2622, 
        NET_CC_CONFIG2623, NET_CC_CONFIG2624, NET_CC_CONFIG2625, 
        NET_CC_CONFIG2626, NET_CC_CONFIG2627, NET_CC_CONFIG2628, 
        NET_CC_CONFIG2629, NET_CC_CONFIG2630, NET_CC_CONFIG2631, 
        NET_CC_CONFIG2632, NET_CC_CONFIG2633, NET_CC_CONFIG2634, 
        NET_CC_CONFIG2635, NET_CC_CONFIG2636, NET_CC_CONFIG2637, 
        NET_CC_CONFIG2638, NET_CC_CONFIG2639, NET_CC_CONFIG2640, 
        NET_CC_CONFIG2641, NET_CC_CONFIG2642, NET_CC_CONFIG2643, 
        NET_CC_CONFIG2644, NET_CC_CONFIG2645, NET_CC_CONFIG2646, 
        NET_CC_CONFIG2647, NET_CC_CONFIG2648, NET_CC_CONFIG2649, 
        NET_CC_CONFIG2650, NET_CC_CONFIG2651, NET_CC_CONFIG2652, 
        NET_CC_CONFIG2653, NET_CC_CONFIG2654, NET_CC_CONFIG2655, 
        NET_CC_CONFIG2656, NET_CC_CONFIG2657, NET_CC_CONFIG2658, 
        NET_CC_CONFIG2659, NET_CC_CONFIG2660, NET_CC_CONFIG2661, 
        NET_CC_CONFIG2662, NET_CC_CONFIG2663, NET_CC_CONFIG2664, 
        NET_CC_CONFIG2665, NET_CC_CONFIG2666, NET_CC_CONFIG2667, 
        NET_CC_CONFIG2668, NET_CC_CONFIG2669, NET_CC_CONFIG2670, 
        NET_CC_CONFIG2671, NET_CC_CONFIG2672, NET_CC_CONFIG2673, 
        NET_CC_CONFIG2674, NET_CC_CONFIG2675, NET_CC_CONFIG2676, 
        NET_CC_CONFIG2677, NET_CC_CONFIG2678, NET_CC_CONFIG2679, 
        NET_CC_CONFIG2680, NET_CC_CONFIG2681, NET_CC_CONFIG2682, 
        NET_CC_CONFIG2683, NET_CC_CONFIG2684, NET_CC_CONFIG2685, 
        NET_CC_CONFIG2686, NET_CC_CONFIG2687, NET_CC_CONFIG2688, 
        NET_CC_CONFIG2689, NET_CC_CONFIG2690, NET_CC_CONFIG2691, 
        NET_CC_CONFIG2692, NET_CC_CONFIG2693, NET_CC_CONFIG2694, 
        NET_CC_CONFIG2695, NET_CC_CONFIG2696, NET_CC_CONFIG2697, 
        NET_CC_CONFIG2698, NET_CC_CONFIG2699, NET_CC_CONFIG2700, 
        NET_CC_CONFIG2701, NET_CC_CONFIG2702, NET_CC_CONFIG2703, 
        NET_CC_CONFIG2704, NET_CC_CONFIG2705, NET_CC_CONFIG2706, 
        NET_CC_CONFIG2707, NET_CC_CONFIG2708, NET_CC_CONFIG2709, 
        NET_CC_CONFIG2710, NET_CC_CONFIG2711, NET_CC_CONFIG2712, 
        NET_CC_CONFIG2713, NET_CC_CONFIG2714, NET_CC_CONFIG2715, 
        NET_CC_CONFIG2716, NET_CC_CONFIG2717, NET_CC_CONFIG2718, 
        NET_CC_CONFIG2719, NET_CC_CONFIG2720, NET_CC_CONFIG2721, 
        NET_CC_CONFIG2722, NET_CC_CONFIG2723, NET_CC_CONFIG2724, 
        NET_CC_CONFIG2725, NET_CC_CONFIG2726, NET_CC_CONFIG2727, 
        NET_CC_CONFIG2728, NET_CC_CONFIG2729, NET_CC_CONFIG2730, 
        NET_CC_CONFIG2731, NET_CC_CONFIG2732, NET_CC_CONFIG2733, 
        NET_CC_CONFIG2734, NET_CC_CONFIG2735, NET_CC_CONFIG2736, 
        AFLSDF_VCC, AFLSDF_GND, \AFLSDF_INV_0\, \AFLSDF_INV_1\, 
        \AFLSDF_INV_2\, \AFLSDF_INV_3\, \AFLSDF_INV_4\, 
        \AFLSDF_INV_5\, \AFLSDF_INV_6\, \AFLSDF_INV_7\, 
        \AFLSDF_INV_8\, \AFLSDF_INV_9\, \AFLSDF_INV_10\, 
        \AFLSDF_INV_11\, \AFLSDF_INV_12\, \AFLSDF_INV_13\, 
        \AFLSDF_INV_14\, \AFLSDF_INV_15\, \AFLSDF_INV_16\, 
        \AFLSDF_INV_17\, \AFLSDF_INV_18\ : std_logic;
    signal GND_power_net1 : std_logic;
    signal VCC_power_net1 : std_logic;
    signal nc228, nc203, nc265, nc216, nc194, nc151, nc23, nc175, 
        nc250, nc58, nc116, nc74, nc133, nc238, nc167, nc84, nc39, 
        nc72, nc256, nc212, nc205, nc82, nc145, nc181, nc160, 
        nc57, nc349, nc156, nc280, nc125, nc211, nc73, nc107, 
        nc329, nc66, nc83, nc9, nc252, nc171, nc54, nc286, nc307, 
        nc135, nc41, nc100, nc270, nc339, nc52, nc251, nc186, 
        nc29, nc269, nc118, nc60, nc141, nc311, nc276, nc193, 
        nc214, nc298, nc282, nc240, nc45, nc53, nc121, nc176, 
        nc220, nc158, nc281, nc209, nc246, nc351, nc162, nc11, 
        nc272, nc131, nc254, nc267, nc96, nc79, nc226, nc146, 
        nc230, nc89, nc119, nc48, nc271, nc213, nc300, nc126, 
        nc195, nc188, nc242, nc15, nc308, nc236, nc102, nc304, 
        nc3, nc207, nc47, nc90, nc284, nc222, nc159, nc136, nc241, 
        nc253, nc178, nc306, nc215, nc59, nc221, nc232, nc274, 
        nc18, nc44, nc117, nc189, nc164, nc148, nc42, nc231, 
        nc191, nc255, nc283, nc341, nc317, nc290, nc17, nc2, 
        nc302, nc110, nc128, nc244, nc321, nc43, nc179, nc157, 
        nc36, nc224, nc296, nc273, nc61, nc104, nc138, nc14, 
        nc285, nc303, nc150, nc331, nc196, nc234, nc149, nc12, 
        nc219, nc30, nc243, nc187, nc65, nc7, nc292, nc129, nc275, 
        nc8, nc223, nc13, nc305, nc180, nc26, nc291, nc177, nc139, 
        nc310, nc259, nc245, nc233, nc163, nc318, nc268, nc112, 
        nc68, nc49, nc314, nc217, nc170, nc91, nc225, nc5, nc20, 
        nc198, nc147, nc350, nc316, nc67, nc289, nc294, nc152, 
        nc127, nc103, nc235, nc76, nc347, nc208, nc354, nc140, 
        nc257, nc86, nc95, nc327, nc120, nc165, nc279, nc137, 
        nc64, nc19, nc312, nc70, nc182, nc62, nc337, nc199, nc80, 
        nc130, nc287, nc98, nc293, nc249, nc114, nc56, nc105, 
        nc63, nc352, nc313, nc309, nc172, nc229, nc277, nc97, 
        nc161, nc31, nc340, nc295, nc154, nc50, nc260, nc239, 
        nc353, nc348, nc142, nc320, nc344, nc315, nc247, nc94, 
        nc197, nc328, nc122, nc266, nc35, nc324, nc4, nc227, nc92, 
        nc101, nc346, nc330, nc184, nc200, nc190, nc166, nc355, 
        nc338, nc326, nc132, nc334, nc21, nc237, nc93, nc262, 
        nc69, nc206, nc174, nc38, nc113, nc336, nc218, nc342, 
        nc106, nc261, nc25, nc1, nc322, nc299, nc37, nc202, nc144, 
        nc153, nc46, nc258, nc343, nc71, nc124, nc332, nc81, 
        nc201, nc168, nc323, nc34, nc28, nc115, nc264, nc192, 
        nc319, nc134, nc32, nc40, nc297, nc99, nc75, nc183, nc345, 
        nc333, nc288, nc85, nc27, nc108, nc325, nc16, nc155, nc51, 
        nc301, nc33, nc204, nc173, nc278, nc169, nc78, nc263, 
        nc335, nc24, nc88, nc111, nc55, nc10, nc22, nc210, nc185, 
        nc143, nc248, nc77, nc6, nc109, nc87, nc123 : std_logic;

begin 

    ADLIB_GND <= GND_power_net1;
    AFLSDF_GND <= GND_power_net1;
    AFLSDF_VCC <= VCC_power_net1;
    ADLIB_VCC <= VCC_power_net1;

    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[471]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[5]\ : 
        CFG4
      generic map(INIT => x"FF08")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_170_1\, 
        B => \can_top_level_inst/txtb_hw_cmd.failed\, C => 
        \can_top_level_inst/txtb_parity_error_valid[7]\, D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_0_1[22]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[322]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_0_1_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0_1\ : 
        CFG3
      generic map(INIT => x"13")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt_rep1\, 
        B => \can_top_level_inst/drv_bus_475_rep2\, C => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[11]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[30]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un17_same_bits_erase\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un2_stuff_rule_violate_Z\, 
        B => \can_top_level_inst/can_core_inst/stuff_length[0]\, 
        C => \can_top_level_inst/rx_triggers[1]\, D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un17_same_bits_erase_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un17_same_bits_erase_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_3_sqmuxa_i_RNI941B_0\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_886_i\, 
        B => \can_top_level_inst/sync_edge\, C => 
        \can_top_level_inst/can_core_inst/bds_data_out\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un22_rx_data_nbs_i_m\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_raw_0[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNID7LB_Y[0]\, 
        B => \can_top_level_inst/drv_bus[375]\, C => 
        \can_top_level_inst/drv_bus[374]\, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_36\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_RNO[24]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2680\, D
         => \can_top_level_inst/memory_registers_inst/N_2713\, Y
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[3]\, 
        B => \can_top_level_inst/drv_bus[59]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_2_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_3_Z\, 
        CC => NET_CC_CONFIG2462, P => NET_CC_CONFIG2459, Y3 => 
        NET_CC_CONFIG2460, Y3A => NET_CC_CONFIG2461);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_0\ : 
        ARI1_CC
      generic map(INIT => x"45500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[79]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_7\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_0_FCNET1\, 
        CC => NET_CC_CONFIG2094, P => NET_CC_CONFIG2091, Y3 => 
        NET_CC_CONFIG2092, Y3A => NET_CC_CONFIG2093);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lto6_tz\ : 
        CFG4
      generic map(INIT => x"FAF4")

      port map(A => \can_top_level_inst/rec_dlc[0]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lt4\, 
        C => \can_top_level_inst/rec_dlc[2]\, D => 
        \can_top_level_inst/rec_dlc[1]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21_tz\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/pexs_proc.un1_drv_cpexs\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_875\, 
        B => \can_top_level_inst/drv_bus[512]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_drv_cpexs\);
    
    \timestamp_ibuf[37]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[37]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[37]/YIN\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[210]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_12\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[12]\, B => 
        \can_top_level_inst/drv_bus[312]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_11\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_12\, 
        CC => NET_CC_CONFIG577, P => NET_CC_CONFIG574, Y3 => 
        NET_CC_CONFIG575, Y3A => NET_CC_CONFIG576);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2_0[1]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txtb_parity_error_valid[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_140\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.lock_i\ : 
        CFG4
      generic map(INIT => x"FAFB")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_970\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_933\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_905_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram16_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe16\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram16_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[16]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[1]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[1]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram47__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_354\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe47\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_RNIHLET1[5]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_4\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_5\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_5\, 
        CC => NET_CC_CONFIG2199, P => NET_CC_CONFIG2196, Y3 => 
        NET_CC_CONFIG2197, Y3A => NET_CC_CONFIG2198);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_13\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[13]\, B => 
        \can_top_level_inst/drv_bus[313]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_12\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_13\, 
        CC => NET_CC_CONFIG581, P => NET_CC_CONFIG578, Y3 => 
        NET_CC_CONFIG579, Y3A => NET_CC_CONFIG580);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/un4_tst_ena_2\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[19]\, B
         => 
        \can_top_level_inst/test_registers_out.tst_control[0]\, Y
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm_0_RNO[8]\ : 
        ARI1_CC
      generic map(INIT => x"45500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[8]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[7]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[8]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG2057, P => 
        NET_CC_CONFIG2054, Y3 => NET_CC_CONFIG2055, Y3A => 
        NET_CC_CONFIG2056);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[18]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[16]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_857\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_851\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[4]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[23]\ : 
        CFG4
      generic map(INIT => x"F0EE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[23]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[23]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[22]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2[0]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1255\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_934\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1327\);
    
    AFLSDF_INV_6 : INV_BA
      port map(A => \AFLSDF_INV_6\, Y => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa14\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe14\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[5]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[37]\, 
        C => \can_top_level_inst/rec_ident[5]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2250\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa12\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe12\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/un73_txtb_hw_cmd_int_0_a3_1_0\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[0]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => \can_top_level_inst/txtb_hw_cmd_index[1]\, D => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_un73_txtb_hw_cmd_int_0_a3_1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa3\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram35__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_350\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe35\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/bit_gen.2.reg_present_gen.reg_autoclear_gen.reg_value_r_5[2]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select\, 
        C => \data_in_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/tx_command[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[4]\ : 
        CFG4
      generic map(INIT => x"FFA2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[3]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_170\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_RNIJIHQ1[2]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\, C
         => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[2]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[1]\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[2]\, 
        CC => NET_CC_CONFIG1103, P => NET_CC_CONFIG1100, Y3 => 
        NET_CC_CONFIG1101, Y3A => NET_CC_CONFIG1102);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m35_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[2]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[2]\, 
        C => \can_top_level_inst/txtb_parity_error_valid[7]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[8]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[8]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m44_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram105_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe105\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram105_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[8]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[10]\ : 
        CFG4
      generic map(INIT => x"F088")

      port map(A => \can_top_level_inst/rec_esi\, B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        C => \can_top_level_inst/rx_buffer_inst/N_2288\, D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un4_compl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[12]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0_RNI9BUS42[1]\ : 
        ARI1_CC
      generic map(INIT => x"58778")

      port map(A => \can_top_level_inst/drv_bus[30]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_48\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_0\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_1\, 
        CC => NET_CC_CONFIG1131, P => NET_CC_CONFIG1128, Y3 => 
        NET_CC_CONFIG1129, Y3A => NET_CC_CONFIG1130);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[27]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[27]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[27]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[6]\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q_c1\ : CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[1]\, 
        Y => \can_top_level_inst/rx_buffer_inst/rx_mof_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_err_frm_req_3_0_a4_RNIV93N\ : 
        CFG3
      generic map(INIT => x"32")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1103_i\, 
        B => \can_top_level_inst/drv_bus[513]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_drv_rom_ena_m\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_14\ : 
        ARI1_CC
      generic map(INIT => x"56633")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[14]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[14]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_13_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_14_FCNET1\, 
        CC => NET_CC_CONFIG454, P => NET_CC_CONFIG451, Y3 => 
        NET_CC_CONFIG452, Y3A => NET_CC_CONFIG453);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m64_1_0\ : 
        CFG4
      generic map(INIT => x"0027")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[7]\, C => 
        \can_top_level_inst/stat_bus[195]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_65_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[15]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[15]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[15]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[28]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c2\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[0]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[1]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c2_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_a4_0_0[1]\ : 
        CFG4
      generic map(INIT => x"0B08")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        B => \can_top_level_inst/drv_bus[460]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1013\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_a4_0_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m43_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[6]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_176\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[31]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[21]\ : 
        SLE
      port map(D => \timestamp_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[21]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry_cy[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO1670, P(0) => 
        NET_CC_CONFIG1671, P(1) => NET_CC_CONFIG1675, P(2) => 
        NET_CC_CONFIG1679, P(3) => NET_CC_CONFIG1683, P(4) => 
        NET_CC_CONFIG1687, P(5) => NET_CC_CONFIG1691, P(6) => 
        NET_CC_CONFIG1695, P(7) => NET_CC_CONFIG1699, P(8) => 
        NET_CC_CONFIG1703, P(9) => NET_CC_CONFIG1707, P(10) => 
        NET_CC_CONFIG1711, P(11) => NET_CC_CONFIG1715, Y3(0) => 
        NET_CC_CONFIG1672, Y3(1) => NET_CC_CONFIG1676, Y3(2) => 
        NET_CC_CONFIG1680, Y3(3) => NET_CC_CONFIG1684, Y3(4) => 
        NET_CC_CONFIG1688, Y3(5) => NET_CC_CONFIG1692, Y3(6) => 
        NET_CC_CONFIG1696, Y3(7) => NET_CC_CONFIG1700, Y3(8) => 
        NET_CC_CONFIG1704, Y3(9) => NET_CC_CONFIG1708, Y3(10) => 
        NET_CC_CONFIG1712, Y3(11) => NET_CC_CONFIG1716, Y3A(0)
         => NET_CC_CONFIG1673, Y3A(1) => NET_CC_CONFIG1677, 
        Y3A(2) => NET_CC_CONFIG1681, Y3A(3) => NET_CC_CONFIG1685, 
        Y3A(4) => NET_CC_CONFIG1689, Y3A(5) => NET_CC_CONFIG1693, 
        Y3A(6) => NET_CC_CONFIG1697, Y3A(7) => NET_CC_CONFIG1701, 
        Y3A(8) => NET_CC_CONFIG1705, Y3A(9) => NET_CC_CONFIG1709, 
        Y3A(10) => NET_CC_CONFIG1713, Y3A(11) => 
        NET_CC_CONFIG1717, CC(0) => NET_CC_CONFIG1674, CC(1) => 
        NET_CC_CONFIG1678, CC(2) => NET_CC_CONFIG1682, CC(3) => 
        NET_CC_CONFIG1686, CC(4) => NET_CC_CONFIG1690, CC(5) => 
        NET_CC_CONFIG1694, CC(6) => NET_CC_CONFIG1698, CC(7) => 
        NET_CC_CONFIG1702, CC(8) => NET_CC_CONFIG1706, CC(9) => 
        NET_CC_CONFIG1710, CC(10) => NET_CC_CONFIG1714, CC(11)
         => NET_CC_CONFIG1718);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[22]\);
    
    \can_top_level_inst/rx_buffer_inst/commit_rx_frame\ : SLE
      port map(D => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/commit_overrun_abort_1_sqmuxa_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_0_1[20]\ : 
        CFG4
      generic map(INIT => x"3B19")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_159\, 
        D => \can_top_level_inst/stat_bus[75]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_0_1_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[19]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[19]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_nbt_Z[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_1_S\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[1]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_20[0]\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_2\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_15_Z[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_10_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_20_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_3_1_RNIMVGD1_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO325, CO => OPEN, P(0) => 
        NET_CC_CONFIG350, P(1) => NET_CC_CONFIG354, P(2) => 
        NET_CC_CONFIG358, P(3) => ADLIB_VCC, P(4) => ADLIB_VCC, 
        P(5) => ADLIB_VCC, P(6) => ADLIB_VCC, P(7) => ADLIB_VCC, 
        P(8) => ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, 
        P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG351, Y3(1) => 
        NET_CC_CONFIG355, Y3(2) => NET_CC_CONFIG359, Y3(3) => 
        ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5) => ADLIB_VCC, Y3(6)
         => ADLIB_VCC, Y3(7) => ADLIB_VCC, Y3(8) => ADLIB_VCC, 
        Y3(9) => ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => 
        ADLIB_VCC, Y3A(0) => NET_CC_CONFIG352, Y3A(1) => 
        NET_CC_CONFIG356, Y3A(2) => NET_CC_CONFIG360, Y3A(3) => 
        ADLIB_VCC, Y3A(4) => ADLIB_VCC, Y3A(5) => ADLIB_VCC, 
        Y3A(6) => ADLIB_VCC, Y3A(7) => ADLIB_VCC, Y3A(8) => 
        ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => ADLIB_VCC, 
        Y3A(11) => ADLIB_VCC, CC(0) => NET_CC_CONFIG353, CC(1)
         => NET_CC_CONFIG357, CC(2) => NET_CC_CONFIG361, CC(3)
         => nc228, CC(4) => nc203, CC(5) => nc265, CC(6) => nc216, 
        CC(7) => nc194, CC(8) => nc151, CC(9) => nc23, CC(10) => 
        nc175, CC(11) => nc250);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_wmux_3_Y[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[29]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[29]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_19\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_19_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[6]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_stuff_count_ok_1\ : 
        CFG3
      generic map(INIT => x"96")

      port map(A => \can_top_level_inst/stat_bus[304]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_stuff_count[1]\, 
        C => \can_top_level_inst/stat_bus[303]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_stuff_count_ok_1_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa15\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe15\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_buf_cmds_proc.un28_is_receiver_i_a2\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1317_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[10]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[10]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[10]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_18\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[30]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_18_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_3[25]\ : 
        CFG3
      generic map(INIT => x"31")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[139]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1089\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIQA1AR8[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[61]\, B => 
        \can_top_level_inst/tran_word[29]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_28\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_29\, 
        CC => NET_CC_CONFIG1043, P => NET_CC_CONFIG1040, Y3 => 
        NET_CC_CONFIG1041, Y3A => NET_CC_CONFIG1042);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[29]\ : 
        SLE
      port map(D => \data_in_c[29]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_7[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_15[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram56_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram57_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_6[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/un102_txtb_hw_cmd_int_0_a3_1\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[4]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        D => \can_top_level_inst/txtb_sw_cmd.set_abt\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_172\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[18]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1459_2\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_control_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => swr_c, B => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_control_reg_comp/wr_select_4_Z[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_control_reg_comp/wr_select_0_Z[0]\, 
        D => \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_tst_control_reg_comp_wr_select[0]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/int_ena_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/un1_int_ena_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_ena[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[24]\);
    
    \timestamp_ibuf[34]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[34]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[34]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[11]\);
    
    \data_in_ibuf[14]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(14), Y => \data_in_ibuf[14]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_38_1_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_38_1_0_y0[21]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_968\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_970\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y3[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y1[31]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_3_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[24]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[262]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[27]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst[2]\ : 
        CFG4
      generic map(INIT => x"FAFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_0_Z[2]\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_1_Z[2]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_996\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_RNIDS391\ : 
        CFG4
      generic map(INIT => x"0110")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_09\, B => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_19\, C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_i\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_i3_mux\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_34_1[21]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/stat_bus[209]\, B => 
        \can_top_level_inst/drv_bus[380]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_968_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[2]\);
    
    \adress_ibuf[8]/U_IOPAD\ : IOPAD_IN
      port map(PAD => adress(8), Y => \adress_ibuf[8]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[472]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[24]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[5]\, CLK
         => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[93]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm_0_RNO[8]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[8]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[7]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[8]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG1637, P => 
        NET_CC_CONFIG1634, Y3 => NET_CC_CONFIG1635, Y3A => 
        NET_CC_CONFIG1636);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_0[3]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[10]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_0_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNO[7]\ : 
        CFG4
      generic map(INIT => x"2023")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_149\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_151\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_124_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[28]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[28]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[28]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_type_q_RNO[0]\ : 
        CFG3
      generic map(INIT => x"13")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_406\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un6_form_err_i_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_err_type_d_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_type_d_ss0_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[28]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_rst_d_i_0\ : 
        CFG3
      generic map(INIT => x"BF")

      port map(A => can_tx_c, B => 
        \can_top_level_inst/bus_sampling_inst/tx_data_prev\, C
         => \can_top_level_inst/stat_bus[13]\, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_rst_d_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[9]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[9]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[9]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa3\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_2_0[1]\ : 
        CFG4
      generic map(INIT => x"F388")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1190\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1191\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1_0_Z[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_2_0_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[15]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[19]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[19]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[19]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[20]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[20]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[23]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[7]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIG7IO_0[0]\ : 
        CFG3
      generic map(INIT => x"1D")

      port map(A => \can_top_level_inst/stat_bus[188]\, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[220]\, Y => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/sel_value_i[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\);
    
    \timestamp_ibuf[60]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[60]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[60]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[27]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[27]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram7__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_353\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe7\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_1_0[12]\ : 
        CFG3
      generic map(INIT => x"27")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[12]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[29]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[21]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[102]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[14]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2259\, B
         => \can_top_level_inst/rx_buffer_inst/N_4274\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2292\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[29]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[9]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[9]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[9]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_RNO[8]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[0]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[1]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[0]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2458\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[35]\ : 
        SLE
      port map(D => \timestamp_c[35]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[35]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_RNO[12]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_RNO_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_length[3]\ : 
        CFG4
      generic map(INIT => x"FC74")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/frame_type\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/un2_data_len_can_2_0_i\, 
        C => \can_top_level_inst/tran_dlc[3]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_8_to_64_integer[3]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_clear_1_0\ : 
        CFG4
      generic map(INIT => x"5140")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_clear_sm0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_717_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_ide_is_arbitration\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_arbitration_lost_condition_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_clear_i_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[252]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[14]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[29]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[29]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[29]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[284]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m44_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un17_same_bits_erase_1\ : 
        CFG4
      generic map(INIT => x"F777")

      port map(A => 
        \can_top_level_inst/can_core_inst/destuff_enable\, B => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/reg_q_0\, 
        C => \can_top_level_inst/rx_triggers[1]\, D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_discard_stuff_bit_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un17_same_bits_erase_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_10[6]\ : 
        CFG4
      generic map(INIT => x"ECCC")

      port map(A => \can_top_level_inst/rx_frame_count[2]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1364_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1364\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[215]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[3]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[3]\, 
        B => \can_top_level_inst/txtb_hw_cmd.unlock\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_136\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_28[17]\ : 
        CFG4
      generic map(INIT => x"ACF0")

      port map(A => \can_top_level_inst/stat_bus[290]\, B => 
        \can_top_level_inst/drv_bus[98]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_810\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_15_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_GND, CO => OPEN, P(0) => 
        NET_CC_CONFIG1982, P(1) => NET_CC_CONFIG1986, P(2) => 
        NET_CC_CONFIG1990, P(3) => NET_CC_CONFIG1994, P(4) => 
        NET_CC_CONFIG1998, P(5) => NET_CC_CONFIG2002, P(6) => 
        NET_CC_CONFIG2006, P(7) => NET_CC_CONFIG2010, P(8) => 
        NET_CC_CONFIG2014, P(9) => NET_CC_CONFIG2018, P(10) => 
        ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1983, 
        Y3(1) => NET_CC_CONFIG1987, Y3(2) => NET_CC_CONFIG1991, 
        Y3(3) => NET_CC_CONFIG1995, Y3(4) => NET_CC_CONFIG1999, 
        Y3(5) => NET_CC_CONFIG2003, Y3(6) => NET_CC_CONFIG2007, 
        Y3(7) => NET_CC_CONFIG2011, Y3(8) => NET_CC_CONFIG2015, 
        Y3(9) => NET_CC_CONFIG2019, Y3(10) => ADLIB_VCC, Y3(11)
         => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG1984, Y3A(1) => 
        NET_CC_CONFIG1988, Y3A(2) => NET_CC_CONFIG1992, Y3A(3)
         => NET_CC_CONFIG1996, Y3A(4) => NET_CC_CONFIG2000, 
        Y3A(5) => NET_CC_CONFIG2004, Y3A(6) => NET_CC_CONFIG2008, 
        Y3A(7) => NET_CC_CONFIG2012, Y3A(8) => NET_CC_CONFIG2016, 
        Y3A(9) => NET_CC_CONFIG2020, Y3A(10) => ADLIB_VCC, 
        Y3A(11) => ADLIB_VCC, CC(0) => NET_CC_CONFIG1985, CC(1)
         => NET_CC_CONFIG1989, CC(2) => NET_CC_CONFIG1993, CC(3)
         => NET_CC_CONFIG1997, CC(4) => NET_CC_CONFIG2001, CC(5)
         => NET_CC_CONFIG2005, CC(6) => NET_CC_CONFIG2009, CC(7)
         => NET_CC_CONFIG2013, CC(8) => NET_CC_CONFIG2017, CC(9)
         => NET_CC_CONFIG2021, CC(10) => nc58, CC(11) => nc116);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[35]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[11]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8_2[3]\ : 
        CFG4
      generic map(INIT => x"A820")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => \can_top_level_inst/rx_mem_free[3]\, D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_3_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1_0_1[1]\ : 
        CFG4
      generic map(INIT => x"2637")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/drv_bus[272]\, D => 
        \can_top_level_inst/drv_bus[144]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1_0_1_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m43_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_0_1[23]\ : 
        CFG4
      generic map(INIT => x"193B")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => \can_top_level_inst/txtb_state_5[3]\, D => 
        \can_top_level_inst/stat_bus[78]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_0_1_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_4[1]\ : 
        CFG4
      generic map(INIT => x"FFF8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1046\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_1_Z[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1202\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_2_Z[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_4_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_RNIR4JN15\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => \can_top_level_inst/drv_bus[34]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        D => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[5]\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_4\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_5\, 
        CC => NET_CC_CONFIG1147, P => NET_CC_CONFIG1144, Y3 => 
        NET_CC_CONFIG1145, Y3A => NET_CC_CONFIG1146);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test_dbl_buf.sdlc\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[2]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/load_ffmt_w_addr\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/sdlc\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[6]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[6]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[23]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_segm_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un6_form_err_i\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/can_core_inst/stuff_length[0]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/form_err\, 
        C => \can_top_level_inst/can_core_inst/stuff_err\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un6_form_err_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_o2_1\ : 
        CFG2
      generic map(INIT => x"D")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reinteg_ctr_expired\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_956\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_15[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram24_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram25_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_6[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/alc_Z[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_761_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/stat_bus[6]\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[294]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m24_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.lock_i_o2\ : 
        CFG3
      generic map(INIT => x"AB")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_931\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_945\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[8]\ : 
        CFG4
      generic map(INIT => x"0E02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[8]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[8]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[31]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_RNO[0]\ : 
        CFG4
      generic map(INIT => x"0302")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, B
         => \can_top_level_inst/tx_arbitrator_inst/N_158\, C => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[7]\, D
         => \can_top_level_inst/stat_bus[68]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_91_i\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[20]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[25]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1219\, 
        B => \can_top_level_inst/tran_word[1]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_57\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[24]\, B => 
        \can_top_level_inst/drv_bus[167]\, C => 
        \can_top_level_inst/drv_bus[200]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_26\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[11]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[12]\, 
        CC => NET_CC_CONFIG738, P => NET_CC_CONFIG735, Y3 => 
        NET_CC_CONFIG736, Y3A => NET_CC_CONFIG737);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[13]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_RNO_5[28]\ : 
        CFG4
      generic map(INIT => x"207F")

      port map(A => \can_top_level_inst/drv_bus[477]\, B => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, C => 
        \can_top_level_inst/tran_frame_test.sdlc\, D => 
        \can_top_level_inst/tran_dlc[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_21\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[30]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_36_1[16]\ : 
        CFG4
      generic map(INIT => x"3120")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/stat_bus[380]\, D => 
        \can_top_level_inst/stat_bus[236]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_780_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_d\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_14_Z\, 
        B => \can_top_level_inst/btmc_reset\, Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[22]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[22]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[13]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[13]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[13]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[1]\ : 
        CFG4
      generic map(INIT => x"0A0B")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        B => \can_top_level_inst/txtb_sw_cmd.set_ety\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_141\, 
        D => \can_top_level_inst/txtb_allow_bb[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_stuff_count_0_a4\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_950\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_stuff_count\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[10]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[6]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[4]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_15_trigger\ : 
        CFG4
      generic map(INIT => x"88B8")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/crc_calc_from_rx\, C
         => \can_top_level_inst/tx_trigger\, D => 
        \can_top_level_inst/can_core_inst/data_halt\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_15_trigger_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[24]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[17]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[17]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[17]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_o4[2]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[7]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_936_i\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_953\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_988\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[10]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m23_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count_RNI1VDV[1]\ : 
        ARI1_CC
      generic map(INIT => x"599CC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un10_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, C
         => \can_top_level_inst/rx_frame_count[1]\, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_0\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI1VDV_S[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_1\, 
        CC => NET_CC_CONFIG2636, P => NET_CC_CONFIG2633, Y3 => 
        NET_CC_CONFIG2634, Y3A => NET_CC_CONFIG2635);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_o2[3]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => \can_top_level_inst/tx_arbitrator_inst/N_109\, 
        B => \can_top_level_inst/tx_arbitrator_inst/N_104\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_72\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[12]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[8]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[6]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/command_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_0_RNIRO131\ : 
        CFG4
      generic map(INIT => x"7FFF")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/access_active_1[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        D => \data_in_c[3]\, Y => 
        \can_top_level_inst/drv_bus_i[353]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un4_txtb_port_b_address[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_ptr[0]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q_Z[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, Y
         => \can_top_level_inst/txtb_port_b_address[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[30]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[11]\ : 
        CFG4
      generic map(INIT => x"0078")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[0]\, 
        B => \can_top_level_inst/tran_frame_test.fcrc\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[0]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_909\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[17]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[17]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[17]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[16]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_2_y0[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[179]\, D => 
        \can_top_level_inst/drv_bus[303]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_38_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_873\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_888\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_38_1_0_y0[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_25[12]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[12]\, B
         => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_617\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNI2S6H2[4]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[4]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[3]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[4]\, 
        CC => NET_CC_CONFIG1694, P => NET_CC_CONFIG1691, Y3 => 
        NET_CC_CONFIG1692, Y3A => NET_CC_CONFIG1693);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[25]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_38_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1217\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1232\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_38_1_0_y0[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_2_1[5]\ : 
        CFG4
      generic map(INIT => x"3302")

      port map(A => \can_top_level_inst/rx_write_pointer[5]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_50_0_2\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_2_1_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[314]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_dest[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_0[1]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => swr_c, B => 
        \can_top_level_inst/memory_registers_inst/un3_test_registers_cs_Z\, 
        C => \adress_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_0_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[31]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/address_decoder_control_registers_comp/addr_dec_3[1]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \adress_c[5]\, B => \adress_c[6]\, Y => 
        \can_top_level_inst/memory_registers_inst/addr_dec_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[15]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1[27]\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[27]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_0_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG2144, P(1) => NET_CC_CONFIG2148, P(2) => 
        NET_CC_CONFIG2152, P(3) => NET_CC_CONFIG2156, P(4) => 
        NET_CC_CONFIG2160, P(5) => NET_CC_CONFIG2164, P(6) => 
        NET_CC_CONFIG2168, P(7) => NET_CC_CONFIG2172, P(8) => 
        ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11)
         => ADLIB_VCC, Y3(0) => NET_CC_CONFIG2145, Y3(1) => 
        NET_CC_CONFIG2149, Y3(2) => NET_CC_CONFIG2153, Y3(3) => 
        NET_CC_CONFIG2157, Y3(4) => NET_CC_CONFIG2161, Y3(5) => 
        NET_CC_CONFIG2165, Y3(6) => NET_CC_CONFIG2169, Y3(7) => 
        NET_CC_CONFIG2173, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG2146, Y3A(1) => NET_CC_CONFIG2150, Y3A(2)
         => NET_CC_CONFIG2154, Y3A(3) => NET_CC_CONFIG2158, 
        Y3A(4) => NET_CC_CONFIG2162, Y3A(5) => NET_CC_CONFIG2166, 
        Y3A(6) => NET_CC_CONFIG2170, Y3A(7) => NET_CC_CONFIG2174, 
        Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => 
        ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG2147, CC(1) => NET_CC_CONFIG2151, CC(2) => 
        NET_CC_CONFIG2155, CC(3) => NET_CC_CONFIG2159, CC(4) => 
        NET_CC_CONFIG2163, CC(5) => NET_CC_CONFIG2167, CC(6) => 
        NET_CC_CONFIG2171, CC(7) => NET_CC_CONFIG2175, CC(8) => 
        nc74, CC(9) => nc133, CC(10) => nc238, CC(11) => nc167);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[15]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[15]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[15]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\);
    
    \can_top_level_inst/rx_buffer_inst/un40_frame_form_w\ : CFG2
      generic map(INIT => x"4")

      port map(A => \can_top_level_inst/stat_bus[156]\, B => 
        \can_top_level_inst/rec_dlc[3]\, Y => 
        \can_top_level_inst/rx_buffer_inst/un40_frame_form_w_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[0]\ : 
        CFG4
      generic map(INIT => x"8F88")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_141\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        C => \can_top_level_inst/txtb_allow_bb[4]\, D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_load_base_id_i_0_a2\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_929\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1330\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[3]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[3]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[34]\ : 
        SLE
      port map(D => \timestamp_c[34]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[34]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[14]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[14]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[5]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[5]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[5]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2629\);
    
    \can_top_level_inst/rx_buffer_inst/rx_read_buff[8]\ : CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/rx_buffer_inst_RAM_data_out[8]\, 
        Y => \can_top_level_inst/rx_read_buff[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[6]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un12_btmc_meas_running_d\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => \can_top_level_inst/tx_trigger\, B => 
        \can_top_level_inst/btmc_reset\, Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un12_btmc_meas_running_d_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[31]\ : 
        SLE
      port map(D => \data_in_c[31]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_27[0]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => \can_top_level_inst/stat_bus[100]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1153\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_28_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/N_3900\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_pos_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/err_detected\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[102]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_meas_running_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un7_btmc_meas_running_d_Z\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un12_btmc_meas_running_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_meas_running_q_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[6]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[96]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_enable\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_enable_1\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => \AFLSDF_INV_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/destuff_enable\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE_0\ : 
        CFG3
      generic map(INIT => x"F9")

      port map(A => \can_top_level_inst/drv_bus[56]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[0]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[6]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE_0_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_36[4]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => \can_top_level_inst/stat_bus[304]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1314_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1314\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_9\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[1]\, B
         => \can_top_level_inst/can_core_inst/crc_21[0]\, C => 
        \can_top_level_inst/can_core_inst/crc_21[1]\, D => 
        \can_top_level_inst/store_data_word[0]\, FCI => ADLIB_GND, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[0]\, 
        CC => NET_CC_CONFIG2, P => NET_CC_CONFIG, Y3 => 
        NET_CC_CONFIG0, Y3A => NET_CC_CONFIG1);
    
    \timestamp_ibuf[43]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(43), Y => 
        \timestamp_ibuf[43]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[14]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[22]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[22]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[22]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4824\);
    
    \data_in_ibuf[18]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(18), Y => \data_in_ibuf[18]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[26]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[326]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.3.next_bytes_gen.shift_reg_in_6[3]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/stat_bus[372]\, B => 
        \can_top_level_inst/store_data_word[23]\, C => 
        \can_top_level_inst/can_core_inst/bds_data_out\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/shift_reg_in_6[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram108_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram109_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r_fast[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/drv_bus_fast[475]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram3_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram3_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[18]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2707\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2674\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m35_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\);
    
    \data_out_obuf[1]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(1), D => \data_out_obuf[1]/DOUT\, 
        E => \data_out_obuf[1]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[4]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_1_i\, 
        SD => ADLIB_VCC, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4[1]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[17]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un3_crc_d\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_nxt_Z\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, Y
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[2]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_5_RNID1G24\ : 
        ARI1_CC
      generic map(INIT => x"5E44E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_sn_N_2_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_5_S\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_Z[0]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_4\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2377\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_5\, 
        CC => NET_CC_CONFIG1973, P => NET_CC_CONFIG1970, Y3 => 
        NET_CC_CONFIG1971, Y3A => NET_CC_CONFIG1972);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/mode[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[6]\);
    
    \data_out_obuf[20]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[20]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[20]/DOUT\, EOUT => 
        \data_out_obuf[20]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[14]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm_0[2]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/int_stat_proc.un1_int_status_clear\ : 
        CFG4
      generic map(INIT => x"8F88")

      port map(A => \can_top_level_inst/N_138\, B => 
        \data_in_c[2]\, C => \can_top_level_inst/int_mask[2]\, D
         => \can_top_level_inst/stat_bus[253]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/un1_int_status_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[15]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test_dbl_buf.tprm[1]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[9]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/load_ffmt_w_addr\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[10]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2634\, Y
         => \can_top_level_inst/memory_registers_inst/N_2699\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[13]\);
    
    \data_out_obuf[6]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(6), D => \data_out_obuf[6]/DOUT\, 
        E => \data_out_obuf[6]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[27]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[13]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_ac0_7_RNI92UDM\ : 
        ARI1_CC
      generic map(INIT => x"56699")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[5]\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c5\, 
        C => \can_top_level_inst/stat_bus[86]\, D => ADLIB_GND, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_4\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_5\, 
        CC => NET_CC_CONFIG107, P => NET_CC_CONFIG104, Y3 => 
        NET_CC_CONFIG105, Y3A => NET_CC_CONFIG106);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[0]\);
    
    \data_out_obuf[15]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(15), D => \data_out_obuf[15]/DOUT\, 
        E => \data_out_obuf[15]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[17]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_840_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram41__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_344\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe41\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_0_0[14]\ : 
        CFG4
      generic map(INIT => x"48C0")

      port map(A => \can_top_level_inst/rec_dlc[0]\, B => 
        \can_top_level_inst/rec_dlc[3]\, C => 
        \can_top_level_inst/rec_dlc[2]\, D => 
        \can_top_level_inst/rec_dlc[1]\, Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_0_0_Z[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[7]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[6]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[1]\, 
        D => \can_top_level_inst/tran_word[31]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[8]\ : 
        SLE
      port map(D => \timestamp_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_arb\ : 
        CFG4
      generic map(INIT => x"0010")

      port map(A => can_tx_c, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_arb_1_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1299_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/bit_err_arb\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un6_ctrl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_address_i_i_m2[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_13[11]\ : 
        CFG4
      generic map(INIT => x"F4F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1557_2\, 
        D => \can_top_level_inst/drv_bus[216]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1557\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[29]\);
    
    \timestamp_ibuf[10]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(10), Y => 
        \timestamp_ibuf[10]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[8]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_wdata[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_11[17]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/drv_bus[222]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_793\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_i_a2_0_RNIA25P[3]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => \can_top_level_inst/is_overload\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[6]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[9]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm_0[7]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[7]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_address_i_i_m2[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_185\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[10]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_1\ : 
        CFG3
      generic map(INIT => x"13")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt_rep1\, 
        B => \can_top_level_inst/drv_bus_475_rep2\, C => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[13]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[26]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_nbt_Z[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_5_S\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_2[26]\ : 
        CFG4
      generic map(INIT => x"FFF8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5_Z\, 
        B => \can_top_level_inst/stat_bus[214]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_0_Z[26]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[1210]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_2_Z[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[16]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[16]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[16]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_6\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[1]\, B
         => \can_top_level_inst/can_core_inst/crc_15[0]\, C => 
        \can_top_level_inst/can_core_inst/crc_15[1]\, D => 
        \can_top_level_inst/store_data_word[0]\, FCI => ADLIB_GND, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[0]\, 
        CC => NET_CC_CONFIG50, P => NET_CC_CONFIG47, Y3 => 
        NET_CC_CONFIG48, Y3A => NET_CC_CONFIG49);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[8]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_12[2]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => \can_top_level_inst/rx_mof\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/txtb_prorities_0[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1214\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[7]\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[7]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_Z[7]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_2_0\, 
        D => \adress_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tick_state_reg_23_sqmuxa_0_a4\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\, 
        Y => \can_top_level_inst/stat_bus[378]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un6_ctrl_ctr_ce\ : 
        CFG4
      generic map(INIT => x"ECCC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_7_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1215_1\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_5_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un6_ctrl_ctr_ce_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[27]\);
    
    \data_in_ibuf[1]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[1]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[1]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m44_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[11]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_2[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram110_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram111_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y7[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_38_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1141\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1156\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_38_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[7]\ : 
        CFG4
      generic map(INIT => x"F888")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/txtb_parity_error_valid[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_9[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => ADLIB_VCC, B => ADLIB_VCC, C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[4]\, 
        D => ADLIB_VCC, Y => OPEN);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_0_1[0]\ : 
        CFG4
      generic map(INIT => x"7525")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/txtb_prorities_0[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1_0_Z[0]\, 
        D => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_0_1_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI19RLA_S[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[287]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[8]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_pointer_inc_1_i_ac0_5\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \can_top_level_inst/rx_read_pointer[3]\, B
         => \can_top_level_inst/rx_read_pointer[2]\, C => 
        \can_top_level_inst/rx_read_pointer[1]\, D => 
        \can_top_level_inst/rx_read_pointer[0]\, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_pointer_inc_1_i_c4\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_28\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_23_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_16_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_28_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[13]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[13]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[13]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m43_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a4_0_0[23]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[15]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[17]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a4_0_0_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[22]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[22]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[22]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[1]\, 
        C => \can_top_level_inst/txtb_port_b_address[1]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram39__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_341\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe39\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => \can_top_level_inst/drv_bus[17]\, B => 
        \can_top_level_inst/drv_bus[24]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_0_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_1_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_1_Z\, 
        CC => NET_CC_CONFIG2151, P => NET_CC_CONFIG2148, Y3 => 
        NET_CC_CONFIG2149, Y3A => NET_CC_CONFIG2150);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/command_reg_comp/bit_gen.7.reg_present_gen.reg_autoclear_gen.reg_value_r_14[7]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/access_active_1[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        D => \data_in_c[7]\, Y => 
        \can_top_level_inst/drv_bus[512]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNI3Q2C[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG1602, P(1) => NET_CC_CONFIG1606, P(2) => 
        NET_CC_CONFIG1610, P(3) => NET_CC_CONFIG1614, P(4) => 
        NET_CC_CONFIG1618, P(5) => NET_CC_CONFIG1622, P(6) => 
        NET_CC_CONFIG1626, P(7) => NET_CC_CONFIG1630, P(8) => 
        NET_CC_CONFIG1634, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, 
        P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1603, Y3(1) => 
        NET_CC_CONFIG1607, Y3(2) => NET_CC_CONFIG1611, Y3(3) => 
        NET_CC_CONFIG1615, Y3(4) => NET_CC_CONFIG1619, Y3(5) => 
        NET_CC_CONFIG1623, Y3(6) => NET_CC_CONFIG1627, Y3(7) => 
        NET_CC_CONFIG1631, Y3(8) => NET_CC_CONFIG1635, Y3(9) => 
        ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, 
        Y3A(0) => NET_CC_CONFIG1604, Y3A(1) => NET_CC_CONFIG1608, 
        Y3A(2) => NET_CC_CONFIG1612, Y3A(3) => NET_CC_CONFIG1616, 
        Y3A(4) => NET_CC_CONFIG1620, Y3A(5) => NET_CC_CONFIG1624, 
        Y3A(6) => NET_CC_CONFIG1628, Y3A(7) => NET_CC_CONFIG1632, 
        Y3A(8) => NET_CC_CONFIG1636, Y3A(9) => ADLIB_VCC, Y3A(10)
         => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG1605, CC(1) => NET_CC_CONFIG1609, CC(2) => 
        NET_CC_CONFIG1613, CC(3) => NET_CC_CONFIG1617, CC(4) => 
        NET_CC_CONFIG1621, CC(5) => NET_CC_CONFIG1625, CC(6) => 
        NET_CC_CONFIG1629, CC(7) => NET_CC_CONFIG1633, CC(8) => 
        NET_CC_CONFIG1637, CC(9) => nc84, CC(10) => nc39, CC(11)
         => nc72);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[17]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_4[16]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => \can_top_level_inst/drv_bus[465]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => \can_top_level_inst/drv_bus[159]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_748\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_17[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/int_mask[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1143\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[17]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1[21]\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[21]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram76_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram77_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_2[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_1_RNO[30]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \adress_c[7]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => \timestamp_c[30]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[1214]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.7.int_module_inst/int_mask_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.7.int_module_inst/int_mask_load_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_mask[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_2[3]\ : 
        CFG4
      generic map(INIT => x"AFAB")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_Z[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_854_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/next_state_17_sqmuxa\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_2_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[2]\ : 
        CFG4
      generic map(INIT => x"F088")

      port map(A => \can_top_level_inst/rec_dlc[2]\, B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        C => \can_top_level_inst/rx_buffer_inst/N_2280\, D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_6_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[513]\, D => 
        \can_top_level_inst/drv_bus[149]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_6_1_0_y0[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[20]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[14]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => swr_c, B => \sbe_c[0]\, C => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_0\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m22_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[13]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[22]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[22]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[18]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry[4]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[3]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[4]\, 
        CC => NET_CC_CONFIG2362, P => NET_CC_CONFIG2359, Y3 => 
        NET_CC_CONFIG2360, Y3A => NET_CC_CONFIG2361);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[16]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[16]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2640\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_0[2]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[1]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa13\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe13\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a2[3]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_192\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un4_phase_err_sjw_by_one_1.CO2\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => \can_top_level_inst/drv_bus[53]\, B => 
        \can_top_level_inst/drv_bus[52]\, C => 
        \can_top_level_inst/drv_bus[51]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/CO2\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/op_eq.un2_ctrl_ctr_one_2_0\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[7]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[6]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[5]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[4]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_one_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i[1]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[1]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[85]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[8]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/g0_0_0\ : 
        CFG4
      generic map(INIT => x"FFEF")

      port map(A => \can_top_level_inst/drv_bus[9]\, B => 
        \can_top_level_inst/drv_bus[10]\, C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/g0_i_a3_4\, 
        D => \can_top_level_inst/drv_bus[15]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un2_tq_counter_allow\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram84__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_345\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe84\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8_RNICES6G6\ : 
        ARI1_CC
      generic map(INIT => x"49CCC")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[7]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_6\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_7\, 
        CC => NET_CC_CONFIG2728, P => NET_CC_CONFIG2725, Y3 => 
        NET_CC_CONFIG2726, Y3A => NET_CC_CONFIG2727);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[19]\);
    
    \timestamp_ibuf[30]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[30]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[30]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[7]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[17]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[17]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[12]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[12]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[12]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_34_1[22]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/stat_bus[210]\, B => 
        \can_top_level_inst/drv_bus[381]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1006_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram78_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe78\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram78_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[18]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[6]\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[6]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_Z[6]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_1\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[17]\, B
         => \can_top_level_inst/can_core_inst/crc_21[16]\, C => 
        \can_top_level_inst/can_core_inst/crc_21[17]\, D => 
        \can_top_level_inst/store_data_word[16]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[7]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[8]\, 
        CC => NET_CC_CONFIG34, P => NET_CC_CONFIG31, Y3 => 
        NET_CC_CONFIG32, Y3A => NET_CC_CONFIG33);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram95_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe95\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram95_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_2[14]\ : 
        CFG4
      generic map(INIT => x"4D6F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_1_1_Z[14]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[32]\ : 
        SLE
      port map(D => \timestamp_c[32]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[32]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[9]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[5]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[3]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[11]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m25_2\ : 
        CFG4
      generic map(INIT => x"DDA0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i77_mux\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i75_mux\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m25_1_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_26_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_8_RNO[27]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/stat_bus[284]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac25_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[443]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[5]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[4]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[5]\);
    
    \data_in_ibuf[7]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[7]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[7]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNII8D41[1]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[1]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_0\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_1\, 
        CC => NET_CC_CONFIG1281, P => NET_CC_CONFIG1278, Y3 => 
        NET_CC_CONFIG1279, Y3A => NET_CC_CONFIG1280);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_02_m_3_1_0_wmux\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_16_i\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06_i\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_020\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_022\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_02_m_3_1_0_y0\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[0]\ : 
        CFG4
      generic map(INIT => x"F088")

      port map(A => \can_top_level_inst/rec_dlc[0]\, B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        C => \can_top_level_inst/rx_buffer_inst/N_2278\, D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[0]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[14]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[24]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[24]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[24]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[23]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[23]\);
    
    \data_in_ibuf[17]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[17]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[17]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[180]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[19]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_0_1[17]\ : 
        CFG3
      generic map(INIT => x"46")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[193]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_0_1_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[23]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0_RNIE94KR2[2]\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[2]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_51\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_1\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_2\, 
        CC => NET_CC_CONFIG1421, P => NET_CC_CONFIG1418, Y3 => 
        NET_CC_CONFIG1419, Y3A => NET_CC_CONFIG1420);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[20]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[20]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[20]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un1_tq_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[47]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_shift_cmd_gen.0.un1_rx_shift_cmd_0\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[19]\);
    
    \can_top_level_inst/rx_buffer_inst/un1_read_counter_dlto4\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[2]\, 
        B => \can_top_level_inst/rx_buffer_inst/rx_mof_1\, C => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[3]\, 
        Y => \can_top_level_inst/rx_mof\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[2]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un2_commit_rx_frame\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_mem_free[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[25]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[3]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un6_first_ssp_d\ : 
        CFG3
      generic map(INIT => x"D5")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_14_Z\, 
        B => \can_top_level_inst/tx_trigger\, C => 
        \can_top_level_inst/gen_first_ssp\, Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un6_first_ssp_d_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_length[1]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/un2_data_len_can_2_0_i\, 
        B => \can_top_level_inst/tran_dlc[1]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[30]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[10]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_51\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[3]\, B => 
        \can_top_level_inst/drv_bus[146]\, C => 
        \can_top_level_inst/drv_bus[179]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_32\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[0]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[1]\, 
        CC => NET_CC_CONFIG694, P => NET_CC_CONFIG691, Y3 => 
        NET_CC_CONFIG692, Y3A => NET_CC_CONFIG693);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[8]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNILDM31[6]\ : 
        CFG3
      generic map(INIT => x"2A")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => \can_top_level_inst/txtb_sw_cmd.set_abt\, C => 
        \can_top_level_inst/txtb_sw_cmd_index[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_135_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[417]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[29]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe10\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g1_0_0_a3_0_4\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => \can_top_level_inst/drv_bus[11]\, B => 
        \can_top_level_inst/drv_bus[15]\, C => 
        \can_top_level_inst/drv_bus[14]\, D => 
        \can_top_level_inst/drv_bus[13]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g1_0_0_a3_0_4_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_34[9]\ : 
        CFG4
      generic map(INIT => x"CECC")

      port map(A => \can_top_level_inst/stat_bus[197]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1502_2\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1502\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_2[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram78_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram79_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y7[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_parity_check_valid_0_RNO_1\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[7]\, B
         => \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[1]\, Y
         => \can_top_level_inst/tx_arbitrator_inst/txtb_m1_e_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[156]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8[5]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2377\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[5]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_parity_check_valid_0_RNO_2\ : 
        CFG2
      generic map(INIT => x"9")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[0]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_parity_check_valid_0_RNO_2_Z\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[20]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[20]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa_0_a2_4\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_189\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_186\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_6[2]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_36_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3_Z[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_1_0\ : 
        ARI1_CC
      generic map(INIT => x"5CC66")

      port map(A => \can_top_level_inst/drv_bus[47]\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[1]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_0\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_1\, 
        CC => NET_CC_CONFIG2664, P => NET_CC_CONFIG2661, Y3 => 
        NET_CC_CONFIG2662, Y3A => NET_CC_CONFIG2663);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_21[25]\ : 
        CFG4
      generic map(INIT => x"E4A0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1106\, 
        D => \can_top_level_inst/drv_bus[230]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1107\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_9[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => ADLIB_VCC, B => ADLIB_VCC, C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[4]\, 
        D => ADLIB_VCC, Y => OPEN);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/un41_txtb_pointer_meta_d_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_RNIIFE91[0]\ : 
        CFG3
      generic map(INIT => x"0E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[2]\, 
        B => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, C
         => \can_top_level_inst/rx_buffer_inst/stored_ts\, Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe20\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[4]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_139\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_170_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_2_0[10]\ : 
        CFG4
      generic map(INIT => x"DDA0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1533\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1532\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1_0_Z[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_2_0_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[7]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[15]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[15]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[15]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4275\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_2_1.CO2\ : 
        CFG3
      generic map(INIT => x"D4")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_Z[2]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO2_1_1\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[18]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[27]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[211]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_address_i_i_m2[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_189\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram79_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe79\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram79_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[21]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[21]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[0]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un1_tq_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram6_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe6\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram6_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_0[1]\ : 
        CFG4
      generic map(INIT => x"F1F0")

      port map(A => \can_top_level_inst/txtb_hw_cmd.err\, B => 
        \can_top_level_inst/txtb_hw_cmd.arbl\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_163\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_0_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_832_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[3]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[404]\, B => 
        \can_top_level_inst/stat_bus[85]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_3_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_4_Z\, 
        CC => NET_CC_CONFIG1545, P => NET_CC_CONFIG1542, Y3 => 
        NET_CC_CONFIG1543, Y3A => NET_CC_CONFIG1544);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0_RNINH5LI3[3]\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[3]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_52\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_2\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_3\, 
        CC => NET_CC_CONFIG1425, P => NET_CC_CONFIG1422, Y3 => 
        NET_CC_CONFIG1423, Y3A => NET_CC_CONFIG1424);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o2[26]\ : 
        CFG2
      generic map(INIT => x"D")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_go_to_suspend_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_952\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1048\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNILUVN[31]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_ena\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/tx_sr_output\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_output_i_m_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[145]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_0_2\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_0_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[7]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[7]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[7]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_25_2[8]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/txtb_prorities_2[0]\, D => 
        \can_top_level_inst/rx_frame_count[4]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1455_2\);
    
    \data_in_ibuf[3]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[3]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[3]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[31]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[31]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[31]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_0_y0[24]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1058\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1061\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_0_y1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate_3_0_.m20\ : 
        CFG4
      generic map(INIT => x"1504")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[6]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[4]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lt4\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/i5_mux\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[25]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[31]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_2_0\ : 
        ARI1_CC
      generic map(INIT => x"5CC66")

      port map(A => \can_top_level_inst/drv_bus[48]\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[2]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_1\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_2\, 
        CC => NET_CC_CONFIG2668, P => NET_CC_CONFIG2665, Y3 => 
        NET_CC_CONFIG2666, Y3A => NET_CC_CONFIG2667);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask[0]\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1192\, 
        C => \can_top_level_inst/drv_bus[477]\, D => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[24]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[17]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_3\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[9]\, B
         => \can_top_level_inst/can_core_inst/crc_17[8]\, C => 
        \can_top_level_inst/can_core_inst/crc_17[9]\, D => 
        \can_top_level_inst/store_data_word[8]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[3]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[4]\, 
        CC => NET_CC_CONFIG2543, P => NET_CC_CONFIG2540, Y3 => 
        NET_CC_CONFIG2541, Y3A => NET_CC_CONFIG2542);
    
    
        \can_top_level_inst/bus_sampling_inst/can_rx_sig_sync_inst/sync\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/can_rx_sig_sync_inst/rff_Z\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rx_data_wbs\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[18]\ : 
        CFG4
      generic map(INIT => x"3120")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_975\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_808_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[20]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_38_1_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_38_1_0_y0[23]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1044\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1046\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/next_state_17_sqmuxa_0_a4\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_980\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/next_state_17_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[15]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_0\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \can_top_level_inst/stat_bus[89]\, B => 
        \can_top_level_inst/stat_bus[88]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNIQLJA1_S\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_0_Z\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_4_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_5_Z\, 
        CC => NET_CC_CONFIG2398, P => NET_CC_CONFIG2395, Y3 => 
        NET_CC_CONFIG2396, Y3A => NET_CC_CONFIG2397);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[4]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[3]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/crc_enable_i_a4_1\ : 
        CFG4
      generic map(INIT => x"5100")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_944\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1299_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/crc_enable_i_a4_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_write_i\ : 
        CFG4
      generic map(INIT => x"8F80")

      port map(A => \data_in_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_tst_control_reg_comp_wr_select[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/rx_buffer_inst/un5_ram_write_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_write_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[12]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[8]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[12]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[12]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[25]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[30]\);
    
    \can_top_level_inst/tx_arbitrator_inst/validated_buffer_1\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[1]\, B
         => \can_top_level_inst/txtb_hw_cmd_index[0]\, C => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_10_1_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_10_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[43]\, D => 
        \can_top_level_inst/rx_frame_count[5]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1478\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_4[1]\ : 
        CFG4
      generic map(INIT => x"0888")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_9[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_2[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ide_is_arbitration_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_4[1]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_dat_sel_ctr_i[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[273]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\);
    
    \timestamp_ibuf[35]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[35]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[35]/YIN\);
    
    \can_top_level_inst/memory_registers_inst/ctr_pres_sel_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/ctr_pres[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => res_n_out_c, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD
         => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/drv_bus[426]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[1]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_3[26]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac25_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac4_Z\, 
        C => \can_top_level_inst/stat_bus[283]\, D => 
        \can_top_level_inst/stat_bus[246]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_3_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[46]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_4\ : 
        CFG4
      generic map(INIT => x"956A")

      port map(A => \can_top_level_inst/drv_bus[60]\, B => 
        \can_top_level_inst/drv_bus[59]\, C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/CO2\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[4]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_4_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[23]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39\ : 
        CFG4
      generic map(INIT => x"F0F7")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_129\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_135\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_39\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_12\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[19]\ : 
        SLE
      port map(D => \timestamp_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry[2]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[2]\, 
        D => \can_top_level_inst/rx_write_pointer[2]\, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[1]\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[2]\, 
        CC => NET_CC_CONFIG377, P => NET_CC_CONFIG374, Y3 => 
        NET_CC_CONFIG375, Y3A => NET_CC_CONFIG376);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[16]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[20]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2644\, Y
         => \can_top_level_inst/memory_registers_inst/N_2709\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[18]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/load_ts_lw_addr_i_RNI576T51\ : 
        CFG4
      generic map(INIT => x"2022")

      port map(A => \can_top_level_inst/tx_arbitrator_inst/N_94\, 
        B => \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, 
        C => \can_top_level_inst/drv_bus[472]\, D => 
        \can_top_level_inst/tx_arbitrator_inst/un1_m3_0_a3_0\, Y
         => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/un1_N_7_mux_i_0_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un3_reinteg_ctr_ce\ : 
        CFG4
      generic map(INIT => x"0F8F")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        B => \can_top_level_inst/can_core_inst/pc_rx_trigger\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy_Y[0]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_980\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un3_reinteg_ctr_ce_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un2_commit_rx_frame\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_mem_free[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\);
    
    \timestamp_ibuf[29]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(29), Y => 
        \timestamp_ibuf[29]/YIN\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[16]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[16]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[16]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped[1]\ : 
        CFG4
      generic map(INIT => x"6AC0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        C => \can_top_level_inst/can_core_inst/crc_21[1]\, D => 
        \can_top_level_inst/tran_frame_test.fcrc\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[6]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[300]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_5_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[5]\);
    
    \timestamp_ibuf[45]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(45), Y => 
        \timestamp_ibuf[45]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_38_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1179\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1194\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_38_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_mem_free[1]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_s_1_2644_FCO\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_1_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_1_Z\, 
        CC => NET_CC_CONFIG2422, P => NET_CC_CONFIG2419, Y3 => 
        NET_CC_CONFIG2420, Y3A => NET_CC_CONFIG2421);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[1]\, 
        C => \can_top_level_inst/txtb_port_b_address[1]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_23[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[275]\, C => 
        \can_top_level_inst/drv_bus[147]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1301\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_RNO[9]\ : 
        ARI1_CC
      generic map(INIT => x"4EE00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, C => 
        \can_top_level_inst/prescaler_inst/is_tseg2\, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_8\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[9]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG2736, P => 
        NET_CC_CONFIG2733, Y3 => NET_CC_CONFIG2734, Y3A => 
        NET_CC_CONFIG2735);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIILP4F_S[21]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[209]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIF6TG5[5]\ : 
        ARI1_CC
      generic map(INIT => x"5D827")

      port map(A => \timestamp_c[37]\, B => 
        \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[5]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[5]\, 
        FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_4\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_5\, 
        CC => NET_CC_CONFIG947, P => NET_CC_CONFIG944, Y3 => 
        NET_CC_CONFIG945, Y3A => NET_CC_CONFIG946);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[14]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry[3]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[2]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[3]\, 
        CC => NET_CC_CONFIG1067, P => NET_CC_CONFIG1064, Y3 => 
        NET_CC_CONFIG1065, Y3A => NET_CC_CONFIG1066);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[1]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2527\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[181]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_1_1.CO1_0_0\ : 
        CFG4
      generic map(INIT => x"0A8E")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3_Z[1]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3_Z[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0_Z[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0_Z[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_0[20]\ : 
        CFG4
      generic map(INIT => x"C9C8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_0_1_Z[20]\, 
        D => \can_top_level_inst/drv_bus[258]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_0_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[7]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2533\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[10]\);
    
    \can_top_level_inst/memory_registers_inst/status_comb_2[2]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => \can_top_level_inst/txtb_state_4[3]\, B => 
        \can_top_level_inst/memory_registers_inst_txnf_calc_proc_6_un6_txtb_state_2\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_147\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_147\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/status_comb_2_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[17]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1[1]\ : 
        CFG4
      generic map(INIT => x"B9A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1181\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1180\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[30]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_20[25]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[25]\, B
         => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1106\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[15]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[29]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram6_[0]\ : 
        SLE
      port map(D => can_tx_c, CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, 
        EN => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe6\, 
        ALn => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn
         => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram6_0\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un1_bst_ctr_q3\ : 
        CFG4
      generic map(INIT => x"ACFF")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_data_in\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un3_same_bits_rst_i_0\, 
        C => \can_top_level_inst/can_core_inst/tx_frame_no_sof\, 
        D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un1_bst_ctr_q3_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[18]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/bit_gen.10.reg_present_gen.reg_autoclear_gen.reg_value_r_66[10]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[1]\, 
        B => \data_in_c[10]\, Y => 
        \can_top_level_inst/memory_registers_inst/ctr_pres[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[16]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[16]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[16]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_28[21]\ : 
        CFG4
      generic map(INIT => x"AC00")

      port map(A => \can_top_level_inst/stat_bus[294]\, B => 
        \can_top_level_inst/drv_bus[102]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_962\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[29]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[2]\, 
        D => \can_top_level_inst/prescaler_inst/tseg1_dbt[2]\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_a3[3]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        B => \can_top_level_inst/txtb_allow_bb[6]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst_txnf_calc_proc_6_un6_txtb_state_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[2]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[4]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[4]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[4]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[4]\);
    
    \timestamp_ibuf[56]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(56), Y => 
        \timestamp_ibuf[56]/YIN\);
    
    
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/tx_trigger_0_1\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \can_top_level_inst/prescaler_inst/N_27\, B
         => 
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/tx_trig_req_flag_q_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/tx_trigger_0_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[14]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_data_access_signaller_comp/access_active_0[0]\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_data_access_signaller_comp/be_active_Z\, 
        B => \adress_c[6]\, C => \adress_c[5]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_data_access_signaller_comp/access_active_0_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[10]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[10]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[10]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2634\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[26]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[26]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[26]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[31]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_1[30]\ : 
        CFG4
      generic map(INIT => x"F8F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_12_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_12_0_0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_19_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_12_0_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[29]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_02_m_3_1_wmux_0\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_02_m_3_1_0_y0\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06_i\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_021\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_023\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_02\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[18]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un8_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[19]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[19]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[19]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI7GBIP4[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[52]\, B => 
        \can_top_level_inst/tran_word[20]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_19\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_20\, 
        CC => NET_CC_CONFIG1007, P => NET_CC_CONFIG1004, Y3 => 
        NET_CC_CONFIG1005, Y3A => NET_CC_CONFIG1006);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_45\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[14]\, B => 
        \can_top_level_inst/drv_bus[157]\, C => 
        \can_top_level_inst/drv_bus[190]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_36\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[6]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[7]\, 
        CC => NET_CC_CONFIG718, P => NET_CC_CONFIG715, Y3 => 
        NET_CC_CONFIG716, Y3A => NET_CC_CONFIG717);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_1[18]\ : 
        CFG4
      generic map(INIT => x"0B08")

      port map(A => \can_top_level_inst/tran_identifier[4]\, B
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[18]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm_0[2]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[2]\, 
        B => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        C => \can_top_level_inst/prescaler_inst/h_sync_valid\, D
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[14]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[24]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[24]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram6_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram7_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[1]\, 
        C => \can_top_level_inst/txtb_port_b_address[1]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNILPAQ1[4]\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[4]\, 
        B => \can_top_level_inst/drv_bus[379]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_3\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNILPAQ1_S[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_4\, 
        CC => NET_CC_CONFIG1585, P => NET_CC_CONFIG1582, Y3 => 
        NET_CC_CONFIG1583, Y3A => NET_CC_CONFIG1584);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m34_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOOLCL_S[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[219]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2_0[1]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txtb_parity_error_valid[5]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_140\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_6[15]\ : 
        CFG4
      generic map(INIT => x"6420")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[158]\, D => 
        \can_top_level_inst/drv_bus[31]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_712\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIUFAP[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[2]\, 
        Y => \can_top_level_inst/tran_word[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[22]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[22]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[23]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[23]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_RNIJ6NH[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_GND, CO => OPEN, P(0) => 
        NET_CC_CONFIG1950, P(1) => NET_CC_CONFIG1954, P(2) => 
        NET_CC_CONFIG1958, P(3) => NET_CC_CONFIG1962, P(4) => 
        NET_CC_CONFIG1966, P(5) => NET_CC_CONFIG1970, P(6) => 
        NET_CC_CONFIG1974, P(7) => NET_CC_CONFIG1978, P(8) => 
        ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11)
         => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1951, Y3(1) => 
        NET_CC_CONFIG1955, Y3(2) => NET_CC_CONFIG1959, Y3(3) => 
        NET_CC_CONFIG1963, Y3(4) => NET_CC_CONFIG1967, Y3(5) => 
        NET_CC_CONFIG1971, Y3(6) => NET_CC_CONFIG1975, Y3(7) => 
        NET_CC_CONFIG1979, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG1952, Y3A(1) => NET_CC_CONFIG1956, Y3A(2)
         => NET_CC_CONFIG1960, Y3A(3) => NET_CC_CONFIG1964, 
        Y3A(4) => NET_CC_CONFIG1968, Y3A(5) => NET_CC_CONFIG1972, 
        Y3A(6) => NET_CC_CONFIG1976, Y3A(7) => NET_CC_CONFIG1980, 
        Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => 
        ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG1953, CC(1) => NET_CC_CONFIG1957, CC(2) => 
        NET_CC_CONFIG1961, CC(3) => NET_CC_CONFIG1965, CC(4) => 
        NET_CC_CONFIG1969, CC(5) => NET_CC_CONFIG1973, CC(6) => 
        NET_CC_CONFIG1977, CC(7) => NET_CC_CONFIG1981, CC(8) => 
        nc256, CC(9) => nc212, CC(10) => nc205, CC(11) => nc82);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/load_ts_lw_addr_i\ : 
        CFG3
      generic map(INIT => x"CD")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[7]\, B
         => \can_top_level_inst/tx_arbitrator_inst/N_120\, C => 
        \can_top_level_inst/tx_arbitrator_inst/N_109\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/N_94\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4[3]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s_2638_RNILHR9\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un8_read_increment_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa19\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe19\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[4]\ : 
        CFG4
      generic map(INIT => x"F888")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIK7JE8[11]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[284]\, C => 
        \can_top_level_inst/stat_bus[268]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_10\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIK7JE8_S[11]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_11\, 
        CC => NET_CC_CONFIG2612, P => NET_CC_CONFIG2609, Y3 => 
        NET_CC_CONFIG2610, Y3A => NET_CC_CONFIG2611);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa12\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe12\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[16]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[16]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[31]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[31]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[31]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\);
    
    
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/sync_flag\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/un1_h_or_re_sync_edge_Z\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/un6_sync_flag_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/sync_flag\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram0__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_346\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_18_1[11]\ : 
        CFG4
      generic map(INIT => x"3210")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/int_vector[11]\, D => 
        \can_top_level_inst/drv_bus[249]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1562_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNIACKK2[6]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[5]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[6]\, 
        CC => NET_CC_CONFIG1629, P => NET_CC_CONFIG1626, Y3 => 
        NET_CC_CONFIG1627, Y3A => NET_CC_CONFIG1628);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o2[2]\ : 
        CFG2
      generic map(INIT => x"B")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_go_to_suspend_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1047\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[4]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/int_ena_proc.un1_int_ena_clear\ : 
        CFG4
      generic map(INIT => x"A800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        D => \data_in_c[10]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/un1_int_ena_clear\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram64_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe64\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram64_0\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_o2_0[6]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \can_top_level_inst/drv_bus[476]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_122\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_29\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_20_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_19_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_18_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_17_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_29_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[26]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_9_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO, CO => OPEN, P(0) => 
        NET_CC_CONFIG15, P(1) => NET_CC_CONFIG19, P(2) => 
        NET_CC_CONFIG23, P(3) => NET_CC_CONFIG27, P(4) => 
        NET_CC_CONFIG31, P(5) => NET_CC_CONFIG35, P(6) => 
        NET_CC_CONFIG39, P(7) => NET_CC_CONFIG43, P(8) => 
        ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11)
         => ADLIB_VCC, Y3(0) => NET_CC_CONFIG16, Y3(1) => 
        NET_CC_CONFIG20, Y3(2) => NET_CC_CONFIG24, Y3(3) => 
        NET_CC_CONFIG28, Y3(4) => NET_CC_CONFIG32, Y3(5) => 
        NET_CC_CONFIG36, Y3(6) => NET_CC_CONFIG40, Y3(7) => 
        NET_CC_CONFIG44, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG17, Y3A(1) => NET_CC_CONFIG21, Y3A(2) => 
        NET_CC_CONFIG25, Y3A(3) => NET_CC_CONFIG29, Y3A(4) => 
        NET_CC_CONFIG33, Y3A(5) => NET_CC_CONFIG37, Y3A(6) => 
        NET_CC_CONFIG41, Y3A(7) => NET_CC_CONFIG45, Y3A(8) => 
        ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => ADLIB_VCC, 
        Y3A(11) => ADLIB_VCC, CC(0) => NET_CC_CONFIG18, CC(1) => 
        NET_CC_CONFIG22, CC(2) => NET_CC_CONFIG26, CC(3) => 
        NET_CC_CONFIG30, CC(4) => NET_CC_CONFIG34, CC(5) => 
        NET_CC_CONFIG38, CC(6) => NET_CC_CONFIG42, CC(7) => 
        NET_CC_CONFIG46, CC(8) => nc145, CC(9) => nc181, CC(10)
         => nc160, CC(11) => nc57);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[29]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_segm_counter_ce\ : 
        CFG4
      generic map(INIT => x"FEEE")

      port map(A => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, B => 
        \can_top_level_inst/prescaler_inst/segment_end\, C => 
        \can_top_level_inst/dbt_ctrs_en\, D => 
        \can_top_level_inst/prescaler_inst_tq_edge_dbt\, Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_segm_counter_ce_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[1]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram86__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_345\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe86\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram2__RNO[0]\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[2]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[1]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_34_2[5]\ : 
        CFG4
      generic map(INIT => x"D800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[37]\, C => 
        \can_top_level_inst/stat_bus[225]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1350_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[17]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[16]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[17]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[27]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_2_0[6]\ : 
        CFG4
      generic map(INIT => x"DDA0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1381\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1380\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1_0_Z[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_2_0_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a2[1]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_190\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[26]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2552\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2484\, Y
         => \can_top_level_inst/memory_registers_inst/N_2682\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[1]\, 
        C => \can_top_level_inst/txtb_port_b_address[1]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[11]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un8_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_segm_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_34[20]\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => \timestamp_c[20]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_930_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_930\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[27]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIA6981_S[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[6]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[23]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[23]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[14]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNI1CH82[5]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/CO0\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[4]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[5]\, 
        CC => NET_CC_CONFIG1625, P => NET_CC_CONFIG1622, Y3 => 
        NET_CC_CONFIG1623, Y3A => NET_CC_CONFIG1624);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[11]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram8_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram9_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[17]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram23__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_345\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe23\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[304]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_23[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[273]\, C => 
        \can_top_level_inst/drv_bus[145]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1225\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[24]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/int_status\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/un2_int_status_set\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/un1_int_status_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_vector[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_3[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_4_1\ : 
        CFG3
      generic map(INIT => x"A2")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_26\, 
        C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[3]\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_4_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8_RNI5K8B67\ : 
        ARI1_CC
      generic map(INIT => x"49CCC")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[8]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_7\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[8]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_8\, 
        CC => NET_CC_CONFIG1445, P => NET_CC_CONFIG1442, Y3 => 
        NET_CC_CONFIG1443, Y3A => NET_CC_CONFIG1444);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[9]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[31]\);
    
    \timestamp_ibuf[62]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(62), Y => 
        \timestamp_ibuf[62]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[10]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[10]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[10]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\);
    
    \data_out_obuf[24]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[24]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[24]/DOUT\, EOUT => 
        \data_out_obuf[24]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4[0]\ : 
        CFG4
      generic map(INIT => x"0504")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_926\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1189\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_2[2]\ : 
        CFG4
      generic map(INIT => x"F1F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_968\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[2]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1047\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_2_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[20]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[20]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram4_[0]\ : 
        SLE
      port map(D => can_tx_c, CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, 
        EN => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe4\, 
        ALn => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn
         => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram4_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[7]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[7]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[7]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/tx_double_parity_error_1_sqmuxa_1_0\ : 
        CFG4
      generic map(INIT => x"444C")

      port map(A => \can_top_level_inst/drv_bus[475]\, B => 
        \can_top_level_inst/memory_registers_inst/command[10]\, C
         => \can_top_level_inst/txtb_parity_error_valid[7]\, D
         => \can_top_level_inst/txtb_parity_error_valid[1]\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tx_double_parity_error_1_sqmuxa_1_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa16\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe16\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa17\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8_2\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa17\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/commit_dbl_bufs_3_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4C00")

      port map(A => \can_top_level_inst/drv_bus[476]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[3]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/tx_arb_parity_check_valid_1\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un1_bt_ctr_clear_i\ : 
        CFG4
      generic map(INIT => x"FFEA")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segment_end_1\, 
        B => \can_top_level_inst/prescaler_inst/N_5058\, C => 
        \can_top_level_inst/prescaler_inst/is_tseg1\, D => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, Y => 
        \can_top_level_inst/prescaler_inst/bt_ctr_clear\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2[5]\ : 
        CFG4
      generic map(INIT => x"5708")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_1[5]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_1_Z[5]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/int_mask_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/int_mask_load_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_mask[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_6_1[1]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_6_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa18\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe18\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.7.int_module_inst/int_mask_load\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[7]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.7.int_module_inst/int_mask_load_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram54__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_349\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe54\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\);
    
    \data_out_obuf[15]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[15]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[15]/DOUT\, EOUT => 
        \data_out_obuf[15]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[23]\);
    
    \I_1/U0_RGB1\ : RGB
      port map(A => \I_1/U0_Y\, EN => ADLIB_VCC, Y => 
        \AFLSDF_INV_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m112_2_1\ : 
        CFG4
      generic map(INIT => x"159D")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/stat_bus[262]\, D => 
        \can_top_level_inst/drv_bus[86]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m112_2_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_147\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1149\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1150\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_y0_0[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[12]\ : 
        CFG4
      generic map(INIT => x"E020")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        D => \can_top_level_inst/tran_word[20]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0_RNI2V1ID1[0]\ : 
        ARI1_CC
      generic map(INIT => x"59C63")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[0]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_49\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[0]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0\, 
        CC => NET_CC_CONFIG1413, P => NET_CC_CONFIG1410, Y3 => 
        NET_CC_CONFIG1411, Y3A => NET_CC_CONFIG1412);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNIPCES1[4]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[3]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[4]\, 
        CC => NET_CC_CONFIG1621, P => NET_CC_CONFIG1618, Y3 => 
        NET_CC_CONFIG1619, Y3A => NET_CC_CONFIG1620);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[31]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/test_registers_out.tst_wdata[31]\, C
         => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram126__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_347\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe126\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_3[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_932\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_934\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_928_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1193\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI16B69_S[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[269]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[16]\);
    
    \sbe_ibuf[3]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \sbe_c[3]\, E => ADLIB_GND, YIN => 
        \sbe_ibuf[3]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[81]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[10]\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count_RNO[5]\ : 
        ARI1_CC
      generic map(INIT => x"47800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_i_0\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/un10_read_increment\, 
        D => \can_top_level_inst/rx_frame_count[5]\, FCI => 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_4\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNO_S[5]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG2652, P => 
        NET_CC_CONFIG2649, Y3 => NET_CC_CONFIG2650, Y3A => 
        NET_CC_CONFIG2651);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1[1]\ : 
        CFG4
      generic map(INIT => x"EAAA")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_182\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_1\ : 
        CFG4
      generic map(INIT => x"8421")

      port map(A => \can_top_level_inst/drv_bus[468]\, B => 
        \can_top_level_inst/drv_bus[467]\, C => 
        \can_top_level_inst/stat_bus[4]\, D => 
        \can_top_level_inst/stat_bus[3]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_1_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram42_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe42\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram42_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un41_txtb_pointer_meta_d[0]\ : 
        CFG4
      generic map(INIT => x"F040")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/load_ffmt_w_addr\, 
        B => \can_top_level_inst/tx_arbitrator_inst/buffer_md_w\, 
        C => \can_top_level_inst/tx_arbitrator_inst/N_94\, D => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, Y
         => 
        \can_top_level_inst/tx_arbitrator_inst/un41_txtb_pointer_meta_d_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_36[14]\ : 
        CFG3
      generic map(INIT => x"EA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_704_1\, 
        B => \can_top_level_inst/stat_bus[378]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_704\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[31]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2557\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2489\, Y
         => \can_top_level_inst/memory_registers_inst/N_2687\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[11]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[11]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[11]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[16]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m30_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[22]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[22]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_c5\ : 
        CFG4
      generic map(INIT => x"FFE4")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[4]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[4]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_c4_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_c5_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m36_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_crc_err\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/bit_err_arb\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_err\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_1250\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_crc_err_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[19]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[22]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2548\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2480\, Y
         => \can_top_level_inst/memory_registers_inst/N_2678\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_2_0\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[30]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_2_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_RNIL9NS[0]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\, C
         => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[0]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry_cy\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[0]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[0]\, 
        CC => NET_CC_CONFIG1095, P => NET_CC_CONFIG1092, Y3 => 
        NET_CC_CONFIG1093, Y3A => NET_CC_CONFIG1094);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_5_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_6_Z\, 
        CC => NET_CC_CONFIG2474, P => NET_CC_CONFIG2471, Y3 => 
        NET_CC_CONFIG2472, Y3A => NET_CC_CONFIG2473);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[16]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2542\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2474\, Y
         => \can_top_level_inst/memory_registers_inst/N_2672\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_41\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[155]\, B => 
        \can_top_level_inst/rec_ident[12]\, C => 
        \can_top_level_inst/drv_bus[188]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_42\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_3[6]\ : 
        CFG3
      generic map(INIT => x"31")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[120]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1357\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[15]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[14]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[16]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2542\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[9]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2254\, B
         => \can_top_level_inst/rx_buffer_inst/N_4816\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2287\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_1.CO1\ : 
        CFG4
      generic map(INIT => x"0A8E")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[1]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_7\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[25]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2551\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[1]\ : 
        CFG3
      generic map(INIT => x"E0")

      port map(A => \can_top_level_inst/txtb_hw_cmd.arbl\, B => 
        \can_top_level_inst/txtb_hw_cmd.err\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_5\ : 
        CFG4
      generic map(INIT => x"777F")

      port map(A => \can_top_level_inst/txtb_parity_check_valid\, 
        B => \can_top_level_inst/txtb_parity_error_valid_1[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_m1_e_1\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_m1_e_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_5_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_18[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_7[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram126_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram127_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y7_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.un4_ctrl_counted_byte_1\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un4_ctrl_counted_byte_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[11]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_0_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_0_FCNET1\, 
        CO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_8\, 
        CC => NET_CC_CONFIG2098, P => NET_CC_CONFIG2095, Y3 => 
        NET_CC_CONFIG2096, Y3A => NET_CC_CONFIG2097);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2\ : 
        CFG4
      generic map(INIT => x"1110")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[10]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_one\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_eight\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_5\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[413]\, B => 
        \can_top_level_inst/stat_bus[86]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_4\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_5\, 
        CC => NET_CC_CONFIG191, P => NET_CC_CONFIG188, Y3 => 
        NET_CC_CONFIG189, Y3A => NET_CC_CONFIG190);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[7]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_4_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_5_Z\, 
        CC => NET_CC_CONFIG2470, P => NET_CC_CONFIG2467, Y3 => 
        NET_CC_CONFIG2468, Y3A => NET_CC_CONFIG2469);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/N_4784\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1\ : 
        RGB
      port map(A => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_Y\, 
        EN => ADLIB_VCC, Y => \AFLSDF_INV_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_3_1_RNIMVGD1_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO325, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => 
        ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_GND, P(6) => 
        NET_CC_CONFIG326, P(7) => NET_CC_CONFIG330, P(8) => 
        NET_CC_CONFIG334, P(9) => NET_CC_CONFIG338, P(10) => 
        NET_CC_CONFIG342, P(11) => NET_CC_CONFIG346, Y3(0) => 
        ADLIB_VCC, Y3(1) => ADLIB_VCC, Y3(2) => ADLIB_VCC, Y3(3)
         => ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5) => ADLIB_VCC, 
        Y3(6) => NET_CC_CONFIG327, Y3(7) => NET_CC_CONFIG331, 
        Y3(8) => NET_CC_CONFIG335, Y3(9) => NET_CC_CONFIG339, 
        Y3(10) => NET_CC_CONFIG343, Y3(11) => NET_CC_CONFIG347, 
        Y3A(0) => ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => 
        ADLIB_VCC, Y3A(3) => ADLIB_VCC, Y3A(4) => ADLIB_VCC, 
        Y3A(5) => ADLIB_VCC, Y3A(6) => NET_CC_CONFIG328, Y3A(7)
         => NET_CC_CONFIG332, Y3A(8) => NET_CC_CONFIG336, Y3A(9)
         => NET_CC_CONFIG340, Y3A(10) => NET_CC_CONFIG344, 
        Y3A(11) => NET_CC_CONFIG348, CC(0) => nc349, CC(1) => 
        nc156, CC(2) => nc280, CC(3) => nc125, CC(4) => nc211, 
        CC(5) => nc73, CC(6) => NET_CC_CONFIG329, CC(7) => 
        NET_CC_CONFIG333, CC(8) => NET_CC_CONFIG337, CC(9) => 
        NET_CC_CONFIG341, CC(10) => NET_CC_CONFIG345, CC(11) => 
        NET_CC_CONFIG349);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[2]\, 
        D => \can_top_level_inst/prescaler_inst/tseg1_nbt[2]\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[11]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[10]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_1\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => \can_top_level_inst/txtb_index_muxed[1]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/N_3901\, 
        C => \can_top_level_inst/txtb_index_muxed[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_m1_e_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_0_y3[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_0_y1[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1156\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[9]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[9]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[9]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8_2[2]\ : 
        CFG4
      generic map(INIT => x"A820")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => \can_top_level_inst/rx_mem_free[2]\, D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_2_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c4_a0\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[3]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[2]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[1]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[0]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c4_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[15]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un10_tq_counter_d\ : 
        CFG4
      generic map(INIT => x"FF06")

      port map(A => \can_top_level_inst/drv_bus[15]\, B => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_a_4[7]\, 
        C => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE_5\, 
        D => \can_top_level_inst/prescaler_inst/bt_ctr_clear\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un10_tq_counter_d_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[10]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[30]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[13]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[13]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[13]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[1]\, 
        C => \can_top_level_inst/txtb_port_b_address[1]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[23]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4[4]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[7]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2533\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2465\, Y
         => \can_top_level_inst/memory_registers_inst/N_2663\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[4]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[24]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[329]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[3]\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[3]\, 
        C => \can_top_level_inst/txtb_parity_error_valid[7]\, D
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_168\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m25_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[188]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[3]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[0]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1277\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1271\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[3]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_shadowed[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/trv_delay[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_9[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => ADLIB_VCC, B => ADLIB_VCC, C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[4]\, 
        D => ADLIB_VCC, Y => OPEN);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_17[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram92_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram93_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_7[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[8]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_7_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8_FCNET1\, 
        CC => NET_CC_CONFIG2482, P => NET_CC_CONFIG2479, Y3 => 
        NET_CC_CONFIG2480, Y3A => NET_CC_CONFIG2481);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/bit_gen.0.reg_present_gen.reg_autoclear_gen.reg_value_r_1[0]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select\, 
        C => \data_in_c[0]\, Y => 
        \can_top_level_inst/memory_registers_inst/tx_command[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_14\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[401]\, B => 
        \can_top_level_inst/stat_bus[82]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_0\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_1\, 
        CC => NET_CC_CONFIG1172, P => NET_CC_CONFIG1169, Y3 => 
        NET_CC_CONFIG1170, Y3A => NET_CC_CONFIG1171);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_pos_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/err_detected\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[104]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[48]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[20]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[20]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[19]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[19]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram109__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_354\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe109\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g1_1\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => 
        \can_top_level_inst/can_core_inst/trigger_mux_inst_tx_trigger_q\, 
        B => \can_top_level_inst/stat_bus[184]\, C => 
        \can_top_level_inst/can_core_inst/crc_calc_from_rx_1_0\, 
        D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g1_0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g1_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[20]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram7_[0]\ : 
        SLE
      port map(D => can_tx_c, CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, 
        EN => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe7\, 
        ALn => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn
         => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram7_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_0[6]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[1]\, 
        B => \can_top_level_inst/stat_bus[68]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_146\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[28]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[28]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un10_sync_control_d\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1457\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_sync_control_d_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un10_sync_control_d_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[22]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[22]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[22]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[17]\ : 
        SLE
      port map(D => \timestamp_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_8[22]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/drv_bus[260]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_980\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_21_RNI8QEQ1\ : 
        CFG4
      generic map(INIT => x"9669")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_28_Z\, 
        Y => \can_top_level_inst/N_3902_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[10]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[18]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[50]\, 
        C => \can_top_level_inst/rec_ident[18]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2263\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_159\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_0\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => \can_top_level_inst/drv_bus[476]\, Y => 
        \can_top_level_inst/txtb_parity_error_valid_1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa13\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe13\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[12]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2538\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2470\, Y
         => \can_top_level_inst/memory_registers_inst/N_2668\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz[3]\ : 
        CFG4
      generic map(INIT => x"DC50")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_192\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[1]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_crc_match_c_0\ : 
        CFG4
      generic map(INIT => x"3332")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_stuff_count_ok_1_Z\, 
        B => \can_top_level_inst/drv_bus[510]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_stuff_count_ok_2_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_stuff_count_ok_NE_0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_crc_match_c_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[12]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2538\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[151]\, D => 
        \can_top_level_inst/drv_bus[279]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_y0_0[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_vals2_RNIMTNC\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_19_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[26]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[15]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1456_2\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_RNO[23]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2679\, D
         => \can_top_level_inst/memory_registers_inst/N_2712\, Y
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram102__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_341\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe102\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[0]\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_ety\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[6]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[18]\);
    
    \timestamp_ibuf[11]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(11), Y => 
        \timestamp_ibuf[11]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m24_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_58\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[168]\, B => 
        \can_top_level_inst/rec_ident[25]\, C => 
        \can_top_level_inst/drv_bus[201]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_26\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[303]\);
    
    \data_out_obuf[12]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(12), D => \data_out_obuf[12]/DOUT\, 
        E => \data_out_obuf[12]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/read_pointer_i[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_read_pointer[3]\);
    
    \data_in_ibuf[8]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(8), Y => \data_in_ibuf[8]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[116]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m36_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[12]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[209]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_25[11]\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[311]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1569\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[6]\ : 
        SLE
      port map(D => \timestamp_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_09_RNIBGIRC_0\ : 
        CFG4
      generic map(INIT => x"0110")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_19\, B => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_09\, C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_RNI791Q4_Z\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_RNIU99F7_Z\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/output_i3_mux\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m23_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[30]\ : 
        SLE
      port map(D => \data_in_c[30]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[54]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_9_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO, P(0) => ADLIB_VCC, 
        P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => ADLIB_VCC, 
        P(4) => ADLIB_VCC, P(5) => ADLIB_VCC, P(6) => ADLIB_VCC, 
        P(7) => ADLIB_GND, P(8) => NET_CC_CONFIG, P(9) => 
        NET_CC_CONFIG3, P(10) => NET_CC_CONFIG7, P(11) => 
        NET_CC_CONFIG11, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => ADLIB_VCC, Y3(3) => ADLIB_VCC, Y3(4) => 
        ADLIB_VCC, Y3(5) => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7)
         => ADLIB_VCC, Y3(8) => NET_CC_CONFIG0, Y3(9) => 
        NET_CC_CONFIG4, Y3(10) => NET_CC_CONFIG8, Y3(11) => 
        NET_CC_CONFIG12, Y3A(0) => ADLIB_VCC, Y3A(1) => ADLIB_VCC, 
        Y3A(2) => ADLIB_VCC, Y3A(3) => ADLIB_VCC, Y3A(4) => 
        ADLIB_VCC, Y3A(5) => ADLIB_VCC, Y3A(6) => ADLIB_VCC, 
        Y3A(7) => ADLIB_VCC, Y3A(8) => NET_CC_CONFIG1, Y3A(9) => 
        NET_CC_CONFIG5, Y3A(10) => NET_CC_CONFIG9, Y3A(11) => 
        NET_CC_CONFIG13, CC(0) => nc107, CC(1) => nc329, CC(2)
         => nc66, CC(3) => nc83, CC(4) => nc9, CC(5) => nc252, 
        CC(6) => nc171, CC(7) => nc54, CC(8) => NET_CC_CONFIG2, 
        CC(9) => NET_CC_CONFIG6, CC(10) => NET_CC_CONFIG10, 
        CC(11) => NET_CC_CONFIG14);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[5]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate_3_0_.m7_1_0\ : 
        CFG4
      generic map(INIT => x"4001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[5]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_27\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[2]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lt4_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_8_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[25]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_d_4\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[6]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[5]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[4]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[3]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_d_4_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[3]\ : 
        CFG3
      generic map(INIT => x"CE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst_priority_decoder_inst_un1_m2_0_a2_0_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_139\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[12]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[5]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[0]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/test_registers_out.tst_dest[5]\, D
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[5]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[32]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[29]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[29]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[29]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_d[2]\ : 
        CFG4
      generic map(INIT => x"060C")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[2]\, 
        C => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un17_same_bits_erase_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/CO0_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_d_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[30]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_28[19]\ : 
        CFG4
      generic map(INIT => x"AC00")

      port map(A => \can_top_level_inst/stat_bus[292]\, B => 
        \can_top_level_inst/drv_bus[100]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_886\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_shadowed[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/trv_delay[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[28]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/bit_gen.2.reg_present_gen.reg_autoclear_gen.reg_value_r_5_rep1[2]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select\, 
        C => \data_in_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/tx_command_rep1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[25]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[25]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[8]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[8]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[7]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s_Z[8]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG1087, P => 
        NET_CC_CONFIG1084, Y3 => NET_CC_CONFIG1085, Y3A => 
        NET_CC_CONFIG1086);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_21\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_21_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[26]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[18]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[18]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[18]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_1_0_0[2]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        B => \adress_c[5]\, C => \adress_c[3]\, D => 
        \adress_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_1_0_0_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un3_reinteg_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_2_2[8]\ : 
        CFG4
      generic map(INIT => x"3347")

      port map(A => \can_top_level_inst/int_ena[8]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/drv_bus[477]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_Z[8]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_2_1[8]\);
    
    \timestamp_ibuf[44]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(44), Y => 
        \timestamp_ibuf[44]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[405]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[11]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_1_1.CO2\ : 
        CFG3
      generic map(INIT => x"B2")

      port map(A => \can_top_level_inst/txtb_prorities_2[2]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_3\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_022\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[17]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[17]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[17]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_RNI11FC[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        Y => \can_top_level_inst/stat_bus[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1\ : 
        CFG4
      generic map(INIT => x"0305")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_FCO\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_crc_match_c_0_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_clear_match_flag\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[2]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[2]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[8]\);
    
    
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_16\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => \data_in_c[19]\, B => \data_in_c[18]\, C => 
        \data_in_c[17]\, D => \data_in_c[16]\, Y => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_16_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO[31]\ : 
        CFG4
      generic map(INIT => x"FF20")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_19\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_14\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_18_0_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[31]\);
    
    \swr_ibuf/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => swr_c, E => ADLIB_GND, YIN => \swr_ibuf/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[20]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[20]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIRV4S4_S[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[279]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_1_0\ : 
        ARI1_CC
      generic map(INIT => x"5CC66")

      port map(A => \can_top_level_inst/drv_bus[30]\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_fast\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[1]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_0\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_1\, 
        CC => NET_CC_CONFIG892, P => NET_CC_CONFIG889, Y3 => 
        NET_CC_CONFIG890, Y3A => NET_CC_CONFIG891);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[18]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[8]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2466\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm_0[1]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[412]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa15\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe15\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[27]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[27]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[27]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[13]\ : 
        SLE
      port map(D => \timestamp_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_5[30]\ : 
        CFG4
      generic map(INIT => x"8D88")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[29]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_11_0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_12_0_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[15]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m38\ : 
        CFG4
      generic map(INIT => x"2755")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[39]\, C => 
        \can_top_level_inst/stat_bus[227]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_39_0\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/int_stat_proc.un1_int_status_clear\ : 
        CFG4
      generic map(INIT => x"88F8")

      port map(A => \data_in_c[6]\, B => 
        \can_top_level_inst/N_138\, C => 
        \can_top_level_inst/err_detected\, D => 
        \can_top_level_inst/int_mask[6]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/un1_int_status_clear\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_dat_sel_ctr_i[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[257]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0[2]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst_priority_decoder_inst_un1_m2_0_a2_0_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c[0]\ : 
        CFG4
      generic map(INIT => x"CE02")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/stat_bus[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_data_len_can_2_0_i_0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[6]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_start_edge\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/drv_ena_reg_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/drv_ena_reg_2_Z\, 
        Y => \can_top_level_inst/prescaler_inst/start_edge\);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_ns_1_0_.m4\ : 
        CFG4
      generic map(INIT => x"8ABA")

      port map(A => \can_top_level_inst/can_core_inst/set_idle\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst/N_20\, 
        C => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/operation_control_inst/un2_go_to_off_i_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/operation_control_inst/N_21_mux\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un1_tq_edge_i\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        B => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        Y => \can_top_level_inst/prescaler_inst_tq_edge_nbt\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[8]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[8]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[8]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[25]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[25]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[25]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_data_access_signaller_comp/access_active_1[0]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \adress_c[2]\, B => \adress_c[3]\, C => 
        \adress_c[4]\, D => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/access_active_1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[26]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram67_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe67\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram67_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/drv_bus_off_reset_q\ : 
        SLE
      port map(D => \can_top_level_inst/drv_bus[429]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_rx_trigger\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/drv_bus_off_reset_q_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[13]\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q[4]\ : SLE
      port map(D => \can_top_level_inst/rx_buffer_inst/N_3928_i\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_8[27]\ : 
        CFG4
      generic map(INIT => x"FEEE")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[443]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_4_Z[27]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac4_Z\, 
        D => \can_top_level_inst/stat_bus[247]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_8_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram80__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_355\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe80\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[6]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[1]\ : 
        SLE
      port map(D => \timestamp_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_38_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1369\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1384\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_38_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_28_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/N_3899\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_RNO[31]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2687\, D
         => \can_top_level_inst/memory_registers_inst/N_2720\, Y
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[29]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[29]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_s_14_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[8]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[27]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_2[20]\ : 
        CFG4
      generic map(INIT => x"AFC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_901\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_900\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_Z[20]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_2_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[19]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram119_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe119\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram119_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram56__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_352\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe56\);
    
    \data_in_ibuf[0]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[0]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[0]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/command_reg_comp/bit_gen.10.reg_present_gen.reg_autoclear_gen.reg_value_r_20[10]\ : 
        CFG4
      generic map(INIT => x"7FFF")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/access_active_1[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        D => \data_in_c[10]\, Y => 
        \can_top_level_inst/memory_registers_inst/command[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[17]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram14__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_342\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe14\);
    
    \can_top_level_inst/tx_arbitrator_inst/validated_buffer_4\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[1]\, B
         => \can_top_level_inst/txtb_hw_cmd_index[0]\, C => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[5]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[250]\, D => 
        \can_top_level_inst/drv_bus[283]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_y0_0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m16_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[19]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_dest[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNIKSD3Q7[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_port_b_address[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/N_4796\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/N_4234\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_y0_0[11]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1569\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1572\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_0_y3[11]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIG63U9[13]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[286]\, C => 
        \can_top_level_inst/stat_bus[270]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_12\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIG63U9_S[13]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_13\, 
        CC => NET_CC_CONFIG2620, P => NET_CC_CONFIG2617, Y3 => 
        NET_CC_CONFIG2618, Y3A => NET_CC_CONFIG2619);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[20]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g1_0_0_a3\ : 
        CFG3
      generic map(INIT => x"14")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE_5\, 
        B => \can_top_level_inst/drv_bus[15]\, C => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_a_4[7]\, 
        Y => \can_top_level_inst/prescaler_inst/N_7\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[2]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[16]\);
    
    \data_out_obuf[10]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(10), D => \data_out_obuf[10]/DOUT\, 
        E => \data_out_obuf[10]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[9]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[21]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[292]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\);
    
    \can_top_level_inst/tx_arbitrator_inst/validated_buffer_6\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[1]\, B
         => \can_top_level_inst/txtb_hw_cmd_index[0]\, C => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_2_0\ : 
        ARI1_CC
      generic map(INIT => x"5CC66")

      port map(A => \can_top_level_inst/drv_bus[31]\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_fast\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[2]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_1\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_2\, 
        CC => NET_CC_CONFIG896, P => NET_CC_CONFIG893, Y3 => 
        NET_CC_CONFIG894, Y3A => NET_CC_CONFIG895);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI7L9M[0]_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO2564, CO => OPEN, P(0) => 
        NET_CC_CONFIG2613, P(1) => NET_CC_CONFIG2617, P(2) => 
        NET_CC_CONFIG2621, P(3) => NET_CC_CONFIG2625, P(4) => 
        ADLIB_VCC, P(5) => ADLIB_VCC, P(6) => ADLIB_VCC, P(7) => 
        ADLIB_VCC, P(8) => ADLIB_VCC, P(9) => ADLIB_VCC, P(10)
         => ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => 
        NET_CC_CONFIG2614, Y3(1) => NET_CC_CONFIG2618, Y3(2) => 
        NET_CC_CONFIG2622, Y3(3) => NET_CC_CONFIG2626, Y3(4) => 
        ADLIB_VCC, Y3(5) => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7)
         => ADLIB_VCC, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG2615, Y3A(1) => NET_CC_CONFIG2619, Y3A(2)
         => NET_CC_CONFIG2623, Y3A(3) => NET_CC_CONFIG2627, 
        Y3A(4) => ADLIB_VCC, Y3A(5) => ADLIB_VCC, Y3A(6) => 
        ADLIB_VCC, Y3A(7) => ADLIB_VCC, Y3A(8) => ADLIB_VCC, 
        Y3A(9) => ADLIB_VCC, Y3A(10) => ADLIB_VCC, Y3A(11) => 
        ADLIB_VCC, CC(0) => NET_CC_CONFIG2616, CC(1) => 
        NET_CC_CONFIG2620, CC(2) => NET_CC_CONFIG2624, CC(3) => 
        NET_CC_CONFIG2628, CC(4) => nc286, CC(5) => nc307, CC(6)
         => nc135, CC(7) => nc41, CC(8) => nc100, CC(9) => nc270, 
        CC(10) => nc339, CC(11) => nc52);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[11]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[11]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[11]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/current_state_fast[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/N_33_i_fast\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/is_tseg2_fast\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[12]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa_0_a2_1\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_189\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_186\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[5]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_881\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_909\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[49]\ : 
        SLE
      port map(D => \timestamp_c[49]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[49]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[23]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[23]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNIQAUE[18]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_962_i\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO[6]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_42\, 
        B => 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[24]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[24]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[14]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa14\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe14\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[11]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[11]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[3]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_24[2]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[207]\, C => 
        \can_top_level_inst/drv_bus[112]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1226\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un7_txtb_parity_error_valid_i_2\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_index_muxed[1]\, B
         => \can_top_level_inst/txtb_index_muxed[2]\, Y => 
        \can_top_level_inst/txtb_parity_error_valid_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[28]\);
    
    \timestamp_ibuf[48]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(48), Y => 
        \timestamp_ibuf[48]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[22]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[22]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[23]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[228]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[16]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[16]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[16]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[20]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q_RNIAJTQ1[0]\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_3_1_RNIMVGD1_Y\, 
        B => \can_top_level_inst/stat_bus[81]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => OPEN, 
        FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_0\, 
        CC => NET_CC_CONFIG87, P => NET_CC_CONFIG84, Y3 => 
        NET_CC_CONFIG85, Y3A => NET_CC_CONFIG86);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[25]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[14]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[14]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[14]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[7]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[7]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[26]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_RNO[9]\ : 
        ARI1_CC
      generic map(INIT => x"4EE00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, C => 
        \can_top_level_inst/prescaler_inst/is_tseg2\, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_8\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[9]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG1163, P => 
        NET_CC_CONFIG1160, Y3 => NET_CC_CONFIG1161, Y3A => 
        NET_CC_CONFIG1162);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[3]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_0_wmux\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[2]\, B
         => \can_top_level_inst/txtb_hw_cmd_index[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_129_i\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_129_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_0_y0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[25]\ : 
        CFG4
      generic map(INIT => x"3210")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1035\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_794_i\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[14]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[46]\, 
        C => \can_top_level_inst/rec_ident[14]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2259\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_0[6]\ : 
        CFG4
      generic map(INIT => x"20A0")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[4]\, D => 
        \can_top_level_inst/stat_bus[68]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/set_idle_i_2_sqmuxa_0_a4\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        Y => \can_top_level_inst/stat_bus[384]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[0]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m31_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_segm_counter_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[8]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[8]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[8]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[415]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[4]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_18[11]\ : 
        CFG4
      generic map(INIT => x"EAAA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1562_1\, 
        B => \can_top_level_inst/txtb_state_2[3]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1562\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[0]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[0]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[3]\ : 
        CFG4
      generic map(INIT => x"7FFF")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[0]\, B
         => \can_top_level_inst/txtb_hw_cmd.unlock\, C => 
        \can_top_level_inst/txtb_hw_cmd_index[1]\, D => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_136\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_crc_calc_from_rx\ : 
        CFG3
      generic map(INIT => x"BF")

      port map(A => \can_top_level_inst/stat_bus[370]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_N_865\, 
        C => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        Y => \can_top_level_inst/can_core_inst/crc_calc_from_rx\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[31]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[31]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[31]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2655\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[127]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[10]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[3]\, 
        D => \can_top_level_inst/drv_bus[49]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[28]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[24]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[167]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[2]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa11\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa11\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_0\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0_1\, 
        B => \adress_c[6]\, C => \adress_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_0_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a2_0_1\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a2_0_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_1[23]\ : 
        CFG4
      generic map(INIT => x"A989")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_1_1_Z[23]\, 
        D => \can_top_level_inst/drv_bus[12]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_1_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[25]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q_RNI5N1U4[6]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[87]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_5\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_6\, 
        CC => NET_CC_CONFIG353, P => NET_CC_CONFIG350, Y3 => 
        NET_CC_CONFIG351, Y3A => NET_CC_CONFIG352);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[2]\ : 
        CFG4
      generic map(INIT => x"35FF")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[9]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[0]\, C => 
        \can_top_level_inst/drv_bus[475]\, D => 
        \can_top_level_inst/txtb_sw_cmd.set_abt\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_141\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un26_err_frm_req_i\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un26_err_frm_req_i_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_1250\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/form_err\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_406\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[30]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[2]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[2]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4811\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_162\ : 
        CFG4
      generic map(INIT => x"4F00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_cnst[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1324\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_881\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_388\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[15]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2260\, B
         => \can_top_level_inst/rx_buffer_inst/N_4275\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2293\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_7\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_4_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_988\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_2\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_3_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_7_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_RNI4BHF1[2]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_179\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[15]\);
    
    \adress_ibuf[10]/U_IOPAD\ : IOPAD_IN
      port map(PAD => adress(10), Y => \adress_ibuf[10]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_RNO_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe46_0_a4\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_354\);
    
    \can_top_level_inst/tx_arbitrator_inst/int_txtb_index[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_hw_cmd_index[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[8]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_i_a2_0[3]\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451_1\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_RNO_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[7]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_0\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_i[0]\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_GND, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_0_Z\, 
        CC => NET_CC_CONFIG127, P => NET_CC_CONFIG124, Y3 => 
        NET_CC_CONFIG125, Y3A => NET_CC_CONFIG126);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[38]\ : 
        SLE
      port map(D => \timestamp_c[38]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[38]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\);
    
    \I_1/U0_RGB1_RGB6\ : RGB
      port map(A => \I_1/U0_Y\, EN => ADLIB_VCC, Y => 
        \AFLSDF_INV_3\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count[3]\ : SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI62SU1_S[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_frame_count[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count[2]\ : SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI305F1_S[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_frame_count[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_31[13]\ : 
        CFG4
      generic map(INIT => x"D951")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_31_1_Z[13]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_0_Z[13]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_Z[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_661\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d_RNO[2]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/can_core_inst/stuff_length[0]\, B => 
        \can_top_level_inst/tx_trigger\, C => 
        \can_top_level_inst/stat_bus[300]\, D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un9_stuff_lvl_reached_NE\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/CO0_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_36[16]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \timestamp_c[48]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_780_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_780\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0RIT8[11]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[199]\, D => 
        \can_top_level_inst/stat_bus[231]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_10\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0RIT8_S[11]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_11\, 
        CC => NET_CC_CONFIG800, P => NET_CC_CONFIG797, Y3 => 
        NET_CC_CONFIG798, Y3A => NET_CC_CONFIG799);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[375]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[23]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[55]\, 
        C => \can_top_level_inst/rec_ident[23]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2268\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe10\);
    
    \clk_sys_ibuf/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => OPEN, E => ADLIB_GND, YIN => ADLIB_VCC);
    
    \data_out_obuf[29]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(29), D => \data_out_obuf[29]/DOUT\, 
        E => \data_out_obuf[29]/EOUT\);
    
    \can_top_level_inst/tx_arbitrator_inst/un1_tran_frame_valid\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_valid_com_Z\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, C
         => 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer\, 
        Y => \can_top_level_inst/tran_frame_valid\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[20]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[9]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_tx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[72]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/set_idle_i_16_sqmuxa_0_a4\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => \can_top_level_inst/stat_bus[373]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/read_pointer_i[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_read_pointer[1]\);
    
    AFLSDF_INV_13 : INV_BA
      port map(A => \AFLSDF_INV_13\, Y => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[24]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[24]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[24]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c4_a0_RNICCRB3\ : 
        CFG4
      generic map(INIT => x"F9F6")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[4]\, 
        B => \can_top_level_inst/drv_bus[4]\, C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_RNIG6M52_Z[7]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c4_i\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_NE_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_9_RNO\ : 
        ARI1_CC
      generic map(INIT => x"45500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[9]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_8\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_9\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG288, P => 
        NET_CC_CONFIG285, Y3 => NET_CC_CONFIG286, Y3A => 
        NET_CC_CONFIG287);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un10_destuffed_d\ : 
        CFG2
      generic map(INIT => x"B")

      port map(A => \can_top_level_inst/rx_triggers[1]\, B => 
        \can_top_level_inst/can_core_inst/destuff_enable\, Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un10_destuffed_d_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[31]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[31]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[31]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_27\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[10]\, B => 
        \can_top_level_inst/drv_bus[153]\, C => 
        \can_top_level_inst/drv_bus[186]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_51\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[4]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[5]\, 
        CC => NET_CC_CONFIG710, P => NET_CC_CONFIG707, Y3 => 
        NET_CC_CONFIG708, Y3A => NET_CC_CONFIG709);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_clear\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_clear_i_1\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_962_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_clear_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[21]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[17]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_nxt\ : 
        CFG4
      generic map(INIT => x"66C3")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/crc_15[14]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_tx_shift_reg_inst_tx_data_nbs_iv\, 
        D => \can_top_level_inst/can_core_inst/crc_calc_from_rx\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_nxt_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[2]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIA5EO2[2]\ : 
        ARI1_CC
      generic map(INIT => x"5D827")

      port map(A => \timestamp_c[34]\, B => 
        \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[2]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[2]\, 
        FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_1\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_2\, 
        CC => NET_CC_CONFIG935, P => NET_CC_CONFIG932, Y3 => 
        NET_CC_CONFIG933, Y3A => NET_CC_CONFIG934);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0\ : 
        CFG4
      generic map(INIT => x"FCDC")

      port map(A => 
        \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_1_0\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g1_1_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/can_crc_inst_crc_17_21_trigger_1_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[26]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_no_data_field\ : 
        CFG4
      generic map(INIT => x"CE00")

      port map(A => \can_top_level_inst/tran_is_rtr\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_no_data_transmitter_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/frame_type\, 
        D => \can_top_level_inst/stat_bus[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_no_data_field_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[28]\ : 
        SLE
      port map(D => \data_in_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[204]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[22]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[22]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_11[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram112_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram113_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_4[0]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/int_mask_load\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[2]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/int_mask_load_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[416]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[10]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[10]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[30]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/int_ena_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/un1_int_ena_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_ena[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[3]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[3]\);
    
    \data_out_obuf[7]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[7]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[7]/DOUT\, EOUT => \data_out_obuf[7]/EOUT\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm_0[5]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[5]\, 
        B => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        C => \can_top_level_inst/prescaler_inst/h_sync_valid\, D
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[20]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIA6981[4]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_3\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIA6981_S[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_4\, 
        CC => NET_CC_CONFIG1661, P => NET_CC_CONFIG1658, Y3 => 
        NET_CC_CONFIG1659, Y3A => NET_CC_CONFIG1660);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/drv_ena_reg\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/drv_ena_reg_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[243]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask[8]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2[4]\, 
        B => \can_top_level_inst/drv_bus[477]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_0[8]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g1_0_0_a3_0\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => \can_top_level_inst/drv_bus[9]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g1_0_0_a3_0_4_Z\, 
        C => \can_top_level_inst/drv_bus[12]\, D => 
        \can_top_level_inst/drv_bus[10]\, Y => 
        \can_top_level_inst/prescaler_inst/N_8\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1_i_o2_RNIE72D\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_860_i\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_19_m[4]\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => \can_top_level_inst/drv_bus[510]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_854_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_19_m_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[30]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[30]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[30]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[235]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/tx_data_wbs\ : 
        CFG3
      generic map(INIT => x"FB")

      port map(A => 
        \can_top_level_inst/can_core_inst/bst_data_out\, B => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        C => \can_top_level_inst/drv_bus[470]\, Y => can_tx_c);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI2H4H1_S[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[221]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1[2]\ : 
        CFG4
      generic map(INIT => x"0010")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        D => \can_top_level_inst/N_4606\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[205]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/un2_res_core_d_n_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => res_n_out_c, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD
         => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst_global_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_808_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[27]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[27]\);
    
    \timestamp_ibuf[6]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(6), Y => \timestamp_ibuf[6]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_raw_0[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIDSV21_S[2]\, 
        B => \can_top_level_inst/drv_bus[377]\, C => 
        \can_top_level_inst/drv_bus[374]\, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_38\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[28]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_0\ : 
        CFG3
      generic map(INIT => x"4D")

      port map(A => \can_top_level_inst/txtb_prorities_3[2]\, B
         => \can_top_level_inst/txtb_prorities_2[2]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_0_1_1_Z\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_i1_mux_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_d_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm_0[8]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[8]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_16\ : 
        CFG3
      generic map(INIT => x"96")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_16_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_11\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[11]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_10_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_11_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_11_Z\, 
        CC => NET_CC_CONFIG1258, P => NET_CC_CONFIG1255, Y3 => 
        NET_CC_CONFIG1256, Y3A => NET_CC_CONFIG1257);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[16]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[16]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[24]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[24]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[24]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[312]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[21]\);
    
    
        \can_top_level_inst/can_core_inst/trigger_mux_inst/crc_trig_tx_wbs_reg/reg_q\ : 
        SLE
      port map(D => \can_top_level_inst/tx_trigger\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/trigger_mux_inst_tx_trigger_q\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[17]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[17]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[16]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_0\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[0]\, 
        B => \can_top_level_inst/drv_bus[51]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_3906\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[271]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/address_decoder_control_registers_comp/addr_dec_2_0_0[8]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \adress_c[6]\, B => \adress_c[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2_0_0[8]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un10_trv_meas_progress_d\ : 
        CFG4
      generic map(INIT => x"F4FF")

      port map(A => \can_top_level_inst/rx_data_wbs\, B => 
        \can_top_level_inst/bus_sampling_inst/rx_data_prev\, C
         => \can_top_level_inst/bus_sampling_inst/edge_tx_valid\, 
        D => \can_top_level_inst/stat_bus[13]\, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un10_trv_meas_progress_d_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[330]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[8]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_segm_counter_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[8]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm_0[5]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[5]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[28]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_5\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[5]\, B
         => \can_top_level_inst/can_core_inst/crc_17[4]\, C => 
        \can_top_level_inst/can_core_inst/crc_17[5]\, D => 
        \can_top_level_inst/store_data_word[4]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[1]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[2]\, 
        CC => NET_CC_CONFIG2535, P => NET_CC_CONFIG2532, Y3 => 
        NET_CC_CONFIG2533, Y3A => NET_CC_CONFIG2534);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count[0]\ : SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI0VMF_Y[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_frame_count[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[0]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[0]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[0]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[33]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[2]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[31]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/dff_data_out_reg/reg_q_RNO_0\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un6_same_bits_rst_trig_i\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/dff_data_out_reg/data_out_d_sn_N_4_mux\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[31]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[31]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[31]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[1]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[2]\, B
         => \can_top_level_inst/txtb_sw_cmd.set_abt_rep1\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_131\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[4]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_5\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[143]\, B => 
        \can_top_level_inst/rec_ident[0]\, C => 
        \can_top_level_inst/drv_bus[176]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_72\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_5[31]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac29_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac28_Z\, 
        C => \can_top_level_inst/drv_bus[60]\, D => 
        \can_top_level_inst/drv_bus[55]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_5_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[5]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_switch_to_ssp\ : 
        CFG4
      generic map(INIT => x"D000")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\, 
        B => \can_top_level_inst/drv_bus[374]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_switch_data\, 
        D => \can_top_level_inst/stat_bus[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_switch_to_ssp_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[13]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_RNID49GN5\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[6]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        D => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[6]\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_5\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_6\, 
        CC => NET_CC_CONFIG2131, P => NET_CC_CONFIG2128, Y3 => 
        NET_CC_CONFIG2129, Y3A => NET_CC_CONFIG2130);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m48_e\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/rx_buffer_inst_RAM_data_out[5]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_140\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_13[24]\ : 
        CFG4
      generic map(INIT => x"0AC0")

      port map(A => \can_top_level_inst/stat_bus[281]\, B => 
        \can_top_level_inst/drv_bus[295]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1061\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"46600")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[6]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_5\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_6_Z\, 
        CC => NET_CC_CONFIG912, P => NET_CC_CONFIG909, Y3 => 
        NET_CC_CONFIG910, Y3A => NET_CC_CONFIG911);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[25]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_2\ : 
        CFG4
      generic map(INIT => x"963C")

      port map(A => \can_top_level_inst/drv_bus[51]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[2]\, 
        C => \can_top_level_inst/drv_bus[53]\, D => 
        \can_top_level_inst/drv_bus[52]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_RNIT43C2[5]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[4]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[5]\, 
        CC => NET_CC_CONFIG2045, P => NET_CC_CONFIG2042, Y3 => 
        NET_CC_CONFIG2043, Y3A => NET_CC_CONFIG2044);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram28_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe28\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram28_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[1]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/int_stat_proc.un2_int_status_set\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/fcs_changed\, B => 
        \can_top_level_inst/int_mask[4]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/un2_int_status_set\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_2[2]\ : 
        CFG4
      generic map(INIT => x"5554")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/status_comb_2_Z[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/status_comb_3_Z[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/status_comb_4_Z[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1204\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce\ : 
        CFG4
      generic map(INIT => x"FFE0")

      port map(A => 
        \can_top_level_inst/can_core_inst/can_crc_inst/un2_crc_ena_15_i\, 
        B => 
        \can_top_level_inst/can_core_inst/can_crc_inst_crc_enable\, 
        C => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_15_trigger_Z\, 
        D => \can_top_level_inst/can_core_inst/load_init_vect\, Y
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0[16]\ : 
        CFG3
      generic map(INIT => x"70")

      port map(A => \adress_c[7]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \sbe_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[5]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_38_1_wmux_RNO_0[4]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_wmux_0_Y[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_2_Z[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1308\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_tx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[73]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[0]\ : 
        CFG4
      generic map(INIT => x"8F88")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_141\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        C => \can_top_level_inst/txtb_allow_bb[0]\, D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_18\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_18_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[380]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_0[3]\ : 
        CFG4
      generic map(INIT => x"CE0A")

      port map(A => \can_top_level_inst/drv_bus[511]\, B => 
        \can_top_level_inst/drv_bus[513]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_879_i\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_err_frm_req_3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_0_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[5]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[17]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[17]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[9]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2535\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2467\, Y
         => \can_top_level_inst/memory_registers_inst/N_2665\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_1[19]\ : 
        CFG4
      generic map(INIT => x"0B08")

      port map(A => \can_top_level_inst/tran_identifier[5]\, B
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[19]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_26[2]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/rx_write_pointer[2]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1228\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[20]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIAMOOF[22]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[210]\, D => 
        \can_top_level_inst/stat_bus[242]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_21\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIAMOOF_S[22]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_22\, 
        CC => NET_CC_CONFIG844, P => NET_CC_CONFIG841, Y3 => 
        NET_CC_CONFIG842, Y3A => NET_CC_CONFIG843);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[24]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[24]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[24]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram16__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_355\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe16\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[30]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[0]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[0]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[5]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_txtb_hw_cmd_d.err_0_sqmuxa_2_0\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_717_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_arbitration_lost_condition_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa_2_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa13_1\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \adress_c[2]\, B => \adress_c[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_1[5]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \can_top_level_inst/txtb_hw_cmd.unlock\, B
         => \can_top_level_inst/txtb_hw_cmd.failed\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_1_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m52_1_0\ : 
        CFG4
      generic map(INIT => x"0213")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/int_ena[5]\, D => 
        \can_top_level_inst/drv_bus[472]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_53_0_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/drv_ena_reg_2\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/drv_ena_reg_Z\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/drv_ena_reg_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[17]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[9]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[9]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[14]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[26]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2484\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[20]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[20]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[20]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[7]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0_RNIH5T2S2[2]\ : 
        ARI1_CC
      generic map(INIT => x"58778")

      port map(A => \can_top_level_inst/drv_bus[31]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_49\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_1\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_2\, 
        CC => NET_CC_CONFIG1135, P => NET_CC_CONFIG1132, Y3 => 
        NET_CC_CONFIG1133, Y3A => NET_CC_CONFIG1134);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_38_1_0_wmux_RNO[9]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_2_0_Z[9]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_2_Z[9]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1498\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[24]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_10_N_4L5\ : 
        CFG4
      generic map(INIT => x"75FD")

      port map(A => \can_top_level_inst/prescaler_inst/g0_0_7\, B
         => \can_top_level_inst/prescaler_inst_un2_tq_edge\, C
         => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_10_N_3L3_Z\, 
        D => \can_top_level_inst/prescaler_inst_tq_edge_nbt\, Y
         => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[2]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIOJMH4[7]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[7]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_6\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_7\, 
        CC => NET_CC_CONFIG1305, P => NET_CC_CONFIG1302, Y3 => 
        NET_CC_CONFIG1303, Y3A => NET_CC_CONFIG1304);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_rx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[85]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[29]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[29]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2[23]\ : 
        CFG4
      generic map(INIT => x"B30C")

      port map(A => \can_top_level_inst/drv_bus[4]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_1[23]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_Z[23]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[8]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2632\, Y
         => \can_top_level_inst/memory_registers_inst/N_2697\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[11]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[11]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[11]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[13]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/current_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/N_33_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/is_tseg2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[10]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[28]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[10]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[9]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_RNIIVU81[3]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_2\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_3\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_3\, 
        CC => NET_CC_CONFIG2191, P => NET_CC_CONFIG2188, Y3 => 
        NET_CC_CONFIG2189, Y3A => NET_CC_CONFIG2190);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[3]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[2]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_2_y0[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1304\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1305\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[424]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[26]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[14]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[27]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_28\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_23_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_16_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_28_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m22_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[7]\ : 
        CFG3
      generic map(INIT => x"0E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_950\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_936_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[30]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_830_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_20[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_8[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y3_0[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y1_0[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y21[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram50__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_343\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe50\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_q.arbl\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/arbl_0\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/txtb_hw_cmd.arbl\);
    
    \res_n_out_obuf/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => res_n_out_c, E => ADLIB_VCC, DOUT => 
        \res_n_out_obuf/DOUT\, EOUT => \res_n_out_obuf/EOUT\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[47]\ : 
        SLE
      port map(D => \timestamp_c[47]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[47]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram58_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe58\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram58_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[16]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram114_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe114\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram114_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram29_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe29\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram29_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[30]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_data_nbs_prev\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_data_nbs_prev_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[26]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_1_2_Z[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_1_1_Z[1]\, 
        D => \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_RNIT1PT[2]\ : 
        CFG3
      generic map(INIT => x"DF")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un9_stuff_lvl_reached_NE\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_34_2[10]\ : 
        CFG4
      generic map(INIT => x"C480")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/stat_bus[374]\, D => 
        \can_top_level_inst/stat_bus[230]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1540_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIKHKCM5[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[54]\, B => 
        \can_top_level_inst/tran_word[22]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_21\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_22\, 
        CC => NET_CC_CONFIG1015, P => NET_CC_CONFIG1012, Y3 => 
        NET_CC_CONFIG1013, Y3A => NET_CC_CONFIG1014);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[21]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[21]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[21]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_RNI3HIK[1]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_0\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_1\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_1\, 
        CC => NET_CC_CONFIG256, P => NET_CC_CONFIG253, Y3 => 
        NET_CC_CONFIG254, Y3A => NET_CC_CONFIG255);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un1_tq_edge_i_RNI2TGIB\ : 
        CFG4
      generic map(INIT => x"AAAC")

      port map(A => 
        \can_top_level_inst/prescaler_inst_tq_edge_dbt\, B => 
        \can_top_level_inst/prescaler_inst_tq_edge_nbt\, C => 
        \can_top_level_inst/stat_bus[10]\, D => 
        \can_top_level_inst/stat_bus[11]\, Y => 
        \can_top_level_inst/tq_edge\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/un15_txtb_available_i\ : 
        CFG3
      generic map(INIT => x"D5")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => \can_top_level_inst/txtb_sw_cmd.set_abt\, C => 
        \can_top_level_inst/txtb_sw_cmd_index[5]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_135\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram70_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe70\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram70_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[30]\ : 
        CFG4
      generic map(INIT => x"5040")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/data_selector[0]\, B
         => \can_top_level_inst/rx_buffer_inst/N_2308_1\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_10\, 
        D => \can_top_level_inst/rx_buffer_inst/N_2308_2\, Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[30]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_1\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_shorten_tseg1_after_tseg2_i\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[7]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[7]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[7]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIJ2PC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[19]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_7\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[407]\, B => 
        \can_top_level_inst/stat_bus[88]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_6_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_7_Z\, 
        CC => NET_CC_CONFIG1557, P => NET_CC_CONFIG1554, Y3 => 
        NET_CC_CONFIG1555, Y3A => NET_CC_CONFIG1556);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_9[27]\ : 
        CFG4
      generic map(INIT => x"FFF8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac23_Z\, 
        B => \can_top_level_inst/drv_bus[108]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_6_Z[27]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[667]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_9_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[25]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[25]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_o2[3]\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_926\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[21]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[21]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[21]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[20]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[20]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[20]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[21]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa19\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe19\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[26]\);
    
    
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_21\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => \data_in_c[7]\, B => \data_in_c[6]\, C => 
        \data_in_c[5]\, D => \data_in_c[4]\, Y => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_21_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[9]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count_RNIA5JE2[4]\ : 
        ARI1_CC
      generic map(INIT => x"599CC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un10_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, C
         => \can_top_level_inst/rx_frame_count[4]\, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_3\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNIA5JE2_S[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_4\, 
        CC => NET_CC_CONFIG2648, P => NET_CC_CONFIG2645, Y3 => 
        NET_CC_CONFIG2646, Y3A => NET_CC_CONFIG2647);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_1\ : 
        CFG4
      generic map(INIT => x"FEEE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/drv_pex_li_m_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_sqmuxa_m\, 
        C => \can_top_level_inst/can_core_inst/bds_data_out\, D
         => \can_top_level_inst/stat_bus[384]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_1_Z\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNITPEU[2]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_1\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNITPEU_S[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_2\, 
        CC => NET_CC_CONFIG1653, P => NET_CC_CONFIG1650, Y3 => 
        NET_CC_CONFIG1651, Y3A => NET_CC_CONFIG1652);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[12]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[12]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[5]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[8]\ : 
        CFG4
      generic map(INIT => x"ACAA")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        B => \can_top_level_inst/rx_buffer_inst/N_2286\, C => 
        \can_top_level_inst/rx_buffer_inst/data_selector[0]\, D
         => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_10\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1_0[1]\ : 
        CFG4
      generic map(INIT => x"F083")

      port map(A => \can_top_level_inst/rx_write_pointer[1]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1_0_1_Z[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1_0_Z[1]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_5_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_6_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_6_Z\, 
        CC => NET_CC_CONFIG1238, P => NET_CC_CONFIG1235, Y3 => 
        NET_CC_CONFIG1236, Y3A => NET_CC_CONFIG1237);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[22]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2480\);
    
    \adress_ibuf[3]/U_IOPAD\ : IOPAD_IN
      port map(PAD => adress(3), Y => \adress_ibuf[3]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[23]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_15\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[4]\, B => 
        \can_top_level_inst/drv_bus[209]\, C => 
        \can_top_level_inst/drv_bus[242]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_61\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[1]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[2]\, 
        CC => NET_CC_CONFIG471, P => NET_CC_CONFIG468, Y3 => 
        NET_CC_CONFIG469, Y3A => NET_CC_CONFIG470);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[16]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[16]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[28]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[28]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[27]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[27]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[27]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/txtb_ptr[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[2]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram66_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe66\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram66_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/int_stat_proc.un1_int_status_clear\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_stat_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => \sbe_c[0]\, C => \data_in_c[1]\, D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/un2_int_status_set\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/un1_int_status_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[3]\, 
        B => \can_top_level_inst/drv_bus[54]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_2_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_3_Z\, 
        CC => NET_CC_CONFIG2390, P => NET_CC_CONFIG2387, Y3 => 
        NET_CC_CONFIG2388, Y3A => NET_CC_CONFIG2389);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[19]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_26\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[297]\, B => 
        \can_top_level_inst/rec_ident[26]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_25\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_26\, 
        CC => NET_CC_CONFIG1937, P => NET_CC_CONFIG1934, Y3 => 
        NET_CC_CONFIG1935, Y3A => NET_CC_CONFIG1936);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[7]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[3]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[1]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ide_is_arbitration\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        B => \can_top_level_inst/stat_bus[66]\, C => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ide_is_arbitration_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_RNILVNT1[5]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_4\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_5\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_5\, 
        CC => NET_CC_CONFIG272, P => NET_CC_CONFIG269, Y3 => 
        NET_CC_CONFIG270, Y3A => NET_CC_CONFIG271);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa16\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe16\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram59_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe59\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram59_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_26\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[26]\, B => 
        \can_top_level_inst/drv_bus[326]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_25\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_26\, 
        CC => NET_CC_CONFIG633, P => NET_CC_CONFIG630, Y3 => 
        NET_CC_CONFIG631, Y3A => NET_CC_CONFIG632);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/pexs_set_i_0\ : 
        CFG3
      generic map(INIT => x"F2")

      port map(A => \can_top_level_inst/drv_bus[460]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/pexs_set_i_0_Z\);
    
    \data_in_ibuf[25]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(25), Y => \data_in_ibuf[25]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o2_0[24]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[14]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1050\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m18_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_address_i_i_m2[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_186\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram48__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_343\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe48\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIBBIS\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIUDL8I_S[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[246]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[29]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1_i_o2\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[5]\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        B => \can_top_level_inst/txtb_parity_error_valid[3]\, C
         => \can_top_level_inst/txtb_hw_cmd.failed\, D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_139\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_173\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1547\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1550\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_parity_check_valid_0_RNO\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_m1_e_2\, B
         => \can_top_level_inst/tx_arbitrator_inst/N_104\, C => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[2]\, D
         => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        Y => \can_top_level_inst/tx_arbitrator_inst/txtb_N_3_mux\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[19]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[19]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[19]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[44]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_mask_set_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        D => \adress_c[4]\, Y => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m31_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_22\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[212]\, B => 
        \can_top_level_inst/rec_ident[7]\, C => 
        \can_top_level_inst/drv_bus[245]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_56\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_21\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[6]\, B => 
        \can_top_level_inst/drv_bus[149]\, C => 
        \can_top_level_inst/drv_bus[182]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_56\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[2]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[3]\, 
        CC => NET_CC_CONFIG702, P => NET_CC_CONFIG699, Y3 => 
        NET_CC_CONFIG700, Y3A => NET_CC_CONFIG701);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_a2[4]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_cnst[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[43]\ : 
        SLE
      port map(D => \timestamp_c[43]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[43]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[1]\, 
        C => \can_top_level_inst/txtb_port_b_address[1]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[3]\);
    
    \timestamp_ibuf[42]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(42), Y => 
        \timestamp_ibuf[42]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_2[6]\ : 
        CFG4
      generic map(INIT => x"D05A")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \can_top_level_inst/stat_bus[263]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1_Z[6]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_2_1[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_2_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[1]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt\, B
         => \can_top_level_inst/txtb_sw_cmd_index[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_131\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram82__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_355\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe82\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[29]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[29]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[29]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_0_iv[0]\ : 
        CFG4
      generic map(INIT => x"22F2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_switch_data\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_switch_to_ssp_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[0]\, 
        D => \can_top_level_inst/br_shifted\, Y => 
        \can_top_level_inst/stat_bus[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[12]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[27]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[27]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_data_nbs_iv\ : 
        CFG4
      generic map(INIT => x"BFB8")

      port map(A => \can_top_level_inst/stat_bus[109]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_dominant\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_output_i_m_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_tx_shift_reg_inst_tx_data_nbs_iv\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[25]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[25]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIASAP[8]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[8]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[8]\, 
        Y => \can_top_level_inst/tran_word[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram71_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe71\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram71_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[476]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_1[31]\ : 
        CFG4
      generic map(INIT => x"F8F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_12\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_18_0_0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_19\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_18_0_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[120]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[272]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI2H4H1_S[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[189]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        B => srd_c, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un3_reinteg_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_mem_free[4]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_3_Z\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_4_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_4_Z\, 
        CC => NET_CC_CONFIG2434, P => NET_CC_CONFIG2431, Y3 => 
        NET_CC_CONFIG2432, Y3A => NET_CC_CONFIG2433);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[5]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[5]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[5]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe8\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[28]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[144]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_frame_type_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_860_i\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[156]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[11]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_2_Z\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_3_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_3_Z\, 
        CC => NET_CC_CONFIG2231, P => NET_CC_CONFIG2228, Y3 => 
        NET_CC_CONFIG2229, Y3A => NET_CC_CONFIG2230);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => 
        \can_top_level_inst/prescaler_inst_tq_edge_dbt\, B => 
        \can_top_level_inst/stat_bus[10]\, C => 
        \can_top_level_inst/stat_bus[11]\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lto3\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[3]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lt4\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/un2_index_satlto4\ : 
        CFG4
      generic map(INIT => x"FEEE")

      port map(A => \adress_c[5]\, B => \adress_c[6]\, C => 
        \adress_c[4]\, D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q_RNO[1]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_15[0]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[17]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[23]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[23]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[23]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4825\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI3KBC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0_RNI2S6M94[4]\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[4]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_53\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_3\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_4\, 
        CC => NET_CC_CONFIG1429, P => NET_CC_CONFIG1426, Y3 => 
        NET_CC_CONFIG1427, Y3A => NET_CC_CONFIG1428);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_m_3_1_0_wmux\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_i\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_i\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner0\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner2\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_m_3_1_0_y0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_1[1]\ : 
        CFG4
      generic map(INIT => x"CECC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_933\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_0_Z[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        D => \can_top_level_inst/is_bus_off\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[10]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[10]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[14]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un12_crc_use_17\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[1]\, 
        B => \can_top_level_inst/drv_bus[57]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_0_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_1_Z\, 
        CC => NET_CC_CONFIG2454, P => NET_CC_CONFIG2451, Y3 => 
        NET_CC_CONFIG2452, Y3A => NET_CC_CONFIG2453);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[22]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[22]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_16\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[287]\, B => 
        \can_top_level_inst/rec_ident[16]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_15\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_16\, 
        CC => NET_CC_CONFIG1897, P => NET_CC_CONFIG1894, Y3 => 
        NET_CC_CONFIG1895, Y3A => NET_CC_CONFIG1896);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_6_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[6]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/load_ts_lw_addr_i_RNI5LQHBE\ : 
        CFG4
      generic map(INIT => x"DCFF")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i_1_0\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i_1\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/un1_m3_0_a3_0\, D
         => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/un1_N_7_mux_i_0_1\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/un4_tst_ena\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => \can_top_level_inst/reg_value_r_rep[17]\, B
         => \can_top_level_inst/test_registers_out.tst_dest[19]\, 
        C => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena_2\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[5]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_meas_progress_q_Z\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_meas_progress_del_Z\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[18]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[11]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[8]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1581\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1575\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_type_q_RNO[2]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_err\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_1250\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un24_err_type_d_1_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un6_form_err_i_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un24_err_type_d_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/alc_Z[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_763_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/stat_bus[6]\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[295]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[14]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[22]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[244]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_29\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_20_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_19_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_18_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_17_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_29_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[10]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe20\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_0[4]\ : 
        CFG2
      generic map(INIT => x"B")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_136\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[1]\);
    
    
        \can_top_level_inst/int_manager_inst/dff_int_output_reg/reg_q_RNO\ : 
        CFG3
      generic map(INIT => x"7F")

      port map(A => 
        \can_top_level_inst/int_manager_inst/un1_int_i_4_Z\, B
         => \can_top_level_inst/int_manager_inst/un1_int_i_9_Z\, 
        C => \can_top_level_inst/int_manager_inst/un1_int_i_5_Z\, 
        Y => 
        \can_top_level_inst/int_manager_inst/dff_int_output_reg/un1_int_i_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_26[6]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => \can_top_level_inst/int_mask[6]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => \can_top_level_inst/drv_bus[174]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1380\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[28]\ : 
        SLE
      port map(D => \data_in_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[266]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_23\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_23_Z\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[62]\ : 
        SLE
      port map(D => \timestamp_c[62]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[62]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[17]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1458_2\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_11\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[223]\, B => 
        \can_top_level_inst/rec_ident[18]\, C => 
        \can_top_level_inst/drv_bus[256]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_67\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[30]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_RNIOH58E\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/un8_exit_segm_regular_tseg1_0\, 
        C => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g2_0_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_RNIOH58E_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60[6]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_3[6]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[29]\, CLK
         => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un25_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[7]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2631\, Y
         => \can_top_level_inst/memory_registers_inst/N_2696\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[7]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[6]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s_Z[7]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG1481, P => 
        NET_CC_CONFIG1478, Y3 => NET_CC_CONFIG1479, Y3A => 
        NET_CC_CONFIG1480);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNO[7]\ : 
        CFG4
      generic map(INIT => x"2023")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_149\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_151\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_124_i\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/pexs_set_i_o4\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/drv_bus[511]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1013\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[5]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[5]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNIERIU_Y[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_tx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[71]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c5\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c4_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[4]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c5_Z\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/int_stat_proc.un2_int_status_set\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => \can_top_level_inst/int_mask[0]\, B => 
        \can_top_level_inst/rec_valid\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/un2_int_status_set\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[15]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry[2]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[1]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[2]\, 
        CC => NET_CC_CONFIG1063, P => NET_CC_CONFIG1060, Y3 => 
        NET_CC_CONFIG1061, Y3A => NET_CC_CONFIG1062);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1[0]\ : 
        CFG4
      generic map(INIT => x"C989")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1_1_Z[0]\, 
        D => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[15]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[15]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[15]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2639\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_4\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_3_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_4_Z\, 
        CC => NET_CC_CONFIG143, P => NET_CC_CONFIG140, Y3 => 
        NET_CC_CONFIG141, Y3A => NET_CC_CONFIG142);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\);
    
    \can_top_level_inst/int_manager_inst/un1_int_i_3\ : CFG4
      generic map(INIT => x"135F")

      port map(A => \can_top_level_inst/int_vector[3]\, B => 
        \can_top_level_inst/int_vector[2]\, C => 
        \can_top_level_inst/int_ena[3]\, D => 
        \can_top_level_inst/int_ena[2]\, Y => 
        \can_top_level_inst/int_manager_inst/un1_int_i_3_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[24]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[24]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[0]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2526\);
    
    \data_out_obuf[2]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(2), D => \data_out_obuf[2]/DOUT\, 
        E => \data_out_obuf[2]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[6]\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        C => \can_top_level_inst/txtb_parity_error_valid[1]\, D
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_195\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_174\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.1.l2_prio_0_1.CO1\ : 
        CFG4
      generic map(INIT => x"7150")

      port map(A => \can_top_level_inst/txtb_prorities_4[1]\, B
         => \can_top_level_inst/txtb_prorities_4[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_5\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_87\ : 
        ARI1_CC
      generic map(INIT => x"6D700")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/drv_bus[233]\, C => 
        \can_top_level_inst/drv_bus[266]\, D => 
        \can_top_level_inst/rec_ident[28]\, FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[13]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_87_FCNET1\, 
        CC => NET_CC_CONFIG519, P => NET_CC_CONFIG516, Y3 => 
        NET_CC_CONFIG517, Y3A => NET_CC_CONFIG518);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram2__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_346\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_stuff_count_ok_2\ : 
        CFG3
      generic map(INIT => x"96")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_stuff_count[2]\, 
        B => \can_top_level_inst/stat_bus[305]\, C => 
        \can_top_level_inst/stat_bus[304]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_stuff_count_ok_2_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[30]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[30]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[17]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[17]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/txtb_prorities_3[2]\, B
         => \can_top_level_inst/txtb_prorities_2[2]\, C => 
        \can_top_level_inst/drv_bus_475_rep2\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[32]\, 
        C => \can_top_level_inst/rec_ident[0]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2245\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m22_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[15]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[7]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[7]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[7]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m22_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2[0]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[5]\, B
         => \can_top_level_inst/txtb_sw_cmd.set_ety\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_151\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[10]\);
    
    \timestamp_ibuf[13]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(13), Y => 
        \timestamp_ibuf[13]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[31]\ : 
        SLE
      port map(D => \timestamp_c[31]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[13]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[2]\ : 
        SLE
      port map(D => \timestamp_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_1[30]\ : 
        CFG4
      generic map(INIT => x"FEFC")

      port map(A => \timestamp_c[62]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[1214]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac2_Z\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_1_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => \can_top_level_inst/drv_bus[39]\, B => 
        \can_top_level_inst/drv_bus[45]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_3_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_4_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_4_Z\, 
        CC => NET_CC_CONFIG1754, P => NET_CC_CONFIG1751, Y3 => 
        NET_CC_CONFIG1752, Y3A => NET_CC_CONFIG1753);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2[1]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_1_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2_Z[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm_0[8]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s_Z[8]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_0[2]\ : 
        CFG4
      generic map(INIT => x"4C00")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[4]\, D => 
        \can_top_level_inst/stat_bus[68]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[12]\);
    
    
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/g0_0_3\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => \can_top_level_inst/rx_data_wbs\, B => 
        \can_top_level_inst/prescaler_inst/g1\, C => 
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/g0_0_0_0_Z\, 
        D => \can_top_level_inst/prescaler_inst/g0_0_2\, Y => 
        \can_top_level_inst/prescaler_inst/g0_0_3\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_15\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[408]\, B => 
        \can_top_level_inst/stat_bus[81]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_0\, 
        CC => NET_CC_CONFIG171, P => NET_CC_CONFIG168, Y3 => 
        NET_CC_CONFIG169, Y3A => NET_CC_CONFIG170);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_2[13]\ : 
        CFG4
      generic map(INIT => x"C840")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1454_2\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[23]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[23]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_1_0_1[19]\ : 
        CFG4
      generic map(INIT => x"3F0A")

      port map(A => \can_top_level_inst/drv_bus[8]\, B => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[19]\, C
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_1_0_1_Z[19]\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_ident_type_com\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_ident_type_dbl_buf_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[66]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[16]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[16]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/test_registers_out.tst_dest[16]\, D
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[24]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[8]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2253\, B
         => \can_top_level_inst/rx_buffer_inst/N_4815\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2286\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO\ : 
        ARI1_CC
      generic map(INIT => x"69900")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/crc_15[14]\, C => 
        \can_top_level_inst/store_data_word[14]\, D => ADLIB_GND, 
        FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[6]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_FCNET1\, 
        CC => NET_CC_CONFIG78, P => NET_CC_CONFIG75, Y3 => 
        NET_CC_CONFIG76, Y3A => NET_CC_CONFIG77);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[28]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[18]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[256]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[24]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[24]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[24]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram10__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_348\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe10\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[11]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[10]\ : 
        SLE
      port map(D => \timestamp_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[10]\);
    
    \I_1/U0_RGB1_RGB2\ : RGB
      port map(A => \I_1/U0_Y\, EN => ADLIB_VCC, Y => 
        \AFLSDF_INV_4\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[5]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[5]\);
    
    \data_in_ibuf[7]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(7), Y => \data_in_ibuf[7]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/int_mask_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/int_mask_load_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_mask[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram5__RNO[0]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[2]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[1]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[24]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[14]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2703\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2670\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_mem_free[2]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_1_Z\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_2_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_2_Z\, 
        CC => NET_CC_CONFIG2426, P => NET_CC_CONFIG2423, Y3 => 
        NET_CC_CONFIG2424, Y3A => NET_CC_CONFIG2425);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNI1SKV[6]\ : 
        CFG3
      generic map(INIT => x"4C")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_129_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[28]\ : 
        SLE
      port map(D => \data_in_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[328]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/un3_test_registers_cs_Z\, 
        B => srd_c, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[9]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[27]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[27]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[9]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[24]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_21[17]\ : 
        CFG4
      generic map(INIT => x"0AC0")

      port map(A => \can_top_level_inst/stat_bus[274]\, B => 
        \can_top_level_inst/drv_bus[288]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_803\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_dbt_Z[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_3_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[6]\ : 
        CFG4
      generic map(INIT => x"0E00")

      port map(A => \can_top_level_inst/txtb_hw_cmd.arbl\, B => 
        \can_top_level_inst/txtb_hw_cmd.err\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_stuff_count_Z[2]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[1]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_stuff_count\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_stuff_count[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[13]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[13]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNI4L2H4[8]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[8]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[7]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[8]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[8]\, 
        CC => NET_CC_CONFIG1710, P => NET_CC_CONFIG1707, Y3 => 
        NET_CC_CONFIG1708, Y3A => NET_CC_CONFIG1709);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[23]\);
    
    \timestamp_ibuf[26]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[26]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[26]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI817P2_S[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[260]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram88_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe88\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram88_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_3[18]\ : 
        CFG4
      generic map(INIT => x"C0AF")

      port map(A => \can_top_level_inst/drv_bus[132]\, B => 
        \can_top_level_inst/rx_mem_free[2]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_823\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_22\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_22_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1_0[0]\ : 
        CFG4
      generic map(INIT => x"D9C8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1135\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1134\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1_0_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[6]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[6]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_7_2_Z[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_7_2_0_Z[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_0_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[2]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[2]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_0_y1[9]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_5_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[5]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[9]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[9]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO2099, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_GND, P(2) => NET_CC_CONFIG2100, 
        P(3) => NET_CC_CONFIG2104, P(4) => NET_CC_CONFIG2108, 
        P(5) => NET_CC_CONFIG2112, P(6) => NET_CC_CONFIG2116, 
        P(7) => NET_CC_CONFIG2120, P(8) => NET_CC_CONFIG2124, 
        P(9) => NET_CC_CONFIG2128, P(10) => NET_CC_CONFIG2132, 
        P(11) => NET_CC_CONFIG2136, Y3(0) => ADLIB_VCC, Y3(1) => 
        ADLIB_GND, Y3(2) => NET_CC_CONFIG2101, Y3(3) => 
        NET_CC_CONFIG2105, Y3(4) => NET_CC_CONFIG2109, Y3(5) => 
        NET_CC_CONFIG2113, Y3(6) => NET_CC_CONFIG2117, Y3(7) => 
        NET_CC_CONFIG2121, Y3(8) => NET_CC_CONFIG2125, Y3(9) => 
        NET_CC_CONFIG2129, Y3(10) => NET_CC_CONFIG2133, Y3(11)
         => NET_CC_CONFIG2137, Y3A(0) => ADLIB_VCC, Y3A(1) => 
        ADLIB_GND, Y3A(2) => NET_CC_CONFIG2102, Y3A(3) => 
        NET_CC_CONFIG2106, Y3A(4) => NET_CC_CONFIG2110, Y3A(5)
         => NET_CC_CONFIG2114, Y3A(6) => NET_CC_CONFIG2118, 
        Y3A(7) => NET_CC_CONFIG2122, Y3A(8) => NET_CC_CONFIG2126, 
        Y3A(9) => NET_CC_CONFIG2130, Y3A(10) => NET_CC_CONFIG2134, 
        Y3A(11) => NET_CC_CONFIG2138, CC(0) => nc251, CC(1) => 
        nc186, CC(2) => NET_CC_CONFIG2103, CC(3) => 
        NET_CC_CONFIG2107, CC(4) => NET_CC_CONFIG2111, CC(5) => 
        NET_CC_CONFIG2115, CC(6) => NET_CC_CONFIG2119, CC(7) => 
        NET_CC_CONFIG2123, CC(8) => NET_CC_CONFIG2127, CC(9) => 
        NET_CC_CONFIG2131, CC(10) => NET_CC_CONFIG2135, CC(11)
         => NET_CC_CONFIG2139);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_RNICDPSA[7]\ : 
        CFG4
      generic map(INIT => x"575D")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un2_tq_counter_allow\, 
        B => \can_top_level_inst/drv_bus[15]\, C => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE_5\, 
        D => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_a_4[7]\, 
        Y => \can_top_level_inst/prescaler_inst_tq_edge_dbt\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[18]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[18]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[18]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o2[11]\ : 
        CFG3
      generic map(INIT => x"73")

      port map(A => \can_top_level_inst/drv_bus[460]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1\, 
        C => \can_top_level_inst/drv_bus[511]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1069\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m19_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1[13]\ : 
        CFG4
      generic map(INIT => x"C0E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_1_Z[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac21_1\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac21_1_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_0[2]\, 
        B => \sbe_c[1]\, C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_1_0_Z[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un4_compl_ctr_ce\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un4_compl_ctr_ce_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[15]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[15]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_34\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[232]\, B => 
        \can_top_level_inst/rec_ident[27]\, C => 
        \can_top_level_inst/drv_bus[265]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_46\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_set_idle_0\ : 
        CFG4
      generic map(INIT => x"FFEC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_set_idle_0_a4_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1113\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1062\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1183\, 
        Y => \can_top_level_inst/can_core_inst/set_idle\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask.un7_flip_mask_c3\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[2]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[1]\, C => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_c3\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[28]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[28]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[28]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[6]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => \can_top_level_inst/N_4606\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_139\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_0\, 
        D => \can_top_level_inst/txtb_parity_error_valid[2]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_165\);
    
    \timestamp_ibuf[56]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[56]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[56]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[4]\, 
        B => \can_top_level_inst/drv_bus[60]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_3_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_4_Z\, 
        CC => NET_CC_CONFIG2466, P => NET_CC_CONFIG2463, Y3 => 
        NET_CC_CONFIG2464, Y3A => NET_CC_CONFIG2465);
    
    
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/rx_trig_req_q\ : 
        SLE
      port map(D => \can_top_level_inst/rx_triggers[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[184]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_u[2]\ : 
        CFG4
      generic map(INIT => x"2203")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_mem_index[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1465\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_RNI3DKB1[1]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\, C
         => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[1]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[0]\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[1]\, 
        CC => NET_CC_CONFIG1099, P => NET_CC_CONFIG1096, Y3 => 
        NET_CC_CONFIG1097, Y3A => NET_CC_CONFIG1098);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa18\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe18\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[2]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_control_reg_comp/bit_gen.1.reg_present_gen.reg_autoclear_gen.reg_value_r_8[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_tst_control_reg_comp_wr_select[0]\, 
        B => \data_in_c[1]\, Y => 
        \can_top_level_inst/test_registers_out.tst_control[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0\ : 
        CFG3
      generic map(INIT => x"F4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_909\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_877\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[94]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[15]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[14]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_wmux_1\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[2]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[1]\, 
        C => \can_top_level_inst/N_3900_i\, D => 
        \can_top_level_inst/N_3897_i\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_y0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_0\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[400]\, B => 
        \can_top_level_inst/stat_bus[71]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_0_Z\, 
        CC => NET_CC_CONFIG212, P => NET_CC_CONFIG209, Y3 => 
        NET_CC_CONFIG210, Y3A => NET_CC_CONFIG211);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[13]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa15\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe15\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m31_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_20\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_20_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_RNI189A[0]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_GND, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_0\, 
        CC => NET_CC_CONFIG252, P => NET_CC_CONFIG249, Y3 => 
        NET_CC_CONFIG250, Y3A => NET_CC_CONFIG251);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[9]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10[9]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4_2[7]\ : 
        CFG4
      generic map(INIT => x"C808")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_s_7_S\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, D
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_s_7_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[15]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[15]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[14]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1104\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1107\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_y0_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a2[1]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/txtb_parity_error_valid[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_178\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[22]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/int_mask_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/int_mask_load_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_mask[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_2[4]\ : 
        CFG3
      generic map(INIT => x"35")

      port map(A => \can_top_level_inst/drv_bus[39]\, B => 
        \can_top_level_inst/stat_bus[85]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_1[4]\ : 
        CFG4
      generic map(INIT => x"4657")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/int_ena[4]\, D => 
        \can_top_level_inst/drv_bus[460]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_1_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[25]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2245\, B
         => \can_top_level_inst/rx_buffer_inst/N_4810\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2278\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[86]\);
    
    \data_out_obuf[12]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[12]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[12]/DOUT\, EOUT => 
        \data_out_obuf[12]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_iv_2[1]\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_iv_0_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_drv_rom_ena_m\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_is_fd_frame_m\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_iv_2_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa15\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa15\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un4_use_basic_segm_length\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un4_use_basic_segm_length_i\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_09\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_135\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_19\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_129\, 
        Y => \can_top_level_inst/tx_arbitrator_inst/l2_prio_09\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[20]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[20]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[20]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1[25]\ : 
        CFG4
      generic map(INIT => x"FFF1")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_960\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1138\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c4_a0_RNIRD4R2\ : 
        CFG4
      generic map(INIT => x"F9F6")

      port map(A => \can_top_level_inst/drv_bus[12]\, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[4]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_NE_5_1\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c4_i\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_NE_5_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa14\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe14\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[2]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[19]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram15_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe15\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram15_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_ena_reg_proc.un21_ctrl_signal_upd_i_o2_0\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/is_bus_off\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_944\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_RNO[2]\ : 
        CFG4
      generic map(INIT => x"6AAA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[2]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[0]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[1]\, 
        D => \can_top_level_inst/bus_sampling_inst/sample_sec\, Y
         => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram89_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe89\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram89_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[113]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[18]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[34]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[28]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[28]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[19]\);
    
    \can_top_level_inst/frame_filters_inst/un1_rec_abort_f\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/frame_filters_inst_ident_valid_q\, B
         => \can_top_level_inst/rec_abort\, Y => 
        \can_top_level_inst/rec_abort_f\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0[10]\ : 
        CFG3
      generic map(INIT => x"70")

      port map(A => \adress_c[7]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \sbe_c[1]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_i[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_write_pointer[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_792_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram112_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe112\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram112_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_1\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6_1\, 
        B => swr_c, C => \adress_c[5]\, D => \adress_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/op_eq.un4_ctrl_ctr_zero\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un2_ctrl_ctr_one_1\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[8]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[8]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[8]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[8]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2632\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_2[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram14_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram15_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y7[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_36_1[5]\ : 
        CFG4
      generic map(INIT => x"00D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[5]\, C => 
        \can_top_level_inst/stat_bus[193]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1352_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\);
    
    \can_rx_ibuf/U_IOPAD\ : IOPAD_IN
      port map(PAD => can_rx, Y => \can_rx_ibuf/YIN\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[15]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[8]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[15]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[15]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_1[12]\ : 
        CFG4
      generic map(INIT => x"8D89")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_1_1_Z[12]\, 
        D => \can_top_level_inst/drv_bus[412]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_1_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[15]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s_2641\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_VCC, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s_2641_FCO\, 
        CC => NET_CC_CONFIG292, P => NET_CC_CONFIG289, Y3 => 
        NET_CC_CONFIG290, Y3A => NET_CC_CONFIG291);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[10]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[19]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[19]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[19]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/int_ena_proc.un1_int_ena_clear\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[1]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/un1_int_ena_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\);
    
    \data_out_obuf[18]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(18), D => \data_out_obuf[18]/DOUT\, 
        E => \data_out_obuf[18]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/retr_ctr_add\ : 
        CFG3
      generic map(INIT => x"C8")

      port map(A => \can_top_level_inst/stat_bus[6]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/retr_ctr_add_sn_N_5_mux\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retr_ctr_add\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_6\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_5_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_6_Z\, 
        CC => NET_CC_CONFIG151, P => NET_CC_CONFIG148, Y3 => 
        NET_CC_CONFIG149, Y3A => NET_CC_CONFIG150);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[15]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[15]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[301]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[19]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[0]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_36_1[14]\ : 
        CFG4
      generic map(INIT => x"00D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[14]\, C => 
        \can_top_level_inst/stat_bus[202]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_704_1\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[15]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[15]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_write_pointer_ts_ce_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_1\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[13]\, B
         => \can_top_level_inst/can_core_inst/crc_17[12]\, C => 
        \can_top_level_inst/can_core_inst/crc_17[13]\, D => 
        \can_top_level_inst/store_data_word[12]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[5]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[6]\, 
        CC => NET_CC_CONFIG2551, P => NET_CC_CONFIG2548, Y3 => 
        NET_CC_CONFIG2549, Y3A => NET_CC_CONFIG2550);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[3]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_139\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_q.failed\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/failed_0\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_hw_cmd.failed\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_3[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4_2[3]\ : 
        CFG4
      generic map(INIT => x"C808")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_3_S\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, D
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_3_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[30]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[25]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_1_1.CO1\ : 
        CFG4
      generic map(INIT => x"0A8E")

      port map(A => \can_top_level_inst/txtb_prorities_2[1]\, B
         => \can_top_level_inst/txtb_prorities_2[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d[5]\ : 
        CFG4
      generic map(INIT => x"BBB8")

      port map(A => \can_top_level_inst/drv_bus[421]\, B => 
        \can_top_level_inst/drv_bus[426]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[5]\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_24_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[28]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[28]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[7]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[29]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/test_registers_out.tst_wdata[29]\, C
         => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[8]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[8]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[23]\);
    
    \timestamp_ibuf[27]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[27]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[27]/YIN\);
    
    \data_in_ibuf[30]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(30), Y => \data_in_ibuf[30]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => \adress_c[5]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_2\, 
        C => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        D => \sbe_c[0]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[191]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g1\ : 
        CFG3
      generic map(INIT => x"AE")

      port map(A => \can_top_level_inst/prescaler_inst/is_tseg2\, 
        B => can_tx_c, C => 
        \can_top_level_inst/prescaler_inst/current_state[1]\, Y
         => \can_top_level_inst/prescaler_inst/g1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_sm0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_1\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_sn_N_7_mux\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_dominant\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[6]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \can_top_level_inst/N_4606\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_139\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_179\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_165\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q_RNO[5]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_15[4]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[6]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_24\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[295]\, B => 
        \can_top_level_inst/rec_ident[24]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_23\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_24\, 
        CC => NET_CC_CONFIG1929, P => NET_CC_CONFIG1926, Y3 => 
        NET_CC_CONFIG1927, Y3A => NET_CC_CONFIG1928);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[24]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[200]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_38_1_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_38_1_0_y0[22]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1006\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1008\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[13]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[8]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[13]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[13]\);
    
    
        \can_top_level_inst/frame_filters_inst/filt_sup_gen_true.un6_filter_result\ : 
        CFG4
      generic map(INIT => x"EEEF")

      port map(A => 
        \can_top_level_inst/frame_filters_inst/int_filter_C_valid\, 
        B => 
        \can_top_level_inst/frame_filters_inst/un6_filter_result_0\, 
        C => 
        \can_top_level_inst/frame_filters_inst/un1_filter_b_enable_Z\, 
        D => 
        \can_top_level_inst/frame_filters_inst/valid_0_I_87_FCO_0\, 
        Y => 
        \can_top_level_inst/frame_filters_inst/un6_filter_result\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI83GC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_cnst_i[2]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1465\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[5]\ : 
        CFG4
      generic map(INIT => x"FEFA")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_163\, 
        B => \can_top_level_inst/txtb_allow_bb[2]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_164\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram52__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_349\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe52\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_0[2]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_988\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[14]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_0_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped[13]\ : 
        CFG4
      generic map(INIT => x"78F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_0[12]\, 
        B => \can_top_level_inst/tran_frame_test.fcrc\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[13]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1_Z[13]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[13]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIESGG6[12]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[12]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[12]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_11\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_12\, 
        CC => NET_CC_CONFIG1325, P => NET_CC_CONFIG1322, Y3 => 
        NET_CC_CONFIG1323, Y3A => NET_CC_CONFIG1324);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[30]\);
    
    \timestamp_ibuf[62]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[62]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[62]/YIN\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_24\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[24]\, B => 
        \can_top_level_inst/drv_bus[324]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_23\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_24\, 
        CC => NET_CC_CONFIG625, P => NET_CC_CONFIG622, Y3 => 
        NET_CC_CONFIG623, Y3A => NET_CC_CONFIG624);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[29]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[15]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[15]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_0[4]\ : 
        CFG4
      generic map(INIT => x"FF35")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[15]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[6]\, C => 
        \can_top_level_inst/drv_bus[475]\, D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_143\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram74__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_348\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe74\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[4]\, 
        D => \can_top_level_inst/drv_bus[33]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[4]\);
    
    \timestamp_ibuf[57]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[57]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[57]/YIN\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[13]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2471\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_load_init_vect_0\ : 
        CFG4
      generic map(INIT => x"CCEC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1183\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        Y => \can_top_level_inst/can_core_inst/load_init_vect\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0_RNITT9ER2[2]\ : 
        ARI1_CC
      generic map(INIT => x"58778")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[2]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_49\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_1\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_2\, 
        CC => NET_CC_CONFIG2115, P => NET_CC_CONFIG2112, Y3 => 
        NET_CC_CONFIG2113, Y3A => NET_CC_CONFIG2114);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_0[2]\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \can_top_level_inst/txtb_hw_cmd.err\, B => 
        \can_top_level_inst/txtb_hw_cmd.arbl\, C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_57\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[24]\, B => 
        \can_top_level_inst/drv_bus[229]\, C => 
        \can_top_level_inst/drv_bus[262]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_26\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[11]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[12]\, 
        CC => NET_CC_CONFIG511, P => NET_CC_CONFIG508, Y3 => 
        NET_CC_CONFIG509, Y3A => NET_CC_CONFIG510);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_147\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[4]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_139\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_5[31]\ : 
        CFG4
      generic map(INIT => x"8D88")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[30]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_11\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_18_0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNIN59H1[2]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[2]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[1]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[2]\, 
        CC => NET_CC_CONFIG1686, P => NET_CC_CONFIG1683, Y3 => 
        NET_CC_CONFIG1684, Y3A => NET_CC_CONFIG1685);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[14]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[14]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry_cy[0]_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO1670, CO => OPEN, P(0) => 
        NET_CC_CONFIG1719, P(1) => NET_CC_CONFIG1723, P(2) => 
        NET_CC_CONFIG1727, P(3) => NET_CC_CONFIG1731, P(4) => 
        ADLIB_VCC, P(5) => ADLIB_VCC, P(6) => ADLIB_VCC, P(7) => 
        ADLIB_VCC, P(8) => ADLIB_VCC, P(9) => ADLIB_VCC, P(10)
         => ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => 
        NET_CC_CONFIG1720, Y3(1) => NET_CC_CONFIG1724, Y3(2) => 
        NET_CC_CONFIG1728, Y3(3) => NET_CC_CONFIG1732, Y3(4) => 
        ADLIB_VCC, Y3(5) => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7)
         => ADLIB_VCC, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG1721, Y3A(1) => NET_CC_CONFIG1725, Y3A(2)
         => NET_CC_CONFIG1729, Y3A(3) => NET_CC_CONFIG1733, 
        Y3A(4) => ADLIB_VCC, Y3A(5) => ADLIB_VCC, Y3A(6) => 
        ADLIB_VCC, Y3A(7) => ADLIB_VCC, Y3A(8) => ADLIB_VCC, 
        Y3A(9) => ADLIB_VCC, Y3A(10) => ADLIB_VCC, Y3A(11) => 
        ADLIB_VCC, CC(0) => NET_CC_CONFIG1722, CC(1) => 
        NET_CC_CONFIG1726, CC(2) => NET_CC_CONFIG1730, CC(3) => 
        NET_CC_CONFIG1734, CC(4) => nc29, CC(5) => nc269, CC(6)
         => nc118, CC(7) => nc60, CC(8) => nc141, CC(9) => nc311, 
        CC(10) => nc276, CC(11) => nc193);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_abort\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid_d_0_sqmuxa_1_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_abort\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[9]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2535\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[31]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa11\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe11\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_RNISO8N1[5]\ : 
        CFG4
      generic map(INIT => x"0116")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[2]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, D
         => \can_top_level_inst/rx_buffer_inst/stored_ts\, Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_10\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[21]\);
    
    \can_top_level_inst/rx_buffer_inst/data_overrun_i_RNO\ : CFG4
      generic map(INIT => x"FFF8")

      port map(A => \data_in_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_command_reg_comp_wr_select[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_Z\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/reset_overrun_flag\, Y
         => \can_top_level_inst/rx_buffer_inst/un1_drv_bus_2_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_0_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG1735, P(1) => NET_CC_CONFIG1739, P(2) => 
        NET_CC_CONFIG1743, P(3) => NET_CC_CONFIG1747, P(4) => 
        NET_CC_CONFIG1751, P(5) => NET_CC_CONFIG1755, P(6) => 
        NET_CC_CONFIG1759, P(7) => ADLIB_VCC, P(8) => ADLIB_VCC, 
        P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11) => ADLIB_VCC, 
        Y3(0) => NET_CC_CONFIG1736, Y3(1) => NET_CC_CONFIG1740, 
        Y3(2) => NET_CC_CONFIG1744, Y3(3) => NET_CC_CONFIG1748, 
        Y3(4) => NET_CC_CONFIG1752, Y3(5) => NET_CC_CONFIG1756, 
        Y3(6) => NET_CC_CONFIG1760, Y3(7) => ADLIB_VCC, Y3(8) => 
        ADLIB_VCC, Y3(9) => ADLIB_VCC, Y3(10) => ADLIB_VCC, 
        Y3(11) => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG1737, Y3A(1)
         => NET_CC_CONFIG1741, Y3A(2) => NET_CC_CONFIG1745, 
        Y3A(3) => NET_CC_CONFIG1749, Y3A(4) => NET_CC_CONFIG1753, 
        Y3A(5) => NET_CC_CONFIG1757, Y3A(6) => NET_CC_CONFIG1761, 
        Y3A(7) => ADLIB_VCC, Y3A(8) => ADLIB_VCC, Y3A(9) => 
        ADLIB_VCC, Y3A(10) => ADLIB_VCC, Y3A(11) => ADLIB_VCC, 
        CC(0) => NET_CC_CONFIG1738, CC(1) => NET_CC_CONFIG1742, 
        CC(2) => NET_CC_CONFIG1746, CC(3) => NET_CC_CONFIG1750, 
        CC(4) => NET_CC_CONFIG1754, CC(5) => NET_CC_CONFIG1758, 
        CC(6) => NET_CC_CONFIG1762, CC(7) => nc214, CC(8) => 
        nc298, CC(9) => nc282, CC(10) => nc240, CC(11) => nc45);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[28]\);
    
    \can_top_level_inst/int_manager_inst/int_input_active_5[11]\ : 
        CFG4
      generic map(INIT => x"FF20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[3]\, 
        D => 
        \can_top_level_inst/int_manager_inst/int_input_active_1_Z[11]\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_5_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m24_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_3_1_RNIMVGD1\ : 
        ARI1_CC
      generic map(INIT => x"5A6AA")

      port map(A => \can_top_level_inst/err_detected\, B => 
        \can_top_level_inst/stat_bus[81]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_0_1\, 
        D => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        FCI => ADLIB_GND, S => OPEN, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_3_1_RNIMVGD1_Y\, 
        FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_0\, 
        CC => NET_CC_CONFIG329, P => NET_CC_CONFIG326, Y3 => 
        NET_CC_CONFIG327, Y3A => NET_CC_CONFIG328);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[2]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m112_2\ : 
        CFG4
      generic map(INIT => x"B0B5")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[21]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m112_2_1_Z\, 
        D => \can_top_level_inst/drv_bus[405]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_113\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm_0[2]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[2]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un7_rx_err_ctr_declto8_i_a2_0_a2_RNIR2DS\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/stat_bus[82]\, B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_0\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNIR2DS_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_1\, 
        CC => NET_CC_CONFIG2322, P => NET_CC_CONFIG2319, Y3 => 
        NET_CC_CONFIG2320, Y3A => NET_CC_CONFIG2321);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIILP4F[21]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[209]\, D => 
        \can_top_level_inst/stat_bus[241]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_20\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIILP4F_S[21]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_21\, 
        CC => NET_CC_CONFIG840, P => NET_CC_CONFIG837, Y3 => 
        NET_CC_CONFIG838, Y3A => NET_CC_CONFIG839);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_36[1]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => \can_top_level_inst/stat_bus[301]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1200_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1200\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o2_1[35]\ : 
        CFG2
      generic map(INIT => x"B")

      port map(A => \can_top_level_inst/is_bus_off\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_952\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[2]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2528\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_81\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[16]\, B => 
        \can_top_level_inst/drv_bus[221]\, C => 
        \can_top_level_inst/drv_bus[254]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_6\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[7]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[8]\, 
        CC => NET_CC_CONFIG495, P => NET_CC_CONFIG492, Y3 => 
        NET_CC_CONFIG493, Y3A => NET_CC_CONFIG494);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_5\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_4_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_5_Z\, 
        CC => NET_CC_CONFIG147, P => NET_CC_CONFIG144, Y3 => 
        NET_CC_CONFIG145, Y3A => NET_CC_CONFIG146);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2[14]\ : 
        CFG3
      generic map(INIT => x"07")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_1[14]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_Z[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un4_crc_match_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_match\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[27]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m23_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_pointer_inc_1_i_axbxc6\ : 
        CFG4
      generic map(INIT => x"6AAA")

      port map(A => \can_top_level_inst/rx_read_pointer[6]\, B
         => \can_top_level_inst/rx_read_pointer[5]\, C => 
        \can_top_level_inst/rx_read_pointer[4]\, D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_pointer_inc_1_i_c4\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa19\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe19\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a2[3]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_192\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[10]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[15]\);
    
    \timestamp_ibuf[24]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[24]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[24]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe20\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOD6T6[8]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[196]\, D => 
        \can_top_level_inst/stat_bus[228]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_7\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOD6T6_S[8]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_8\, 
        CC => NET_CC_CONFIG788, P => NET_CC_CONFIG785, Y3 => 
        NET_CC_CONFIG786, Y3A => NET_CC_CONFIG787);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_0\ : 
        CFG3
      generic map(INIT => x"82")

      port map(A => \can_top_level_inst/drv_bus[465]\, B => 
        \can_top_level_inst/stat_bus[5]\, C => 
        \can_top_level_inst/drv_bus[469]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_0_Z\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_28\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[154]\, B => 
        \can_top_level_inst/rec_ident[11]\, C => 
        \can_top_level_inst/drv_bus[187]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_51\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select[3]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_0\, 
        D => \sbe_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/pexs_set_i\ : 
        CFG4
      generic map(INIT => x"0032")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1013\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/pexs_set_i_0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_875\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[4]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[4]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[18]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_RNO[27]\ : 
        CFG4
      generic map(INIT => x"CC93")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[16]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/m42_1\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[16]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1218\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[17]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_14\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[285]\, B => 
        \can_top_level_inst/rec_ident[14]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_13\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_14\, 
        CC => NET_CC_CONFIG1889, P => NET_CC_CONFIG1886, Y3 => 
        NET_CC_CONFIG1887, Y3A => NET_CC_CONFIG1888);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\);
    
    \data_out_obuf[27]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(27), D => \data_out_obuf[27]/DOUT\, 
        E => \data_out_obuf[27]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_17\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_17_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_782_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[3]\ : 
        CFG4
      generic map(INIT => x"F7FF")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[0]\, B
         => \can_top_level_inst/txtb_hw_cmd.unlock\, C => 
        \can_top_level_inst/txtb_hw_cmd_index[1]\, D => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_136\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[29]\ : 
        SLE
      port map(D => \data_in_c[29]\, CLK => 
        \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[58]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[16]\);
    
    \can_top_level_inst/tx_arbitrator_inst/last_txtb_index[1]\ : 
        SLE
      port map(D => \can_top_level_inst/txtb_hw_cmd_index[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_last_txtb_index\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/last_txtb_index_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_1[1]\ : 
        CFG4
      generic map(INIT => x"EAFA")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_163\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        D => \can_top_level_inst/N_4606\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[21]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[21]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNIJSV84[0]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/txtb_port_b_address[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/N_4197\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/N_4198\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_18[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_7[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram94_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram95_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y7_0[0]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[3]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[2]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[22]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm_0[0]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[25]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[25]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[25]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_2_2[20]\ : 
        CFG4
      generic map(INIT => x"4F7F")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[20]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_Z[20]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => \can_top_level_inst/drv_bus[225]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_2_1[20]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[7]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[7]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[7]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[0]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/int_mask_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/int_mask_load_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_mask[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[23]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[23]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[23]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lto6\ : 
        CFG4
      generic map(INIT => x"FFF8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lt4\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[6]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[5]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_d_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[21]\);
    
    \timestamp_ibuf[54]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[54]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[54]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[29]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2555\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[18]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_15\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[4]\, B => 
        \can_top_level_inst/drv_bus[147]\, C => 
        \can_top_level_inst/drv_bus[180]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_61\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[1]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[2]\, 
        CC => NET_CC_CONFIG698, P => NET_CC_CONFIG695, Y3 => 
        NET_CC_CONFIG696, Y3A => NET_CC_CONFIG697);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[5]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[30]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_1\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[415]\, B => 
        \can_top_level_inst/stat_bus[88]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_6\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_7\, 
        CC => NET_CC_CONFIG199, P => NET_CC_CONFIG196, Y3 => 
        NET_CC_CONFIG197, Y3A => NET_CC_CONFIG198);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_0_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO394, CO => OPEN, P(0) => 
        NET_CC_CONFIG431, P(1) => NET_CC_CONFIG435, P(2) => 
        NET_CC_CONFIG439, P(3) => NET_CC_CONFIG443, P(4) => 
        NET_CC_CONFIG447, P(5) => NET_CC_CONFIG451, P(6) => 
        NET_CC_CONFIG455, P(7) => ADLIB_VCC, P(8) => ADLIB_VCC, 
        P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11) => ADLIB_VCC, 
        Y3(0) => NET_CC_CONFIG432, Y3(1) => NET_CC_CONFIG436, 
        Y3(2) => NET_CC_CONFIG440, Y3(3) => NET_CC_CONFIG444, 
        Y3(4) => NET_CC_CONFIG448, Y3(5) => NET_CC_CONFIG452, 
        Y3(6) => NET_CC_CONFIG456, Y3(7) => ADLIB_VCC, Y3(8) => 
        ADLIB_VCC, Y3(9) => ADLIB_VCC, Y3(10) => ADLIB_VCC, 
        Y3(11) => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG433, Y3A(1)
         => NET_CC_CONFIG437, Y3A(2) => NET_CC_CONFIG441, Y3A(3)
         => NET_CC_CONFIG445, Y3A(4) => NET_CC_CONFIG449, Y3A(5)
         => NET_CC_CONFIG453, Y3A(6) => NET_CC_CONFIG457, Y3A(7)
         => ADLIB_VCC, Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, 
        Y3A(10) => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG434, CC(1) => NET_CC_CONFIG438, CC(2) => 
        NET_CC_CONFIG442, CC(3) => NET_CC_CONFIG446, CC(4) => 
        NET_CC_CONFIG450, CC(5) => NET_CC_CONFIG454, CC(6) => 
        NET_CC_CONFIG458, CC(7) => nc53, CC(8) => nc121, CC(9)
         => nc176, CC(10) => nc220, CC(11) => nc158);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[29]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/int_status\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/un2_int_status_set\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/un1_int_status_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_vector[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO[28]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[28]\, B
         => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac11_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[892]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[5]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_0[6]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[3]\, 
        B => \can_top_level_inst/stat_bus[68]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_146\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/fixed_prev_d_u\ : 
        CFG4
      generic map(INIT => x"7400")

      port map(A => 
        \can_top_level_inst/can_core_inst/stuff_length[0]\, B => 
        \can_top_level_inst/rx_triggers[1]\, C => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/fixed_prev_q\, 
        D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/reg_q_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/fixed_prev_d\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_19\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[290]\, B => 
        \can_top_level_inst/rec_ident[19]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_18\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_19\, 
        CC => NET_CC_CONFIG1909, P => NET_CC_CONFIG1906, Y3 => 
        NET_CC_CONFIG1907, Y3A => NET_CC_CONFIG1908);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[6]\ : 
        CFG4
      generic map(INIT => x"FFA2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[2]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_165\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/int_status\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/un2_int_status_set\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/un1_int_status_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_vector[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[10]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_dbt_Z[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_5_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa13\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe13\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\);
    
    \timestamp_ibuf[15]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(15), Y => 
        \timestamp_ibuf[15]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q_RNIAJTQ1[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO83, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => 
        ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_GND, P(6) => 
        NET_CC_CONFIG84, P(7) => NET_CC_CONFIG88, P(8) => 
        NET_CC_CONFIG92, P(9) => NET_CC_CONFIG96, P(10) => 
        NET_CC_CONFIG100, P(11) => NET_CC_CONFIG104, Y3(0) => 
        ADLIB_VCC, Y3(1) => ADLIB_VCC, Y3(2) => ADLIB_VCC, Y3(3)
         => ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5) => ADLIB_VCC, 
        Y3(6) => NET_CC_CONFIG85, Y3(7) => NET_CC_CONFIG89, Y3(8)
         => NET_CC_CONFIG93, Y3(9) => NET_CC_CONFIG97, Y3(10) => 
        NET_CC_CONFIG101, Y3(11) => NET_CC_CONFIG105, Y3A(0) => 
        ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => ADLIB_VCC, 
        Y3A(3) => ADLIB_VCC, Y3A(4) => ADLIB_VCC, Y3A(5) => 
        ADLIB_VCC, Y3A(6) => NET_CC_CONFIG86, Y3A(7) => 
        NET_CC_CONFIG90, Y3A(8) => NET_CC_CONFIG94, Y3A(9) => 
        NET_CC_CONFIG98, Y3A(10) => NET_CC_CONFIG102, Y3A(11) => 
        NET_CC_CONFIG106, CC(0) => nc281, CC(1) => nc209, CC(2)
         => nc246, CC(3) => nc351, CC(4) => nc162, CC(5) => nc11, 
        CC(6) => NET_CC_CONFIG87, CC(7) => NET_CC_CONFIG91, CC(8)
         => NET_CC_CONFIG95, CC(9) => NET_CC_CONFIG99, CC(10) => 
        NET_CC_CONFIG103, CC(11) => NET_CC_CONFIG107);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe16_0_a4\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[0]\, 
        B => \can_top_level_inst/rx_buffer_inst/un5_ram_write_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[15]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[15]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[15]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe31_0_a4_0\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_351\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[1]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1_0[4]\ : 
        CFG4
      generic map(INIT => x"D9C8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1287\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1286\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1_0_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_2_0[21]\ : 
        CFG4
      generic map(INIT => x"3708")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_2_0_1_Z[21]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_1_0_Z[21]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_2_0_Z[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/set_idle_i_5_sqmuxa_0_a4\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/set_idle_i_5_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.err_0_sqmuxa\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa_5\, 
        B => \can_top_level_inst/stat_bus[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa_4\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_s2_0_a2\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\, 
        Y => \can_top_level_inst/stat_bus[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[7]\);
    
    \I_1/U0_RGB1_RGB7\ : RGB
      port map(A => \I_1/U0_Y\, EN => ADLIB_VCC, Y => 
        \AFLSDF_INV_5\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[31]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[31]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_18\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_18_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[12]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[11]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[12]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[30]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[30]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/int_stat_proc.un1_int_status_clear\ : 
        CFG4
      generic map(INIT => x"F8F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_stat_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => \sbe_c[1]\, C => 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/un2_int_status_set\, 
        D => \data_in_c[9]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/un1_int_status_clear\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[7]\ : 
        CFG4
      generic map(INIT => x"F888")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        C => \can_top_level_inst/txtb_parity_error_valid[5]\, D
         => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_5_Z\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_6_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_6_Z\, 
        CC => NET_CC_CONFIG2243, P => NET_CC_CONFIG2240, Y3 => 
        NET_CC_CONFIG2241, Y3A => NET_CC_CONFIG2242);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram33_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe33\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram33_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m76\ : 
        CFG3
      generic map(INIT => x"1B")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[212]\, C => 
        \can_top_level_inst/drv_bus[175]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i81_mux\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[5]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        B => \can_top_level_inst/txtb_hw_cmd.failed\, C => 
        \can_top_level_inst/txtb_parity_error_valid[2]\, D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_164\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => \can_top_level_inst/drv_bus[36]\, B => 
        \can_top_level_inst/drv_bus[42]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_0_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_1_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_1_Z\, 
        CC => NET_CC_CONFIG1742, P => NET_CC_CONFIG1739, Y3 => 
        NET_CC_CONFIG1740, Y3A => NET_CC_CONFIG1741);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa_0_a2_2\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_189\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_186\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[16]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[16]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_34[1]\ : 
        CFG4
      generic map(INIT => x"CCFE")

      port map(A => \can_top_level_inst/trv_delay[1]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1198_2\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1198\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[4]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_nbt_Z[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_4_S\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram64_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram65_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0[0]\);
    
    \can_top_level_inst/rx_buffer_inst/rx_read_buff[18]\ : CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/rx_buffer_inst_RAM_data_out[18]\, 
        Y => \can_top_level_inst/rx_read_buff[18]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[25]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[25]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[25]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d[6]\ : 
        CFG4
      generic map(INIT => x"BBB8")

      port map(A => \can_top_level_inst/drv_bus[422]\, B => 
        \can_top_level_inst/drv_bus[426]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[6]\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_20_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[131]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[10]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[22]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_d_0_RNI1FNF_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG1638, P(1) => NET_CC_CONFIG1642, P(2) => 
        NET_CC_CONFIG1646, P(3) => NET_CC_CONFIG1650, P(4) => 
        NET_CC_CONFIG1654, P(5) => NET_CC_CONFIG1658, P(6) => 
        NET_CC_CONFIG1662, P(7) => NET_CC_CONFIG1666, P(8) => 
        ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11)
         => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1639, Y3(1) => 
        NET_CC_CONFIG1643, Y3(2) => NET_CC_CONFIG1647, Y3(3) => 
        NET_CC_CONFIG1651, Y3(4) => NET_CC_CONFIG1655, Y3(5) => 
        NET_CC_CONFIG1659, Y3(6) => NET_CC_CONFIG1663, Y3(7) => 
        NET_CC_CONFIG1667, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG1640, Y3A(1) => NET_CC_CONFIG1644, Y3A(2)
         => NET_CC_CONFIG1648, Y3A(3) => NET_CC_CONFIG1652, 
        Y3A(4) => NET_CC_CONFIG1656, Y3A(5) => NET_CC_CONFIG1660, 
        Y3A(6) => NET_CC_CONFIG1664, Y3A(7) => NET_CC_CONFIG1668, 
        Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => 
        ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG1641, CC(1) => NET_CC_CONFIG1645, CC(2) => 
        NET_CC_CONFIG1649, CC(3) => NET_CC_CONFIG1653, CC(4) => 
        NET_CC_CONFIG1657, CC(5) => NET_CC_CONFIG1661, CC(6) => 
        NET_CC_CONFIG1665, CC(7) => NET_CC_CONFIG1669, CC(8) => 
        nc272, CC(9) => nc131, CC(10) => nc254, CC(11) => nc267);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[15]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un5_tx_err_ctr_declto8_i_a2_4\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => \can_top_level_inst/stat_bus[78]\, B => 
        \can_top_level_inst/stat_bus[77]\, C => 
        \can_top_level_inst/stat_bus[79]\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un5_tx_err_ctr_declto8_i_a2_4\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[5]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[4]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[16]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_ena_reg_proc.stuff_enable_1_f0\ : 
        CFG3
      generic map(INIT => x"0D")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_905_i\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_clear_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_rx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[87]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\);
    
    \data_out_obuf[30]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[30]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[30]/DOUT\, EOUT => 
        \data_out_obuf[30]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_89_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[1]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_RNI2BTRM\ : 
        CFG4
      generic map(INIT => x"4F44")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_sx_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_8_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/un7_exit_segm_regular_cry_9_0\, 
        D => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_12_1_sx\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_RNO[6]\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_priority_decoder_inst_un1_m2_0_a2_0_1\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_nxt\ : 
        CFG4
      generic map(INIT => x"656A")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[16]\, 
        B => \can_top_level_inst/can_core_inst/crc_data_rx_wbs\, 
        C => \can_top_level_inst/can_core_inst/crc_calc_from_rx\, 
        D => \can_top_level_inst/can_core_inst/bst_data_out\, Y
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_nxt_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_load_base_id_i_0_0\ : 
        CFG4
      generic map(INIT => x"333B")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_load_base_id_i_0_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[25]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[296]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[20]\ : 
        CFG4
      generic map(INIT => x"B830")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[20]\, 
        D => \can_top_level_inst/tran_word[12]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[31]\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q_RNO[3]\ : 
        CFG4
      generic map(INIT => x"C693")

      port map(A => \can_top_level_inst/rx_mof\, B => 
        \can_top_level_inst/rx_buffer_inst/N_2833\, C => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[3]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[14]\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_3927_i\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIEM93A[25]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[25]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[25]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_24\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_25\, 
        CC => NET_CC_CONFIG1377, P => NET_CC_CONFIG1374, Y3 => 
        NET_CC_CONFIG1375, Y3A => NET_CC_CONFIG1376);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[19]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO[7]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_43\, 
        B => 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[10]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_17[26]\ : 
        CFG4
      generic map(INIT => x"FFEF")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[922]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_12_Z[26]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_17_1_Z[26]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_8_Z[26]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_17_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_RNI73352[6]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_10\, 
        B => \can_top_level_inst/rx_buffer_inst/data_selector[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[285]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_34_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/trv_delay[6]\, D => 
        \can_top_level_inst/stat_bus[194]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_34_1_0_y0[6]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNIPO513[5]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[5]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[4]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[5]\, 
        CC => NET_CC_CONFIG1698, P => NET_CC_CONFIG1695, Y3 => 
        NET_CC_CONFIG1696, Y3A => NET_CC_CONFIG1697);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_2_2[10]\ : 
        CFG4
      generic map(INIT => x"551B")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => \can_top_level_inst/int_ena[10]\, C => 
        \can_top_level_inst/drv_bus[281]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1_Z[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_2_1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[1]\ : 
        CFG4
      generic map(INIT => x"0A0B")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        B => \can_top_level_inst/txtb_sw_cmd.set_ety\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_141\, 
        D => \can_top_level_inst/txtb_allow_bb[6]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[1]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_35_sqmuxa_0_a4\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        Y => \can_top_level_inst/stat_bus[379]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[28]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[28]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_segm_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[214]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => \adress_c[8]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_un1_ram_write_0\, 
        D => \adress_c[9]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[31]\);
    
    \can_top_level_inst/int_manager_inst/un1_int_i_9\ : CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/int_manager_inst/un1_int_i_3_Z\, B
         => \can_top_level_inst/int_manager_inst/un1_int_i_2_Z\, 
        C => \can_top_level_inst/int_manager_inst/un1_int_i_1_Z\, 
        D => \can_top_level_inst/int_manager_inst/un1_int_i_0_Z\, 
        Y => \can_top_level_inst/int_manager_inst/un1_int_i_9_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI8MMFB[30]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[30]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[30]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_29\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_30\, 
        CC => NET_CC_CONFIG1397, P => NET_CC_CONFIG1394, Y3 => 
        NET_CC_CONFIG1395, Y3A => NET_CC_CONFIG1396);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[23]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[23]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[23]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un8_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[22]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[25]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[25]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[25]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1[21]\ : 
        CFG4
      generic map(INIT => x"A989")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_1_Z[21]\, 
        D => \can_top_level_inst/drv_bus[10]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[24]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[1]\, 
        C => \can_top_level_inst/txtb_port_b_address[1]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un2_commit_rx_frame\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_mem_free[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[18]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/N_4137\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_buf_cmds_proc.store_data_2\ : 
        CFG4
      generic map(INIT => x"C088")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data_d_0_sqmuxa_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid_d_1_sqmuxa_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data_d_1_sqmuxa_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data_2\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_0[1]\, B
         => \can_top_level_inst/txtb_prorities_1[1]\, C => 
        \can_top_level_inst/drv_bus_fast[475]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[25]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[157]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_10_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/rx_parity_error\, D => 
        \can_top_level_inst/drv_bus[185]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_10_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[1]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2625\, Y
         => \can_top_level_inst/memory_registers_inst/N_2690\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_8_1\ : 
        CFG3
      generic map(INIT => x"A2")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_26\, 
        C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_8_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns[1]\ : 
        CFG4
      generic map(INIT => x"FEFA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1113\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_a4_1_0_Z[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_0_Z[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram12__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_342\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe12\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_124_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[11]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[11]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe16_0_a4_0\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_355\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[2]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[2]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_11\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[161]\, B => 
        \can_top_level_inst/rec_ident[18]\, C => 
        \can_top_level_inst/drv_bus[194]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_67\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m36_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIRV4S4[6]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[279]\, C => 
        \can_top_level_inst/stat_bus[263]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_5\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIRV4S4_S[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_6\, 
        CC => NET_CC_CONFIG2592, P => NET_CC_CONFIG2589, Y3 => 
        NET_CC_CONFIG2590, Y3A => NET_CC_CONFIG2591);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_38_1_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_38_1_0_y0[2]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1236\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1238\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[15]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[47]\, 
        C => \can_top_level_inst/rec_ident[15]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2260\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[6]\ : 
        CFG4
      generic map(INIT => x"0010")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[6]\, 
        D => \can_top_level_inst/txtb_parity_error_valid[7]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_175\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"46600")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[6]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_5_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_6_Z\, 
        CC => NET_CC_CONFIG2684, P => NET_CC_CONFIG2681, Y3 => 
        NET_CC_CONFIG2682, Y3A => NET_CC_CONFIG2683);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[40]\ : 
        SLE
      port map(D => \timestamp_c[40]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[40]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_2_3[1]\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_2_1_Z[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_9\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1012\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_2_3_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_2_1[1]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[10]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_2_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram76__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_342\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe76\);
    
    
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/g0_0_0_0\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst_prev_Sample\, B => 
        \can_top_level_inst/stat_bus[15]\, Y => 
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/g0_0_0_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[8]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[2]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[2]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[2]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q_RNO[6]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_15[5]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data_d_1_sqmuxa\ : 
        CFG4
      generic map(INIT => x"D0F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un4_ctrl_counted_byte_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data_d_1_sqmuxa_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m24_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_0[13]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_0_1_Z[13]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_655\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_0_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_5[26]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac23_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac20_Z\, 
        C => \can_top_level_inst/drv_bus[202]\, D => 
        \can_top_level_inst/drv_bus[107]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_5_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y1[31]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO[4]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_40\, 
        B => 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0\ : 
        CFG4
      generic map(INIT => x"FCF4")

      port map(A => 
        \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g1_1_Z\, 
        C => \can_top_level_inst/can_core_inst/load_init_vect\, D
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst_crc_17_21_trigger_1_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[6]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[6]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[6]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4814\);
    
    \can_top_level_inst/rx_buffer_inst/rx_read_buff[16]\ : CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/rx_buffer_inst_RAM_data_out[16]\, 
        Y => \can_top_level_inst/rx_read_buff[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_RNIFIEJ1[4]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_3\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_4\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_4\, 
        CC => NET_CC_CONFIG268, P => NET_CC_CONFIG265, Y3 => 
        NET_CC_CONFIG266, Y3A => NET_CC_CONFIG267);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m24_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[17]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2641\, Y
         => \can_top_level_inst/memory_registers_inst/N_2706\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d[3]\ : 
        CFG4
      generic map(INIT => x"BBB8")

      port map(A => \can_top_level_inst/drv_bus[419]\, B => 
        \can_top_level_inst/drv_bus[426]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[3]\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_4_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_10[26]\ : 
        CFG4
      generic map(INIT => x"FFF8")

      port map(A => \can_top_level_inst/drv_bus[140]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac22_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_5_Z[26]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[634]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_10_Z[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_8_to_64_integer_0_a3[3]\ : 
        CFG3
      generic map(INIT => x"13")

      port map(A => \can_top_level_inst/tran_dlc[2]\, B => 
        \can_top_level_inst/tran_dlc[1]\, C => 
        \can_top_level_inst/tran_dlc[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_8_to_64_integer[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_51\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[3]\, B => 
        \can_top_level_inst/drv_bus[208]\, C => 
        \can_top_level_inst/drv_bus[241]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_32\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[0]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[1]\, 
        CC => NET_CC_CONFIG467, P => NET_CC_CONFIG464, Y3 => 
        NET_CC_CONFIG465, Y3A => NET_CC_CONFIG466);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[28]\ : 
        SLE
      port map(D => \data_in_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[171]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_1[4]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[11]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un9_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[27]\);
    
    \timestamp_ibuf[32]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[32]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[32]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m23_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[25]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2551\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2483\, Y
         => \can_top_level_inst/memory_registers_inst/N_2681\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIG3FI5_S[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[280]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0UM0H_S[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[212]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[8]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[8]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[8]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[16]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[15]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac37\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac37_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac21_1_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac37_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_2[22]\ : 
        CFG4
      generic map(INIT => x"A820")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1463_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\);
    
    \data_in_ibuf[27]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(27), Y => \data_in_ibuf[27]/YIN\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_gate_mem_read.AXB[2]\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/AXB[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m23_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[13]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[12]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_7\ : 
        ARI1_CC
      generic map(INIT => x"553AC")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[7]\, 
        B => \can_top_level_inst/bus_sampling_inst/ssp_delay[7]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[7]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_6_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_7_Z\, 
        CC => NET_CC_CONFIG426, P => NET_CC_CONFIG423, Y3 => 
        NET_CC_CONFIG424, Y3A => NET_CC_CONFIG425);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_21_RNI4IMQ1\ : 
        CFG4
      generic map(INIT => x"9669")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_28_Z\, 
        Y => \can_top_level_inst/N_3904_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[20]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_18\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[18]\, B => 
        \can_top_level_inst/drv_bus[318]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_17\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_18\, 
        CC => NET_CC_CONFIG601, P => NET_CC_CONFIG598, Y3 => 
        NET_CC_CONFIG599, Y3A => NET_CC_CONFIG600);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_26[0]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/rx_write_pointer[0]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1152\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[6]\ : 
        CFG4
      generic map(INIT => x"AE00")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        D => \can_top_level_inst/N_4606\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram45__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_354\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe45\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[17]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[17]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[17]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm_0[7]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s_Z[7]\, 
        B => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        C => \can_top_level_inst/prescaler_inst/h_sync_valid\, D
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1[1]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[30]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[12]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un2_commit_rx_frame\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_mem_free[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_1_3_1_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_1_3_1_0_y0[0]\, 
        B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_dest[0]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_64\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[164]\, B => 
        \can_top_level_inst/rec_ident[21]\, C => 
        \can_top_level_inst/drv_bus[197]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_21\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o3[1]\ : 
        CFG3
      generic map(INIT => x"F2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_138\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask[7]\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[4]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[3]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1_0_Z[7]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram93__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_351\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe93\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_pos_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/err_detected\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[103]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[36]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_818_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/int_stat_proc.un2_int_status_set\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/br_shifted\, B => 
        \can_top_level_inst/int_mask[9]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/un2_int_status_set\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[29]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[1]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[1]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[1]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[27]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[4]\ : 
        CFG3
      generic map(INIT => x"D0")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[6]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_160\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\);
    
    \timestamp_ibuf[9]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(9), Y => \timestamp_ibuf[9]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[3]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_83_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[18]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[28]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[28]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_0_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_GND, CO => OPEN, P(0) => 
        NET_CC_CONFIG2447, P(1) => NET_CC_CONFIG2451, P(2) => 
        NET_CC_CONFIG2455, P(3) => NET_CC_CONFIG2459, P(4) => 
        NET_CC_CONFIG2463, P(5) => NET_CC_CONFIG2467, P(6) => 
        NET_CC_CONFIG2471, P(7) => NET_CC_CONFIG2475, P(8) => 
        NET_CC_CONFIG2479, P(9) => NET_CC_CONFIG2483, P(10) => 
        ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG2448, 
        Y3(1) => NET_CC_CONFIG2452, Y3(2) => NET_CC_CONFIG2456, 
        Y3(3) => NET_CC_CONFIG2460, Y3(4) => NET_CC_CONFIG2464, 
        Y3(5) => NET_CC_CONFIG2468, Y3(6) => NET_CC_CONFIG2472, 
        Y3(7) => NET_CC_CONFIG2476, Y3(8) => NET_CC_CONFIG2480, 
        Y3(9) => NET_CC_CONFIG2484, Y3(10) => ADLIB_VCC, Y3(11)
         => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG2449, Y3A(1) => 
        NET_CC_CONFIG2453, Y3A(2) => NET_CC_CONFIG2457, Y3A(3)
         => NET_CC_CONFIG2461, Y3A(4) => NET_CC_CONFIG2465, 
        Y3A(5) => NET_CC_CONFIG2469, Y3A(6) => NET_CC_CONFIG2473, 
        Y3A(7) => NET_CC_CONFIG2477, Y3A(8) => NET_CC_CONFIG2481, 
        Y3A(9) => NET_CC_CONFIG2485, Y3A(10) => ADLIB_VCC, 
        Y3A(11) => ADLIB_VCC, CC(0) => NET_CC_CONFIG2450, CC(1)
         => NET_CC_CONFIG2454, CC(2) => NET_CC_CONFIG2458, CC(3)
         => NET_CC_CONFIG2462, CC(4) => NET_CC_CONFIG2466, CC(5)
         => NET_CC_CONFIG2470, CC(6) => NET_CC_CONFIG2474, CC(7)
         => NET_CC_CONFIG2478, CC(8) => NET_CC_CONFIG2482, CC(9)
         => NET_CC_CONFIG2486, CC(10) => nc96, CC(11) => nc79);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[30]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram7__RNO[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[2]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[1]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe7\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[7]\ : 
        CFG4
      generic map(INIT => x"FFA2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[6]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_168\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_pointer_inc_1_i_axbxc1\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => \can_top_level_inst/rx_read_pointer[0]\, B
         => \can_top_level_inst/rx_read_pointer[1]\, Y => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[25]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[25]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[25]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_0[28]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => \timestamp_c[28]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac2_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac1_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_0_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_RNO[22]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2678\, D
         => \can_top_level_inst/memory_registers_inst/N_2711\, Y
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[3]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[3]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa15\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe15\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[28]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/int_stat_proc.un2_int_status_set\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => \can_top_level_inst/int_mask[3]\, B => 
        \can_top_level_inst/rx_data_overrun\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/un2_int_status_set\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_2[2]\ : 
        CFG4
      generic map(INIT => x"5708")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_2_1[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_1_Z[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_2_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_38_1_wmux_RNO_1[19]\ : 
        CFG4
      generic map(INIT => x"0F3D")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_2_0[19]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_2_1_1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[15]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_RNO[7]\ : 
        CFG4
      generic map(INIT => x"000D")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, B
         => \can_top_level_inst/txtb_hw_cmd.unlock\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_128\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/store_last_txtb_index\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_78_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[3]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2529\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[18]\ : 
        CFG4
      generic map(INIT => x"0078")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[7]\, 
        B => \can_top_level_inst/tran_frame_test.fcrc\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[7]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_909\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[15]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2541\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2473\, Y
         => \can_top_level_inst/memory_registers_inst/N_2671\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa17\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe17\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[15]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2541\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[11]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[11]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[11]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram20_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe20\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram20_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_0[29]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => \timestamp_c[61]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_0_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_data_access_signaller_comp/be_active\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => \sbe_c[3]\, B => \sbe_c[2]\, C => \sbe_c[1]\, 
        D => \sbe_c[0]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_data_access_signaller_comp/be_active_Z\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_7\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[7]\, B => 
        \can_top_level_inst/drv_bus[307]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_6\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_7\, 
        CC => NET_CC_CONFIG557, P => NET_CC_CONFIG554, Y3 => 
        NET_CC_CONFIG555, Y3A => NET_CC_CONFIG556);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_a2_1[1]\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_111\, 
        B => \can_top_level_inst/tx_arbitrator_inst/N_104\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, D
         => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_155\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_132\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_vals2_RNIMTNC_0\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_19\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/int_stat_proc.un2_int_status_set\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => \can_top_level_inst/int_mask[8]\, B => 
        \can_top_level_inst/rx_buffer_inst_rx_full_1_4\, C => 
        \can_top_level_inst/rx_buffer_inst_rx_full_1_5\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/un2_int_status_set\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa4\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe4\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2_2[1]\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_930\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1230\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_wmux_3\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_0_y3\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_0_y1\, 
        C => \can_top_level_inst/txtb_hw_cmd_index[0]\, D => 
        ADLIB_VCC, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[3]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[3]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4812\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/un5_transient_state_0_o3\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_0_1.CO2\ : 
        CFG3
      generic map(INIT => x"D4")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_Z[2]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO2_1_0\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[24]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[24]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[27]\);
    
    \timestamp_ibuf[20]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[20]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[20]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[14]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[14]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[2]\ : 
        CFG4
      generic map(INIT => x"0E0A")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_147\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[7]\ : 
        SLE
      port map(D => \timestamp_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2[4]\ : 
        CFG4
      generic map(INIT => x"D585")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_1[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[33]\ : 
        CFG4
      generic map(INIT => x"5551")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_961\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[33]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_778_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_i[16]\ : 
        CFG4
      generic map(INIT => x"BBBE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_909\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[5]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_i_o2_1_Z[16]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1194\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1187\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[27]\);
    
    
        \can_top_level_inst/bus_sampling_inst/data_edge_detector_inst/un7_sync_edge\ : 
        CFG4
      generic map(INIT => x"A088")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst_data_edge_detector_inst_un7_sync_edge_1\, 
        B => \can_top_level_inst/prescaler_inst_tq_edge_dbt\, C
         => \can_top_level_inst/prescaler_inst_tq_edge_nbt\, D
         => \can_top_level_inst/prescaler_inst_un2_tq_edge\, Y
         => \can_top_level_inst/sync_edge\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1[8]\ : 
        CFG4
      generic map(INIT => x"C9C8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_1_Z[8]\, 
        D => \can_top_level_inst/int_vector[8]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_Z[8]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[16]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[16]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[16]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[16]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un7_rx_err_ctr_declto8_i_a2_0_a2_RNITK7L2\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/stat_bus[86]\, B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_4\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNITK7L2_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_5\, 
        CC => NET_CC_CONFIG2338, P => NET_CC_CONFIG2335, Y3 => 
        NET_CC_CONFIG2336, Y3A => NET_CC_CONFIG2337);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_24_RNI2QFV1\ : 
        CFG4
      generic map(INIT => x"9669")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_27_Z\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_26_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_24_Z\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_25_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[20]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[20]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[20]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_RNI256J6[4]\ : 
        ARI1_CC
      generic map(INIT => x"48800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy_Y[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[4]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[3]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[4]\, 
        CC => NET_CC_CONFIG669, P => NET_CC_CONFIG666, Y3 => 
        NET_CC_CONFIG667, Y3A => NET_CC_CONFIG668);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_12[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_4[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram82_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram83_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y1_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[23]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_810_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[24]\);
    
    \I_1/U0_RGB1_RGB3\ : RGB
      port map(A => \I_1/U0_Y\, EN => ADLIB_VCC, Y => 
        \AFLSDF_INV_6\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s_2638\ : 
        ARI1_CC
      generic map(INIT => x"40100")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/data_overrun_i_Z\, C
         => \can_top_level_inst/rx_buffer_inst/N_57\, D => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_Z\, 
        FCI => ADLIB_VCC, S => OPEN, Y => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s_2638_FCO\, 
        CC => NET_CC_CONFIG365, P => NET_CC_CONFIG362, Y3 => 
        NET_CC_CONFIG363, Y3A => NET_CC_CONFIG364);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_0_CC_2\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO525, CO => OPEN, P(0) => 
        NET_CC_CONFIG610, P(1) => NET_CC_CONFIG614, P(2) => 
        NET_CC_CONFIG618, P(3) => NET_CC_CONFIG622, P(4) => 
        NET_CC_CONFIG626, P(5) => NET_CC_CONFIG630, P(6) => 
        NET_CC_CONFIG634, P(7) => NET_CC_CONFIG638, P(8) => 
        NET_CC_CONFIG642, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, 
        P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG611, Y3(1) => 
        NET_CC_CONFIG615, Y3(2) => NET_CC_CONFIG619, Y3(3) => 
        NET_CC_CONFIG623, Y3(4) => NET_CC_CONFIG627, Y3(5) => 
        NET_CC_CONFIG631, Y3(6) => NET_CC_CONFIG635, Y3(7) => 
        NET_CC_CONFIG639, Y3(8) => NET_CC_CONFIG643, Y3(9) => 
        ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, 
        Y3A(0) => NET_CC_CONFIG612, Y3A(1) => NET_CC_CONFIG616, 
        Y3A(2) => NET_CC_CONFIG620, Y3A(3) => NET_CC_CONFIG624, 
        Y3A(4) => NET_CC_CONFIG628, Y3A(5) => NET_CC_CONFIG632, 
        Y3A(6) => NET_CC_CONFIG636, Y3A(7) => NET_CC_CONFIG640, 
        Y3A(8) => NET_CC_CONFIG644, Y3A(9) => ADLIB_VCC, Y3A(10)
         => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG613, CC(1) => NET_CC_CONFIG617, CC(2) => 
        NET_CC_CONFIG621, CC(3) => NET_CC_CONFIG625, CC(4) => 
        NET_CC_CONFIG629, CC(5) => NET_CC_CONFIG633, CC(6) => 
        NET_CC_CONFIG637, CC(7) => NET_CC_CONFIG641, CC(8) => 
        NET_CC_CONFIG645, CC(9) => nc226, CC(10) => nc146, CC(11)
         => nc230);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, D => 
        \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un7_txtb_parity_error_valid_i_1_0\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/N_3900\, 
        B => \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/txtb_index_muxed[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un7_txtb_parity_error_valid_i_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1[25]\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[25]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[25]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNITPEU_S[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_16[0]\ : 
        CFG4
      generic map(INIT => x"E4A0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_Z[0]\, 
        C => \can_top_level_inst/int_ena[0]\, D => \data_in_c[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1142\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\);
    
    \timestamp_ibuf[50]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[50]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[50]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\);
    
    \data_out_obuf[23]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(23), D => \data_out_obuf[23]/DOUT\, 
        E => \data_out_obuf[23]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[19]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[16]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/test_registers_out.tst_dest[19]\, D
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[19]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[2]\ : 
        CFG4
      generic map(INIT => x"FF40")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_178\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[1]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_N_3L4\ : 
        CFG4
      generic map(INIT => x"0888")

      port map(A => \can_top_level_inst/prescaler_inst/g1\, B => 
        \can_top_level_inst/prescaler_inst/g0_0_5\, C => 
        \can_top_level_inst/prescaler_inst/un6_phase_err_sjw_by_one_NE\, 
        D => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_N_3L4_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram117__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_349\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe117\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_9[21]\ : 
        CFG4
      generic map(INIT => x"E2EE")

      port map(A => \can_top_level_inst/stat_bus[76]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_139\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_190\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_943\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_40\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[94]\, B => 
        \can_top_level_inst/rec_ident[13]\, C => 
        \can_top_level_inst/drv_bus[127]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_41\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_6\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[7]\, B
         => \can_top_level_inst/can_core_inst/crc_21[6]\, C => 
        \can_top_level_inst/can_core_inst/crc_21[7]\, D => 
        \can_top_level_inst/store_data_word[6]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[2]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[3]\, 
        CC => NET_CC_CONFIG14, P => NET_CC_CONFIG11, Y3 => 
        NET_CC_CONFIG12, Y3A => NET_CC_CONFIG13);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\);
    
    \data_out_obuf[31]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(31), D => \data_out_obuf[31]/DOUT\, 
        E => \data_out_obuf[31]/EOUT\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_1[1]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_2\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[17]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q\ : 
        SLE
      port map(D => \can_top_level_inst/tran_valid\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram50_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe50\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram50_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m88\ : 
        CFG3
      generic map(INIT => x"53")

      port map(A => \can_top_level_inst/stat_bus[264]\, B => 
        \can_top_level_inst/drv_bus[23]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i75_mux\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un1_tq_counter_ce\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/dbt_ctrs_en\, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un2_tq_counter_allow\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un1_tq_counter_ce_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[7]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[7]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI1IBC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a3_0[1]\ : 
        CFG4
      generic map(INIT => x"0010")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        C => \can_top_level_inst/txtb_sw_cmd.set_ety\, D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_163\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_9\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[9]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_8_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_9_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_9_Z\, 
        CC => NET_CC_CONFIG1250, P => NET_CC_CONFIG1247, Y3 => 
        NET_CC_CONFIG1248, Y3A => NET_CC_CONFIG1249);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIMSM92_S[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[222]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/txtb_prorities_7[2]\, B
         => \can_top_level_inst/txtb_prorities_6[2]\, C => 
        \can_top_level_inst/drv_bus_475_rep2\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[23]\);
    
    \timestamp_ibuf[14]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(14), Y => 
        \timestamp_ibuf[14]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[2]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[14]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[14]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[19]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/current_state_fast_RNO[0]\ : 
        CFG4
      generic map(INIT => x"0028")

      port map(A => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        B => \can_top_level_inst/prescaler_inst/segment_end\, C
         => \can_top_level_inst/prescaler_inst/is_tseg2_fast\, D
         => \can_top_level_inst/prescaler_inst/current_state[1]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/N_33_i_fast\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[2]\ : 
        CFG4
      generic map(INIT => x"F088")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_RNIH1071_Z\, 
        B => \can_top_level_inst/tran_word[26]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o2_2[35]\ : 
        CFG2
      generic map(INIT => x"D")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/bds_data_out\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_973\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_is_rtr_com\ : SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_is_rtr_dbl_buf_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/tran_is_rtr\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNO[7]\ : 
        CFG4
      generic map(INIT => x"2023")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_149\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_151\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_124_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram104_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram105_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0[5]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        B => \can_top_level_inst/txtb_hw_cmd.failed\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/first_err_delim_q_RNO\ : 
        CFG4
      generic map(INIT => x"080A")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_9\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[8]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_980\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_936_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_869_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_29\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_20_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_19_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_18_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_17_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_29_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select[3]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20_1\, 
        B => \sbe_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2[6]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/NN_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/frame_count_ctr_proc.un10_read_increment_1\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/un10_read_increment_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[288]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[22]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[7]\ : 
        CFG4
      generic map(INIT => x"F808")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        B => \can_top_level_inst/stat_bus[156]\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        D => \can_top_level_inst/rx_buffer_inst/N_2285\, Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1\ : 
        RGB
      port map(A => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_Y\, 
        EN => ADLIB_VCC, Y => \AFLSDF_INV_7\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram21_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe21\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram21_0\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_0\ : 
        ARI1_CC
      generic map(INIT => x"553AC")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[0]\, 
        B => \can_top_level_inst/bus_sampling_inst/ssp_delay[0]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[0]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_0_Z\, 
        CC => NET_CC_CONFIG398, P => NET_CC_CONFIG395, Y3 => 
        NET_CC_CONFIG396, Y3A => NET_CC_CONFIG397);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_1[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram10_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram11_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[28]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[22]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[22]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[22]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dff_destuffed_flag_reg/reg_q_RNO\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_discard_stuff_bit_Z\, 
        B => \can_top_level_inst/can_core_inst/destuff_enable\, Y
         => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dff_destuffed_flag_reg/N_26_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO\ : 
        ARI1_CC
      generic map(INIT => x"69900")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/crc_17[16]\, C => 
        \can_top_level_inst/store_data_word[16]\, D => ADLIB_GND, 
        FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[7]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_FCNET1\, 
        CC => NET_CC_CONFIG2559, P => NET_CC_CONFIG2556, Y3 => 
        NET_CC_CONFIG2557, Y3A => NET_CC_CONFIG2558);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[10]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[10]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[10]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"553AC")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[1]\, 
        B => \can_top_level_inst/bus_sampling_inst/ssp_delay[1]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[1]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_0_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_1_Z\, 
        CC => NET_CC_CONFIG402, P => NET_CC_CONFIG399, Y3 => 
        NET_CC_CONFIG400, Y3A => NET_CC_CONFIG401);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_0[0]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_109_0\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_114\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_128\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[6]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[4]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_1\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[0]\, B => 
        \can_top_level_inst/drv_bus[205]\, C => 
        \can_top_level_inst/drv_bus[238]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_71\, 
        FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[0]\, 
        CC => NET_CC_CONFIG463, P => NET_CC_CONFIG460, Y3 => 
        NET_CC_CONFIG461, Y3A => NET_CC_CONFIG462);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[3]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[3]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[5]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[5]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[5]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_6_1_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_6_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[22]\, D => 
        \can_top_level_inst/rx_write_pointer[6]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1360\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[20]\);
    
    \data_in_ibuf[20]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(20), Y => \data_in_ibuf[20]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_0[6]\ : 
        CFG4
      generic map(INIT => x"F7FF")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[0]\, B
         => \can_top_level_inst/stat_bus[68]\, C => 
        \can_top_level_inst/txtb_hw_cmd_index[1]\, D => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_146\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[5]\ : 
        CFG4
      generic map(INIT => x"FEEE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_162\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_164\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\);
    
    \data_out_obuf[21]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(21), D => \data_out_obuf[21]/DOUT\, 
        E => \data_out_obuf[21]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[6]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_0\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[0]\, B => 
        \can_top_level_inst/drv_bus[300]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_0\, 
        CC => NET_CC_CONFIG529, P => NET_CC_CONFIG526, Y3 => 
        NET_CC_CONFIG527, Y3A => NET_CC_CONFIG528);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_26\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_21_Z\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_11_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_26_Z\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s_2640_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG1450, P(1) => NET_CC_CONFIG1454, P(2) => 
        NET_CC_CONFIG1458, P(3) => NET_CC_CONFIG1462, P(4) => 
        NET_CC_CONFIG1466, P(5) => NET_CC_CONFIG1470, P(6) => 
        NET_CC_CONFIG1474, P(7) => NET_CC_CONFIG1478, P(8) => 
        ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11)
         => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1451, Y3(1) => 
        NET_CC_CONFIG1455, Y3(2) => NET_CC_CONFIG1459, Y3(3) => 
        NET_CC_CONFIG1463, Y3(4) => NET_CC_CONFIG1467, Y3(5) => 
        NET_CC_CONFIG1471, Y3(6) => NET_CC_CONFIG1475, Y3(7) => 
        NET_CC_CONFIG1479, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG1452, Y3A(1) => NET_CC_CONFIG1456, Y3A(2)
         => NET_CC_CONFIG1460, Y3A(3) => NET_CC_CONFIG1464, 
        Y3A(4) => NET_CC_CONFIG1468, Y3A(5) => NET_CC_CONFIG1472, 
        Y3A(6) => NET_CC_CONFIG1476, Y3A(7) => NET_CC_CONFIG1480, 
        Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => 
        ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG1453, CC(1) => NET_CC_CONFIG1457, CC(2) => 
        NET_CC_CONFIG1461, CC(3) => NET_CC_CONFIG1465, CC(4) => 
        NET_CC_CONFIG1469, CC(5) => NET_CC_CONFIG1473, CC(6) => 
        NET_CC_CONFIG1477, CC(7) => NET_CC_CONFIG1481, CC(8) => 
        nc89, CC(9) => nc119, CC(10) => nc48, CC(11) => nc271);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[8]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dff_err_reg/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_stuff_err_d_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/stuff_err\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_1[1]\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \adress_c[2]\, B => \adress_c[4]\, C => 
        \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/addr_dec_3[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_9\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_9\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_8_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_9_FCNET1\, 
        CC => NET_CC_CONFIG163, P => NET_CC_CONFIG160, Y3 => 
        NET_CC_CONFIG161, Y3A => NET_CC_CONFIG162);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[20]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[20]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram125__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_347\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe125\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_9_RNO\ : 
        ARI1_CC
      generic map(INIT => x"45500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[9]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_8\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_9\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG2215, P => 
        NET_CC_CONFIG2212, Y3 => NET_CC_CONFIG2213, Y3A => 
        NET_CC_CONFIG2214);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m30_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\);
    
    \data_in_ibuf[18]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[18]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[18]/YIN\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_4[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_26_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_1[3]\ : 
        CFG4
      generic map(INIT => x"0EFF")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_930\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_926\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_1_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNO[7]\ : 
        CFG4
      generic map(INIT => x"080B")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_149\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_151\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_124_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_ctr_proc.same_bits_q_3_iv[0]\ : 
        CFG4
      generic map(INIT => x"CDCE")

      port map(A => \can_top_level_inst/tx_trigger\, B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un1_bst_ctr_q3_Z\, 
        C => \can_top_level_inst/can_core_inst/tx_frame_no_sof\, 
        D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_18\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_18_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_38_1_0_wmux_RNO[17]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_2_0_Z[17]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_2_Z[17]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_797\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[25]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[25]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_1[20]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[196]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_1_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[5]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIB9FT[0]_CC_3\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO923, CO => OPEN, P(0) => 
        NET_CC_CONFIG1032, P(1) => NET_CC_CONFIG1036, P(2) => 
        NET_CC_CONFIG1040, P(3) => NET_CC_CONFIG1044, P(4) => 
        NET_CC_CONFIG1048, P(5) => ADLIB_VCC, P(6) => ADLIB_VCC, 
        P(7) => ADLIB_VCC, P(8) => ADLIB_VCC, P(9) => ADLIB_VCC, 
        P(10) => ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => 
        NET_CC_CONFIG1033, Y3(1) => NET_CC_CONFIG1037, Y3(2) => 
        NET_CC_CONFIG1041, Y3(3) => NET_CC_CONFIG1045, Y3(4) => 
        NET_CC_CONFIG1049, Y3(5) => ADLIB_VCC, Y3(6) => ADLIB_VCC, 
        Y3(7) => ADLIB_VCC, Y3(8) => ADLIB_VCC, Y3(9) => 
        ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, 
        Y3A(0) => NET_CC_CONFIG1034, Y3A(1) => NET_CC_CONFIG1038, 
        Y3A(2) => NET_CC_CONFIG1042, Y3A(3) => NET_CC_CONFIG1046, 
        Y3A(4) => NET_CC_CONFIG1050, Y3A(5) => ADLIB_VCC, Y3A(6)
         => ADLIB_VCC, Y3A(7) => ADLIB_VCC, Y3A(8) => ADLIB_VCC, 
        Y3A(9) => ADLIB_VCC, Y3A(10) => ADLIB_VCC, Y3A(11) => 
        ADLIB_VCC, CC(0) => NET_CC_CONFIG1035, CC(1) => 
        NET_CC_CONFIG1039, CC(2) => NET_CC_CONFIG1043, CC(3) => 
        NET_CC_CONFIG1047, CC(4) => NET_CC_CONFIG1051, CC(5) => 
        nc213, CC(6) => nc300, CC(7) => nc126, CC(8) => nc195, 
        CC(9) => nc188, CC(10) => nc242, CC(11) => nc15);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[6]\ : 
        CFG3
      generic map(INIT => x"D0")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[3]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_176_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/int_mask_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/int_mask_load_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_mask[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[17]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[2]\);
    
    \can_top_level_inst/int_manager_inst/int_input_active_0[11]\ : 
        CFG4
      generic map(INIT => x"CCEC")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_172\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_0_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_nbt_Z[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_3_S\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[22]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[260]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[28]\ : 
        CFG4
      generic map(INIT => x"F0E0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_16_Z[28]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[892]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[24]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_15_Z[28]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_5[17]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/drv_bus[33]\, B => 
        \can_top_level_inst/rx_read_pointer[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_787\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2248\, B
         => \can_top_level_inst/rx_buffer_inst/N_4812\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2281\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_0_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[0]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\);
    
    \can_top_level_inst/rx_buffer_inst/rx_full_1_5\ : CFG4
      generic map(INIT => x"0001")

      port map(A => \can_top_level_inst/rx_mem_free[3]\, B => 
        \can_top_level_inst/rx_mem_free[2]\, C => 
        \can_top_level_inst/rx_mem_free[1]\, D => 
        \can_top_level_inst/rx_mem_free[0]\, Y => 
        \can_top_level_inst/rx_buffer_inst_rx_full_1_5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_138\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/txtb_ptr[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[8]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[6]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[6]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[13]\);
    
    \timestamp_ibuf[38]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[38]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[38]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram51_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe51\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram51_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1[22]\ : 
        CFG4
      generic map(INIT => x"A989")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_1_Z[22]\, 
        D => \can_top_level_inst/drv_bus[11]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[14]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[14]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[13]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[13]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIC47PS3[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[50]\, B => 
        \can_top_level_inst/tran_word[18]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_17\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_18\, 
        CC => NET_CC_CONFIG999, P => NET_CC_CONFIG996, Y3 => 
        NET_CC_CONFIG997, Y3A => NET_CC_CONFIG998);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_RNO[8]\ : 
        ARI1_CC
      generic map(INIT => x"48800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy_Y[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[8]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[7]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[8]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG685, P => 
        NET_CC_CONFIG682, Y3 => NET_CC_CONFIG683, Y3A => 
        NET_CC_CONFIG684);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa4\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe4\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\);
    
    \timestamp_ibuf[25]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[25]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[25]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[22]\);
    
    \timestamp_ibuf[18]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(18), Y => 
        \timestamp_ibuf[18]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[9]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram70__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_353\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe70\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[17]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[8]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[8]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[8]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[2]\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa18\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe18\);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_ns_1_0_.m8\ : 
        CFG4
      generic map(INIT => x"4F4C")

      port map(A => \can_top_level_inst/can_core_inst/set_idle\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_set_transmitter\, 
        C => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/operation_control_inst/un2_go_to_off_i_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/operation_control_inst/N_9\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[27]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2651\, Y
         => \can_top_level_inst/memory_registers_inst/N_2716\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\);
    
    \adress_ibuf[10]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \adress_c[10]\, E => ADLIB_GND, YIN => 
        \adress_ibuf[10]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[119]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/dff_halt_reg/reg_q\ : 
        SLE
      port map(D => \can_top_level_inst/can_core_inst/data_halt\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_data_halt_q\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/alc_Z[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/stat_bus[6]\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[289]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[31]\);
    
    \data_out_obuf[23]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[23]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[23]/DOUT\, EOUT => 
        \data_out_obuf[23]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[23]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[261]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2[27]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/tran_identifier[13]\, B
         => \can_top_level_inst/tran_identifier[24]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0[10]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => \can_top_level_inst/drv_bus[477]\, B => 
        \can_top_level_inst/tran_frame_test.tprm[1]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc3\, 
        D => \can_top_level_inst/tran_frame_test.tprm[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[23]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[23]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[23]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_dbt_Z[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_1_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[1]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_139\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_160\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0[3]\ : 
        CFG4
      generic map(INIT => x"0010")

      port map(A => \can_top_level_inst/txtb_hw_cmd.failed\, B
         => \can_top_level_inst/N_4606\, C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        D => \can_top_level_inst/tran_valid\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[30]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[23]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un4_txtb_index_muxed_i[2]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[2]\, B
         => \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[2]\, 
        Y => \can_top_level_inst/txtb_index_muxed[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[20]\ : 
        CFG4
      generic map(INIT => x"EC00")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[20]\, B
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[125]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c2_RNIVV371\ : 
        CFG4
      generic map(INIT => x"9699")

      port map(A => \can_top_level_inst/drv_bus[3]\, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[3]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c2_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[2]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/N_3910\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_2_1.CO2_1\ : 
        CFG4
      generic map(INIT => x"7150")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_Z[1]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_Z[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2_Z[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2_Z[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO2_1_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[16]\ : 
        CFG4
      generic map(INIT => x"3100")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1024\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_812_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[20]\);
    
    \timestamp_ibuf[55]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[55]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[55]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[8]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[8]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[8]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram72_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram73_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[184]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[16]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[16]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[15]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_30_1_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_30_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_807\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_810\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_812\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_0_y3[15]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_0_y1[15]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_737\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => \can_top_level_inst/drv_bus[476]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_m1_e_1\, 
        D => \can_top_level_inst/txtb_parity_check_valid\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/txtb_parity_error_valid[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[250]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[8]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_34\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[108]\, B => 
        \can_top_level_inst/rec_ident[27]\, C => 
        \can_top_level_inst/drv_bus[141]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_46\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_2[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[13]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[13]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram87__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_345\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe87\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[17]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d[0]\ : 
        CFG4
      generic map(INIT => x"BBB8")

      port map(A => \can_top_level_inst/drv_bus[416]\, B => 
        \can_top_level_inst/drv_bus[426]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[0]\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_16_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_7\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[407]\, B => 
        \can_top_level_inst/stat_bus[78]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_6_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_7_Z\, 
        CC => NET_CC_CONFIG240, P => NET_CC_CONFIG237, Y3 => 
        NET_CC_CONFIG238, Y3A => NET_CC_CONFIG239);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[17]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/int_stat_proc.un2_int_status_set\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/int_mask[10]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/un2_int_status_set\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe8\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_124_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[13]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_control[1]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[20]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[291]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_1_Z[0]\, 
        B => \sbe_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz[3]\ : 
        CFG4
      generic map(INIT => x"FF04")

      port map(A => \can_top_level_inst/N_4606\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\);
    
    \data_out_obuf[10]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[10]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[10]/DOUT\, EOUT => 
        \data_out_obuf[10]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry[1]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[1]\, 
        D => \can_top_level_inst/rx_write_pointer[1]\, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[0]\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[1]\, 
        CC => NET_CC_CONFIG373, P => NET_CC_CONFIG370, Y3 => 
        NET_CC_CONFIG371, Y3A => NET_CC_CONFIG372);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask.un7_flip_mask_c3_RNIHAD21\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[4]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[3]\, C => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_c3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_20_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[8]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[7]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[1]\, 
        D => \can_top_level_inst/tran_word[16]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[26]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[21]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[26]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[169]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[22]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[103]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[1]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_148\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG646, P(1) => NET_CC_CONFIG650, P(2) => 
        NET_CC_CONFIG654, P(3) => NET_CC_CONFIG658, P(4) => 
        NET_CC_CONFIG662, P(5) => NET_CC_CONFIG666, P(6) => 
        NET_CC_CONFIG670, P(7) => NET_CC_CONFIG674, P(8) => 
        NET_CC_CONFIG678, P(9) => NET_CC_CONFIG682, P(10) => 
        ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG647, 
        Y3(1) => NET_CC_CONFIG651, Y3(2) => NET_CC_CONFIG655, 
        Y3(3) => NET_CC_CONFIG659, Y3(4) => NET_CC_CONFIG663, 
        Y3(5) => NET_CC_CONFIG667, Y3(6) => NET_CC_CONFIG671, 
        Y3(7) => NET_CC_CONFIG675, Y3(8) => NET_CC_CONFIG679, 
        Y3(9) => NET_CC_CONFIG683, Y3(10) => ADLIB_VCC, Y3(11)
         => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG648, Y3A(1) => 
        NET_CC_CONFIG652, Y3A(2) => NET_CC_CONFIG656, Y3A(3) => 
        NET_CC_CONFIG660, Y3A(4) => NET_CC_CONFIG664, Y3A(5) => 
        NET_CC_CONFIG668, Y3A(6) => NET_CC_CONFIG672, Y3A(7) => 
        NET_CC_CONFIG676, Y3A(8) => NET_CC_CONFIG680, Y3A(9) => 
        NET_CC_CONFIG684, Y3A(10) => ADLIB_VCC, Y3A(11) => 
        ADLIB_VCC, CC(0) => NET_CC_CONFIG649, CC(1) => 
        NET_CC_CONFIG653, CC(2) => NET_CC_CONFIG657, CC(3) => 
        NET_CC_CONFIG661, CC(4) => NET_CC_CONFIG665, CC(5) => 
        NET_CC_CONFIG669, CC(6) => NET_CC_CONFIG673, CC(7) => 
        NET_CC_CONFIG677, CC(8) => NET_CC_CONFIG681, CC(9) => 
        NET_CC_CONFIG685, CC(10) => nc308, CC(11) => nc236);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_0_1[1]\ : 
        CFG4
      generic map(INIT => x"2637")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/drv_bus[239]\, D => 
        \can_top_level_inst/drv_bus[115]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_0_1_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_11[26]\ : 
        CFG4
      generic map(INIT => x"FFF8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac16_Z\, 
        B => \can_top_level_inst/drv_bus[326]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_6_Z[26]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[666]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_11_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_19\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[27]\, B
         => \can_top_level_inst/rx_buffer_inst_RAM_data_out[26]\, 
        C => \can_top_level_inst/rx_buffer_inst_RAM_data_out[25]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_19_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[12]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_ext_id_0_a4\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_950\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[30]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m19_1_0_wmux_0\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m19_1_0_y0\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i79_mux\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i81_mux\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_20_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[28]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[28]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[28]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_set_idle_0_a4_1\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_941\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_set_idle_0_a4_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_14[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_5[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram86_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram87_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y3_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_i[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_write_pointer[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m29_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[24]\);
    
    \can_top_level_inst/memory_registers_inst/ctr_pres_sel_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/ctr_pres[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => res_n_out_c, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD
         => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/drv_bus[425]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[251]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_17_0_a4\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1304\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_17_0_a4_0_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1239\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_962_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1186_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_19\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_19_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[24]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_can_fd_1_sqmuxa_RNIDGQL_1\ : 
        CFG4
      generic map(INIT => x"2800")

      port map(A => \can_top_level_inst/tran_dlc[2]\, B => 
        \can_top_level_inst/tran_dlc[1]\, C => 
        \can_top_level_inst/tran_dlc[0]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_can_fd_1_sqmuxa_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.un1_is_transmitter\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/stat_bus[66]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_is_transmitter\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_38_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_911\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_926\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_38_1_0_y0[20]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[4]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[4]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[14]\ : 
        CFG4
      generic map(INIT => x"E020")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_46_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        D => \can_top_level_inst/tran_word[22]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[14]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_m_3_1_0_wmux_0_RNI8VH11\ : 
        CFG3
      generic map(INIT => x"32")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_09\, B => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_19\, C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[18]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[23]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[323]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram49__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_343\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe49\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_s_7\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_mem_free[7]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_6_Z\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_s_7_S\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG2446, P => 
        NET_CC_CONFIG2443, Y3 => NET_CC_CONFIG2444, Y3A => 
        NET_CC_CONFIG2445);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[6]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_182\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[22]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0_0\ : 
        CFG3
      generic map(INIT => x"2A")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[4]\, C => 
        \can_top_level_inst/txtb_sw_cmd.set_abt_rep1\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram72_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe72\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram72_0\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[31]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[31]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_22\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_22_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[24]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_36[22]\ : 
        CFG4
      generic map(INIT => x"88D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[54]\, C => 
        \can_top_level_inst/stat_bus[242]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1008\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[14]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[403]\, B => 
        \can_top_level_inst/stat_bus[74]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_2_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_3_Z\, 
        CC => NET_CC_CONFIG224, P => NET_CC_CONFIG221, Y3 => 
        NET_CC_CONFIG222, Y3A => NET_CC_CONFIG223);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_m2_0_a2_1\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => \can_top_level_inst/drv_bus_475_rep2\, B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_un1_txtb_available_i_0_out_tz\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_m2_0_a2_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNISMQGE_S[20]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[240]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m16_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[12]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[12]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[12]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[20]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[20]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_7\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_7\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_6_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_7_Z\, 
        CC => NET_CC_CONFIG155, P => NET_CC_CONFIG152, Y3 => 
        NET_CC_CONFIG153, Y3A => NET_CC_CONFIG154);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI8D4E8[19]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[19]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[19]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_18\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_19\, 
        CC => NET_CC_CONFIG1353, P => NET_CC_CONFIG1350, Y3 => 
        NET_CC_CONFIG1351, Y3A => NET_CC_CONFIG1352);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_18\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_18_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_2[1]\, B
         => \can_top_level_inst/txtb_prorities_3[1]\, C => 
        \can_top_level_inst/drv_bus_fast[475]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[16]\);
    
    \data_in_ibuf[27]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[27]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[27]/YIN\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_0_4\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_0_2_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_9[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[10]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_0_4_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_58\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[230]\, B => 
        \can_top_level_inst/rec_ident[25]\, C => 
        \can_top_level_inst/drv_bus[263]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_26\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2_0[1]\ : 
        CFG4
      generic map(INIT => x"888C")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1230\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2_0_1_Z[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\, 
        D => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1244\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm_0[4]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[4]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val[4]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[0]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_124_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => swr_c, B => \sbe_c[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_1[0]\, 
        D => \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/wr_select_Z[0]\);
    
    \adress_ibuf[4]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \adress_c[4]\, E => ADLIB_GND, YIN => 
        \adress_ibuf[4]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram113__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_343\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe113\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_1_2_Z[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_1_1_Z[1]\, 
        D => \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[3]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_15[2]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[23]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[23]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        D => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/N_4154\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIGCJC1_S[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[258]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[16]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIC2DE6[6]\ : 
        ARI1_CC
      generic map(INIT => x"5D827")

      port map(A => \timestamp_c[38]\, B => 
        \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[6]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[6]\, 
        FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_5\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_6\, 
        CC => NET_CC_CONFIG951, P => NET_CC_CONFIG948, Y3 => 
        NET_CC_CONFIG949, Y3A => NET_CC_CONFIG950);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_25[18]\ : 
        CFG4
      generic map(INIT => x"C0A0")

      port map(A => \can_top_level_inst/drv_bus[318]\, B => 
        \can_top_level_inst/txtb_prorities_4[2]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_845\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[238]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[30]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIMSM92_S[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[190]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0MKOK_S[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[250]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm_0[3]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[3]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[30]\ : 
        SLE
      port map(D => \data_in_c[30]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[405]\, B => 
        \can_top_level_inst/stat_bus[76]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_4_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_5_Z\, 
        CC => NET_CC_CONFIG232, P => NET_CC_CONFIG229, Y3 => 
        NET_CC_CONFIG230, Y3A => NET_CC_CONFIG231);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_0[14]\ : 
        CFG4
      generic map(INIT => x"286C")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_1_0_Z[14]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_1_0_1_Z[14]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_0_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_28[20]\ : 
        CFG4
      generic map(INIT => x"AC00")

      port map(A => \can_top_level_inst/stat_bus[293]\, B => 
        \can_top_level_inst/drv_bus[101]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_924\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_14[0]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_14_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[151]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2[21]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/tran_identifier[7]\, B
         => \can_top_level_inst/tran_identifier[18]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_m2_2_0\ : 
        CFG3
      generic map(INIT => x"A2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_sm0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_m2_2_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[11]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[247]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un3_segment_end_i_0_1_N_2L1_RNI7UT6B2\ : 
        CFG4
      generic map(INIT => x"FEFC")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_nbt_tq_active\, 
        B => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/N_4\, 
        C => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g1_0\, 
        D => 
        \can_top_level_inst/prescaler_inst/exit_segm_req_nbt_1\, 
        Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segment_end_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[31]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe2_0_a4\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[0]\, 
        B => \can_top_level_inst/rx_buffer_inst/un5_ram_write_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\);
    
    
        \can_top_level_inst/int_manager_inst/dff_int_output_reg/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst/dff_int_output_reg/un1_int_i_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => int_c);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_828_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_2\ : 
        CFG4
      generic map(INIT => x"F2F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        B => \can_top_level_inst/drv_bus[460]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_0_sqmuxa\, 
        D => \can_top_level_inst/can_core_inst/bds_data_out\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_1[2]\ : 
        CFG4
      generic map(INIT => x"FFCE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_Z[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_drv_rom_ena_m\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_913_i\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/next_state_16_sqmuxa\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[4]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2249\, B
         => \can_top_level_inst/rx_buffer_inst/N_4813\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[1]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2459\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[3]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/txtb_prorities_5[2]\, B
         => \can_top_level_inst/txtb_prorities_4[2]\, C => 
        \can_top_level_inst/drv_bus_475_rep2\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_enable_set_i_o4\ : 
        CFG4
      generic map(INIT => x"FEFC")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1268\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1330\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_945\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1003\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram98_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe98\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram98_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[17]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[17]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[17]\);
    
    \data_in_ibuf[11]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(11), Y => \data_in_ibuf[11]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_1_0[3]\ : 
        CFG3
      generic map(INIT => x"1B")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_1[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/next_state_17_sqmuxa_0_a4_RNII9IR\ : 
        CFG3
      generic map(INIT => x"8A")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/next_state_17_sqmuxa\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_length_i_m\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram80_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe80\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram80_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[9]\ : 
        CFG3
      generic map(INIT => x"45")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[28]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1269\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_826_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[15]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy[0]\ : 
        ARI1_CC
      generic map(INIT => x"4EE00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_966\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => ADLIB_GND, FCI => ADLIB_VCC, S => OPEN, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy_Y[0]\, 
        FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy\, 
        CC => NET_CC_CONFIG649, P => NET_CC_CONFIG646, Y3 => 
        NET_CC_CONFIG647, Y3A => NET_CC_CONFIG648);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram4__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_353\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe4\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram4__RNO[0]\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[2]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[1]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe4\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[11]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[11]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[11]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_2[27]\ : 
        CFG4
      generic map(INIT => x"FFF8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5_Z\, 
        B => \can_top_level_inst/stat_bus[215]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_0_Z[27]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[1211]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_2_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[4]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_133\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[26]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[26]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[26]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[28]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un6_sspc_ena_d\ : 
        CFG4
      generic map(INIT => x"EAFF")

      port map(A => \can_top_level_inst/stat_bus[10]\, B => 
        \can_top_level_inst/gen_first_ssp\, C => 
        \can_top_level_inst/tx_trigger\, D => 
        \can_top_level_inst/stat_bus[11]\, Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un6_sspc_ena_d_Z\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_a3[3]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        B => \can_top_level_inst/txtb_allow_bb[2]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        Y => \can_top_level_inst/txtb_state_2[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[31]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2489\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[9]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[9]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_75\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[9]\, B => 
        \timestamp_c[40]\, C => \timestamp_c[41]\, D => 
        \can_top_level_inst/tran_word[8]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[3]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[4]\, 
        CC => NET_CC_CONFIG2269, P => NET_CC_CONFIG2266, Y3 => 
        NET_CC_CONFIG2267, Y3A => NET_CC_CONFIG2268);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[22]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[22]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[22]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2646\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram24__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_340\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe24\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry_cy[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG1088, P(1) => NET_CC_CONFIG1092, P(2) => 
        NET_CC_CONFIG1096, P(3) => NET_CC_CONFIG1100, P(4) => 
        NET_CC_CONFIG1104, P(5) => NET_CC_CONFIG1108, P(6) => 
        NET_CC_CONFIG1112, P(7) => NET_CC_CONFIG1116, P(8) => 
        ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11)
         => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1089, Y3(1) => 
        NET_CC_CONFIG1093, Y3(2) => NET_CC_CONFIG1097, Y3(3) => 
        NET_CC_CONFIG1101, Y3(4) => NET_CC_CONFIG1105, Y3(5) => 
        NET_CC_CONFIG1109, Y3(6) => NET_CC_CONFIG1113, Y3(7) => 
        NET_CC_CONFIG1117, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG1090, Y3A(1) => NET_CC_CONFIG1094, Y3A(2)
         => NET_CC_CONFIG1098, Y3A(3) => NET_CC_CONFIG1102, 
        Y3A(4) => NET_CC_CONFIG1106, Y3A(5) => NET_CC_CONFIG1110, 
        Y3A(6) => NET_CC_CONFIG1114, Y3A(7) => NET_CC_CONFIG1118, 
        Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => 
        ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG1091, CC(1) => NET_CC_CONFIG1095, CC(2) => 
        NET_CC_CONFIG1099, CC(3) => NET_CC_CONFIG1103, CC(4) => 
        NET_CC_CONFIG1107, CC(5) => NET_CC_CONFIG1111, CC(6) => 
        NET_CC_CONFIG1115, CC(7) => NET_CC_CONFIG1119, CC(8) => 
        nc102, CC(9) => nc304, CC(10) => nc3, CC(11) => nc207);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un2_commit_rx_frame\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_mem_free[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[28]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[28]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[28]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2652\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid_2\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_valid\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[5]\ : 
        CFG4
      generic map(INIT => x"ACA0")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[5]\, 
        B => \can_top_level_inst/can_core_inst/crc_17[1]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m29_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[9]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[3]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1_1[0]\ : 
        CFG3
      generic map(INIT => x"46")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/int_vector[0]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1_1_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_0_y0[12]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[188]\, D => 
        \can_top_level_inst/drv_bus[217]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_0_y1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_1[1]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[6]\, 
        B => \can_top_level_inst/stat_bus[68]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_137\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[5]\ : 
        CFG4
      generic map(INIT => x"FF08")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_170_1\, 
        B => \can_top_level_inst/txtb_hw_cmd.failed\, C => 
        \can_top_level_inst/txtb_parity_error_valid[5]\, D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\);
    
    \can_top_level_inst/rx_buffer_inst/rx_read_buff[3]\ : CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/rx_buffer_inst_RAM_data_out[3]\, 
        Y => \can_top_level_inst/rx_read_buff[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_frm_req\ : 
        SLE
      port map(D => \can_top_level_inst/err_detected\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\);
    
    \timestamp_ibuf[30]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(30), Y => 
        \timestamp_ibuf[30]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[2]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram5__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_353\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[24]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[229]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[15]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_11[2]\ : 
        CFG4
      generic map(INIT => x"F0E0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1213\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_1[17]\ : 
        CFG4
      generic map(INIT => x"0B08")

      port map(A => \can_top_level_inst/tran_identifier[3]\, B
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[17]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_1[20]\ : 
        CFG4
      generic map(INIT => x"0B08")

      port map(A => \can_top_level_inst/tran_identifier[6]\, B
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[20]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_flag\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1533_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_ack_err_flag_clr\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_flag_Z\);
    
    \timestamp_ibuf[39]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[39]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[39]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[6]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_meas_progress_del\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_meas_progress_q_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_meas_progress_del_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[12]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_7\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_6_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_7_Z\, 
        CC => NET_CC_CONFIG2406, P => NET_CC_CONFIG2403, Y3 => 
        NET_CC_CONFIG2404, Y3A => NET_CC_CONFIG2405);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2246\, B
         => \can_top_level_inst/rx_buffer_inst/N_4261\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2279\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[25]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[106]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_RNI6RRU[2]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_1\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_2\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_2\, 
        CC => NET_CC_CONFIG260, P => NET_CC_CONFIG257, Y3 => 
        NET_CC_CONFIG258, Y3A => NET_CC_CONFIG259);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[2]\ : 
        CFG4
      generic map(INIT => x"FF20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_178\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[2]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, B => 
        \can_top_level_inst/prescaler_inst/segment_end\, C => 
        \can_top_level_inst/prescaler_inst/start_edge\, D => 
        \can_top_level_inst/prescaler_inst/resync_edge_valid\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m127_2_1_0\ : 
        CFG4
      generic map(INIT => x"4657")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[243]\, D => 
        \can_top_level_inst/drv_bus[119]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m127_2_1_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[7]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => \can_top_level_inst/txtb_allow_bb[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4QRQ3_S[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[224]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[9]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[9]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[9]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2633\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[14]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[22]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[22]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram99_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe99\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram99_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[6]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIOVDQ9[24]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[24]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[24]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_23\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_24\, 
        CC => NET_CC_CONFIG1373, P => NET_CC_CONFIG1370, Y3 => 
        NET_CC_CONFIG1371, Y3A => NET_CC_CONFIG1372);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[1]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[1]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIKPVM8[20]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[20]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[20]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_19\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_20\, 
        CC => NET_CC_CONFIG1357, P => NET_CC_CONFIG1354, Y3 => 
        NET_CC_CONFIG1355, Y3A => NET_CC_CONFIG1356);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[26]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[26]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[26]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[8]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[8]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[8]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[26]\);
    
    AFLSDF_INV_2 : INV_BA
      port map(A => \AFLSDF_INV_2\, Y => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_RNO_6[28]\ : 
        CFG3
      generic map(INIT => x"53")

      port map(A => \can_top_level_inst/tran_identifier[25]\, B
         => \can_top_level_inst/tran_identifier[14]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_24\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram0_[0]\ : 
        SLE
      port map(D => can_tx_c, CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, 
        EN => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe0\, 
        ALn => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn
         => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un4_txtb_port_b_address[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_ptr[1]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q_Z[1]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, Y
         => \can_top_level_inst/txtb_port_b_address[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[27]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_10\ : 
        ARI1_CC
      generic map(INIT => x"56633")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[10]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[10]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_9_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_10_Z\, 
        CC => NET_CC_CONFIG438, P => NET_CC_CONFIG435, Y3 => 
        NET_CC_CONFIG436, Y3A => NET_CC_CONFIG437);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[8]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[5]\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[5]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_Z[5]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_21\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_21_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[8]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\);
    
    \data_out_obuf[28]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[28]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[28]/DOUT\, EOUT => 
        \data_out_obuf[28]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[29]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[29]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[29]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[17]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[17]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[17]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4821\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram81_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe81\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram81_0\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_m_3_1_0_wmux_0\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_m_3_1_0_y0\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_i\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner1\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner3\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_i\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI79P86_S[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[281]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[2]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI2KAP[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[4]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[4]\, 
        Y => \can_top_level_inst/tran_word[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNID0CK[32]\ : 
        CFG3
      generic map(INIT => x"32")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[32]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_763_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2\ : 
        RGB
      port map(A => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_Y\, 
        EN => ADLIB_VCC, Y => \AFLSDF_INV_8\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[30]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/un22_txtb_available_i\ : 
        CFG3
      generic map(INIT => x"B3")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_129\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[4]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_shorten_tseg1_after_tseg2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/resync_edge_valid\, C
         => \can_top_level_inst/prescaler_inst/is_tseg2\, D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_shorten_tseg1_after_tseg2_i\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_27[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/stat_bus[3]\, B => 
        \can_top_level_inst/drv_bus[90]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1495\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[61]\ : 
        SLE
      port map(D => \timestamp_c[61]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[61]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIMNFH9_S[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[232]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        B => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[18]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[4]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un7_txtb_parity_error_valid_i_0\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/N_3898\, 
        B => \can_top_level_inst/drv_bus[476]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un7_txtb_parity_error_valid_i_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[29]\ : 
        CFG4
      generic map(INIT => x"2888")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[18]\, 
        C => \can_top_level_inst/tran_frame_test.fcrc\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[18]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[10]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_x2[30]\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_data_nbs_prev_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_936_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIU4MKH_S[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[213]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_s_0_2637_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => ADLIB_GND, 
        P(1) => NET_CC_CONFIG881, P(2) => NET_CC_CONFIG885, P(3)
         => NET_CC_CONFIG889, P(4) => NET_CC_CONFIG893, P(5) => 
        NET_CC_CONFIG897, P(6) => NET_CC_CONFIG901, P(7) => 
        NET_CC_CONFIG905, P(8) => NET_CC_CONFIG909, P(9) => 
        NET_CC_CONFIG913, P(10) => NET_CC_CONFIG917, P(11) => 
        ADLIB_VCC, Y3(0) => ADLIB_GND, Y3(1) => NET_CC_CONFIG882, 
        Y3(2) => NET_CC_CONFIG886, Y3(3) => NET_CC_CONFIG890, 
        Y3(4) => NET_CC_CONFIG894, Y3(5) => NET_CC_CONFIG898, 
        Y3(6) => NET_CC_CONFIG902, Y3(7) => NET_CC_CONFIG906, 
        Y3(8) => NET_CC_CONFIG910, Y3(9) => NET_CC_CONFIG914, 
        Y3(10) => NET_CC_CONFIG918, Y3(11) => ADLIB_VCC, Y3A(0)
         => ADLIB_GND, Y3A(1) => NET_CC_CONFIG883, Y3A(2) => 
        NET_CC_CONFIG887, Y3A(3) => NET_CC_CONFIG891, Y3A(4) => 
        NET_CC_CONFIG895, Y3A(5) => NET_CC_CONFIG899, Y3A(6) => 
        NET_CC_CONFIG903, Y3A(7) => NET_CC_CONFIG907, Y3A(8) => 
        NET_CC_CONFIG911, Y3A(9) => NET_CC_CONFIG915, Y3A(10) => 
        NET_CC_CONFIG919, Y3A(11) => ADLIB_VCC, CC(0) => nc47, 
        CC(1) => NET_CC_CONFIG884, CC(2) => NET_CC_CONFIG888, 
        CC(3) => NET_CC_CONFIG892, CC(4) => NET_CC_CONFIG896, 
        CC(5) => NET_CC_CONFIG900, CC(6) => NET_CC_CONFIG904, 
        CC(7) => NET_CC_CONFIG908, CC(8) => NET_CC_CONFIG912, 
        CC(9) => NET_CC_CONFIG916, CC(10) => NET_CC_CONFIG920, 
        CC(11) => nc90);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m30_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_control[1]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[29]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_19\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_19_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[25]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2483\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_1_1[14]\ : 
        CFG3
      generic map(INIT => x"27")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[252]\, C => 
        \can_top_level_inst/drv_bus[128]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_1_1_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_a3[3]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        Y => \can_top_level_inst/txtb_state_5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un7_txtb_parity_error_valid_i\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \can_top_level_inst/drv_bus[476]\, B => 
        \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un7_txtb_parity_error_valid_i_1\, 
        D => \can_top_level_inst/txtb_parity_check_valid\, Y => 
        \can_top_level_inst/txtb_parity_error_valid[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[19]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[11]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_m_3_1_0_wmux_0_RNIOGPC4\ : 
        CFG4
      generic map(INIT => x"0B07")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_i\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_i3_mux\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/select_buf_index_i_0_i_1[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_12[8]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => \can_top_level_inst/drv_bus[408]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/drv_bus[89]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1442\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_830_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNII94QU7[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_port_b_address[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/N_4158\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/N_4154\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[28]\ : 
        CFG4
      generic map(INIT => x"00CA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[29]\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_788_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[19]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_29\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_20_Z\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_19_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_18_Z\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_17_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/parity_mismatch_1_29\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_1[1]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[2]\, 
        B => \can_top_level_inst/stat_bus[68]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_137\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\);
    
    \can_top_level_inst/rx_buffer_inst/un1_overrun_condition\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_4_Z\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_3_Z\, 
        C => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        D => \can_top_level_inst/rx_buffer_inst/N_57\, Y => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[20]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[258]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_9_RNIHPDK[1]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_9[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1299_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_3[29]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac25_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac4_Z\, 
        C => \can_top_level_inst/stat_bus[286]\, D => 
        \can_top_level_inst/stat_bus[249]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_3_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[1]\);
    
    \timestamp_ibuf[0]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(0), Y => \timestamp_ibuf[0]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_y0_0[24]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1073\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1076\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_0_y3[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2[13]\ : 
        CFG3
      generic map(INIT => x"74")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_1[13]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_648\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_Z[13]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_64\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[226]\, B => 
        \can_top_level_inst/rec_ident[21]\, C => 
        \can_top_level_inst/drv_bus[259]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_21\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[25]\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_dlc_com[3]\ : SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/tran_dlc[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[21]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[10]\, 
        B => \can_top_level_inst/tran_word[13]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIQ2NJ7[16]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[16]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[16]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_15\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_16\, 
        CC => NET_CC_CONFIG1341, P => NET_CC_CONFIG1338, Y3 => 
        NET_CC_CONFIG1339, Y3A => NET_CC_CONFIG1340);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[16]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[5]\ : 
        CFG4
      generic map(INIT => x"FEFA")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_163\, 
        B => \can_top_level_inst/txtb_allow_bb[6]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_164\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[19]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[24]\);
    
    
        \can_top_level_inst/bus_sampling_inst/data_edge_detector_inst/rx_data_prev\ : 
        SLE
      port map(D => \can_top_level_inst/rx_data_wbs\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/rx_data_prev\);
    
    \timestamp_ibuf[57]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(57), Y => 
        \timestamp_ibuf[57]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_4[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/drv_bus[165]\, B => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_976\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[269]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2_2[1]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[1]\, B
         => \can_top_level_inst/txtb_sw_cmd.set_rdy\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_149\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_1_0\ : 
        CFG4
      generic map(INIT => x"0D08")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1462_3\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_is_receiver\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_sm0\, 
        D => \can_top_level_inst/stat_bus[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[13]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_0_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_GND, CO => OPEN, P(0) => 
        NET_CC_CONFIG124, P(1) => NET_CC_CONFIG128, P(2) => 
        NET_CC_CONFIG132, P(3) => NET_CC_CONFIG136, P(4) => 
        NET_CC_CONFIG140, P(5) => NET_CC_CONFIG144, P(6) => 
        NET_CC_CONFIG148, P(7) => NET_CC_CONFIG152, P(8) => 
        NET_CC_CONFIG156, P(9) => NET_CC_CONFIG160, P(10) => 
        NET_CC_CONFIG164, P(11) => ADLIB_VCC, Y3(0) => 
        NET_CC_CONFIG125, Y3(1) => NET_CC_CONFIG129, Y3(2) => 
        NET_CC_CONFIG133, Y3(3) => NET_CC_CONFIG137, Y3(4) => 
        NET_CC_CONFIG141, Y3(5) => NET_CC_CONFIG145, Y3(6) => 
        NET_CC_CONFIG149, Y3(7) => NET_CC_CONFIG153, Y3(8) => 
        NET_CC_CONFIG157, Y3(9) => NET_CC_CONFIG161, Y3(10) => 
        NET_CC_CONFIG165, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG126, Y3A(1) => NET_CC_CONFIG130, Y3A(2) => 
        NET_CC_CONFIG134, Y3A(3) => NET_CC_CONFIG138, Y3A(4) => 
        NET_CC_CONFIG142, Y3A(5) => NET_CC_CONFIG146, Y3A(6) => 
        NET_CC_CONFIG150, Y3A(7) => NET_CC_CONFIG154, Y3A(8) => 
        NET_CC_CONFIG158, Y3A(9) => NET_CC_CONFIG162, Y3A(10) => 
        NET_CC_CONFIG166, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG127, CC(1) => NET_CC_CONFIG131, CC(2) => 
        NET_CC_CONFIG135, CC(3) => NET_CC_CONFIG139, CC(4) => 
        NET_CC_CONFIG143, CC(5) => NET_CC_CONFIG147, CC(6) => 
        NET_CC_CONFIG151, CC(7) => NET_CC_CONFIG155, CC(8) => 
        NET_CC_CONFIG159, CC(9) => NET_CC_CONFIG163, CC(10) => 
        NET_CC_CONFIG167, CC(11) => nc284);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[6]\);
    
    \can_top_level_inst/rx_buffer_inst/rwcnt_com_4_0_.m9\ : CFG4
      generic map(INIT => x"3B76")

      port map(A => \can_top_level_inst/rec_dlc[0]\, B => 
        \can_top_level_inst/rec_dlc[3]\, C => 
        \can_top_level_inst/rec_dlc[2]\, D => 
        \can_top_level_inst/rec_dlc[1]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_10\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[11]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[10]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[4]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_139\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m103_2_1\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/drv_bus[210]\, D => 
        \can_top_level_inst/drv_bus[173]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m103_2_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_first_ssp_d\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/tx_trigger\, B => 
        \can_top_level_inst/gen_first_ssp\, Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_first_ssp_d_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[378]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un7_txtb_parity_error_valid_i\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un7_txtb_parity_error_valid_i_1_Z\, 
        B => \can_top_level_inst/txtb_index_muxed[2]\, C => 
        \can_top_level_inst/txtb_parity_check_valid\, D => 
        \can_top_level_inst/txtb_index_muxed[1]\, Y => 
        \can_top_level_inst/txtb_parity_error_valid[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[128]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_3[7]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[30]\, CLK
         => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un25_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_34_1[20]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/stat_bus[208]\, B => 
        \can_top_level_inst/drv_bus[379]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_930_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask[9]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc1\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_0[8]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[15]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_2_2[9]\ : 
        CFG4
      generic map(INIT => x"551B")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => \can_top_level_inst/int_ena[9]\, C => 
        \can_top_level_inst/drv_bus[280]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_Z[9]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_2_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[11]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[17]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa11\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe11\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram81__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_355\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe81\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_RNIJLCE05\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[5]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        D => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[5]\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_4\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_5\, 
        CC => NET_CC_CONFIG2127, P => NET_CC_CONFIG2124, Y3 => 
        NET_CC_CONFIG2125, Y3A => NET_CC_CONFIG2126);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[3]\ : 
        SLE
      port map(D => \timestamp_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram57__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_352\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe57\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_9_RNIIKAMS\ : 
        CFG4
      generic map(INIT => x"FFF4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un7_exit_segm_regular_cry_9_0\, 
        B => \can_top_level_inst/prescaler_inst/is_tseg2\, C => 
        \can_top_level_inst/prescaler_inst/un8_exit_segm_regular_tseg1_0\, 
        D => 
        \can_top_level_inst/prescaler_inst/un5_exit_ph2_immediate_0\, 
        Y => 
        \can_top_level_inst/prescaler_inst/exit_segm_req_dbt\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[22]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[198]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[56]\ : 
        SLE
      port map(D => \timestamp_c[56]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[56]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[15]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[27]\);
    
    \timestamp_ibuf[12]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(12), Y => 
        \timestamp_ibuf[12]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[18]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[18]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[17]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram113_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe113\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram113_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram102_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram103_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y3[0]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q_RNI0IEB5[7]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[88]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_6\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_7\, 
        CC => NET_CC_CONFIG357, P => NET_CC_CONFIG354, Y3 => 
        NET_CC_CONFIG355, Y3A => NET_CC_CONFIG356);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[15]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[26]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[58]\, 
        C => \can_top_level_inst/rec_ident[26]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2271\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a2[1]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_190\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[3]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2529\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2461\, Y
         => \can_top_level_inst/memory_registers_inst/N_2659\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_0\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => \adress_c[10]\, B => \adress_c[11]\, C => 
        scs_c, D => swr_c, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_un1_ram_write_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_17_RNO[26]\ : 
        CFG4
      generic map(INIT => x"F0E0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[922]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[248]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[1]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[9]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[9]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[9]\);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_Z[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/operation_control_inst/N_10_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[20]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_5[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[32]\ : 
        CFG4
      generic map(INIT => x"3210")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_975\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[32]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[33]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_780_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_0_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[0]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_i[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[28]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIB6V53[6]\ : 
        CFG3
      generic map(INIT => x"FB")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_7\, 
        C => \can_top_level_inst/drv_bus[374]\, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_1_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[19]\);
    
    
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/g1_0\ : 
        CFG4
      generic map(INIT => x"AAAC")

      port map(A => 
        \can_top_level_inst/prescaler_inst_tq_edge_dbt\, B => 
        \can_top_level_inst/prescaler_inst_tq_edge_nbt\, C => 
        \can_top_level_inst/stat_bus[10]\, D => 
        \can_top_level_inst/stat_bus[11]\, Y => 
        \can_top_level_inst/prescaler_inst/g1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[25]\);
    
    \can_top_level_inst/int_manager_inst/int_input_active_11[11]\ : 
        CFG4
      generic map(INIT => x"0CAE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_11_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_shift_res_reg_inst/rx_shift_res_reg_inst/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_shift_res_reg_inst/rx_shift_res_reg_inst/N_1251_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa4\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe4\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_settings_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_settings_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[351]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_segm_counter_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_control[1]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\);
    
    \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/bus_sampling_inst/tx_trigger_q\, C
         => \can_top_level_inst/stat_bus[10]\, Y => 
        \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[25]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[19]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[19]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[19]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4279\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[16]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2_2[1]\ : 
        CFG4
      generic map(INIT => x"53FF")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[6]\, B
         => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[15]\, 
        C => \can_top_level_inst/drv_bus[475]\, D => 
        \can_top_level_inst/txtb_sw_cmd.set_rdy\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_149\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_18\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_18_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31[7]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[13]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[13]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[13]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2[3]\ : 
        CFG4
      generic map(INIT => x"D585")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_1[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_nxt\ : 
        CFG4
      generic map(INIT => x"656A")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[20]\, 
        B => \can_top_level_inst/can_core_inst/crc_data_rx_wbs\, 
        C => \can_top_level_inst/can_core_inst/crc_calc_from_rx\, 
        D => \can_top_level_inst/can_core_inst/bst_data_out\, Y
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_nxt_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[27]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[27]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa7\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe7\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[308]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_N_4L6\ : 
        CFG4
      generic map(INIT => x"75FD")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_N_3L4_Z\, 
        B => \can_top_level_inst/prescaler_inst_un2_tq_edge\, C
         => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_10_N_3L3_Z\, 
        D => \can_top_level_inst/prescaler_inst_tq_edge_nbt\, Y
         => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_1_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[0]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_rdy\, B
         => \can_top_level_inst/txtb_sw_cmd_index[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_141\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_load_data_word\ : 
        CFG4
      generic map(INIT => x"AA20")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1059\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data_d_0_sqmuxa_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[10]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_1\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[1]\, B => 
        \can_top_level_inst/drv_bus[144]\, C => 
        \can_top_level_inst/drv_bus[177]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_72\, 
        FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[0]\, 
        CC => NET_CC_CONFIG690, P => NET_CC_CONFIG687, Y3 => 
        NET_CC_CONFIG688, Y3A => NET_CC_CONFIG689);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram72__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_348\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe72\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i[0]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[0]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[16]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_RNIBV4GM4\ : 
        CFG4
      generic map(INIT => x"AEFF")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g1_0\, 
        B => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_nbt_tq_active\, 
        C => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_0_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1\, 
        Y => \can_top_level_inst/prescaler_inst/segment_end\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_19\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_19_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/is_arbitration_RNIDSD21\ : 
        CFG3
      generic map(INIT => x"BF")

      port map(A => \can_top_level_inst/stat_bus[370]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_N_865\, 
        C => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        Y => 
        \can_top_level_inst/can_core_inst/crc_calc_from_rx_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[28]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_7\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[412]\, B => 
        \can_top_level_inst/stat_bus[85]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_3\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_4\, 
        CC => NET_CC_CONFIG187, P => NET_CC_CONFIG184, Y3 => 
        NET_CC_CONFIG185, Y3A => NET_CC_CONFIG186);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[15]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO2099, CO => OPEN, P(0) => 
        NET_CC_CONFIG2140, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, 
        P(3) => ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_VCC, 
        P(6) => ADLIB_VCC, P(7) => ADLIB_VCC, P(8) => ADLIB_VCC, 
        P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11) => ADLIB_VCC, 
        Y3(0) => NET_CC_CONFIG2141, Y3(1) => ADLIB_VCC, Y3(2) => 
        ADLIB_VCC, Y3(3) => ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5)
         => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7) => ADLIB_VCC, 
        Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, Y3(10) => 
        ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG2142, Y3A(1) => ADLIB_VCC, Y3A(2) => 
        ADLIB_VCC, Y3A(3) => ADLIB_VCC, Y3A(4) => ADLIB_VCC, 
        Y3A(5) => ADLIB_VCC, Y3A(6) => ADLIB_VCC, Y3A(7) => 
        ADLIB_VCC, Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, 
        Y3A(10) => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG2143, CC(1) => nc222, CC(2) => nc159, CC(3)
         => nc136, CC(4) => nc241, CC(5) => nc253, CC(6) => nc178, 
        CC(7) => nc306, CC(8) => nc215, CC(9) => nc59, CC(10) => 
        nc221, CC(11) => nc232);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_dlc_com[1]\ : SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/tran_dlc[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[18]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_0_y3[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_0_y1[0]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[2]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/bus_sampling_inst/data_tx_delayed\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_709\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_712\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_0_y0[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_1[29]\ : 
        CFG4
      generic map(INIT => x"F2F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_14_1\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_17_i_0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_29\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_17_i_1\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_11\ : 
        ARI1_CC
      generic map(INIT => x"56633")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[11]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[11]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_10_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_11_Z\, 
        CC => NET_CC_CONFIG442, P => NET_CC_CONFIG439, Y3 => 
        NET_CC_CONFIG440, Y3A => NET_CC_CONFIG441);
    
    
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_20_RNI0ERE\ : 
        CFG4
      generic map(INIT => x"9669")

      port map(A => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_28_Z\, 
        B => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_27_Z\, 
        C => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_21_Z\, 
        D => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_20_Z\, 
        Y => \can_top_level_inst/N_3896_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[16]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2[23]\ : 
        CFG4
      generic map(INIT => x"4F50")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_1[23]\, 
        B => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_1_Z[23]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_8[29]\ : 
        CFG4
      generic map(INIT => x"DFFF")

      port map(A => \can_top_level_inst/tran_frame_test.fstc\, B
         => \can_top_level_inst/tran_frame_test.tprm[4]\, C => 
        \can_top_level_inst/drv_bus[477]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_17_i_o4_0_4\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIBGHO99[10]_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIBGHO99[10]_FCNET1\, 
        CO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_30\, 
        CC => NET_CC_CONFIG1051, P => NET_CC_CONFIG1048, Y3 => 
        NET_CC_CONFIG1049, Y3A => NET_CC_CONFIG1050);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[17]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[129]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[18]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_RNO[0]\ : 
        CFG4
      generic map(INIT => x"BF40")

      port map(A => \can_top_level_inst/stat_bus[10]\, B => 
        \can_top_level_inst/stat_bus[11]\, C => 
        \can_top_level_inst/bus_sampling_inst/tx_trigger_q\, D
         => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[30]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/un3_crc_d\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_nxt_Z\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, Y
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram124_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe124\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram124_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[17]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIUDL8I_S[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[214]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram26__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_340\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe26\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_46\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[158]\, B => 
        \can_top_level_inst/rec_ident[15]\, C => 
        \can_top_level_inst/drv_bus[191]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_36\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[310]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa18\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe18\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8[7]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2379\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_724\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_727\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_y0_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_segm_counter_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[0]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[0]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[3]\ : 
        CFG4
      generic map(INIT => x"F0F2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_168\, 
        D => \can_top_level_inst/txtb_parity_error_valid[7]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_ac0_7\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => \can_top_level_inst/stat_bus[85]\, B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c3\, 
        C => \can_top_level_inst/stat_bus[84]\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_2[0]\ : 
        CFG3
      generic map(INIT => x"35")

      port map(A => \can_top_level_inst/drv_bus[35]\, B => 
        \can_top_level_inst/stat_bus[81]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[28]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[10]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[7]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_12[28]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac28_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac17_Z\, 
        C => \can_top_level_inst/drv_bus[299]\, D => 
        \can_top_level_inst/drv_bus[57]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_12_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[24]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/next_state_3_sqmuxa_0_a2\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/tx_arb_parity_check_valid_1\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[2]\, Y
         => 
        \can_top_level_inst/tx_arbitrator_inst/load_ffmt_w_addr\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\);
    
    \data_in_ibuf[9]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[9]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[9]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\);
    
    \can_top_level_inst/rx_buffer_inst/un5_ram_write\ : CFG3
      generic map(INIT => x"F1")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/data_overrun_i_Z\, B
         => \can_top_level_inst/rx_buffer_inst/N_48\, C => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        Y => \can_top_level_inst/rx_buffer_inst/un5_ram_write_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_2[2]\ : 
        CFG4
      generic map(INIT => x"F588")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1221\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1222\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_Z[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_2_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_vals2_RNIMTNC_1\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_29\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_div_32_plus_5_sat[2]\ : 
        CFG4
      generic map(INIT => x"0787")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[6]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/CO0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ANB2\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[8]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_mem_index[2]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_15_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_GND, CO => OPEN, P(0) => 
        NET_CC_CONFIG168, P(1) => NET_CC_CONFIG172, P(2) => 
        NET_CC_CONFIG176, P(3) => NET_CC_CONFIG180, P(4) => 
        NET_CC_CONFIG184, P(5) => NET_CC_CONFIG188, P(6) => 
        NET_CC_CONFIG192, P(7) => NET_CC_CONFIG196, P(8) => 
        NET_CC_CONFIG200, P(9) => NET_CC_CONFIG204, P(10) => 
        ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG169, 
        Y3(1) => NET_CC_CONFIG173, Y3(2) => NET_CC_CONFIG177, 
        Y3(3) => NET_CC_CONFIG181, Y3(4) => NET_CC_CONFIG185, 
        Y3(5) => NET_CC_CONFIG189, Y3(6) => NET_CC_CONFIG193, 
        Y3(7) => NET_CC_CONFIG197, Y3(8) => NET_CC_CONFIG201, 
        Y3(9) => NET_CC_CONFIG205, Y3(10) => ADLIB_VCC, Y3(11)
         => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG170, Y3A(1) => 
        NET_CC_CONFIG174, Y3A(2) => NET_CC_CONFIG178, Y3A(3) => 
        NET_CC_CONFIG182, Y3A(4) => NET_CC_CONFIG186, Y3A(5) => 
        NET_CC_CONFIG190, Y3A(6) => NET_CC_CONFIG194, Y3A(7) => 
        NET_CC_CONFIG198, Y3A(8) => NET_CC_CONFIG202, Y3A(9) => 
        NET_CC_CONFIG206, Y3A(10) => ADLIB_VCC, Y3A(11) => 
        ADLIB_VCC, CC(0) => NET_CC_CONFIG171, CC(1) => 
        NET_CC_CONFIG175, CC(2) => NET_CC_CONFIG179, CC(3) => 
        NET_CC_CONFIG183, CC(4) => NET_CC_CONFIG187, CC(5) => 
        NET_CC_CONFIG191, CC(6) => NET_CC_CONFIG195, CC(7) => 
        NET_CC_CONFIG199, CC(8) => NET_CC_CONFIG203, CC(9) => 
        NET_CC_CONFIG207, CC(10) => nc274, CC(11) => nc18);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[12]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[12]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[14]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[14]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[14]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2638\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_after_ack_err\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_ack_err_flag\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[7]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_err_after_ack_err\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa19\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe19\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un7_txtb_parity_error_valid_i_1\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/N_3897\, 
        B => \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/txtb_index_muxed[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un7_txtb_parity_error_valid_i_1_Z\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_4\ : 
        CFG4
      generic map(INIT => x"FEFA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_2_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_1_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_926\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_4_Z\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q_RNIUN418[2]\ : 
        ARI1_CC
      generic map(INIT => x"5E11E")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[2]\, 
        B => \can_top_level_inst/stat_bus[81]\, C => 
        \can_top_level_inst/stat_bus[82]\, D => 
        \can_top_level_inst/stat_bus[83]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_1\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_2\, 
        CC => NET_CC_CONFIG95, P => NET_CC_CONFIG92, Y3 => 
        NET_CC_CONFIG93, Y3A => NET_CC_CONFIG94);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[13]\);
    
    \data_out_obuf[14]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[14]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[14]/DOUT\, EOUT => 
        \data_out_obuf[14]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram70_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram71_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/next_state_15_sqmuxa_i\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_955\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_980\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_854_i\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry[1]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s_2641_FCO\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[1]\, 
        CC => NET_CC_CONFIG296, P => NET_CC_CONFIG293, Y3 => 
        NET_CC_CONFIG294, Y3A => NET_CC_CONFIG295);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_45\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[14]\, B => 
        \can_top_level_inst/drv_bus[95]\, C => 
        \can_top_level_inst/drv_bus[128]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_36\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[6]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[7]\, 
        CC => NET_CC_CONFIG1795, P => NET_CC_CONFIG1792, Y3 => 
        NET_CC_CONFIG1793, Y3A => NET_CC_CONFIG1794);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[13]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2702\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2669\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_53\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[145]\, B => 
        \can_top_level_inst/rec_ident[2]\, C => 
        \can_top_level_inst/drv_bus[178]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_32\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_RNIIFE91_0[0]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[2]\, 
        B => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, C
         => \can_top_level_inst/rx_buffer_inst/stored_ts\, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/memory_write_data_sn_N_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNIAORRI6[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_port_b_address[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/N_4138\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/N_4134\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[12]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[12]\);
    
    \adress_ibuf[5]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \adress_c[5]\, E => ADLIB_GND, YIN => 
        \adress_ibuf[5]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[206]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4QRQ3_S[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[192]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[19]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_9\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[19]\, B => 
        \can_top_level_inst/drv_bus[162]\, C => 
        \can_top_level_inst/drv_bus[195]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_67\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[8]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[9]\, 
        CC => NET_CC_CONFIG726, P => NET_CC_CONFIG723, Y3 => 
        NET_CC_CONFIG724, Y3A => NET_CC_CONFIG725);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.is_control_2_i_a2_1\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1462_3\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[23]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/is_control_2_i_a2_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[9]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[1]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[1]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4261\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[278]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[4]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[3]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI16B69_S[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[285]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[12]\);
    
    \data_in_ibuf[6]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(6), Y => \data_in_ibuf[6]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_2[8]\ : 
        CFG4
      generic map(INIT => x"3B0C")

      port map(A => \can_top_level_inst/int_mask[8]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_2_1[8]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_Z[8]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_2_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_wmux_3\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_0_y3\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_0_y1\, 
        C => \can_top_level_inst/txtb_hw_cmd_index[0]\, D => 
        ADLIB_VCC, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_0\, 
        D => \sbe_c[1]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_control[1]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate_3_0_.m12\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[3]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_13\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_5\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[3]\, B
         => \can_top_level_inst/can_core_inst/crc_15[2]\, C => 
        \can_top_level_inst/can_core_inst/crc_15[3]\, D => 
        \can_top_level_inst/store_data_word[2]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[0]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[1]\, 
        CC => NET_CC_CONFIG54, P => NET_CC_CONFIG51, Y3 => 
        NET_CC_CONFIG52, Y3A => NET_CC_CONFIG53);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/reg_value_r_fast[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_tx_command_reg_comp_reg_value_r_fast[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[0]\, 
        D => \can_top_level_inst/prescaler_inst/tseg1_dbt[0]\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select\, 
        B => \sbe_c[1]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[9]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_34[19]\ : 
        CFG4
      generic map(INIT => x"FF8C")

      port map(A => \timestamp_c[19]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_892_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_892\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[3]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_2[0]\ : 
        CFG4
      generic map(INIT => x"CFCE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1408\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_1_Z[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[0]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_2_Z[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[11]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[11]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[11]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/current_state_0_rep1_RNO\ : 
        CFG4
      generic map(INIT => x"0208")

      port map(A => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        B => \can_top_level_inst/prescaler_inst/segment_end\, C
         => \can_top_level_inst/prescaler_inst/current_state[1]\, 
        D => \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/N_33_i_rep1\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06\ : 
        CFG4
      generic map(INIT => x"7077")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_129\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_135\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_19\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_02\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[17]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[17]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[21]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m30_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_7\ : 
        ARI1_CC
      generic map(INIT => x"46600")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[7]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_6_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_7_Z\, 
        CC => NET_CC_CONFIG916, P => NET_CC_CONFIG913, Y3 => 
        NET_CC_CONFIG914, Y3A => NET_CC_CONFIG915);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[301]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[11]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[11]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[1]\ : 
        CFG4
      generic map(INIT => x"0A0B")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        B => \can_top_level_inst/txtb_sw_cmd.set_ety\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_141\, 
        D => \can_top_level_inst/txtb_allow_bb[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[19]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_7_i_a2\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1304\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[15]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[4]\ : 
        CFG4
      generic map(INIT => x"CE0A")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[19]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[19]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[18]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[13]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[13]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/int_mask_load\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[0]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/int_mask_load_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_12[17]\ : 
        CFG4
      generic map(INIT => x"CA0A")

      port map(A => \can_top_level_inst/stat_bus[108]\, B => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[17]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_794\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_9\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[280]\, B => 
        \can_top_level_inst/rec_ident[9]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_8\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_9\, 
        CC => NET_CC_CONFIG1869, P => NET_CC_CONFIG1866, Y3 => 
        NET_CC_CONFIG1867, Y3A => NET_CC_CONFIG1868);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[26]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIG7IO[0]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[188]\, D => 
        \can_top_level_inst/stat_bus[220]\, FCI => ADLIB_VCC, S
         => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_0\, 
        CC => NET_CC_CONFIG756, P => NET_CC_CONFIG753, Y3 => 
        NET_CC_CONFIG754, Y3A => NET_CC_CONFIG755);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[9]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_1[15]\ : 
        CFG4
      generic map(INIT => x"0B08")

      port map(A => \can_top_level_inst/tran_identifier[1]\, B
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[15]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI48LTK1[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[45]\, B => 
        \can_top_level_inst/tran_word[13]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_12\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_13\, 
        CC => NET_CC_CONFIG979, P => NET_CC_CONFIG976, Y3 => 
        NET_CC_CONFIG977, Y3A => NET_CC_CONFIG978);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_shift_ena_cnst_i_a2[0]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[30]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[33]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1238\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[17]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[17]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[17]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/current_state[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/current_state[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_0[23]\ : 
        CFG4
      generic map(INIT => x"8D8C")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_0_1_Z[23]\, 
        D => \can_top_level_inst/drv_bus[261]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_0_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[23]\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count[4]\ : SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNIA5JE2_S[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_frame_count[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[19]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[19]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_9[22]\ : 
        CFG4
      generic map(INIT => x"FA3A")

      port map(A => \can_top_level_inst/stat_bus[77]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_190\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_981\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[14]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2638\, Y
         => \can_top_level_inst/memory_registers_inst/N_2703\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_20\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_20_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/un15_txtb_hw_cmd_int_0_a3_1_0\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[0]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => \can_top_level_inst/txtb_hw_cmd_index[1]\, D => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_un15_txtb_hw_cmd_int_0_a3_1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_13[9]\ : 
        CFG4
      generic map(INIT => x"F4F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1481_2\, 
        D => \can_top_level_inst/drv_bus[214]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1481\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNIKDRP1[31]\ : 
        CFG4
      generic map(INIT => x"4047")

      port map(A => \can_top_level_inst/stat_bus[109]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_dominant\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_output_i_m_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_data_in\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_a5_1_0[1]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_223\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[8]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[8]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1467\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1461\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_18[3]\ : 
        CFG4
      generic map(INIT => x"0AC0")

      port map(A => \can_top_level_inst/drv_bus[38]\, B => 
        \can_top_level_inst/stat_bus[84]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1258\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[2]\);
    
    \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_1\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/crc_calc_from_rx_0\, B
         => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un1_data_halt_d_i\, 
        C => \can_top_level_inst/can_core_inst/stuff_length[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa13\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe13\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[26]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_d[0]\ : 
        CFG3
      generic map(INIT => x"F6")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[0]\, 
        B => \can_top_level_inst/rx_triggers[1]\, C => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un17_same_bits_erase_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_d_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[4]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[12]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[12]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[29]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o2[16]\ : 
        CFG2
      generic map(INIT => x"B")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_no_data_field_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1059\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un73_rx_data_nbs_0_a2\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1255\);
    
    \can_top_level_inst/tx_arbitrator_inst/validated_buffer_0\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[1]\, B
         => \can_top_level_inst/txtb_hw_cmd_index[0]\, C => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[11]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_13\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[409]\, B => 
        \can_top_level_inst/stat_bus[82]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_0\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_1\, 
        CC => NET_CC_CONFIG175, P => NET_CC_CONFIG172, Y3 => 
        NET_CC_CONFIG173, Y3A => NET_CC_CONFIG174);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[20]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[163]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/gen_first_ssp\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[23]\, 
        Y => \can_top_level_inst/gen_first_ssp\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_read_address[3]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => \can_top_level_inst/rx_read_pointer[3]\, C => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_24[0]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[205]\, C => 
        \can_top_level_inst/drv_bus[110]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1150\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c5_RNIB9471\ : 
        CFG4
      generic map(INIT => x"9699")

      port map(A => \can_top_level_inst/drv_bus[6]\, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[6]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c5_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[5]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/N_3913\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un2_select_index_changed_2\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[2]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[2]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un2_select_index_changed_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_2\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[11]\, B
         => \can_top_level_inst/can_core_inst/crc_17[10]\, C => 
        \can_top_level_inst/can_core_inst/crc_17[11]\, D => 
        \can_top_level_inst/store_data_word[10]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[4]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[5]\, 
        CC => NET_CC_CONFIG2547, P => NET_CC_CONFIG2544, Y3 => 
        NET_CC_CONFIG2545, Y3A => NET_CC_CONFIG2546);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[273]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid_d_0_sqmuxa_1\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un26_is_receiver\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid_d_0_sqmuxa_1_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/int_status\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/un2_int_status_set\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/un1_int_status_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_vector[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[0]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_rdy\, B
         => \can_top_level_inst/txtb_sw_cmd_index[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_141\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa7\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe7\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[20]\);
    
    
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_17\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => \data_in_c[23]\, B => \data_in_c[22]\, C => 
        \data_in_c[21]\, D => \data_in_c[20]\, Y => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_17_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_38_1_0_wmux_RNO[20]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_2_0_Z[20]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_2_Z[20]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_911\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNIGOOR2[2]\ : 
        ARI1_CC
      generic map(INIT => x"58DD8")

      port map(A => \can_top_level_inst/stat_bus[73]\, B => 
        \can_top_level_inst/drv_bus[425]\, C => 
        \can_top_level_inst/drv_bus[418]\, D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[4]\, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_1\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_2\, 
        CC => NET_CC_CONFIG2498, P => NET_CC_CONFIG2495, Y3 => 
        NET_CC_CONFIG2496, Y3A => NET_CC_CONFIG2497);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[13]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[12]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_788_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_RNIQ8RR1_1\ : 
        CFG3
      generic map(INIT => x"48")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/CO2\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_s1\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[8]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_138\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_1[1]\ : 
        CFG3
      generic map(INIT => x"37")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/int_vector[1]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[31]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_tran_frame_parity_error\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => \can_top_level_inst/drv_bus[476]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_clk_en_q_Z\, 
        Y => \can_top_level_inst/tran_frame_parity_error\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[11]\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_write_address[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[3]\, 
        C => \can_top_level_inst/rx_buffer_inst/N_48\, Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_RNO[21]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2677\, D
         => \can_top_level_inst/memory_registers_inst/N_2710\, Y
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram17__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_355\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe17\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_0_y0[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_wmux_0_Y[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_wmux_1_Y[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_0_y1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[19]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[19]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[21]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[18]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[161]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[7]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[18]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[18]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[18]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_0\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \adress_c[4]\, B => \adress_c[5]\, C => 
        \adress_c[7]\, D => swr_c, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, D => 
        \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[15]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_27[4]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => \can_top_level_inst/stat_bus[104]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1305\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[114]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/m8\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[0]\, 
        B => \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_m3[1]\ : 
        CFG4
      generic map(INIT => x"EFE0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => \can_top_level_inst/txtb_parity_error_valid[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_148\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_149\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_150\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[21]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[21]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_6[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_36_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_1_1.CO2\ : 
        CFG3
      generic map(INIT => x"D4")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0_Z[2]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3_Z[2]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[27]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[27]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[27]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_form_err_i_0_sqmuxa\ : 
        CFG4
      generic map(INIT => x"4404")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        D => \can_top_level_inst/stat_bus[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_form_err_i_0_sqmuxa_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[6]\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[6]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_Z[6]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/masked_data[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_RNI7L0T41[2]\ : 
        CFG3
      generic map(INIT => x"1F")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[4]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[2]\, C
         => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/tx_arb_parity_check_valid_1\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_N_7_mux_i_0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[29]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa18\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10_2\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa18\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[6]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[6]\, 
        D => \can_top_level_inst/rx_write_pointer[6]\, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[5]\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s_Z[6]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG393, P => 
        NET_CC_CONFIG390, Y3 => NET_CC_CONFIG391, Y3A => 
        NET_CC_CONFIG392);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[12]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/txtb_ptr[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[14]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[14]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[14]\);
    
    \can_top_level_inst/tx_arbitrator_inst/last_txtb_index[2]\ : 
        SLE
      port map(D => \can_top_level_inst/txtb_hw_cmd_index[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_last_txtb_index\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/last_txtb_index_Z[2]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[275]\, B => 
        \can_top_level_inst/rec_ident[4]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_3\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_4\, 
        CC => NET_CC_CONFIG1849, P => NET_CC_CONFIG1846, Y3 => 
        NET_CC_CONFIG1847, Y3A => NET_CC_CONFIG1848);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram51__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_343\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe51\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[20]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2546\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[4]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2462\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_h_sync_valid_2\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_shorten_tseg1_after_tseg2_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        C => \can_top_level_inst/prescaler_inst/h_sync_valid\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_h_sync_valid_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_13[10]\ : 
        CFG4
      generic map(INIT => x"F4F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1519_2\, 
        D => \can_top_level_inst/drv_bus[215]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1519\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/un3_test_registers_cs_Z\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => res_n_out_c, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD
         => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.unlock_2_2_0\ : 
        CFG4
      generic map(INIT => x"008A")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_717_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/unlock_5\, 
        C => \can_top_level_inst/stat_bus[378]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/set_idle_i_5_sqmuxa\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1497_2\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[19]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[19]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[19]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2643\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIUAAP6[13]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[13]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[13]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_12\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_13\, 
        CC => NET_CC_CONFIG1329, P => NET_CC_CONFIG1326, Y3 => 
        NET_CC_CONFIG1327, Y3A => NET_CC_CONFIG1328);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[8]\);
    
    \can_top_level_inst/rx_buffer_inst/commit_rx_frame_RNO\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => \can_top_level_inst/rx_buffer_inst/data_overrun_i_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/commit_overrun_abort_1_sqmuxa_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_0[1]\ : 
        CFG4
      generic map(INIT => x"F800")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        B => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_a4_0_Z[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_0_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[13]\ : 
        CFG4
      generic map(INIT => x"AC0C")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[13]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        D => \can_top_level_inst/tran_word[21]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[10]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_36_1[4]\ : 
        CFG4
      generic map(INIT => x"00D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[4]\, C => 
        \can_top_level_inst/stat_bus[192]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1314_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_38_1_0_wmux_RNO[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_0_Z[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_Z[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1141\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_RNO[3]\ : 
        CFG4
      generic map(INIT => x"3210")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_121\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[4]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[3]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_85_i\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/int_mask_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/int_mask_load_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_mask[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_s_0_2637\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_fast\, C => 
        ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_VCC, S => OPEN, Y
         => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_s_0_2637_FCO\, 
        CC => NET_CC_CONFIG884, P => NET_CC_CONFIG881, Y3 => 
        NET_CC_CONFIG882, Y3A => NET_CC_CONFIG883);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[5]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_8_x_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_6_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_sx_Z\, 
        Y => \can_top_level_inst/prescaler_inst/h_sync_valid\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_2[3]\ : 
        CFG4
      generic map(INIT => x"BBC0")

      port map(A => \timestamp_c[35]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/stat_bus[223]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_1_Z[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_2_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[26]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[26]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[26]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4828\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_9\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[19]\, B => 
        \can_top_level_inst/drv_bus[224]\, C => 
        \can_top_level_inst/drv_bus[257]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_67\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[8]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[9]\, 
        CC => NET_CC_CONFIG499, P => NET_CC_CONFIG496, Y3 => 
        NET_CC_CONFIG497, Y3A => NET_CC_CONFIG498);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[25]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[25]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1[5]\ : 
        CFG4
      generic map(INIT => x"0E02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[5]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[5]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO[12]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_37_2_0_Z[12]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_37_2_Z[12]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_629\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[15]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        C => \can_top_level_inst/drv_bus[29]\, D => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[0]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_7[2]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_4[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1462_3\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_2[2]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_3[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_7[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[8]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIQBAP[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[0]\, 
        Y => \can_top_level_inst/tran_word[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_7[29]\ : 
        CFG3
      generic map(INIT => x"53")

      port map(A => \can_top_level_inst/tran_identifier[26]\, B
         => \can_top_level_inst/tran_identifier[15]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_13_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[12]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/crc_err_1\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[14]\, 
        Y => \can_top_level_inst/stat_bus[377]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa17\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe17\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram109_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe109\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram109_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[267]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m34_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_21[11]\ : 
        CFG4
      generic map(INIT => x"CECC")

      port map(A => \can_top_level_inst/stat_bus[268]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1565_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1565\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram65_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe65\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram65_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[8]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_27\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[10]\, B => 
        \can_top_level_inst/drv_bus[215]\, C => 
        \can_top_level_inst/drv_bus[248]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_51\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[4]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[5]\, 
        CC => NET_CC_CONFIG483, P => NET_CC_CONFIG480, Y3 => 
        NET_CC_CONFIG481, Y3A => NET_CC_CONFIG482);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0[3]\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => \can_top_level_inst/txtb_hw_cmd.arbl\, B => 
        \can_top_level_inst/txtb_hw_cmd.err\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_raw_0[6]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI5VLH2_S[6]\, 
        B => \can_top_level_inst/drv_bus[381]\, C => 
        \can_top_level_inst/drv_bus[374]\, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_42\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_dest[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIB9FT[0]\ : 
        ARI1_CC
      generic map(INIT => x"5D827")

      port map(A => \timestamp_c[32]\, B => 
        \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[0]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[0]\, 
        FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_0\, 
        CC => NET_CC_CONFIG927, P => NET_CC_CONFIG924, Y3 => 
        NET_CC_CONFIG925, Y3A => NET_CC_CONFIG926);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_9\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[403]\, B => 
        \can_top_level_inst/stat_bus[74]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_2\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_3\, 
        CC => NET_CC_CONFIG1997, P => NET_CC_CONFIG1994, Y3 => 
        NET_CC_CONFIG1995, Y3A => NET_CC_CONFIG1996);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m19_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[6]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2532\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped[11]\ : 
        CFG4
      generic map(INIT => x"6CCC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_0[8]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[11]\, 
        C => \can_top_level_inst/tran_frame_test.fcrc\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_0_Z[11]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[25]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNID7LB[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_GND, CO => OPEN, P(0) => 
        NET_CC_CONFIG1566, P(1) => NET_CC_CONFIG1570, P(2) => 
        NET_CC_CONFIG1574, P(3) => NET_CC_CONFIG1578, P(4) => 
        NET_CC_CONFIG1582, P(5) => NET_CC_CONFIG1586, P(6) => 
        NET_CC_CONFIG1590, P(7) => NET_CC_CONFIG1594, P(8) => 
        NET_CC_CONFIG1598, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, 
        P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1567, Y3(1) => 
        NET_CC_CONFIG1571, Y3(2) => NET_CC_CONFIG1575, Y3(3) => 
        NET_CC_CONFIG1579, Y3(4) => NET_CC_CONFIG1583, Y3(5) => 
        NET_CC_CONFIG1587, Y3(6) => NET_CC_CONFIG1591, Y3(7) => 
        NET_CC_CONFIG1595, Y3(8) => NET_CC_CONFIG1599, Y3(9) => 
        ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, 
        Y3A(0) => NET_CC_CONFIG1568, Y3A(1) => NET_CC_CONFIG1572, 
        Y3A(2) => NET_CC_CONFIG1576, Y3A(3) => NET_CC_CONFIG1580, 
        Y3A(4) => NET_CC_CONFIG1584, Y3A(5) => NET_CC_CONFIG1588, 
        Y3A(6) => NET_CC_CONFIG1592, Y3A(7) => NET_CC_CONFIG1596, 
        Y3A(8) => NET_CC_CONFIG1600, Y3A(9) => ADLIB_VCC, Y3A(10)
         => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG1569, CC(1) => NET_CC_CONFIG1573, CC(2) => 
        NET_CC_CONFIG1577, CC(3) => NET_CC_CONFIG1581, CC(4) => 
        NET_CC_CONFIG1585, CC(5) => NET_CC_CONFIG1589, CC(6) => 
        NET_CC_CONFIG1593, CC(7) => NET_CC_CONFIG1597, CC(8) => 
        NET_CC_CONFIG1601, CC(9) => nc44, CC(10) => nc117, CC(11)
         => nc189);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\);
    
    \adress_ibuf[8]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \adress_c[8]\, E => ADLIB_GND, YIN => 
        \adress_ibuf[8]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[5]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        B => \can_top_level_inst/txtb_hw_cmd.failed\, C => 
        \can_top_level_inst/txtb_parity_error_valid[6]\, D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_164\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[305]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[18]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_nbt_Z[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_6_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[23]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[23]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[23]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[25]\);
    
    \data_in_ibuf[19]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[19]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[19]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1[6]\ : 
        CFG4
      generic map(INIT => x"0E02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[6]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[6]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram1_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram1_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram20__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_345\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe20\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/no_data_receiver\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => \can_top_level_inst/store_data_word[1]\, B
         => \can_top_level_inst/store_data_word[2]\, C => 
        \can_top_level_inst/can_core_inst/bds_data_out\, D => 
        \can_top_level_inst/store_data_word[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/no_data_receiver_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[9]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[8]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_15[7]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[8]\);
    
    \can_top_level_inst/can_core_inst/trigger_mux_inst/g0\ : CFG3
      generic map(INIT => x"47")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un6_same_bits_rst_trig_i\, 
        B => \can_top_level_inst/tx_trigger\, C => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_data_halt_q\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst_crc_17_21_trigger_1_1\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[20]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[20]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/un44_txtb_hw_cmd_int_0_a3_1\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst_priority_decoder_inst_un1_m2_0_a2_0_1\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_172\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[26]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s_2639\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_VCC, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s_2639_FCO\, 
        CC => NET_CC_CONFIG1055, P => NET_CC_CONFIG1052, Y3 => 
        NET_CC_CONFIG1053, Y3A => NET_CC_CONFIG1054);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1[1]\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \can_top_level_inst/stat_bus[89]\, B => 
        \can_top_level_inst/stat_bus[88]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNIR2DS_S\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_13[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram84_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram85_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\);
    
    \data_in_ibuf[9]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(9), Y => \data_in_ibuf[9]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[6]\ : 
        CFG4
      generic map(INIT => x"FF08")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_195\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_176\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[30]\ : 
        CFG4
      generic map(INIT => x"EC00")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[30]\, B
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[24]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/un5_transient_state_0_o3\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[3]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[3]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un6_ctrl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[7]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[7]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[7]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_a3[3]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst_txnf_calc_proc_7_un7_txtb_state_2\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[29]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2555\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2487\, Y
         => \can_top_level_inst/memory_registers_inst/N_2685\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_13[6]\ : 
        CFG4
      generic map(INIT => x"F4F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1367_2\, 
        D => \can_top_level_inst/drv_bus[211]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1367\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_784_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[28]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_3_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_4_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_4_Z\, 
        CC => NET_CC_CONFIG1230, P => NET_CC_CONFIG1227, Y3 => 
        NET_CC_CONFIG1228, Y3A => NET_CC_CONFIG1229);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.lock_i_o2_0\ : 
        CFG4
      generic map(INIT => x"BAFF")

      port map(A => \can_top_level_inst/is_bus_off\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_go_to_suspend_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_933\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_958\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"553AC")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[4]\, 
        B => \can_top_level_inst/bus_sampling_inst/ssp_delay[4]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[4]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_3_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_4_Z\, 
        CC => NET_CC_CONFIG414, P => NET_CC_CONFIG411, Y3 => 
        NET_CC_CONFIG412, Y3A => NET_CC_CONFIG413);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30[8]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[1]\);
    
    \timestamp_ibuf[31]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(31), Y => 
        \timestamp_ibuf[31]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[5]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[6]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[6]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[6]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_2_0[22]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1000\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_0_Z[22]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_2_0_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO[29]\ : 
        CFG4
      generic map(INIT => x"00FD")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_29\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_18\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_17_i_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        D => \adress_c[4]\, Y => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_stat_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[15]\);
    
    
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_13\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => \data_in_c[10]\, B => \data_in_c[11]\, Y => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_13_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa15\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe15\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[21]\ : 
        CFG4
      generic map(INIT => x"EC00")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[21]\, B
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[19]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\);
    
    \adress_ibuf[6]/U_IOPAD\ : IOPAD_IN
      port map(PAD => adress(6), Y => \adress_ibuf[6]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_31_RNO_0[12]\ : 
        CFG4
      generic map(INIT => x"2273")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_2_1[12]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_2_1_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_2[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_16\ : 
        CFG3
      generic map(INIT => x"96")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_16_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[29]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[30]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_RNIG6M52[7]\ : 
        CFG4
      generic map(INIT => x"9666")

      port map(A => \can_top_level_inst/drv_bus[7]\, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[7]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c4_i\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_m1_0_a2_1\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_RNIG6M52_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[21]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[2]\, CLK
         => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[11]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[23]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[23]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[23]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[16]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[16]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un3_same_bits_rst\ : 
        CFG4
      generic map(INIT => x"F3B3")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un6_same_bits_rst_trig_i\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/reg_q\, 
        C => \can_top_level_inst/tx_trigger\, D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un1_same_bits_rst_trig_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un3_same_bits_rst_i_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram122_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe122\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram122_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[95]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\);
    
    \can_top_level_inst/memory_registers_inst/status_comb_3[2]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_147\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_147\, 
        C => \can_top_level_inst/txtb_state_2[3]\, D => 
        \can_top_level_inst/txtb_state_0[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/status_comb_3_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_d_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_4\ : 
        CFG4
      generic map(INIT => x"3B0A")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_968\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1013\, 
        C => \can_top_level_inst/can_core_inst/bds_data_out\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_957\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_4_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNIU3941[7]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[6]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ANB2\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[8]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un2_compl_ctr_div_32_plus_5_sat_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_11\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_11_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[6]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select[3]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => swr_c, B => \sbe_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_0\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_RNI6L9G1\ : 
        CFG4
      generic map(INIT => x"0888")

      port map(A => \can_top_level_inst/prescaler_inst/g1\, B => 
        \can_top_level_inst/prescaler_inst/g0_0_5\, C => 
        \can_top_level_inst/prescaler_inst/un6_phase_err_sjw_by_one_NE\, 
        D => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        Y => \can_top_level_inst/prescaler_inst/g0_0_7\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[12]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[12]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[12]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_0\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0_1\, 
        B => \adress_c[6]\, C => \adress_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[14]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[186]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[31]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[31]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[8]\);
    
    \can_top_level_inst/int_manager_inst/int_input_active_24[11]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => 
        \can_top_level_inst/int_manager_inst/int_input_active_24_tz_Z[11]\, 
        C => \can_top_level_inst/is_bus_off\, Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_24_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNI6UT21[24]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => \can_top_level_inst/can_core_inst/pc_rx_trigger\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_switch_data\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa19\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa19\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[25]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[263]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_set_receiver_i_o4\ : 
        CFG3
      generic map(INIT => x"AE")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_929\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_961\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[22]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_RNO[10]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_0_Z[10]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_RNO_Z[10]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un23_inc_eight_i_0\ : 
        CFG4
      generic map(INIT => x"CCEC")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/bit_err_after_ack_err\, 
        C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un1_inc_eight_i_2_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un23_inc_eight_i_0_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[152]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[24]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2648\, Y
         => \can_top_level_inst/memory_registers_inst/N_2713\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[19]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[19]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[177]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_17\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_17_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[28]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[15]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[4]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_28[18]\ : 
        CFG4
      generic map(INIT => x"ACF0")

      port map(A => \can_top_level_inst/stat_bus[291]\, B => 
        \can_top_level_inst/drv_bus[99]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_848\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[22]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[22]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[22]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[22]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[6]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[38]\, 
        C => \can_top_level_inst/rec_ident[6]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2251\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[19]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2545\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2477\, Y
         => \can_top_level_inst/memory_registers_inst/N_2675\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_2[6]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        B => \can_top_level_inst/can_core_inst/crc_21[6]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_2_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[16]\ : 
        CFG4
      generic map(INIT => x"C055")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1187\, 
        B => \can_top_level_inst/tran_word[8]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[19]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2545\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[6]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_s_0_2645_CC_2\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO1206, CO => OPEN, P(0) => 
        NET_CC_CONFIG1259, P(1) => NET_CC_CONFIG1263, P(2) => 
        NET_CC_CONFIG1267, P(3) => ADLIB_VCC, P(4) => ADLIB_VCC, 
        P(5) => ADLIB_VCC, P(6) => ADLIB_VCC, P(7) => ADLIB_VCC, 
        P(8) => ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, 
        P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1260, Y3(1) => 
        NET_CC_CONFIG1264, Y3(2) => NET_CC_CONFIG1268, Y3(3) => 
        ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5) => ADLIB_VCC, Y3(6)
         => ADLIB_VCC, Y3(7) => ADLIB_VCC, Y3(8) => ADLIB_VCC, 
        Y3(9) => ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => 
        ADLIB_VCC, Y3A(0) => NET_CC_CONFIG1261, Y3A(1) => 
        NET_CC_CONFIG1265, Y3A(2) => NET_CC_CONFIG1269, Y3A(3)
         => ADLIB_VCC, Y3A(4) => ADLIB_VCC, Y3A(5) => ADLIB_VCC, 
        Y3A(6) => ADLIB_VCC, Y3A(7) => ADLIB_VCC, Y3A(8) => 
        ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => ADLIB_VCC, 
        Y3A(11) => ADLIB_VCC, CC(0) => NET_CC_CONFIG1262, CC(1)
         => NET_CC_CONFIG1266, CC(2) => NET_CC_CONFIG1270, CC(3)
         => nc164, CC(4) => nc148, CC(5) => nc42, CC(6) => nc231, 
        CC(7) => nc191, CC(8) => nc255, CC(9) => nc283, CC(10)
         => nc341, CC(11) => nc317);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d_1\ : 
        CFG4
      generic map(INIT => x"1001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_txtb_clk_en_d_NE_0_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_txtb_clk_en_d_NE_2_Z\, 
        C => \can_top_level_inst/txtb_ptr[2]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d_1_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[27]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[27]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[27]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[12]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[12]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[12]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac19_0\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac19_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_25[8]\ : 
        CFG4
      generic map(INIT => x"AABA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1455_2\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_147\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1455\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_frame_type_i\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/drv_bus[460]\, B => 
        \can_top_level_inst/stat_bus[67]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/frame_type\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe20\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[27]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/un3_test_registers_cs\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => scs_c, B => 
        \can_top_level_inst/memory_registers_inst/un3_test_registers_cs_2_Z\, 
        C => \adress_c[11]\, D => \adress_c[8]\, Y => 
        \can_top_level_inst/memory_registers_inst/un3_test_registers_cs_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[5]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[9]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\);
    
    \can_top_level_inst/rx_buffer_inst/rwcnt_com_4_0_.m11\ : CFG4
      generic map(INIT => x"773E")

      port map(A => \can_top_level_inst/rec_dlc[0]\, B => 
        \can_top_level_inst/rec_dlc[3]\, C => 
        \can_top_level_inst/rec_dlc[2]\, D => 
        \can_top_level_inst/rec_dlc[1]\, Y => 
        \can_top_level_inst/rx_buffer_inst/rwcnt_com[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_12[1]\ : 
        CFG4
      generic map(INIT => x"F808")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_full_1_4\, B => 
        \can_top_level_inst/rx_buffer_inst_rx_full_1_5\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => \can_top_level_inst/txtb_prorities_0[1]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1176\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[110]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[3]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[1]\, 
        B => \can_top_level_inst/txtb_hw_cmd.unlock\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_136\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram22_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe22\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram22_0\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_1_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO686, CO => OPEN, P(0) => 
        NET_CC_CONFIG715, P(1) => NET_CC_CONFIG719, P(2) => 
        NET_CC_CONFIG723, P(3) => NET_CC_CONFIG727, P(4) => 
        NET_CC_CONFIG731, P(5) => NET_CC_CONFIG735, P(6) => 
        NET_CC_CONFIG739, P(7) => NET_CC_CONFIG743, P(8) => 
        NET_CC_CONFIG747, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, 
        P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG716, Y3(1) => 
        NET_CC_CONFIG720, Y3(2) => NET_CC_CONFIG724, Y3(3) => 
        NET_CC_CONFIG728, Y3(4) => NET_CC_CONFIG732, Y3(5) => 
        NET_CC_CONFIG736, Y3(6) => NET_CC_CONFIG740, Y3(7) => 
        NET_CC_CONFIG744, Y3(8) => NET_CC_CONFIG748, Y3(9) => 
        ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, 
        Y3A(0) => NET_CC_CONFIG717, Y3A(1) => NET_CC_CONFIG721, 
        Y3A(2) => NET_CC_CONFIG725, Y3A(3) => NET_CC_CONFIG729, 
        Y3A(4) => NET_CC_CONFIG733, Y3A(5) => NET_CC_CONFIG737, 
        Y3A(6) => NET_CC_CONFIG741, Y3A(7) => NET_CC_CONFIG745, 
        Y3A(8) => NET_CC_CONFIG749, Y3A(9) => ADLIB_VCC, Y3A(10)
         => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG718, CC(1) => NET_CC_CONFIG722, CC(2) => 
        NET_CC_CONFIG726, CC(3) => NET_CC_CONFIG730, CC(4) => 
        NET_CC_CONFIG734, CC(5) => NET_CC_CONFIG738, CC(6) => 
        NET_CC_CONFIG742, CC(7) => NET_CC_CONFIG746, CC(8) => 
        NET_CC_CONFIG750, CC(9) => nc290, CC(10) => nc17, CC(11)
         => nc2);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[28]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[28]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[28]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[2]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[21]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[21]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => \can_top_level_inst/drv_bus[19]\, B => 
        \can_top_level_inst/drv_bus[26]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_2_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_3_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_3_Z\, 
        CC => NET_CC_CONFIG2159, P => NET_CC_CONFIG2156, Y3 => 
        NET_CC_CONFIG2157, Y3A => NET_CC_CONFIG2158);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_wmux_2\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_y0_0\, 
        B => \can_top_level_inst/txtb_hw_cmd_index[1]\, C => 
        \can_top_level_inst/N_3899_i\, D => 
        \can_top_level_inst/N_3898_i\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_0_y3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[253]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[8]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[11]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_3[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/un2_crc_ena_15\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_N_865\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, Y
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/un2_crc_ena_15_i\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_36[9]\ : 
        CFG4
      generic map(INIT => x"ACAF")

      port map(A => \timestamp_c[41]\, B => \timestamp_c[9]\, C
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1504\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[19]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIGNT35[8]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[8]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[8]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_7\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_8\, 
        CC => NET_CC_CONFIG1309, P => NET_CC_CONFIG1306, Y3 => 
        NET_CC_CONFIG1307, Y3A => NET_CC_CONFIG1308);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[9]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[18]\);
    
    \can_top_level_inst/memory_registers_inst/un31_status_comb\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_8\, 
        B => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_8\, 
        Y => \can_top_level_inst/memory_registers_inst/N_4605\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram11__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_348\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe11\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_31_1_1[13]\ : 
        CFG4
      generic map(INIT => x"404B")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_2_1[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_31_1_1_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[22]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[22]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[25]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[325]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_388\, 
        B => \can_top_level_inst/tran_word[24]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[11]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_a3[3]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        Y => \can_top_level_inst/txtb_state_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[28]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[28]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_7\ : 
        CFG4
      generic map(INIT => x"1F00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state_0_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1246\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_7_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_RNO\ : 
        CFG4
      generic map(INIT => x"3233")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_2\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_25_sn\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1462_3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_sn_N_7_mux\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_26[10]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => \can_top_level_inst/int_mask[10]\, C => 
        \can_top_level_inst/drv_bus[236]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1532\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[2]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_dec_proc.1.l1_prio_19\ : 
        CFG4
      generic map(INIT => x"135F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_19\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[2]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2626\, Y
         => \can_top_level_inst/memory_registers_inst/N_2691\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[4]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[25]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_1_3\ : 
        CFG4
      generic map(INIT => x"0A8E")

      port map(A => \can_top_level_inst/txtb_prorities_7[1]\, B
         => \can_top_level_inst/txtb_prorities_7[0]\, C => 
        \can_top_level_inst/txtb_prorities_6[1]\, D => 
        \can_top_level_inst/txtb_prorities_6[0]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_1_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[21]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_2\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[415]\, B => 
        \can_top_level_inst/stat_bus[78]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_6\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_7\, 
        CC => NET_CC_CONFIG2090, P => NET_CC_CONFIG2087, Y3 => 
        NET_CC_CONFIG2088, Y3A => NET_CC_CONFIG2089);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[31]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_mem_free[5]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_4_Z\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_5_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_5_Z\, 
        CC => NET_CC_CONFIG2438, P => NET_CC_CONFIG2435, Y3 => 
        NET_CC_CONFIG2436, Y3A => NET_CC_CONFIG2437);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/un3_test_registers_cs_2\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \adress_c[9]\, B => \adress_c[10]\, Y => 
        \can_top_level_inst/memory_registers_inst/un3_test_registers_cs_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_1_RNI1T881\ : 
        ARI1_CC
      generic map(INIT => x"5E44E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_sn_N_2_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_1_S\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_Z[0]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_0\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2373\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_1\, 
        CC => NET_CC_CONFIG1957, P => NET_CC_CONFIG1954, Y3 => 
        NET_CC_CONFIG1955, Y3A => NET_CC_CONFIG1956);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[14]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[14]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[14]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_1_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_2_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_2_Z\, 
        CC => NET_CC_CONFIG1222, P => NET_CC_CONFIG1219, Y3 => 
        NET_CC_CONFIG1220, Y3A => NET_CC_CONFIG1221);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_RNILQRU1[2]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/txtb_parity_error_valid[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_182\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[7]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[179]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_20\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_20_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_33[25]\ : 
        CFG2
      generic map(INIT => x"D")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \timestamp_c[25]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1119\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[10]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[10]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[10]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[4]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[4]\);
    
    \timestamp_ibuf[63]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[63]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[63]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_2_0[2]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[32]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[30]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_2[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_7[30]\ : 
        CFG4
      generic map(INIT => x"FFF8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac28_Z\, 
        B => \can_top_level_inst/drv_bus[59]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_5_Z[30]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_4_Z[30]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_7_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/sw_dor_proc.un8_overrun_condition\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => \data_in_c[3]\, B => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_command_reg_comp_wr_select[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/un8_overrun_condition\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[11]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2635\, Y
         => \can_top_level_inst/memory_registers_inst/N_2700\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[30]\ : 
        CFG4
      generic map(INIT => x"0054")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1124\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_784_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[28]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[28]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_RNO_0[2]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[15]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_87_i_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[159]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNI1KJN5[5]\ : 
        ARI1_CC
      generic map(INIT => x"58DD8")

      port map(A => \can_top_level_inst/stat_bus[76]\, B => 
        \can_top_level_inst/drv_bus[425]\, C => 
        \can_top_level_inst/drv_bus[421]\, D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[4]\, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_4\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_5\, 
        CC => NET_CC_CONFIG2510, P => NET_CC_CONFIG2507, Y3 => 
        NET_CC_CONFIG2508, Y3A => NET_CC_CONFIG2509);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[29]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[24]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[24]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[24]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[3]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[31]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[31]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[31]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram52_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe52\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram52_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[2]\);
    
    
        \can_top_level_inst/bus_sampling_inst/data_edge_detector_inst/rx_data_sync_prev\ : 
        SLE
      port map(D => \can_top_level_inst/rx_data_wbs\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/tq_edge\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst_data_edge_detector_inst_rx_data_sync_prev\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa3\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \adress_c[4]\, B => \adress_c[5]\, C => 
        \adress_c[6]\, D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa3\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_0_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_1_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_1_Z\, 
        CC => NET_CC_CONFIG1218, P => NET_CC_CONFIG1215, Y3 => 
        NET_CC_CONFIG1216, Y3A => NET_CC_CONFIG1217);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/N_4117\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[11]\ : 
        CFG4
      generic map(INIT => x"ACA0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[10]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_388\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[11]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[16]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[12]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[10]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[29]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[29]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[29]\);
    
    \data_in_ibuf[4]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(4), Y => \data_in_ibuf[4]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_33\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[26]\, B => 
        \can_top_level_inst/drv_bus[231]\, C => 
        \can_top_level_inst/drv_bus[264]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_46\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[12]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[13]\, 
        CC => NET_CC_CONFIG515, P => NET_CC_CONFIG512, Y3 => 
        NET_CC_CONFIG513, Y3A => NET_CC_CONFIG514);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_m2_1_0\ : 
        CFG4
      generic map(INIT => x"4045")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_sm0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ide_is_arbitration_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_is_receiver\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_m2_1_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[25]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_pointer_inc_1_i_axbxc2\ : 
        CFG3
      generic map(INIT => x"6A")

      port map(A => \can_top_level_inst/rx_read_pointer[2]\, B
         => \can_top_level_inst/rx_read_pointer[1]\, C => 
        \can_top_level_inst/rx_read_pointer[0]\, Y => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[5]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[18]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[18]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[23]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[31]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIEMC5A[13]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[201]\, D => 
        \can_top_level_inst/stat_bus[233]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_12\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIEMC5A_S[13]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_13\, 
        CC => NET_CC_CONFIG808, P => NET_CC_CONFIG805, Y3 => 
        NET_CC_CONFIG806, Y3A => NET_CC_CONFIG807);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_brs_dbl_buf\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/buffer_md_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tran_brs_dbl_buf_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNO[6]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_195\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_147\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_150\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_126_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm_0[1]\ : 
        CFG4
      generic map(INIT => x"0C5C")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_iv_4[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un22_rx_data_nbs_i_m\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[30]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_dbt_RNO[0]\ : 
        CFG2
      generic map(INIT => x"9")

      port map(A => \can_top_level_inst/drv_bus[35]\, B => 
        \can_top_level_inst/drv_bus[41]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_axb_0_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_21\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[6]\, B => 
        \can_top_level_inst/drv_bus[211]\, C => 
        \can_top_level_inst/drv_bus[244]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_56\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[2]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[3]\, 
        CC => NET_CC_CONFIG475, P => NET_CC_CONFIG472, Y3 => 
        NET_CC_CONFIG473, Y3A => NET_CC_CONFIG474);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[16]\);
    
    \timestamp_ibuf[46]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(46), Y => 
        \timestamp_ibuf[46]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNIGJBH[0]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[0]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry_cy\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[0]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[0]\, 
        CC => NET_CC_CONFIG1678, P => NET_CC_CONFIG1675, Y3 => 
        NET_CC_CONFIG1676, Y3A => NET_CC_CONFIG1677);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[23]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[27]\ : 
        CFG4
      generic map(INIT => x"F0EE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[27]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[27]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[26]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[35]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_774_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[4]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_9\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[19]\, B => 
        \can_top_level_inst/drv_bus[100]\, C => 
        \can_top_level_inst/drv_bus[133]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_67\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[8]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[9]\, 
        CC => NET_CC_CONFIG1803, P => NET_CC_CONFIG1800, Y3 => 
        NET_CC_CONFIG1801, Y3A => NET_CC_CONFIG1802);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[10]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[10]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[1]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[1]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[1]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31[6]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_38_1_0_wmux_RNO[10]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_2_0_Z[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_2_Z[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1536\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram104_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe104\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram104_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[17]\);
    
    \can_top_level_inst/rx_buffer_inst/rx_full_1_4\ : CFG4
      generic map(INIT => x"0001")

      port map(A => \can_top_level_inst/rx_mem_free[7]\, B => 
        \can_top_level_inst/rx_mem_free[6]\, C => 
        \can_top_level_inst/rx_mem_free[5]\, D => 
        \can_top_level_inst/rx_mem_free[4]\, Y => 
        \can_top_level_inst/rx_buffer_inst_rx_full_1_4\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped[15]\ : 
        CFG4
      generic map(INIT => x"13EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[15]\, 
        C => \can_top_level_inst/can_core_inst/crc_21[15]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_1_Z[15]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_30_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_762\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_765\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_30_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[22]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/g0\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \can_top_level_inst/drv_bus[513]\, B => 
        \can_top_level_inst/can_core_inst/act_err_ovr_flag\, C
         => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        D => \can_top_level_inst/err_detected\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_one\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q_RNIBKTQ1[1]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[82]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_0\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_1\, 
        CC => NET_CC_CONFIG333, P => NET_CC_CONFIG330, Y3 => 
        NET_CC_CONFIG331, Y3A => NET_CC_CONFIG332);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram43_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe43\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram43_0\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf[2]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[2]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/buffer_md_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[3]\ : 
        CFG4
      generic map(INIT => x"ECCC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[4]\, D => 
        \can_top_level_inst/txtb_sw_cmd.set_abt\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_139\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_838\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_841\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_y0_0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa11\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe11\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_10_1[11]\ : 
        CFG4
      generic map(INIT => x"00CA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tx_double_parity_error_Z\, 
        B => \can_top_level_inst/drv_bus[187]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1554_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[19]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[319]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_2[23]\ : 
        CFG4
      generic map(INIT => x"4F7F")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[23]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_1_Z[23]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => \can_top_level_inst/drv_bus[228]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_1[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_2_3[19]\ : 
        CFG4
      generic map(INIT => x"0D3D")

      port map(A => \can_top_level_inst/drv_bus[224]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_2_1[19]\, 
        D => \can_top_level_inst/drv_bus[195]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_2_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_RNIR6DV1[4]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[3]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[4]\, 
        CC => NET_CC_CONFIG2041, P => NET_CC_CONFIG2038, Y3 => 
        NET_CC_CONFIG2039, Y3A => NET_CC_CONFIG2040);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un24_err_type_d_1\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/bit_err_arb\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_ack_err\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un24_err_type_d_1_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select[2]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_1\, 
        D => \adress_c[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[11]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[10]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIO8KOB[31]_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIO8KOB[31]_FCNET1\, 
        CO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid\, 
        CC => NET_CC_CONFIG1405, P => NET_CC_CONFIG1402, Y3 => 
        NET_CC_CONFIG1403, Y3A => NET_CC_CONFIG1404);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_RNIM3Q7[2]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_135_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_18\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_18_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[14]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[10]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[8]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_38_1_0_wmux_RNO[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_1_wmux_0_Y[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_2_Z[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1217\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNO_S[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[9]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[9]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[9]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[30]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[30]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[30]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_q.lock\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/operation_control_inst_set_transmitter\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[68]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_22\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[7]\, B
         => \can_top_level_inst/rx_buffer_inst_RAM_data_out[6]\, 
        C => \can_top_level_inst/rx_buffer_inst_RAM_data_out[5]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_22_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2_2[1]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[3]\, B
         => \can_top_level_inst/txtb_sw_cmd.set_rdy\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_149\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[16]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2474\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[25]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_21\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[292]\, B => 
        \can_top_level_inst/rec_ident[21]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_20\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_21\, 
        CC => NET_CC_CONFIG1917, P => NET_CC_CONFIG1914, Y3 => 
        NET_CC_CONFIG1915, Y3A => NET_CC_CONFIG1916);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_2[1]\ : 
        CFG3
      generic map(INIT => x"8C")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_183\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_195\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_70\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[166]\, B => 
        \can_top_level_inst/rec_ident[23]\, C => 
        \can_top_level_inst/drv_bus[199]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_16\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_21_RNIGQC6\ : 
        CFG4
      generic map(INIT => x"9669")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_28_Z\, 
        Y => \can_top_level_inst/N_3897_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[29]\ : 
        SLE
      port map(D => \data_in_c[29]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[15]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[15]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[15]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_36[17]\ : 
        CFG4
      generic map(INIT => x"ACA0")

      port map(A => \timestamp_c[49]\, B => \timestamp_c[17]\, C
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_818\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_34_2[0]\ : 
        CFG4
      generic map(INIT => x"D800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[32]\, C => 
        \can_top_level_inst/stat_bus[220]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1160_2\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_gate_mem_read.ALTB[0]\ : 
        CFG4
      generic map(INIT => x"3500")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_8\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_28_mux\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[6]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ALTB[0]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[4]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[3]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI19RLA[14]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[287]\, C => 
        \can_top_level_inst/stat_bus[271]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_13\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI19RLA_S[14]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_14\, 
        CC => NET_CC_CONFIG2624, P => NET_CC_CONFIG2621, Y3 => 
        NET_CC_CONFIG2622, Y3A => NET_CC_CONFIG2623);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_arbitration_lost_condition\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/can_core_inst/bds_data_out\, C => 
        can_tx_c, D => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_arbitration_lost_condition_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/txtb_prorities_1[2]\, B
         => \can_top_level_inst/txtb_prorities_0[2]\, C => 
        \can_top_level_inst/drv_bus_475_rep2\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/reset_overrun_flag\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_21\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[21]\, B => 
        \can_top_level_inst/drv_bus[321]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_20\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_21\, 
        CC => NET_CC_CONFIG613, P => NET_CC_CONFIG610, Y3 => 
        NET_CC_CONFIG611, Y3A => NET_CC_CONFIG612);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO_0[15]\ : 
        CFG4
      generic map(INIT => x"2755")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[15]\, C => 
        \can_top_level_inst/stat_bus[203]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_37_2_1_1_0[15]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_37_2_1_1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[16]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram90_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe90\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram90_0\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_0[13]\ : 
        CFG4
      generic map(INIT => x"0E00")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rwcnt_com[2]\, B => 
        \can_top_level_inst/rx_buffer_inst/un40_frame_form_w_i\, 
        C => \can_top_level_inst/rec_is_rtr\, D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2206\);
    
    
        \can_top_level_inst/rx_buffer_inst/internal_dor_proc.un12_overrun_condition\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => \data_in_c[2]\, B => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_command_reg_comp_wr_select[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/un12_overrun_condition\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[5]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[5]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[24]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[19]\ : 
        CFG4
      generic map(INIT => x"B830")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[19]\, 
        D => \can_top_level_inst/tran_word[11]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_23\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[9]\, B
         => \can_top_level_inst/rx_buffer_inst_RAM_data_out[8]\, 
        C => \can_top_level_inst/rx_buffer_inst_RAM_data_out[12]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_23_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_clears2\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1299_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_962_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_clear_sm0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[1]\, 
        B => \can_top_level_inst/drv_bus[52]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_0_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_1_Z\, 
        CC => NET_CC_CONFIG2382, P => NET_CC_CONFIG2379, Y3 => 
        NET_CC_CONFIG2380, Y3A => NET_CC_CONFIG2381);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[0]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/int_mask_load\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[1]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/int_mask_load_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[23]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1464_2\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry_cy[0]\ : 
        ARI1_CC
      generic map(INIT => x"45500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\, C
         => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_VCC, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry_cy\, 
        CC => NET_CC_CONFIG1091, P => NET_CC_CONFIG1088, Y3 => 
        NET_CC_CONFIG1089, Y3A => NET_CC_CONFIG1090);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram63__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_347\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe63\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[2]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.1.l2_prio_1_1.CO1\ : 
        CFG4
      generic map(INIT => x"0A8E")

      port map(A => \can_top_level_inst/txtb_prorities_6[1]\, B
         => \can_top_level_inst/txtb_prorities_6[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[23]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_can_fd_1_sqmuxa_RNIDGQL_0\ : 
        CFG4
      generic map(INIT => x"4F00")

      port map(A => \can_top_level_inst/tran_dlc[2]\, B => 
        \can_top_level_inst/tran_dlc[1]\, C => 
        \can_top_level_inst/tran_dlc[0]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_can_fd_1_sqmuxa_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[4]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_139\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62[11]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_Z[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_1_0_wmux_0_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[21]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_mem_free[3]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_2_Z\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_3_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_3_Z\, 
        CC => NET_CC_CONFIG2430, P => NET_CC_CONFIG2427, Y3 => 
        NET_CC_CONFIG2428, Y3A => NET_CC_CONFIG2429);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[27]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_0_y3[12]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_0_y1[12]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_606\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[222]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1[1]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_1_Z[1]\, 
        B => \can_top_level_inst/stat_bus[82]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe1_0_a4\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[0]\, 
        B => \can_top_level_inst/rx_buffer_inst/un5_ram_write_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/un7_index_sat[4]\ : 
        CFG3
      generic map(INIT => x"70")

      port map(A => \adress_c[7]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[6]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_0_cy\ : 
        ARI1_CC
      generic map(INIT => x"4EE00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, C => 
        \can_top_level_inst/prescaler_inst/is_tseg2\, D => 
        ADLIB_GND, FCI => ADLIB_VCC, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_0_cy_Z\, 
        CC => NET_CC_CONFIG1123, P => NET_CC_CONFIG1120, Y3 => 
        NET_CC_CONFIG1121, Y3A => NET_CC_CONFIG1122);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_1_1[21]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[135]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_1_1_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe5\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_6[10]\ : 
        CFG4
      generic map(INIT => x"F2F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1512_1\, 
        D => \can_top_level_inst/drv_bus[26]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1512\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_0[16]\ : 
        CFG4
      generic map(INIT => x"C9C8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_0_1_Z[16]\, 
        D => \can_top_level_inst/drv_bus[254]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_0_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_4[1]\, B
         => \can_top_level_inst/txtb_prorities_5[1]\, C => 
        \can_top_level_inst/drv_bus_475_rep1\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[25]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un4_dst_ctr_d\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => 
        \can_top_level_inst/can_core_inst/stuff_length[0]\, B => 
        \can_top_level_inst/rx_triggers[1]\, C => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_stuff_lvl_reached_NE_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un4_dst_ctr_d_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.lock_i_o4\ : 
        CFG3
      generic map(INIT => x"F4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_958\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_945\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_970\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_0\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[15]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[15]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[15]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[10]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[2]\, 
        D => \can_top_level_inst/drv_bus[48]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_0[22]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_0_1_Z[22]\, 
        B => \can_top_level_inst/txtb_prorities_5[2]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_0_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[15]\);
    
    \timestamp_ibuf[3]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(3), Y => \timestamp_ibuf[3]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_flag_RNO\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_i_3_Z\, 
        B => \can_top_level_inst/can_core_inst/pc_rx_trigger\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1533_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[24]\);
    
    \can_top_level_inst/frame_filters_inst/un1_store_data_f\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/frame_filters_inst_ident_valid_q\, B
         => \can_top_level_inst/store_data\, Y => 
        \can_top_level_inst/store_data_f\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_1[5]\ : 
        CFG4
      generic map(INIT => x"F805")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_1_1_Z[5]\, 
        B => \can_top_level_inst/stat_bus[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_1_Z[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_dbt_Z[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_5_Z\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_14\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[409]\, B => 
        \can_top_level_inst/stat_bus[72]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_0\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_1\, 
        CC => NET_CC_CONFIG2066, P => NET_CC_CONFIG2063, Y3 => 
        NET_CC_CONFIG2064, Y3A => NET_CC_CONFIG2065);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_2[1]\ : 
        CFG4
      generic map(INIT => x"08CC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_183_1\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_2_1_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_195\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_RNO[26]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2682\, D
         => \can_top_level_inst/memory_registers_inst/N_2715\, Y
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_2_0[20]\ : 
        CFG4
      generic map(INIT => x"3708")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_2_0_1_Z[20]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_0_Z[20]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_2_0_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[5]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0_RNIR8GC94[4]\ : 
        ARI1_CC
      generic map(INIT => x"58778")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[4]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_51\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_3\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_4\, 
        CC => NET_CC_CONFIG2123, P => NET_CC_CONFIG2120, Y3 => 
        NET_CC_CONFIG2121, Y3A => NET_CC_CONFIG2122);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa12\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe12\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[10]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[15]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[15]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIHPEQ9[10]\ : 
        ARI1_CC
      generic map(INIT => x"5D827")

      port map(A => \timestamp_c[42]\, B => 
        \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[10]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[10]\, 
        FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_9\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_10\, 
        CC => NET_CC_CONFIG967, P => NET_CC_CONFIG964, Y3 => 
        NET_CC_CONFIG965, Y3A => NET_CC_CONFIG966);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[4]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[12]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2470\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[13]\ : 
        CFG4
      generic map(INIT => x"0023")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1050\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1137\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_818_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[21]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_6_1_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_6_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[34]\, D => 
        \can_top_level_inst/rx_read_pointer[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_826\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[9]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_2[13]\ : 
        CFG4
      generic map(INIT => x"66E6")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_2_1_Z[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_190\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_139\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[3]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIC0M98_S[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[198]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un2_commit_rx_frame\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_mem_free[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => \adress_c[8]\, B => 
        \can_top_level_inst/txtb_allow_bb[0]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_un1_ram_write_0\, 
        D => \adress_c[9]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[17]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_11\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[282]\, B => 
        \can_top_level_inst/rec_ident[11]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_10\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_11\, 
        CC => NET_CC_CONFIG1877, P => NET_CC_CONFIG1874, Y3 => 
        NET_CC_CONFIG1875, Y3A => NET_CC_CONFIG1876);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm_0[4]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[4]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[4]\);
    
    \can_top_level_inst/frame_filters_inst/un2_filter_a_enable_1\ : 
        CFG4
      generic map(INIT => x"F3F5")

      port map(A => \can_top_level_inst/drv_bus[110]\, B => 
        \can_top_level_inst/drv_bus[111]\, C => 
        \can_top_level_inst/stat_bus[156]\, D => 
        \can_top_level_inst/stat_bus[155]\, Y => 
        \can_top_level_inst/frame_filters_inst/un2_filter_a_enable_1_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[23]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[23]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[23]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_87_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_87_FCNET1\, 
        CO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[14]\, 
        CC => NET_CC_CONFIG2313, P => NET_CC_CONFIG2310, Y3 => 
        NET_CC_CONFIG2311, Y3A => NET_CC_CONFIG2312);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y7[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y5[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/txtb_sw_cmd.set_abt_fast\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/tx_command_fast[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_sw_cmd.set_abt_fast\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c[2]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[2]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_18[25]\ : 
        CFG4
      generic map(INIT => x"6420")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[201]\, D => 
        \can_top_level_inst/drv_bus[14]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1104\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_1[5]\ : 
        CFG4
      generic map(INIT => x"98BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_1_1_Z[5]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_113\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_1_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[8]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[7]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[8]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_16\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[210]\, B => 
        \can_top_level_inst/rec_ident[5]\, C => 
        \can_top_level_inst/drv_bus[243]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_61\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram91_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe91\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram91_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_36_1[1]\ : 
        CFG4
      generic map(INIT => x"00D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[1]\, C => 
        \can_top_level_inst/stat_bus[189]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1200_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[28]\ : 
        SLE
      port map(D => \data_in_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[233]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[25]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[5]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[5]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[5]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[18]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2642\, Y
         => \can_top_level_inst/memory_registers_inst/N_2707\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[21]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[29]\ : 
        SLE
      port map(D => \timestamp_c[29]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[29]\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q_1076\ : 
        CFG3
      generic map(INIT => x"FD")

      port map(A => \can_top_level_inst/rx_mof\, B => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[2]\, 
        C => \can_top_level_inst/rx_buffer_inst/rx_mof_1\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2833\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram22__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_345\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe22\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[27]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[7]\ : 
        CFG4
      generic map(INIT => x"FFA2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[2]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_168\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_2\ : 
        CFG4
      generic map(INIT => x"F9F6")

      port map(A => \can_top_level_inst/drv_bus[51]\, B => 
        \can_top_level_inst/drv_bus[52]\, C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_2_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[1]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_2_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[4]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI6BEO2[6]_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI6BEO2[6]_FCNET1\, 
        CO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_7\, 
        CC => NET_CC_CONFIG1601, P => NET_CC_CONFIG1598, Y3 => 
        NET_CC_CONFIG1599, Y3A => NET_CC_CONFIG1600);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[18]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[22]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1463_2\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[22]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8_RNI0A0OE\ : 
        CFG4
      generic map(INIT => x"A820")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_2_1\, 
        B => \can_top_level_inst/prescaler_inst_un2_tq_edge\, C
         => \can_top_level_inst/prescaler_inst_tq_edge_dbt\, D
         => \can_top_level_inst/prescaler_inst_tq_edge_nbt\, Y
         => 
        \can_top_level_inst/prescaler_inst/un5_exit_ph2_immediate_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[27]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_6[1]\, B
         => \can_top_level_inst/txtb_prorities_7[1]\, C => 
        \can_top_level_inst/drv_bus_475_rep1\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\);
    
    \can_top_level_inst/rx_buffer_inst/un3_is_free_wordlto5\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => \can_top_level_inst/rx_frame_count[1]\, B => 
        \can_top_level_inst/rx_frame_count[0]\, C => 
        \can_top_level_inst/rx_buffer_inst/un3_is_free_wordlto5_3_Z\, 
        Y => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[22]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[293]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_5_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_5_FCNET1\, 
        CO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_5_Z\, 
        CC => NET_CC_CONFIG1762, P => NET_CC_CONFIG1759, Y3 => 
        NET_CC_CONFIG1760, Y3A => NET_CC_CONFIG1761);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[3]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_16[10]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => \can_top_level_inst/int_vector[10]\, C => 
        \can_top_level_inst/drv_bus[248]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1522\);
    
    \timestamp_ibuf[46]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[46]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[46]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[12]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[37]\ : 
        CFG4
      generic map(INIT => x"004C")

      port map(A => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_771_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_1[1]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[0]\, 
        B => \can_top_level_inst/stat_bus[68]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_137\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa13\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe13\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y3[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y1[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y9[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[5]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_10_N_2L1\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un7_exit_segm_regular_cry_9\, 
        B => \can_top_level_inst/prescaler_inst/is_tseg2\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_10_N_2L1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_21_RNIK256\ : 
        CFG4
      generic map(INIT => x"9669")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_28_Z\, 
        Y => \can_top_level_inst/N_3899_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[15]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[4]\ : 
        SLE
      port map(D => \timestamp_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[143]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI6F5CA[26]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[26]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[26]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_25\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_26\, 
        CC => NET_CC_CONFIG1381, P => NET_CC_CONFIG1378, Y3 => 
        NET_CC_CONFIG1379, Y3A => NET_CC_CONFIG1380);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => res_n_out_c, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD
         => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[20]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_dest[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa7\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe7\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[21]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60[2]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa20\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \adress_c[2]\, B => \adress_c[3]\, C => 
        \adress_c[5]\, D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI261LC_S[17]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[205]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[31]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[24]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/set_idle_i_17_sqmuxa_0_a4\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => \can_top_level_inst/stat_bus[374]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_21[0]\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_9\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_18_Z[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_11_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_21_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[4]\, 
        C => \can_top_level_inst/txtb_port_b_address[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_len_can_fd_1_sqmuxa\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_data_len_can_2_0_i_0\, 
        B => \can_top_level_inst/stat_bus[156]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_len_can_fd_1_sqmuxa_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[25]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[201]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_34[4]\ : 
        CFG4
      generic map(INIT => x"CCCE")

      port map(A => \can_top_level_inst/trv_delay[4]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1312_2\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1312\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m36_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[406]\, B => 
        \can_top_level_inst/stat_bus[77]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_5_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_6_Z\, 
        CC => NET_CC_CONFIG236, P => NET_CC_CONFIG233, Y3 => 
        NET_CC_CONFIG234, Y3A => NET_CC_CONFIG235);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_o2[2]\ : 
        CFG4
      generic map(INIT => x"CECC")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_72\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        D => \can_top_level_inst/drv_bus[476]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_121\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram34_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe34\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram34_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/bit_gen.1.reg_present_gen.reg_autoclear_gen.reg_value_r_3[1]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select\, 
        C => \data_in_c[1]\, Y => 
        \can_top_level_inst/memory_registers_inst/tx_command[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_m_3_1_0_wmux_0_RNIVBG0V\ : 
        CFG4
      generic map(INIT => x"FF28")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/output_i3_mux\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_i\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_RNI791Q4_Z\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/select_buf_index_i_0_1[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/int_mask_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/int_mask_load_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_mask[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[26]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIKDJDB_S[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[288]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[22]\);
    
    \timestamp_ibuf[33]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[33]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[33]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_5[22]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/drv_bus[3]\, B => 
        \can_top_level_inst/rx_read_pointer[6]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_977\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIEMC5A_S[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[201]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[22]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[22]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[22]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[10]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[10]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[21]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2645\, Y
         => \can_top_level_inst/memory_registers_inst/N_2710\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/un5_transient_state_i_o2\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => \can_top_level_inst/txtb_allow_bb[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_1_0_0_Z[2]\, 
        B => \sbe_c[1]\, C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_0[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI4QDS\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram82_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe82\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram82_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/un1_retr_ctr_q_1.CO1\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => \can_top_level_inst/stat_bus[3]\, B => 
        \can_top_level_inst/stat_bus[2]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retr_ctr_add\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/CO1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_3[29]\ : 
        CFG4
      generic map(INIT => x"BF80")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[1]\, 
        B => \can_top_level_inst/drv_bus[477]\, C => 
        \can_top_level_inst/tran_frame_test.sdlc\, D => 
        \can_top_level_inst/tran_dlc[1]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_14_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2[3]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_Z[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"46600")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[5]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_4\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_5_Z\, 
        CC => NET_CC_CONFIG2680, P => NET_CC_CONFIG2677, Y3 => 
        NET_CC_CONFIG2678, Y3A => NET_CC_CONFIG2679);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[19]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_16\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[16]\, B => 
        \can_top_level_inst/drv_bus[316]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_15\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_16\, 
        CC => NET_CC_CONFIG593, P => NET_CC_CONFIG590, Y3 => 
        NET_CC_CONFIG591, Y3A => NET_CC_CONFIG592);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO_1[13]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_37_2_1_1_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_21_RNIOAT5\ : 
        CFG4
      generic map(INIT => x"9669")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_28_Z\, 
        Y => \can_top_level_inst/N_3900_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIEJKC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un1_tq_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[0]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_4_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_5_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_5_Z\, 
        CC => NET_CC_CONFIG1234, P => NET_CC_CONFIG1231, Y3 => 
        NET_CC_CONFIG1232, Y3A => NET_CC_CONFIG1233);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[13]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[13]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[21]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG1406, P(1) => NET_CC_CONFIG1410, P(2) => 
        NET_CC_CONFIG1414, P(3) => NET_CC_CONFIG1418, P(4) => 
        NET_CC_CONFIG1422, P(5) => NET_CC_CONFIG1426, P(6) => 
        NET_CC_CONFIG1430, P(7) => NET_CC_CONFIG1434, P(8) => 
        NET_CC_CONFIG1438, P(9) => NET_CC_CONFIG1442, P(10) => 
        NET_CC_CONFIG1446, P(11) => ADLIB_VCC, Y3(0) => 
        NET_CC_CONFIG1407, Y3(1) => NET_CC_CONFIG1411, Y3(2) => 
        NET_CC_CONFIG1415, Y3(3) => NET_CC_CONFIG1419, Y3(4) => 
        NET_CC_CONFIG1423, Y3(5) => NET_CC_CONFIG1427, Y3(6) => 
        NET_CC_CONFIG1431, Y3(7) => NET_CC_CONFIG1435, Y3(8) => 
        NET_CC_CONFIG1439, Y3(9) => NET_CC_CONFIG1443, Y3(10) => 
        NET_CC_CONFIG1447, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG1408, Y3A(1) => NET_CC_CONFIG1412, Y3A(2)
         => NET_CC_CONFIG1416, Y3A(3) => NET_CC_CONFIG1420, 
        Y3A(4) => NET_CC_CONFIG1424, Y3A(5) => NET_CC_CONFIG1428, 
        Y3A(6) => NET_CC_CONFIG1432, Y3A(7) => NET_CC_CONFIG1436, 
        Y3A(8) => NET_CC_CONFIG1440, Y3A(9) => NET_CC_CONFIG1444, 
        Y3A(10) => NET_CC_CONFIG1448, Y3A(11) => ADLIB_VCC, CC(0)
         => NET_CC_CONFIG1409, CC(1) => NET_CC_CONFIG1413, CC(2)
         => NET_CC_CONFIG1417, CC(3) => NET_CC_CONFIG1421, CC(4)
         => NET_CC_CONFIG1425, CC(5) => NET_CC_CONFIG1429, CC(6)
         => NET_CC_CONFIG1433, CC(7) => NET_CC_CONFIG1437, CC(8)
         => NET_CC_CONFIG1441, CC(9) => NET_CC_CONFIG1445, CC(10)
         => NET_CC_CONFIG1449, CC(11) => nc302);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[11]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[11]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_m2[20]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[17]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1023\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_3_sqmuxa_i_a2\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_956\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1300\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y7[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y5[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_3[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.err_0_sqmuxa_5\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[8]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa_5\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1[34]\ : 
        CFG4
      generic map(INIT => x"ABBB")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[3]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_arb_1_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_4\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[34]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        C => \can_top_level_inst/drv_bus[34]\, D => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[5]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[5]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[1]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_is_rtr_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_715_i\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_is_rtr_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_ac0_3_RNIA5S8C\ : 
        ARI1_CC
      generic map(INIT => x"56699")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c3\, 
        C => \can_top_level_inst/stat_bus[84]\, D => ADLIB_GND, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_2\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_3\, 
        CC => NET_CC_CONFIG99, P => NET_CC_CONFIG96, Y3 => 
        NET_CC_CONFIG97, Y3A => NET_CC_CONFIG98);
    
    \data_in_ibuf[31]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[31]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[31]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[3]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_20_1\ : 
        CFG3
      generic map(INIT => x"A2")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_26\, 
        C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[6]\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_20_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_0_1[8]\ : 
        CFG3
      generic map(INIT => x"51")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[42]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_0_1_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_7[1]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_7_1_0_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1327\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_968\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1203\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[1]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[8]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNIIJ7M6[6]\ : 
        ARI1_CC
      generic map(INIT => x"58DD8")

      port map(A => \can_top_level_inst/stat_bus[77]\, B => 
        \can_top_level_inst/drv_bus[425]\, C => 
        \can_top_level_inst/drv_bus[422]\, D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[4]\, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_5\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_6\, 
        CC => NET_CC_CONFIG2514, P => NET_CC_CONFIG2511, Y3 => 
        NET_CC_CONFIG2512, Y3A => NET_CC_CONFIG2513);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[25]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_7\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[278]\, B => 
        \can_top_level_inst/rec_ident[7]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_6\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_7\, 
        CC => NET_CC_CONFIG1861, P => NET_CC_CONFIG1858, Y3 => 
        NET_CC_CONFIG1859, Y3A => NET_CC_CONFIG1860);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_FCO\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[23]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_7_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO2523, CO => OPEN, P(0) => 
        NET_CC_CONFIG2536, P(1) => NET_CC_CONFIG2540, P(2) => 
        NET_CC_CONFIG2544, P(3) => NET_CC_CONFIG2548, P(4) => 
        NET_CC_CONFIG2552, P(5) => NET_CC_CONFIG2556, P(6) => 
        NET_CC_CONFIG2560, P(7) => ADLIB_VCC, P(8) => ADLIB_VCC, 
        P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11) => ADLIB_VCC, 
        Y3(0) => NET_CC_CONFIG2537, Y3(1) => NET_CC_CONFIG2541, 
        Y3(2) => NET_CC_CONFIG2545, Y3(3) => NET_CC_CONFIG2549, 
        Y3(4) => NET_CC_CONFIG2553, Y3(5) => NET_CC_CONFIG2557, 
        Y3(6) => NET_CC_CONFIG2561, Y3(7) => ADLIB_VCC, Y3(8) => 
        ADLIB_VCC, Y3(9) => ADLIB_VCC, Y3(10) => ADLIB_VCC, 
        Y3(11) => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG2538, Y3A(1)
         => NET_CC_CONFIG2542, Y3A(2) => NET_CC_CONFIG2546, 
        Y3A(3) => NET_CC_CONFIG2550, Y3A(4) => NET_CC_CONFIG2554, 
        Y3A(5) => NET_CC_CONFIG2558, Y3A(6) => NET_CC_CONFIG2562, 
        Y3A(7) => ADLIB_VCC, Y3A(8) => ADLIB_VCC, Y3A(9) => 
        ADLIB_VCC, Y3A(10) => ADLIB_VCC, Y3A(11) => ADLIB_VCC, 
        CC(0) => NET_CC_CONFIG2539, CC(1) => NET_CC_CONFIG2543, 
        CC(2) => NET_CC_CONFIG2547, CC(3) => NET_CC_CONFIG2551, 
        CC(4) => NET_CC_CONFIG2555, CC(5) => NET_CC_CONFIG2559, 
        CC(6) => NET_CC_CONFIG2563, CC(7) => nc110, CC(8) => 
        nc128, CC(9) => nc244, CC(10) => nc321, CC(11) => nc43);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_7\ : 
        ARI1_CC
      generic map(INIT => x"46600")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[7]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_6_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_7_Z\, 
        CC => NET_CC_CONFIG2688, P => NET_CC_CONFIG2685, Y3 => 
        NET_CC_CONFIG2686, Y3A => NET_CC_CONFIG2687);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_1_0_1[14]\ : 
        CFG3
      generic map(INIT => x"27")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[314]\, C => 
        \can_top_level_inst/drv_bus[190]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_1_0_1_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNIUMJT[6]\ : 
        CFG3
      generic map(INIT => x"2A")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => \can_top_level_inst/txtb_sw_cmd.set_abt\, C => 
        \can_top_level_inst/txtb_sw_cmd_index[5]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_135_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_2[21]\ : 
        CFG4
      generic map(INIT => x"4F50")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_2_1[21]\, 
        B => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_Z[21]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_2_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\);
    
    \data_in_ibuf[10]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[10]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[10]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_RNO[25]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2681\, D
         => \can_top_level_inst/memory_registers_inst/N_2714\, Y
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_o2_0[1]\ : 
        CFG4
      generic map(INIT => x"FF04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_957\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2_0_Z[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_939\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1244\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_978\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[17]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[17]\);
    
    \timestamp_ibuf[47]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[47]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[47]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i[4]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[4]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_28[3]\ : 
        CFG4
      generic map(INIT => x"CCCE")

      port map(A => \can_top_level_inst/rx_write_pointer[3]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1268_2\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1268\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[7]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[7]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[7]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2631\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[4]\ : 
        CFG4
      generic map(INIT => x"ACA0")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[4]\, 
        B => \can_top_level_inst/can_core_inst/crc_17[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_segm_counter_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_25[3]\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => \can_top_level_inst/txtb_state_0[3]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1265\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[7]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2465\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[8]\ : 
        CFG4
      generic map(INIT => x"0E02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[8]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[8]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[8]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_raw_0[5]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNICB062_S[5]\, 
        B => \can_top_level_inst/drv_bus[380]\, C => 
        \can_top_level_inst/drv_bus[374]\, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_41\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[23]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[23]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_3[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y3[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y1[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y9[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[27]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[10]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0\ : 
        RGB
      port map(A => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_Y\, 
        EN => ADLIB_VCC, Y => \AFLSDF_INV_9\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_0\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO[15]\ : 
        CFG4
      generic map(INIT => x"CB0B")

      port map(A => \timestamp_c[47]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_37_2_1_1[15]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_37_1_0_Z[15]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_743\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_0[1]\ : 
        CFG4
      generic map(INIT => x"FF04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/drv_bus_off_reset_q_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[10]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1196\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_0_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/un2_index_satlto2_1\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \adress_c[2]\, B => \adress_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[8]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[8]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[13]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[29]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2487\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[8]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30[5]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[24]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid_d_1_sqmuxa\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un26_is_receiver\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid_d_1_sqmuxa_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram77__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_342\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe77\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[27]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[27]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[27]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[12]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[12]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[12]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[12]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m35_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIT4BLH2[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[47]\, B => 
        \can_top_level_inst/tran_word[15]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_14\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_15\, 
        CC => NET_CC_CONFIG987, P => NET_CC_CONFIG984, Y3 => 
        NET_CC_CONFIG985, Y3A => NET_CC_CONFIG986);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[6]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2532\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2464\, Y
         => \can_top_level_inst/memory_registers_inst/N_2662\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[30]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[30]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_8\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[8]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_7_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_8_FCNET1\, 
        CC => NET_CC_CONFIG2410, P => NET_CC_CONFIG2407, Y3 => 
        NET_CC_CONFIG2408, Y3A => NET_CC_CONFIG2409);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO[24]\ : 
        CFG4
      generic map(INIT => x"3266")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_37_2_1_1[24]\, 
        C => \can_top_level_inst/stat_bus[244]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_37_2_1_2[24]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1085\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select[3]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_1_0_0_Z[2]\, 
        B => \sbe_c[3]\, C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_0[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_8\ : 
        ARI1_CC
      generic map(INIT => x"45500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[89]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_7_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_8_FCNET1\, 
        CC => NET_CC_CONFIG1561, P => NET_CC_CONFIG1558, Y3 => 
        NET_CC_CONFIG1559, Y3A => NET_CC_CONFIG1560);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[409]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_10_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[27]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_0_0_o3[0]\ : 
        CFG3
      generic map(INIT => x"DB")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[4]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[3]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_c3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1192\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI19RLA_S[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[271]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[276]\, B => 
        \can_top_level_inst/rec_ident[5]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_4\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_5\, 
        CC => NET_CC_CONFIG1853, P => NET_CC_CONFIG1850, Y3 => 
        NET_CC_CONFIG1851, Y3A => NET_CC_CONFIG1852);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a4_0[24]\ : 
        CFG4
      generic map(INIT => x"0045")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[14]\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1137\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[0]\ : 
        CFG3
      generic map(INIT => x"23")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[37]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_844_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[30]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[30]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[30]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[13]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.txtb_hw_cmd_d.unlock_5\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/stat_bus[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/unlock_5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa17\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe17\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram102_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe102\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram102_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_a2[0]\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_switch_to_ssp_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1420\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_9_RNI12ECF\ : 
        CFG4
      generic map(INIT => x"2AAA")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_0\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_sjw_mt_zero_0\, 
        C => \can_top_level_inst/prescaler_inst/g1_0\, D => 
        \can_top_level_inst/prescaler_inst/g0_0_3\, Y => 
        \can_top_level_inst/prescaler_inst/un8_exit_segm_regular_tseg1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_0_0\, 
        D => \adress_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram5_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe5\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram5_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[20]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_5\ : 
        CFG4
      generic map(INIT => x"FE0E")

      port map(A => \can_top_level_inst/prescaler_inst/N_7\, B
         => \can_top_level_inst/prescaler_inst/N_8\, C => 
        \can_top_level_inst/prescaler_inst_un2_tq_edge\, D => 
        \can_top_level_inst/prescaler_inst_tq_edge_nbt\, Y => 
        \can_top_level_inst/prescaler_inst/g1_0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/un3_crc_d\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_nxt_Z\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, Y
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[2]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[2]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[2]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_38_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1025\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1040\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_38_1_0_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_806_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_36_1[8]\ : 
        CFG4
      generic map(INIT => x"00D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[8]\, C => 
        \can_top_level_inst/stat_bus[196]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1466_1\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/int_ena_proc.un1_int_ena_clear\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[5]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/un1_int_ena_clear\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_4[2]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[23]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_4[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram1__RNO[0]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[2]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[1]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe0_0_a4\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_346\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[216]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI0ALE1[3]\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[3]\, 
        B => \can_top_level_inst/drv_bus[378]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_2\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI0ALE1_S[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_3\, 
        CC => NET_CC_CONFIG1581, P => NET_CC_CONFIG1578, Y3 => 
        NET_CC_CONFIG1579, Y3A => NET_CC_CONFIG1580);
    
    \data_in_ibuf[15]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[15]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[15]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[4]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[4]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[12]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[12]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[3]\, 
        D => \can_top_level_inst/prescaler_inst/tseg1_dbt[3]\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_28\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[216]\, B => 
        \can_top_level_inst/rec_ident[11]\, C => 
        \can_top_level_inst/drv_bus[249]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_51\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa4\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe4\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m29_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[21]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_63\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[20]\, B => 
        \can_top_level_inst/drv_bus[163]\, C => 
        \can_top_level_inst/drv_bus[196]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_21\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[9]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[10]\, 
        CC => NET_CC_CONFIG730, P => NET_CC_CONFIG727, Y3 => 
        NET_CC_CONFIG728, Y3A => NET_CC_CONFIG729);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_3_1.CO2\ : 
        CFG3
      generic map(INIT => x"D4")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0_Z[2]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO2_1\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2_Z[2]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/un2_data_len_can_2_0_1.CO3\ : 
        CFG4
      generic map(INIT => x"AAA8")

      port map(A => \can_top_level_inst/store_data_word[2]\, B
         => \can_top_level_inst/store_data_word[1]\, C => 
        \can_top_level_inst/store_data_word[0]\, D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_data_len_can_2_0_i_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_9\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[3]\, B => 
        \timestamp_c[34]\, C => \timestamp_c[35]\, D => 
        \can_top_level_inst/tran_word[2]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[0]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[1]\, 
        CC => NET_CC_CONFIG2257, P => NET_CC_CONFIG2254, Y3 => 
        NET_CC_CONFIG2255, Y3A => NET_CC_CONFIG2256);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_rtr_i_a2\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[29]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[32]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1234\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_81_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram18_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe18\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram18_0\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[8]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2534\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2466\, Y
         => \can_top_level_inst/memory_registers_inst/N_2664\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_13[15]\ : 
        CFG4
      generic map(INIT => x"0AC0")

      port map(A => \can_top_level_inst/stat_bus[272]\, B => 
        \can_top_level_inst/drv_bus[286]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_719\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_8\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[279]\, B => 
        \can_top_level_inst/rec_ident[8]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_7\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_8\, 
        CC => NET_CC_CONFIG1865, P => NET_CC_CONFIG1862, Y3 => 
        NET_CC_CONFIG1863, Y3A => NET_CC_CONFIG1864);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_2_0[0]\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_1[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_6_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_GND, CO => OPEN, P(0) => 
        NET_CC_CONFIG47, P(1) => NET_CC_CONFIG51, P(2) => 
        NET_CC_CONFIG55, P(3) => NET_CC_CONFIG59, P(4) => 
        NET_CC_CONFIG63, P(5) => NET_CC_CONFIG67, P(6) => 
        NET_CC_CONFIG71, P(7) => NET_CC_CONFIG75, P(8) => 
        NET_CC_CONFIG79, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, 
        P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG48, Y3(1) => 
        NET_CC_CONFIG52, Y3(2) => NET_CC_CONFIG56, Y3(3) => 
        NET_CC_CONFIG60, Y3(4) => NET_CC_CONFIG64, Y3(5) => 
        NET_CC_CONFIG68, Y3(6) => NET_CC_CONFIG72, Y3(7) => 
        NET_CC_CONFIG76, Y3(8) => NET_CC_CONFIG80, Y3(9) => 
        ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, 
        Y3A(0) => NET_CC_CONFIG49, Y3A(1) => NET_CC_CONFIG53, 
        Y3A(2) => NET_CC_CONFIG57, Y3A(3) => NET_CC_CONFIG61, 
        Y3A(4) => NET_CC_CONFIG65, Y3A(5) => NET_CC_CONFIG69, 
        Y3A(6) => NET_CC_CONFIG73, Y3A(7) => NET_CC_CONFIG77, 
        Y3A(8) => NET_CC_CONFIG81, Y3A(9) => ADLIB_VCC, Y3A(10)
         => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG50, CC(1) => NET_CC_CONFIG54, CC(2) => 
        NET_CC_CONFIG58, CC(3) => NET_CC_CONFIG62, CC(4) => 
        NET_CC_CONFIG66, CC(5) => NET_CC_CONFIG70, CC(6) => 
        NET_CC_CONFIG74, CC(7) => NET_CC_CONFIG78, CC(8) => 
        NET_CC_CONFIG82, CC(9) => nc179, CC(10) => nc157, CC(11)
         => nc36);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe91_0_a4\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[0]\, 
        B => \can_top_level_inst/rx_buffer_inst/un5_ram_write_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_7[28]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac22_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac20_Z\, 
        C => \can_top_level_inst/drv_bus[204]\, D => 
        \can_top_level_inst/drv_bus[142]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_7_Z[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_err_delim_late_i_o4\ : 
        CFG3
      generic map(INIT => x"F7")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_950\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[8]\);
    
    \timestamp_ibuf[44]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[44]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[44]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram88__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_340\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe88\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[25]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[25]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[19]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[19]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[19]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[28]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[0]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => \can_top_level_inst/drv_bus[18]\, B => 
        \can_top_level_inst/drv_bus[25]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_1_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_2_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_2_Z\, 
        CC => NET_CC_CONFIG2155, P => NET_CC_CONFIG2152, Y3 => 
        NET_CC_CONFIG2153, Y3A => NET_CC_CONFIG2154);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram122__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_352\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe122\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.un3_is_receiver\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        C => \can_top_level_inst/drv_bus[507]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_match\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_is_receiver\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[23]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram34_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram35_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\);
    
    \timestamp_ibuf[22]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[22]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[22]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[21]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22[21]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[29]\);
    
    \timestamp_ibuf[33]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(33), Y => 
        \timestamp_ibuf[33]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI2IFV3[6]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[6]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_5\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_6\, 
        CC => NET_CC_CONFIG1301, P => NET_CC_CONFIG1298, Y3 => 
        NET_CC_CONFIG1299, Y3A => NET_CC_CONFIG1300);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[317]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_1_Z[0]\, 
        B => \sbe_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNICHKC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\);
    
    \timestamp_ibuf[59]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(59), Y => 
        \timestamp_ibuf[59]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\);
    
    \can_top_level_inst/rx_buffer_inst/data_overrun_flg\ : SLE
      port map(D => \can_top_level_inst/drv_bus_i[353]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un8_overrun_condition\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_data_overrun\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_2[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_16_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram116_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe116\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram116_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_30_1_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_30_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_769\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_772\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_774\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_31_1[13]\ : 
        CFG4
      generic map(INIT => x"6765")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_31_1_1_Z[13]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_2_0_1_Z[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_31_1_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_0_1.CO1\ : 
        CFG4
      generic map(INIT => x"7150")

      port map(A => \can_top_level_inst/txtb_prorities_0[1]\, B
         => \can_top_level_inst/txtb_prorities_0[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_4\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[4]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_161_2\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_139\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_170\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2[0]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_14_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_24\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_21_Z[0]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_20_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[274]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[24]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[324]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[10]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[10]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[10]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4817\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1[16]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_1_Z[16]\, 
        B => \can_top_level_inst/rx_mem_free[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[9]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[9]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa18\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe18\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[27]\ : 
        SLE
      port map(D => \timestamp_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[4]\, 
        C => \can_top_level_inst/txtb_port_b_address[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[25]\ : 
        CFG4
      generic map(INIT => x"F0EE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[25]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[25]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[24]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[3]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_126_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[33]\, 
        C => \can_top_level_inst/rec_ident[1]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2246\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[27]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8_RNO[7]\ : 
        ARI1_CC
      generic map(INIT => x"4E22E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_s_7_S\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_sn_N_2_i\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[7]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_6\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2379\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG1981, P => 
        NET_CC_CONFIG1978, Y3 => NET_CC_CONFIG1979, Y3A => 
        NET_CC_CONFIG1980);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q_RNO[13]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_15[12]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_2\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1012\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[10]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select\, 
        B => \sbe_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_1_1[23]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[199]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_1_1_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[16]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[15]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[16]\);
    
    \srd_ibuf/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => srd_c, E => ADLIB_GND, YIN => \srd_ibuf/YIN\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[19]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[19]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[19]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[411]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/int_stat_proc.un2_int_status_set\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/stat_bus[253]\, B => 
        \can_top_level_inst/int_mask[2]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/un2_int_status_set\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_2\ : 
        CFG4
      generic map(INIT => x"963C")

      port map(A => \can_top_level_inst/drv_bus[56]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[2]\, 
        C => \can_top_level_inst/drv_bus[58]\, D => 
        \can_top_level_inst/drv_bus[57]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_2_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_10_RNO[26]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac19_0_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => \can_top_level_inst/drv_bus[231]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[634]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[13]\);
    
    \timestamp_ibuf[52]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[52]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[52]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa4\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe4\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[28]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2652\, Y
         => \can_top_level_inst/memory_registers_inst/N_2717\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[6]\ : 
        CFG4
      generic map(INIT => x"F8F0")

      port map(A => \can_top_level_inst/N_4606\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_139\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_167\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_182\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.1.l2_prio_1.CO2\ : 
        CFG3
      generic map(INIT => x"B2")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[2]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_6\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_120\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[21]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1562\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1565\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_y0_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[15]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[15]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[15]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[15]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[4]\, 
        B => \can_top_level_inst/tran_word[23]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[14]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q_RNISDRO5[8]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[89]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_7\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[8]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG361, P => 
        NET_CC_CONFIG358, Y3 => NET_CC_CONFIG359, Y3A => 
        NET_CC_CONFIG360);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/un2_res_core_d_n_i\, 
        CLK => \I_1/U0_RGB1_RGB7_rgb_net_1\, EN => ADLIB_VCC, ALn
         => res_n_out_c, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD
         => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram19_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe19\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram19_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa11\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe11\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m35_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[24]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_a2[1]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => \can_top_level_inst/tx_arbitrator_inst/N_109\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_109_0\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, D
         => \can_top_level_inst/tx_arbitrator_inst/N_104\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_130\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNIMA301[6]\ : 
        CFG3
      generic map(INIT => x"4C")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_129_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31[10]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram36_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram37_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m17_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/int_status\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/un2_int_status_set\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/un1_int_status_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_vector[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_1[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8_2\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_1_2_Z[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/addr_dec_3[1]\, 
        D => \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_1_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[153]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[4]\, 
        C => \can_top_level_inst/txtb_port_b_address[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.1.next_bytes_gen.shift_reg_in_2[1]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/stat_bus[372]\, B => 
        \can_top_level_inst/store_data_word[7]\, C => 
        \can_top_level_inst/can_core_inst/bds_data_out\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/shift_reg_in_2[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[234]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_9[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[304]\, C => 
        \can_top_level_inst/drv_bus[180]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1287\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[27]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[19]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1[0]\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \can_top_level_inst/stat_bus[89]\, B => 
        \can_top_level_inst/stat_bus[88]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNITG6E_Y\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa19\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe19\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_798_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[17]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/fsm_wait_state_q_RNO_0\ : 
        CFG4
      generic map(INIT => x"0A02")

      port map(A => \can_top_level_inst/drv_bus[472]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid\, 
        C => \can_top_level_inst/tx_arbitrator_inst/N_109\, D => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[15]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_100_i_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_0_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[0]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_i[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[8]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[8]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[7]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s_Z[8]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG324, P => 
        NET_CC_CONFIG321, Y3 => NET_CC_CONFIG322, Y3A => 
        NET_CC_CONFIG323);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[11]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d[2]\ : 
        CFG4
      generic map(INIT => x"4888")

      port map(A => \can_top_level_inst/stat_bus[302]\, B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/reg_q\, 
        C => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/CO0_0\, 
        D => \can_top_level_inst/stat_bus[301]\, Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[13]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_16\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[400]\, B => 
        \can_top_level_inst/stat_bus[81]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_0\, 
        CC => NET_CC_CONFIG1168, P => NET_CC_CONFIG1165, Y3 => 
        NET_CC_CONFIG1166, Y3A => NET_CC_CONFIG1167);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[6]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[34]\, 
        C => \can_top_level_inst/rec_ident[2]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2247\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err\ : 
        CFG4
      generic map(INIT => x"A2A0")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_form_err_i_0_sqmuxa_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/form_err\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/command_reg_comp/wr_select_cZ[0]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/access_active_1[0]\, 
        C => \sbe_c[0]\, Y => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_command_reg_comp_wr_select[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[23]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8_2[5]\ : 
        CFG4
      generic map(INIT => x"A820")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => \can_top_level_inst/rx_mem_free[5]\, D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_5_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNINUGF3_S[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[261]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[26]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[231]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[11]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO[11]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_37_2_0_Z[11]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_37_2_Z[11]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1581\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[22]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_5[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_0_y0[18]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_830\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_833\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_0_y1[18]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_s_0_2645\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_ena_q_Z\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_VCC, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_s_0_2645_FCO\, 
        CC => NET_CC_CONFIG1210, P => NET_CC_CONFIG1207, Y3 => 
        NET_CC_CONFIG1208, Y3A => NET_CC_CONFIG1209);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => \adress_c[10]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_un1_ram_write_0\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        D => \adress_c[8]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[3]\, 
        D => \can_top_level_inst/drv_bus[32]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[23]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[23]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[23]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2647\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_ena_reg_proc.un21_ctrl_signal_upd_i_2\ : 
        CFG4
      generic map(INIT => x"FFF4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_944\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un21_ctrl_signal_upd_i_0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1267_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un21_ctrl_signal_upd_i_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNI5MUH3[0]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/txtb_port_b_address[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/N_4157\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/N_4158\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/un4_tst_ena\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[19]\, B
         => 
        \can_top_level_inst/test_registers_out.tst_control[0]\, C
         => 
        \can_top_level_inst/memory_registers_inst_test_registers_in_tst_rdata_sn_N_8\, 
        D => \can_top_level_inst/test_registers_out.tst_dest[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[9]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[9]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[7]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[6]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s_Z[7]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG2374, P => 
        NET_CC_CONFIG2371, Y3 => NET_CC_CONFIG2372, Y3A => 
        NET_CC_CONFIG2373);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[4]\, 
        C => \can_top_level_inst/txtb_port_b_address[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[466]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_13\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[13]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_12_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_13_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_13_Z\, 
        CC => NET_CC_CONFIG1266, P => NET_CC_CONFIG1263, Y3 => 
        NET_CC_CONFIG1264, Y3A => NET_CC_CONFIG1265);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe31_0_a4\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[0]\, 
        B => \can_top_level_inst/rx_buffer_inst/un5_ram_write_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[22]\);
    
    \timestamp_ibuf[20]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(20), Y => 
        \timestamp_ibuf[20]/YIN\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[28]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[60]\, 
        C => \can_top_level_inst/rec_ident[28]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2273\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominants2\ : 
        CFG4
      generic map(INIT => x"FDFC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_2\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_25_sn\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1462_3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_sm0\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[31]\ : 
        CFG4
      generic map(INIT => x"EC00")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[31]\, B
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[24]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram94__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_351\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe94\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_34[10]\ : 
        CFG4
      generic map(INIT => x"CECC")

      port map(A => \can_top_level_inst/stat_bus[198]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1540_2\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1540\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[31]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIUNTL3[3]\ : 
        ARI1_CC
      generic map(INIT => x"5D827")

      port map(A => \timestamp_c[35]\, B => 
        \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[3]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[3]\, 
        FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_2\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_3\, 
        CC => NET_CC_CONFIG939, P => NET_CC_CONFIG936, Y3 => 
        NET_CC_CONFIG937, Y3A => NET_CC_CONFIG938);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[26]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram37_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe37\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram37_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[2]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_RNIV3EQ[6]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[3]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[6]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/g1_0_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[40]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[23]\ : 
        SLE
      port map(D => \timestamp_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_a3[1]\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \can_top_level_inst/store_metadata\, B => 
        \can_top_level_inst/rec_abort\, C => 
        \can_top_level_inst/rx_buffer_inst/reset_overrun_flag\, D
         => \can_top_level_inst/frame_filters_inst_ident_valid_q\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[8]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lto3\ : 
        CFG4
      generic map(INIT => x"F2FC")

      port map(A => \can_top_level_inst/rec_dlc[0]\, B => 
        \can_top_level_inst/rec_dlc[2]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lto3_0_0_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_data_len_can_2_0_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lt4\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/tx_double_parity_error\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/command[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/tx_double_parity_error_1_sqmuxa_i_Z\, 
        ALn => res_n_out_c, ADn => ADLIB_VCC, SLn => ADLIB_VCC, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/tx_double_parity_error_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIS1AF[10]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[10]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[10]\, 
        Y => \can_top_level_inst/tran_word[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[29]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => \adress_c[11]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_1_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        D => \adress_c[8]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[26]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[26]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_buf_cmds_proc.un26_is_receiver\ : 
        CFG3
      generic map(INIT => x"0B")

      port map(A => \can_top_level_inst/drv_bus[508]\, B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1317_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un26_is_receiver\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_5[21]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/drv_bus[2]\, B => 
        \can_top_level_inst/rx_read_pointer[5]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_939\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[7]\);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_s1_0_a2\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        Y => \can_top_level_inst/stat_bus[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[15]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[3]\, 
        D => \can_top_level_inst/prescaler_inst/tseg1_nbt[3]\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_i_o2[17]\ : 
        CFG4
      generic map(INIT => x"FFDF")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1194\, 
        C => \can_top_level_inst/tran_frame_test.tprm[1]\, D => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1214\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_ena_0_a2\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1246\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_36[23]\ : 
        CFG4
      generic map(INIT => x"88D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[55]\, C => 
        \can_top_level_inst/stat_bus[243]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1046\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[0]\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_dlc_com[2]\ : SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/tran_dlc[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/retr_ctr_add_block_RNIIK831\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \can_top_level_inst/drv_bus[465]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/retr_ctr_add_block_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retr_ctr_clear\, 
        D => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/retr_ctr_add_sn_N_5_mux\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_1_2[1]\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        B => \adress_c[7]\, C => \adress_c[5]\, D => 
        \adress_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_1_2_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_19[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/stat_bus[261]\, B => 
        \can_top_level_inst/drv_bus[20]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1297\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1[2]\ : 
        CFG3
      generic map(INIT => x"5D")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_26\, 
        C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un3_reinteg_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_3_Z\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_4_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_4_Z\, 
        CC => NET_CC_CONFIG2235, P => NET_CC_CONFIG2232, Y3 => 
        NET_CC_CONFIG2233, Y3A => NET_CC_CONFIG2234);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1[10]\ : 
        CFG4
      generic map(INIT => x"D9C8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1523\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1522\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_21_RNI093B\ : 
        CFG4
      generic map(INIT => x"9669")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_28_Z\, 
        Y => \can_top_level_inst/N_3903_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_go_to_suspend\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/stat_bus[108]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_go_to_suspend_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[23]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[104]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_790_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe10\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask.un7_flip_mask_axbxc3\ : 
        CFG4
      generic map(INIT => x"666A")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[3]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[2]\, C => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, D => 
        \can_top_level_inst/tran_frame_test.tprm[1]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a4[2]\ : 
        CFG4
      generic map(INIT => x"002A")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_931\, 
        C => \can_top_level_inst/can_core_inst/bds_data_out\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_Z[4]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1116\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un31_err_frm_req_i\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/form_err\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un26_err_frm_req_i_1_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_1250\, 
        D => \can_top_level_inst/tran_frame_parity_error\, Y => 
        \can_top_level_inst/err_detected\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_4[21]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => \can_top_level_inst/drv_bus[508]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => \can_top_level_inst/drv_bus[164]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_938\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/op_eq.un2_ctrl_ctr_one\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un2_ctrl_ctr_one_1\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[8]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_one\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_820_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_RNI39U72\ : 
        CFG4
      generic map(INIT => x"F060")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/CO0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[6]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_s1\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un2_compl_ctr_div_32_plus_5_sat_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[6]\ : 
        CFG3
      generic map(INIT => x"D0")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[6]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_167\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_14\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[14]\, B => 
        \can_top_level_inst/drv_bus[314]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_13\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_14\, 
        CC => NET_CC_CONFIG585, P => NET_CC_CONFIG582, Y3 => 
        NET_CC_CONFIG583, Y3A => NET_CC_CONFIG584);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/int_stat_proc.un1_int_status_clear\ : 
        CFG4
      generic map(INIT => x"F8F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_stat_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => \sbe_c[1]\, C => 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/un2_int_status_set\, 
        D => \data_in_c[11]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/un1_int_status_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_1_0[4]\ : 
        CFG3
      generic map(INIT => x"1B")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_1[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram96_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram97_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[14]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[14]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[14]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_8\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_8\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[8]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_7_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_8_Z\, 
        CC => NET_CC_CONFIG159, P => NET_CC_CONFIG156, Y3 => 
        NET_CC_CONFIG157, Y3A => NET_CC_CONFIG158);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[11]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_ena_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_first_ssp_d_Z\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un6_sspc_ena_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_ena_q_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/N_4157\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_dlc_Z[2]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[1]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_dlc\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_dlc[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_2[22]\ : 
        CFG4
      generic map(INIT => x"4F50")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_2_1[22]\, 
        B => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_Z[22]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_2_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_842_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state_0_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram107__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_344\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe107\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[25]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[6]\, CLK
         => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[20]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_0[2]\, 
        B => \sbe_c[0]\, C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_1_0_Z[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIMNFH9_S[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[200]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1[2]\ : 
        CFG4
      generic map(INIT => x"CECC")

      port map(A => \can_top_level_inst/is_bus_off\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[2]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_968\, 
        D => \can_top_level_inst/can_core_inst/pc_rx_trigger\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_16\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[148]\, B => 
        \can_top_level_inst/rec_ident[5]\, C => 
        \can_top_level_inst/drv_bus[181]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_61\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_base_id_0_a4\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_950\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[33]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[13]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[9]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[7]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[4]\, 
        C => \can_top_level_inst/txtb_port_b_address[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe56_0_a4\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_352\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask.un7_flip_mask_axbxc1_RNI8DQ31\ : 
        CFG4
      generic map(INIT => x"0063")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[3]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[4]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_c3\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[1]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[1]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[1]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_19\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[19]\, B => 
        \can_top_level_inst/drv_bus[319]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_18\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_19\, 
        CC => NET_CC_CONFIG605, P => NET_CC_CONFIG602, Y3 => 
        NET_CC_CONFIG603, Y3A => NET_CC_CONFIG604);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[30]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[30]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\);
    
    \data_in_ibuf[23]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(23), Y => \data_in_ibuf[23]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram71__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_353\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe71\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[9]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_75\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[8]\, B => 
        \can_top_level_inst/drv_bus[151]\, C => 
        \can_top_level_inst/drv_bus[184]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_11\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[3]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[4]\, 
        CC => NET_CC_CONFIG706, P => NET_CC_CONFIG703, Y3 => 
        NET_CC_CONFIG704, Y3A => NET_CC_CONFIG705);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[4]\ : 
        CFG4
      generic map(INIT => x"F888")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[15]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[0]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[0]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4810\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30[7]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[30]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[30]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[30]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2654\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1[2]\ : 
        CFG4
      generic map(INIT => x"8D89")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_1_Z[2]\, 
        D => \can_top_level_inst/int_mask[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[25]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[21]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2266\, B
         => \can_top_level_inst/rx_buffer_inst/N_4281\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[3]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI8CSH61[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[44]\, B => 
        \can_top_level_inst/tran_word[12]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_11\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_12\, 
        CC => NET_CC_CONFIG975, P => NET_CC_CONFIG972, Y3 => 
        NET_CC_CONFIG973, Y3A => NET_CC_CONFIG974);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[4]\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_161_2\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_179\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_160\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r_10_rep1\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus_475_rep1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[12]\ : 
        CFG4
      generic map(INIT => x"0B08")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_939\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[25]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_820_i\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI2E6D1_S[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[280]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31[12]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[147]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[21]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[15]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_10[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y21[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y9[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[4]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_wmux_10_Y[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.1.l2_prio_1.CO1\ : 
        CFG4
      generic map(INIT => x"0A8E")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[1]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_6\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/parity_flag_proc.un3_drv_clr_rxpe\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => \data_in_c[8]\, B => 
        \can_top_level_inst/rx_buffer_inst/un5_drv_read_start\, C
         => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_command_reg_comp_wr_select[1]\, 
        Y => \can_top_level_inst/rx_buffer_inst/un3_drv_clr_rxpe\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_h_sync_valid_2_RNISN3L05\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[5]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_h_sync_valid_2_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[5]\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_4\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_5\, 
        CC => NET_CC_CONFIG1433, P => NET_CC_CONFIG1430, Y3 => 
        NET_CC_CONFIG1431, Y3A => NET_CC_CONFIG1432);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[14]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[8]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[14]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[14]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_0_y3[24]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_0_y1[24]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1079\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[14]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[14]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[14]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_0_0[2]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[6]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_0_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[7]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/int_ena_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/un1_int_ena_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_ena[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[187]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2[0]\ : 
        CFG4
      generic map(INIT => x"AAA8")

      port map(A => \can_top_level_inst/stat_bus[68]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_111\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[6]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/store_last_txtb_index\);
    
    \timestamp_ibuf[28]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[28]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[28]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_0[26]\ : 
        CFG4
      generic map(INIT => x"FEFC")

      port map(A => \timestamp_c[58]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac2_Z\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_0_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_1_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_1_1_y0[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1213\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1214\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_1_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[10]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_33\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[26]\, B => 
        \can_top_level_inst/drv_bus[107]\, C => 
        \can_top_level_inst/drv_bus[140]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_46\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[12]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[13]\, 
        CC => NET_CC_CONFIG1819, P => NET_CC_CONFIG1816, Y3 => 
        NET_CC_CONFIG1817, Y3A => NET_CC_CONFIG1818);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_21\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_21_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[16]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_0_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO524, CO => CI_TO_CO525, P(0) => 
        NET_CC_CONFIG562, P(1) => NET_CC_CONFIG566, P(2) => 
        NET_CC_CONFIG570, P(3) => NET_CC_CONFIG574, P(4) => 
        NET_CC_CONFIG578, P(5) => NET_CC_CONFIG582, P(6) => 
        NET_CC_CONFIG586, P(7) => NET_CC_CONFIG590, P(8) => 
        NET_CC_CONFIG594, P(9) => NET_CC_CONFIG598, P(10) => 
        NET_CC_CONFIG602, P(11) => NET_CC_CONFIG606, Y3(0) => 
        NET_CC_CONFIG563, Y3(1) => NET_CC_CONFIG567, Y3(2) => 
        NET_CC_CONFIG571, Y3(3) => NET_CC_CONFIG575, Y3(4) => 
        NET_CC_CONFIG579, Y3(5) => NET_CC_CONFIG583, Y3(6) => 
        NET_CC_CONFIG587, Y3(7) => NET_CC_CONFIG591, Y3(8) => 
        NET_CC_CONFIG595, Y3(9) => NET_CC_CONFIG599, Y3(10) => 
        NET_CC_CONFIG603, Y3(11) => NET_CC_CONFIG607, Y3A(0) => 
        NET_CC_CONFIG564, Y3A(1) => NET_CC_CONFIG568, Y3A(2) => 
        NET_CC_CONFIG572, Y3A(3) => NET_CC_CONFIG576, Y3A(4) => 
        NET_CC_CONFIG580, Y3A(5) => NET_CC_CONFIG584, Y3A(6) => 
        NET_CC_CONFIG588, Y3A(7) => NET_CC_CONFIG592, Y3A(8) => 
        NET_CC_CONFIG596, Y3A(9) => NET_CC_CONFIG600, Y3A(10) => 
        NET_CC_CONFIG604, Y3A(11) => NET_CC_CONFIG608, CC(0) => 
        NET_CC_CONFIG565, CC(1) => NET_CC_CONFIG569, CC(2) => 
        NET_CC_CONFIG573, CC(3) => NET_CC_CONFIG577, CC(4) => 
        NET_CC_CONFIG581, CC(5) => NET_CC_CONFIG585, CC(6) => 
        NET_CC_CONFIG589, CC(7) => NET_CC_CONFIG593, CC(8) => 
        NET_CC_CONFIG597, CC(9) => NET_CC_CONFIG601, CC(10) => 
        NET_CC_CONFIG605, CC(11) => NET_CC_CONFIG609);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[22]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q[3]\ : SLE
      port map(D => \can_top_level_inst/rx_buffer_inst/N_3927_i\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[30]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_3\ : 
        CFG4
      generic map(INIT => x"BBFB")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1324\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_949\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_3_Z\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c2_RNIAUK91\ : 
        CFG4
      generic map(INIT => x"9699")

      port map(A => \can_top_level_inst/drv_bus[11]\, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[3]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c2_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[2]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/N_3918\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[26]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2_2[1]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[5]\, B
         => \can_top_level_inst/txtb_sw_cmd.set_rdy\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_149\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_5\ : 
        CFG4
      generic map(INIT => x"CCEC")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_2_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/frame_type\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_5_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[6]\);
    
    \sbe_ibuf[3]/U_IOPAD\ : IOPAD_IN
      port map(PAD => sbe(3), Y => \sbe_ibuf[3]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c2_RNIN6GE2\ : 
        CFG4
      generic map(INIT => x"F6F9")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[2]\, 
        B => \can_top_level_inst/drv_bus[2]\, C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_NE_0\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c2_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_NE_1\);
    
    \can_top_level_inst/frame_filters_inst/un2_filter_a_enable_0\ : 
        CFG4
      generic map(INIT => x"3F5F")

      port map(A => \can_top_level_inst/drv_bus[112]\, B => 
        \can_top_level_inst/drv_bus[113]\, C => 
        \can_top_level_inst/stat_bus[156]\, D => 
        \can_top_level_inst/stat_bus[155]\, Y => 
        \can_top_level_inst/frame_filters_inst/un2_filter_a_enable_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNIIMVO4[4]\ : 
        ARI1_CC
      generic map(INIT => x"58DD8")

      port map(A => \can_top_level_inst/stat_bus[75]\, B => 
        \can_top_level_inst/drv_bus[425]\, C => 
        \can_top_level_inst/drv_bus[420]\, D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[4]\, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_3\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_4\, 
        CC => NET_CC_CONFIG2506, P => NET_CC_CONFIG2503, Y3 => 
        NET_CC_CONFIG2504, Y3A => NET_CC_CONFIG2505);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[29]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI8J6I[0]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[0]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_GND, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_0\, 
        CC => NET_CC_CONFIG1277, P => NET_CC_CONFIG1274, Y3 => 
        NET_CC_CONFIG1275, Y3A => NET_CC_CONFIG1276);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[24]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[56]\, 
        C => \can_top_level_inst/rec_ident[24]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2269\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[25]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[25]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[29]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_1\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_cnst[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1012\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m16\ : 
        CFG4
      generic map(INIT => x"A8FD")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_16_0_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_16_0_2\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_13_0\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_17_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/destuff_enable\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[305]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram58__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_352\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe58\);
    
    \timestamp_ibuf[58]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[58]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[58]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_2_0_1[21]\ : 
        CFG3
      generic map(INIT => x"35")

      port map(A => \can_top_level_inst/stat_bus[278]\, B => 
        \can_top_level_inst/drv_bus[292]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_2_0_1_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[508]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\);
    
    \data_out_obuf[21]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[21]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[21]/DOUT\, EOUT => 
        \data_out_obuf[21]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[475]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_28_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/int_mask[11]\, D => 
        \can_top_level_inst/drv_bus[237]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_28_1_0_y0[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_bds_data_in\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[0]\, 
        Y => \can_top_level_inst/can_core_inst/un2_bds_data_in_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[21]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_0[9]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_0_1_Z[9]\, 
        B => \can_top_level_inst/txtb_prorities_2[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_0_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r_RNICJ1F[6]\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        Y => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate_3_0_.m7\ : 
        CFG3
      generic map(INIT => x"AE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_8_1\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[5]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lt4\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_8\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/curr_state[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/curr_state_ns[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/is_bus_off\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[10]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[15]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[11]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[9]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_19[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/stat_bus[259]\, B => 
        \can_top_level_inst/drv_bus[18]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1221\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_len_8_to_64_integer_0_a3[3]\ : 
        CFG3
      generic map(INIT => x"13")

      port map(A => \can_top_level_inst/store_data_word[1]\, B
         => \can_top_level_inst/store_data_word[0]\, C => 
        \can_top_level_inst/can_core_inst/bds_data_out\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_len_8_to_64_integer[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_3[35]\ : 
        CFG4
      generic map(INIT => x"FFAE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_3_1_Z[35]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_973\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[35]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_3_Z[35]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[14]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[14]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[14]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[25]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[6]\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        C => \can_top_level_inst/txtb_parity_error_valid[7]\, D
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_195\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_174\);
    
    \timestamp_ibuf[40]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[40]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[40]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[1]\ : 
        CFG4
      generic map(INIT => x"0A0B")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        B => \can_top_level_inst/txtb_sw_cmd.set_ety\, C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_141\, 
        D => \can_top_level_inst/txtb_allow_bb[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_nbt_Z[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_2_S\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[28]\ : 
        CFG4
      generic map(INIT => x"EC00")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[28]\, B
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[24]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI28RV8[21]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[21]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[21]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_20\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_21\, 
        CC => NET_CC_CONFIG1361, P => NET_CC_CONFIG1358, Y3 => 
        NET_CC_CONFIG1359, Y3A => NET_CC_CONFIG1360);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[20]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21[20]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\);
    
    
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_18\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => \data_in_c[27]\, B => \data_in_c[26]\, C => 
        \data_in_c[25]\, D => \data_in_c[24]\, Y => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_18_Z\);
    
    \timestamp_ibuf[35]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(35), Y => 
        \timestamp_ibuf[35]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO[0]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_36\, 
        B => 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_28_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/N_3898\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_tx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[78]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[31]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNILDDJ4[4]\ : 
        ARI1_CC
      generic map(INIT => x"5D827")

      port map(A => \timestamp_c[36]\, B => 
        \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[4]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[4]\, 
        FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_3\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_4\, 
        CC => NET_CC_CONFIG943, P => NET_CC_CONFIG940, Y3 => 
        NET_CC_CONFIG941, Y3A => NET_CC_CONFIG942);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[17]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[49]\, 
        C => \can_top_level_inst/rec_ident[17]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2262\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_tx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[74]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[18]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[99]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[15]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_14_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_14_FCNET1\, 
        CO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_14_Z\, 
        CC => NET_CC_CONFIG458, P => NET_CC_CONFIG455, Y3 => 
        NET_CC_CONFIG456, Y3A => NET_CC_CONFIG457);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2[0]\ : 
        CFG4
      generic map(INIT => x"35FF")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[15]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[6]\, C => 
        \can_top_level_inst/drv_bus[475]\, D => 
        \can_top_level_inst/txtb_sw_cmd.set_ety\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_151\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_1_3_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_control[0]\, D
         => \can_top_level_inst/test_registers_out.tst_wdata[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_1_3_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[24]\);
    
    \adress_ibuf[9]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \adress_c[9]\, E => ADLIB_GND, YIN => 
        \adress_ibuf[9]/YIN\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_0\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[15]\, B
         => \can_top_level_inst/can_core_inst/crc_17[14]\, C => 
        \can_top_level_inst/can_core_inst/crc_17[15]\, D => 
        \can_top_level_inst/store_data_word[14]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[6]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[7]\, 
        CC => NET_CC_CONFIG2555, P => NET_CC_CONFIG2552, Y3 => 
        NET_CC_CONFIG2553, Y3A => NET_CC_CONFIG2554);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_0[1]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => \can_top_level_inst/txtb_hw_cmd.arbl\, B => 
        \can_top_level_inst/txtb_hw_cmd.err\, Y => 
        \can_top_level_inst/N_4606\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa2\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \adress_c[4]\, B => \adress_c[5]\, C => 
        \adress_c[6]\, D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_34_2[2]\ : 
        CFG4
      generic map(INIT => x"D800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[34]\, C => 
        \can_top_level_inst/stat_bus[222]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1236_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[29]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[3]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_8\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[8]\, B => 
        \can_top_level_inst/drv_bus[308]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_7\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_8\, 
        CC => NET_CC_CONFIG561, P => NET_CC_CONFIG558, Y3 => 
        NET_CC_CONFIG559, Y3A => NET_CC_CONFIG560);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[31]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/next_state_16_sqmuxa_0_a4\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1059\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_980\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/next_state_16_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, D => 
        \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_2[14]\ : 
        CFG4
      generic map(INIT => x"C840")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1455_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[24]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[24]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[24]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[29]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram5_[0]\ : 
        SLE
      port map(D => can_tx_c, CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, 
        EN => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe5\, 
        ALn => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn
         => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram5_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[6]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[5]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[1]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[1]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa15\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe15\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram96__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_350\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe96\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_load_base_id_i_0_a4\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_931\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1180\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[0]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2689\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2656\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/retr_ctr_add_block_proc.un7_retr_ctr_add_block_clr\ : 
        CFG4
      generic map(INIT => x"FFC8")

      port map(A => \can_top_level_inst/stat_bus[6]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/retr_ctr_add_sn_N_5_mux\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => \can_top_level_inst/stat_bus[379]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_retr_ctr_add_block_clr\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_8\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_8\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un12_tx_err_ctr_mt_erp\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_4\ : 
        CFG4
      generic map(INIT => x"956A")

      port map(A => \can_top_level_inst/drv_bus[55]\, B => 
        \can_top_level_inst/drv_bus[54]\, C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/CO2\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[4]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_4_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_RNIL1KA1[2]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => \can_top_level_inst/tran_valid\, B => 
        \can_top_level_inst/txtb_hw_cmd.failed\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m17_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm_0[7]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[7]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_15_1_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_15_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1478\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1481\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1483\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram123_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe123\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram123_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm_0[0]\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[0]\, 
        B => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        C => \can_top_level_inst/prescaler_inst/h_sync_valid\, D
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\);
    
    \can_top_level_inst/rx_buffer_inst/un1_read_increment\ : CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/drv_bus[352]\, Y => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[5]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[5]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_0_wmux\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[2]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[1]\, 
        C => \can_top_level_inst/N_3902_i\, D => 
        \can_top_level_inst/N_3901_i\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_0_y0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[13]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[5]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => \can_top_level_inst/txtb_allow_bb[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_162\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_segm_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\);
    
    \can_top_level_inst/tx_arbitrator_inst/int_txtb_index[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_hw_cmd_index[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_45\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[14]\, B => 
        \can_top_level_inst/drv_bus[219]\, C => 
        \can_top_level_inst/drv_bus[252]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_36\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[6]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[7]\, 
        CC => NET_CC_CONFIG491, P => NET_CC_CONFIG488, Y3 => 
        NET_CC_CONFIG489, Y3A => NET_CC_CONFIG490);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[2]\, 
        B => \can_top_level_inst/drv_bus[53]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_1_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_2_Z\, 
        CC => NET_CC_CONFIG2386, P => NET_CC_CONFIG2383, Y3 => 
        NET_CC_CONFIG2384, Y3A => NET_CC_CONFIG2385);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0_RNIHAQKD1[0]\ : 
        ARI1_CC
      generic map(INIT => x"59C63")

      port map(A => \can_top_level_inst/drv_bus[46]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_49\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_0_cy_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[0]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_0\, 
        CC => NET_CC_CONFIG2700, P => NET_CC_CONFIG2697, Y3 => 
        NET_CC_CONFIG2698, Y3A => NET_CC_CONFIG2699);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_9_RNO[27]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_0_Z\, 
        D => \can_top_level_inst/drv_bus[265]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[667]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un4_crc_match_d\ : 
        CFG4
      generic map(INIT => x"AAEA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_clear_match_flag\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        D => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un4_crc_match_d_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_0[8]\ : 
        CFG4
      generic map(INIT => x"F0A4")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_0_1_Z[8]\, 
        B => \can_top_level_inst/stat_bus[89]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_0_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5_1\, 
        B => \sbe_c[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/NN_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[2]\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => \can_top_level_inst/N_4606\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_0\, 
        D => \can_top_level_inst/txtb_parity_error_valid[2]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_156\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q_RNI1AA82[2]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[83]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_1\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_2\, 
        CC => NET_CC_CONFIG337, P => NET_CC_CONFIG334, Y3 => 
        NET_CC_CONFIG335, Y3A => NET_CC_CONFIG336);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[11]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[11]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[11]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_3[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[10]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[10]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[10]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[5]\ : 
        CFG4
      generic map(INIT => x"4CCC")

      port map(A => 
        \can_top_level_inst/txtb_parity_error_valid_1[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_1_Z[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_m1_e_1\, 
        D => \can_top_level_inst/txtb_parity_check_valid\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_164\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[3]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2692\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2659\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[3]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa17\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe17\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.1.l2_prio_2_1.CO2\ : 
        CFG3
      generic map(INIT => x"B2")

      port map(A => \can_top_level_inst/txtb_prorities_6[2]\, B
         => \can_top_level_inst/txtb_prorities_4[2]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_1\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_123\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_RNO_0[9]\ : 
        ARI1_CC
      generic map(INIT => x"4EE00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, C => 
        \can_top_level_inst/prescaler_inst/is_tseg2\, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_8\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[9]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG2143, P => 
        NET_CC_CONFIG2140, Y3 => NET_CC_CONFIG2141, Y3A => 
        NET_CC_CONFIG2142);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIAGCK17[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[57]\, B => 
        \can_top_level_inst/tran_word[25]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_24\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_25\, 
        CC => NET_CC_CONFIG1027, P => NET_CC_CONFIG1024, Y3 => 
        NET_CC_CONFIG1025, Y3A => NET_CC_CONFIG1026);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[5]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_cZ[0]\ : 
        CFG4
      generic map(INIT => x"FABA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_4[0]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ide_is_arbitration_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa16\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe16\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[29]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2[0]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[1]\, B
         => \can_top_level_inst/txtb_sw_cmd.set_ety\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_151\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNIC4P91[6]\ : 
        CFG3
      generic map(INIT => x"2A")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => \can_top_level_inst/txtb_sw_cmd.set_abt\, C => 
        \can_top_level_inst/txtb_sw_cmd_index[1]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_135_i\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[0]\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_data_mux_test_registers_comp_un2_index_sat_1\, 
        C => \adress_c[7]\, D => \adress_c[6]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[0]\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_write_address[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[1]\, 
        C => \can_top_level_inst/rx_buffer_inst/N_48\, Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0[3]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[3]\, 
        C => \can_top_level_inst/drv_bus[54]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_50\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[4]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/int_mask_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/int_mask_load_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_mask[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[2]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_Z[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_156\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a2_1_1\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state_0_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a2_1_1_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe36_0_a4\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_341\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4\ : 
        RGB
      port map(A => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_Y\, 
        EN => ADLIB_VCC, Y => \AFLSDF_INV_10\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select[2]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_1_0_0_Z[2]\, 
        B => \sbe_c[2]\, C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_0[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[4]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_RNIOTR3A[7]\ : 
        ARI1_CC
      generic map(INIT => x"48800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy_Y[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[7]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[6]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[7]\, 
        CC => NET_CC_CONFIG681, P => NET_CC_CONFIG678, Y3 => 
        NET_CC_CONFIG679, Y3A => NET_CC_CONFIG680);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[146]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8_RNIUCF53\ : 
        CFG4
      generic map(INIT => x"40C0")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        B => \can_top_level_inst/prescaler_inst/g0_0_3\, C => 
        \can_top_level_inst/prescaler_inst/is_tseg2\, D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_2_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_38_1_0_wmux_RNO[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_0_Z[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_Z[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1293\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[21]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select\, 
        B => \sbe_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[0]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[28]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[28]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[28]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4829\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[18]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[2]\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/txtb_parity_error_valid[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_154\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[11]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2700\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2667\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_4[0]\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1300\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1194\);
    
    \can_top_level_inst/int_manager_inst/int_input_active_18[11]\ : 
        CFG4
      generic map(INIT => x"FFFD")

      port map(A => 
        \can_top_level_inst/int_manager_inst/int_input_active_18_1_Z[11]\, 
        B => 
        \can_top_level_inst/int_manager_inst/int_input_active_15_Z[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        D => 
        \can_top_level_inst/int_manager_inst/int_input_active_4_0_Z[11]\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_18_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_1_Z[0]\, 
        B => \sbe_c[0]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[15]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/crc_spec_enable_i_RNI0EDM1\ : 
        CFG4
      generic map(INIT => x"73FF")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/g2\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_N_865\, 
        Y => 
        \can_top_level_inst/can_core_inst/crc_calc_from_rx_1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_1_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO2248, CO => CI_TO_CO2249, P(0) => 
        NET_CC_CONFIG2262, P(1) => NET_CC_CONFIG2266, P(2) => 
        NET_CC_CONFIG2270, P(3) => NET_CC_CONFIG2274, P(4) => 
        NET_CC_CONFIG2278, P(5) => NET_CC_CONFIG2282, P(6) => 
        NET_CC_CONFIG2286, P(7) => NET_CC_CONFIG2290, P(8) => 
        NET_CC_CONFIG2294, P(9) => NET_CC_CONFIG2298, P(10) => 
        NET_CC_CONFIG2302, P(11) => NET_CC_CONFIG2306, Y3(0) => 
        NET_CC_CONFIG2263, Y3(1) => NET_CC_CONFIG2267, Y3(2) => 
        NET_CC_CONFIG2271, Y3(3) => NET_CC_CONFIG2275, Y3(4) => 
        NET_CC_CONFIG2279, Y3(5) => NET_CC_CONFIG2283, Y3(6) => 
        NET_CC_CONFIG2287, Y3(7) => NET_CC_CONFIG2291, Y3(8) => 
        NET_CC_CONFIG2295, Y3(9) => NET_CC_CONFIG2299, Y3(10) => 
        NET_CC_CONFIG2303, Y3(11) => NET_CC_CONFIG2307, Y3A(0)
         => NET_CC_CONFIG2264, Y3A(1) => NET_CC_CONFIG2268, 
        Y3A(2) => NET_CC_CONFIG2272, Y3A(3) => NET_CC_CONFIG2276, 
        Y3A(4) => NET_CC_CONFIG2280, Y3A(5) => NET_CC_CONFIG2284, 
        Y3A(6) => NET_CC_CONFIG2288, Y3A(7) => NET_CC_CONFIG2292, 
        Y3A(8) => NET_CC_CONFIG2296, Y3A(9) => NET_CC_CONFIG2300, 
        Y3A(10) => NET_CC_CONFIG2304, Y3A(11) => 
        NET_CC_CONFIG2308, CC(0) => NET_CC_CONFIG2265, CC(1) => 
        NET_CC_CONFIG2269, CC(2) => NET_CC_CONFIG2273, CC(3) => 
        NET_CC_CONFIG2277, CC(4) => NET_CC_CONFIG2281, CC(5) => 
        NET_CC_CONFIG2285, CC(6) => NET_CC_CONFIG2289, CC(7) => 
        NET_CC_CONFIG2293, CC(8) => NET_CC_CONFIG2297, CC(9) => 
        NET_CC_CONFIG2301, CC(10) => NET_CC_CONFIG2305, CC(11)
         => NET_CC_CONFIG2309);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[8]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[19]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[51]\, 
        C => \can_top_level_inst/rec_ident[19]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2264\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy\ : 
        ARI1_CC
      generic map(INIT => x"4EE00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, C => 
        \can_top_level_inst/prescaler_inst/is_tseg2\, D => 
        ADLIB_GND, FCI => ADLIB_VCC, S => OPEN, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Z\, 
        CC => NET_CC_CONFIG1409, P => NET_CC_CONFIG1406, Y3 => 
        NET_CC_CONFIG1407, Y3A => NET_CC_CONFIG1408);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un2_commit_rx_frame\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_mem_free[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a4_2_2[31]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_data_nbs_prev_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1151_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_4_RNIN6UB3\ : 
        ARI1_CC
      generic map(INIT => x"5E44E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_sn_N_2_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_4_S\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_Z[0]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_3\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2376\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_4\, 
        CC => NET_CC_CONFIG1969, P => NET_CC_CONFIG1966, Y3 => 
        NET_CC_CONFIG1967, Y3A => NET_CC_CONFIG1968);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_34[22]\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => \timestamp_c[22]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1006_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1006\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[26]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_87_RNIG6KEM2\ : 
        CFG4
      generic map(INIT => x"333B")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un1_m3_0_a3_0\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/un1_N_7_mux_i_0_0\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[14]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i_1_1\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[3]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_19[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y7_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y5_0[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_8[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram103__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_341\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe103\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[6]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[0]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/test_registers_out.tst_dest[6]\, D
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[16]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[20]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2546\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2478\, Y
         => \can_top_level_inst/memory_registers_inst/N_2676\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_2_RNO[27]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \adress_c[7]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => \timestamp_c[27]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[1211]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI94GC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a0_0[5]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        B => \can_top_level_inst/drv_bus[472]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a0_0_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[55]\ : 
        SLE
      port map(D => \timestamp_c[55]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[55]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[178]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_6\ : 
        CFG4
      generic map(INIT => x"2300")

      port map(A => can_tx_c, B => 
        \can_top_level_inst/stat_bus[10]\, C => 
        \can_top_level_inst/prescaler_inst/g0_0\, D => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_4_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_6_Z\);
    
    \timestamp_ibuf[45]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[45]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[45]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[19]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[19]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[19]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m42_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[20]\ : 
        CFG4
      generic map(INIT => x"553C")

      port map(A => \can_top_level_inst/drv_bus[510]\, B => 
        \can_top_level_inst/can_core_inst/crc_21[19]\, C => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_nxt_Z\, 
        D => \can_top_level_inst/can_core_inst/load_init_vect\, Y
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[15]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_dbt_Z[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_axb_0_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO[13]\ : 
        CFG4
      generic map(INIT => x"CB0B")

      port map(A => \timestamp_c[45]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_37_2_1_1[13]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_37_1_0_Z[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_667\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_0_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_GND, CO => OPEN, P(0) => 
        NET_CC_CONFIG1526, P(1) => NET_CC_CONFIG1530, P(2) => 
        NET_CC_CONFIG1534, P(3) => NET_CC_CONFIG1538, P(4) => 
        NET_CC_CONFIG1542, P(5) => NET_CC_CONFIG1546, P(6) => 
        NET_CC_CONFIG1550, P(7) => NET_CC_CONFIG1554, P(8) => 
        NET_CC_CONFIG1558, P(9) => NET_CC_CONFIG1562, P(10) => 
        ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1527, 
        Y3(1) => NET_CC_CONFIG1531, Y3(2) => NET_CC_CONFIG1535, 
        Y3(3) => NET_CC_CONFIG1539, Y3(4) => NET_CC_CONFIG1543, 
        Y3(5) => NET_CC_CONFIG1547, Y3(6) => NET_CC_CONFIG1551, 
        Y3(7) => NET_CC_CONFIG1555, Y3(8) => NET_CC_CONFIG1559, 
        Y3(9) => NET_CC_CONFIG1563, Y3(10) => ADLIB_VCC, Y3(11)
         => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG1528, Y3A(1) => 
        NET_CC_CONFIG1532, Y3A(2) => NET_CC_CONFIG1536, Y3A(3)
         => NET_CC_CONFIG1540, Y3A(4) => NET_CC_CONFIG1544, 
        Y3A(5) => NET_CC_CONFIG1548, Y3A(6) => NET_CC_CONFIG1552, 
        Y3A(7) => NET_CC_CONFIG1556, Y3A(8) => NET_CC_CONFIG1560, 
        Y3A(9) => NET_CC_CONFIG1564, Y3A(10) => ADLIB_VCC, 
        Y3A(11) => ADLIB_VCC, CC(0) => NET_CC_CONFIG1529, CC(1)
         => NET_CC_CONFIG1533, CC(2) => NET_CC_CONFIG1537, CC(3)
         => NET_CC_CONFIG1541, CC(4) => NET_CC_CONFIG1545, CC(5)
         => NET_CC_CONFIG1549, CC(6) => NET_CC_CONFIG1553, CC(7)
         => NET_CC_CONFIG1557, CC(8) => NET_CC_CONFIG1561, CC(9)
         => NET_CC_CONFIG1565, CC(10) => nc224, CC(11) => nc296);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[0]\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_ety\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_4[0]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_4\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[19]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[19]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_stuff_lvl_reached_NE\ : 
        CFG3
      generic map(INIT => x"DF")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_stuff_lvl_reached_NE_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_RNO[0]\ : 
        CFG4
      generic map(INIT => x"0A0B")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1421\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_2_Z[0]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1420\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1367_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[8]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[4]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[2]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_0_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => ADLIB_GND, 
        P(1) => NET_CC_CONFIG1482, P(2) => NET_CC_CONFIG1486, 
        P(3) => NET_CC_CONFIG1490, P(4) => NET_CC_CONFIG1494, 
        P(5) => NET_CC_CONFIG1498, P(6) => NET_CC_CONFIG1502, 
        P(7) => NET_CC_CONFIG1506, P(8) => NET_CC_CONFIG1510, 
        P(9) => NET_CC_CONFIG1514, P(10) => NET_CC_CONFIG1518, 
        P(11) => NET_CC_CONFIG1522, Y3(0) => ADLIB_VCC, Y3(1) => 
        NET_CC_CONFIG1483, Y3(2) => NET_CC_CONFIG1487, Y3(3) => 
        NET_CC_CONFIG1491, Y3(4) => NET_CC_CONFIG1495, Y3(5) => 
        NET_CC_CONFIG1499, Y3(6) => NET_CC_CONFIG1503, Y3(7) => 
        NET_CC_CONFIG1507, Y3(8) => NET_CC_CONFIG1511, Y3(9) => 
        NET_CC_CONFIG1515, Y3(10) => NET_CC_CONFIG1519, Y3(11)
         => NET_CC_CONFIG1523, Y3A(0) => ADLIB_VCC, Y3A(1) => 
        NET_CC_CONFIG1484, Y3A(2) => NET_CC_CONFIG1488, Y3A(3)
         => NET_CC_CONFIG1492, Y3A(4) => NET_CC_CONFIG1496, 
        Y3A(5) => NET_CC_CONFIG1500, Y3A(6) => NET_CC_CONFIG1504, 
        Y3A(7) => NET_CC_CONFIG1508, Y3A(8) => NET_CC_CONFIG1512, 
        Y3A(9) => NET_CC_CONFIG1516, Y3A(10) => NET_CC_CONFIG1520, 
        Y3A(11) => NET_CC_CONFIG1524, CC(0) => nc273, CC(1) => 
        NET_CC_CONFIG1485, CC(2) => NET_CC_CONFIG1489, CC(3) => 
        NET_CC_CONFIG1493, CC(4) => NET_CC_CONFIG1497, CC(5) => 
        NET_CC_CONFIG1501, CC(6) => NET_CC_CONFIG1505, CC(7) => 
        NET_CC_CONFIG1509, CC(8) => NET_CC_CONFIG1513, CC(9) => 
        NET_CC_CONFIG1517, CC(10) => NET_CC_CONFIG1521, CC(11)
         => NET_CC_CONFIG1525);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_16[28]\ : 
        CFG4
      generic map(INIT => x"FCFE")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_10_Z[28]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_11_Z[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_159\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_16_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_2[1]\ : 
        CFG4
      generic map(INIT => x"135F")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[4]\, D => 
        \can_top_level_inst/stat_bus[68]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_11_RNO[26]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_0_Z\, 
        D => \can_top_level_inst/drv_bus[264]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[666]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_0[1]\ : 
        CFG3
      generic map(INIT => x"E0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1196\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"553AC")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[3]\, 
        B => \can_top_level_inst/bus_sampling_inst/ssp_delay[3]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[3]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_2_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_3_Z\, 
        CC => NET_CC_CONFIG410, P => NET_CC_CONFIG407, Y3 => 
        NET_CC_CONFIG408, Y3A => NET_CC_CONFIG409);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_tx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[76]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[19]\);
    
    \data_in_ibuf[28]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[28]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[28]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[16]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[29]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[29]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2[25]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/tran_identifier[11]\, B
         => \can_top_level_inst/tran_identifier[22]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram36_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe36\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram36_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_no_data_field\ : 
        CFG4
      generic map(INIT => x"FF32")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/no_data_receiver_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        C => \can_top_level_inst/rec_is_rtr\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_no_data_field_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_no_data_field_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8_2[6]\ : 
        CFG4
      generic map(INIT => x"A820")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => \can_top_level_inst/rx_mem_free[6]\, D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_6_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_12[0]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[33]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[14]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[15]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[32]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_12_Z[0]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNIBVF65[12]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[12]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[11]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[12]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[12]\, 
        CC => NET_CC_CONFIG1726, P => NET_CC_CONFIG1723, Y3 => 
        NET_CC_CONFIG1724, Y3A => NET_CC_CONFIG1725);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a2[2]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_181\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[24]\);
    
    \timestamp_ibuf[29]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[29]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[29]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_2\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_1_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_2_Z\, 
        CC => NET_CC_CONFIG135, P => NET_CC_CONFIG132, Y3 => 
        NET_CC_CONFIG133, Y3A => NET_CC_CONFIG134);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_39\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[13]\, B => 
        \timestamp_c[44]\, C => \timestamp_c[45]\, D => 
        \can_top_level_inst/tran_word[12]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[5]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[6]\, 
        CC => NET_CC_CONFIG2277, P => NET_CC_CONFIG2274, Y3 => 
        NET_CC_CONFIG2275, Y3A => NET_CC_CONFIG2276);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select[2]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5_1\, 
        B => \sbe_c[2]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/NN_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_13_2[10]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/drv_bus[410]\, B => 
        \can_top_level_inst/rx_read_buff[10]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1519_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/crc_enable_i_a4_1_RNIC8TA1\ : 
        CFG4
      generic map(INIT => x"070F")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1246\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/crc_enable_i_a4_1_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1236\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst_crc_enable\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_type_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_type_d_ss0_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/err_detected\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[105]\);
    
    \adress_ibuf[3]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \adress_c[3]\, E => ADLIB_GND, YIN => 
        \adress_ibuf[3]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[26]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[26]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[26]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_18[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_7[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram30_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram31_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y7_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[4]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_161_2\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_161\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d\ : 
        CFG4
      generic map(INIT => x"0023")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d_1_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/AXB[4]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_gate_mem_read_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d_Z\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz[3]\ : 
        CFG4
      generic map(INIT => x"FF04")

      port map(A => \can_top_level_inst/N_4606\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[12]\);
    
    \clk_sys_ibuf/U_IOPAD\ : IOPAD_IN
      port map(PAD => clk_sys, Y => \clk_sys_ibuf/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_41\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[217]\, B => 
        \can_top_level_inst/rec_ident[12]\, C => 
        \can_top_level_inst/drv_bus[250]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_42\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_1\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[29]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_1_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[19]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNICIA58[18]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[18]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[18]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_17\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_18\, 
        CC => NET_CC_CONFIG1349, P => NET_CC_CONFIG1346, Y3 => 
        NET_CC_CONFIG1347, Y3A => NET_CC_CONFIG1348);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[118]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[276]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_0[4]\ : 
        CFG2
      generic map(INIT => x"B")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[1]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_143\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_63\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[20]\, B => 
        \can_top_level_inst/drv_bus[225]\, C => 
        \can_top_level_inst/drv_bus[258]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_21\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[9]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[10]\, 
        CC => NET_CC_CONFIG503, P => NET_CC_CONFIG500, Y3 => 
        NET_CC_CONFIG501, Y3A => NET_CC_CONFIG502);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[16]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\);
    
    \timestamp_ibuf[59]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[59]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[59]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm_0[5]\ : 
        CFG4
      generic map(INIT => x"B830")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/next_state_16_sqmuxa\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[5]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un3_data_length_sub_c_axbxc2\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[12]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test_dbl_buf.fcrc\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[1]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/load_ffmt_w_addr\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/fcrc\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[3]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_2_1.CO1\ : 
        CFG4
      generic map(INIT => x"0A8E")

      port map(A => \can_top_level_inst/txtb_prorities_2[1]\, B
         => \can_top_level_inst/txtb_prorities_2[0]\, C => 
        \can_top_level_inst/txtb_prorities_0[1]\, D => 
        \can_top_level_inst/txtb_prorities_0[0]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram18__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_355\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe18\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[10]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2536\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2468\, Y
         => \can_top_level_inst/memory_registers_inst/N_2666\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m64\ : 
        CFG3
      generic map(INIT => x"BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_65_1\, 
        B => \can_top_level_inst/stat_bus[371]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_65\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0_RNI5QJO42[1]\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => \can_top_level_inst/drv_bus[47]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_50\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_0\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_1\, 
        CC => NET_CC_CONFIG2704, P => NET_CC_CONFIG2701, Y3 => 
        NET_CC_CONFIG2702, Y3A => NET_CC_CONFIG2703);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[27]\ : 
        SLE
      port map(D => \data_in_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[51]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_826_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe20\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_0_y3[11]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_0_y1[11]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1575\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[10]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2536\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_37_2_0[12]\ : 
        CFG4
      generic map(INIT => x"FD0D")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \timestamp_c[12]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => \timestamp_c[44]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_37_2_0_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_wmux_0\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_0_y0\, 
        B => \can_top_level_inst/txtb_hw_cmd_index[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_129_i\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_129_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_0_y1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[2]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2691\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2658\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[21]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[21]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[21]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[26]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[9]\);
    
    \data_in_ibuf[22]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(22), Y => \data_in_ibuf[22]/YIN\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[0]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[0]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[7]\);
    
    \can_top_level_inst/tx_arbitrator_inst/txtb_clk_en_q\ : SLE
      port map(D => \can_top_level_inst/txtb_clk_en\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_clk_en_q_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[13]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[23]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[4]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[54]\ : 
        SLE
      port map(D => \timestamp_c[54]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[54]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s_2640\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_VCC, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s_2640_FCO\, 
        CC => NET_CC_CONFIG1453, P => NET_CC_CONFIG1450, Y3 => 
        NET_CC_CONFIG1451, Y3A => NET_CC_CONFIG1452);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_11\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[410]\, B => 
        \can_top_level_inst/stat_bus[83]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_1\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_2\, 
        CC => NET_CC_CONFIG179, P => NET_CC_CONFIG176, Y3 => 
        NET_CC_CONFIG177, Y3A => NET_CC_CONFIG178);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_m3[1]\ : 
        CFG4
      generic map(INIT => x"F1E0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_149\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_150\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIIPGS7[17]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[17]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[17]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_16\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_17\, 
        CC => NET_CC_CONFIG1345, P => NET_CC_CONFIG1342, Y3 => 
        NET_CC_CONFIG1343, Y3A => NET_CC_CONFIG1344);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_retr_ctr_clear_i\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/unlock_5\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_retr_ctr_clear_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIU99S\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[12]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[8]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_629\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_623\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[11]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_3\ : 
        CFG4
      generic map(INIT => x"F2F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_928_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_2_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1255\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_3_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s_2638_RNIGG5A\ : 
        CFG2
      generic map(INIT => x"B")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_sn_N_2_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[31]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[31]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[31]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[31]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[6]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[6]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[6]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1[13]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        C => \can_top_level_inst/tran_frame_test.tprm[3]\, D => 
        \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1_Z[13]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_53\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[207]\, B => 
        \can_top_level_inst/rec_ident[2]\, C => 
        \can_top_level_inst/drv_bus[240]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_32\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0[2]\ : 
        CFG4
      generic map(INIT => x"FF02")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reinteg_ctr_expired\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_0\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => swr_c, B => scs_c, C => \adress_c[11]\, D => 
        \adress_c[9]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_un1_ram_write_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_1\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_934\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[22]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI8QRSD[19]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[207]\, D => 
        \can_top_level_inst/stat_bus[239]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_18\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI8QRSD_S[19]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_19\, 
        CC => NET_CC_CONFIG832, P => NET_CC_CONFIG829, Y3 => 
        NET_CC_CONFIG830, Y3A => NET_CC_CONFIG831);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[28]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[27]\ : 
        CFG4
      generic map(INIT => x"88C0")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_4287\, B
         => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        C => \can_top_level_inst/rx_buffer_inst/N_2272\, D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_13\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a2[3]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_192\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe6\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[26]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[17]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_RNO_0[2]\ : 
        CFG4
      generic map(INIT => x"6AAA")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[0]\, 
        D => \can_top_level_inst/tx_trigger\, Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/N_10\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[8]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[10]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2255\, B
         => \can_top_level_inst/rx_buffer_inst/N_4817\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2288\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[19]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[19]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[19]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/current_state_s0_0_a2_RNIQ6VIM4\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg1\, Y => 
        \can_top_level_inst/rx_triggers_i[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[20]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[225]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[26]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[23]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[23]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1[6]\ : 
        CFG4
      generic map(INIT => x"0E02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[6]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[6]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[24]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[24]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[24]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4826\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un4_tst_ena\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[16]\, B
         => \can_top_level_inst/reg_value_r_rep[17]\, C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena_2\, 
        D => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[29]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_0_CC_2\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO1829, CO => OPEN, P(0) => 
        NET_CC_CONFIG1914, P(1) => NET_CC_CONFIG1918, P(2) => 
        NET_CC_CONFIG1922, P(3) => NET_CC_CONFIG1926, P(4) => 
        NET_CC_CONFIG1930, P(5) => NET_CC_CONFIG1934, P(6) => 
        NET_CC_CONFIG1938, P(7) => NET_CC_CONFIG1942, P(8) => 
        NET_CC_CONFIG1946, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, 
        P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1915, Y3(1) => 
        NET_CC_CONFIG1919, Y3(2) => NET_CC_CONFIG1923, Y3(3) => 
        NET_CC_CONFIG1927, Y3(4) => NET_CC_CONFIG1931, Y3(5) => 
        NET_CC_CONFIG1935, Y3(6) => NET_CC_CONFIG1939, Y3(7) => 
        NET_CC_CONFIG1943, Y3(8) => NET_CC_CONFIG1947, Y3(9) => 
        ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, 
        Y3A(0) => NET_CC_CONFIG1916, Y3A(1) => NET_CC_CONFIG1920, 
        Y3A(2) => NET_CC_CONFIG1924, Y3A(3) => NET_CC_CONFIG1928, 
        Y3A(4) => NET_CC_CONFIG1932, Y3A(5) => NET_CC_CONFIG1936, 
        Y3A(6) => NET_CC_CONFIG1940, Y3A(7) => NET_CC_CONFIG1944, 
        Y3A(8) => NET_CC_CONFIG1948, Y3A(9) => ADLIB_VCC, Y3A(10)
         => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG1917, CC(1) => NET_CC_CONFIG1921, CC(2) => 
        NET_CC_CONFIG1925, CC(3) => NET_CC_CONFIG1929, CC(4) => 
        NET_CC_CONFIG1933, CC(5) => NET_CC_CONFIG1937, CC(6) => 
        NET_CC_CONFIG1941, CC(7) => NET_CC_CONFIG1945, CC(8) => 
        NET_CC_CONFIG1949, CC(9) => nc61, CC(10) => nc104, CC(11)
         => nc138);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[9]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[9]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI8QRSD_S[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[207]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_2_0_1[16]\ : 
        CFG3
      generic map(INIT => x"35")

      port map(A => \can_top_level_inst/stat_bus[273]\, B => 
        \can_top_level_inst/drv_bus[287]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_2_0_1_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIFPMS\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[14]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[14]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[14]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_6[28]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac21_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac19_Z\, 
        C => \can_top_level_inst/drv_bus[233]\, D => 
        \can_top_level_inst/drv_bus[171]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_6_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[31]\ : 
        CFG4
      generic map(INIT => x"00CA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[32]\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_782_i\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[1]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[1]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un6_ctrl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[4]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_2[18]\ : 
        CFG4
      generic map(INIT => x"C840")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1459_2\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_50\ : 
        CFG4
      generic map(INIT => x"8421")

      port map(A => \timestamp_c[63]\, B => \timestamp_c[62]\, C
         => \can_top_level_inst/tran_word[31]\, D => 
        \can_top_level_inst/tran_word[30]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_I_50\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/dff_fc_reset_inst/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/reg_q\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[15]\);
    
    
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/un6_bit_err_ssp_valid\ : 
        CFG4
      generic map(INIT => x"F4F8")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/data_tx_delayed\, B
         => \can_top_level_inst/bus_sampling_inst/sample_sec\, C
         => 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_ssp_capt_q_Z\, 
        D => \can_top_level_inst/rx_data_wbs\, Y => 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/un6_bit_err_ssp_valid_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[6]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test_dbl_buf.tprm[0]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[8]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/load_ffmt_w_addr\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un1_err_warning_limit_d\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_8_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_8_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un1_err_warning_limit_d_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[26]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[26]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[26]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[26]\ : 
        CFG4
      generic map(INIT => x"1032")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_960\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1069\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_792_i\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_38_1_0_wmux_RNO[6]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_2_0_Z[6]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_2_Z[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1384\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram85__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_345\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe85\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\);
    
    \data_out_obuf[13]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[13]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[13]/DOUT\, EOUT => 
        \data_out_obuf[13]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[0]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[15]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[302]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_0\ : 
        ARI1_CC
      generic map(INIT => x"45500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[89]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_7\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_0_FCNET1\, 
        CC => NET_CC_CONFIG1200, P => NET_CC_CONFIG1197, Y3 => 
        NET_CC_CONFIG1198, Y3A => NET_CC_CONFIG1199);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_38_1_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_38_1_0_y0[4]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1312\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1314\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_25[16]\ : 
        CFG4
      generic map(INIT => x"C0A0")

      port map(A => \can_top_level_inst/drv_bus[316]\, B => 
        \can_top_level_inst/txtb_prorities_4[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_769\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/command_reg_comp/bit_gen.6.reg_present_gen.reg_autoclear_gen.reg_value_r_12[6]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/access_active_1[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        D => \data_in_c[6]\, Y => 
        \can_top_level_inst/drv_bus[474]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m29_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/int_ena_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/un1_int_ena_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_ena[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[18]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[18]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[18]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\);
    
    \timestamp_ibuf[34]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(34), Y => 
        \timestamp_ibuf[34]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[6]\ : 
        CFG4
      generic map(INIT => x"FCFE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_1\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_167\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_165\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_ctr_proc.same_bits_q_3_iv_i[1]\ : 
        CFG4
      generic map(INIT => x"3312")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/CO0_1\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un1_bst_ctr_q3_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[1]\, 
        D => \can_top_level_inst/can_core_inst/tx_frame_no_sof\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_3_iv_i[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un5_tx_err_ctr_declto8_i_a2_5\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => \can_top_level_inst/stat_bus[76]\, B => 
        \can_top_level_inst/stat_bus[75]\, C => 
        \can_top_level_inst/stat_bus[74]\, D => 
        \can_top_level_inst/stat_bus[73]\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un5_tx_err_ctr_declto8_i_a2_5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[24]\ : 
        CFG4
      generic map(INIT => x"F0EE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[24]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[24]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[23]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[9]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[9]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[9]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[18]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_6[29]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[3]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_17_i_o4_0_4\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_10\);
    
    
        \can_top_level_inst/bus_sampling_inst/sample_mux_inst/prev_sample_q\ : 
        SLE
      port map(D => \can_top_level_inst/rx_data_wbs\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/sample_mux_inst/prev_sample_q_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst_prev_Sample\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_22\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_22_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_0_wmux\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[2]\, B
         => \can_top_level_inst/txtb_hw_cmd_index[1]\, C => 
        \can_top_level_inst/N_3902_i\, D => 
        \can_top_level_inst/N_3901_i\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_0_y0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[27]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[27]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_24\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[13]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_0_Z\, 
        C => \can_top_level_inst/rx_buffer_inst_RAM_data_out[15]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_24_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram90__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_340\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe90\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[126]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_10_RNO[28]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_0_Z\, 
        D => \can_top_level_inst/drv_bus[266]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[668]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_RNO_0[25]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc3\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        C => \can_top_level_inst/tran_frame_test.tprm[4]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/m50_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/m51_1\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_8_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_8_FCNET1\, 
        CO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_8_Z\, 
        CC => NET_CC_CONFIG248, P => NET_CC_CONFIG245, Y3 => 
        NET_CC_CONFIG246, Y3A => NET_CC_CONFIG247);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\);
    
    \timestamp_ibuf[21]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(21), Y => 
        \timestamp_ibuf[21]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_1[0]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => swr_c, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0_1\, 
        C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_1_2[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_1_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[7]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_1\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => \can_top_level_inst/txtb_index_muxed[1]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/N_3904\, 
        C => \can_top_level_inst/txtb_index_muxed[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un8_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/drv_bus[40]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_4_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_5_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_5_FCNET1\, 
        CC => NET_CC_CONFIG1758, P => NET_CC_CONFIG1755, Y3 => 
        NET_CC_CONFIG1756, Y3A => NET_CC_CONFIG1757);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3\ : 
        GB
      port map(A => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_Z\, 
        EN => ADLIB_VCC, Y => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_Y\);
    
    
        \can_top_level_inst/rx_buffer_inst/parity_flag_proc.un5_drv_read_start\ : 
        CFG4
      generic map(INIT => x"8008")

      port map(A => \can_top_level_inst/drv_bus[352]\, B => 
        \can_top_level_inst/drv_bus[476]\, C => 
        \can_top_level_inst/rx_buffer_inst/parity_mismatch_1_29\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/parity_mismatch_1_30\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/un5_drv_read_start\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram27__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_340\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe27\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[97]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[18]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[18]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[18]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[5]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[5]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[21]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[21]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[21]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_2\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => swr_c, B => scs_c, C => \adress_c[11]\, D => 
        \adress_c[8]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_2_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[5]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[15]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm_0[3]\ : 
        CFG4
      generic map(INIT => x"FCAC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_0_iv_3[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[3]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un22_rx_data_nbs_m\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa19\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe19\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[6]\ : 
        CFG4
      generic map(INIT => x"FEEE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_174\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[2]\, 
        B => \can_top_level_inst/drv_bus[58]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_1_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_2_Z\, 
        CC => NET_CC_CONFIG2458, P => NET_CC_CONFIG2455, Y3 => 
        NET_CC_CONFIG2456, Y3A => NET_CC_CONFIG2457);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[7]\);
    
    \can_top_level_inst/rx_buffer_inst/rx_read_buff[6]\ : CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/rx_buffer_inst_RAM_data_out[6]\, 
        Y => \can_top_level_inst/rx_read_buff[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_39\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[13]\, B => 
        \can_top_level_inst/drv_bus[218]\, C => 
        \can_top_level_inst/drv_bus[251]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_42\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[5]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[6]\, 
        CC => NET_CC_CONFIG487, P => NET_CC_CONFIG484, Y3 => 
        NET_CC_CONFIG485, Y3A => NET_CC_CONFIG486);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[5]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[21]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/dff_fixed_stuff_reg/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/fixed_reg_d\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/fixed_reg_q\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_discard_stuff_bit\ : 
        CFG3
      generic map(INIT => x"37")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/fixed_prev_q\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_stuff_lvl_reached_NE_Z\, 
        C => \can_top_level_inst/can_core_inst/stuff_length[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_discard_stuff_bit_Z\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/int_mask_load\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[6]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/int_mask_load_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa14\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe14\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[26]\ : 
        CFG4
      generic map(INIT => x"F0EE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[26]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[26]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[25]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select[2]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[20]\ : 
        SLE
      port map(D => \timestamp_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_147\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[7]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[7]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[7]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4267\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_836_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select[3]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram92_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe92\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram92_0\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un1_tq_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_4[28]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac25_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac4_Z\, 
        C => \can_top_level_inst/stat_bus[285]\, D => 
        \can_top_level_inst/stat_bus[248]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_4_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[13]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm_0[3]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un10_tq_counter_d_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[3]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_18[15]\ : 
        CFG4
      generic map(INIT => x"6420")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[191]\, D => 
        \can_top_level_inst/drv_bus[48]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_724\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a4[25]\ : 
        CFG4
      generic map(INIT => x"00B3")

      port map(A => \can_top_level_inst/is_bus_off\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_939\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1138\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[34]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_776_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[10]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[10]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[10]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[7]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[28]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_69\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[25]\, B => 
        \timestamp_c[56]\, C => \timestamp_c[57]\, D => 
        \can_top_level_inst/tran_word[24]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[11]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[12]\, 
        CC => NET_CC_CONFIG2301, P => NET_CC_CONFIG2298, Y3 => 
        NET_CC_CONFIG2299, Y3A => NET_CC_CONFIG2300);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[26]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[297]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[8]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[26]\);
    
    
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_d_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/bit_err\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/buffer_md_w_0_a2\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/tx_arb_parity_check_valid_1\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[4]\, 
        Y => \can_top_level_inst/tx_arbitrator_inst/buffer_md_w\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[3]\, 
        C => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_RNI46NU[2]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_1\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_2\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_2\, 
        CC => NET_CC_CONFIG2187, P => NET_CC_CONFIG2184, Y3 => 
        NET_CC_CONFIG2185, Y3A => NET_CC_CONFIG2186);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_o2_0_RNIF2VL41[3]\ : 
        CFG4
      generic map(INIT => x"0900")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[1]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[1]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/un2_select_index_changed_2_Z\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/commit_m5_1\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/tx_arb_parity_check_valid_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[421]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/alc_Z[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_765_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/stat_bus[6]\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[296]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[5]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[52]\ : 
        SLE
      port map(D => \timestamp_c[52]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[52]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[30]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_7_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO2523, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => 
        ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_VCC, P(6) => 
        ADLIB_VCC, P(7) => ADLIB_VCC, P(8) => ADLIB_GND, P(9) => 
        NET_CC_CONFIG2524, P(10) => NET_CC_CONFIG2528, P(11) => 
        NET_CC_CONFIG2532, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => ADLIB_VCC, Y3(3) => ADLIB_VCC, Y3(4) => 
        ADLIB_VCC, Y3(5) => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7)
         => ADLIB_VCC, Y3(8) => ADLIB_VCC, Y3(9) => 
        NET_CC_CONFIG2525, Y3(10) => NET_CC_CONFIG2529, Y3(11)
         => NET_CC_CONFIG2533, Y3A(0) => ADLIB_VCC, Y3A(1) => 
        ADLIB_VCC, Y3A(2) => ADLIB_VCC, Y3A(3) => ADLIB_VCC, 
        Y3A(4) => ADLIB_VCC, Y3A(5) => ADLIB_VCC, Y3A(6) => 
        ADLIB_VCC, Y3A(7) => ADLIB_VCC, Y3A(8) => ADLIB_VCC, 
        Y3A(9) => NET_CC_CONFIG2526, Y3A(10) => NET_CC_CONFIG2530, 
        Y3A(11) => NET_CC_CONFIG2534, CC(0) => nc14, CC(1) => 
        nc285, CC(2) => nc303, CC(3) => nc150, CC(4) => nc331, 
        CC(5) => nc196, CC(6) => nc234, CC(7) => nc149, CC(8) => 
        nc12, CC(9) => NET_CC_CONFIG2527, CC(10) => 
        NET_CC_CONFIG2531, CC(11) => NET_CC_CONFIG2535);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[29]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[239]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/int_mask_load\ : 
        CFG4
      generic map(INIT => x"A800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        D => \data_in_c[8]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/int_mask_load_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_18[16]\ : 
        CFG4
      generic map(INIT => x"F5F1")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_762_2\, 
        D => \can_top_level_inst/drv_bus[192]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_762\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_RNO[1]\ : 
        CFG3
      generic map(INIT => x"6A")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[1]\, 
        B => \can_top_level_inst/bus_sampling_inst/sample_sec\, C
         => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[0]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_2[1]\);
    
    \timestamp_ibuf[38]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(38), Y => 
        \timestamp_ibuf[38]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm_0[4]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[4]\, 
        B => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        C => \can_top_level_inst/prescaler_inst/h_sync_valid\, D
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_0\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1239\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_0_4_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_951\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1213\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m34_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[26]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa4\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe4\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\);
    
    AFLSDF_INV_18 : INV_BA
      port map(A => \AFLSDF_INV_18\, Y => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1[20]\ : 
        CFG4
      generic map(INIT => x"A989")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_1_Z[20]\, 
        D => \can_top_level_inst/drv_bus[9]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_36[0]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => \can_top_level_inst/stat_bus[300]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1162_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1162\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_0[30]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[30]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_13_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_14_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[2]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNI52AS[6]\ : 
        CFG3
      generic map(INIT => x"4C")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_129_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[460]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[12]\);
    
    
        \can_top_level_inst/int_manager_inst/int_input_active_24_tz[11]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txtb_allow_bb[2]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_24_tz_Z[11]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_2[0]\, B
         => \can_top_level_inst/txtb_prorities_3[0]\, C => 
        \can_top_level_inst/drv_bus_fast[475]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_bit_err_enable_i_a2\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1246\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1236\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1310\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m34\ : 
        CFG4
      generic map(INIT => x"AF3C")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_139_mux\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m34_1_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_35_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_0\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_i[0]\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_GND, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_0_Z\, 
        CC => NET_CC_CONFIG1485, P => NET_CC_CONFIG1482, Y3 => 
        NET_CC_CONFIG1483, Y3A => NET_CC_CONFIG1484);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[10]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q_RNII4834[4]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[85]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_3\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_4\, 
        CC => NET_CC_CONFIG345, P => NET_CC_CONFIG342, Y3 => 
        NET_CC_CONFIG343, Y3A => NET_CC_CONFIG344);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_17\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[19]\, B
         => \can_top_level_inst/rx_buffer_inst_RAM_data_out[18]\, 
        C => \can_top_level_inst/rx_buffer_inst_RAM_data_out[17]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_17_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un4_compl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[0]\);
    
    \timestamp_ibuf[7]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(7), Y => \timestamp_ibuf[7]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_pointer_inc_1_i_axbxc3\ : 
        CFG4
      generic map(INIT => x"6AAA")

      port map(A => \can_top_level_inst/rx_read_pointer[3]\, B
         => \can_top_level_inst/rx_read_pointer[2]\, C => 
        \can_top_level_inst/rx_read_pointer[1]\, D => 
        \can_top_level_inst/rx_read_pointer[0]\, Y => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un3_reinteg_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[11]\);
    
    \can_top_level_inst/rst_sync_inst/rff\ : SLE
      port map(D => ADLIB_VCC, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn => 
        res_n_c, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rst_sync_inst/rff_Z\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0_RNIHR6F42[1]\ : 
        ARI1_CC
      generic map(INIT => x"58778")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[1]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_48\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_1\, 
        CC => NET_CC_CONFIG2111, P => NET_CC_CONFIG2108, Y3 => 
        NET_CC_CONFIG2109, Y3A => NET_CC_CONFIG2110);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[11]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[2]\ : 
        CFG4
      generic map(INIT => x"AEAA")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_154\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_182\, 
        C => \can_top_level_inst/N_4606\, D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_RNI4TAI2[7]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_6\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_7\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_7\, 
        CC => NET_CC_CONFIG280, P => NET_CC_CONFIG277, Y3 => 
        NET_CC_CONFIG278, Y3A => NET_CC_CONFIG279);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q_RNO[5]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_17[4]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_20\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => \data_in_c[3]\, B => \data_in_c[2]\, C => 
        \data_in_c[1]\, D => \data_in_c[0]\, Y => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_20_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate_3_0_.m5_e\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[3]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_27\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[28]\);
    
    \timestamp_ibuf[61]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[61]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[61]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_17\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_17_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[3]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_RNIBV4GM4_0\ : 
        CFG4
      generic map(INIT => x"5515")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segment_end_1\, 
        B => \can_top_level_inst/prescaler_inst/N_5058\, C => 
        \can_top_level_inst/prescaler_inst/is_tseg1\, D => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segment_end_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a4[18]\ : 
        CFG4
      generic map(INIT => x"1011")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_955\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_971\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1131\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[8]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[2]\ : 
        CFG3
      generic map(INIT => x"4C")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[15]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0_RNIB2DDI3[3]\ : 
        ARI1_CC
      generic map(INIT => x"58778")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[3]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_50\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_2\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_3\, 
        CC => NET_CC_CONFIG2119, P => NET_CC_CONFIG2116, Y3 => 
        NET_CC_CONFIG2117, Y3A => NET_CC_CONFIG2118);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_5_RNIDKG81\ : 
        CFG4
      generic map(INIT => x"FF96")

      port map(A => \can_top_level_inst/drv_bus[59]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/CO2\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[3]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_5_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_1_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[4]\, 
        C => \can_top_level_inst/txtb_port_b_address[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[23]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_wmux_0\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_0_y0\, 
        B => \can_top_level_inst/txtb_hw_cmd_index[1]\, C => 
        \can_top_level_inst/N_3904_i\, D => 
        \can_top_level_inst/N_3903_i\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_0_y1\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_4_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[4]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa3\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe3\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_87\ : 
        ARI1_CC
      generic map(INIT => x"6D700")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/drv_bus[109]\, C => 
        \can_top_level_inst/drv_bus[142]\, D => 
        \can_top_level_inst/rec_ident[28]\, FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[13]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_87_FCNET1\, 
        CC => NET_CC_CONFIG1823, P => NET_CC_CONFIG1820, Y3 => 
        NET_CC_CONFIG1821, Y3A => NET_CC_CONFIG1822);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[6]\);
    
    \timestamp_ibuf[16]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(16), Y => 
        \timestamp_ibuf[16]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[23]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[23]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_6[24]\ : 
        CFG4
      generic map(INIT => x"F2F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1054_1\, 
        D => \can_top_level_inst/drv_bus[5]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1054\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[31]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_3[3]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_3_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_13_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[13]\);
    
    \data_out_obuf[2]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[2]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[2]/DOUT\, EOUT => \data_out_obuf[2]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_21\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_21_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_21[18]\ : 
        CFG4
      generic map(INIT => x"0AC0")

      port map(A => \can_top_level_inst/stat_bus[275]\, B => 
        \can_top_level_inst/drv_bus[289]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_841\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[10]\ : 
        CFG4
      generic map(INIT => x"0045")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[10]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1045\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1142\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_824_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[10]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[10]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_is_rtr\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => \can_top_level_inst/stat_bus[156]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_is_rtr_i_Z\, 
        Y => \can_top_level_inst/rec_is_rtr\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[24]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30[10]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[10]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[11]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/dff_data_out_reg/reg_q_RNO\ : 
        CFG4
      generic map(INIT => x"5CAA")

      port map(A => 
        \can_top_level_inst/can_core_inst/bst_data_out\, B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_data_in\, 
        C => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/dff_data_out_reg/data_out_d_sn_N_4_mux\, 
        D => \can_top_level_inst/tx_trigger\, Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/dff_data_out_reg/N_374_mux\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[12]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[12]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_dlc_Z[3]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[2]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_dlc\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_dlc[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI72GC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/N_4257\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/int_mask_load\ : 
        CFG4
      generic map(INIT => x"A800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        D => \data_in_c[11]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/int_mask_load_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[28]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\);
    
    
        \can_top_level_inst/rx_buffer_inst/rst_reg_inst/rx_shift_res_reg_inst/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rst_reg_inst/rx_shift_res_reg_inst/un1_rx_buf_res_n_d_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/reg_q\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm_0[0]\ : 
        CFG4
      generic map(INIT => x"FCB8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_0_iv_2[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_RNITOLC_Y[0]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_cnst_m[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[24]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/int_mask_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/int_mask_load_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_mask[9]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_raw_0[7]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI6BEO2_S[6]\, 
        B => \can_top_level_inst/drv_bus[382]\, C => 
        \can_top_level_inst/drv_bus[374]\, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_43\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[19]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[195]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select[2]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_1_Z[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_0_Z[1]\, 
        D => \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_0[6]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[5]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[19]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[224]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[27]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4QRQ3[4]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[192]\, D => 
        \can_top_level_inst/stat_bus[224]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_3\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4QRQ3_S[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_4\, 
        CC => NET_CC_CONFIG772, P => NET_CC_CONFIG769, Y3 => 
        NET_CC_CONFIG770, Y3A => NET_CC_CONFIG771);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[10]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_2[17]\ : 
        CFG4
      generic map(INIT => x"C840")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1458_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[30]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/test_registers_out.tst_wdata[30]\, C
         => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45[9]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[9]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[22]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[6]\ : 
        CFG4
      generic map(INIT => x"FF40")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_178\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_9_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_9_FCNET1\, 
        CO => 
        \can_top_level_inst/prescaler_inst/un7_exit_segm_regular_cry_9_0\, 
        CC => NET_CC_CONFIG167, P => NET_CC_CONFIG164, Y3 => 
        NET_CC_CONFIG165, Y3A => NET_CC_CONFIG166);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0_RNIR1S8J3[3]\ : 
        ARI1_CC
      generic map(INIT => x"58778")

      port map(A => \can_top_level_inst/drv_bus[32]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_50\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_2\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_3\, 
        CC => NET_CC_CONFIG1139, P => NET_CC_CONFIG1136, Y3 => 
        NET_CC_CONFIG1137, Y3A => NET_CC_CONFIG1138);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[9]\ : 
        CFG4
      generic map(INIT => x"0E02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[9]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[9]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[5]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram10_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe10\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram10_0\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI8N9PA_S[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[234]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_enable_set_i_a2\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_go_to_suspend_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1268\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_pointer_inc_1_i_axbxc5\ : 
        CFG3
      generic map(INIT => x"6A")

      port map(A => \can_top_level_inst/rx_read_pointer[5]\, B
         => \can_top_level_inst/rx_read_pointer[4]\, C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_pointer_inc_1_i_c4\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data_2\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/store_data\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/next_state_15_sqmuxa_i_o4\ : 
        CFG3
      generic map(INIT => x"15")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_no_data_field_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_955\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[23]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_4\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[406]\, B => 
        \can_top_level_inst/stat_bus[87]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_5\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_6\, 
        CC => NET_CC_CONFIG1192, P => NET_CC_CONFIG1189, Y3 => 
        NET_CC_CONFIG1190, Y3A => NET_CC_CONFIG1191);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[11]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3_2[29]\ : 
        CFG4
      generic map(INIT => x"A0C0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[29]\, 
        B => \can_top_level_inst/store_data_word[29]\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_2307_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[10]\);
    
    AFLSDF_INV_1 : INV_BA
      port map(A => \AFLSDF_INV_1\, Y => NN_1);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[1]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt_fast\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_tx_command_reg_comp_reg_value_r_fast[14]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst_priority_decoder_inst_un1_m2_0_a2_0_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[17]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[16]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[14]\);
    
    
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_27\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_23_Z\, 
        B => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_13_Z\, 
        C => \data_in_c[9]\, D => \data_in_c[8]\, Y => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_27_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[6]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[6]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d[2]\ : 
        CFG4
      generic map(INIT => x"4888")

      port map(A => \can_top_level_inst/stat_bus[305]\, B => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/reg_q_0\, 
        C => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/CO0_0\, 
        D => \can_top_level_inst/stat_bus[304]\, Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_70\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[104]\, B => 
        \can_top_level_inst/rec_ident[23]\, C => 
        \can_top_level_inst/drv_bus[137]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_16\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_31_1[8]\ : 
        CFG4
      generic map(INIT => x"2637")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_2_0_Z[8]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_2_Z[8]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_31_1_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[15]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[15]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[15]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[3]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[1]\, 
        D => \can_top_level_inst/tran_word[27]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[25]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[2]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[2]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_26[1]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[206]\, C => 
        \can_top_level_inst/drv_bus[111]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1190\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        D => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/N_4174\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[29]\);
    
    \data_out_obuf[18]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[18]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[18]/DOUT\, EOUT => 
        \data_out_obuf[18]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un23_inc_eight_i_2\ : 
        CFG4
      generic map(INIT => x"FDF5")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_N_846\, 
        B => \can_top_level_inst/err_detected\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un23_inc_eight_i_0_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un9_inc_eight_i_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un23_inc_eight_i_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[16]\);
    
    \can_top_level_inst/memory_registers_inst/un9_status_comb\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => \can_top_level_inst/stat_bus[9]\, B => 
        \can_top_level_inst/is_bus_off\, Y => 
        \can_top_level_inst/memory_registers_inst/un9_status_comb_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_0_1[5]\ : 
        CFG4
      generic map(INIT => x"1131")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_190\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_139\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_0_1_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\);
    
    \adress_ibuf[7]/U_IOPAD\ : IOPAD_IN
      port map(PAD => adress(7), Y => \adress_ibuf[7]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_i[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_write_pointer[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m25_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un5_tx_err_ctr_declto8_i_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => \can_top_level_inst/stat_bus[71]\, B => 
        \can_top_level_inst/stat_bus[72]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un5_tx_err_ctr_declto8_i_a2_5\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un5_tx_err_ctr_declto8_i_a2_4\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_238\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[17]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18[17]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[11]\);
    
    \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/crc_calc_from_rx_1_0\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un1_data_halt_d_i\, 
        C => \can_top_level_inst/can_core_inst/stuff_length[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_1_0\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_6[0]\, B
         => \can_top_level_inst/txtb_prorities_7[0]\, C => 
        \can_top_level_inst/drv_bus_475_rep1\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_6[11]\ : 
        CFG4
      generic map(INIT => x"F2F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1550_1\, 
        D => \can_top_level_inst/drv_bus[27]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1550\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_10[27]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac29_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac28_Z\, 
        C => \can_top_level_inst/drv_bus[56]\, D => 
        \can_top_level_inst/drv_bus[51]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_10_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[0]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNO[6]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_195\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_147\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_150\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_126_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[21]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[21]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[21]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2645\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram103_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe103\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram103_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[19]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[27]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2553\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI7L9M_0[0]\ : 
        CFG4
      generic map(INIT => x"3335")

      port map(A => \can_top_level_inst/stat_bus[257]\, B => 
        \can_top_level_inst/stat_bus[273]\, C => 
        \can_top_level_inst/stat_bus[11]\, D => 
        \can_top_level_inst/stat_bus[10]\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_dat_sel_ctr_i[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[12]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[12]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[12]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[41]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un6_ctrl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\);
    
    \can_top_level_inst/int_manager_inst/un1_int_i_2\ : CFG4
      generic map(INIT => x"135F")

      port map(A => \can_top_level_inst/int_vector[1]\, B => 
        \can_top_level_inst/int_vector[0]\, C => 
        \can_top_level_inst/int_ena[1]\, D => 
        \can_top_level_inst/int_ena[0]\, Y => 
        \can_top_level_inst/int_manager_inst/un1_int_i_2_Z\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_reg_rst_inst/rx_shift_res_reg_inst/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_reg_rst_inst/rx_shift_res_reg_inst/drv_bus_i[473]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[30]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m44_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[15]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2473\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_2\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => \can_top_level_inst/drv_bus[51]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_0_1_Z\, 
        C => \can_top_level_inst/drv_bus[54]\, D => 
        \can_top_level_inst/drv_bus[53]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_sjw_mt_zero_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[3]\, 
        C => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[22]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[322]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram55__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_349\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe55\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[1]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_9\ : 
        ARI1_CC
      generic map(INIT => x"56633")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[9]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[9]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_8_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_9_Z\, 
        CC => NET_CC_CONFIG434, P => NET_CC_CONFIG431, Y3 => 
        NET_CC_CONFIG432, Y3A => NET_CC_CONFIG433);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_9_RNINK6FD\ : 
        CFG4
      generic map(INIT => x"F444")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un7_exit_segm_regular_cry_9\, 
        B => \can_top_level_inst/prescaler_inst/is_tseg2\, C => 
        \can_top_level_inst/prescaler_inst/g1_0_0\, D => 
        \can_top_level_inst/prescaler_inst/g0_0_7\, Y => 
        \can_top_level_inst/prescaler_inst/exit_segm_req_nbt_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_33_RNIDJQ01[24]\ : 
        CFG4
      generic map(INIT => x"434F")

      port map(A => \timestamp_c[56]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1081\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_37_2_1_1[24]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[7]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[6]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[3]\ : 
        CFG4
      generic map(INIT => x"F080")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram11_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe11\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram11_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[26]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[26]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[16]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_3[1]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \can_top_level_inst/drv_bus[475]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_183_1\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[0]\, 
        D => \can_top_level_inst/txtb_allow_bb[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_183\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[9]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[9]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[9]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60[4]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[25]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test.tprm[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[4]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_frame_test.tprm[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_23\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_23_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m43_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[1]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_16_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO2058, CO => OPEN, P(0) => 
        NET_CC_CONFIG2095, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, 
        P(3) => ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_VCC, 
        P(6) => ADLIB_VCC, P(7) => ADLIB_VCC, P(8) => ADLIB_VCC, 
        P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11) => ADLIB_VCC, 
        Y3(0) => NET_CC_CONFIG2096, Y3(1) => ADLIB_VCC, Y3(2) => 
        ADLIB_VCC, Y3(3) => ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5)
         => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7) => ADLIB_VCC, 
        Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, Y3(10) => 
        ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG2097, Y3A(1) => ADLIB_VCC, Y3A(2) => 
        ADLIB_VCC, Y3A(3) => ADLIB_VCC, Y3A(4) => ADLIB_VCC, 
        Y3A(5) => ADLIB_VCC, Y3A(6) => ADLIB_VCC, Y3A(7) => 
        ADLIB_VCC, Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, 
        Y3A(10) => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG2098, CC(1) => nc219, CC(2) => nc30, CC(3)
         => nc243, CC(4) => nc187, CC(5) => nc65, CC(6) => nc7, 
        CC(7) => nc292, CC(8) => nc129, CC(9) => nc275, CC(10)
         => nc8, CC(11) => nc223);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[6]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[403]\);
    
    \timestamp_ibuf[16]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[16]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[16]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_159\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[20]\ : 
        CFG4
      generic map(INIT => x"0078")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[9]\, 
        B => \can_top_level_inst/tran_frame_test.fcrc\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[9]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_909\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[20]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_1.CO2\ : 
        CFG3
      generic map(INIT => x"B2")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[2]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_7\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_020\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[10]\);
    
    \data_in_ibuf[29]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(29), Y => \data_in_ibuf[29]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_err_type_d\ : 
        CFG4
      generic map(INIT => x"FFAB")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_1250\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_err\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un24_err_type_d_1_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/bit_err_arb\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_err_type_d_Z\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"553AC")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[5]\, 
        B => \can_top_level_inst/bus_sampling_inst/ssp_delay[5]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[5]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_4_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_5_Z\, 
        CC => NET_CC_CONFIG418, P => NET_CC_CONFIG415, Y3 => 
        NET_CC_CONFIG416, Y3A => NET_CC_CONFIG417);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNIKDN03[7]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ANB2\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[6]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[7]\, 
        CC => NET_CC_CONFIG1633, P => NET_CC_CONFIG1630, Y3 => 
        NET_CC_CONFIG1631, Y3A => NET_CC_CONFIG1632);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram21__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_345\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe21\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[5]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[5]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select[3]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0_1\, 
        B => \sbe_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[3]\, 
        C => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d[2]\ : 
        CFG4
      generic map(INIT => x"DD8D")

      port map(A => \can_top_level_inst/drv_bus[426]\, B => 
        \can_top_level_inst/drv_bus[418]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[2]\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_8_RNICV5067\ : 
        ARI1_CC
      generic map(INIT => x"4D2F0")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[8]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_7\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[8]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_8\, 
        CC => NET_CC_CONFIG1159, P => NET_CC_CONFIG1156, Y3 => 
        NET_CC_CONFIG1157, Y3A => NET_CC_CONFIG1158);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[30]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[30]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[30]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ssp_reset\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ssp_reset_i_4_Z\, 
        Y => \can_top_level_inst/ssp_reset\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa15\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe15\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa3\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram89__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_340\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe89\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_6_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[467]\, D => 
        \can_top_level_inst/drv_bus[161]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_6_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[23]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[8]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_4\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst_prev_Sample\, B => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_3_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst_data_edge_detector_inst_rx_data_sync_prev\, 
        D => \can_top_level_inst/stat_bus[15]\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_4_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_36[5]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => \can_top_level_inst/stat_bus[305]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1352_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1352\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_stuff_count_ok_NE_0\ : 
        CFG4
      generic map(INIT => x"7DBE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_stuff_count[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_stuff_count[3]\, 
        C => \can_top_level_inst/stat_bus[305]\, D => 
        \can_top_level_inst/stat_bus[303]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_stuff_count_ok_NE_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[31]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/is_control\ : 
        CFG4
      generic map(INIT => x"0545")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/is_control_2_i_a2_2\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ide_is_arbitration_i\, 
        Y => \can_top_level_inst/stat_bus[371]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[8]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2697\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2664\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/un5_transient_state_0_o3\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[7]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/wr_select_1_1[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/reg_sel_2[8]\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        D => \adress_c[5]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_1[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_27[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/stat_bus[106]\, B => 
        \can_top_level_inst/drv_bus[87]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1381\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[15]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[15]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[15]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[28]\ : 
        SLE
      port map(D => \data_in_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[52]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_segm_counter_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa18\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe18\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[7]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[20]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30[12]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[12]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[12]\);
    
    \can_tx_obuf/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => can_tx_c, E => ADLIB_VCC, DOUT => 
        \can_tx_obuf/DOUT\, EOUT => \can_tx_obuf/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_enable_set_i\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1003\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_892_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        D => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/N_4194\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m[0]\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_3_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1327\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_913_i\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1194\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_cnst_m[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[0]\);
    
    \data_out_obuf[26]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(26), D => \data_out_obuf[26]/DOUT\, 
        E => \data_out_obuf[26]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa16_2\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \adress_c[3]\, B => \adress_c[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[3]\, 
        C => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe5\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[20]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2478\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[28]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/tx_sr_pload_val[28]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[27]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[4]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_139\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[24]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/int_mask_load\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[3]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/int_mask_load_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa11\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe11\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c2_RNISQ759\ : 
        CFG4
      generic map(INIT => x"FFEF")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_NE_1\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_NE_4\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/N_3913\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_NE_2\, 
        Y => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[31]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[31]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[31]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m25_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[23]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0_RNI733J42[1]\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[1]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_50\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_1\, 
        CC => NET_CC_CONFIG1417, P => NET_CC_CONFIG1414, Y3 => 
        NET_CC_CONFIG1415, Y3A => NET_CC_CONFIG1416);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[7]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_RNI1Q6J1[4]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_3\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_4\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_4\, 
        CC => NET_CC_CONFIG2195, P => NET_CC_CONFIG2192, Y3 => 
        NET_CC_CONFIG2193, Y3A => NET_CC_CONFIG2194);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/int_status\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/un2_int_status_set\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/un1_int_status_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_vector[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[26]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_6[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un26_err_frm_req_i_1\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_err\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/bit_err_arb\, 
        C => \can_top_level_inst/can_core_inst/stuff_err\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_ack_err\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un26_err_frm_req_i_1_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[14]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm_0[6]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[6]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_a2[3]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_944\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_933\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1250\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[29]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[29]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_cZ[4]\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_1_0_Z[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_Z[3]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_Z[4]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3_1[30]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[62]\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2308_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\);
    
    
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_23\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => \data_in_c[15]\, B => \data_in_c[14]\, C => 
        \data_in_c[13]\, D => \data_in_c[12]\, Y => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_23_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[17]\ : 
        CFG4
      generic map(INIT => x"1300")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1023\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_810_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/int_ena_proc.un1_int_ena_clear\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[2]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/un1_int_ena_clear\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m79\ : 
        CFG3
      generic map(INIT => x"27")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[278]\, C => 
        \can_top_level_inst/drv_bus[150]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i79_mux\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[3]\ : 
        CFG3
      generic map(INIT => x"32")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[34]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_4\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_838_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[24]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_28[24]\ : 
        CFG3
      generic map(INIT => x"C8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[105]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1076\);
    
    \timestamp_ibuf[6]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[6]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[6]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_RNI5V9U8[6]\ : 
        ARI1_CC
      generic map(INIT => x"48800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy_Y[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[6]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[5]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[6]\, 
        CC => NET_CC_CONFIG677, P => NET_CC_CONFIG674, Y3 => 
        NET_CC_CONFIG675, Y3A => NET_CC_CONFIG676);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[21]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_21\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_21_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[21]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_81\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[17]\, B => 
        \can_top_level_inst/drv_bus[98]\, C => 
        \can_top_level_inst/drv_bus[131]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_7\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[7]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[8]\, 
        CC => NET_CC_CONFIG1799, P => NET_CC_CONFIG1796, Y3 => 
        NET_CC_CONFIG1797, Y3A => NET_CC_CONFIG1798);
    
    \data_in_ibuf[16]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(16), Y => \data_in_ibuf[16]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNI85I2E8[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_port_b_address[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/N_4198\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/N_4194\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa0_1\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \adress_c[2]\, B => \adress_c[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[16]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram92__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_351\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe92\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un41_txtb_pointer_meta_d[1]\ : 
        CFG4
      generic map(INIT => x"F5F4")

      port map(A => \can_top_level_inst/tx_arbitrator_inst/N_120\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[7]\, C
         => \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, 
        D => \can_top_level_inst/tx_arbitrator_inst/N_109\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un41_txtb_pointer_meta_d_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0[15]\ : 
        CFG4
      generic map(INIT => x"FF0D")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1324\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_16[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_6[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram58_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram59_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y5_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[23]\);
    
    \timestamp_ibuf[31]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[31]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[31]/YIN\);
    
    \timestamp_ibuf[17]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[17]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[17]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_RNI8GFC2\ : 
        CFG4
      generic map(INIT => x"0E02")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_i\, 
        B => \can_top_level_inst/tx_arbitrator_inst/l2_prio_19\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_i3_mux\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/select_buf_index_i_0_i_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[19]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[18]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_gate_mem_read.AXB[3]\ : 
        CFG3
      generic map(INIT => x"A6")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_138\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_24\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[6]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/AXB[3]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[16]\ : 
        SLE
      port map(D => \timestamp_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[5]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[14]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[14]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[14]\);
    
    \can_top_level_inst/int_manager_inst/int_input_active_9[11]\ : 
        CFG4
      generic map(INIT => x"0ACE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_9_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m34_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[20]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un7_rx_err_ctr_declto8_i_a2_0_a2_RNIQ9QO1\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/stat_bus[84]\, B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_2\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNIQ9QO1_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_3\, 
        CC => NET_CC_CONFIG2330, P => NET_CC_CONFIG2327, Y3 => 
        NET_CC_CONFIG2328, Y3A => NET_CC_CONFIG2329);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_38_1_0_wmux_RNO[1]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_2_0_Z[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_2_Z[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1179\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram118_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe118\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram118_0\);
    
    \sbe_ibuf[1]/U_IOPAD\ : IOPAD_IN
      port map(PAD => sbe(1), Y => \sbe_ibuf[1]/YIN\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_ac0_3\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => \can_top_level_inst/stat_bus[83]\, B => 
        \can_top_level_inst/stat_bus[82]\, C => 
        \can_top_level_inst/stat_bus[81]\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[17]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[17]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[4]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[30]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[30]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_1\ : 
        CFG4
      generic map(INIT => x"0032")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_m2_2_0_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_m2_1_0_Z\, 
        D => \can_top_level_inst/can_core_inst/act_err_ovr_flag\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/bit_err_disable_1_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[30]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIALJ4K[29]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[217]\, D => 
        \can_top_level_inst/stat_bus[249]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_28\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIALJ4K_S[29]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_29\, 
        CC => NET_CC_CONFIG872, P => NET_CC_CONFIG869, Y3 => 
        NET_CC_CONFIG870, Y3A => NET_CC_CONFIG871);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3_2[30]\ : 
        CFG4
      generic map(INIT => x"A0C0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[30]\, 
        B => \can_top_level_inst/store_data_word[30]\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_2308_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_control[1]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_segm_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[130]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[22]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\);
    
    \timestamp_ibuf[32]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(32), Y => 
        \timestamp_ibuf[32]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_69\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[22]\, B => 
        \can_top_level_inst/drv_bus[165]\, C => 
        \can_top_level_inst/drv_bus[198]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_16\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[10]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[11]\, 
        CC => NET_CC_CONFIG734, P => NET_CC_CONFIG731, Y3 => 
        NET_CC_CONFIG732, Y3A => NET_CC_CONFIG733);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[10]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_2_1_1\ : 
        CFG4
      generic map(INIT => x"0A8E")

      port map(A => \can_top_level_inst/txtb_prorities_5[1]\, B
         => \can_top_level_inst/txtb_prorities_5[0]\, C => 
        \can_top_level_inst/txtb_prorities_4[1]\, D => 
        \can_top_level_inst/txtb_prorities_4[0]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_2_1_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_18\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_18_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_a3_0[0]\ : 
        CFG4
      generic map(INIT => x"0111")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[2]\, 
        B => \can_top_level_inst/rx_buffer_inst/data_selector[0]\, 
        C => \can_top_level_inst/store_metadata\, D => 
        \can_top_level_inst/frame_filters_inst_ident_valid_q\, Y
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/N_54\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_1\ : 
        CFG4
      generic map(INIT => x"0007")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        B => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        C => \can_top_level_inst/stat_bus[10]\, D => 
        \can_top_level_inst/stat_bus[11]\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_nbt_tq_active\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[22]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[22]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[22]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram33__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_350\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe33\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[17]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[17]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[17]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[10]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI6RD98[8]\ : 
        ARI1_CC
      generic map(INIT => x"5D827")

      port map(A => \timestamp_c[40]\, B => 
        \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[8]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[8]\, 
        FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_7\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_8\, 
        CC => NET_CC_CONFIG959, P => NET_CC_CONFIG956, Y3 => 
        NET_CC_CONFIG957, Y3A => NET_CC_CONFIG958);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m35_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m32\ : 
        CFG4
      generic map(INIT => x"4777")

      port map(A => \can_top_level_inst/stat_bus[107]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[7]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_139_mux\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_1[22]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_1_1_Z[22]\, 
        B => \can_top_level_inst/rx_mem_free[6]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_1_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_29\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_20_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_19_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_18_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_17_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_29_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_9[1]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[33]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[32]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[29]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[30]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_9[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa3\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe3\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_7[31]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_5_Z[31]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[895]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_2_Z[31]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_3_Z[31]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_7_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[5]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[5]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_23\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_23_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[16]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[15]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[15]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[15]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNIGMQ017[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_port_b_address[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/N_4218\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/N_4214\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_2_0_1[12]\ : 
        CFG4
      generic map(INIT => x"5F5C")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_619\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_159\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_2_0_1_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_dest[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_25_2[13]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/drv_bus[413]\, B => 
        \can_top_level_inst/rx_read_buff[13]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_655_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\);
    
    AFLSDF_INV_14 : INV_BA
      port map(A => \AFLSDF_INV_14\, Y => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[13]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[31]\ : 
        SLE
      port map(D => \data_in_c[31]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[55]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[10]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[122]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[28]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_34_2[9]\ : 
        CFG4
      generic map(INIT => x"C480")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/stat_bus[373]\, D => 
        \can_top_level_inst/stat_bus[229]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1502_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNIUO5T1[1]\ : 
        ARI1_CC
      generic map(INIT => x"58DD8")

      port map(A => \can_top_level_inst/stat_bus[72]\, B => 
        \can_top_level_inst/drv_bus[425]\, C => 
        \can_top_level_inst/drv_bus[417]\, D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[4]\, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_0\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_1\, 
        CC => NET_CC_CONFIG2494, P => NET_CC_CONFIG2491, Y3 => 
        NET_CC_CONFIG2492, Y3A => NET_CC_CONFIG2493);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[3]\, 
        C => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[4]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_133\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_1_Z[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_0_Z[1]\, 
        D => \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNICVMP[33]\ : 
        CFG4
      generic map(INIT => x"0F0E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[29]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[33]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_761_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\);
    
    \can_top_level_inst/int_manager_inst/un1_int_i_0\ : CFG4
      generic map(INIT => x"135F")

      port map(A => \can_top_level_inst/int_vector[11]\, B => 
        \can_top_level_inst/int_vector[10]\, C => 
        \can_top_level_inst/int_ena[11]\, D => 
        \can_top_level_inst/int_ena[10]\, Y => 
        \can_top_level_inst/int_manager_inst/un1_int_i_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[4]\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => \can_top_level_inst/tran_valid\, B => 
        \can_top_level_inst/txtb_hw_cmd.failed\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_170_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[3]\ : 
        CFG4
      generic map(INIT => x"F0F2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_168\, 
        D => \can_top_level_inst/txtb_parity_error_valid[1]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_3\ : 
        CFG4
      generic map(INIT => x"FF96")

      port map(A => \can_top_level_inst/drv_bus[54]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/CO2\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[3]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_4_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_3_Z\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_46\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[96]\, B => 
        \can_top_level_inst/rec_ident[15]\, C => 
        \can_top_level_inst/drv_bus[129]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_36\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[28]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2273\, B
         => \can_top_level_inst/rx_buffer_inst/N_4829\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m15_2_0\ : 
        CFG3
      generic map(INIT => x"4C")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/int_mask[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_16_0_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_1_2[0]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => swr_c, B => 
        \can_top_level_inst/memory_registers_inst/un3_test_registers_cs_Z\, 
        C => \adress_c[7]\, D => \adress_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_1_2_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz[3]\ : 
        CFG4
      generic map(INIT => x"F0F4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm_0[1]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un10_tq_counter_d_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[1]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[1]\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_178\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m41\ : 
        CFG4
      generic map(INIT => x"0415")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_65\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_39_0\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_42_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_0_1[21]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[321]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_0_1_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_0\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[400]\, B => 
        \can_top_level_inst/stat_bus[81]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_0_Z\, 
        CC => NET_CC_CONFIG1529, P => NET_CC_CONFIG1526, Y3 => 
        NET_CC_CONFIG1527, Y3A => NET_CC_CONFIG1528);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[11]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_4\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_3_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_4_Z\, 
        CC => NET_CC_CONFIG1501, P => NET_CC_CONFIG1498, Y3 => 
        NET_CC_CONFIG1499, Y3A => NET_CC_CONFIG1500);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[5]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g1_0\ : 
        CFG4
      generic map(INIT => x"0EEE")

      port map(A => 
        \can_top_level_inst/can_core_inst/can_crc_inst_crc_enable\, 
        B => 
        \can_top_level_inst/can_core_inst/can_crc_inst/un2_crc_ena_17_21_i\, 
        C => 
        \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_2_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/crc_calc_from_rx_1_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g1_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIQV0C5_S[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[226]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[27]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[27]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI9BRM7_S[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[283]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_i\ : 
        CFG4
      generic map(INIT => x"DFFF")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_5_Z\, 
        B => \can_top_level_inst/txtb_parity_error_valid[2]\, C
         => 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_0_Z\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_3_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_16[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/int_ena[1]\, C => 
        \can_top_level_inst/drv_bus[470]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1180\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[4]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[4]\);
    
    \timestamp_ibuf[14]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[14]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[14]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[20]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[20]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[20]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_err_delim_late_i_o2\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_932\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_13\ : 
        ARI1_CC
      generic map(INIT => x"56633")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[13]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[13]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_12_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_13_Z\, 
        CC => NET_CC_CONFIG450, P => NET_CC_CONFIG447, Y3 => 
        NET_CC_CONFIG448, Y3A => NET_CC_CONFIG449);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_RNO[11]\ : 
        CFG3
      generic map(INIT => x"EF")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un40_frame_form_w_i\, 
        B => \can_top_level_inst/rec_is_rtr\, C => 
        \can_top_level_inst/rx_buffer_inst/N_5\, Y => 
        \can_top_level_inst/rx_buffer_inst/frame_form_w_9[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[9]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[9]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[9]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[31]\ : 
        SLE
      port map(D => \data_in_c[31]\, CLK => 
        \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[60]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[2]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[2]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[2]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_i[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_write_pointer[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d[3]\ : 
        CFG4
      generic map(INIT => x"6A00")

      port map(A => \can_top_level_inst/stat_bus[5]\, B => 
        \can_top_level_inst/stat_bus[4]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/CO1\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/un1_retr_ctr_add_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d_Z[3]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.1.l2_prio_2_1.CO1\ : 
        CFG4
      generic map(INIT => x"0A8E")

      port map(A => \can_top_level_inst/txtb_prorities_6[1]\, B
         => \can_top_level_inst/txtb_prorities_6[0]\, C => 
        \can_top_level_inst/txtb_prorities_4[1]\, D => 
        \can_top_level_inst/txtb_prorities_4[0]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[91]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, D => 
        \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[2]\ : 
        CFG4
      generic map(INIT => x"5455")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_3_Z[35]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_928_i\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_930\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_840_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/int_stat_proc.un2_int_status_set\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/stat_bus[6]\, B => 
        \can_top_level_inst/int_mask[5]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/un2_int_status_set\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm_0[8]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s_Z[8]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q_RNO[9]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_17[8]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[18]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[18]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[18]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[39]\ : 
        SLE
      port map(D => \timestamp_c[39]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[39]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[26]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_1_i\, 
        SD => ADLIB_VCC, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram15__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_342\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe15\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[22]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[245]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[10]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[10]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[10]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_4\ : 
        CFG4
      generic map(INIT => x"CE0A")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_is_transmitter\, 
        C => \can_top_level_inst/tran_is_rtr\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_4_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_length[0]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/un2_data_len_can_2_0_i\, 
        B => \can_top_level_inst/tran_dlc[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[26]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[26]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[25]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[57]\, 
        C => \can_top_level_inst/rec_ident[25]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2270\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[5]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[5]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un3_reinteg_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_axbxc3\ : 
        CFG4
      generic map(INIT => x"CCC9")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[3]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[2]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un3_data_length_sub_c_axbxc3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un3_reinteg_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[20]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[196]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_14\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_14_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[28]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[28]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[28]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram73_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe73\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram73_0\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[5]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[5]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_1[20]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[134]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_1_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[27]\ : 
        SLE
      port map(D => \data_in_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_proc.txtb_hw_cmd_q.err_2\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/int_stat_proc.un2_int_status_set\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/err_detected\, B => 
        \can_top_level_inst/int_mask[6]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/un2_int_status_set\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_38_1_wmux_RNO_0[2]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_wmux_0_Y[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_2_Z[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1232\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNID7LB[0]\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[0]\, 
        B => \can_top_level_inst/drv_bus[375]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNID7LB_Y[0]\, 
        FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_0\, 
        CC => NET_CC_CONFIG1569, P => NET_CC_CONFIG1566, Y3 => 
        NET_CC_CONFIG1567, Y3A => NET_CC_CONFIG1568);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[21]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[21]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[21]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram44_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe44\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram44_0\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO[1]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_37\, 
        B => 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_RNINDFK[1]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_0\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_1\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_1\, 
        CC => NET_CC_CONFIG2183, P => NET_CC_CONFIG2180, Y3 => 
        NET_CC_CONFIG2181, Y3A => NET_CC_CONFIG2182);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_10[18]\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[194]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_830\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d[0]\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/un1_retr_ctr_add_Z\, 
        B => \can_top_level_inst/stat_bus[2]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retr_ctr_add\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/address_decoder_control_registers_comp/addr_dec_2_0[8]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => \adress_c[6]\, B => \adress_c[7]\, C => 
        \adress_c[4]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/reg_sel_2[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_RNIQ9NI1[3]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[2]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[3]\, 
        CC => NET_CC_CONFIG2037, P => NET_CC_CONFIG2034, Y3 => 
        NET_CC_CONFIG2035, Y3A => NET_CC_CONFIG2036);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[26]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[5]\, B => 
        \can_top_level_inst/drv_bus[305]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_4\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_5\, 
        CC => NET_CC_CONFIG549, P => NET_CC_CONFIG546, Y3 => 
        NET_CC_CONFIG547, Y3A => NET_CC_CONFIG548);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/op_eq.un2_ctrl_ctr_one_1\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_one_2\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[3]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un2_ctrl_ctr_one_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[1]\ : 
        CFG4
      generic map(INIT => x"40C0")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[4]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        C => \can_top_level_inst/N_4606\, D => 
        \can_top_level_inst/txtb_sw_cmd.set_abt\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[23]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2268\, B
         => \can_top_level_inst/rx_buffer_inst/N_4825\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_o2[1]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_933\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_rx_err_ctr_ce\ : 
        CFG4
      generic map(INIT => x"F0FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/dec_one\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        C => \can_top_level_inst/drv_bus[426]\, D => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_rx_err_ctr_ce_Z\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[20]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram114__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_343\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe114\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[13]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[13]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_0[0]\ : 
        CFG2
      generic map(INIT => x"D")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1372\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1408\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[19]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[30]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_4\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[206]\, B => 
        \can_top_level_inst/rec_ident[1]\, C => 
        \can_top_level_inst/drv_bus[239]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_71\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m17_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_37_1_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_37_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/stat_bus[238]\, D => 
        \can_top_level_inst/stat_bus[384]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_37_1_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[28]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[28]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y3[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y1[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y9[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lto3_0_0\ : 
        CFG4
      generic map(INIT => x"BBB9")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lto3_0_0_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_data_len_can_2_0_i\, 
        C => \can_top_level_inst/rec_dlc[3]\, D => 
        \can_top_level_inst/rec_dlc[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lto3_0_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[5]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[5]\);
    
    \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_2\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/stuff_length[0]\, B => 
        \can_top_level_inst/can_core_inst/destuffed\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/un218_txtb_hw_cmd_int_0_o2\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => \can_top_level_inst/txtb_hw_cmd.failed\, B
         => \can_top_level_inst/tran_valid\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[11]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[11]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/soft_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/mode_i[0]\, CLK
         => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => res_n_out_c, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD
         => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m42_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[6]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_167\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_38_1_0_wmux_RNO[22]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_2_0_Z[22]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_2_Z[22]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_987\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[14]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[14]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_2\ : 
        CFG4
      generic map(INIT => x"FDDD")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_966\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1250\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        D => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m31_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[20]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[12]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[12]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[12]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[6]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_0_y0[25]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1096\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1099\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_0_y1[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIG63U9_S[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[286]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[3]\ : 
        CFG4
      generic map(INIT => x"F080")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[282]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[11]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[9]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\);
    
    \data_in_ibuf[29]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[29]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[29]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[31]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[31]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[31]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[3]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_eight\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_one\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram78__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_342\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe78\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[3]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[3]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[17]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI2J6DE3[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[49]\, B => 
        \can_top_level_inst/tran_word[17]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_16\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_17\, 
        CC => NET_CC_CONFIG995, P => NET_CC_CONFIG992, Y3 => 
        NET_CC_CONFIG993, Y3A => NET_CC_CONFIG994);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNISDAP[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[1]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[1]\, 
        Y => \can_top_level_inst/tran_word[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[7]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_8_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m15_1_0\ : 
        CFG4
      generic map(INIT => x"0213")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/int_ena[7]\, D => 
        \can_top_level_inst/drv_bus[507]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_16_0_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_1_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO459, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => 
        ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_GND, P(6) => 
        NET_CC_CONFIG460, P(7) => NET_CC_CONFIG464, P(8) => 
        NET_CC_CONFIG468, P(9) => NET_CC_CONFIG472, P(10) => 
        NET_CC_CONFIG476, P(11) => NET_CC_CONFIG480, Y3(0) => 
        ADLIB_VCC, Y3(1) => ADLIB_VCC, Y3(2) => ADLIB_VCC, Y3(3)
         => ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5) => ADLIB_VCC, 
        Y3(6) => NET_CC_CONFIG461, Y3(7) => NET_CC_CONFIG465, 
        Y3(8) => NET_CC_CONFIG469, Y3(9) => NET_CC_CONFIG473, 
        Y3(10) => NET_CC_CONFIG477, Y3(11) => NET_CC_CONFIG481, 
        Y3A(0) => ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => 
        ADLIB_VCC, Y3A(3) => ADLIB_VCC, Y3A(4) => ADLIB_VCC, 
        Y3A(5) => ADLIB_VCC, Y3A(6) => NET_CC_CONFIG462, Y3A(7)
         => NET_CC_CONFIG466, Y3A(8) => NET_CC_CONFIG470, Y3A(9)
         => NET_CC_CONFIG474, Y3A(10) => NET_CC_CONFIG478, 
        Y3A(11) => NET_CC_CONFIG482, CC(0) => nc13, CC(1) => 
        nc305, CC(2) => nc180, CC(3) => nc26, CC(4) => nc291, 
        CC(5) => nc177, CC(6) => NET_CC_CONFIG463, CC(7) => 
        NET_CC_CONFIG467, CC(8) => NET_CC_CONFIG471, CC(9) => 
        NET_CC_CONFIG475, CC(10) => NET_CC_CONFIG479, CC(11) => 
        NET_CC_CONFIG483);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m49_2_0\ : 
        CFG4
      generic map(INIT => x"5300")

      port map(A => \can_top_level_inst/stat_bus[105]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_140\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_50_0_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram59__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_352\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe59\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe8\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_nbt_RNO[0]\ : 
        CFG2
      generic map(INIT => x"9")

      port map(A => \can_top_level_inst/drv_bus[16]\, B => 
        \can_top_level_inst/drv_bus[23]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_axb_0_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[19]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20[19]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[37]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[21]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[21]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[0]\);
    
    \timestamp_ibuf[23]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(23), Y => 
        \timestamp_ibuf[23]/YIN\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[4]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_15[3]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[23]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[23]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[9]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNIC0812[3]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[3]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[2]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[3]\, 
        CC => NET_CC_CONFIG1690, P => NET_CC_CONFIG1687, Y3 => 
        NET_CC_CONFIG1688, Y3A => NET_CC_CONFIG1689);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[22]\ : 
        CFG4
      generic map(INIT => x"EC00")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[22]\, B
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[5]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[5]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram68_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe68\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram68_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_set_receiver_i_a4_0\ : 
        CFG3
      generic map(INIT => x"15")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_go_to_suspend_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_set_receiver_i_a4_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[24]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[24]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_1[14]\ : 
        CFG4
      generic map(INIT => x"0B08")

      port map(A => \can_top_level_inst/tran_identifier[0]\, B
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[14]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_16[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_6[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram26_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram27_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y5_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[2]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_34_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[376]\, D => 
        \can_top_level_inst/stat_bus[205]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_34_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_15_1_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_15_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1516\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1519\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1521\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m34_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[1]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_priority_decoder_inst_un1_m2_0_a2_0_1\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_182\, 
        C => \can_top_level_inst/N_4606\, D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_153\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/arbitration_lost_iv\ : 
        CFG4
      generic map(INIT => x"C444")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_717_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_arbitration_lost_condition_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ide_is_arbitration_i\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/set_idle_i_5_sqmuxa\, 
        Y => \can_top_level_inst/stat_bus[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI8QRSD_S[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[239]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8[4]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2376\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_6\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_5_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_6_Z\, 
        CC => NET_CC_CONFIG1509, P => NET_CC_CONFIG1506, Y3 => 
        NET_CC_CONFIG1507, Y3A => NET_CC_CONFIG1508);
    
    \data_out_obuf[24]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(24), D => \data_out_obuf[24]/DOUT\, 
        E => \data_out_obuf[24]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[27]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_segm_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[26]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[26]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[3]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[30]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[30]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[30]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[15]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_2[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram32_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram33_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[277]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[4]\, 
        C => \can_top_level_inst/txtb_port_b_address[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[9]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2467\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_28[16]\ : 
        CFG4
      generic map(INIT => x"ACF0")

      port map(A => \can_top_level_inst/stat_bus[289]\, B => 
        \can_top_level_inst/drv_bus[97]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_772\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[28]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.un4_read_increment\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => \can_top_level_inst/rec_abort_f\, B => 
        \can_top_level_inst/rx_buffer_inst/commit_overrun_abort_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un19_timestamp_valid_cry_31\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_30\, 
        B => \timestamp_c[63]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_axb_31_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/un2_index_satlto5_1\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => \adress_c[4]\, B => \adress_c[5]\, Y => 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_data_mux_test_registers_comp_un2_index_sat_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[10]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/address_decoder_control_registers_comp/addr_dec_1[16]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \adress_c[6]\, B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\);
    
    \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_0\ : 
        CFG4
      generic map(INIT => x"9009")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[5]\, 
        B => \can_top_level_inst/rx_read_pointer[5]\, C => 
        \can_top_level_inst/rx_read_pointer[4]\, D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_0_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[26]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[26]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[7]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[2]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry[1]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s_2640_FCO\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[1]\, 
        CC => NET_CC_CONFIG1457, P => NET_CC_CONFIG1454, Y3 => 
        NET_CC_CONFIG1455, Y3A => NET_CC_CONFIG1456);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un1_inc_eight_i_2\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_first_err_delim_q\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state[6]\, 
        C => \can_top_level_inst/can_core_inst/bds_data_out\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un1_inc_eight_i_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[9]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[9]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[13]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[13]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[13]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[22]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_2[15]\ : 
        CFG4
      generic map(INIT => x"C840")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1456_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_36[10]\ : 
        CFG4
      generic map(INIT => x"ACAF")

      port map(A => \timestamp_c[42]\, B => \timestamp_c[10]\, C
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1542\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[4]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[4]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_127_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_127_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_wmux_10_Y[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_wmux_10_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_o2[0]\ : 
        CFG4
      generic map(INIT => x"0A4F")

      port map(A => \can_top_level_inst/drv_bus[460]\, B => 
        \can_top_level_inst/drv_bus[511]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_984\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_0[19]\ : 
        CFG4
      generic map(INIT => x"F302")

      port map(A => \can_top_level_inst/rx_mem_free[3]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_0_1_Z[19]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_0_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[17]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[17]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[17]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un4_compl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[22]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.un4_rx_shift_ena_0\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\);
    
    \data_in_ibuf[12]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[12]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[12]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[31]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_FCNET1\, 
        CO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_FCO\, 
        CC => NET_CC_CONFIG46, P => NET_CC_CONFIG43, Y3 => 
        NET_CC_CONFIG44, Y3A => NET_CC_CONFIG45);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[5]\ : 
        CFG4
      generic map(INIT => x"FF04")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/txtb_parity_error_valid[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIQ7PL7_S[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[229]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa17\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe17\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask[1]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[0]\, 
        B => \can_top_level_inst/drv_bus[477]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1192\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[17]\);
    
    
        \can_top_level_inst/int_manager_inst/int_input_active_13_0[11]\ : 
        CFG4
      generic map(INIT => x"F8F0")

      port map(A => \can_top_level_inst/txtb_hw_cmd.unlock\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        C => 
        \can_top_level_inst/int_manager_inst/int_input_active_24_Z[11]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_13_0_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/fsm_wait_state_d_i_0\ : 
        CFG4
      generic map(INIT => x"FF0E")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[3]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[1]\, C
         => \can_top_level_inst/tx_arbitrator_inst/N_109\, D => 
        \can_top_level_inst/tx_arbitrator_inst/N_120\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/fsm_wait_state_d_i_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_ns_1_0_.N_10_i\ : 
        CFG3
      generic map(INIT => x"74")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst/N_9\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/operation_control_inst/N_21_mux\, 
        Y => 
        \can_top_level_inst/can_core_inst/operation_control_inst/N_10_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60[1]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIR5T22_S[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[275]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram69_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe69\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram69_0\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_first_ssp_d_Z\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un6_first_ssp_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\);
    
    
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/un1_h_or_re_sync_edge\ : 
        CFG4
      generic map(INIT => x"0880")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst_data_edge_detector_inst_un7_sync_edge_1\, 
        B => \can_top_level_inst/tq_edge\, C => 
        \can_top_level_inst/stat_bus[14]\, D => 
        \can_top_level_inst/stat_bus[15]\, Y => 
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/un1_h_or_re_sync_edge_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m31_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_7_1_0[1]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[10]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_933\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1012\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_7_1_0_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[31]\);
    
    \data_in_ibuf[14]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[14]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[14]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[8]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[58]\ : 
        SLE
      port map(D => \timestamp_c[58]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[58]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7_2_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_Z\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_39\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[12]\, B => 
        \can_top_level_inst/drv_bus[93]\, C => 
        \can_top_level_inst/drv_bus[126]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_41\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[5]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[6]\, 
        CC => NET_CC_CONFIG1791, P => NET_CC_CONFIG1788, Y3 => 
        NET_CC_CONFIG1789, Y3A => NET_CC_CONFIG1790);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_2[5]\ : 
        CFG4
      generic map(INIT => x"8ADA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_1_Z[5]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_2_1_Z[5]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_104\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1346\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_can_fd_1_sqmuxa\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/un2_data_len_can_2_0_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/frame_type\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_can_fd_1_sqmuxa_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[27]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[27]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[27]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m22_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_0_cy_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => ADLIB_GND, 
        P(1) => NET_CC_CONFIG1120, P(2) => NET_CC_CONFIG1124, 
        P(3) => NET_CC_CONFIG1128, P(4) => NET_CC_CONFIG1132, 
        P(5) => NET_CC_CONFIG1136, P(6) => NET_CC_CONFIG1140, 
        P(7) => NET_CC_CONFIG1144, P(8) => NET_CC_CONFIG1148, 
        P(9) => NET_CC_CONFIG1152, P(10) => NET_CC_CONFIG1156, 
        P(11) => NET_CC_CONFIG1160, Y3(0) => ADLIB_GND, Y3(1) => 
        NET_CC_CONFIG1121, Y3(2) => NET_CC_CONFIG1125, Y3(3) => 
        NET_CC_CONFIG1129, Y3(4) => NET_CC_CONFIG1133, Y3(5) => 
        NET_CC_CONFIG1137, Y3(6) => NET_CC_CONFIG1141, Y3(7) => 
        NET_CC_CONFIG1145, Y3(8) => NET_CC_CONFIG1149, Y3(9) => 
        NET_CC_CONFIG1153, Y3(10) => NET_CC_CONFIG1157, Y3(11)
         => NET_CC_CONFIG1161, Y3A(0) => ADLIB_GND, Y3A(1) => 
        NET_CC_CONFIG1122, Y3A(2) => NET_CC_CONFIG1126, Y3A(3)
         => NET_CC_CONFIG1130, Y3A(4) => NET_CC_CONFIG1134, 
        Y3A(5) => NET_CC_CONFIG1138, Y3A(6) => NET_CC_CONFIG1142, 
        Y3A(7) => NET_CC_CONFIG1146, Y3A(8) => NET_CC_CONFIG1150, 
        Y3A(9) => NET_CC_CONFIG1154, Y3A(10) => NET_CC_CONFIG1158, 
        Y3A(11) => NET_CC_CONFIG1162, CC(0) => nc139, CC(1) => 
        NET_CC_CONFIG1123, CC(2) => NET_CC_CONFIG1127, CC(3) => 
        NET_CC_CONFIG1131, CC(4) => NET_CC_CONFIG1135, CC(5) => 
        NET_CC_CONFIG1139, CC(6) => NET_CC_CONFIG1143, CC(7) => 
        NET_CC_CONFIG1147, CC(8) => NET_CC_CONFIG1151, CC(9) => 
        NET_CC_CONFIG1155, CC(10) => NET_CC_CONFIG1159, CC(11)
         => NET_CC_CONFIG1163);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_a5[1]\ : 
        CFG3
      generic map(INIT => x"32")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1408\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1415\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[21]\);
    
    \data_in_ibuf[0]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(0), Y => \data_in_ibuf[0]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[18]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[132]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[8]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[8]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[8]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask.un7_flip_mask_axbxc2\ : 
        CFG3
      generic map(INIT => x"A9")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[2]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[1]\, C => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[18]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[18]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[18]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_2[19]\ : 
        CFG4
      generic map(INIT => x"225F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[290]\, C => 
        \can_top_level_inst/drv_bus[257]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_Z[19]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_1[19]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIQV0C5_S[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[194]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram64__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_346\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe64\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[27]\ : 
        CFG4
      generic map(INIT => x"EC00")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[27]\, B
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[24]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_frame_test.sdlc\ : 
        SLE
      port map(D => \can_top_level_inst/tx_arbitrator_inst/sdlc\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_frame_test.sdlc\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_20\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_20_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/read_pointer_i[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_read_pointer[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[14]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_2[9]\ : 
        CFG4
      generic map(INIT => x"D05A")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \can_top_level_inst/stat_bus[266]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_Z[9]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_2_1[9]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_2_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[20]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/int_ena_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/un1_int_ena_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_ena[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram126_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe126\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram126_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_17[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram60_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram61_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_7[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[469]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[382]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[19]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_5\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_4_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_5_Z\, 
        CC => NET_CC_CONFIG1505, P => NET_CC_CONFIG1502, Y3 => 
        NET_CC_CONFIG1503, Y3A => NET_CC_CONFIG1504);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_15_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1509\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1512\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_15_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un9_valid\ : 
        CFG4
      generic map(INIT => x"0007")

      port map(A => 
        \can_top_level_inst/frame_filters_inst/un1_filter_range_enable_0_Z\, 
        B => 
        \can_top_level_inst/frame_filters_inst/un1_filter_range_enable_1_Z\, 
        C => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid\, 
        D => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid\, 
        Y => 
        \can_top_level_inst/frame_filters_inst/int_filter_ran_valid\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1_0_1[6]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[306]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1_0_1_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a2[33]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state_0_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[4]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_27[1]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => \can_top_level_inst/stat_bus[101]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1191\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/int_ena_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/un1_int_ena_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_ena[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[27]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[13]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_82\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[160]\, B => 
        \can_top_level_inst/rec_ident[17]\, C => 
        \can_top_level_inst/drv_bus[193]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_6\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_38_1_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_38_1_0_y0[14]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_702\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_704\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[31]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_a3[3]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        Y => \can_top_level_inst/txtb_state_1[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_5[29]\ : 
        CFG3
      generic map(INIT => x"EA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_3_Z[29]\, 
        B => \can_top_level_inst/drv_bus[53]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac29_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_5_Z[29]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_1\ : 
        CFG3
      generic map(INIT => x"4B")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[4]\, 
        B => \can_top_level_inst/drv_bus[55]\, C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[5]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[9]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_12\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[12]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_11_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_12_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_12_Z\, 
        CC => NET_CC_CONFIG1262, P => NET_CC_CONFIG1259, Y3 => 
        NET_CC_CONFIG1260, Y3A => NET_CC_CONFIG1261);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[24]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_RNIEHDN1\ : 
        CFG4
      generic map(INIT => x"8F88")

      port map(A => \can_top_level_inst/N_138\, B => 
        \data_in_c[7]\, C => \can_top_level_inst/int_mask[7]\, D
         => \can_top_level_inst/is_overload\, Y => 
        \can_top_level_inst/N_141_mux_i\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_21_2[15]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/drv_bus[415]\, B => 
        \can_top_level_inst/rx_read_buff[15]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_727_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_1[16]\ : 
        CFG4
      generic map(INIT => x"0B08")

      port map(A => \can_top_level_inst/tran_identifier[2]\, B
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[16]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_15_1_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_15_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1364\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1367\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1369\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \adress_c[8]\, B => 
        \can_top_level_inst/txtb_allow_bb[4]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_un1_ram_write_0\, 
        D => \adress_c[10]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[37]\ : 
        SLE
      port map(D => \timestamp_c[37]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[37]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_RNO[28]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2684\, D
         => \can_top_level_inst/memory_registers_inst/N_2717\, Y
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[20]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o5[1]\ : 
        CFG4
      generic map(INIT => x"11F1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1372\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_switch_to_ssp_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1370\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q_RNIAJTQ1[0]_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO83, CO => OPEN, P(0) => 
        NET_CC_CONFIG108, P(1) => NET_CC_CONFIG112, P(2) => 
        NET_CC_CONFIG116, P(3) => NET_CC_CONFIG120, P(4) => 
        ADLIB_VCC, P(5) => ADLIB_VCC, P(6) => ADLIB_VCC, P(7) => 
        ADLIB_VCC, P(8) => ADLIB_VCC, P(9) => ADLIB_VCC, P(10)
         => ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => 
        NET_CC_CONFIG109, Y3(1) => NET_CC_CONFIG113, Y3(2) => 
        NET_CC_CONFIG117, Y3(3) => NET_CC_CONFIG121, Y3(4) => 
        ADLIB_VCC, Y3(5) => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7)
         => ADLIB_VCC, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG110, Y3A(1) => NET_CC_CONFIG114, Y3A(2) => 
        NET_CC_CONFIG118, Y3A(3) => NET_CC_CONFIG122, Y3A(4) => 
        ADLIB_VCC, Y3A(5) => ADLIB_VCC, Y3A(6) => ADLIB_VCC, 
        Y3A(7) => ADLIB_VCC, Y3A(8) => ADLIB_VCC, Y3A(9) => 
        ADLIB_VCC, Y3A(10) => ADLIB_VCC, Y3A(11) => ADLIB_VCC, 
        CC(0) => NET_CC_CONFIG111, CC(1) => NET_CC_CONFIG115, 
        CC(2) => NET_CC_CONFIG119, CC(3) => NET_CC_CONFIG123, 
        CC(4) => nc310, CC(5) => nc259, CC(6) => nc245, CC(7) => 
        nc233, CC(8) => nc163, CC(9) => nc318, CC(10) => nc268, 
        CC(11) => nc112);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_tx_err_ctr_ce\ : 
        CFG4
      generic map(INIT => x"FEF0")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/dec_one\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_eight\, 
        C => \can_top_level_inst/drv_bus[425]\, D => 
        \can_top_level_inst/stat_bus[0]\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_tx_err_ctr_ce_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_load_base_id_i_0\ : 
        CFG4
      generic map(INIT => x"EFEE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_load_base_id_i_0_0_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1180\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1330\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_881\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[89]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[10]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[29]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[10]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[42]\, 
        C => \can_top_level_inst/rec_ident[10]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2255\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram2_[0]\ : 
        SLE
      port map(D => can_tx_c, CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, 
        EN => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe2\, 
        ALn => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn
         => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram2_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[20]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec[5]\ : 
        CFG2
      generic map(INIT => x"B")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_eight\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_238\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_75\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[8]\, B => 
        \can_top_level_inst/drv_bus[89]\, C => 
        \can_top_level_inst/drv_bus[122]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_11\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[3]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[4]\, 
        CC => NET_CC_CONFIG1783, P => NET_CC_CONFIG1780, Y3 => 
        NET_CC_CONFIG1781, Y3A => NET_CC_CONFIG1782);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[6]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2464\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[6]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[5]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[6]\);
    
    \data_in_ibuf[24]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(24), Y => \data_in_ibuf[24]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.perform_hsync_2_i_a2_2\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/perform_hsync_2_i_a2_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_9[0]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state_0_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[10]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_9_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_3\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[13]\, B
         => \can_top_level_inst/can_core_inst/crc_21[12]\, C => 
        \can_top_level_inst/can_core_inst/crc_21[13]\, D => 
        \can_top_level_inst/store_data_word[12]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[5]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[6]\, 
        CC => NET_CC_CONFIG26, P => NET_CC_CONFIG23, Y3 => 
        NET_CC_CONFIG24, Y3A => NET_CC_CONFIG25);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[374]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[1]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram4_0\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram5_0\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_y0_0[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_09_RNIBGIRC\ : 
        CFG4
      generic map(INIT => x"F4A0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_19\, B => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_09\, C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_RNI791Q4_Z\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_RNIU99F7_Z\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/select_buf_index_i_0_1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[5]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[5]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[6]\, B => 
        \can_top_level_inst/drv_bus[306]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_5\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_6\, 
        CC => NET_CC_CONFIG553, P => NET_CC_CONFIG550, Y3 => 
        NET_CC_CONFIG551, Y3A => NET_CC_CONFIG552);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_settings_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => swr_c, B => \sbe_c[2]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_settings_reg_comp/wr_select_0_Z[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_settings_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_2[1]\ : 
        CFG4
      generic map(INIT => x"3F08")

      port map(A => \can_top_level_inst/int_mask[1]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_2_1[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1_Z[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_2_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa3\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_1_0\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => \can_top_level_inst/txtb_index_muxed[1]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/N_3902\, 
        C => \can_top_level_inst/txtb_index_muxed[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_m1_e_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[10]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[10]\);
    
    \timestamp_ibuf[23]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[23]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[23]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[19]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[6]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[6]\);
    
    \timestamp_ibuf[10]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[10]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[10]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/int_stat_proc.un2_int_status_set\ : 
        CFG4
      generic map(INIT => x"0F0E")

      port map(A => 
        \can_top_level_inst/int_manager_inst/int_input_active_17_0_Z[11]\, 
        B => 
        \can_top_level_inst/int_manager_inst/int_input_active_18_Z[11]\, 
        C => \can_top_level_inst/int_mask[11]\, D => 
        \can_top_level_inst/int_manager_inst/int_input_active_16_Z[11]\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/un2_int_status_set\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m29_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[21]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[46]\ : 
        SLE
      port map(D => \timestamp_c[46]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[46]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_0_2[14]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un40_frame_form_w_i\, 
        C => \can_top_level_inst/rec_is_rtr\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2207_2\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_0_1[14]\ : 
        CFG4
      generic map(INIT => x"2604")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_1_0_1_Z[14]\, 
        D => \can_top_level_inst/drv_bus[95]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_0_1_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[29]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dff_destuffed_flag_reg/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dff_destuffed_flag_reg/N_26_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un10_destuffed_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/destuffed\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_read_address[0]\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => \can_top_level_inst/rx_read_pointer[0]\, Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[10]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_RNI0HB9[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/rx_triggers[1]\, B => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/CO0_1\);
    
    \timestamp_ibuf[47]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(47), Y => 
        \timestamp_ibuf[47]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[30]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[25]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[25]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[25]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4827\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm_0[6]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[6]\, 
        B => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        C => \can_top_level_inst/prescaler_inst/h_sync_valid\, D
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram47_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe47\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram47_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_address_i_i_m2[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[26]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[26]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_2230\ : 
        CFG3
      generic map(INIT => x"62")

      port map(A => \can_top_level_inst/prescaler_inst/is_tseg1\, 
        B => \can_top_level_inst/prescaler_inst/segment_end\, C
         => \can_top_level_inst/prescaler_inst/is_tseg2\, Y => 
        \can_top_level_inst/prescaler_inst/un18_sel_tseg1_2230\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm_0[6]\ : 
        CFG4
      generic map(INIT => x"B830")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/next_state_16_sqmuxa\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[6]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un3_data_length_sub_c_axbxc3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[27]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_0[31]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[31]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_13\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_14\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_14[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_5[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram22_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram23_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y3_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\);
    
    \I_1/U0_RGB1_RGB1\ : RGB
      port map(A => \I_1/U0_Y\, EN => ADLIB_VCC, Y => 
        \AFLSDF_INV_11\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram111__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_354\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe111\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[6]\);
    
    \timestamp_ibuf[53]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[53]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[53]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_m3[1]\ : 
        CFG4
      generic map(INIT => x"EFE0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => \can_top_level_inst/txtb_parity_error_valid[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_148\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_149\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_150\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_1[21]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_1_1_Z[21]\, 
        B => \can_top_level_inst/rx_mem_free[5]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_1_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m17_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[1]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[25]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un4_txtb_port_b_address[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_ptr[2]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, Y
         => \can_top_level_inst/txtb_port_b_address[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[16]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram19__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_355\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe19\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIR4RP46[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[55]\, B => 
        \can_top_level_inst/tran_word[23]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_22\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_23\, 
        CC => NET_CC_CONFIG1019, P => NET_CC_CONFIG1016, Y3 => 
        NET_CC_CONFIG1017, Y3A => NET_CC_CONFIG1018);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_0\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[13]\, B
         => \can_top_level_inst/can_core_inst/crc_15[12]\, C => 
        \can_top_level_inst/can_core_inst/crc_15[13]\, D => 
        \can_top_level_inst/store_data_word[12]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[5]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[6]\, 
        CC => NET_CC_CONFIG74, P => NET_CC_CONFIG71, Y3 => 
        NET_CC_CONFIG72, Y3A => NET_CC_CONFIG73);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_1\ : 
        CFG4
      generic map(INIT => x"FFAE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        C => \can_top_level_inst/can_core_inst/bds_data_out\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[30]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[30]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[11]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[6]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[6]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIAT4M5[9]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[9]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[9]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_8\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_9\, 
        CC => NET_CC_CONFIG1313, P => NET_CC_CONFIG1310, Y3 => 
        NET_CC_CONFIG1311, Y3A => NET_CC_CONFIG1312);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa9\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe9\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[1]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0PKSI[27]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[215]\, D => 
        \can_top_level_inst/stat_bus[247]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_26\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0PKSI_S[27]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_27\, 
        CC => NET_CC_CONFIG864, P => NET_CC_CONFIG861, Y3 => 
        NET_CC_CONFIG862, Y3A => NET_CC_CONFIG863);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[33]\ : 
        SLE
      port map(D => \timestamp_c[33]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[33]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[18]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[7]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[7]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[11]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[11]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[11]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4271\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_iv_0[1]\ : 
        CFG4
      generic map(INIT => x"5703")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_890_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_879_i\, 
        C => \can_top_level_inst/drv_bus[511]\, D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_iv_0_Z[1]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO[3]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_39\, 
        B => 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[5]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa7\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe7\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/test_registers_out.tst_control[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_rx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[89]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[21]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[21]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[21]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_10\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[10]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_9_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_10_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_10_Z\, 
        CC => NET_CC_CONFIG1254, P => NET_CC_CONFIG1251, Y3 => 
        NET_CC_CONFIG1252, Y3A => NET_CC_CONFIG1253);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_RNO[11]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        B => \can_top_level_inst/can_core_inst/crc_21[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_a3[3]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        B => \can_top_level_inst/txtb_allow_bb[4]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        Y => \can_top_level_inst/txtb_state_4[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_10[0]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[29]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_10_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_7\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[404]\, B => 
        \can_top_level_inst/stat_bus[75]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_3\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_4\, 
        CC => NET_CC_CONFIG2001, P => NET_CC_CONFIG1998, Y3 => 
        NET_CC_CONFIG1999, Y3A => NET_CC_CONFIG2000);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_11[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/stat_bus[265]\, B => 
        \can_top_level_inst/drv_bus[24]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1441\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[8]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_RNI9L5IE6\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[7]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        D => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[7]\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_6\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_7\, 
        CC => NET_CC_CONFIG2135, P => NET_CC_CONFIG2132, Y3 => 
        NET_CC_CONFIG2133, Y3A => NET_CC_CONFIG2134);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.1.l2_prio_19\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_135\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_39\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_129\, 
        Y => \can_top_level_inst/tx_arbitrator_inst/l2_prio_19\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1369_i\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[24]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[24]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, D => 
        \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped[12]\ : 
        CFG4
      generic map(INIT => x"78F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_0[12]\, 
        B => \can_top_level_inst/tran_frame_test.fcrc\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[12]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_Z[12]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[8]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[7]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0_RNI3JVMD1[0]\ : 
        ARI1_CC
      generic map(INIT => x"5B44B")

      port map(A => \can_top_level_inst/drv_bus[29]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_47\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_0_cy_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[0]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_0\, 
        CC => NET_CC_CONFIG1127, P => NET_CC_CONFIG1124, Y3 => 
        NET_CC_CONFIG1125, Y3A => NET_CC_CONFIG1126);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_20\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_20_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[28]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_34[0]\ : 
        CFG4
      generic map(INIT => x"CCFE")

      port map(A => \can_top_level_inst/trv_delay[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1160_2\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1160\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI261LC[17]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[205]\, D => 
        \can_top_level_inst/stat_bus[237]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_16\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI261LC_S[17]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_17\, 
        CC => NET_CC_CONFIG824, P => NET_CC_CONFIG821, Y3 => 
        NET_CC_CONFIG822, Y3A => NET_CC_CONFIG823);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[6]\);
    
    \data_out_obuf[3]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(3), D => \data_out_obuf[3]/DOUT\, 
        E => \data_out_obuf[3]/EOUT\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[12]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[11]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[0]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_20\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[291]\, B => 
        \can_top_level_inst/rec_ident[20]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_19\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_20\, 
        CC => NET_CC_CONFIG1913, P => NET_CC_CONFIG1910, Y3 => 
        NET_CC_CONFIG1911, Y3A => NET_CC_CONFIG1912);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[10]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_a2_0[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1421\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\);
    
    AFLSDF_INV_11 : INV_BA
      port map(A => \AFLSDF_INV_11\, Y => 
        \I_1/U0_RGB1_RGB1_rgb_net_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[16]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[16]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[16]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[29]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[29]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[29]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/decrement_rec_0_0\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_2\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_decrement_rec_0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[5]\ : 
        CFG4
      generic map(INIT => x"F088")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_RNIH1071_Z\, 
        B => \can_top_level_inst/tran_word[29]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[4]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[25]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_20\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[20]\, B => 
        \can_top_level_inst/drv_bus[320]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_19\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_20\, 
        CC => NET_CC_CONFIG609, P => NET_CC_CONFIG606, Y3 => 
        NET_CC_CONFIG607, Y3A => NET_CC_CONFIG608);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[465]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNO[6]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_195\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_147\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_150\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_126_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_read_address[4]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => \can_top_level_inst/rx_read_pointer[4]\, C => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram4_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe4\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram4_0\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[21]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[21]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNIF9273[0]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/txtb_port_b_address[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/N_4795\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/N_4796\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm_0[0]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, B => 
        \can_top_level_inst/prescaler_inst/segment_end\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[0]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[4]\ : 
        CFG3
      generic map(INIT => x"EA")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_170\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[24]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[105]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_shorten_tseg1_after_tseg2_i\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\);
    
    \timestamp_ibuf[25]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(25), Y => 
        \timestamp_ibuf[25]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[21]\ : 
        CFG4
      generic map(INIT => x"F0EE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[21]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[21]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[20]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[21]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIQ7PL7_S[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[197]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_nbt_Z[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_axb_0_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_ena_reg_proc.un21_ctrl_signal_upd_i\ : 
        CFG4
      generic map(INIT => x"FFEC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un21_ctrl_signal_upd_i_1\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1221\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un21_ctrl_signal_upd_i_2\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_920_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_length[3]\ : 
        CFG4
      generic map(INIT => x"B8FC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_len_8_to_64_integer[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_data_len_can_2_0_i_0\, 
        C => \can_top_level_inst/store_data_word[2]\, D => 
        \can_top_level_inst/stat_bus[156]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[3]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIR5T22_S[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[259]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[12]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1_1[4]\ : 
        CFG3
      generic map(INIT => x"46")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/int_vector[4]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1_1_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6_1\, 
        B => \sbe_c[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_2_0\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[18]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_1\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_0_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_1_Z\, 
        CC => NET_CC_CONFIG131, P => NET_CC_CONFIG128, Y3 => 
        NET_CC_CONFIG129, Y3A => NET_CC_CONFIG130);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa18\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe18\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[15]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60[8]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[8]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/int_ena_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/un1_int_ena_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_ena[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[21]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[21]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[7]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[6]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[6]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[6]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_838_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[8]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIQ58103[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[48]\, B => 
        \can_top_level_inst/tran_word[16]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_15\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_16\, 
        CC => NET_CC_CONFIG991, P => NET_CC_CONFIG988, Y3 => 
        NET_CC_CONFIG989, Y3A => NET_CC_CONFIG990);
    
    
        \can_top_level_inst/bus_sampling_inst/sample_mux_inst/prev_sample_q_2_0\ : 
        CFG4
      generic map(INIT => x"88C0")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/sample_sec\, B => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        C => \can_top_level_inst/rx_triggers[1]\, D => 
        \can_top_level_inst/bus_sampling_inst/un1_ssp_enable_Z\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/sample_mux_inst/prev_sample_q_2\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_ide_0_a4\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ide\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_3[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\);
    
    \timestamp_ibuf[8]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[8]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[8]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[19]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram0__RNO[0]\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[2]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[1]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[17]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[0]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2526\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2458\, Y
         => \can_top_level_inst/memory_registers_inst/N_2656\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[23]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_RNO[0]\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/sample_sec\, B => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[0]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_2[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIIOM89[22]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[22]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[22]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_21\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_22\, 
        CC => NET_CC_CONFIG1365, P => NET_CC_CONFIG1362, Y3 => 
        NET_CC_CONFIG1363, Y3A => NET_CC_CONFIG1364);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_38_1_wmux_RNO_0[21]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_2_0_Z[21]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_2_Z[21]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_964\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI2E6D1[5]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_4\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI2E6D1_S[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_5\, 
        CC => NET_CC_CONFIG1665, P => NET_CC_CONFIG1662, Y3 => 
        NET_CC_CONFIG1663, Y3A => NET_CC_CONFIG1664);
    
    \timestamp_ibuf[15]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[15]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[15]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_147\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_11[30]\ : 
        CFG4
      generic map(INIT => x"0503")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[15]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[13]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_1226_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select[3]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5_1\, 
        B => \sbe_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/NN_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[28]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[28]\);
    
    
        \can_top_level_inst/int_manager_inst/int_input_active_4_0[11]\ : 
        CFG4
      generic map(INIT => x"0CAE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_un15_txtb_hw_cmd_int_0_a3_1_0\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_un73_txtb_hw_cmd_int_0_a3_1_0\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_4_0_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNO[6]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_195\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_147\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_150\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_126_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[18]\);
    
    
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/un4_bit_err_ssp_capt_d\ : 
        CFG4
      generic map(INIT => x"F6F0")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/data_tx_delayed\, B
         => \can_top_level_inst/rx_data_wbs\, C => 
        \can_top_level_inst/rx_triggers[1]\, D => 
        \can_top_level_inst/bus_sampling_inst/sample_sec\, Y => 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/un4_bit_err_ssp_capt_d_Z\);
    
    \data_in_ibuf[28]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(28), Y => \data_in_ibuf[28]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[0]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[0]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m17_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_y0_0[15]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_731\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_734\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_0_y3[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[15]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[8]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_743\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_737\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_786_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram66__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_346\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe66\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_69\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[22]\, B => 
        \can_top_level_inst/drv_bus[227]\, C => 
        \can_top_level_inst/drv_bus[260]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_16\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[10]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[11]\, 
        CC => NET_CC_CONFIG507, P => NET_CC_CONFIG504, Y3 => 
        NET_CC_CONFIG505, Y3A => NET_CC_CONFIG506);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIAMOOF_S[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[242]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d[2]\ : 
        CFG3
      generic map(INIT => x"60")

      port map(A => \can_top_level_inst/stat_bus[4]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/CO1\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/un1_retr_ctr_add_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_1_0[2]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => \adress_c[3]\, D => \adress_c[5]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_1_0_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20_1\, 
        B => \sbe_c[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2[6]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/NN_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_5\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[405]\, B => 
        \can_top_level_inst/stat_bus[76]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_4\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_5\, 
        CC => NET_CC_CONFIG2005, P => NET_CC_CONFIG2002, Y3 => 
        NET_CC_CONFIG2003, Y3A => NET_CC_CONFIG2004);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_8\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[3]\, B
         => \can_top_level_inst/can_core_inst/crc_21[2]\, C => 
        \can_top_level_inst/can_core_inst/crc_21[3]\, D => 
        \can_top_level_inst/store_data_word[2]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[0]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[1]\, 
        CC => NET_CC_CONFIG6, P => NET_CC_CONFIG3, Y3 => 
        NET_CC_CONFIG4, Y3A => NET_CC_CONFIG5);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNIR2RS3[0]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/txtb_port_b_address[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/N_4784\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/N_4218\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[26]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[5]\ : 
        CFG4
      generic map(INIT => x"CCA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena_3\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2463\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_28\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_23_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_16_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_28_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[0]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_10\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[281]\, B => 
        \can_top_level_inst/rec_ident[10]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_9\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_10\, 
        CC => NET_CC_CONFIG1873, P => NET_CC_CONFIG1870, Y3 => 
        NET_CC_CONFIG1871, Y3A => NET_CC_CONFIG1872);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[22]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[13]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_1_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[1]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[401]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[4]\ : 
        CFG3
      generic map(INIT => x"EA")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_170\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_2_0[20]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_924\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_0_Z[20]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_2_0_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/address_decoder_control_registers_comp/addr_dec_2[10]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_36[6]\ : 
        CFG4
      generic map(INIT => x"ACAF")

      port map(A => \timestamp_c[38]\, B => \timestamp_c[6]\, C
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1390\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[5]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_15\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[4]\, B => 
        \can_top_level_inst/drv_bus[85]\, C => 
        \can_top_level_inst/drv_bus[118]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_61\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[1]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[2]\, 
        CC => NET_CC_CONFIG1775, P => NET_CC_CONFIG1772, Y3 => 
        NET_CC_CONFIG1773, Y3A => NET_CC_CONFIG1774);
    
    \data_out_obuf[5]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(5), D => \data_out_obuf[5]/DOUT\, 
        E => \data_out_obuf[5]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_159\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[13]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[8]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_667\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_661\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[13]\);
    
    
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_ssp_capt_q\ : 
        SLE
      port map(D => \can_top_level_inst/rx_triggers_i[1]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/un4_bit_err_ssp_capt_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_ssp_capt_q_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_21_RNI4OR\ : 
        CFG4
      generic map(INIT => x"9669")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_28_Z\, 
        Y => \can_top_level_inst/N_3898_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[21]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1462_2\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_1_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[146]\, D => 
        \can_top_level_inst/drv_bus[274]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_1_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz[3]\ : 
        CFG4
      generic map(INIT => x"F444")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_192\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[29]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[29]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[29]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[21]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[5]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2247\, B
         => \can_top_level_inst/rx_buffer_inst/N_4811\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2280\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5_2\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5_2_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0[35]\ : 
        CFG4
      generic map(INIT => x"F0F4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\, 
        B => \can_top_level_inst/drv_bus[513]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_928_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[35]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sample_sec\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_14_Z\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_ena_q_Z\, 
        Y => \can_top_level_inst/bus_sampling_inst/sample_sec\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[18]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_17[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/stat_bus[258]\, B => 
        \can_top_level_inst/drv_bus[17]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1181\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[3]\, 
        C => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8_2\, 
        B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        D => \adress_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry[4]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[3]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[4]\, 
        CC => NET_CC_CONFIG308, P => NET_CC_CONFIG305, Y3 => 
        NET_CC_CONFIG306, Y3A => NET_CC_CONFIG307);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_set_receiver_i_a4_0_RNIICVQ\ : 
        CFG4
      generic map(INIT => x"0C04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_set_receiver_i_a4_0_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_961\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_931\, 
        Y => 
        \can_top_level_inst/can_core_inst/operation_control_inst_set_receiver\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[23]\ : 
        CFG4
      generic map(INIT => x"0C0A")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[23]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => \can_top_level_inst/can_core_inst/pc_rx_trigger\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_798_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m44_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[11]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[11]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[11]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[11]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/g0_0\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => \can_top_level_inst/drv_bus[14]\, B => 
        \can_top_level_inst/drv_bus[13]\, C => 
        \can_top_level_inst/drv_bus[12]\, D => 
        \can_top_level_inst/drv_bus[11]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/g0_i_a3_4\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[26]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_4[29]\ : 
        CFG4
      generic map(INIT => x"7222")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[28]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_13_1\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_17_i_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/frame_valid_com_clear_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, B
         => \can_top_level_inst/tx_arbitrator_inst/un1_l3_valid\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[7]\, D
         => \can_top_level_inst/stat_bus[68]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/N_144\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m29_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_10\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[411]\, B => 
        \can_top_level_inst/stat_bus[74]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_2\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_3\, 
        CC => NET_CC_CONFIG2074, P => NET_CC_CONFIG2071, Y3 => 
        NET_CC_CONFIG2072, Y3A => NET_CC_CONFIG2073);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_0_1[16]\ : 
        CFG4
      generic map(INIT => x"3B19")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_147\, 
        D => \can_top_level_inst/stat_bus[71]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_0_1_Z[16]\);
    
    
        \can_top_level_inst/frame_filters_inst/un1_filter_range_enable_1\ : 
        CFG4
      generic map(INIT => x"F3F5")

      port map(A => \can_top_level_inst/drv_bus[267]\, B => 
        \can_top_level_inst/drv_bus[268]\, C => 
        \can_top_level_inst/stat_bus[156]\, D => 
        \can_top_level_inst/stat_bus[155]\, Y => 
        \can_top_level_inst/frame_filters_inst/un1_filter_range_enable_1_Z\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNIERIU[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => ADLIB_VCC, 
        P(1) => ADLIB_VCC, P(2) => ADLIB_GND, P(3) => 
        NET_CC_CONFIG2487, P(4) => NET_CC_CONFIG2491, P(5) => 
        NET_CC_CONFIG2495, P(6) => NET_CC_CONFIG2499, P(7) => 
        NET_CC_CONFIG2503, P(8) => NET_CC_CONFIG2507, P(9) => 
        NET_CC_CONFIG2511, P(10) => NET_CC_CONFIG2515, P(11) => 
        NET_CC_CONFIG2519, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => ADLIB_VCC, Y3(3) => NET_CC_CONFIG2488, Y3(4) => 
        NET_CC_CONFIG2492, Y3(5) => NET_CC_CONFIG2496, Y3(6) => 
        NET_CC_CONFIG2500, Y3(7) => NET_CC_CONFIG2504, Y3(8) => 
        NET_CC_CONFIG2508, Y3(9) => NET_CC_CONFIG2512, Y3(10) => 
        NET_CC_CONFIG2516, Y3(11) => NET_CC_CONFIG2520, Y3A(0)
         => ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => ADLIB_VCC, 
        Y3A(3) => NET_CC_CONFIG2489, Y3A(4) => NET_CC_CONFIG2493, 
        Y3A(5) => NET_CC_CONFIG2497, Y3A(6) => NET_CC_CONFIG2501, 
        Y3A(7) => NET_CC_CONFIG2505, Y3A(8) => NET_CC_CONFIG2509, 
        Y3A(9) => NET_CC_CONFIG2513, Y3A(10) => NET_CC_CONFIG2517, 
        Y3A(11) => NET_CC_CONFIG2521, CC(0) => nc68, CC(1) => 
        nc49, CC(2) => nc314, CC(3) => NET_CC_CONFIG2490, CC(4)
         => NET_CC_CONFIG2494, CC(5) => NET_CC_CONFIG2498, CC(6)
         => NET_CC_CONFIG2502, CC(7) => NET_CC_CONFIG2506, CC(8)
         => NET_CC_CONFIG2510, CC(9) => NET_CC_CONFIG2514, CC(10)
         => NET_CC_CONFIG2518, CC(11) => NET_CC_CONFIG2522);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[401]\, B => 
        \can_top_level_inst/stat_bus[72]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_0_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_1_Z\, 
        CC => NET_CC_CONFIG216, P => NET_CC_CONFIG213, Y3 => 
        NET_CC_CONFIG214, Y3A => NET_CC_CONFIG215);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d[1]\ : 
        CFG4
      generic map(INIT => x"BBB8")

      port map(A => \can_top_level_inst/drv_bus[417]\, B => 
        \can_top_level_inst/drv_bus[426]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_8_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_1[9]\ : 
        CFG4
      generic map(INIT => x"4567")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/int_vector[9]\, D => 
        \can_top_level_inst/drv_bus[247]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_1_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\);
    
    \can_top_level_inst/rx_buffer_inst/data_overrun_i\ : SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/un12_overrun_condition\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_drv_bus_2_i\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/data_overrun_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa4\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe4\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[5]\);
    
    
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/tx_trigger_0\ : 
        CFG4
      generic map(INIT => x"4505")

      port map(A => \can_top_level_inst/stat_bus[184]\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2\, C => 
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/tx_trigger_0_1_Z\, 
        D => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => \can_top_level_inst/tx_trigger\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m29_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m43_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un4_compl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[6]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q_RNO[11]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_15[10]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_2[2]\ : 
        CFG4
      generic map(INIT => x"AABA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_1_Z[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_971\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_854_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_0_iv_2[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1_0[7]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[1]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[2]\, C => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, D => 
        \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1_0_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[13]\);
    
    \can_top_level_inst/memory_registers_inst/status_comb_4_1[2]\ : 
        CFG4
      generic map(INIT => x"135F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_txnf_calc_proc_7_un7_txtb_state_2\, 
        B => \can_top_level_inst/txtb_state_1[3]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_159\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_159\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/status_comb_4_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\);
    
    \srd_ibuf/U_IOPAD\ : IOPAD_IN
      port map(PAD => srd, Y => \srd_ibuf/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[31]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_2[1]\ : 
        CFG4
      generic map(INIT => x"22A2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_183_1\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_2_1_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_195\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[16]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[24]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2550\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/bit_gen.2.reg_present_gen.reg_autoclear_gen.reg_value_r_5_fast[2]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select\, 
        C => \data_in_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/tx_command_fast[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[6]\ : 
        CFG4
      generic map(INIT => x"F2F0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_195\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_176_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[2]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[6]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[5]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[6]\);
    
    \can_top_level_inst/frame_filters_inst/un1_filter_c_enable_0\ : 
        CFG4
      generic map(INIT => x"3F5F")

      port map(A => \can_top_level_inst/drv_bus[236]\, B => 
        \can_top_level_inst/drv_bus[237]\, C => 
        \can_top_level_inst/stat_bus[156]\, D => 
        \can_top_level_inst/stat_bus[155]\, Y => 
        \can_top_level_inst/frame_filters_inst/un1_filter_c_enable_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_s_0_2636_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG2653, P(1) => NET_CC_CONFIG2657, P(2) => 
        NET_CC_CONFIG2661, P(3) => NET_CC_CONFIG2665, P(4) => 
        NET_CC_CONFIG2669, P(5) => NET_CC_CONFIG2673, P(6) => 
        NET_CC_CONFIG2677, P(7) => NET_CC_CONFIG2681, P(8) => 
        NET_CC_CONFIG2685, P(9) => NET_CC_CONFIG2689, P(10) => 
        ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG2654, 
        Y3(1) => NET_CC_CONFIG2658, Y3(2) => NET_CC_CONFIG2662, 
        Y3(3) => NET_CC_CONFIG2666, Y3(4) => NET_CC_CONFIG2670, 
        Y3(5) => NET_CC_CONFIG2674, Y3(6) => NET_CC_CONFIG2678, 
        Y3(7) => NET_CC_CONFIG2682, Y3(8) => NET_CC_CONFIG2686, 
        Y3(9) => NET_CC_CONFIG2690, Y3(10) => ADLIB_VCC, Y3(11)
         => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG2655, Y3A(1) => 
        NET_CC_CONFIG2659, Y3A(2) => NET_CC_CONFIG2663, Y3A(3)
         => NET_CC_CONFIG2667, Y3A(4) => NET_CC_CONFIG2671, 
        Y3A(5) => NET_CC_CONFIG2675, Y3A(6) => NET_CC_CONFIG2679, 
        Y3A(7) => NET_CC_CONFIG2683, Y3A(8) => NET_CC_CONFIG2687, 
        Y3A(9) => NET_CC_CONFIG2691, Y3A(10) => ADLIB_VCC, 
        Y3A(11) => ADLIB_VCC, CC(0) => NET_CC_CONFIG2656, CC(1)
         => NET_CC_CONFIG2660, CC(2) => NET_CC_CONFIG2664, CC(3)
         => NET_CC_CONFIG2668, CC(4) => NET_CC_CONFIG2672, CC(5)
         => NET_CC_CONFIG2676, CC(6) => NET_CC_CONFIG2680, CC(7)
         => NET_CC_CONFIG2684, CC(8) => NET_CC_CONFIG2688, CC(9)
         => NET_CC_CONFIG2692, CC(10) => nc217, CC(11) => nc170);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_16_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO2058, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_GND, P(3) => 
        NET_CC_CONFIG2059, P(4) => NET_CC_CONFIG2063, P(5) => 
        NET_CC_CONFIG2067, P(6) => NET_CC_CONFIG2071, P(7) => 
        NET_CC_CONFIG2075, P(8) => NET_CC_CONFIG2079, P(9) => 
        NET_CC_CONFIG2083, P(10) => NET_CC_CONFIG2087, P(11) => 
        NET_CC_CONFIG2091, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => ADLIB_VCC, Y3(3) => NET_CC_CONFIG2060, Y3(4) => 
        NET_CC_CONFIG2064, Y3(5) => NET_CC_CONFIG2068, Y3(6) => 
        NET_CC_CONFIG2072, Y3(7) => NET_CC_CONFIG2076, Y3(8) => 
        NET_CC_CONFIG2080, Y3(9) => NET_CC_CONFIG2084, Y3(10) => 
        NET_CC_CONFIG2088, Y3(11) => NET_CC_CONFIG2092, Y3A(0)
         => ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => ADLIB_VCC, 
        Y3A(3) => NET_CC_CONFIG2061, Y3A(4) => NET_CC_CONFIG2065, 
        Y3A(5) => NET_CC_CONFIG2069, Y3A(6) => NET_CC_CONFIG2073, 
        Y3A(7) => NET_CC_CONFIG2077, Y3A(8) => NET_CC_CONFIG2081, 
        Y3A(9) => NET_CC_CONFIG2085, Y3A(10) => NET_CC_CONFIG2089, 
        Y3A(11) => NET_CC_CONFIG2093, CC(0) => nc91, CC(1) => 
        nc225, CC(2) => nc5, CC(3) => NET_CC_CONFIG2062, CC(4)
         => NET_CC_CONFIG2066, CC(5) => NET_CC_CONFIG2070, CC(6)
         => NET_CC_CONFIG2074, CC(7) => NET_CC_CONFIG2078, CC(8)
         => NET_CC_CONFIG2082, CC(9) => NET_CC_CONFIG2086, CC(10)
         => NET_CC_CONFIG2090, CC(11) => NET_CC_CONFIG2094);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_4[30]\ : 
        CFG4
      generic map(INIT => x"FFEC")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac8_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_1_Z[30]\, 
        C => \can_top_level_inst/txtb_prorities_7[2]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[446]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_4_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[19]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[17]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm_0[4]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[4]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[124]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/un1_retr_ctr_add\ : 
        CFG3
      generic map(INIT => x"F1")

      port map(A => \can_top_level_inst/txtb_changed\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retr_ctr_clear\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retr_ctr_add\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/un1_retr_ctr_add_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[2]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[4]\, B => 
        \can_top_level_inst/drv_bus[304]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_3\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_4\, 
        CC => NET_CC_CONFIG545, P => NET_CC_CONFIG542, Y3 => 
        NET_CC_CONFIG543, Y3A => NET_CC_CONFIG544);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/alc_Z[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/stat_bus[6]\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[290]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_0\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_s_0_2645_FCO\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_0_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_0_Z\, 
        CC => NET_CC_CONFIG1214, P => NET_CC_CONFIG1211, Y3 => 
        NET_CC_CONFIG1212, Y3A => NET_CC_CONFIG1213);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_dest[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/un50_txtb_available_i\ : 
        CFG3
      generic map(INIT => x"D5")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[0]\, C => 
        \can_top_level_inst/txtb_sw_cmd.set_abt_fast\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_129\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[4]\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => \can_top_level_inst/tran_valid\, B => 
        \can_top_level_inst/txtb_hw_cmd.failed\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE_0_RNIQRRN1\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE_4_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE_0_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_1_1\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_4_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_11\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[11]\, B => 
        \can_top_level_inst/drv_bus[311]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_10\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_11\, 
        CC => NET_CC_CONFIG573, P => NET_CC_CONFIG570, Y3 => 
        NET_CC_CONFIG571, Y3A => NET_CC_CONFIG572);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_18_2[17]\ : 
        CFG4
      generic map(INIT => x"2A08")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_148\, 
        D => \can_top_level_inst/stat_bus[72]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_800_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[4]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[21]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_22\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[88]\, B => 
        \can_top_level_inst/rec_ident[7]\, C => 
        \can_top_level_inst/drv_bus[121]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_56\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[31]\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_write_address[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[2]\, 
        C => \can_top_level_inst/rx_buffer_inst/N_48\, Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[9]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un1_tq_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[29]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/int_ena_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/un1_int_ena_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_ena[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNIQAUE_0[18]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => \can_top_level_inst/stat_bus[375]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_11[31]\ : 
        CFG4
      generic map(INIT => x"FFEF")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[3]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[0]\, C => 
        \can_top_level_inst/tran_frame_test.fstc\, D => 
        \can_top_level_inst/tran_frame_test.tprm[4]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_18_0_o4_0_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_RNIH40J\ : 
        CFG3
      generic map(INIT => x"7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select\, 
        B => \sbe_c[0]\, C => \data_in_c[0]\, Y => 
        \can_top_level_inst/memory_registers_inst/mode_i[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_2_2[21]\ : 
        CFG4
      generic map(INIT => x"4F7F")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[21]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_Z[21]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => \can_top_level_inst/drv_bus[226]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_2_1[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[4]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_139\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_170_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[25]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_1_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO1763, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => 
        ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_GND, P(6) => 
        NET_CC_CONFIG1764, P(7) => NET_CC_CONFIG1768, P(8) => 
        NET_CC_CONFIG1772, P(9) => NET_CC_CONFIG1776, P(10) => 
        NET_CC_CONFIG1780, P(11) => NET_CC_CONFIG1784, Y3(0) => 
        ADLIB_VCC, Y3(1) => ADLIB_VCC, Y3(2) => ADLIB_VCC, Y3(3)
         => ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5) => ADLIB_VCC, 
        Y3(6) => NET_CC_CONFIG1765, Y3(7) => NET_CC_CONFIG1769, 
        Y3(8) => NET_CC_CONFIG1773, Y3(9) => NET_CC_CONFIG1777, 
        Y3(10) => NET_CC_CONFIG1781, Y3(11) => NET_CC_CONFIG1785, 
        Y3A(0) => ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => 
        ADLIB_VCC, Y3A(3) => ADLIB_VCC, Y3A(4) => ADLIB_VCC, 
        Y3A(5) => ADLIB_VCC, Y3A(6) => NET_CC_CONFIG1766, Y3A(7)
         => NET_CC_CONFIG1770, Y3A(8) => NET_CC_CONFIG1774, 
        Y3A(9) => NET_CC_CONFIG1778, Y3A(10) => NET_CC_CONFIG1782, 
        Y3A(11) => NET_CC_CONFIG1786, CC(0) => nc20, CC(1) => 
        nc198, CC(2) => nc147, CC(3) => nc350, CC(4) => nc316, 
        CC(5) => nc67, CC(6) => NET_CC_CONFIG1767, CC(7) => 
        NET_CC_CONFIG1771, CC(8) => NET_CC_CONFIG1775, CC(9) => 
        NET_CC_CONFIG1779, CC(10) => NET_CC_CONFIG1783, CC(11)
         => NET_CC_CONFIG1787);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[0]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[19]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[51]\ : 
        SLE
      port map(D => \timestamp_c[51]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[51]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_37_2[11]\ : 
        CFG4
      generic map(INIT => x"DA8A")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_37_1_Z[11]\, 
        B => \can_top_level_inst/stat_bus[375]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => \can_top_level_inst/stat_bus[231]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_37_2_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_control[1]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[19]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[257]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_15\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_15_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_26[14]\ : 
        CFG4
      generic map(INIT => x"E444")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => \can_top_level_inst/drv_bus[414]\, C => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[14]\, D
         => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_694\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_0[1]\ : 
        CFG4
      generic map(INIT => x"F015")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_0_1_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_148\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_0_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_RNI44F53[7]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[6]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[7]\, 
        CC => NET_CC_CONFIG2053, P => NET_CC_CONFIG2050, Y3 => 
        NET_CC_CONFIG2051, Y3A => NET_CC_CONFIG2052);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[23]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[23]\);
    
    \sbe_ibuf[0]/U_IOPAD\ : IOPAD_IN
      port map(PAD => sbe(0), Y => \sbe_ibuf[0]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[20]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIKDJDB[15]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[288]\, C => 
        \can_top_level_inst/stat_bus[272]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_14\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIKDJDB_S[15]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG2628, P => 
        NET_CC_CONFIG2625, Y3 => NET_CC_CONFIG2626, Y3A => 
        NET_CC_CONFIG2627);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_frame_valid_com\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/un1_frame_valid_com_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_valid_com_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/read_pointer_i[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_read_pointer_i[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_read_pointer[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm_0[2]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un10_tq_counter_d_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[2]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_23[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[271]\, C => 
        \can_top_level_inst/drv_bus[143]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1149\);
    
    \data_in_ibuf[20]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[20]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[20]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_RNIQD161[2]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[1]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[2]\, 
        CC => NET_CC_CONFIG2033, P => NET_CC_CONFIG2030, Y3 => 
        NET_CC_CONFIG2031, Y3A => NET_CC_CONFIG2032);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[88]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_11\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[99]\, B => 
        \can_top_level_inst/rec_ident[18]\, C => 
        \can_top_level_inst/drv_bus[132]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_67\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[14]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a2[1]\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => \can_top_level_inst/txtb_parity_error_valid[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_178\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_address_i_i_m2[6]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_183\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[7]\ : 
        CFG3
      generic map(INIT => x"EA")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_168\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_1[27]\ : 
        CFG4
      generic map(INIT => x"007F")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[27]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac11_Z\, 
        C => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_15_Z[27]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_1_Z[27]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[9]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[41]\, 
        C => \can_top_level_inst/rec_ident[9]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2254\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7_2\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7_2_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\);
    
    
        \can_top_level_inst/int_manager_inst/int_input_active_15_1[11]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txtb_allow_bb[0]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_15_1_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_0[17]\ : 
        CFG4
      generic map(INIT => x"A989")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_0_1_Z[17]\, 
        D => \can_top_level_inst/drv_bus[50]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_0_Z[17]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/int_ena_proc.un1_int_ena_clear\ : 
        CFG4
      generic map(INIT => x"A800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        D => \data_in_c[8]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/un1_int_ena_clear\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[16]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[16]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_57\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[24]\, B => 
        \can_top_level_inst/drv_bus[105]\, C => 
        \can_top_level_inst/drv_bus[138]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_26\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[11]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[12]\, 
        CC => NET_CC_CONFIG1815, P => NET_CC_CONFIG1812, Y3 => 
        NET_CC_CONFIG1813, Y3A => NET_CC_CONFIG1814);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_28_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/N_3902\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[18]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[270]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIR5T22[2]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[275]\, C => 
        \can_top_level_inst/stat_bus[259]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_1\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIR5T22_S[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_2\, 
        CC => NET_CC_CONFIG2576, P => NET_CC_CONFIG2573, Y3 => 
        NET_CC_CONFIG2574, Y3A => NET_CC_CONFIG2575);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_8\ : 
        ARI1_CC
      generic map(INIT => x"45500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[79]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_7_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_8_FCNET1\, 
        CC => NET_CC_CONFIG244, P => NET_CC_CONFIG241, Y3 => 
        NET_CC_CONFIG242, Y3A => NET_CC_CONFIG243);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[0]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[28]\);
    
    \can_top_level_inst/int_manager_inst/int_input_active_15[11]\ : 
        CFG4
      generic map(INIT => x"F0F8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => \can_top_level_inst/is_bus_off\, C => 
        \can_top_level_inst/int_manager_inst/int_input_active_9_Z[11]\, 
        D => 
        \can_top_level_inst/int_manager_inst/int_input_active_15_1_Z[11]\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_15_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[26]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_18[24]\ : 
        CFG4
      generic map(INIT => x"6420")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[200]\, D => 
        \can_top_level_inst/drv_bus[13]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1066\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/crc_clear_match_flag_0\ : 
        CFG3
      generic map(INIT => x"EA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_clear_match_flag\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m24_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_RNO[2]\ : 
        CFG4
      generic map(INIT => x"5551")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_0_1_Z[5]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a0_0_Z[5]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_87_i_1\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_87_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/int_stat_proc.un1_int_status_clear\ : 
        CFG4
      generic map(INIT => x"F8F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_stat_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => \sbe_c[1]\, C => 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/un2_int_status_set\, 
        D => \data_in_c[8]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.8.int_module_inst/un1_int_status_clear\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_y0_0[12]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[312]\, D => 
        \can_top_level_inst/drv_bus[267]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_0_y3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\);
    
    \timestamp_ibuf[42]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[42]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[42]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped[4]\ : 
        CFG4
      generic map(INIT => x"F078")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_Z[4]\, 
        B => \can_top_level_inst/tran_frame_test.fcrc\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[4]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1192\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/is_ack_field_i_a2_RNI42VB\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1239\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => \can_top_level_inst/stat_bus[376]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[12]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_9\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_9\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_8_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_9_FCNET1\, 
        CC => NET_CC_CONFIG1521, P => NET_CC_CONFIG1518, Y3 => 
        NET_CC_CONFIG1519, Y3A => NET_CC_CONFIG1520);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[23]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_raw_0[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNILPAQ1_S[4]\, 
        B => \can_top_level_inst/drv_bus[379]\, C => 
        \can_top_level_inst/drv_bus[374]\, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_40\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_800_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_7\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_6_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_7_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_7_Z\, 
        CC => NET_CC_CONFIG1242, P => NET_CC_CONFIG1239, Y3 => 
        NET_CC_CONFIG1240, Y3A => NET_CC_CONFIG1241);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[5]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2531\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m23_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_0[4]\ : 
        CFG3
      generic map(INIT => x"37")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_ety\, B
         => \can_top_level_inst/txtb_sw_cmd_index[5]\, C => 
        \can_top_level_inst/txtb_sw_cmd.set_rdy\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_143\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI0GN76[11]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[11]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[11]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_10\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_11\, 
        CC => NET_CC_CONFIG1321, P => NET_CC_CONFIG1318, Y3 => 
        NET_CC_CONFIG1319, Y3A => NET_CC_CONFIG1320);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_34[23]\ : 
        CFG4
      generic map(INIT => x"FF8C")

      port map(A => \timestamp_c[23]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1044_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1044\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[11]\ : 
        CFG4
      generic map(INIT => x"0C01")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1048\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_960\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_822_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\);
    
    \data_in_ibuf[6]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[6]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[6]/YIN\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[117]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram46_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe46\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram46_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[16]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIGQMS\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[16]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[12]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2701\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2668\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_185\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_183\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_184\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[10]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[10]\);
    
    \data_out_obuf[31]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[31]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[31]/DOUT\, EOUT => 
        \data_out_obuf[31]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[6]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[6]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[28]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_18[13]\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => \can_top_level_inst/drv_bus[29]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_648\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[26]\ : 
        CFG4
      generic map(INIT => x"88C0")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_4828\, B
         => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        C => \can_top_level_inst/rx_buffer_inst/N_2271\, D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_13\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_0_1[5]\ : 
        CFG4
      generic map(INIT => x"CEDF")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_121\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[2]\, D
         => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_0_1_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_0[23]\ : 
        CFG4
      generic map(INIT => x"3708")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_0_1_Z[23]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_0_Z[23]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_0_Z[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_sw_cmd_index[0]\);
    
    \data_in_ibuf[25]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[25]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[25]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un6_ctrl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2_0_1[1]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_932\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_933\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2_0_1_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q[0]\ : SLE
      port map(D => \can_top_level_inst/rx_buffer_inst/N_3924_i\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8[3]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2375\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram12_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe12\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram12_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_ena_clr_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \adress_c[3]\, B => \adress_c[4]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2[6]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        Y => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.failed\ : 
        CFG4
      generic map(INIT => x"AA80")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_ide_is_arbitration\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa_2_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/failed_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c2\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[0]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[1]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[20]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_1_0[23]\ : 
        CFG4
      generic map(INIT => x"530F")

      port map(A => \can_top_level_inst/stat_bus[296]\, B => 
        \can_top_level_inst/drv_bus[104]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_1_0_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[23]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[137]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram75__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_348\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe75\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_3[2]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[33]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[29]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_3[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry[1]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s_2639_FCO\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[1]\, 
        CC => NET_CC_CONFIG1059, P => NET_CC_CONFIG1056, Y3 => 
        NET_CC_CONFIG1057, Y3A => NET_CC_CONFIG1058);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.un4_rx_shift_ena_2_0\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4_2[5]\ : 
        CFG4
      generic map(INIT => x"C808")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_5_S\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, D
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_5_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNINFTA3[0]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/txtb_port_b_address[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/N_4257\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/N_4258\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[29]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe6\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/int_stat_proc.un2_int_status_set\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/tran_valid\, B => 
        \can_top_level_inst/int_mask[1]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/un2_int_status_set\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un2_stuff_rule_violate\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_in\, B => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/prev_val_q\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un2_stuff_rule_violate_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[2]\ : 
        CFG4
      generic map(INIT => x"FF40")

      port map(A => \can_top_level_inst/N_4606\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_182\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_11[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram48_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram49_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_4[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNI3Q2C[0]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_VCC, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNI3Q2C_FCO[0]\, 
        CC => NET_CC_CONFIG1605, P => NET_CC_CONFIG1602, Y3 => 
        NET_CC_CONFIG1603, Y3A => NET_CC_CONFIG1604);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[7]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0[3]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[3]\, 
        C => \can_top_level_inst/drv_bus[59]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_52\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[12]\);
    
    
        \can_top_level_inst/bus_sampling_inst/shift_regs_rst_reg_inst/rx_shift_res_reg_inst/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/shift_regs_rst_reg_inst/rx_shift_res_reg_inst/ssp_reset_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/reg_q\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_8[30]\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => \can_top_level_inst/stat_bus[301]\, B => 
        \can_top_level_inst/stat_bus[302]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_12_0_x4_0\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[11]\);
    
    \timestamp_ibuf[24]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(24), Y => 
        \timestamp_ibuf[24]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa9\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe9\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[18]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[17]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_3_1[1]\ : 
        CFG3
      generic map(INIT => x"E0")

      port map(A => \can_top_level_inst/tran_valid\, B => 
        \can_top_level_inst/txtb_hw_cmd.failed\, C => 
        \can_top_level_inst/txtb_hw_cmd.unlock\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_183_1\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_8\ : 
        CFG4
      generic map(INIT => x"0070")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        B => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        C => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_6_Z\, 
        D => \can_top_level_inst/stat_bus[11]\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_8_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[9]\ : 
        CFG4
      generic map(INIT => x"F088")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_RNIH1071_Z\, 
        B => \can_top_level_inst/tran_word[17]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[8]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram7_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe7\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram7_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[5]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_2[4]\ : 
        CFG4
      generic map(INIT => x"F588")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1297\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1298\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_Z[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_2_Z[4]\);
    
    
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/tx_trig_req_flag_q\ : 
        SLE
      port map(D => \can_top_level_inst/stat_bus[184]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/un9_tx_trig_req_flag_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/tx_trig_req_flag_q_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[14]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[83]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\);
    
    \data_out_obuf[27]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[27]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[27]/DOUT\, EOUT => 
        \data_out_obuf[27]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_RNIBM7A[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_GND, CO => OPEN, P(0) => 
        NET_CC_CONFIG2176, P(1) => NET_CC_CONFIG2180, P(2) => 
        NET_CC_CONFIG2184, P(3) => NET_CC_CONFIG2188, P(4) => 
        NET_CC_CONFIG2192, P(5) => NET_CC_CONFIG2196, P(6) => 
        NET_CC_CONFIG2200, P(7) => NET_CC_CONFIG2204, P(8) => 
        NET_CC_CONFIG2208, P(9) => NET_CC_CONFIG2212, P(10) => 
        ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG2177, 
        Y3(1) => NET_CC_CONFIG2181, Y3(2) => NET_CC_CONFIG2185, 
        Y3(3) => NET_CC_CONFIG2189, Y3(4) => NET_CC_CONFIG2193, 
        Y3(5) => NET_CC_CONFIG2197, Y3(6) => NET_CC_CONFIG2201, 
        Y3(7) => NET_CC_CONFIG2205, Y3(8) => NET_CC_CONFIG2209, 
        Y3(9) => NET_CC_CONFIG2213, Y3(10) => ADLIB_VCC, Y3(11)
         => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG2178, Y3A(1) => 
        NET_CC_CONFIG2182, Y3A(2) => NET_CC_CONFIG2186, Y3A(3)
         => NET_CC_CONFIG2190, Y3A(4) => NET_CC_CONFIG2194, 
        Y3A(5) => NET_CC_CONFIG2198, Y3A(6) => NET_CC_CONFIG2202, 
        Y3A(7) => NET_CC_CONFIG2206, Y3A(8) => NET_CC_CONFIG2210, 
        Y3A(9) => NET_CC_CONFIG2214, Y3A(10) => ADLIB_VCC, 
        Y3A(11) => ADLIB_VCC, CC(0) => NET_CC_CONFIG2179, CC(1)
         => NET_CC_CONFIG2183, CC(2) => NET_CC_CONFIG2187, CC(3)
         => NET_CC_CONFIG2191, CC(4) => NET_CC_CONFIG2195, CC(5)
         => NET_CC_CONFIG2199, CC(6) => NET_CC_CONFIG2203, CC(7)
         => NET_CC_CONFIG2207, CC(8) => NET_CC_CONFIG2211, CC(9)
         => NET_CC_CONFIG2215, CC(10) => nc289, CC(11) => nc294);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[14]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1455_2\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram60__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_347\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe60\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[7]\ : 
        CFG4
      generic map(INIT => x"F888")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        C => \can_top_level_inst/txtb_parity_error_valid[1]\, D
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60[3]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[4]\ : 
        CFG4
      generic map(INIT => x"0405")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451_1\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[4]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1035\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_836_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[31]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[31]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[21]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d[0]\ : 
        CFG3
      generic map(INIT => x"48")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un4_bst_ctr_d_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/reg_q\, 
        C => \can_top_level_inst/stat_bus[300]\, Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac19\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac19_0_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac19_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[19]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2477\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[3]\ : 
        CFG4
      generic map(INIT => x"A0E0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[17]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[17]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[17]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[19]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[9]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_len_can_fd_1_sqmuxa_RNIRPME\ : 
        CFG4
      generic map(INIT => x"4F00")

      port map(A => \can_top_level_inst/store_data_word[1]\, B
         => \can_top_level_inst/store_data_word[0]\, C => 
        \can_top_level_inst/can_core_inst/bds_data_out\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_len_can_fd_1_sqmuxa_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[25]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[24]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4FU8D_S[18]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[206]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram106_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe106\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram106_0\);
    
    \data_in_ibuf[1]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(1), Y => \data_in_ibuf[1]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[26]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe8\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q_RNO[9]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_15[8]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[14]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/int_stat_proc.un1_int_status_clear\ : 
        CFG4
      generic map(INIT => x"88F8")

      port map(A => \data_in_c[4]\, B => 
        \can_top_level_inst/N_138\, C => 
        \can_top_level_inst/fcs_changed\, D => 
        \can_top_level_inst/int_mask[4]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/un1_int_status_clear\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_RNIF4O01[1]\ : 
        CFG4
      generic map(INIT => x"E7DB")

      port map(A => \can_top_level_inst/drv_bus[9]\, B => 
        \can_top_level_inst/drv_bus[8]\, C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[0]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[1]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_NE_5_0\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[13]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2637\, Y
         => \can_top_level_inst/memory_registers_inst/N_2702\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram23_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe23\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram23_0\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_RNO[27]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2683\, D
         => \can_top_level_inst/memory_registers_inst/N_2716\, Y
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram97__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_350\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe97\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_3_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[21]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[21]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_9_RNIABLB\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un7_exit_segm_regular_cry_9\, 
        B => \can_top_level_inst/prescaler_inst/is_tseg1\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[149]\);
    
    
        \can_top_level_inst/frame_filters_inst/un1_filter_b_enable_1_0\ : 
        CFG4
      generic map(INIT => x"0F35")

      port map(A => \can_top_level_inst/drv_bus[172]\, B => 
        \can_top_level_inst/drv_bus[174]\, C => 
        \can_top_level_inst/stat_bus[156]\, D => 
        \can_top_level_inst/stat_bus[155]\, Y => 
        \can_top_level_inst/frame_filters_inst/un1_filter_b_enable_1_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI817P2[3]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[276]\, C => 
        \can_top_level_inst/stat_bus[260]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_2\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI817P2_S[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_3\, 
        CC => NET_CC_CONFIG2580, P => NET_CC_CONFIG2577, Y3 => 
        NET_CC_CONFIG2578, Y3A => NET_CC_CONFIG2579);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_3_1_RNIQCRL3\ : 
        ARI1_CC
      generic map(INIT => x"508F7")

      port map(A => \can_top_level_inst/stat_bus[84]\, B => 
        \can_top_level_inst/err_detected\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_0_1\, 
        D => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_2\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_3\, 
        CC => NET_CC_CONFIG341, P => NET_CC_CONFIG338, Y3 => 
        NET_CC_CONFIG339, Y3A => NET_CC_CONFIG340);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1[5]\ : 
        CFG4
      generic map(INIT => x"0F0E")

      port map(A => \can_top_level_inst/stat_bus[89]\, B => 
        \can_top_level_inst/stat_bus[88]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNITK7L2_S\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\);
    
    
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_28\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_19_Z\, 
        B => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_18_Z\, 
        C => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_17_Z\, 
        D => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_16_Z\, 
        Y => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_28_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un1_same_bits_rst_trig\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => 
        \can_top_level_inst/can_core_inst/stuff_length[0]\, B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_data_in\, 
        C => \can_top_level_inst/can_core_inst/bst_data_out\, Y
         => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un1_same_bits_rst_trig_Z\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count_RNO_0[5]\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, Y
         => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_i_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_3[30]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac4_Z\, 
        C => \can_top_level_inst/stat_bus[250]\, D => 
        \can_top_level_inst/stat_bus[218]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_3_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_6_1[25]\ : 
        CFG4
      generic map(INIT => x"00CA")

      port map(A => \can_top_level_inst/txt_buf_failed_bof\, B
         => \can_top_level_inst/drv_bus[168]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1092_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[31]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_dest[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_20\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_20_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[26]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_3[30]\ : 
        CFG4
      generic map(INIT => x"F909")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_12_0_x4_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_8_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        D => \can_top_level_inst/tran_word[6]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_13_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[12]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[12]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[12]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa11\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe11\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa7\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe7\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1[0]\ : 
        CFG3
      generic map(INIT => x"21")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_1[5]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[12]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un9_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[13]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[6]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[6]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[20]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[20]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m36_1_0_wmux\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_17_0\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_20_0\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m36_1_0_y0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[9]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram28__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_351\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe28\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIL4PC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[7]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[7]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNITF3E3[0]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/txtb_port_b_address[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/N_4137\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/N_4138\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_0[20]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_0_1_Z[20]\, 
        B => \can_top_level_inst/txtb_prorities_5[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_0_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/txtb_cmd_index_gen.5.txtb_priority_odd_gen.txtb_sw_cmd_index_8[5]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[4]\, B
         => \can_top_level_inst/drv_bus[475]\, C => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[13]\, 
        Y => \can_top_level_inst/txtb_sw_cmd_index[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_2[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/shift_reg_in_4[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un17_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_1_i\, 
        SD => ADLIB_VCC, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[8]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry[5]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[4]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[5]\, 
        CC => NET_CC_CONFIG1075, P => NET_CC_CONFIG1072, Y3 => 
        NET_CC_CONFIG1073, Y3A => NET_CC_CONFIG1074);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIC0M98[10]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[198]\, D => 
        \can_top_level_inst/stat_bus[230]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_9\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIC0M98_S[10]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_10\, 
        CC => NET_CC_CONFIG796, P => NET_CC_CONFIG793, Y3 => 
        NET_CC_CONFIG794, Y3A => NET_CC_CONFIG795);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[6]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[13]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_37_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[377]\, D => 
        \can_top_level_inst/stat_bus[206]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_37_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[18]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.unlock_2_1_0\ : 
        CFG4
      generic map(INIT => x"BA10")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/set_idle_i_5_sqmuxa\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_717_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_arbitration_lost_condition_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_ide_is_arbitration\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1497_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[27]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q_RNO[12]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_17[11]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06\ : 
        CFG4
      generic map(INIT => x"F0FE")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_i1_mux_1\, 
        B => \can_top_level_inst/drv_bus_475_rep2\, C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_06\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_129\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[13]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_1_1[14]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/drv_bus[157]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_1_1_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[21]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[21]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        B => \can_top_level_inst/txtb_hw_cmd.unlock\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[29]\);
    
    \timestamp_ibuf[28]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(28), Y => 
        \timestamp_ibuf[28]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[20]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[20]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_1[3]\ : 
        CFG3
      generic map(INIT => x"F4")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => \can_top_level_inst/trv_delay[3]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_1_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1[7]\ : 
        CFG4
      generic map(INIT => x"0E02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[7]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[7]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[21]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[197]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[30]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2654\, Y
         => \can_top_level_inst/memory_registers_inst/N_2719\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[23]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[26]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[26]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[1]\ : 
        CFG4
      generic map(INIT => x"F088")

      port map(A => \can_top_level_inst/rec_dlc[1]\, B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        C => \can_top_level_inst/rx_buffer_inst/N_2279\, D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/next_state_15_sqmuxa_i_o4_0\ : 
        CFG2
      generic map(INIT => x"D")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_980\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_7\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_7\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_6_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_7_Z\, 
        CC => NET_CC_CONFIG1513, P => NET_CC_CONFIG1510, Y3 => 
        NET_CC_CONFIG1511, Y3A => NET_CC_CONFIG1512);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_15_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1471\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1474\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_15_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[28]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[14]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[13]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_can_fd_1_sqmuxa_RNIDGQL\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \can_top_level_inst/tran_dlc[2]\, B => 
        \can_top_level_inst/tran_dlc[1]\, C => 
        \can_top_level_inst/tran_dlc[0]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_len_can_fd_1_sqmuxa_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram53_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe53\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram53_0\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[26]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[26]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[26]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_16\ : 
        CFG3
      generic map(INIT => x"96")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_16_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[3]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[3]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2627\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[19]\);
    
    \data_out_obuf[0]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[0]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[0]/DOUT\, EOUT => \data_out_obuf[0]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_24\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[21]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_17_Z\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_3_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_24_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8_2[4]\ : 
        CFG4
      generic map(INIT => x"A820")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => \can_top_level_inst/rx_mem_free[4]\, D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_4_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[19]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1_0_1[10]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[310]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1_0_1_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[29]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[29]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[29]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[13]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[13]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[13]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[13]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI0A1LA[27]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[27]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[27]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_26\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_27\, 
        CC => NET_CC_CONFIG1385, P => NET_CC_CONFIG1382, Y3 => 
        NET_CC_CONFIG1383, Y3A => NET_CC_CONFIG1384);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_38_1_wmux_RNO_0[19]\ : 
        CFG4
      generic map(INIT => x"AD0D")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_886\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_2_1_1[19]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_1_0_Z[19]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_888\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_3[9]\ : 
        CFG3
      generic map(INIT => x"31")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[123]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1471\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[14]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[14]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[20]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[12]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_1_i\, 
        SD => ADLIB_VCC, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_3\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_2_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_3_Z\, 
        CC => NET_CC_CONFIG139, P => NET_CC_CONFIG136, Y3 => 
        NET_CC_CONFIG137, Y3A => NET_CC_CONFIG138);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[8]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_1_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\);
    
    \data_in_ibuf[13]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[13]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[13]/YIN\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_RNI3F284[2]\ : 
        ARI1_CC
      generic map(INIT => x"48800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy_Y[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[2]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[1]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[2]\, 
        CC => NET_CC_CONFIG661, P => NET_CC_CONFIG658, Y3 => 
        NET_CC_CONFIG659, Y3A => NET_CC_CONFIG660);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[27]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_3\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[7]\, B
         => \can_top_level_inst/can_core_inst/crc_15[6]\, C => 
        \can_top_level_inst/can_core_inst/crc_15[7]\, D => 
        \can_top_level_inst/store_data_word[6]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[2]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[3]\, 
        CC => NET_CC_CONFIG62, P => NET_CC_CONFIG59, Y3 => 
        NET_CC_CONFIG60, Y3A => NET_CC_CONFIG61);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8[1]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2373\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[1]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[16]\ : 
        CFG4
      generic map(INIT => x"553C")

      port map(A => \can_top_level_inst/drv_bus[510]\, B => 
        \can_top_level_inst/can_core_inst/crc_17[15]\, C => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_nxt_Z\, 
        D => \can_top_level_inst/can_core_inst/load_init_vect\, Y
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_2\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20_1\, 
        B => swr_c, C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        D => \adress_c[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI99IS\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\);
    
    \sbe_ibuf[1]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \sbe_c[1]\, E => ADLIB_GND, YIN => 
        \sbe_ibuf[1]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[11]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m12\ : 
        CFG4
      generic map(INIT => x"0CAF")

      port map(A => \can_top_level_inst/int_vector[7]\, B => 
        \can_top_level_inst/memory_registers_inst/un9_status_comb_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_13_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[30]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm_0[7]\ : 
        CFG4
      generic map(INIT => x"B830")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/next_state_16_sqmuxa\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[7]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un3_data_length_sub_c_axbxc4\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[22]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[6]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31[11]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[23]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[23]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[23]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[23]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[23]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[23]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[26]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_51\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[3]\, B => 
        \can_top_level_inst/drv_bus[84]\, C => 
        \can_top_level_inst/drv_bus[117]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_32\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[0]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[1]\, 
        CC => NET_CC_CONFIG1771, P => NET_CC_CONFIG1768, Y3 => 
        NET_CC_CONFIG1769, Y3A => NET_CC_CONFIG1770);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/retr_ctr_add_block\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retr_ctr_add\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_retr_ctr_add_block_clr\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/retr_ctr_add_block_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_RNO[6]\ : 
        ARI1_CC
      generic map(INIT => x"49900")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2\, 
        C => \can_top_level_inst/stat_bus[87]\, D => ADLIB_GND, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_5\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_RNO_S[6]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG2342, P => 
        NET_CC_CONFIG2339, Y3 => NET_CC_CONFIG2340, Y3A => 
        NET_CC_CONFIG2341);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[12]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_8_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_8_FCNET1\, 
        CO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_8_Z\, 
        CC => NET_CC_CONFIG1565, P => NET_CC_CONFIG1562, Y3 => 
        NET_CC_CONFIG1563, Y3A => NET_CC_CONFIG1564);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[22]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_RNI791Q4\ : 
        CFG3
      generic map(INIT => x"47")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06_i\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_i\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_16_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l2_prio_06_RNI791Q4_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4[7]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_1[15]\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[4]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[3]\, C => 
        \can_top_level_inst/tran_frame_test.fcrc\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1_0_Z[7]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_1_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram35_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe35\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram35_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe6\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram110__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_354\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe110\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_sw_cmd_index[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_2_0_1[20]\ : 
        CFG3
      generic map(INIT => x"35")

      port map(A => \can_top_level_inst/stat_bus[277]\, B => 
        \can_top_level_inst/drv_bus[291]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_2_0_1_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[4]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_shadowed[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/trv_delay[3]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI2H4H1[1]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[189]\, D => 
        \can_top_level_inst/stat_bus[221]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_0\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI2H4H1_S[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_1\, 
        CC => NET_CC_CONFIG760, P => NET_CC_CONFIG757, Y3 => 
        NET_CC_CONFIG758, Y3A => NET_CC_CONFIG759);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[7]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[10]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[10]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[10]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[8]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[8]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[8]\);
    
    \data_out_obuf[19]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(19), D => \data_out_obuf[19]/DOUT\, 
        E => \data_out_obuf[19]/EOUT\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[1]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram0_0\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram1_0\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m36_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_0[13]\ : 
        CFG4
      generic map(INIT => x"C0E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_0_1_Z[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_0_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[14]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[14]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\);
    
    \timestamp_ibuf[48]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[48]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[48]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[27]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2553\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2485\, Y
         => \can_top_level_inst/memory_registers_inst/N_2683\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_13[30]\ : 
        CFG4
      generic map(INIT => x"FFFB")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[2]\, 
        B => \can_top_level_inst/tran_frame_test.fstc\, C => 
        \can_top_level_inst/tran_frame_test.tprm[3]\, D => 
        \can_top_level_inst/tran_frame_test.tprm[4]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_12_0_o4_0_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_0_0\, 
        D => \adress_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_37_2[12]\ : 
        CFG4
      generic map(INIT => x"DA8A")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_37_1_Z[12]\, 
        B => \can_top_level_inst/stat_bus[376]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => \can_top_level_inst/stat_bus[232]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_37_2_Z[12]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram1_[0]\ : 
        SLE
      port map(D => can_tx_c, CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, 
        EN => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe1\, 
        ALn => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn
         => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_38_1_wmux_RNO_0[1]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_2_0_Z[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_2_Z[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1194\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[2]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_d_0_RNI1FNF\ : 
        ARI1_CC
      generic map(INIT => x"47F00")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_meas_progress_q_Z\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_d_0_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_d_4_Z\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[2]\, 
        FCI => ADLIB_VCC, S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_0_cy\, 
        CC => NET_CC_CONFIG1641, P => NET_CC_CONFIG1638, Y3 => 
        NET_CC_CONFIG1639, Y3A => NET_CC_CONFIG1640);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[17]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[217]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[13]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[13]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[13]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[4]\);
    
    \timestamp_ibuf[4]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(4), Y => \timestamp_ibuf[4]/YIN\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[30]\ : 
        SLE
      port map(D => \timestamp_c[30]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[30]\);
    
    \adress_ibuf[2]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \adress_c[2]\, E => ADLIB_GND, YIN => 
        \adress_ibuf[2]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[8]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[31]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[11]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2256\, B
         => \can_top_level_inst/rx_buffer_inst/N_4271\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2289\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[30]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4[0]\ : 
        CFG4
      generic map(INIT => x"53AA")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[0]\, 
        B => \can_top_level_inst/rx_mem_free[0]\, C => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, D
         => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[0]\);
    
    AFLSDF_INV_7 : INV_BA
      port map(A => \AFLSDF_INV_7\, Y => 
        \can_top_level_inst/reg_q_arst\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o2[35]\ : 
        CFG3
      generic map(INIT => x"7F")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\, 
        C => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_930\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[24]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2269\, B
         => \can_top_level_inst/rx_buffer_inst/N_4826\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[5]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[26]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[26]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[26]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[21]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2547\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un7_btmc_meas_running_d\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => \can_top_level_inst/btmc_reset\, B => 
        \can_top_level_inst/gen_first_ssp\, Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un7_btmc_meas_running_d_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.un2_commit_rx_frame\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, Y
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un2_commit_rx_frame\);
    
    \data_in_ibuf[16]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[16]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[16]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_pointer_inc_1_i_axbxc4\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_pointer_inc_1_i_c4\, 
        B => \can_top_level_inst/rx_read_pointer[4]\, Y => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[3]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[408]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[0]\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_ety\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[0]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_36_1[0]\ : 
        CFG4
      generic map(INIT => x"00D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[0]\, C => 
        \can_top_level_inst/stat_bus[188]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1162_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6\ : 
        RGB
      port map(A => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_Y\, 
        EN => ADLIB_VCC, Y => \AFLSDF_INV_12\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[87]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[6]\ : 
        CFG4
      generic map(INIT => x"CDCC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[2]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2528\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2460\, Y
         => \can_top_level_inst/memory_registers_inst/N_2658\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[1]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_148\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m18_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[286]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[10]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_shadowed[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/trv_delay[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.un109_ctrl_ctr_zero\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_one\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un109_ctrl_ctr_zero\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[21]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram60_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe60\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram60_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[507]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[7]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[7]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[7]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram118__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_349\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe118\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_0_1[4]\ : 
        CFG4
      generic map(INIT => x"4373")

      port map(A => \can_top_level_inst/txtb_prorities_1[0]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1_0_Z[4]\, 
        D => \can_top_level_inst/rx_frame_count[0]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_0_1_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[6]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2695\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2662\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dff_prev_val_reg/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/prev_val_d_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_triggers[1]\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/prev_val_q\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60[5]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[25]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[25]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[2]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[2]\, 
        B => \can_top_level_inst/txtb_hw_cmd.unlock\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_134\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[6]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[6]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_1_i\, 
        SD => ADLIB_VCC, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[27]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_3[3]\ : 
        CFG4
      generic map(INIT => x"FEEE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_1_Z[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_2\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        D => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_3_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_38_1_0_wmux_RNO[19]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_0_Z[19]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_Z[19]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_873\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[379]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[2]\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst_priority_decoder_inst_un1_m2_0_a2_0_1\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_2_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_155\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_28_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/N_3901\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[17]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[17]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[17]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_822_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_RNO_7[28]\ : 
        CFG4
      generic map(INIT => x"FFBF")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[2]\, 
        B => \can_top_level_inst/tran_frame_test.fstc\, C => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, D => 
        \can_top_level_inst/tran_frame_test.tprm[4]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_o4_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m25_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_0[2]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[8]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[8]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[8]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[8]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_type_dbl_buf\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[7]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/buffer_md_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_type_dbl_buf_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_38_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_759\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_774\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_38_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_1[2]\ : 
        CFG2
      generic map(INIT => x"B")

      port map(A => \can_top_level_inst/int_vector[2]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1203\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_sync_control_d\ : 
        CFG4
      generic map(INIT => x"FCEC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[1]\, 
        C => \can_top_level_inst/stat_bus[0]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_switch_data\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_sync_control_d_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_2[31]\ : 
        CFG4
      generic map(INIT => x"FEFA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac2_Z\, 
        B => \can_top_level_inst/stat_bus[219]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_0_Z[31]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_2_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[24]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[24]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_9[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[300]\, C => 
        \can_top_level_inst/drv_bus[176]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1135\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_2[17]\ : 
        CFG4
      generic map(INIT => x"AFC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_787\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_786\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_Z[17]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_2_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_1[21]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[197]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_1_Z[21]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dff_data_out_val_reg/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_in\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_triggers[1]\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bds_data_out\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[17]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2543\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2475\, Y
         => \can_top_level_inst/memory_registers_inst/N_2673\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate_3_0_.m21\ : 
        CFG4
      generic map(INIT => x"0110")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[3]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_22\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[17]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2543\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[23]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[23]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[23]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/command_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_0_RNI20231\ : 
        CFG4
      generic map(INIT => x"7FFF")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/access_active_1[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        D => \data_in_c[9]\, Y => 
        \can_top_level_inst/memory_registers_inst/command_i[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[18]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[18]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[25]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0[13]\ : 
        CFG4
      generic map(INIT => x"FF35")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4Q6DB_S[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[235]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/first_err_delim_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_869_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_first_err_delim_q\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_36_1[2]\ : 
        CFG4
      generic map(INIT => x"00D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[2]\, C => 
        \can_top_level_inst/stat_bus[190]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1238_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[5]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_0_1[23]\ : 
        CFG3
      generic map(INIT => x"35")

      port map(A => \can_top_level_inst/stat_bus[280]\, B => 
        \can_top_level_inst/drv_bus[294]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_0_1_Z[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_19[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/stat_bus[257]\, B => 
        \can_top_level_inst/drv_bus[16]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1145\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[306]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un3_reinteg_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[25]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0[2]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_6[30]\ : 
        CFG3
      generic map(INIT => x"BA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_1226_1\, 
        B => \can_top_level_inst/can_core_inst/crc_21[19]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_1226\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[11]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[11]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[22]\ : 
        CFG4
      generic map(INIT => x"5554")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[15]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_934\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_800_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[12]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_1\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/N_3903\, 
        B => \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/drv_bus[476]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[0]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[0]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2624\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_2[10]\ : 
        CFG4
      generic map(INIT => x"D05A")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \can_top_level_inst/stat_bus[267]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1_Z[10]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_2_1[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_2_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[7]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[3]\, 
        C => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[20]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_21_2[16]\ : 
        CFG4
      generic map(INIT => x"C840")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/stat_bus[109]\, D => 
        \can_top_level_inst/rx_read_buff[16]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_765_2\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNO[14]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[14]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[13]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[14]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG1734, P => 
        NET_CC_CONFIG1731, Y3 => NET_CC_CONFIG1732, Y3A => 
        NET_CC_CONFIG1733);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_phase_err_sjw_by_one_1.g0\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => \can_top_level_inst/drv_bus[58]\, B => 
        \can_top_level_inst/drv_bus[57]\, C => 
        \can_top_level_inst/drv_bus[56]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/CO2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[26]\ : 
        CFG4
      generic map(INIT => x"EC00")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[26]\, B
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[24]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_length[2]\ : 
        CFG4
      generic map(INIT => x"2C0C")

      port map(A => \can_top_level_inst/stat_bus[156]\, B => 
        \can_top_level_inst/store_data_word[1]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_data_len_can_2_0_i_0\, 
        D => \can_top_level_inst/can_core_inst/bds_data_out\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram79__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_342\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe79\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[17]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[23]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2647\, Y
         => \can_top_level_inst/memory_registers_inst/N_2712\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[14]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[13]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram61_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe61\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram61_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[1]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un4_tst_ena\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[16]\, B
         => \can_top_level_inst/reg_value_r_rep[17]\, C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena_2\, 
        D => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sync_control_q_RNO[1]\ : 
        CFG3
      generic map(INIT => x"32")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1457\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_sync_control_d_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un12_sync_control_d_i\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/next_state_i[0]\ : 
        CFG4
      generic map(INIT => x"1300")

      port map(A => \can_top_level_inst/stat_bus[108]\, B => 
        \can_top_level_inst/is_bus_off\, C => 
        \can_top_level_inst/stat_bus[79]\, D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un12_tx_err_ctr_mt_erp\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/N_27_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_1_Z\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_2_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_2_Z\, 
        CC => NET_CC_CONFIG2227, P => NET_CC_CONFIG2224, Y3 => 
        NET_CC_CONFIG2225, Y3A => NET_CC_CONFIG2226);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/un1_txtb_available_i_0_s_tz\ : 
        CFG4
      generic map(INIT => x"8A80")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt_fast\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_tx_command_reg_comp_reg_value_r_fast[14]\, 
        C => \can_top_level_inst/drv_bus_fast[475]\, D => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_un1_txtb_available_i_0_out_tz\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[15]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[15]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/write_raw_intent_i_0\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, B => 
        \can_top_level_inst/rx_buffer_inst/reset_overrun_flag\, Y
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/write_raw_intent_i_0_Z\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[12]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[12]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[12]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4818\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram91__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_340\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe91\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[20]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[26]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[7]\, CLK
         => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_16[6]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => \can_top_level_inst/int_vector[6]\, C => 
        \can_top_level_inst/drv_bus[244]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1370\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_clk_en_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d_Z\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/txtb_clk_en\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[13]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[21]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_76\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[152]\, B => 
        \can_top_level_inst/rec_ident[9]\, C => 
        \can_top_level_inst/drv_bus[185]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_11\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un8_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_load_init_vect_0_a4\ : 
        CFG4
      generic map(INIT => x"0504")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1269\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1183\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_a3[5]\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[2]\, 
        B => \can_top_level_inst/rec_valid_f\, C => 
        \can_top_level_inst/rec_abort_f\, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[15]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[15]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[15]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[220]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_13[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram116_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram117_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_5[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNI7L5O[1]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNI3Q2C_FCO[0]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[1]\, 
        CC => NET_CC_CONFIG1609, P => NET_CC_CONFIG1606, Y3 => 
        NET_CC_CONFIG1607, Y3A => NET_CC_CONFIG1608);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_o2[0]\ : 
        CFG4
      generic map(INIT => x"FFDF")

      port map(A => \can_top_level_inst/drv_bus[476]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_146\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_109_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[10]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[19]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_dest[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[19]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNIN8VE[24]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        Y => \can_top_level_inst/btmc_reset\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06_RNO_2\ : 
        CFG4
      generic map(INIT => x"084C")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt_fast\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[0]\, D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm_0[5]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un10_tq_counter_d_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[5]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_RNI04PO2[6]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[5]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[6]\, 
        CC => NET_CC_CONFIG2049, P => NET_CC_CONFIG2046, Y3 => 
        NET_CC_CONFIG2047, Y3A => NET_CC_CONFIG2048);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[23]\);
    
    \data_out_obuf[11]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[11]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[11]/DOUT\, EOUT => 
        \data_out_obuf[11]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[6]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIHRMS\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[0]\);
    
    \data_out_obuf[25]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(25), D => \data_out_obuf[25]/DOUT\, 
        E => \data_out_obuf[25]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[27]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1218\, 
        B => \can_top_level_inst/tran_word[3]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[27]\ : 
        SLE
      port map(D => \data_in_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[170]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_34[5]\ : 
        CFG4
      generic map(INIT => x"CCFE")

      port map(A => \can_top_level_inst/trv_delay[5]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1350_2\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1350\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe12_0_a4\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_342\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_9_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[18]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[18]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[18]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_wmux_1\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[2]\, B
         => \can_top_level_inst/txtb_hw_cmd_index[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_135_i\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_135_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_y0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[0]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_rdy\, B
         => \can_top_level_inst/txtb_sw_cmd_index[6]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_141\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_0[1]\ : 
        CFG4
      generic map(INIT => x"FF04")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        B => \can_top_level_inst/txtb_sw_cmd.set_ety\, C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_148\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_193\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_0_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram83_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe83\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram83_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[6]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4[2]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[8]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[8]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[8]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry[5]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[4]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[5]\, 
        CC => NET_CC_CONFIG1473, P => NET_CC_CONFIG1470, Y3 => 
        NET_CC_CONFIG1471, Y3A => NET_CC_CONFIG1472);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[24]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[22]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2267\, B
         => \can_top_level_inst/rx_buffer_inst/N_4824\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[23]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_1_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO2248, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => 
        ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_VCC, P(6) => 
        ADLIB_VCC, P(7) => ADLIB_VCC, P(8) => ADLIB_GND, P(9) => 
        NET_CC_CONFIG2250, P(10) => NET_CC_CONFIG2254, P(11) => 
        NET_CC_CONFIG2258, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => ADLIB_VCC, Y3(3) => ADLIB_VCC, Y3(4) => 
        ADLIB_VCC, Y3(5) => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7)
         => ADLIB_VCC, Y3(8) => ADLIB_VCC, Y3(9) => 
        NET_CC_CONFIG2251, Y3(10) => NET_CC_CONFIG2255, Y3(11)
         => NET_CC_CONFIG2259, Y3A(0) => ADLIB_VCC, Y3A(1) => 
        ADLIB_VCC, Y3A(2) => ADLIB_VCC, Y3A(3) => ADLIB_VCC, 
        Y3A(4) => ADLIB_VCC, Y3A(5) => ADLIB_VCC, Y3A(6) => 
        ADLIB_VCC, Y3A(7) => ADLIB_VCC, Y3A(8) => ADLIB_VCC, 
        Y3A(9) => NET_CC_CONFIG2252, Y3A(10) => NET_CC_CONFIG2256, 
        Y3A(11) => NET_CC_CONFIG2260, CC(0) => nc152, CC(1) => 
        nc127, CC(2) => nc103, CC(3) => nc235, CC(4) => nc76, 
        CC(5) => nc347, CC(6) => nc208, CC(7) => nc354, CC(8) => 
        nc140, CC(9) => NET_CC_CONFIG2253, CC(10) => 
        NET_CC_CONFIG2257, CC(11) => NET_CC_CONFIG2261);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[12]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[12]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[12]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2636\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_tx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[75]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_i_o2_1[16]\ : 
        CFG3
      generic map(INIT => x"7F")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[0]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[2]\, C => 
        \can_top_level_inst/tran_frame_test.tprm[1]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_i_o2_1_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[18]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[18]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[18]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2642\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram6__RNO[0]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[2]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[1]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe6\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[31]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[31]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[31]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_27[2]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => \can_top_level_inst/stat_bus[102]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1229\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_0[27]\ : 
        CFG4
      generic map(INIT => x"FEFC")

      port map(A => \timestamp_c[59]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac2_Z\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_0_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa17\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe17\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_read_exp\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_read_exp_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[7]\ : 
        CFG4
      generic map(INIT => x"FFA2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[3]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_177\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[28]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2554\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_8[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[242]\, C => 
        \can_top_level_inst/drv_bus[118]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1286\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un4_txtb_index_muxed_i[0]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[0]\, B
         => \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[0]\, 
        Y => \can_top_level_inst/txtb_index_muxed[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[27]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[27]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[27]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2651\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_6_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[6]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[1]\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_178\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_33[24]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \timestamp_c[24]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1081\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram62__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_347\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe62\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIKHKK[0]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_0_cy\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIKHKK_S[0]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_0\, 
        CC => NET_CC_CONFIG1645, P => NET_CC_CONFIG1642, Y3 => 
        NET_CC_CONFIG1643, Y3A => NET_CC_CONFIG1644);
    
    \timestamp_ibuf[49]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[49]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[49]/YIN\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI8J6I[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO1271, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => 
        ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_GND, P(6) => 
        NET_CC_CONFIG1274, P(7) => NET_CC_CONFIG1278, P(8) => 
        NET_CC_CONFIG1282, P(9) => NET_CC_CONFIG1286, P(10) => 
        NET_CC_CONFIG1290, P(11) => NET_CC_CONFIG1294, Y3(0) => 
        ADLIB_VCC, Y3(1) => ADLIB_VCC, Y3(2) => ADLIB_VCC, Y3(3)
         => ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5) => ADLIB_VCC, 
        Y3(6) => NET_CC_CONFIG1275, Y3(7) => NET_CC_CONFIG1279, 
        Y3(8) => NET_CC_CONFIG1283, Y3(9) => NET_CC_CONFIG1287, 
        Y3(10) => NET_CC_CONFIG1291, Y3(11) => NET_CC_CONFIG1295, 
        Y3A(0) => ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => 
        ADLIB_VCC, Y3A(3) => ADLIB_VCC, Y3A(4) => ADLIB_VCC, 
        Y3A(5) => ADLIB_VCC, Y3A(6) => NET_CC_CONFIG1276, Y3A(7)
         => NET_CC_CONFIG1280, Y3A(8) => NET_CC_CONFIG1284, 
        Y3A(9) => NET_CC_CONFIG1288, Y3A(10) => NET_CC_CONFIG1292, 
        Y3A(11) => NET_CC_CONFIG1296, CC(0) => nc257, CC(1) => 
        nc86, CC(2) => nc95, CC(3) => nc327, CC(4) => nc120, 
        CC(5) => nc165, CC(6) => NET_CC_CONFIG1277, CC(7) => 
        NET_CC_CONFIG1281, CC(8) => NET_CC_CONFIG1285, CC(9) => 
        NET_CC_CONFIG1289, CC(10) => NET_CC_CONFIG1293, CC(11)
         => NET_CC_CONFIG1297);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[22]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_4_1_sx\ : 
        CFG4
      generic map(INIT => x"7775")

      port map(A => \can_top_level_inst/prescaler_inst/g1\, B => 
        \can_top_level_inst/prescaler_inst_tq_edge_dbt\, C => 
        \can_top_level_inst/stat_bus[10]\, D => 
        \can_top_level_inst/stat_bus[11]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_4_1_sx_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_29\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_20_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_19_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_18_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_17_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_29_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[29]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[29]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[29]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/dff_data_out_reg/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/dff_data_out_reg/N_374_mux\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un4_data_out_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bst_data_out\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_10[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y21[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y9[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[4]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_wmux_10_Y[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[20]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_2[20]\ : 
        CFG4
      generic map(INIT => x"4F50")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_2_1[20]\, 
        B => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_Z[20]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_2_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\);
    
    AFLSDF_INV_9 : INV_BA
      port map(A => \AFLSDF_INV_9\, Y => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[19]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[2]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[0]\, 
        B => \can_top_level_inst/txtb_hw_cmd.unlock\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_134\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_1_0[22]\ : 
        CFG4
      generic map(INIT => x"D9C8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_981\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_980\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_1_0_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[1]\);
    
    \adress_ibuf[2]/U_IOPAD\ : IOPAD_IN
      port map(PAD => adress(2), Y => \adress_ibuf[2]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0[31]\ : 
        CFG4
      generic map(INIT => x"F1F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[8]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_data_nbs_prev_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1033\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_arb_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[25]\ : 
        CFG4
      generic map(INIT => x"EC00")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[25]\, B
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[24]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_5\ : 
        CFG3
      generic map(INIT => x"B4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[4]\, 
        B => \can_top_level_inst/drv_bus[60]\, C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[5]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_5_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bus_off_req_capt_proc.un3_rx_trigger\ : 
        CFG3
      generic map(INIT => x"BA")

      port map(A => \can_top_level_inst/drv_bus[429]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_966\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_rx_trigger\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[13]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry_cy[0]\ : 
        ARI1_CC
      generic map(INIT => x"45500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => ADLIB_VCC, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry_cy\, 
        CC => NET_CC_CONFIG1674, P => NET_CC_CONFIG1671, Y3 => 
        NET_CC_CONFIG1672, Y3A => NET_CC_CONFIG1673);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[31]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[31]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_0_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO1828, CO => CI_TO_CO1829, P(0) => 
        NET_CC_CONFIG1866, P(1) => NET_CC_CONFIG1870, P(2) => 
        NET_CC_CONFIG1874, P(3) => NET_CC_CONFIG1878, P(4) => 
        NET_CC_CONFIG1882, P(5) => NET_CC_CONFIG1886, P(6) => 
        NET_CC_CONFIG1890, P(7) => NET_CC_CONFIG1894, P(8) => 
        NET_CC_CONFIG1898, P(9) => NET_CC_CONFIG1902, P(10) => 
        NET_CC_CONFIG1906, P(11) => NET_CC_CONFIG1910, Y3(0) => 
        NET_CC_CONFIG1867, Y3(1) => NET_CC_CONFIG1871, Y3(2) => 
        NET_CC_CONFIG1875, Y3(3) => NET_CC_CONFIG1879, Y3(4) => 
        NET_CC_CONFIG1883, Y3(5) => NET_CC_CONFIG1887, Y3(6) => 
        NET_CC_CONFIG1891, Y3(7) => NET_CC_CONFIG1895, Y3(8) => 
        NET_CC_CONFIG1899, Y3(9) => NET_CC_CONFIG1903, Y3(10) => 
        NET_CC_CONFIG1907, Y3(11) => NET_CC_CONFIG1911, Y3A(0)
         => NET_CC_CONFIG1868, Y3A(1) => NET_CC_CONFIG1872, 
        Y3A(2) => NET_CC_CONFIG1876, Y3A(3) => NET_CC_CONFIG1880, 
        Y3A(4) => NET_CC_CONFIG1884, Y3A(5) => NET_CC_CONFIG1888, 
        Y3A(6) => NET_CC_CONFIG1892, Y3A(7) => NET_CC_CONFIG1896, 
        Y3A(8) => NET_CC_CONFIG1900, Y3A(9) => NET_CC_CONFIG1904, 
        Y3A(10) => NET_CC_CONFIG1908, Y3A(11) => 
        NET_CC_CONFIG1912, CC(0) => NET_CC_CONFIG1869, CC(1) => 
        NET_CC_CONFIG1873, CC(2) => NET_CC_CONFIG1877, CC(3) => 
        NET_CC_CONFIG1881, CC(4) => NET_CC_CONFIG1885, CC(5) => 
        NET_CC_CONFIG1889, CC(6) => NET_CC_CONFIG1893, CC(7) => 
        NET_CC_CONFIG1897, CC(8) => NET_CC_CONFIG1901, CC(9) => 
        NET_CC_CONFIG1905, CC(10) => NET_CC_CONFIG1909, CC(11)
         => NET_CC_CONFIG1913);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[2]\, 
        D => \can_top_level_inst/drv_bus[31]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\);
    
    \timestamp_ibuf[22]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(22), Y => 
        \timestamp_ibuf[22]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac20\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac20_1_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac20_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[11]\ : 
        CFG4
      generic map(INIT => x"AC0C")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[11]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        D => \can_top_level_inst/tran_word[19]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_segm_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[3]\);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_ns_1_0_.m15\ : 
        CFG4
      generic map(INIT => x"F0E0")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_set_receiver\, 
        C => 
        \can_top_level_inst/can_core_inst/operation_control_inst/m15_1\, 
        D => 
        \can_top_level_inst/can_core_inst/operation_control_inst_set_transmitter\, 
        Y => 
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_ns[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2[14]\ : 
        CFG4
      generic map(INIT => x"585A")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_190\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_Z[14]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIQLHSC8[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[60]\, B => 
        \can_top_level_inst/tran_word[28]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_27\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_28\, 
        CC => NET_CC_CONFIG1039, P => NET_CC_CONFIG1036, Y3 => 
        NET_CC_CONFIG1037, Y3A => NET_CC_CONFIG1038);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[14]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_1[2]\ : 
        CFG4
      generic map(INIT => x"F400")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_928_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1043\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a4_0[27]\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_966\, 
        B => \can_top_level_inst/can_core_inst/pc_rx_trigger\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1142\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[39]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_select[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select\, 
        B => \sbe_c[0]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/int_ena_proc.un1_int_ena_clear\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[6]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/un1_int_ena_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_16\ : 
        CFG3
      generic map(INIT => x"96")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_16_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[4]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2628\, Y
         => \can_top_level_inst/memory_registers_inst/N_2693\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_6_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_6_FCNET1\, 
        CO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_6_Z\, 
        CC => NET_CC_CONFIG2175, P => NET_CC_CONFIG2172, Y3 => 
        NET_CC_CONFIG2173, Y3A => NET_CC_CONFIG2174);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_axbxc4\ : 
        CFG4
      generic map(INIT => x"E41B")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[4]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[4]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_c4_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un3_data_length_sub_c_axbxc4\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_dlc_0_a4\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_950\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_dlc\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[28]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[28]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[28]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31[5]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[10]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[6]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[6]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\);
    
    
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/g0_0_0\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst_prev_Sample\, B => 
        \can_top_level_inst/stat_bus[15]\, Y => 
        \can_top_level_inst/prescaler_inst/g0_0_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[467]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[23]\ : 
        CFG4
      generic map(INIT => x"EC00")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[23]\, B
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_RNI8TUS1[0]\ : 
        ARI1_CC
      generic map(INIT => x"48800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy_Y[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[0]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[0]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[0]\, 
        CC => NET_CC_CONFIG653, P => NET_CC_CONFIG650, Y3 => 
        NET_CC_CONFIG651, Y3A => NET_CC_CONFIG652);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/txtb_ptr[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m30_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[16]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI261LC_S[17]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[237]\);
    
    \data_out_obuf[9]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[9]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[9]/DOUT\, EOUT => \data_out_obuf[9]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_37[8]\ : 
        CFG4
      generic map(INIT => x"E4A0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1466\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1463\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1467\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[28]\ : 
        SLE
      port map(D => \data_in_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[299]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_1[8]\ : 
        CFG3
      generic map(INIT => x"0D")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => \can_top_level_inst/stat_bus[385]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_1_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[13]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNICUAP[9]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[9]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[9]\, 
        Y => \can_top_level_inst/tran_word[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m31_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_1\, 
        D => \adress_c[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_2[23]\ : 
        CFG4
      generic map(INIT => x"3347")

      port map(A => \can_top_level_inst/drv_bus[166]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => \can_top_level_inst/drv_bus[510]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_Z[23]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[20]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q_RNO[8]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_17[7]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[29]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_46\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[220]\, B => 
        \can_top_level_inst/rec_ident[15]\, C => 
        \can_top_level_inst/drv_bus[253]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_36\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[14]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[17]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[17]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[30]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[30]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[30]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[24]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[19]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[290]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_wmux_2\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_y0_0\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[1]\, 
        C => \can_top_level_inst/N_3899_i\, D => 
        \can_top_level_inst/N_3898_i\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_0_y3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[25]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_8\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[404]\, B => 
        \can_top_level_inst/stat_bus[85]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_3\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_4\, 
        CC => NET_CC_CONFIG1184, P => NET_CC_CONFIG1181, Y3 => 
        NET_CC_CONFIG1182, Y3A => NET_CC_CONFIG1183);
    
    \can_tx_obuf/U_IOPAD\ : IOPAD_TRI
      port map(PAD => can_tx, D => \can_tx_obuf/DOUT\, E => 
        \can_tx_obuf/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_2_2[19]\ : 
        CFG4
      generic map(INIT => x"474F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => \can_top_level_inst/drv_bus[319]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_2_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/un7_index_sat[0]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \adress_c[7]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31[2]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_36[8]\ : 
        CFG3
      generic map(INIT => x"EA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1466_1\, 
        B => \can_top_level_inst/stat_bus[372]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1466\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[28]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram110_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe110\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram110_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[27]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_0_2[15]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \can_top_level_inst/rec_dlc[0]\, B => 
        \can_top_level_inst/rec_dlc[3]\, C => 
        \can_top_level_inst/rec_dlc[2]\, D => 
        \can_top_level_inst/rec_dlc[1]\, Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_0_2_Z[15]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_8\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[8]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_7_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_8_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_8_Z\, 
        CC => NET_CC_CONFIG1246, P => NET_CC_CONFIG1243, Y3 => 
        NET_CC_CONFIG1244, Y3A => NET_CC_CONFIG1245);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[20]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[320]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o4[5]\ : 
        CFG3
      generic map(INIT => x"F2")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_975\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[12]\ : 
        CFG4
      generic map(INIT => x"ACA0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[11]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_388\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[12]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_6[26]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac37_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac21_Z\, 
        C => \can_top_level_inst/drv_bus[330]\, D => 
        \can_top_level_inst/drv_bus[169]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_6_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m18_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram0_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram1_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_9\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[411]\, B => 
        \can_top_level_inst/stat_bus[84]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_2\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_3\, 
        CC => NET_CC_CONFIG183, P => NET_CC_CONFIG180, Y3 => 
        NET_CC_CONFIG181, Y3A => NET_CC_CONFIG182);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[14]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[2]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_0\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => \can_top_level_inst/drv_bus[35]\, B => 
        \can_top_level_inst/drv_bus[41]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => ADLIB_VCC, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_0_Z\, 
        CC => NET_CC_CONFIG1738, P => NET_CC_CONFIG1735, Y3 => 
        NET_CC_CONFIG1736, Y3A => NET_CC_CONFIG1737);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_23\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_23_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[3]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[0]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/int_mask_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.2.int_module_inst/int_mask_load_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_mask[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[13]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[20]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa13\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe13\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_3[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_cZ[3]\ : 
        CFG4
      generic map(INIT => x"FEFF")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_3_Z[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_0_Z[3]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_Z[3]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[3]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_ac0_11_RNI2GBV81\ : 
        ARI1_CC
      generic map(INIT => x"5D22D")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[8]\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c7\, 
        C => \can_top_level_inst/stat_bus[88]\, D => 
        \can_top_level_inst/stat_bus[89]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_7\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_8\, 
        CC => NET_CC_CONFIG119, P => NET_CC_CONFIG116, Y3 => 
        NET_CC_CONFIG117, Y3A => NET_CC_CONFIG118);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[5]\ : 
        CFG3
      generic map(INIT => x"D0")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[3]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_172\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_1[2]\ : 
        CFG4
      generic map(INIT => x"B9A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1204\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1203\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_1_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60[7]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_58\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[106]\, B => 
        \can_top_level_inst/rec_ident[25]\, C => 
        \can_top_level_inst/drv_bus[139]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_26\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[2]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[1]\, 
        C => \can_top_level_inst/txtb_port_b_address[1]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa6\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6_1\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNISMQGE_S[20]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[208]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_1_0_1[14]\ : 
        CFG3
      generic map(INIT => x"27")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[269]\, C => 
        \can_top_level_inst/drv_bus[219]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_1_0_1_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[3]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2461\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_17\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_17_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o2_0[35]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_928_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_939\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[10]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[9]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_2_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[117]\, D => 
        \can_top_level_inst/drv_bus[241]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_2_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[12]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[12]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_1\ : 
        CFG3
      generic map(INIT => x"96")

      port map(A => \can_top_level_inst/drv_bus[56]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[1]\, 
        C => \can_top_level_inst/drv_bus[57]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_1_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[18]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[18]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[176]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[21]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/status_comb_4_1_0[2]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_139\, 
        B => \can_top_level_inst/txtb_state_5[3]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_159\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_190\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/status_comb_4_1_0_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o3[1]\ : 
        CFG3
      generic map(INIT => x"F2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_138\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segm_end_req_capture.2.un13_segm_end_req_capt_ce\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end\, B => 
        \can_top_level_inst/prescaler_inst/exit_segm_req_dbt\, Y
         => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un13_segm_end_req_capt_ce\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_ena_reg_proc.un21_ctrl_signal_upd_i_1\ : 
        CFG4
      generic map(INIT => x"22F2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_944\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_941\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_931\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un21_ctrl_signal_upd_i_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y7[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y5[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[18]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un4_tst_ena\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => \can_top_level_inst/reg_value_r_rep[17]\, B
         => \can_top_level_inst/test_registers_out.tst_dest[19]\, 
        C => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena_2\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m42_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[22]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3\ : 
        CFG4
      generic map(INIT => x"10FF")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_cnst[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1324\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_881\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[8]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[8]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[8]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\);
    
    \adress_ibuf[5]/U_IOPAD\ : IOPAD_IN
      port map(PAD => adress(5), Y => \adress_ibuf[5]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[23]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0_RNI7R3GD1[0]\ : 
        ARI1_CC
      generic map(INIT => x"5B44B")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[0]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_47\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[0]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0\, 
        CC => NET_CC_CONFIG2107, P => NET_CC_CONFIG2104, Y3 => 
        NET_CC_CONFIG2105, Y3A => NET_CC_CONFIG2106);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_2[1]\ : 
        CFG4
      generic map(INIT => x"0010")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[14]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[15]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_1[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_2[1]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[24]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[24]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m17_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[25]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram1__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_346\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_16_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO1164, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => 
        ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_VCC, P(6) => 
        ADLIB_VCC, P(7) => ADLIB_VCC, P(8) => ADLIB_VCC, P(9) => 
        ADLIB_GND, P(10) => NET_CC_CONFIG1165, P(11) => 
        NET_CC_CONFIG1169, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => ADLIB_VCC, Y3(3) => ADLIB_VCC, Y3(4) => 
        ADLIB_VCC, Y3(5) => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7)
         => ADLIB_VCC, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => NET_CC_CONFIG1166, Y3(11) => NET_CC_CONFIG1170, 
        Y3A(0) => ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => 
        ADLIB_VCC, Y3A(3) => ADLIB_VCC, Y3A(4) => ADLIB_VCC, 
        Y3A(5) => ADLIB_VCC, Y3A(6) => ADLIB_VCC, Y3A(7) => 
        ADLIB_VCC, Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, 
        Y3A(10) => NET_CC_CONFIG1167, Y3A(11) => 
        NET_CC_CONFIG1171, CC(0) => nc279, CC(1) => nc137, CC(2)
         => nc64, CC(3) => nc19, CC(4) => nc312, CC(5) => nc70, 
        CC(6) => nc182, CC(7) => nc62, CC(8) => nc337, CC(9) => 
        nc199, CC(10) => NET_CC_CONFIG1168, CC(11) => 
        NET_CC_CONFIG1172);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[28]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[28]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[28]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[28]\ : 
        SLE
      port map(D => \data_in_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[109]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[30]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[30]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_1\, 
        D => \adress_c[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0MKOK_S[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[218]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_unaliged\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1314\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_Z[4]\, 
        D => \can_top_level_inst/sync_edge\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_unaliged_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_0_1_1\ : 
        CFG4
      generic map(INIT => x"0A8E")

      port map(A => \can_top_level_inst/txtb_prorities_3[1]\, B
         => \can_top_level_inst/txtb_prorities_3[0]\, C => 
        \can_top_level_inst/txtb_prorities_2[1]\, D => 
        \can_top_level_inst/txtb_prorities_2[0]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_0_1_1_Z\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_1\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[1]\, B => 
        \can_top_level_inst/drv_bus[82]\, C => 
        \can_top_level_inst/drv_bus[115]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_72\, 
        FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[0]\, 
        CC => NET_CC_CONFIG1767, P => NET_CC_CONFIG1764, Y3 => 
        NET_CC_CONFIG1765, Y3A => NET_CC_CONFIG1766);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m30_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[36]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[5]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_valid_com_proc.un1_frame_valid_com_clear\ : 
        CFG4
      generic map(INIT => x"FFF4")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[7]\, B
         => \can_top_level_inst/tx_arbitrator_inst/N_158\, C => 
        \can_top_level_inst/tx_arbitrator_inst/N_144\, D => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_frame_valid_com_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_0[2]\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \can_top_level_inst/txtb_hw_cmd.err\, B => 
        \can_top_level_inst/txtb_hw_cmd.arbl\, C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[2]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_4[0]\, B
         => \can_top_level_inst/txtb_prorities_5[0]\, C => 
        \can_top_level_inst/drv_bus_475_rep1\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_frame_no_sof_q_RNO\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_931\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_929\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_frame_no_sof_d_i_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_918_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_0_y3[8]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_0_y1[8]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1452\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[406]\, B => 
        \can_top_level_inst/stat_bus[87]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_5_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_6_Z\, 
        CC => NET_CC_CONFIG1553, P => NET_CC_CONFIG1550, Y3 => 
        NET_CC_CONFIG1551, Y3A => NET_CC_CONFIG1552);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[1]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_0[14]\ : 
        CFG4
      generic map(INIT => x"F2D0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_0_1_Z[14]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_694\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_0_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_17\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_17_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[0]\ : 
        CFG4
      generic map(INIT => x"8F88")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_141\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        C => \can_top_level_inst/txtb_allow_bb[2]\, D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[3]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[2]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[3]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1[3]\ : 
        CFG4
      generic map(INIT => x"0F0E")

      port map(A => \can_top_level_inst/stat_bus[89]\, B => 
        \can_top_level_inst/stat_bus[88]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNIQ9QO1_S\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[406]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[19]\ : 
        CFG4
      generic map(INIT => x"0021")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1131\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_806_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[1]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_2\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[407]\, B => 
        \can_top_level_inst/stat_bus[88]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_6\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_7\, 
        CC => NET_CC_CONFIG1196, P => NET_CC_CONFIG1193, Y3 => 
        NET_CC_CONFIG1194, Y3A => NET_CC_CONFIG1195);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_7_2[3]\ : 
        CFG4
      generic map(INIT => x"0ACF")

      port map(A => \can_top_level_inst/int_vector[3]\, B => 
        \can_top_level_inst/stat_bus[381]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_7_2_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[13]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un4_txtb_index_muxed_i[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[1]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[1]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, Y
         => \can_top_level_inst/txtb_index_muxed[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[20]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIJVB31[3]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_2\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIJVB31_S[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_3\, 
        CC => NET_CC_CONFIG1657, P => NET_CC_CONFIG1654, Y3 => 
        NET_CC_CONFIG1655, Y3A => NET_CC_CONFIG1656);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_m2s2\ : 
        CFG4
      generic map(INIT => x"FF01")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[17]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[16]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_disable_1_sm0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_esi_0_a4\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[23]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_esi\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_1\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[11]\, B
         => \can_top_level_inst/can_core_inst/crc_15[10]\, C => 
        \can_top_level_inst/can_core_inst/crc_15[11]\, D => 
        \can_top_level_inst/store_data_word[10]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[4]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[5]\, 
        CC => NET_CC_CONFIG70, P => NET_CC_CONFIG67, Y3 => 
        NET_CC_CONFIG68, Y3A => NET_CC_CONFIG69);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[63]\ : 
        SLE
      port map(D => \timestamp_c[63]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[63]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0PKSI_S[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[215]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_19\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_19_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[281]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_brs_0_a2\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_brs\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[15]\ : 
        SLE
      port map(D => \timestamp_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_10[11]\ : 
        CFG4
      generic map(INIT => x"F2F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1554_1\, 
        D => \can_top_level_inst/drv_bus[45]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1554\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[1]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_frame_no_sof_d_i_o4_0\ : 
        CFG3
      generic map(INIT => x"EF")

      port map(A => \can_top_level_inst/drv_bus[470]\, B => 
        \can_top_level_inst/drv_bus[513]\, C => 
        \can_top_level_inst/tran_frame_valid\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_931\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[18]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[28]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4FU8D[18]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[206]\, D => 
        \can_top_level_inst/stat_bus[238]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_17\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4FU8D_S[18]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_18\, 
        CC => NET_CC_CONFIG828, P => NET_CC_CONFIG825, Y3 => 
        NET_CC_CONFIG826, Y3A => NET_CC_CONFIG827);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/write_raw_intent_i\ : 
        CFG4
      generic map(INIT => x"FFF4")

      port map(A => \can_top_level_inst/store_data_f\, B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/write_raw_intent_i_0_Z\, 
        D => \can_top_level_inst/rx_buffer_inst/stored_ts\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_57\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[3]\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        C => \can_top_level_inst/N_4606\, D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_168\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_1\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => \adress_c[9]\, B => \adress_c[10]\, C => 
        scs_c, D => swr_c, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_1_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[29]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[29]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[29]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[2]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7_2_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7_Z\);
    
    \timestamp_ibuf[49]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(49), Y => 
        \timestamp_ibuf[49]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4PNCG_S[23]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[211]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\);
    
    \can_top_level_inst/rx_buffer_inst/rx_read_buff[9]\ : CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/rx_buffer_inst_RAM_data_out[9]\, 
        Y => \can_top_level_inst/rx_read_buff[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[10]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_10\ : 
        CFG4
      generic map(INIT => x"A8AA")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_nbt_tq_active\, 
        B => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_10_N_2L1_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/un8_exit_segm_regular_tseg1\, 
        D => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_1\, 
        Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g1_0_0_0\);
    
    
        \can_top_level_inst/int_manager_inst/int_input_active_18_1[11]\ : 
        CFG4
      generic map(INIT => x"00F7")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        D => 
        \can_top_level_inst/int_manager_inst/int_input_active_0_Z[11]\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_18_1_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a2[2]\ : 
        CFG4
      generic map(INIT => x"1333")

      port map(A => 
        \can_top_level_inst/txtb_parity_error_valid_1[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_m1_e_1\, 
        D => \can_top_level_inst/txtb_parity_check_valid\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_175\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_s_7\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_6_Z\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_s_7_S\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG2247, P => 
        NET_CC_CONFIG2244, Y3 => NET_CC_CONFIG2245, Y3A => 
        NET_CC_CONFIG2246);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[21]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[164]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_ac0_7_RNI0M8BS\ : 
        ARI1_CC
      generic map(INIT => x"5D22D")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[6]\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c5\, 
        C => \can_top_level_inst/stat_bus[86]\, D => 
        \can_top_level_inst/stat_bus[87]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_5\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_6\, 
        CC => NET_CC_CONFIG111, P => NET_CC_CONFIG108, Y3 => 
        NET_CC_CONFIG109, Y3A => NET_CC_CONFIG110);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa11\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe11\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI61GC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[17]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_2_3\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_2_0_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1246\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1215_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_2_3_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_26[4]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/rx_write_pointer[4]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1304\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI4ETA7[15]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[15]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[15]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_14\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_15\, 
        CC => NET_CC_CONFIG1337, P => NET_CC_CONFIG1334, Y3 => 
        NET_CC_CONFIG1335, Y3A => NET_CC_CONFIG1336);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m35_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/is_ack_field_i_a2\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_2\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[15]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1239\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[27]\);
    
    AFLSDF_INV_12 : INV_BA
      port map(A => \AFLSDF_INV_12\, Y => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.unlock_sn_m5\ : 
        CFG4
      generic map(INIT => x"0010")

      port map(A => \can_top_level_inst/stat_bus[378]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/set_idle_i_5_sqmuxa\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_717_i\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/unlock_sn_N_8_mux\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_38_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1483\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1498\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_38_1_0_y0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[13]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram111_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe111\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram111_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[23]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[23]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI5D3FU7[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[59]\, B => 
        \can_top_level_inst/tran_word[27]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_26\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_27\, 
        CC => NET_CC_CONFIG1035, P => NET_CC_CONFIG1032, Y3 => 
        NET_CC_CONFIG1033, Y3A => NET_CC_CONFIG1034);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI6SDS\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNICH841[2]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[1]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[2]\, 
        CC => NET_CC_CONFIG1613, P => NET_CC_CONFIG1610, Y3 => 
        NET_CC_CONFIG1611, Y3A => NET_CC_CONFIG1612);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[6]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[6]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_25\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[296]\, B => 
        \can_top_level_inst/rec_ident[25]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_24\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_25\, 
        CC => NET_CC_CONFIG1933, P => NET_CC_CONFIG1930, Y3 => 
        NET_CC_CONFIG1931, Y3A => NET_CC_CONFIG1932);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un7_txtb_parity_error_valid_i\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/N_3903\, 
        B => \can_top_level_inst/txtb_parity_error_valid_1[0]\, C
         => \can_top_level_inst/txtb_parity_check_valid\, D => 
        \can_top_level_inst/txtb_parity_error_valid_2[6]\, Y => 
        \can_top_level_inst/txtb_parity_error_valid[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select[2]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_0_0\, 
        D => \adress_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO\ : 
        ARI1_CC
      generic map(INIT => x"69900")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/crc_21[20]\, C => 
        \can_top_level_inst/store_data_word[20]\, D => ADLIB_GND, 
        FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[9]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_FCNET1\, 
        CC => NET_CC_CONFIG42, P => NET_CC_CONFIG39, Y3 => 
        NET_CC_CONFIG40, Y3A => NET_CC_CONFIG41);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_8_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_8_FCNET1\, 
        CO => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        CC => NET_CC_CONFIG2414, P => NET_CC_CONFIG2411, Y3 => 
        NET_CC_CONFIG2412, Y3A => NET_CC_CONFIG2413);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[193]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram25__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_340\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe25\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[3]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_25\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[25]\, B => 
        \can_top_level_inst/drv_bus[325]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_24\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_25\, 
        CC => NET_CC_CONFIG629, P => NET_CC_CONFIG626, Y3 => 
        NET_CC_CONFIG627, Y3A => NET_CC_CONFIG628);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/data_selector[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_1[2]\ : 
        CFG4
      generic map(INIT => x"4657")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/int_ena[2]\, D => 
        \can_top_level_inst/drv_bus[471]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[17]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[17]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[17]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_28[25]\ : 
        CFG3
      generic map(INIT => x"C8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[106]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1114\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_25\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[13]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_18_Z\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_8_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_25_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[45]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[27]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[27]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[27]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz[3]\ : 
        CFG4
      generic map(INIT => x"FF04")

      port map(A => \can_top_level_inst/N_4606\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns[1]\ : 
        CFG4
      generic map(INIT => x"FFF1")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_114\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_112\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_130\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_132\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[29]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[27]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0RIT8_S[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[231]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[3]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_y0_0[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_34_1[19]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/stat_bus[207]\, B => 
        \can_top_level_inst/drv_bus[378]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_892_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[49]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_frame_no_sof_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_918_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/tx_frame_no_sof\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[14]\ : 
        SLE
      port map(D => \timestamp_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[27]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2485\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m43\ : 
        CFG4
      generic map(INIT => x"80A2")

      port map(A => \sbe_c[0]\, B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_42_0\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_37_0\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/masked_data[7]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/int_mask_load\ : 
        CFG4
      generic map(INIT => x"A800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        D => \data_in_c[9]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/int_mask_load_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[5]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[19]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_12_1_0\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un8_exit_segm_regular_tseg1_0\, 
        B => 
        \can_top_level_inst/prescaler_inst/un5_exit_ph2_immediate_0\, 
        Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_12_1_0_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[14]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_2_RNO[26]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \adress_c[7]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => \timestamp_c[26]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[1210]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_8[31]\ : 
        CFG4
      generic map(INIT => x"FFBF")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_18_0_o4_0_3\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        C => \can_top_level_inst/drv_bus[477]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_12_Z[31]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_8\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_21[16]\ : 
        CFG4
      generic map(INIT => x"F4F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_765_2\, 
        D => \can_top_level_inst/drv_bus[221]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_765\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[31]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[31]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[31]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa9\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8_2\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa9\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\);
    
    \res_n_out_obuf/U_IOPAD\ : IOPAD_TRI
      port map(PAD => res_n_out, D => \res_n_out_obuf/DOUT\, E
         => \res_n_out_obuf/EOUT\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test.tprm[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[1]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_frame_test.tprm[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[21]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[21]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[21]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm_0[1]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[1]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_pos_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/err_detected\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[101]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[5]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIBGF932[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[46]\, B => 
        \can_top_level_inst/tran_word[14]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_13\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_14\, 
        CC => NET_CC_CONFIG983, P => NET_CC_CONFIG980, Y3 => 
        NET_CC_CONFIG981, Y3A => NET_CC_CONFIG982);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI8UQ54[5]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[278]\, C => 
        \can_top_level_inst/stat_bus[262]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_4\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI8UQ54_S[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_5\, 
        CC => NET_CC_CONFIG2588, P => NET_CC_CONFIG2585, Y3 => 
        NET_CC_CONFIG2586, Y3A => NET_CC_CONFIG2587);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[2]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0444")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_968\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a2_0_1_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        D => \can_top_level_inst/is_bus_off\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1228\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_17\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_17_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[2]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[0]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/test_registers_out.tst_dest[2]\, D
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_5\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[10]\, Y => 
        \can_top_level_inst/prescaler_inst_un2_tq_edge\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[15]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[15]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_0[0]\ : 
        CFG4
      generic map(INIT => x"00DF")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        C => \can_top_level_inst/is_bus_off\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1193\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_0_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_2[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[5]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[5]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_err_frm_req_3_0_o2\ : 
        CFG3
      generic map(INIT => x"1D")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_952\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_930\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_937\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_dlc[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_dlc\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_dlc[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[27]\ : 
        SLE
      port map(D => \data_in_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[141]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o3_0[1]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_192\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_147\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac20_1\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac20_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_RNO_0[9]\ : 
        ARI1_CC
      generic map(INIT => x"4EE00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, C => 
        \can_top_level_inst/prescaler_inst/is_tseg2\, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_8\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[9]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG1449, P => 
        NET_CC_CONFIG1446, Y3 => NET_CC_CONFIG1447, Y3A => 
        NET_CC_CONFIG1448);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[30]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2556\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1[4]\ : 
        CFG4
      generic map(INIT => x"0F0E")

      port map(A => \can_top_level_inst/stat_bus[89]\, B => 
        \can_top_level_inst/stat_bus[88]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNIRU072_S\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[4]\);
    
    \can_top_level_inst/rx_buffer_inst/rx_read_buff[15]\ : CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/rx_buffer_inst_RAM_data_out[15]\, 
        Y => \can_top_level_inst/rx_read_buff[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_15\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[286]\, B => 
        \can_top_level_inst/rec_ident[15]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_14\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_15\, 
        CC => NET_CC_CONFIG1893, P => NET_CC_CONFIG1890, Y3 => 
        NET_CC_CONFIG1891, Y3A => NET_CC_CONFIG1892);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[30]\);
    
    \timestamp_ibuf[21]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[21]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[21]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[12]\);
    
    \timestamp_ibuf[17]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(17), Y => 
        \timestamp_ibuf[17]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[5]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_33\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[11]\, B => 
        \timestamp_c[42]\, C => \timestamp_c[43]\, D => 
        \can_top_level_inst/tran_word[10]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[4]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[5]\, 
        CC => NET_CC_CONFIG2273, P => NET_CC_CONFIG2270, Y3 => 
        NET_CC_CONFIG2271, Y3A => NET_CC_CONFIG2272);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[28]\ : 
        SLE
      port map(D => \data_in_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI8LHP[1]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_0\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI8LHP_S[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_1\, 
        CC => NET_CC_CONFIG1649, P => NET_CC_CONFIG1646, Y3 => 
        NET_CC_CONFIG1647, Y3A => NET_CC_CONFIG1648);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\);
    
    
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_19\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => \data_in_c[31]\, B => \data_in_c[30]\, C => 
        \data_in_c[29]\, D => \data_in_c[28]\, Y => 
        \can_top_level_inst/txtb_parity_true_gen.txtb_port_a_parity_calculator_inst/parity_19_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31[0]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[8]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[40]\, 
        C => \can_top_level_inst/rec_ident[8]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2253\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test.tprm[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[0]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_frame_test.tprm[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_834_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lto3_1\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lt4_1\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q_RNO[0]\ : 
        CFG3
      generic map(INIT => x"2E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[11]\, B
         => \can_top_level_inst/rx_mof\, C => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[0]\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_3924_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_2[30]\ : 
        CFG4
      generic map(INIT => x"8222")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_1226\, 
        C => \can_top_level_inst/tran_frame_test.fcrc\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_83_mux\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[2]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[27]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[14]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[21]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/int_ena_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/un1_int_ena_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_ena[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_0_y0[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1145\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1146\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_0_y1[0]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q_RNO[7]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_17[6]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_3[31]\ : 
        CFG4
      generic map(INIT => x"F099")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_8\, 
        B => \can_top_level_inst/stat_bus[302]\, C => 
        \can_top_level_inst/tran_word[7]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_13\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa20_1\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \adress_c[4]\, B => \adress_c[6]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_21\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_21_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[22]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[22]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[17]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_127_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_wmux_10_Y[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_wmux_10_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_127_1_0_y0[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[22]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23[22]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[22]\);
    
    \timestamp_ibuf[51]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[51]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[51]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[12]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_reg_rst_inst/rx_shift_res_reg_inst/reg_q_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => \can_top_level_inst/drv_bus[473]\, Y => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_reg_rst_inst/rx_shift_res_reg_inst/drv_bus_i[473]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa12\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe12\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_o4\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[23]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[27]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_934\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\);
    
    \data_out_obuf[17]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(17), D => \data_out_obuf[17]/DOUT\, 
        E => \data_out_obuf[17]/EOUT\);
    
    \can_top_level_inst/rx_buffer_inst/rx_read_buff[10]\ : CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/rx_buffer_inst_RAM_data_out[10]\, 
        Y => \can_top_level_inst/rx_read_buff[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0[1]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[1]\, 
        C => \can_top_level_inst/drv_bus[57]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_50\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_set_receiver_i_o4_RNIR7SE\ : 
        CFG3
      generic map(INIT => x"8C")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_961\, 
        Y => \can_top_level_inst/sof_pulse\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[468]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_2_0[8]\ : 
        CFG4
      generic map(INIT => x"F588")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1441\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1442\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_1_0_Z[8]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_15_2_0_Z[8]\);
    
    \timestamp_ibuf[4]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[4]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[4]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[2]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m97\ : 
        CFG3
      generic map(INIT => x"27")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[307]\, C => 
        \can_top_level_inst/drv_bus[183]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i71_mux\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/int_stat_proc.un1_int_status_clear\ : 
        CFG4
      generic map(INIT => x"F8F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_stat_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => \sbe_c[1]\, C => 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/un2_int_status_set\, 
        D => \data_in_c[10]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/un1_int_status_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[6]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_3_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[30]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[30]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_15[27]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_10_Z[27]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_11_Z[27]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_9_Z[27]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_12_Z[27]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_15_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_6[25]\ : 
        CFG4
      generic map(INIT => x"F4F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1092_1\, 
        D => \can_top_level_inst/drv_bus[6]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1092\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[20]\ : 
        CFG4
      generic map(INIT => x"2021")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1004\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_804_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o3_0[1]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_192\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_147\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[21]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[4]\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_161_2\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_170_1\, 
        D => \can_top_level_inst/txtb_parity_error_valid[1]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_170\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram74_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe74\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram74_0\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[24]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/command_reg_comp/wr_select_cZ[1]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/access_active_1[0]\, 
        C => \sbe_c[1]\, Y => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_command_reg_comp_wr_select[1]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_5\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[81]\, B => 
        \can_top_level_inst/rec_ident[0]\, C => 
        \can_top_level_inst/drv_bus[114]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_72\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/txtb_sw_cmd.set_abt_rep1\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/tx_command_rep1[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_sw_cmd.set_abt_rep1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un4_compl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/CO0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI4BIH9[23]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[23]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[23]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_22\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_23\, 
        CC => NET_CC_CONFIG1369, P => NET_CC_CONFIG1366, Y3 => 
        NET_CC_CONFIG1367, Y3A => NET_CC_CONFIG1368);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_FCNET1\, 
        CO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_8\, 
        CC => NET_CC_CONFIG207, P => NET_CC_CONFIG204, Y3 => 
        NET_CC_CONFIG205, Y3A => NET_CC_CONFIG206);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[13]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[3]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[21]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[21]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[21]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[21]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0[2]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[2]\, 
        C => \can_top_level_inst/drv_bus[53]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_49\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/txtb_sw_cmd.set_ety\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/tx_command[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_sw_cmd.set_ety\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_23\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_23_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[19]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[19]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[23]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa9\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe9\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_axbxc5\ : 
        CFG4
      generic map(INIT => x"E41B")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[5]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[5]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_c5_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un3_data_length_sub_c_axbxc5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_1[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/shift_reg_in_2[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un9_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[19]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_0\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[271]\, B => 
        \can_top_level_inst/rec_ident[0]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_0\, 
        CC => NET_CC_CONFIG1833, P => NET_CC_CONFIG1830, Y3 => 
        NET_CC_CONFIG1831, Y3A => NET_CC_CONFIG1832);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[20]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[20]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[20]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[28]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_34\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[170]\, B => 
        \can_top_level_inst/rec_ident[27]\, C => 
        \can_top_level_inst/drv_bus[203]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_46\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[13]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[11]\ : 
        CFG4
      generic map(INIT => x"F088")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/frame_form_w_9[11]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        C => \can_top_level_inst/rx_buffer_inst/N_2289\, D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[2]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[1]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[6]\ : 
        CFG4
      generic map(INIT => x"0E00")

      port map(A => \can_top_level_inst/txtb_hw_cmd.arbl\, B => 
        \can_top_level_inst/txtb_hw_cmd.err\, C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_i_o2_0[16]\ : 
        CFG3
      generic map(INIT => x"BF")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1192\, 
        B => \can_top_level_inst/tran_frame_test.fcrc\, C => 
        \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1194\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram34__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_350\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe34\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe6\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[12]\ : 
        SLE
      port map(D => \timestamp_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[12]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2257\, B
         => \can_top_level_inst/rx_buffer_inst/N_4818\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2290\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[30]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test_dbl_buf.tprm[2]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[10]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/load_ffmt_w_addr\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[13]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_nbt_Z[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_6_S\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_nbt[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30[11]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[11]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_0_1[19]\ : 
        CFG4
      generic map(INIT => x"350F")

      port map(A => \can_top_level_inst/drv_bus[0]\, B => 
        \can_top_level_inst/rx_read_pointer[3]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_0_1_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[22]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[22]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[5]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m25_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[29]\ : 
        CFG4
      generic map(INIT => x"EC00")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[29]\, B
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[24]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[13]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa4_2\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \adress_c[3]\, B => \adress_c[5]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/wa4_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_write_pointer_ts_ce_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1[2]\ : 
        CFG4
      generic map(INIT => x"4C5F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_147\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_12\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[402]\, B => 
        \can_top_level_inst/stat_bus[83]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_1\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_2\, 
        CC => NET_CC_CONFIG1176, P => NET_CC_CONFIG1173, Y3 => 
        NET_CC_CONFIG1174, Y3A => NET_CC_CONFIG1175);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[17]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[17]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[17]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_2_RNIHKQU1\ : 
        ARI1_CC
      generic map(INIT => x"5E44E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_sn_N_2_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_2_S\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_Z[0]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_1\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2374\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_2\, 
        CC => NET_CC_CONFIG1961, P => NET_CC_CONFIG1958, Y3 => 
        NET_CC_CONFIG1959, Y3A => NET_CC_CONFIG1960);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_RNO_4[28]\ : 
        CFG4
      generic map(INIT => x"FFBF")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[3]\, 
        B => \can_top_level_inst/tran_frame_test.fcrc\, C => 
        \can_top_level_inst/drv_bus[477]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_o4_3_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_19_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[10]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[10]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_write_pointer_ts_ce_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[26]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[107]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[25]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[25]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[25]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[21]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[21]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[21]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/int_stat_proc.un1_int_status_clear\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_stat_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => \sbe_c[0]\, C => \data_in_c[0]\, D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/un2_int_status_set\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/un1_int_status_clear\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_8_RNILCPFO5\ : 
        ARI1_CC
      generic map(INIT => x"4D2F0")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[6]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_5\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_6\, 
        CC => NET_CC_CONFIG1151, P => NET_CC_CONFIG1148, Y3 => 
        NET_CC_CONFIG1149, Y3A => NET_CC_CONFIG1150);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_0[4]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[3]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[311]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[26]\);
    
    \data_out_obuf[22]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(22), D => \data_out_obuf[22]/DOUT\, 
        E => \data_out_obuf[22]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\);
    
    \can_top_level_inst/rx_buffer_inst/rx_parity_error\ : SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/un5_drv_read_start\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un3_drv_clr_rxpe\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rx_parity_error\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1[26]\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[26]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_2[3]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[18]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un17_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_RNIJ6NH[0]\ : 
        ARI1_CC
      generic map(INIT => x"5559A")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_i[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        FCI => ADLIB_GND, S => OPEN, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_RNIJ6NH_Y[0]\, 
        FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_0\, 
        CC => NET_CC_CONFIG1953, P => NET_CC_CONFIG1950, Y3 => 
        NET_CC_CONFIG1951, Y3A => NET_CC_CONFIG1952);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_34_1_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_34_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/stat_bus[237]\, D => 
        \can_top_level_inst/is_overload\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_816\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_err_frm_req_3_0\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1103_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_err_frm_req_3\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOLJ46[7]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[195]\, D => 
        \can_top_level_inst/stat_bus[227]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_6\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOLJ46_S[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_7\, 
        CC => NET_CC_CONFIG784, P => NET_CC_CONFIG781, Y3 => 
        NET_CC_CONFIG782, Y3A => NET_CC_CONFIG783);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[2]\ : 
        CFG4
      generic map(INIT => x"0E0A")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_147\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_ac0_11_RNI98SFB1\ : 
        CFG3
      generic map(INIT => x"E0")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_one\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_eight\, 
        C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_26\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un3_segment_end_i_0_1_N_2L1_RNIDL3B\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un3_segment_end_i_0_1_N_2L1_Z\, 
        B => \can_top_level_inst/prescaler_inst/is_tseg2\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/N_4\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac1\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \adress_c[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[29]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[29]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[29]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[15]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_2[21]\ : 
        CFG4
      generic map(INIT => x"A820")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1462_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a2_0[1]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_182\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_3_sqmuxa_i\ : 
        CFG3
      generic map(INIT => x"FD")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_Z[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1300\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_886_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[26]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[26]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[15]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[1]\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => \can_top_level_inst/txtb_hw_cmd.err\, B => 
        \can_top_level_inst/txtb_hw_cmd.arbl\, C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/read_pointer_i[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_read_pointer[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac25\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac1_0_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac25_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[8]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[1]\, 
        D => \can_top_level_inst/drv_bus[30]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un4_use_basic_segm_length_i\, 
        B => \can_top_level_inst/prescaler_inst/start_edge\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un8_use_basic_segm_length_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[29]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[29]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[2]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_0[29]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[29]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_15\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_18\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[13]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[23]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[294]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram116__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_349\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe116\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[18]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/int_ena_proc.un1_int_ena_clear\ : 
        CFG4
      generic map(INIT => x"A800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        D => \data_in_c[9]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/un1_int_ena_clear\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[420]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_25[15]\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[315]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_731\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa13\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5_1\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa13\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe4\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[23]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[16]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_axbxc2\ : 
        CFG3
      generic map(INIT => x"C9")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[2]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un3_data_length_sub_c_axbxc2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[28]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram44_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram45_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1[37]\ : 
        CFG4
      generic map(INIT => x"BBAB")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_arb_1_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[37]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNICPQ82[3]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[3]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_2\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_3\, 
        CC => NET_CC_CONFIG1289, P => NET_CC_CONFIG1286, Y3 => 
        NET_CC_CONFIG1287, Y3A => NET_CC_CONFIG1288);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_11_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[6]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, D => 
        \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\);
    
    \timestamp_ibuf[36]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(36), Y => 
        \timestamp_ibuf[36]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[12]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[15]\);
    
    \data_out_obuf[30]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(30), D => \data_out_obuf[30]/DOUT\, 
        E => \data_out_obuf[30]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_5[16]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/drv_bus[32]\, B => 
        \can_top_level_inst/rx_read_pointer[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_749\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_2[1]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[16]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un17_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[24]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2550\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2482\, Y
         => \can_top_level_inst/memory_registers_inst/N_2680\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram104__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_344\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe104\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[4]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[3]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_63\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[21]\, B => 
        \timestamp_c[52]\, C => \timestamp_c[53]\, D => 
        \can_top_level_inst/tran_word[20]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[9]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[10]\, 
        CC => NET_CC_CONFIG2293, P => NET_CC_CONFIG2290, Y3 => 
        NET_CC_CONFIG2291, Y3A => NET_CC_CONFIG2292);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_823\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_826\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_0_y0[18]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4FU8D_S[18]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[238]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[4]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2530\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2462\, Y
         => \can_top_level_inst/memory_registers_inst/N_2660\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_2[22]\ : 
        CFG4
      generic map(INIT => x"BBC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_977\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_976\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_1_Z[22]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_2_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[24]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[24]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[6]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac29\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac21_1_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac29_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6_1\, 
        B => \sbe_c[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_2_0\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_RNITOLC[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_GND, CO => OPEN, P(0) => 
        NET_CC_CONFIG2022, P(1) => NET_CC_CONFIG2026, P(2) => 
        NET_CC_CONFIG2030, P(3) => NET_CC_CONFIG2034, P(4) => 
        NET_CC_CONFIG2038, P(5) => NET_CC_CONFIG2042, P(6) => 
        NET_CC_CONFIG2046, P(7) => NET_CC_CONFIG2050, P(8) => 
        NET_CC_CONFIG2054, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, 
        P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG2023, Y3(1) => 
        NET_CC_CONFIG2027, Y3(2) => NET_CC_CONFIG2031, Y3(3) => 
        NET_CC_CONFIG2035, Y3(4) => NET_CC_CONFIG2039, Y3(5) => 
        NET_CC_CONFIG2043, Y3(6) => NET_CC_CONFIG2047, Y3(7) => 
        NET_CC_CONFIG2051, Y3(8) => NET_CC_CONFIG2055, Y3(9) => 
        ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, 
        Y3A(0) => NET_CC_CONFIG2024, Y3A(1) => NET_CC_CONFIG2028, 
        Y3A(2) => NET_CC_CONFIG2032, Y3A(3) => NET_CC_CONFIG2036, 
        Y3A(4) => NET_CC_CONFIG2040, Y3A(5) => NET_CC_CONFIG2044, 
        Y3A(6) => NET_CC_CONFIG2048, Y3A(7) => NET_CC_CONFIG2052, 
        Y3A(8) => NET_CC_CONFIG2056, Y3A(9) => ADLIB_VCC, Y3A(10)
         => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG2025, CC(1) => NET_CC_CONFIG2029, CC(2) => 
        NET_CC_CONFIG2033, CC(3) => NET_CC_CONFIG2037, CC(4) => 
        NET_CC_CONFIG2041, CC(5) => NET_CC_CONFIG2045, CC(6) => 
        NET_CC_CONFIG2049, CC(7) => NET_CC_CONFIG2053, CC(8) => 
        NET_CC_CONFIG2057, CC(9) => nc80, CC(10) => nc130, CC(11)
         => nc287);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa17\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe17\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[19]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI6OAP[6]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[6]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[6]\, 
        Y => \can_top_level_inst/tran_word[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_len_can_fd_1_sqmuxa_RNIRPME_0\ : 
        CFG4
      generic map(INIT => x"2800")

      port map(A => \can_top_level_inst/store_data_word[1]\, B
         => \can_top_level_inst/store_data_word[0]\, C => 
        \can_top_level_inst/can_core_inst/bds_data_out\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst_comb/data_len_can_fd_1_sqmuxa_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[5]\);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_s3_i\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c5_RNIEUN32\ : 
        CFG4
      generic map(INIT => x"7DD7")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/N_3910\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_a_4_c5_Z\, 
        C => \can_top_level_inst/drv_bus[5]\, D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[5]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_NE_4\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_0_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO524, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_GND, P(3) => 
        NET_CC_CONFIG526, P(4) => NET_CC_CONFIG530, P(5) => 
        NET_CC_CONFIG534, P(6) => NET_CC_CONFIG538, P(7) => 
        NET_CC_CONFIG542, P(8) => NET_CC_CONFIG546, P(9) => 
        NET_CC_CONFIG550, P(10) => NET_CC_CONFIG554, P(11) => 
        NET_CC_CONFIG558, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => ADLIB_VCC, Y3(3) => NET_CC_CONFIG527, Y3(4) => 
        NET_CC_CONFIG531, Y3(5) => NET_CC_CONFIG535, Y3(6) => 
        NET_CC_CONFIG539, Y3(7) => NET_CC_CONFIG543, Y3(8) => 
        NET_CC_CONFIG547, Y3(9) => NET_CC_CONFIG551, Y3(10) => 
        NET_CC_CONFIG555, Y3(11) => NET_CC_CONFIG559, Y3A(0) => 
        ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => ADLIB_VCC, 
        Y3A(3) => NET_CC_CONFIG528, Y3A(4) => NET_CC_CONFIG532, 
        Y3A(5) => NET_CC_CONFIG536, Y3A(6) => NET_CC_CONFIG540, 
        Y3A(7) => NET_CC_CONFIG544, Y3A(8) => NET_CC_CONFIG548, 
        Y3A(9) => NET_CC_CONFIG552, Y3A(10) => NET_CC_CONFIG556, 
        Y3A(11) => NET_CC_CONFIG560, CC(0) => nc98, CC(1) => 
        nc293, CC(2) => nc249, CC(3) => NET_CC_CONFIG529, CC(4)
         => NET_CC_CONFIG533, CC(5) => NET_CC_CONFIG537, CC(6)
         => NET_CC_CONFIG541, CC(7) => NET_CC_CONFIG545, CC(8)
         => NET_CC_CONFIG549, CC(9) => NET_CC_CONFIG553, CC(10)
         => NET_CC_CONFIG557, CC(11) => NET_CC_CONFIG561);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv[4]\ : 
        CFG4
      generic map(INIT => x"EEFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_19_m_Z[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_0_Z[4]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_cnst[4]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_913_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_3\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[406]\, B => 
        \can_top_level_inst/stat_bus[77]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_5\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_6\, 
        CC => NET_CC_CONFIG2009, P => NET_CC_CONFIG2006, Y3 => 
        NET_CC_CONFIG2007, Y3A => NET_CC_CONFIG2008);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_tx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[77]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[24]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec[4]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_eight\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_238\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[189]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[22]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_0\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2_0_0[8]\, 
        C => \adress_c[5]\, D => \adress_c[4]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram127__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_347\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe127\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[27]\ : 
        SLE
      port map(D => \data_in_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[56]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_23\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_23_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => swr_c, B => \sbe_c[1]\, C => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_0\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[27]\);
    
    \timestamp_ibuf[9]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[9]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[9]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[17]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20_1\, 
        B => \sbe_c[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2[6]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/NN_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_RNO[28]\ : 
        CFG4
      generic map(INIT => x"FFA2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_1\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_22\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/tx_sr_pload_val[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_1[3]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[10]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un9_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_28_1_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_28_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[92]\, D => 
        \can_top_level_inst/stat_bus[5]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1572\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m44_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[5]\ : 
        CFG3
      generic map(INIT => x"D0")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[2]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_163\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[8]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[8]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIAMOOF_S[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[210]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        D => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/N_4214\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa_0_a2_2\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_189\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_186\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[6]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[17]\);
    
    \data_out_obuf[20]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(20), D => \data_out_obuf[20]/DOUT\, 
        E => \data_out_obuf[20]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_91_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[14]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram29__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_351\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe29\);
    
    \data_in_ibuf[15]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(15), Y => \data_in_ibuf[15]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_6[31]\ : 
        CFG3
      generic map(INIT => x"BA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_1225_1\, 
        B => \can_top_level_inst/can_core_inst/crc_21[20]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_1225\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1051\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1054\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_0_y0[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_0_sqmuxa_0_a2\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_0_sqmuxa\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/store_ts_l_w_0_a2\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/tx_arb_parity_check_valid_1\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[6]\, 
        Y => \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[45]\ : 
        SLE
      port map(D => \timestamp_c[45]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[45]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[11]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[43]\, 
        C => \can_top_level_inst/rec_ident[11]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2256\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_13_2[6]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/drv_bus[406]\, B => 
        \can_top_level_inst/rx_read_buff[6]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1367_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[12]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m43_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[18]\);
    
    \can_top_level_inst/int_manager_inst/int_input_active_1[11]\ : 
        CFG4
      generic map(INIT => x"F0F8")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_172\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_1_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1[1]\ : 
        CFG4
      generic map(INIT => x"FF20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_0\, 
        B => \can_top_level_inst/txtb_parity_error_valid[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select[3]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_0[2]\, 
        B => \sbe_c[3]\, C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_1_0_Z[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_m0[8]\ : 
        CFG4
      generic map(INIT => x"FF37")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_eight\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_i\, 
        C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_one\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[8]\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_m0_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[27]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_87\ : 
        ARI1_CC
      generic map(INIT => x"6D700")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/drv_bus[171]\, C => 
        \can_top_level_inst/drv_bus[204]\, D => 
        \can_top_level_inst/rec_ident[28]\, FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[13]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_87_FCNET1\, 
        CC => NET_CC_CONFIG746, P => NET_CC_CONFIG743, Y3 => 
        NET_CC_CONFIG744, Y3A => NET_CC_CONFIG745);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[9]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1066\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1069\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_31_2_y0_0[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2[19]\ : 
        CFG3
      generic map(INIT => x"0B")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_1[19]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[12]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_write_pointer_ts_ce_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[16]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8[2]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2374\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_3\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[5]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_3_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[7]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[7]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_segm_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[213]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_38_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_987\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1002\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_38_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2_1[1]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_148\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\);
    
    \I_1/U0_RGB1_RGB4\ : RGB
      port map(A => \I_1/U0_Y\, EN => ADLIB_VCC, Y => 
        \AFLSDF_INV_13\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[27]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram117_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe117\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram117_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_2_2[2]\ : 
        CFG3
      generic map(INIT => x"35")

      port map(A => \can_top_level_inst/drv_bus[37]\, B => 
        \can_top_level_inst/stat_bus[83]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_2_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[27]\ : 
        CFG4
      generic map(INIT => x"3210")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1035\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[27]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_790_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_4[20]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => \can_top_level_inst/drv_bus[469]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => \can_top_level_inst/drv_bus[163]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_900\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[9]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry[2]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[1]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[2]\, 
        CC => NET_CC_CONFIG1461, P => NET_CC_CONFIG1458, Y3 => 
        NET_CC_CONFIG1459, Y3A => NET_CC_CONFIG1460);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_rx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[82]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[17]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[402]\, B => 
        \can_top_level_inst/stat_bus[73]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_1_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_2_Z\, 
        CC => NET_CC_CONFIG220, P => NET_CC_CONFIG217, Y3 => 
        NET_CC_CONFIG218, Y3A => NET_CC_CONFIG219);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[14]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2540\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2472\, Y
         => \can_top_level_inst/memory_registers_inst/N_2670\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_7\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[5]\, B
         => \can_top_level_inst/can_core_inst/crc_21[4]\, C => 
        \can_top_level_inst/can_core_inst/crc_21[5]\, D => 
        \can_top_level_inst/store_data_word[4]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[1]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[2]\, 
        CC => NET_CC_CONFIG10, P => NET_CC_CONFIG7, Y3 => 
        NET_CC_CONFIG8, Y3A => NET_CC_CONFIG9);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram108_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe108\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram108_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45[7]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[23]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"553AC")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[6]\, 
        B => \can_top_level_inst/bus_sampling_inst/ssp_delay[6]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[6]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_5_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_6_Z\, 
        CC => NET_CC_CONFIG422, P => NET_CC_CONFIG419, Y3 => 
        NET_CC_CONFIG420, Y3A => NET_CC_CONFIG421);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[14]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2540\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[3]\);
    
    \timestamp_ibuf[12]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[12]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[12]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_RNI73352[5]\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/data_selector[0]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/memory_write_data_sn_N_3\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[10]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2468\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[29]\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_write_address[5]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[5]\, 
        C => \can_top_level_inst/rx_buffer_inst/N_48\, Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_15\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[400]\, B => 
        \can_top_level_inst/stat_bus[71]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_0\, 
        CC => NET_CC_CONFIG1985, P => NET_CC_CONFIG1982, Y3 => 
        NET_CC_CONFIG1983, Y3A => NET_CC_CONFIG1984);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_29\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_20_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_19_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_18_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_17_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_29_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/txtb_cmd_index_gen.3.txtb_priority_odd_gen.txtb_sw_cmd_index_5[3]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[2]\, B
         => \can_top_level_inst/drv_bus[475]\, C => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[11]\, 
        Y => \can_top_level_inst/txtb_sw_cmd_index[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe8\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[11]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_7_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[7]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[1]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst_priority_decoder_inst_un1_m2_0_a2_0_1\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_2_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_152\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[27]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[27]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram43__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_344\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe43\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/int_status\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/un2_int_status_set\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.6.int_module_inst/un1_int_status_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_vector[6]\);
    
    \sbe_ibuf[0]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \sbe_c[0]\, E => ADLIB_GND, YIN => 
        \sbe_ibuf[0]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_11\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[402]\, B => 
        \can_top_level_inst/stat_bus[73]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_1\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_2\, 
        CC => NET_CC_CONFIG1993, P => NET_CC_CONFIG1990, Y3 => 
        NET_CC_CONFIG1991, Y3A => NET_CC_CONFIG1992);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[15]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[15]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_gate_mem_read.ALTB_1[4]\ : 
        CFG4
      generic map(INIT => x"00D8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/AXB[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_138\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ALTB[2]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/AXB[4]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_gate_mem_read_1\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_0_1.CO2\ : 
        CFG3
      generic map(INIT => x"D4")

      port map(A => \can_top_level_inst/txtb_prorities_0[2]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_4\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_021\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram3__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_346\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[1]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_2\ : 
        CFG4
      generic map(INIT => x"F0F1")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_1_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1085_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[14]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[14]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[14]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[17]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[3]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[10]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_15[9]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[2]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[1]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[17]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[17]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y1[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[400]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[16]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_raw_0[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNISGAN_S[1]\, 
        B => \can_top_level_inst/drv_bus[376]\, C => 
        \can_top_level_inst/drv_bus[374]\, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_37\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_RNITOLC[0]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_GND, S => 
        OPEN, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_RNITOLC_Y[0]\, 
        FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[0]\, 
        CC => NET_CC_CONFIG2025, P => NET_CC_CONFIG2022, Y3 => 
        NET_CC_CONFIG2023, Y3A => NET_CC_CONFIG2024);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe76_0_a4\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[0]\, 
        B => \can_top_level_inst/rx_buffer_inst/un5_ram_write_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_34_1_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_34_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/stat_bus[226]\, D => 
        \can_top_level_inst/stat_bus[370]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_4991\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[36]\, 
        C => \can_top_level_inst/rec_ident[4]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2249\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_0[5]\ : 
        CFG4
      generic map(INIT => x"101F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_1_0_Z[5]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_0_1_Z[5]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_128\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_0_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram0_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram0_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_txtb_clk_en_d_NE_2\ : 
        CFG4
      generic map(INIT => x"7BDE")

      port map(A => \can_top_level_inst/txtb_ptr[1]\, B => 
        \can_top_level_inst/txtb_ptr[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_txtb_clk_en_d_NE_2_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram36__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_341\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe36\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select[2]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_1_Z[0]\, 
        B => \sbe_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI0H3V6_S[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[282]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_0\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_0_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[27]\ : 
        CFG4
      generic map(INIT => x"80AA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[24]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst_txnf_calc_proc_6_un6_txtb_state_2\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_1_Z[27]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2_1[1]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_148\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[9]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[9]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[44]\ : 
        SLE
      port map(D => \timestamp_c[44]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[44]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[17]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[13]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[11]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[17]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[3]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[3]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[9]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2698\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2665\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[9]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segm_end_req_capt_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segment_end_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un13_segm_end_req_capt_ce\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segm_end_req_capt_q_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[0]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[3]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_meas_running_q_RNIFDS9\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => \can_top_level_inst/btmc_reset\, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_meas_running_q_Z\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram77_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe77\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram77_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_RNO[4]\ : 
        CFG4
      generic map(INIT => x"3120")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_72\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[4]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[2]\, Y
         => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_83_i\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_0_0\ : 
        ARI1_CC
      generic map(INIT => x"5CC66")

      port map(A => \can_top_level_inst/drv_bus[46]\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[0]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_s_0_2636_FCO\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[0]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_0\, 
        CC => NET_CC_CONFIG2660, P => NET_CC_CONFIG2657, Y3 => 
        NET_CC_CONFIG2658, Y3A => NET_CC_CONFIG2659);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_19\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_19_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_RNIHMAKB2\ : 
        CFG4
      generic map(INIT => x"0703")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_1\, 
        C => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/N_4\, 
        D => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_RNIOH58E_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un1_tq_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[13]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[13]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[27]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[8]\, CLK
         => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[27]\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_dlc_com[0]\ : SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/tran_dlc[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[34]\ : 
        CFG4
      generic map(INIT => x"020E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        B => \can_top_level_inst/can_core_inst/pc_rx_trigger\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_960\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_970\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_776_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram2_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram3_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_o2\ : 
        CFG4
      generic map(INIT => x"F0F2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a2_1_1_Z\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1228\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_933\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_951\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_12[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_4[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram114_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram115_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y1_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[10]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_4_Z\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_5_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_5_Z\, 
        CC => NET_CC_CONFIG2239, P => NET_CC_CONFIG2236, Y3 => 
        NET_CC_CONFIG2237, Y3A => NET_CC_CONFIG2238);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_0[1]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[1]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO[30]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[30]\, B
         => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac11_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[894]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_flag_proc.un8_ack_err_flag_clr\ : 
        CFG3
      generic map(INIT => x"F4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_flag_clr_1\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_ack_err\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_ack_err_flag_clr\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_20\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[31]\, B
         => \can_top_level_inst/rx_buffer_inst_RAM_data_out[30]\, 
        C => \can_top_level_inst/rx_buffer_inst_RAM_data_out[29]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_20_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_17\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[26]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_17_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[21]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[135]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_4[0]\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_24\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_2[0]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_4[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_1_y0[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[208]\, D => 
        \can_top_level_inst/drv_bus[113]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_14_1_wmux_1_Y[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm_0[3]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[3]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[22]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[12]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4Q6DB[15]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[203]\, D => 
        \can_top_level_inst/stat_bus[235]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_14\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4Q6DB_S[15]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_15\, 
        CC => NET_CC_CONFIG816, P => NET_CC_CONFIG813, Y3 => 
        NET_CC_CONFIG814, Y3A => NET_CC_CONFIG815);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[20]\);
    
    \data_out_obuf[13]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(13), D => \data_out_obuf[13]/DOUT\, 
        E => \data_out_obuf[13]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_0[23]\ : 
        CFG4
      generic map(INIT => x"0908")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_1_0_Z[23]\, 
        D => \can_top_level_inst/drv_bus[323]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_0_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[15]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_rx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[83]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[9]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[60]\ : 
        SLE
      port map(D => \timestamp_c[60]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[60]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_o2[1]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        B => \can_top_level_inst/can_core_inst/pc_rx_trigger\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06_RNO_1\ : 
        CFG4
      generic map(INIT => x"0A8E")

      port map(A => \can_top_level_inst/txtb_prorities_1[1]\, B
         => \can_top_level_inst/txtb_prorities_1[0]\, C => 
        \can_top_level_inst/txtb_prorities_0[1]\, D => 
        \can_top_level_inst/txtb_prorities_0[0]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_1_1_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        D => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/N_4254\);
    
    
        \can_top_level_inst/memory_registers_inst/tx_double_parity_error_1_sqmuxa_i\ : 
        CFG4
      generic map(INIT => x"CF8F")

      port map(A => 
        \can_top_level_inst/txtb_parity_error_valid[3]\, B => 
        \can_top_level_inst/drv_bus[475]\, C => 
        \can_top_level_inst/memory_registers_inst/tx_double_parity_error_1_sqmuxa_1_0_Z\, 
        D => \can_top_level_inst/txtb_parity_error_valid[5]\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tx_double_parity_error_1_sqmuxa_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[6]\ : 
        CFG4
      generic map(INIT => x"FEEE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_174\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[16]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[16]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_reg_rst_inst/rx_shift_res_reg_inst/reg_q_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => \can_top_level_inst/drv_bus[474]\, Y => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_reg_rst_inst/rx_shift_res_reg_inst/drv_bus_i[474]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_vals2\ : 
        CFG3
      generic map(INIT => x"8F")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_cnst[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_881\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\);
    
    \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_2\ : 
        CFG4
      generic map(INIT => x"9009")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[3]\, 
        B => \can_top_level_inst/rx_read_pointer[3]\, C => 
        \can_top_level_inst/rx_read_pointer[2]\, D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_2_Z\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_5_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_6_Z\, 
        CC => NET_CC_CONFIG2402, P => NET_CC_CONFIG2399, Y3 => 
        NET_CC_CONFIG2400, Y3A => NET_CC_CONFIG2401);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[16]\);
    
    \timestamp_ibuf[50]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(50), Y => 
        \timestamp_ibuf[50]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIB9FT[0]_CC_2\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO922, CO => CI_TO_CO923, P(0) => 
        NET_CC_CONFIG984, P(1) => NET_CC_CONFIG988, P(2) => 
        NET_CC_CONFIG992, P(3) => NET_CC_CONFIG996, P(4) => 
        NET_CC_CONFIG1000, P(5) => NET_CC_CONFIG1004, P(6) => 
        NET_CC_CONFIG1008, P(7) => NET_CC_CONFIG1012, P(8) => 
        NET_CC_CONFIG1016, P(9) => NET_CC_CONFIG1020, P(10) => 
        NET_CC_CONFIG1024, P(11) => NET_CC_CONFIG1028, Y3(0) => 
        NET_CC_CONFIG985, Y3(1) => NET_CC_CONFIG989, Y3(2) => 
        NET_CC_CONFIG993, Y3(3) => NET_CC_CONFIG997, Y3(4) => 
        NET_CC_CONFIG1001, Y3(5) => NET_CC_CONFIG1005, Y3(6) => 
        NET_CC_CONFIG1009, Y3(7) => NET_CC_CONFIG1013, Y3(8) => 
        NET_CC_CONFIG1017, Y3(9) => NET_CC_CONFIG1021, Y3(10) => 
        NET_CC_CONFIG1025, Y3(11) => NET_CC_CONFIG1029, Y3A(0)
         => NET_CC_CONFIG986, Y3A(1) => NET_CC_CONFIG990, Y3A(2)
         => NET_CC_CONFIG994, Y3A(3) => NET_CC_CONFIG998, Y3A(4)
         => NET_CC_CONFIG1002, Y3A(5) => NET_CC_CONFIG1006, 
        Y3A(6) => NET_CC_CONFIG1010, Y3A(7) => NET_CC_CONFIG1014, 
        Y3A(8) => NET_CC_CONFIG1018, Y3A(9) => NET_CC_CONFIG1022, 
        Y3A(10) => NET_CC_CONFIG1026, Y3A(11) => 
        NET_CC_CONFIG1030, CC(0) => NET_CC_CONFIG987, CC(1) => 
        NET_CC_CONFIG991, CC(2) => NET_CC_CONFIG995, CC(3) => 
        NET_CC_CONFIG999, CC(4) => NET_CC_CONFIG1003, CC(5) => 
        NET_CC_CONFIG1007, CC(6) => NET_CC_CONFIG1011, CC(7) => 
        NET_CC_CONFIG1015, CC(8) => NET_CC_CONFIG1019, CC(9) => 
        NET_CC_CONFIG1023, CC(10) => NET_CC_CONFIG1027, CC(11)
         => NET_CC_CONFIG1031);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[16]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[16]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[2]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[1]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\);
    
    \can_top_level_inst/rx_buffer_inst/rx_read_buff[11]\ : CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/rx_buffer_inst_RAM_data_out[11]\, 
        Y => \can_top_level_inst/rx_read_buff[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate_3_0_.m15\ : 
        CFG4
      generic map(INIT => x"2E03")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_13\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[6]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lt4\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_3_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[7]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1[31]\ : 
        CFG4
      generic map(INIT => x"3350")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[8]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1151_2\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m16_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[307]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/bds_data_in_u\ : 
        CFG4
      generic map(INIT => x"CDC0")

      port map(A => \can_top_level_inst/drv_bus[470]\, B => 
        \can_top_level_inst/can_core_inst/bst_data_out\, C => 
        \can_top_level_inst/can_core_inst/un2_bds_data_in_i\, D
         => \can_top_level_inst/rx_data_wbs\, Y => 
        \can_top_level_inst/can_core_inst/bds_data_in\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un6_ctrl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[8]\ : 
        CFG4
      generic map(INIT => x"40E0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[8]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_950\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1051\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_828_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa7\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe7\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[10]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[9]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[1]\, 
        D => \can_top_level_inst/tran_word[18]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm_0[7]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un10_tq_counter_d_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s_Z[7]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[17]\);
    
    \int_obuf/U_IOPAD\ : IOPAD_TRI
      port map(PAD => int_Z, D => \int_obuf/DOUT\, E => 
        \int_obuf/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram4_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram5_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_0[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[10]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[10]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[10]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[1]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[2]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_28_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/N_3904\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_2_3_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_control[1]\, D
         => \can_top_level_inst/test_registers_out.tst_wdata[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_2_3_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31[1]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[28]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[28]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45[6]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[28]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06_RNO\ : 
        CFG3
      generic map(INIT => x"4D")

      port map(A => \can_top_level_inst/txtb_prorities_1[2]\, B
         => \can_top_level_inst/txtb_prorities_0[2]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_1_1_1\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_i1_mux_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[7]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[7]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[7]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[31]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[17]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2262\, B
         => \can_top_level_inst/rx_buffer_inst/N_4821\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s_2641_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG289, P(1) => NET_CC_CONFIG293, P(2) => 
        NET_CC_CONFIG297, P(3) => NET_CC_CONFIG301, P(4) => 
        NET_CC_CONFIG305, P(5) => NET_CC_CONFIG309, P(6) => 
        NET_CC_CONFIG313, P(7) => NET_CC_CONFIG317, P(8) => 
        NET_CC_CONFIG321, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, 
        P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG290, Y3(1) => 
        NET_CC_CONFIG294, Y3(2) => NET_CC_CONFIG298, Y3(3) => 
        NET_CC_CONFIG302, Y3(4) => NET_CC_CONFIG306, Y3(5) => 
        NET_CC_CONFIG310, Y3(6) => NET_CC_CONFIG314, Y3(7) => 
        NET_CC_CONFIG318, Y3(8) => NET_CC_CONFIG322, Y3(9) => 
        ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, 
        Y3A(0) => NET_CC_CONFIG291, Y3A(1) => NET_CC_CONFIG295, 
        Y3A(2) => NET_CC_CONFIG299, Y3A(3) => NET_CC_CONFIG303, 
        Y3A(4) => NET_CC_CONFIG307, Y3A(5) => NET_CC_CONFIG311, 
        Y3A(6) => NET_CC_CONFIG315, Y3A(7) => NET_CC_CONFIG319, 
        Y3A(8) => NET_CC_CONFIG323, Y3A(9) => ADLIB_VCC, Y3A(10)
         => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG292, CC(1) => NET_CC_CONFIG296, CC(2) => 
        NET_CC_CONFIG300, CC(3) => NET_CC_CONFIG304, CC(4) => 
        NET_CC_CONFIG308, CC(5) => NET_CC_CONFIG312, CC(6) => 
        NET_CC_CONFIG316, CC(7) => NET_CC_CONFIG320, CC(8) => 
        NET_CC_CONFIG324, CC(9) => nc114, CC(10) => nc56, CC(11)
         => nc105);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram93_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe93\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram93_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[21]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[160]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_stuff_count_Z[3]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[2]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_stuff_count\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_stuff_count[3]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_RNIARSOK1\ : 
        CFG4
      generic map(INIT => x"FFC8")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g2_0_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/un8_exit_segm_regular_tseg1_0\, 
        D => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g1_0_0_0\, 
        Y => \can_top_level_inst/prescaler_inst/N_5058\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_address_i_i_m2[5]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_184\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI8LHP_S[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[29]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[29]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[29]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[10]\);
    
    \data_out_obuf[11]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(11), D => \data_out_obuf[11]/DOUT\, 
        E => \data_out_obuf[11]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[24]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[7]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[7]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[7]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNIR0941[7]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/CO0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ANB2\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[6]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/CO2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[29]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_2[5]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[20]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un17_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[9]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0_1\, 
        B => \sbe_c[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[26]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[26]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[26]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_19[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y7_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y5_0[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_8[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram123__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_352\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe123\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[20]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[20]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[20]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[18]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[27]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[59]\, 
        C => \can_top_level_inst/rec_ident[27]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2272\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_i[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_write_pointer[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram62_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe62\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram62_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[1]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_148\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe10\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r_rep_RNIFQ411[17]\ : 
        CFG4
      generic map(INIT => x"CC50")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[16]\, B
         => 
        \can_top_level_inst/memory_registers_inst_test_registers_in_tst_rdata_sn_N_8\, 
        C => \can_top_level_inst/reg_value_r_rep[17]\, D => 
        \can_top_level_inst/test_registers_out.tst_dest[19]\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[31]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[31]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[16]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[16]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[16]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[27]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[13]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[25]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[25]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[29]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIJVB31_S[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c5_RNIM7L91\ : 
        CFG4
      generic map(INIT => x"96A5")

      port map(A => \can_top_level_inst/drv_bus[14]\, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c5_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[6]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[5]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/N_3921\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[4]\ : 
        CFG4
      generic map(INIT => x"FFA2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[2]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_161\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[0]\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_is_rtr_dbl_buf\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/buffer_md_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tran_is_rtr_dbl_buf_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[9]\ : 
        SLE
      port map(D => \timestamp_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram101__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_341\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe101\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1[4]\ : 
        CFG4
      generic map(INIT => x"8B89")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_1_Z[4]\, 
        D => \can_top_level_inst/int_mask[4]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\);
    
    \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_3\ : 
        CFG3
      generic map(INIT => x"84")

      port map(A => \can_top_level_inst/rx_read_pointer[0]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_0_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_3_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[10]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2_0[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_934\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1236\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[1]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[1]\, 
        D => \can_top_level_inst/tran_word[25]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[15]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[0]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_2_0_1[13]\ : 
        CFG4
      generic map(INIT => x"0A77")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[313]\, C => 
        \can_top_level_inst/drv_bus[268]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_0_Z[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_2_0_1_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1[4]\ : 
        CFG4
      generic map(INIT => x"FF35")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[33]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[4]\);
    
    
        \can_top_level_inst/bus_sampling_inst/data_edge_detector_inst/rx_data_sync_prev_RNIFDTE\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst_data_edge_detector_inst_rx_data_sync_prev\, 
        B => \can_top_level_inst/bus_sampling_inst_prev_Sample\, 
        C => \can_top_level_inst/rx_data_wbs\, Y => 
        \can_top_level_inst/bus_sampling_inst_data_edge_detector_inst_un7_sync_edge_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0[4]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[4]\, 
        C => \can_top_level_inst/drv_bus[55]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_51\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/int_status\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/un2_int_status_set\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/un1_int_status_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_vector[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_wmux_2\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_y0_0\, 
        B => \can_top_level_inst/txtb_hw_cmd_index[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_135_i\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_135_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_validated_buffer_7_2_1_0_y3\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[42]\ : 
        SLE
      port map(D => \timestamp_c[42]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[42]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[6]\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram67__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_346\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe67\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[11]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[11]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_81\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[16]\, B => 
        \can_top_level_inst/drv_bus[159]\, C => 
        \can_top_level_inst/drv_bus[192]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_6\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[7]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[8]\, 
        CC => NET_CC_CONFIG722, P => NET_CC_CONFIG719, Y3 => 
        NET_CC_CONFIG720, Y3A => NET_CC_CONFIG721);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[12]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[12]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_y0_0[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1265\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1268\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_0_y3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_RNIRIBP[1]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[0]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_cry[1]\, 
        CC => NET_CC_CONFIG2029, P => NET_CC_CONFIG2026, Y3 => 
        NET_CC_CONFIG2027, Y3A => NET_CC_CONFIG2028);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[25]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[25]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[25]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[11]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[11]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[11]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1258\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1261\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_y0_0[3]\);
    
    \data_in_ibuf[31]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(31), Y => \data_in_ibuf[31]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1[23]\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[23]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06_RNO_0\ : 
        CFG4
      generic map(INIT => x"2F00")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[9]\, 
        B => \can_top_level_inst/drv_bus_475_rep1\, C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_0\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_06\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[25]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[6]\);
    
    \data_in_ibuf[22]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[22]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[22]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[16]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[19]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[19]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[28]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[28]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[28]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[20]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[20]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[20]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/alc_Z[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/stat_bus[6]\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[293]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_s_8\ : 
        ARI1_CC
      generic map(INIT => x"46600")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[8]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_7_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[8]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG2692, P => 
        NET_CC_CONFIG2689, Y3 => NET_CC_CONFIG2690, Y3A => 
        NET_CC_CONFIG2691);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[10]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_parity_check_valid_0\ : 
        CFG4
      generic map(INIT => x"F808")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_clk_en_q_Z\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, C
         => \can_top_level_inst/tx_arbitrator_inst/txtb_N_3_mux\, 
        D => \can_top_level_inst/tx_arbitrator_inst/r_N_5_mux\, Y
         => \can_top_level_inst/txtb_parity_check_valid\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[17]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60[0]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[31]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\);
    
    \can_top_level_inst/bus_sampling_inst/un1_ssp_enable\ : CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[10]\, Y => 
        \can_top_level_inst/bus_sampling_inst/un1_ssp_enable_Z\);
    
    \timestamp_ibuf[18]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[18]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[18]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa5\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \adress_c[2]\, B => \adress_c[4]\, C => 
        \adress_c[6]\, D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/wa4_2\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[18]\);
    
    \data_in_ibuf[24]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[24]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[24]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[8]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[8]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[8]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa9\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe9\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[17]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[121]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c5\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c4_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[4]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c5_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[24]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[20]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_FCNET1\, 
        CO => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_8\, 
        CC => NET_CC_CONFIG2021, P => NET_CC_CONFIG2018, Y3 => 
        NET_CC_CONFIG2019, Y3A => NET_CC_CONFIG2020);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[4]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_83\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[97]\, B => 
        \can_top_level_inst/rec_ident[16]\, C => 
        \can_top_level_inst/drv_bus[130]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_7\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_0_cy_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => ADLIB_GND, 
        P(1) => NET_CC_CONFIG2693, P(2) => NET_CC_CONFIG2697, 
        P(3) => NET_CC_CONFIG2701, P(4) => NET_CC_CONFIG2705, 
        P(5) => NET_CC_CONFIG2709, P(6) => NET_CC_CONFIG2713, 
        P(7) => NET_CC_CONFIG2717, P(8) => NET_CC_CONFIG2721, 
        P(9) => NET_CC_CONFIG2725, P(10) => NET_CC_CONFIG2729, 
        P(11) => NET_CC_CONFIG2733, Y3(0) => ADLIB_GND, Y3(1) => 
        NET_CC_CONFIG2694, Y3(2) => NET_CC_CONFIG2698, Y3(3) => 
        NET_CC_CONFIG2702, Y3(4) => NET_CC_CONFIG2706, Y3(5) => 
        NET_CC_CONFIG2710, Y3(6) => NET_CC_CONFIG2714, Y3(7) => 
        NET_CC_CONFIG2718, Y3(8) => NET_CC_CONFIG2722, Y3(9) => 
        NET_CC_CONFIG2726, Y3(10) => NET_CC_CONFIG2730, Y3(11)
         => NET_CC_CONFIG2734, Y3A(0) => ADLIB_GND, Y3A(1) => 
        NET_CC_CONFIG2695, Y3A(2) => NET_CC_CONFIG2699, Y3A(3)
         => NET_CC_CONFIG2703, Y3A(4) => NET_CC_CONFIG2707, 
        Y3A(5) => NET_CC_CONFIG2711, Y3A(6) => NET_CC_CONFIG2715, 
        Y3A(7) => NET_CC_CONFIG2719, Y3A(8) => NET_CC_CONFIG2723, 
        Y3A(9) => NET_CC_CONFIG2727, Y3A(10) => NET_CC_CONFIG2731, 
        Y3A(11) => NET_CC_CONFIG2735, CC(0) => nc63, CC(1) => 
        NET_CC_CONFIG2696, CC(2) => NET_CC_CONFIG2700, CC(3) => 
        NET_CC_CONFIG2704, CC(4) => NET_CC_CONFIG2708, CC(5) => 
        NET_CC_CONFIG2712, CC(6) => NET_CC_CONFIG2716, CC(7) => 
        NET_CC_CONFIG2720, CC(8) => NET_CC_CONFIG2724, CC(9) => 
        NET_CC_CONFIG2728, CC(10) => NET_CC_CONFIG2732, CC(11)
         => NET_CC_CONFIG2736);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[20]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram98__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_350\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe98\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[26]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[26]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[26]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[111]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_proc.txtb_hw_cmd_q.unlock_2\ : 
        CFG4
      generic map(INIT => x"0302")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1497_2\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1317_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/unlock_sn_N_8_mux\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1497_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/unlock_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe8_0_a4\ : 
        CFG4
      generic map(INIT => x"0010")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_348\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[10]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2699\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2666\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[10]\);
    
    \can_top_level_inst/frame_filters_inst/un1_filter_c_enable_1\ : 
        CFG4
      generic map(INIT => x"F3F5")

      port map(A => \can_top_level_inst/drv_bus[234]\, B => 
        \can_top_level_inst/drv_bus[235]\, C => 
        \can_top_level_inst/stat_bus[156]\, D => 
        \can_top_level_inst/stat_bus[155]\, Y => 
        \can_top_level_inst/frame_filters_inst/un1_filter_c_enable_1_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1[22]\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[22]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un10_err_ctrs_unchanged\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \can_top_level_inst/can_core_inst/stuff_err\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => can_tx_c, D => \can_top_level_inst/stat_bus[370]\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_detector_inst_un10_err_ctrs_unchanged\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_38_1_0_wmux_RNO[16]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_2_0_Z[16]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_2_Z[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_759\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_c4\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[3]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[2]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_data_length_sub_c_c4_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[26]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[30]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[25]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[27]\ : 
        SLE
      port map(D => \data_in_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[265]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_write_address[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[4]\, 
        C => \can_top_level_inst/rx_buffer_inst/N_48\, Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_14[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_5[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram118_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram119_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y3_0[0]\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q[1]\ : SLE
      port map(D => \can_top_level_inst/rx_buffer_inst/N_3925_i\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[1]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/int_mask_load\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[5]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/int_mask_load_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m18_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[29]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[29]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[21]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2547\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2479\, Y
         => \can_top_level_inst/memory_registers_inst/N_2677\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[38]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[19]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un4_compl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ANB2\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_m0[7]\ : 
        CFG4
      generic map(INIT => x"FF37")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_eight\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_i\, 
        C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_one\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[7]\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_m0_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => \can_top_level_inst/drv_bus[20]\, B => 
        \can_top_level_inst/drv_bus[27]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_3_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_4_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_4_Z\, 
        CC => NET_CC_CONFIG2163, P => NET_CC_CONFIG2160, Y3 => 
        NET_CC_CONFIG2161, Y3A => NET_CC_CONFIG2162);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[16]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[16]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[16]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_38_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_949\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_964\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_38_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_2[1]\ : 
        CFG3
      generic map(INIT => x"07")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_2_1[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_2_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[279]\);
    
    \timestamp_ibuf[43]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[43]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[43]/YIN\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_RNO[25]\ : 
        CFG4
      generic map(INIT => x"336C")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[14]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/m51_1\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[14]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1219\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[13]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[13]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[13]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[20]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[29]\ : 
        CFG4
      generic map(INIT => x"5040")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/data_selector[0]\, B
         => \can_top_level_inst/rx_buffer_inst/N_2307_1\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_10\, 
        D => \can_top_level_inst/rx_buffer_inst/N_2307_2\, Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[29]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0[2]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[2]\, 
        C => \can_top_level_inst/drv_bus[58]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_51\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[17]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[6]\ : 
        CFG4
      generic map(INIT => x"0E00")

      port map(A => \can_top_level_inst/txtb_hw_cmd.err\, B => 
        \can_top_level_inst/txtb_hw_cmd.arbl\, C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[1]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_27\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[10]\, B => 
        \can_top_level_inst/drv_bus[91]\, C => 
        \can_top_level_inst/drv_bus[124]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_51\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[4]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[5]\, 
        CC => NET_CC_CONFIG1787, P => NET_CC_CONFIG1784, Y3 => 
        NET_CC_CONFIG1785, Y3A => NET_CC_CONFIG1786);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[3]\ : 
        CFG4
      generic map(INIT => x"F080")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_2[21]\ : 
        CFG4
      generic map(INIT => x"AFC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_939\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_938\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_1_Z[21]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_2_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[148]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOOLCL[31]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[219]\, D => 
        \can_top_level_inst/stat_bus[251]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_30\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOOLCL_S[31]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG880, P => 
        NET_CC_CONFIG877, Y3 => NET_CC_CONFIG878, Y3A => 
        NET_CC_CONFIG879);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI6BEO2[6]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/drv_bus[382]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_6\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI6BEO2_S[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI6BEO2[6]_FCNET1\, 
        CC => NET_CC_CONFIG1597, P => NET_CC_CONFIG1594, Y3 => 
        NET_CC_CONFIG1595, Y3A => NET_CC_CONFIG1596);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns[7]\ : 
        CFG4
      generic map(INIT => x"AAAE")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[2]\, 
        C => \can_top_level_inst/rec_valid_f\, D => 
        \can_top_level_inst/rec_abort_f\, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[20]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[101]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_1[23]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[137]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_1_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIALJ4K_S[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[217]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_settings_reg_comp/wr_select_0[0]\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10_2\, 
        B => \adress_c[7]\, C => \adress_c[5]\, D => 
        \adress_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_settings_reg_comp/wr_select_0_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[30]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[12]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61[11]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[11]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[26]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[264]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_12[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_4[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram18_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram19_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y1_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[7]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_ena_reg_proc.un21_ctrl_signal_upd_i_0\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_962_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un21_ctrl_signal_upd_i_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_3[11]\ : 
        CFG3
      generic map(INIT => x"31")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[125]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1547\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa19\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe19\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[2]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram30__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_351\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe30\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[23]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[23]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[23]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[25]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[25]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[25]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[25]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[30]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[30]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/op_eq.un6_reinteg_ctr_expired\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[7]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un6_reinteg_ctr_expired_5\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un6_reinteg_ctr_expired_4\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reinteg_ctr_expired\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/fsm_wait_state_d_i_1\ : 
        CFG4
      generic map(INIT => x"FF04")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[7]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        C => \can_top_level_inst/tx_arbitrator_inst/N_109\, D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/fsm_wait_state_d_i_0_Z\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/fsm_wait_state_d_i_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[255]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[3]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_139\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_0_1[19]\ : 
        CFG4
      generic map(INIT => x"1A1F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_0_Z[19]\, 
        B => \can_top_level_inst/txtb_state_4[3]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => \can_top_level_inst/stat_bus[74]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_0_1_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_2[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[11]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[11]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[11]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[27]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[27]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[27]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[22]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[22]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\);
    
    \data_out_obuf[26]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[26]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[26]/DOUT\, EOUT => 
        \data_out_obuf[26]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[20]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_dbt_Z[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_2_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[24]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[28]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_rx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[81]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[16]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[16]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_1_Z[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_0_Z[1]\, 
        D => \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[27]\ : 
        SLE
      port map(D => \data_in_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[327]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry[2]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[1]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[2]\, 
        CC => NET_CC_CONFIG300, P => NET_CC_CONFIG297, Y3 => 
        NET_CC_CONFIG298, Y3A => NET_CC_CONFIG299);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[13]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[13]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[13]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4819\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe8\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNII1PC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_824_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_0_0[0]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[3]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_928_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIVA9S\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[17]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[17]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[14]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[22]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_4[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_1_RNO\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => \can_top_level_inst/drv_bus[511]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_0_sqmuxa\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/drv_pex_li_m_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[11]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2537\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2469\, Y
         => \can_top_level_inst/memory_registers_inst/N_2667\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_wmux_1\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[2]\, B
         => \can_top_level_inst/txtb_hw_cmd_index[1]\, C => 
        \can_top_level_inst/N_3900_i\, D => 
        \can_top_level_inst/N_3897_i\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_tx_7_2_1_y0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_write_pointer_ts_ce_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[43]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4_2[1]\ : 
        CFG4
      generic map(INIT => x"C808")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_1_S\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, D
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_1_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[11]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2537\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[8]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2534\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry[0]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[0]\, 
        D => \can_top_level_inst/rx_write_pointer[0]\, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s_2638_FCO\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[0]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[0]\, 
        CC => NET_CC_CONFIG369, P => NET_CC_CONFIG366, Y3 => 
        NET_CC_CONFIG367, Y3A => NET_CC_CONFIG368);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[2]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[2]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIEMC5A_S[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[233]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[25]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o3_0[1]\ : 
        CFG4
      generic map(INIT => x"C0EA")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_146\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_147\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[14]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a3_0[1]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_ety\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_148\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_163\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[30]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[30]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[30]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[24]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[138]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_1[19]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => \can_top_level_inst/drv_bus[468]\, D => 
        \can_top_level_inst/drv_bus[162]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_1_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[254]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_pos_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/err_detected\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[100]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_21_1[11]\ : 
        CFG4
      generic map(INIT => x"3210")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/int_ena[11]\, D => 
        \can_top_level_inst/drv_bus[282]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1565_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[18]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_enable_clear_0_a2\ : 
        CFG4
      generic map(INIT => x"0B0A")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        B => \can_top_level_inst/drv_bus[460]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1013\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1267_i\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[12]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[15]\);
    
    \data_out_obuf[7]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(7), D => \data_out_obuf[7]/DOUT\, 
        E => \data_out_obuf[7]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[12]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_2_1[1]\ : 
        CFG3
      generic map(INIT => x"7F")

      port map(A => \can_top_level_inst/drv_bus[475]\, B => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[6]\, 
        C => \can_top_level_inst/txtb_allow_bb[6]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_2_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/command_reg_comp/bit_gen.4.reg_present_gen.reg_autoclear_gen.reg_value_r_8[4]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/access_active_1[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        D => \data_in_c[4]\, Y => 
        \can_top_level_inst/drv_bus[429]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_9\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[9]\, B => 
        \can_top_level_inst/drv_bus[309]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_8\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_9\, 
        CC => NET_CC_CONFIG565, P => NET_CC_CONFIG562, Y3 => 
        NET_CC_CONFIG563, Y3A => NET_CC_CONFIG564);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa16\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe16\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_2_0[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/stat_bus[303]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_1_0_Z[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_2_0_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_38_1_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_38_1_0_y0[5]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1350\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1352\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6[5]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/un41_txtb_pointer_meta_d_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q_Z[1]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram3_[0]\ : 
        SLE
      port map(D => can_tx_c, CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, 
        EN => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe3\, 
        ALn => \can_top_level_inst/bus_sampling_inst/reg_q\, ADn
         => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram3_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[15]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_2_1[13]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \can_top_level_inst/stat_bus[270]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_2_1_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac16\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_0_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac16_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_2_0[16]\ : 
        CFG4
      generic map(INIT => x"3708")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_2_0_1_Z[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_0_Z[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_2_0_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_4\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[5]\, B
         => \can_top_level_inst/can_core_inst/crc_15[4]\, C => 
        \can_top_level_inst/can_core_inst/crc_15[5]\, D => 
        \can_top_level_inst/store_data_word[4]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[1]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[2]\, 
        CC => NET_CC_CONFIG58, P => NET_CC_CONFIG55, Y3 => 
        NET_CC_CONFIG56, Y3A => NET_CC_CONFIG57);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_0_o2_0[13]\ : 
        CFG4
      generic map(INIT => x"FFEF")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1194\, 
        C => \can_top_level_inst/tran_frame_test.tprm[1]\, D => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1206\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram76_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe76\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram76_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[14]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m16_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_21_1_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_21_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[403]\, D => 
        \can_top_level_inst/drv_bus[84]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1261\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[29]\ : 
        CFG4
      generic map(INIT => x"08AA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[24]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_160\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_1_Z[29]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[24]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[24]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_2[6]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[21]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un17_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[2]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[18]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[18]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_0\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[10]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_read_exp_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[7]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[7]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[7]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[208]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_RNI5QE92[3]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\, C
         => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[3]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[2]\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[3]\, 
        CC => NET_CC_CONFIG1107, P => NET_CC_CONFIG1104, Y3 => 
        NET_CC_CONFIG1105, Y3A => NET_CC_CONFIG1106);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[18]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_11[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram80_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram81_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_4[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[154]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[28]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_8_RNI5VBA57\ : 
        ARI1_CC
      generic map(INIT => x"4D2F0")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[8]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_7\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[8]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_8\, 
        CC => NET_CC_CONFIG2139, P => NET_CC_CONFIG2136, Y3 => 
        NET_CC_CONFIG2137, Y3A => NET_CC_CONFIG2138);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_1_0_0_Z[2]\, 
        B => \sbe_c[0]\, C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_0[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[14]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram45_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe45\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram45_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[30]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[22]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[22]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[22]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[18]\ : 
        SLE
      port map(D => \timestamp_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[9]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g1_0\ : 
        CFG4
      generic map(INIT => x"0EEE")

      port map(A => 
        \can_top_level_inst/can_core_inst/can_crc_inst/un2_crc_ena_17_21_i\, 
        B => 
        \can_top_level_inst/can_core_inst/can_crc_inst_crc_enable\, 
        C => 
        \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/crc_calc_from_rx_0\, Y
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g1_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[25]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[25]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[25]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[24]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2482\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d[4]\ : 
        CFG4
      generic map(INIT => x"BBB8")

      port map(A => \can_top_level_inst/drv_bus[420]\, B => 
        \can_top_level_inst/drv_bus[426]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[4]\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_12_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => scs_c, B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_2_Z\, 
        C => \adress_c[9]\, D => \adress_c[8]\, Y => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\);
    
    \swr_ibuf/U_IOPAD\ : IOPAD_IN
      port map(PAD => swr, Y => \swr_ibuf/YIN\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_7_2_0[3]\ : 
        CFG4
      generic map(INIT => x"8D05")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_7_2_0_1_Z[3]\, 
        D => \can_top_level_inst/int_mask[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_7_2_0_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_8\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_8\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[8]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_7_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_8_Z\, 
        CC => NET_CC_CONFIG1517, P => NET_CC_CONFIG1514, Y3 => 
        NET_CC_CONFIG1515, Y3A => NET_CC_CONFIG1516);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[30]\);
    
    \can_top_level_inst/int_manager_inst/int_input_active_16[11]\ : 
        CFG4
      generic map(INIT => x"FAFE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_180\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        C => 
        \can_top_level_inst/int_manager_inst/int_input_active_11_Z[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_16_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0[12]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc3\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        C => \can_top_level_inst/drv_bus[477]\, D => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_set_idle_0_o2\ : 
        CFG4
      generic map(INIT => x"CECF")

      port map(A => \can_top_level_inst/drv_bus[470]\, B => 
        \can_top_level_inst/drv_bus[513]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => \can_top_level_inst/tran_frame_valid\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1062\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[28]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_6_1[11]\ : 
        CFG4
      generic map(INIT => x"3210")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode[11]\, 
        D => \can_top_level_inst/drv_bus[154]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1550_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO[26]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[26]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac11_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m_0[890]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_2[31]\ : 
        CFG4
      generic map(INIT => x"8222")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_1225\, 
        C => \can_top_level_inst/tran_frame_test.fcrc\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_82_mux\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_185\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_183\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_184\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/form_err_i_1_iv_1_RNO_0\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1314\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_sqmuxa_m\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        C => \can_top_level_inst/drv_bus[47]\, D => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[1]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1[9]\ : 
        CFG4
      generic map(INIT => x"F015")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_1_Z[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_148\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[10]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe61_0_a4\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_347\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a3_0[1]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_ety\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_148\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_163\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_17_0_a4_0\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[23]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_17_0_a4_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[2]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_76\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[90]\, B => 
        \can_top_level_inst/rec_ident[9]\, C => 
        \can_top_level_inst/drv_bus[123]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_11\);
    
    
        \can_top_level_inst/int_manager_inst/int_input_active_17_0[11]\ : 
        CFG4
      generic map(INIT => x"FFF4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_134\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        C => 
        \can_top_level_inst/int_manager_inst/int_input_active_13_0_Z[11]\, 
        D => 
        \can_top_level_inst/int_manager_inst/int_input_active_5_Z[11]\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_input_active_17_0_Z[11]\);
    
    \adress_ibuf[7]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \adress_c[7]\, E => ADLIB_GND, YIN => 
        \adress_ibuf[7]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[6]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[316]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[26]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[26]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[26]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2650\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_8[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[238]\, C => 
        \can_top_level_inst/drv_bus[114]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1134\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[27]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[27]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[27]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4287\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.err_0_sqmuxa_4\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state_0_Z[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[7]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_0_sqmuxa_4\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un2_tq_counter_allowlto7_4\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => \can_top_level_inst/drv_bus[7]\, B => 
        \can_top_level_inst/drv_bus[6]\, C => 
        \can_top_level_inst/drv_bus[5]\, D => 
        \can_top_level_inst/drv_bus[4]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un2_tq_counter_allowlto7_4_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[16]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2705\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2672\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[6]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[25]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[139]\);
    
    \timestamp_ibuf[19]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[19]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[19]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[16]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2640\, Y
         => \can_top_level_inst/memory_registers_inst/N_2705\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[14]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[14]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_write_pointer_ts_ce_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[1]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_0_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO394, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_GND, P(3) => 
        NET_CC_CONFIG395, P(4) => NET_CC_CONFIG399, P(5) => 
        NET_CC_CONFIG403, P(6) => NET_CC_CONFIG407, P(7) => 
        NET_CC_CONFIG411, P(8) => NET_CC_CONFIG415, P(9) => 
        NET_CC_CONFIG419, P(10) => NET_CC_CONFIG423, P(11) => 
        NET_CC_CONFIG427, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => ADLIB_VCC, Y3(3) => NET_CC_CONFIG396, Y3(4) => 
        NET_CC_CONFIG400, Y3(5) => NET_CC_CONFIG404, Y3(6) => 
        NET_CC_CONFIG408, Y3(7) => NET_CC_CONFIG412, Y3(8) => 
        NET_CC_CONFIG416, Y3(9) => NET_CC_CONFIG420, Y3(10) => 
        NET_CC_CONFIG424, Y3(11) => NET_CC_CONFIG428, Y3A(0) => 
        ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => ADLIB_VCC, 
        Y3A(3) => NET_CC_CONFIG397, Y3A(4) => NET_CC_CONFIG401, 
        Y3A(5) => NET_CC_CONFIG405, Y3A(6) => NET_CC_CONFIG409, 
        Y3A(7) => NET_CC_CONFIG413, Y3A(8) => NET_CC_CONFIG417, 
        Y3A(9) => NET_CC_CONFIG421, Y3A(10) => NET_CC_CONFIG425, 
        Y3A(11) => NET_CC_CONFIG429, CC(0) => nc352, CC(1) => 
        nc313, CC(2) => nc309, CC(3) => NET_CC_CONFIG398, CC(4)
         => NET_CC_CONFIG402, CC(5) => NET_CC_CONFIG406, CC(6)
         => NET_CC_CONFIG410, CC(7) => NET_CC_CONFIG414, CC(8)
         => NET_CC_CONFIG418, CC(9) => NET_CC_CONFIG422, CC(10)
         => NET_CC_CONFIG426, CC(11) => NET_CC_CONFIG430);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un4_tst_ena_2\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[16]\, B
         => 
        \can_top_level_inst/test_registers_out.tst_control[0]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena_2\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[28]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2554\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2486\, Y
         => \can_top_level_inst/memory_registers_inst/N_2684\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_5\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[9]\, B
         => \can_top_level_inst/can_core_inst/crc_21[8]\, C => 
        \can_top_level_inst/can_core_inst/crc_21[9]\, D => 
        \can_top_level_inst/store_data_word[8]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[3]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[4]\, 
        CC => NET_CC_CONFIG18, P => NET_CC_CONFIG15, Y3 => 
        NET_CC_CONFIG16, Y3A => NET_CC_CONFIG17);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[21]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_1_0[19]\ : 
        CFG4
      generic map(INIT => x"9C98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_1_0_1_Z[19]\, 
        D => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_30_1_0_Z[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test_dbl_buf.fstc\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[0]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/load_ffmt_w_addr\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/fstc\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[28]\);
    
    \can_top_level_inst/tx_arbitrator_inst/un4_txtb_changed_NE_0\ : 
        CFG4
      generic map(INIT => x"7DBE")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/last_txtb_index_Z[0]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/last_txtb_index_Z[1]\, 
        C => \can_top_level_inst/txtb_hw_cmd_index[1]\, D => 
        \can_top_level_inst/txtb_hw_cmd_index[0]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un4_txtb_changed_NE_0_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.1.l2_prio_12_m_3_1_0_wmux\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_36_i\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_26_i\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_120\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_122\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_12_m_3_1_0_y0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m36_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[17]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[10]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_19[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y7_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y5_0[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_8[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_1[3]\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_1_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_3[0]\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1192\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_0_Z[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1189\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1190\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_3_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[23]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[23]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_0_0\ : 
        ARI1_CC
      generic map(INIT => x"5CC66")

      port map(A => \can_top_level_inst/drv_bus[29]\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_fast\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[0]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_s_0_2637_FCO\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[0]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_0\, 
        CC => NET_CC_CONFIG888, P => NET_CC_CONFIG885, Y3 => 
        NET_CC_CONFIG886, Y3A => NET_CC_CONFIG887);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[23]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_1_Z\, 
        B => \can_top_level_inst/txtb_parity_check_valid\, C => 
        \can_top_level_inst/txtb_parity_error_valid_2[6]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m100\ : 
        CFG3
      generic map(INIT => x"27")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[245]\, C => 
        \can_top_level_inst/drv_bus[121]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i69_mux\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_64\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[102]\, B => 
        \can_top_level_inst/rec_ident[21]\, C => 
        \can_top_level_inst/drv_bus[135]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_21\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_3[31]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac25_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac4_Z\, 
        C => \can_top_level_inst/stat_bus[288]\, D => 
        \can_top_level_inst/stat_bus[251]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_3_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[22]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[22]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[22]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[16]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[16]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_q.err\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_2\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/txtb_hw_cmd.err\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\);
    
    AFLSDF_INV_4 : INV_BA
      port map(A => \AFLSDF_INV_4\, Y => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_28_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_28_FCNET1\, 
        CO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid\, 
        CC => NET_CC_CONFIG1949, P => NET_CC_CONFIG1946, Y3 => 
        NET_CC_CONFIG1947, Y3A => NET_CC_CONFIG1948);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram61__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_347\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe61\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_70\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[228]\, B => 
        \can_top_level_inst/rec_ident[23]\, C => 
        \can_top_level_inst/drv_bus[261]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_16\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un6_ctrl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[2]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNI6RL45[11]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[11]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[10]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[11]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[11]\, 
        CC => NET_CC_CONFIG1722, P => NET_CC_CONFIG1719, Y3 => 
        NET_CC_CONFIG1720, Y3A => NET_CC_CONFIG1721);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_i_3_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_ack_err\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_28_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/N_3903\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNI5RKV[3]\ : 
        CFG4
      generic map(INIT => x"3332")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_4\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_9\, 
        Y => \can_top_level_inst/stat_bus[381]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[28]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[28]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[7]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_6_1[10]\ : 
        CFG4
      generic map(INIT => x"5410")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => \can_top_level_inst/drv_bus[475]\, D => 
        \can_top_level_inst/drv_bus[153]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1512_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram24_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe24\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram24_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_iv_4[1]\ : 
        CFG4
      generic map(INIT => x"FFF4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_913_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_900\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_19_m_Z[4]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_iv_2_Z[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_iv_4[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry[2]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[1]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[2]\, 
        CC => NET_CC_CONFIG2354, P => NET_CC_CONFIG2351, Y3 => 
        NET_CC_CONFIG2352, Y3A => NET_CC_CONFIG2353);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[24]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[12]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0[0]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[0]\, 
        C => \can_top_level_inst/drv_bus[51]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_47\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_a5_2[0]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_switch_to_ssp_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1408\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1414\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m30_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[30]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[7]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2252\, B
         => \can_top_level_inst/rx_buffer_inst/N_4267\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2285\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_7_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[30]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_wmux_3\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_0_y3\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_0_y1\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[0]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_1[16]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[130]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_1_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[477]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[12]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[12]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_20[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_8[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y3_0[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y1_0[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y21[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[20]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/drv_ena_reg_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[6]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[5]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[1]\, 
        D => \can_top_level_inst/tran_word[30]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[6]\);
    
    \can_top_level_inst/tx_arbitrator_inst/validated_buffer_3\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[1]\, B
         => \can_top_level_inst/txtb_hw_cmd_index[0]\, C => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[12]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[18]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[18]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[12]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_28[15]\ : 
        CFG4
      generic map(INIT => x"6420")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[270]\, D => 
        \can_top_level_inst/drv_bus[96]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_734\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[218]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIUVJM1[2]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[2]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_1\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_2\, 
        CC => NET_CC_CONFIG1285, P => NET_CC_CONFIG1282, Y3 => 
        NET_CC_CONFIG1283, Y3A => NET_CC_CONFIG1284);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[30]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[22]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[22]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[22]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIK5UU5[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[10]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[10]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_9\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_10\, 
        CC => NET_CC_CONFIG1317, P => NET_CC_CONFIG1314, Y3 => 
        NET_CC_CONFIG1315, Y3A => NET_CC_CONFIG1316);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[1]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0UM0H_S[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[244]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d[1]\ : 
        CFG4
      generic map(INIT => x"60C0")

      port map(A => \can_top_level_inst/stat_bus[2]\, B => 
        \can_top_level_inst/stat_bus[3]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/un1_retr_ctr_add_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retr_ctr_add\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d_Z[1]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_21\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[6]\, B => 
        \can_top_level_inst/drv_bus[87]\, C => 
        \can_top_level_inst/drv_bus[120]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_56\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[2]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[3]\, 
        CC => NET_CC_CONFIG1779, P => NET_CC_CONFIG1776, Y3 => 
        NET_CC_CONFIG1777, Y3A => NET_CC_CONFIG1778);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_ns_1_0_.m15_1\ : 
        CFG4
      generic map(INIT => x"084E")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\, 
        C => \can_top_level_inst/can_core_inst/set_idle\, D => 
        \can_top_level_inst/can_core_inst/operation_control_inst/un2_go_to_off_i_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/operation_control_inst/m15_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[12]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2636\, Y
         => \can_top_level_inst/memory_registers_inst/N_2701\);
    
    \timestamp_ibuf[3]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[3]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[3]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[13]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[13]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[13]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2637\);
    
    \data_in_ibuf[17]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(17), Y => \data_in_ibuf[17]/YIN\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => swr_c, B => \sbe_c[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_1[0]\, 
        D => \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_shorten_tseg1_after_tseg2\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => \can_top_level_inst/prescaler_inst/g1_0_0\, B
         => 
        \can_top_level_inst/prescaler_inst/un6_phase_err_sjw_by_one_NE\, 
        C => \can_top_level_inst/prescaler_inst/g0_0_7\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_shorten_tseg1_after_tseg2_i\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNII4BK[30]\ : 
        CFG3
      generic map(INIT => x"32")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[29]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[30]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_765_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m18_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe65_0_a4\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[0]\, 
        B => \can_top_level_inst/rx_buffer_inst/un5_ram_write_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_RNIQ8RR1\ : 
        CFG3
      generic map(INIT => x"D0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/CO0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un2_compl_ctr_div_32_plus_5_sat_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_s1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[21]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[21]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_err_delim_late_i\ : 
        CFG4
      generic map(INIT => x"FFEF")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_932\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_N_846\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_814_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[6]\ : 
        CFG4
      generic map(INIT => x"FF40")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_178\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_y0_0[8]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[213]\, D => 
        \can_top_level_inst/drv_bus[234]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_0_y3[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_1[1]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[8]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un9_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa12\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe12\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[18]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2544\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2476\, Y
         => \can_top_level_inst/memory_registers_inst/N_2674\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/crc_err\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => \can_top_level_inst/stat_bus[377]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_match\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, D
         => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_err\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[18]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2544\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate_3_0_.m17\ : 
        CFG4
      generic map(INIT => x"005E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[2]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lt4_1\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[5]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/i5_mux\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\);
    
    \data_out_obuf[28]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(28), D => \data_out_obuf[28]/DOUT\, 
        E => \data_out_obuf[28]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30[9]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[9]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[29]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_87_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_87_FCNET1\, 
        CO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_I_87_FCO_1\, 
        CC => NET_CC_CONFIG523, P => NET_CC_CONFIG520, Y3 => 
        NET_CC_CONFIG521, Y3A => NET_CC_CONFIG522);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask.un7_flip_mask_axbxc1\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[0]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[1]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[27]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[27]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram54_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe54\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram54_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_27[12]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/txtb_prorities_3[0]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_619\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe6\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_5_0\ : 
        ARI1_CC
      generic map(INIT => x"5CC66")

      port map(A => \can_top_level_inst/drv_bus[34]\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[5]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_4\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_5\, 
        CC => NET_CC_CONFIG908, P => NET_CC_CONFIG905, Y3 => 
        NET_CC_CONFIG906, Y3A => NET_CC_CONFIG907);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[14]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[273]\, B => 
        \can_top_level_inst/rec_ident[2]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_1\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_2\, 
        CC => NET_CC_CONFIG1841, P => NET_CC_CONFIG1838, Y3 => 
        NET_CC_CONFIG1839, Y3A => NET_CC_CONFIG1840);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\);
    
    
        \can_top_level_inst/bus_sampling_inst/can_rx_sig_sync_inst/rff\ : 
        SLE
      port map(D => can_rx_c, CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, 
        EN => ADLIB_VCC, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/can_rx_sig_sync_inst/rff_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[7]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_2\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => \can_top_level_inst/stat_bus[14]\, B => 
        \can_top_level_inst/bus_sampling_inst_data_edge_detector_inst_rx_data_sync_prev\, 
        C => \can_top_level_inst/prescaler_inst/sync_flag\, Y => 
        \can_top_level_inst/prescaler_inst/g0_0_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[17]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[2]\, B => 
        \can_top_level_inst/drv_bus[302]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_1\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_2\, 
        CC => NET_CC_CONFIG537, P => NET_CC_CONFIG534, Y3 => 
        NET_CC_CONFIG535, Y3A => NET_CC_CONFIG536);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[2]\ : 
        CFG4
      generic map(INIT => x"CECC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_182\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_154\, 
        C => \can_top_level_inst/N_4606\, D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_8[28]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => \can_top_level_inst/drv_bus[328]\, B => 
        \can_top_level_inst/txtb_prorities_7[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac16_Z\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac8_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_8_Z[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm_0[7]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[7]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[28]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[9]\, CLK
         => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[27]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNICA923_S[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[223]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_type\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ide\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[155]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[25]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe6\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa15\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe15\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[283]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[1]\, 
        D => \can_top_level_inst/prescaler_inst/tseg1_nbt[1]\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_1_2[2]\ : 
        CFG4
      generic map(INIT => x"BAAA")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_1_1_Z[2]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[15]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_0_Z[2]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_1_2_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO[25]\ : 
        CFG4
      generic map(INIT => x"3266")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_37_2_1_1[25]\, 
        C => \can_top_level_inst/stat_bus[245]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_37_2_1_2[25]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1123\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8[6]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2378\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[18]\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_read_address[6]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => \can_top_level_inst/rx_read_pointer[6]\, C => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_length_1_0[0]\ : 
        CFG4
      generic map(INIT => x"BBBF")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        Y => \can_top_level_inst/can_core_inst/stuff_length[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\);
    
    AFLSDF_INV_5 : INV_BA
      port map(A => \AFLSDF_INV_5\, Y => 
        \I_1/U0_RGB1_RGB7_rgb_net_1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/un7_index_sat[2]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \adress_c[7]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[4]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[11]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_0\ : 
        CFG4
      generic map(INIT => x"0BBB")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un7_exit_segm_regular_cry_9\, 
        B => \can_top_level_inst/prescaler_inst/is_tseg2\, C => 
        \can_top_level_inst/prescaler_inst/g1_0_0\, D => 
        \can_top_level_inst/prescaler_inst/g0_0_7\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa2_1\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => \adress_c[2]\, B => \adress_c[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[6]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g2_0\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un7_exit_segm_regular_cry_9_0\, 
        B => \can_top_level_inst/prescaler_inst/is_tseg2\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g2_0_Z\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/fixed_reg_d_u\ : 
        CFG4
      generic map(INIT => x"7400")

      port map(A => 
        \can_top_level_inst/can_core_inst/stuff_length[0]\, B => 
        \can_top_level_inst/tx_trigger\, C => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/fixed_reg_q\, 
        D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/reg_q\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/fixed_reg_d\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_0_y3[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_31_2_0_y1[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1271\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[18]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_0[31]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => \timestamp_c[63]\, B => \timestamp_c[31]\, C
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac1_Z\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_0_Z[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[10]\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_data_mux_test_registers_comp_un2_index_sat_1\, 
        C => \adress_c[7]\, D => \adress_c[6]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\);
    
    \timestamp_ibuf[51]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(51), Y => 
        \timestamp_ibuf[51]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[9]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[9]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[9]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[7]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[7]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[7]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m25_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_7[29]\ : 
        CFG4
      generic map(INIT => x"FFF8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac28_Z\, 
        B => \can_top_level_inst/drv_bus[58]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_5_Z[29]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_4_Z[29]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_7_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31[8]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_13_2[18]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/is_bus_off\, B => 
        \can_top_level_inst/rx_read_buff[18]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_833_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_2_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1301\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1302\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_30_1_2_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\);
    
    \data_out_obuf[8]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[8]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[8]/DOUT\, EOUT => \data_out_obuf[8]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[18]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[18]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[12]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[44]\, 
        C => \can_top_level_inst/rec_ident[12]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2257\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_2\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[9]\, B
         => \can_top_level_inst/can_core_inst/crc_15[8]\, C => 
        \can_top_level_inst/can_core_inst/crc_15[9]\, D => 
        \can_top_level_inst/store_data_word[8]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[3]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_15_ok_0_data_tmp[4]\, 
        CC => NET_CC_CONFIG66, P => NET_CC_CONFIG63, Y3 => 
        NET_CC_CONFIG64, Y3A => NET_CC_CONFIG65);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_7_i_RNIVBKF\ : 
        CFG3
      generic map(INIT => x"0D")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_223\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_736_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1186_i\, 
        Y => \can_top_level_inst/dbt_ctrs_en\);
    
    \can_top_level_inst/int_manager_inst/un1_int_i_4\ : CFG4
      generic map(INIT => x"135F")

      port map(A => \can_top_level_inst/int_vector[7]\, B => 
        \can_top_level_inst/int_vector[6]\, C => 
        \can_top_level_inst/int_ena[7]\, D => 
        \can_top_level_inst/int_ena[6]\, Y => 
        \can_top_level_inst/int_manager_inst/un1_int_i_4_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_10\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[10]\, B => 
        \can_top_level_inst/drv_bus[310]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_9\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_10\, 
        CC => NET_CC_CONFIG569, P => NET_CC_CONFIG566, Y3 => 
        NET_CC_CONFIG567, Y3A => NET_CC_CONFIG568);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[11]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_2[1]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[1]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_16_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_12[30]\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[4]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[3]\, C => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_c3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_77_mux\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_shorten_tseg1_after_tseg2_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/un18_sel_tseg1_2230\, 
        C => \can_top_level_inst/prescaler_inst/start_edge\, D
         => \can_top_level_inst/prescaler_inst/h_sync_valid\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[418]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[27]\ : 
        SLE
      port map(D => \data_in_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[232]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[25]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[25]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[14]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[14]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[14]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un7_txtb_parity_error_valid_i\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un7_txtb_parity_error_valid_i_0_Z\, 
        B => \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/txtb_parity_check_valid\, D => 
        \can_top_level_inst/txtb_parity_error_valid_2[6]\, Y => 
        \can_top_level_inst/txtb_parity_error_valid[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[17]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[5]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[5]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2_1[1]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_148\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[1]\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un4_compl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[8]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[25]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2270\, B
         => \can_top_level_inst/rx_buffer_inst/N_4827\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO\ : 
        ARI1_CC
      generic map(INIT => x"45500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[79]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_7\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_FCNET1\, 
        CC => NET_CC_CONFIG2017, P => NET_CC_CONFIG2014, Y3 => 
        NET_CC_CONFIG2015, Y3A => NET_CC_CONFIG2016);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[16]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[16]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[16]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[3]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[18]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2263\, B
         => \can_top_level_inst/rx_buffer_inst/N_4822\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[18]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_16\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[86]\, B => 
        \can_top_level_inst/rec_ident[5]\, C => 
        \can_top_level_inst/drv_bus[119]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_61\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[20]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[20]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[20]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2644\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa14\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe14\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[30]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_segm_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m18_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[183]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[272]\, B => 
        \can_top_level_inst/rec_ident[1]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_0\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_1\, 
        CC => NET_CC_CONFIG1837, P => NET_CC_CONFIG1834, Y3 => 
        NET_CC_CONFIG1835, Y3A => NET_CC_CONFIG1836);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.un3_ack_err_flag\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_flag_Z\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => \can_top_level_inst/can_core_inst/pc_rx_trigger\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_ack_err_flag\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m19_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[18]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_RNIQCF6[0]\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_i[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[6]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_16\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[408]\, B => 
        \can_top_level_inst/stat_bus[71]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_0\, 
        CC => NET_CC_CONFIG2062, P => NET_CC_CONFIG2059, Y3 => 
        NET_CC_CONFIG2060, Y3A => NET_CC_CONFIG2061);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[9]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_8[21]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/drv_bus[259]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_942\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[12]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[274]\, B => 
        \can_top_level_inst/rec_ident[3]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_2\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_3\, 
        CC => NET_CC_CONFIG1845, P => NET_CC_CONFIG1842, Y3 => 
        NET_CC_CONFIG1843, Y3A => NET_CC_CONFIG1844);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[3]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz[3]\ : 
        CFG4
      generic map(INIT => x"F444")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_192\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sync_control_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un12_sync_control_d_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[28]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[1]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[0]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sync_control_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un10_sync_control_d_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_FCNET1\, 
        CO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_FCO\, 
        CC => NET_CC_CONFIG2563, P => NET_CC_CONFIG2560, Y3 => 
        NET_CC_CONFIG2561, Y3A => NET_CC_CONFIG2562);
    
    \I_1/U0_RGB1_RGB5\ : RGB
      port map(A => \I_1/U0_Y\, EN => ADLIB_VCC, Y => 
        \AFLSDF_INV_14\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[21]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[15]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4PNCG[23]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[211]\, D => 
        \can_top_level_inst/stat_bus[243]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_22\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4PNCG_S[23]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_23\, 
        CC => NET_CC_CONFIG848, P => NET_CC_CONFIG845, Y3 => 
        NET_CC_CONFIG846, Y3A => NET_CC_CONFIG847);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_0[3]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[2]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[24]\);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/un2_go_to_off\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/is_bus_off\, Y => 
        \can_top_level_inst/can_core_inst/operation_control_inst/un2_go_to_off_i_0\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy\ : 
        ARI1_CC
      generic map(INIT => x"4EE00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, C => 
        \can_top_level_inst/prescaler_inst/is_tseg2\, D => 
        ADLIB_GND, FCI => ADLIB_VCC, S => OPEN, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Z\, 
        CC => NET_CC_CONFIG2103, P => NET_CC_CONFIG2100, Y3 => 
        NET_CC_CONFIG2101, Y3A => NET_CC_CONFIG2102);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNO_0[8]\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => \can_top_level_inst/drv_bus[425]\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/drv_bus_i[425]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[19]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_7\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_6_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_7_Z\, 
        CC => NET_CC_CONFIG2478, P => NET_CC_CONFIG2475, Y3 => 
        NET_CC_CONFIG2476, Y3A => NET_CC_CONFIG2477);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_i[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_write_pointer[4]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_27\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[298]\, B => 
        \can_top_level_inst/rec_ident[27]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_26\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_27\, 
        CC => NET_CC_CONFIG1941, P => NET_CC_CONFIG1938, Y3 => 
        NET_CC_CONFIG1939, Y3A => NET_CC_CONFIG1940);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNI2OR25[10]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[10]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[9]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[10]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[10]\, 
        CC => NET_CC_CONFIG1718, P => NET_CC_CONFIG1715, Y3 => 
        NET_CC_CONFIG1716, Y3A => NET_CC_CONFIG1717);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[26]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[26]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\);
    
    \data_in_ibuf[21]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(21), Y => \data_in_ibuf[21]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => swr_c, B => \adress_c[7]\, C => \adress_c[6]\, 
        D => \adress_c[5]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_2_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[25]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[13]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram32__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_350\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe32\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_7_sqmuxa_i\ : 
        CFG4
      generic map(INIT => x"FF3B")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_949\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_890_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[240]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask.un7_flip_mask_axbxc2_RNIQJ8K\ : 
        CFG3
      generic map(INIT => x"06")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[4]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[3]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_0[8]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_27\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[27]\, B => 
        \can_top_level_inst/drv_bus[327]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_26\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_27\, 
        CC => NET_CC_CONFIG637, P => NET_CC_CONFIG634, Y3 => 
        NET_CC_CONFIG635, Y3A => NET_CC_CONFIG636);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_8\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[412]\, B => 
        \can_top_level_inst/stat_bus[75]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_3\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_4\, 
        CC => NET_CC_CONFIG2078, P => NET_CC_CONFIG2075, Y3 => 
        NET_CC_CONFIG2076, Y3A => NET_CC_CONFIG2077);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_38_1_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_38_1_0_y0[1]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1198\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1200\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_2\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/tran_brs\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_2_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4_2[2]\ : 
        CFG4
      generic map(INIT => x"C808")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_2_S\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, D
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_2_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_2\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[2]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_1_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_2_Z\, 
        CC => NET_CC_CONFIG1493, P => NET_CC_CONFIG1490, Y3 => 
        NET_CC_CONFIG1491, Y3A => NET_CC_CONFIG1492);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_185\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_183\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_184\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[19]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2708\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2675\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe20_0_a4\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_345\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[126]\, D => 
        \can_top_level_inst/drv_bus[155]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_14_2_0_y0[12]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i\, 
        B => \can_top_level_inst/prescaler_inst/start_edge\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[24]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[24]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[24]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[19]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[19]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_11[27]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac17_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac37_Z\, 
        C => \can_top_level_inst/drv_bus[476]\, D => 
        \can_top_level_inst/drv_bus[298]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_11_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[4]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram100__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_341\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe100\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_ena_reg_proc.un21_ctrl_signal_upd_i_a4_2\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        B => \can_top_level_inst/is_bus_off\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_go_to_suspend_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_931\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1221\);
    
    \timestamp_ibuf[19]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(19), Y => 
        \timestamp_ibuf[19]/YIN\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[13]\ : 
        CFG4
      generic map(INIT => x"F2D0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_10\, 
        B => \can_top_level_inst/rx_buffer_inst/data_selector[0]\, 
        C => \can_top_level_inst/rx_buffer_inst/N_2206\, D => 
        \can_top_level_inst/rx_buffer_inst/N_2291\, Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[28]\);
    
    \data_in_ibuf[10]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(10), Y => \data_in_ibuf[10]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[26]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2650\, Y
         => \can_top_level_inst/memory_registers_inst/N_2715\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe66_0_a4\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[0]\, 
        B => \can_top_level_inst/rx_buffer_inst/un5_ram_write_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[15]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[22]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[3]\, CLK
         => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_9_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_9_FCNET1\, 
        CO => 
        \can_top_level_inst/prescaler_inst/un7_exit_segm_regular_cry_9\, 
        CC => NET_CC_CONFIG1525, P => NET_CC_CONFIG1522, Y3 => 
        NET_CC_CONFIG1523, Y3A => NET_CC_CONFIG1524);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[21]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[12]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[12]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[12]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIGCJC1[1]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[274]\, C => 
        \can_top_level_inst/stat_bus[258]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_0\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIGCJC1_S[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_1\, 
        CC => NET_CC_CONFIG2572, P => NET_CC_CONFIG2569, Y3 => 
        NET_CC_CONFIG2570, Y3A => NET_CC_CONFIG2571);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_0_y0[15]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_716\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_719\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_31_2_0_y1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[26]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_type_q_RNO[1]\ : 
        CFG3
      generic map(INIT => x"F1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_crc_err_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un1_err_type_d_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un6_form_err_i_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_1170_i\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[174]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un7_rx_err_ctr_declto8_i_a2_0_a2_RNITG6E_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO2314, CO => OPEN, P(0) => 
        NET_CC_CONFIG2323, P(1) => NET_CC_CONFIG2327, P(2) => 
        NET_CC_CONFIG2331, P(3) => NET_CC_CONFIG2335, P(4) => 
        NET_CC_CONFIG2339, P(5) => ADLIB_VCC, P(6) => ADLIB_VCC, 
        P(7) => ADLIB_VCC, P(8) => ADLIB_VCC, P(9) => ADLIB_VCC, 
        P(10) => ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => 
        NET_CC_CONFIG2324, Y3(1) => NET_CC_CONFIG2328, Y3(2) => 
        NET_CC_CONFIG2332, Y3(3) => NET_CC_CONFIG2336, Y3(4) => 
        NET_CC_CONFIG2340, Y3(5) => ADLIB_VCC, Y3(6) => ADLIB_VCC, 
        Y3(7) => ADLIB_VCC, Y3(8) => ADLIB_VCC, Y3(9) => 
        ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, 
        Y3A(0) => NET_CC_CONFIG2325, Y3A(1) => NET_CC_CONFIG2329, 
        Y3A(2) => NET_CC_CONFIG2333, Y3A(3) => NET_CC_CONFIG2337, 
        Y3A(4) => NET_CC_CONFIG2341, Y3A(5) => ADLIB_VCC, Y3A(6)
         => ADLIB_VCC, Y3A(7) => ADLIB_VCC, Y3A(8) => ADLIB_VCC, 
        Y3A(9) => ADLIB_VCC, Y3A(10) => ADLIB_VCC, Y3A(11) => 
        ADLIB_VCC, CC(0) => NET_CC_CONFIG2326, CC(1) => 
        NET_CC_CONFIG2330, CC(2) => NET_CC_CONFIG2334, CC(3) => 
        NET_CC_CONFIG2338, CC(4) => NET_CC_CONFIG2342, CC(5) => 
        nc172, CC(6) => nc229, CC(7) => nc277, CC(8) => nc97, 
        CC(9) => nc161, CC(10) => nc31, CC(11) => nc340);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un4_tst_ena\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[19]\, B
         => 
        \can_top_level_inst/test_registers_out.tst_control[0]\, C
         => 
        \can_top_level_inst/memory_registers_inst_test_registers_in_tst_rdata_sn_N_8\, 
        D => \can_top_level_inst/test_registers_out.tst_dest[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[17]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[16]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[25]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[24]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1123\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1117\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIC1TB7[7]\ : 
        ARI1_CC
      generic map(INIT => x"5D827")

      port map(A => \timestamp_c[39]\, B => 
        \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[7]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[7]\, 
        FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_6\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_7\, 
        CC => NET_CC_CONFIG955, P => NET_CC_CONFIG952, Y3 => 
        NET_CC_CONFIG953, Y3A => NET_CC_CONFIG954);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_3[24]\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[138]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1051\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[16]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_3L3\ : 
        CFG4
      generic map(INIT => x"C4CC")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_8_x_Z\, 
        B => \can_top_level_inst/prescaler_inst/is_tseg1\, C => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_sx_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_6_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[7]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[0]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[7]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[7]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[7]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q_RNO[2]\ : 
        CFG4
      generic map(INIT => x"A5CC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[2]\, 
        B => \can_top_level_inst/rx_buffer_inst_RAM_data_out[13]\, 
        C => \can_top_level_inst/rx_buffer_inst/rx_mof_1\, D => 
        \can_top_level_inst/rx_mof\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_3926_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[21]\ : 
        CFG4
      generic map(INIT => x"2021")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1059\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_802_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram120_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe120\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram120_0\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[5]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2694\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2661\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_12[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_4[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram50_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram51_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y1_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[4]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_139\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_170_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_10_1[10]\ : 
        CFG4
      generic map(INIT => x"00CA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_Z\, 
        B => \can_top_level_inst/drv_bus[186]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1516_1\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIQV0C5[6]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[194]\, D => 
        \can_top_level_inst/stat_bus[226]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_5\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIQV0C5_S[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_6\, 
        CC => NET_CC_CONFIG780, P => NET_CC_CONFIG777, Y3 => 
        NET_CC_CONFIG778, Y3A => NET_CC_CONFIG779);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_12[31]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[1]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[2]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_12_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[13]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0_RNIRBDSR2[2]\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => \can_top_level_inst/drv_bus[48]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_51\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_1\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_2\, 
        CC => NET_CC_CONFIG2708, P => NET_CC_CONFIG2705, Y3 => 
        NET_CC_CONFIG2706, Y3A => NET_CC_CONFIG2707);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0_RNIDL04A4[4]\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => \can_top_level_inst/drv_bus[50]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_53\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_3\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_4\, 
        CC => NET_CC_CONFIG2716, P => NET_CC_CONFIG2713, Y3 => 
        NET_CC_CONFIG2714, Y3A => NET_CC_CONFIG2715);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe20\);
    
    \adress_ibuf[11]/U_IOPAD\ : IOPAD_IN
      port map(PAD => adress(11), Y => \adress_ibuf[11]/YIN\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_o2_0[0]\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[6]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[5]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_111\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_112\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[7]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[6]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[7]\);
    
    \can_top_level_inst/memory_registers_inst/drv_bus_57[352]\ : 
        CFG4
      generic map(INIT => x"ACA0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/rx_data_read\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_command_reg_comp_wr_select[0]\, 
        C => \can_top_level_inst/memory_registers_inst/mode[9]\, 
        D => \data_in_c[1]\, Y => 
        \can_top_level_inst/drv_bus[352]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe48_0_a4\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_343\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[207]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_RNIK7O93[7]\ : 
        CFG4
      generic map(INIT => x"3339")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[0]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[7]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/g1_4_0\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/g1_0_0\, 
        Y => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_a_4[7]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_d_0\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[1]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_d_0_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m4[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_957\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1038\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_802_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram27_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe27\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram27_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[16]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[26]\ : 
        SLE
      port map(D => \timestamp_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_17\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[288]\, B => 
        \can_top_level_inst/rec_ident[17]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_16\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_17\, 
        CC => NET_CC_CONFIG1901, P => NET_CC_CONFIG1898, Y3 => 
        NET_CC_CONFIG1899, Y3A => NET_CC_CONFIG1900);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_38_1_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_38_1_0_y0[19]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_892\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_894\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[23]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24[23]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram108__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_354\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe108\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m36_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_m2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"C040")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_m2_0_a2_1_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst_priority_decoder_inst_un1_m2_0_a2_0_1\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_i1_mux\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_36_i\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dff_ena_reg/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/destuff_enable\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/reg_q_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[10]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[10]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[10]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => \adress_c[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[18]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/drv_bus[22]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_5_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_6_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_6_FCNET1\, 
        CC => NET_CC_CONFIG2171, P => NET_CC_CONFIG2168, Y3 => 
        NET_CC_CONFIG2169, Y3A => NET_CC_CONFIG2170);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[19]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[19]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_1[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[27]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz[3]\ : 
        CFG4
      generic map(INIT => x"F4FC")

      port map(A => \can_top_level_inst/txtb_hw_cmd.unlock\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_139\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[3]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_mem_free[6]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_5_Z\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_6_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_6_Z\, 
        CC => NET_CC_CONFIG2442, P => NET_CC_CONFIG2439, Y3 => 
        NET_CC_CONFIG2440, Y3A => NET_CC_CONFIG2441);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select[2]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_0\, 
        D => \sbe_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/int_mask_load\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[4]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/int_mask_load_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[21]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2479\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate_3_0_.m9\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[2]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lt4_1\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_3_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_28_mux\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_write_address[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[0]\, 
        C => \can_top_level_inst/rx_buffer_inst/N_48\, Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[24]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un4_compl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_12_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/un5_go_to_failed\ : 
        CFG4
      generic map(INIT => x"FFEC")

      port map(A => \can_top_level_inst/is_bus_off\, B => 
        \can_top_level_inst/drv_bus[513]\, C => 
        \can_top_level_inst/txt_buf_failed_bof\, D => 
        \can_top_level_inst/drv_bus[470]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_txtb_clk_en_d_NE_0\ : 
        CFG4
      generic map(INIT => x"7BDE")

      port map(A => \can_top_level_inst/txtb_ptr[4]\, B => 
        \can_top_level_inst/txtb_ptr[3]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[4]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_138\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_txtb_clk_en_d_NE_0_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[122]\, D => 
        \can_top_level_inst/drv_bus[246]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[20]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[4]\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q_RNO[4]\ : 
        CFG4
      generic map(INIT => x"C693")

      port map(A => \can_top_level_inst/rx_mof\, B => 
        \can_top_level_inst/rx_buffer_inst/N_2835\, C => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[4]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[15]\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_3928_i\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/is_pexs\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_875\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_drv_cpexs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[385]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIDIKC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[22]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2646\, Y
         => \can_top_level_inst/memory_registers_inst/N_2711\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_4\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[7]\, B
         => \can_top_level_inst/can_core_inst/crc_17[6]\, C => 
        \can_top_level_inst/can_core_inst/crc_17[7]\, D => 
        \can_top_level_inst/store_data_word[6]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[2]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[3]\, 
        CC => NET_CC_CONFIG2539, P => NET_CC_CONFIG2536, Y3 => 
        NET_CC_CONFIG2537, Y3A => NET_CC_CONFIG2538);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa7\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa7\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[9]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[11]\ : 
        SLE
      port map(D => \timestamp_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[11]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[11]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[11]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_11[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram16_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram17_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_4[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[22]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[11]\, 
        B => \can_top_level_inst/tran_word[14]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[6]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[48]\ : 
        SLE
      port map(D => \timestamp_c[48]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[48]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[18]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[18]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[18]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[29]\ : 
        SLE
      port map(D => \data_in_c[29]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[53]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/frame_valid_com_clear_0_a2_2\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, B
         => \can_top_level_inst/txtb_hw_cmd.unlock\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/N_158\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d[8]\ : 
        CFG4
      generic map(INIT => x"D888")

      port map(A => \can_top_level_inst/drv_bus[426]\, B => 
        \can_top_level_inst/drv_bus[424]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_m0_Z[8]\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNICA923_S[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[191]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[26]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m25_1_1\ : 
        CFG3
      generic map(INIT => x"51")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[41]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m25_1_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_2_2[13]\ : 
        CFG4
      generic map(INIT => x"225F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[284]\, C => 
        \can_top_level_inst/drv_bus[251]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_Z[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_2_1[13]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNINS46O[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[43]\, B => 
        \can_top_level_inst/tran_word[11]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_10\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_11\, 
        CC => NET_CC_CONFIG971, P => NET_CC_CONFIG968, Y3 => 
        NET_CC_CONFIG969, Y3A => NET_CC_CONFIG970);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[50]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_24[0]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[23]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_24\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[21]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m25_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[11]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[11]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[11]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram84_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe84\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram84_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[8]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_2[2]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_3_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_16_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8_RNIEKKVE6\ : 
        ARI1_CC
      generic map(INIT => x"49CCC")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[7]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_6\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_7\, 
        CC => NET_CC_CONFIG1441, P => NET_CC_CONFIG1438, Y3 => 
        NET_CC_CONFIG1439, Y3A => NET_CC_CONFIG1440);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[26]\ : 
        CFG4
      generic map(INIT => x"E0A0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_17_Z[26]\, 
        B => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[24]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m_0[890]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i[2]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[2]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[26]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_0\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => \can_top_level_inst/drv_bus[16]\, B => 
        \can_top_level_inst/drv_bus[23]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => ADLIB_VCC, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_0_Z\, 
        CC => NET_CC_CONFIG2147, P => NET_CC_CONFIG2144, Y3 => 
        NET_CC_CONFIG2145, Y3A => NET_CC_CONFIG2146);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m31_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m25_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0[4]\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        C => \can_top_level_inst/tran_frame_test.tprm[1]\, D => 
        \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_i_3_tz\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_data_nbs_prev_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[17]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[15]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_i_3_tz_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\);
    
    \adress_ibuf[11]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \adress_c[11]\, E => ADLIB_GND, YIN => 
        \adress_ibuf[11]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[4]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram57_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe57\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram57_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_esi\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_esi\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_esi\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_23\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un42_parity_mismatch_23_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.1.l2_prio_12_m_3_1_wmux_0\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_12_m_3_1_0_y0\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_26_i\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_121\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_123\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_12\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram95__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_351\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe95\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_2[14]\ : 
        CFG4
      generic map(INIT => x"434F")

      port map(A => \can_top_level_inst/stat_bus[271]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_1_Z[14]\, 
        D => \can_top_level_inst/drv_bus[30]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\);
    
    \data_out_obuf[17]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[17]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[17]/DOUT\, EOUT => 
        \data_out_obuf[17]/EOUT\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q_1078\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2833\, B
         => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[3]\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2835\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[27]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[27]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_frame_no_sof_d_i_1\ : 
        CFG4
      generic map(INIT => x"FDFC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_go_to_suspend_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_frame_no_sof_d_i_1_Z\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIG7IO[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO751, P(0) => 
        NET_CC_CONFIG753, P(1) => NET_CC_CONFIG757, P(2) => 
        NET_CC_CONFIG761, P(3) => NET_CC_CONFIG765, P(4) => 
        NET_CC_CONFIG769, P(5) => NET_CC_CONFIG773, P(6) => 
        NET_CC_CONFIG777, P(7) => NET_CC_CONFIG781, P(8) => 
        NET_CC_CONFIG785, P(9) => NET_CC_CONFIG789, P(10) => 
        NET_CC_CONFIG793, P(11) => NET_CC_CONFIG797, Y3(0) => 
        NET_CC_CONFIG754, Y3(1) => NET_CC_CONFIG758, Y3(2) => 
        NET_CC_CONFIG762, Y3(3) => NET_CC_CONFIG766, Y3(4) => 
        NET_CC_CONFIG770, Y3(5) => NET_CC_CONFIG774, Y3(6) => 
        NET_CC_CONFIG778, Y3(7) => NET_CC_CONFIG782, Y3(8) => 
        NET_CC_CONFIG786, Y3(9) => NET_CC_CONFIG790, Y3(10) => 
        NET_CC_CONFIG794, Y3(11) => NET_CC_CONFIG798, Y3A(0) => 
        NET_CC_CONFIG755, Y3A(1) => NET_CC_CONFIG759, Y3A(2) => 
        NET_CC_CONFIG763, Y3A(3) => NET_CC_CONFIG767, Y3A(4) => 
        NET_CC_CONFIG771, Y3A(5) => NET_CC_CONFIG775, Y3A(6) => 
        NET_CC_CONFIG779, Y3A(7) => NET_CC_CONFIG783, Y3A(8) => 
        NET_CC_CONFIG787, Y3A(9) => NET_CC_CONFIG791, Y3A(10) => 
        NET_CC_CONFIG795, Y3A(11) => NET_CC_CONFIG799, CC(0) => 
        NET_CC_CONFIG756, CC(1) => NET_CC_CONFIG760, CC(2) => 
        NET_CC_CONFIG764, CC(3) => NET_CC_CONFIG768, CC(4) => 
        NET_CC_CONFIG772, CC(5) => NET_CC_CONFIG776, CC(6) => 
        NET_CC_CONFIG780, CC(7) => NET_CC_CONFIG784, CC(8) => 
        NET_CC_CONFIG788, CC(9) => NET_CC_CONFIG792, CC(10) => 
        NET_CC_CONFIG796, CC(11) => NET_CC_CONFIG800);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[31]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_24_1\ : 
        CFG3
      generic map(INIT => x"A2")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_26\, 
        C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[5]\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_24_1_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_2[29]\ : 
        CFG4
      generic map(INIT => x"F909")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_17_i_x4_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_10\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        D => \can_top_level_inst/tran_word[5]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_15\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_10[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y21[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y9[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[4]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_wmux_10_Y[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un4_compl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[18]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[14]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[12]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[18]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO[5]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_41\, 
        B => 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_2[7]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[22]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un17_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_1_0[2]\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \adress_c[4]\, B => \adress_c[7]\, C => 
        \adress_c[6]\, D => swr_c, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[6]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[2]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[4]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[1]\, 
        D => \can_top_level_inst/tran_word[28]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[42]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_27\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[15]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_15_Z\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_14_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_27_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[7]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[7]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[7]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[12]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_raw_0[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI0ALE1_S[3]\, 
        B => \can_top_level_inst/drv_bus[378]\, C => 
        \can_top_level_inst/drv_bus[374]\, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_39\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_h_sync_valid_2_RNIOL0KN5\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[6]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_h_sync_valid_2_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[6]\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_5\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_6\, 
        CC => NET_CC_CONFIG1437, P => NET_CC_CONFIG1434, Y3 => 
        NET_CC_CONFIG1435, Y3A => NET_CC_CONFIG1436);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_9[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[302]\, C => 
        \can_top_level_inst/drv_bus[178]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1211\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select[2]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => swr_c, B => \sbe_c[2]\, C => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_0\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[13]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2258\, B
         => \can_top_level_inst/rx_buffer_inst/N_4819\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2291\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[10]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[302]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_57\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[19]\, B => 
        \timestamp_c[50]\, C => \timestamp_c[51]\, D => 
        \can_top_level_inst/tran_word[18]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[8]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[9]\, 
        CC => NET_CC_CONFIG2289, P => NET_CC_CONFIG2286, Y3 => 
        NET_CC_CONFIG2287, Y3A => NET_CC_CONFIG2288);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_segm_counter_ce\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end\, B => 
        \can_top_level_inst/prescaler_inst_tq_edge_nbt\, Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_segm_counter_ce_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm_0[6]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[6]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[6]\);
    
    \data_in_ibuf[23]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[23]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[23]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[25]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[25]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[25]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_19\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_19_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_3\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txtb_parity_error_valid[1]\, B => 
        \can_top_level_inst/txtb_parity_error_valid[7]\, C => 
        \can_top_level_inst/txtb_parity_error_valid[3]\, D => 
        \can_top_level_inst/txtb_parity_error_valid[5]\, Y => 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_3_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[16]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[16]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[16]\);
    
    \data_out_obuf[5]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[5]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[5]/DOUT\, EOUT => \data_out_obuf[5]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o2[0]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[3]\, B
         => \can_top_level_inst/txtb_sw_cmd.set_ety\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_151\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[7]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_15[6]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_y0_0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_0_y3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[7]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txtb_parity_error_valid_1[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_m1_e_1\, 
        D => \can_top_level_inst/txtb_parity_check_valid\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_168\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_brs_com\ : SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_brs_dbl_buf_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/tran_brs\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIMSM92[2]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[190]\, D => 
        \can_top_level_inst/stat_bus[222]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_1\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIMSM92_S[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_2\, 
        CC => NET_CC_CONFIG764, P => NET_CC_CONFIG761, Y3 => 
        NET_CC_CONFIG762, Y3A => NET_CC_CONFIG763);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8[0]\ : 
        CFG4
      generic map(INIT => x"7B48")

      port map(A => \can_top_level_inst/rx_mem_free[0]\, B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_RNIJ6NH_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_9[28]\ : 
        CFG4
      generic map(INIT => x"FFEA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_2_Z[28]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5_Z\, 
        C => \can_top_level_inst/stat_bus[216]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_4_Z[28]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_9_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_RNIK5FB\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_s1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\);
    
    \timestamp_ibuf[2]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[2]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[2]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_124_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[24]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[24]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[5]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[5]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[5]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[5]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI9BRM7_S[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[267]\);
    
    \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_2\ : 
        CFG4
      generic map(INIT => x"0010")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/load_ffmt_w_addr\, 
        B => \can_top_level_inst/tx_arbitrator_inst/buffer_md_w\, 
        C => \can_top_level_inst/tx_arbitrator_inst/N_94\, D => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, Y
         => 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_2_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_0[4]\ : 
        CFG4
      generic map(INIT => x"22AE")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1_0_Z[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_159\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_0_1_Z[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_0_Z[4]\);
    
    AFLSDF_INV_0 : INV_BA
      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1317_i\, 
        Y => \AFLSDF_INV_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[2]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[17]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[17]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_28\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[92]\, B => 
        \can_top_level_inst/rec_ident[11]\, C => 
        \can_top_level_inst/drv_bus[125]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_51\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[21]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[226]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[2]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_21_RNI4JJH\ : 
        CFG4
      generic map(INIT => x"9669")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_28_Z\, 
        Y => \can_top_level_inst/N_3901_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/commit_overrun_abort_0_sqmuxa_i\ : 
        CFG2
      generic map(INIT => x"D")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => \can_top_level_inst/rx_buffer_inst/data_overrun_i_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/commit_overrun_abort_0_sqmuxa_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/fsm_wait_state_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_100_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/stuff_enable_1\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \AFLSDF_INV_15\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_cZ[1]\ : 
        CFG4
      generic map(INIT => x"DCCC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_4[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[7]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        B => \can_top_level_inst/txtb_parity_error_valid[7]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[381]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[246]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[20]\);
    
    \adress_ibuf[4]/U_IOPAD\ : IOPAD_IN
      port map(PAD => adress(4), Y => \adress_ibuf[4]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram121_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe121\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram121_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_RNILLG23[1]\ : 
        ARI1_CC
      generic map(INIT => x"48800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy_Y[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[1]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[0]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[1]\, 
        CC => NET_CC_CONFIG657, P => NET_CC_CONFIG654, Y3 => 
        NET_CC_CONFIG655, Y3A => NET_CC_CONFIG656);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_RNO_2[28]\ : 
        CFG4
      generic map(INIT => x"1B0A")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_21\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_24\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac22\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_0_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac22_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/first_err_delim_d_i_o2\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[7]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[8]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_9\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[10]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/curr_state_s0_0_a3\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \can_top_level_inst/is_bus_off\, B => 
        \can_top_level_inst/stat_bus[108]\, Y => 
        \can_top_level_inst/stat_bus[109]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI817P2_S[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[276]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIISMS\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_3\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[414]\, B => 
        \can_top_level_inst/stat_bus[87]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_5\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_6\, 
        CC => NET_CC_CONFIG195, P => NET_CC_CONFIG192, Y3 => 
        NET_CC_CONFIG193, Y3A => NET_CC_CONFIG194);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_sw_cmd_index[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_RNIKFUH2[7]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_6\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_7\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_7\, 
        CC => NET_CC_CONFIG2207, P => NET_CC_CONFIG2204, Y3 => 
        NET_CC_CONFIG2205, Y3A => NET_CC_CONFIG2206);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[4]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2530\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_23\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_23_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m25_1\ : 
        CFG4
      generic map(INIT => x"99AB")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => \can_top_level_inst/stat_bus[88]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m25_1_1_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m25_1_Z\);
    
    \data_out_obuf[6]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[6]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[6]/DOUT\, EOUT => \data_out_obuf[6]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[30]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_1\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => swr_c, B => scs_c, C => \adress_c[9]\, D => 
        \adress_c[8]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[2]\ : 
        CFG4
      generic map(INIT => x"20A0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[2]\, 
        B => \can_top_level_inst/txtb_hw_cmd.unlock\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_175\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_154\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[7]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[7]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIS6TTA[28]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[28]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[28]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_27\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_28\, 
        CC => NET_CC_CONFIG1389, P => NET_CC_CONFIG1386, Y3 => 
        NET_CC_CONFIG1387, Y3A => NET_CC_CONFIG1388);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[14]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[14]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[14]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_38_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1521\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1536\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_38_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[18]\);
    
    \data_in_ibuf[26]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[26]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[26]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_RNO[6]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\, C
         => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[6]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[5]\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[6]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG1119, P => 
        NET_CC_CONFIG1116, Y3 => NET_CC_CONFIG1117, Y3A => 
        NET_CC_CONFIG1118);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNISMQGE[20]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[208]\, D => 
        \can_top_level_inst/stat_bus[240]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_19\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNISMQGE_S[20]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_20\, 
        CC => NET_CC_CONFIG836, P => NET_CC_CONFIG833, Y3 => 
        NET_CC_CONFIG834, Y3A => NET_CC_CONFIG835);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_0_y0[8]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[184]\, D => 
        \can_top_level_inst/drv_bus[308]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_22_2_0_y1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[11]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[2]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[2]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[2]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[5]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[5]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[5]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4265\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m19_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2_3[0]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1320\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o3_0[1]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_192\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_147\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_844_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[25]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[25]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[3]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[3]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[31]\ : 
        CFG4
      generic map(INIT => x"5040")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/data_selector[0]\, B
         => \can_top_level_inst/rx_buffer_inst/N_2309_1\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_10\, 
        D => \can_top_level_inst/rx_buffer_inst/N_2309_2\, Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe10\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI2V31C[16]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[204]\, D => 
        \can_top_level_inst/stat_bus[236]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_15\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI2V31C_S[16]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_16\, 
        CC => NET_CC_CONFIG820, P => NET_CC_CONFIG817, Y3 => 
        NET_CC_CONFIG818, Y3A => NET_CC_CONFIG819);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_ena_reg_proc.destuff_enable_1_iv\ : 
        CFG3
      generic map(INIT => x"5D")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_892_i\, 
        B => \can_top_level_inst/can_core_inst/destuff_enable\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_920_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_enable_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[14]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIALJ4K_S[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[249]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI46KGJ[28]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[216]\, D => 
        \can_top_level_inst/stat_bus[248]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_27\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI46KGJ_S[28]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_28\, 
        CC => NET_CC_CONFIG868, P => NET_CC_CONFIG865, Y3 => 
        NET_CC_CONFIG866, Y3A => NET_CC_CONFIG867);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[29]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un7_rx_err_ctr_declto8_i_a2_0_a2_RNITG6E_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO2314, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => 
        ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_VCC, P(6) => 
        ADLIB_VCC, P(7) => ADLIB_VCC, P(8) => ADLIB_VCC, P(9) => 
        ADLIB_GND, P(10) => NET_CC_CONFIG2315, P(11) => 
        NET_CC_CONFIG2319, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => ADLIB_VCC, Y3(3) => ADLIB_VCC, Y3(4) => 
        ADLIB_VCC, Y3(5) => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7)
         => ADLIB_VCC, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => NET_CC_CONFIG2316, Y3(11) => NET_CC_CONFIG2320, 
        Y3A(0) => ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => 
        ADLIB_VCC, Y3A(3) => ADLIB_VCC, Y3A(4) => ADLIB_VCC, 
        Y3A(5) => ADLIB_VCC, Y3A(6) => ADLIB_VCC, Y3A(7) => 
        ADLIB_VCC, Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, 
        Y3A(10) => NET_CC_CONFIG2317, Y3A(11) => 
        NET_CC_CONFIG2321, CC(0) => nc295, CC(1) => nc154, CC(2)
         => nc50, CC(3) => nc260, CC(4) => nc239, CC(5) => nc353, 
        CC(6) => nc348, CC(7) => nc142, CC(8) => nc320, CC(9) => 
        nc344, CC(10) => NET_CC_CONFIG2318, CC(11) => 
        NET_CC_CONFIG2322);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram38_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe38\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram38_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa9\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe9\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[19]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[5]\ : 
        CFG3
      generic map(INIT => x"D0")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[6]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_163\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_s_14\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[14]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_13_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_s_14_S\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG1270, P => 
        NET_CC_CONFIG1267, Y3 => NET_CC_CONFIG1268, Y3A => 
        NET_CC_CONFIG1269);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_38_1_wmux_RNO_0[22]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_2_0_Z[22]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_2_Z[22]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1002\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[28]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2486\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1367_i\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_38_1_0_wmux_RNO[21]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_2_0_Z[21]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_2_Z[21]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_949\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[22]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[20]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2265\, B
         => \can_top_level_inst/rx_buffer_inst/N_4823\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[20]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_RNO[2]\ : 
        CFG4
      generic map(INIT => x"78F0")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[2]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[1]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_3[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_5\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => \can_top_level_inst/drv_bus[56]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_1\, 
        C => \can_top_level_inst/drv_bus[60]\, D => 
        \can_top_level_inst/drv_bus[59]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_sjw_mt_zero_0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[26]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[26]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[14]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15[14]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry[6]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[5]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[6]\, 
        CC => NET_CC_CONFIG316, P => NET_CC_CONFIG313, Y3 => 
        NET_CC_CONFIG314, Y3A => NET_CC_CONFIG315);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_38_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_797\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_812\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_38_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_28[8]\ : 
        CFG4
      generic map(INIT => x"C480")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/stat_bus[2]\, D => 
        \can_top_level_inst/rx_read_buff[8]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1458\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_12\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[410]\, B => 
        \can_top_level_inst/stat_bus[73]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_1\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_2\, 
        CC => NET_CC_CONFIG2070, P => NET_CC_CONFIG2067, Y3 => 
        NET_CC_CONFIG2068, Y3A => NET_CC_CONFIG2069);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_shorten_tseg1_after_tseg2_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        C => \can_top_level_inst/prescaler_inst/h_sync_valid\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIG7IO[0]_CC_2\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO752, CO => OPEN, P(0) => 
        NET_CC_CONFIG849, P(1) => NET_CC_CONFIG853, P(2) => 
        NET_CC_CONFIG857, P(3) => NET_CC_CONFIG861, P(4) => 
        NET_CC_CONFIG865, P(5) => NET_CC_CONFIG869, P(6) => 
        NET_CC_CONFIG873, P(7) => NET_CC_CONFIG877, P(8) => 
        ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11)
         => ADLIB_VCC, Y3(0) => NET_CC_CONFIG850, Y3(1) => 
        NET_CC_CONFIG854, Y3(2) => NET_CC_CONFIG858, Y3(3) => 
        NET_CC_CONFIG862, Y3(4) => NET_CC_CONFIG866, Y3(5) => 
        NET_CC_CONFIG870, Y3(6) => NET_CC_CONFIG874, Y3(7) => 
        NET_CC_CONFIG878, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG851, Y3A(1) => NET_CC_CONFIG855, Y3A(2) => 
        NET_CC_CONFIG859, Y3A(3) => NET_CC_CONFIG863, Y3A(4) => 
        NET_CC_CONFIG867, Y3A(5) => NET_CC_CONFIG871, Y3A(6) => 
        NET_CC_CONFIG875, Y3A(7) => NET_CC_CONFIG879, Y3A(8) => 
        ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => ADLIB_VCC, 
        Y3A(11) => ADLIB_VCC, CC(0) => NET_CC_CONFIG852, CC(1)
         => NET_CC_CONFIG856, CC(2) => NET_CC_CONFIG860, CC(3)
         => NET_CC_CONFIG864, CC(4) => NET_CC_CONFIG868, CC(5)
         => NET_CC_CONFIG872, CC(6) => NET_CC_CONFIG876, CC(7)
         => NET_CC_CONFIG880, CC(8) => nc315, CC(9) => nc247, 
        CC(10) => nc94, CC(11) => nc197);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNI6I8967[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_port_b_address[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/N_4178\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/N_4174\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity\);
    
    \timestamp_ibuf[5]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[5]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[5]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m36_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[4]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[20]\);
    
    \can_top_level_inst/memory_registers_inst/tx_parity_error\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/command_i[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_i_Z\, 
        ALn => res_n_out_c, ADn => ADLIB_VCC, SLn => ADLIB_VCC, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un6_ctrl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.7.int_module_inst/int_ena_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.7.int_module_inst/un1_int_ena_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_ena[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[23]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_15[28]\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_8_Z[28]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_12_Z[28]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_9_Z[28]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_15_Z[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry[7]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[6]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[7]\, 
        CC => NET_CC_CONFIG320, P => NET_CC_CONFIG317, Y3 => 
        NET_CC_CONFIG318, Y3A => NET_CC_CONFIG319);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI8UQ54_S[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[262]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_0_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60[11]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_RNO[30]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2686\, D
         => \can_top_level_inst/memory_registers_inst/N_2719\, Y
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[16]\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_data_mux_test_registers_comp_un2_index_sat_1\, 
        C => \adress_c[7]\, D => \adress_c[6]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[5]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6[5]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[30]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un1_data_halt_d\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/reg_q\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un1_data_halt_d_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe11\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[13]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[12]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[23]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[23]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[23]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\);
    
    
        \can_top_level_inst/bus_sampling_inst/data_edge_detector_inst/un1_tx_edge_i\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => can_tx_c, B => 
        \can_top_level_inst/bus_sampling_inst/tx_data_prev\, Y
         => \can_top_level_inst/bus_sampling_inst/edge_tx_valid\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[17]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[17]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[13]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[13]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_ac0_11_RNI3BUT91\ : 
        ARI1_CC
      generic map(INIT => x"40200")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c7\, 
        C => \can_top_level_inst/stat_bus[88]\, D => 
        \can_top_level_inst/stat_bus[89]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_8\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_i\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG123, P => 
        NET_CC_CONFIG120, Y3 => NET_CC_CONFIG121, Y3A => 
        NET_CC_CONFIG122);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[19]\ : 
        CFG4
      generic map(INIT => x"0078")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[8]\, 
        B => \can_top_level_inst/tran_frame_test.fcrc\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[8]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_909\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[19]\);
    
    \data_in_ibuf[8]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[8]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[8]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_0[11]\ : 
        CFG4
      generic map(INIT => x"8200")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc1\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_c3\, 
        C => \can_top_level_inst/tran_frame_test.tprm[3]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_0_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        D => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/N_4114\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[15]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe10\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa12\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0_1\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa12\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/command_reg_comp/bit_gen.5.reg_present_gen.reg_autoclear_gen.reg_value_r_10[5]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/access_active_1[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        D => \data_in_c[5]\, Y => 
        \can_top_level_inst/drv_bus[473]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_6_RNI5T1P4\ : 
        ARI1_CC
      generic map(INIT => x"5E44E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_sn_N_2_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_6_S\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_Z[0]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_5\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2378\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_6\, 
        CC => NET_CC_CONFIG1977, P => NET_CC_CONFIG1974, Y3 => 
        NET_CC_CONFIG1975, Y3A => NET_CC_CONFIG1976);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => \adress_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_0\, 
        D => \sbe_c[0]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_75\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[8]\, B => 
        \can_top_level_inst/drv_bus[213]\, C => 
        \can_top_level_inst/drv_bus[246]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_11\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[3]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_data_tmp[4]\, 
        CC => NET_CC_CONFIG479, P => NET_CC_CONFIG476, Y3 => 
        NET_CC_CONFIG477, Y3A => NET_CC_CONFIG478);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram39_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe39\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram39_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[5]\ : 
        CFG4
      generic map(INIT => x"FF08")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_170_1\, 
        B => \can_top_level_inst/txtb_hw_cmd.failed\, C => 
        \can_top_level_inst/txtb_parity_error_valid[1]\, D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i\ : 
        CFG4
      generic map(INIT => x"6660")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[10]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_one\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_eight\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram38_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram39_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_6_i\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_9[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_717_i\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_37_1[11]\ : 
        CFG3
      generic map(INIT => x"A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \can_top_level_inst/stat_bus[199]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_37_1_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_i[17]\ : 
        CFG4
      generic map(INIT => x"BBBE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_909\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1214\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[6]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_2_Z[6]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1186\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[28]\ : 
        SLE
      port map(D => \data_in_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.1.l2_prio_0_1.CO2\ : 
        CFG3
      generic map(INIT => x"D4")

      port map(A => \can_top_level_inst/txtb_prorities_4[2]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_5\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_121\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_10\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[403]\, B => 
        \can_top_level_inst/stat_bus[84]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_2\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_3\, 
        CC => NET_CC_CONFIG1180, P => NET_CC_CONFIG1177, Y3 => 
        NET_CC_CONFIG1178, Y3A => NET_CC_CONFIG1179);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_28\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_23_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_16_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/un30_parity_mismatch_28_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[25]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[25]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[25]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5_1\, 
        B => \sbe_c[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/NN_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry[6]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[5]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[6]\, 
        CC => NET_CC_CONFIG1477, P => NET_CC_CONFIG1474, Y3 => 
        NET_CC_CONFIG1475, Y3A => NET_CC_CONFIG1476);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/tx_trig_req_0_a2_0\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        B => \can_top_level_inst/prescaler_inst/current_state[1]\, 
        Y => \can_top_level_inst/prescaler_inst/N_27\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test_dbl_buf.tprm[4]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[12]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/load_ffmt_w_addr\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns[2]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_38_1_0_wmux_RNO[23]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_0_Z[23]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_2_Z[23]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1025\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[7]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2696\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2663\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retransmitt_counter_inst/retr_ctr_d_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[18]\ : 
        CFG4
      generic map(INIT => x"B830")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_Z[18]\, 
        D => \can_top_level_inst/tran_word[10]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0[30]\ : 
        CFG4
      generic map(INIT => x"F5C5")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[7]\, 
        B => \can_top_level_inst/drv_bus[513]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => \can_top_level_inst/stat_bus[109]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[4]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[7]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[15]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un4_data_out_d\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => \can_top_level_inst/tx_trigger\, B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un4_data_out_d_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO_0[25]\ : 
        CFG4
      generic map(INIT => x"530F")

      port map(A => \can_top_level_inst/stat_bus[213]\, B => 
        \can_top_level_inst/drv_bus[374]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_37_2_1_1[25]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_37_2_1_2[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[17]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segm_end_req_capt_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segment_end_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_req_capt_ce\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segm_end_req_capt_q_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_reg_rst_inst/rx_shift_res_reg_inst/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_reg_rst_inst/rx_shift_res_reg_inst/drv_bus_i[474]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[236]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[10]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[10]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[10]\);
    
    \timestamp_ibuf[26]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(26), Y => 
        \timestamp_ibuf[26]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_RNO_1[28]\ : 
        CFG4
      generic map(INIT => x"ED21")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_19_1\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[17]\, 
        D => \can_top_level_inst/tran_word[4]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_22\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram4_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[112]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNI99S33[0]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/txtb_port_b_address[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/N_4177\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/N_4178\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[14]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_3_0\ : 
        ARI1_CC
      generic map(INIT => x"5CC66")

      port map(A => \can_top_level_inst/drv_bus[49]\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[3]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_2\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_3\, 
        CC => NET_CC_CONFIG2672, P => NET_CC_CONFIG2669, Y3 => 
        NET_CC_CONFIG2670, Y3A => NET_CC_CONFIG2671);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[10]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[7]\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => \can_top_level_inst/txtb_parity_error_valid[2]\, C
         => \can_top_level_inst/txtb_allow_bb[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_168\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o5_0[0]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1372\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m6\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/is_overload\, B => 
        \can_top_level_inst/int_mask[7]\, Y => 
        \can_top_level_inst/int_manager_inst_int_module_gen_7_int_module_inst_int_stat_proc_un2_int_status_set\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[1]\);
    
    \can_top_level_inst/tx_arbitrator_inst/un1_txtb_changed\ : 
        CFG4
      generic map(INIT => x"8CC8")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un4_txtb_changed_NE_0_Z\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/store_last_txtb_index\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/last_txtb_index_Z[2]\, 
        D => \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/txtb_changed\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[2]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[2]\, 
        C => \can_top_level_inst/txtb_parity_error_valid[1]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram40_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram41_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram87_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe87\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram87_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[5]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[3]\, B => 
        \can_top_level_inst/drv_bus[303]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_2\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_3\, 
        CC => NET_CC_CONFIG541, P => NET_CC_CONFIG538, Y3 => 
        NET_CC_CONFIG539, Y3A => NET_CC_CONFIG540);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNID437J6[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[56]\, B => 
        \can_top_level_inst/tran_word[24]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_23\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_24\, 
        CC => NET_CC_CONFIG1023, P => NET_CC_CONFIG1020, Y3 => 
        NET_CC_CONFIG1021, Y3A => NET_CC_CONFIG1022);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[4]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[4]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[3]\);
    
    \data_out_obuf[29]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[29]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[29]/DOUT\, EOUT => 
        \data_out_obuf[29]/EOUT\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[26]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[26]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[3]\ : 
        CFG3
      generic map(INIT => x"CE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_139\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => \can_top_level_inst/txtb_parity_check_valid\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_1_Z\, 
        C => \can_top_level_inst/txtb_parity_error_valid_1[0]\, Y
         => \can_top_level_inst/txtb_parity_error_valid[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[1]\);
    
    
        \can_top_level_inst/frame_filters_inst/filt_sup_gen_true.un6_filter_result_0\ : 
        CFG4
      generic map(INIT => x"F0F7")

      port map(A => 
        \can_top_level_inst/frame_filters_inst/un2_filter_a_enable_0_Z\, 
        B => 
        \can_top_level_inst/frame_filters_inst/un2_filter_a_enable_1_Z\, 
        C => 
        \can_top_level_inst/frame_filters_inst/int_filter_ran_valid\, 
        D => 
        \can_top_level_inst/frame_filters_inst/valid_0_I_87_FCO\, 
        Y => 
        \can_top_level_inst/frame_filters_inst/un6_filter_result_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_4_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/un43_txtb_available_i\ : 
        CFG3
      generic map(INIT => x"D5")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => \can_top_level_inst/txtb_sw_cmd.set_abt\, C => 
        \can_top_level_inst/txtb_sw_cmd_index[1]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_135\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_RNIFF973[5]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\, C
         => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[5]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[4]\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[5]\, 
        CC => NET_CC_CONFIG1115, P => NET_CC_CONFIG1112, Y3 => 
        NET_CC_CONFIG1113, Y3A => NET_CC_CONFIG1114);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[29]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0[3]\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => \can_top_level_inst/txtb_hw_cmd.arbl\, B => 
        \can_top_level_inst/txtb_hw_cmd.err\, C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[5]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2531\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2463\, Y
         => \can_top_level_inst/memory_registers_inst/N_2661\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c4_a0\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[3]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[2]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[1]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[0]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c4_i\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac21_1_0\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac37_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram26_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe26\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram26_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0[28]\ : 
        CFG4
      generic map(INIT => x"FF0B")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_966\, 
        B => \can_top_level_inst/can_core_inst/pc_rx_trigger\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_12[26]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => \can_top_level_inst/drv_bus[297]\, B => 
        \can_top_level_inst/txtb_prorities_6[2]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac17_Z\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac8_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_12_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[29]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[29]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[27]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_o2[1]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[3]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[2]\, C
         => \can_top_level_inst/tx_arbitrator_inst/curr_state[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[4]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_111\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[4]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[27]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_mask_clr_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_0[2]\, 
        D => \adress_c[5]\, Y => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\);
    
    \timestamp_ibuf[1]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(1), Y => \timestamp_ibuf[1]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram4__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa4\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe4\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI2JBC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNI3CA11[1]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[1]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[0]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[1]\, 
        CC => NET_CC_CONFIG1682, P => NET_CC_CONFIG1679, Y3 => 
        NET_CC_CONFIG1680, Y3A => NET_CC_CONFIG1681);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[9]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[9]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[9]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_1[1]\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_926\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_976\, 
        C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        D => \can_top_level_inst/can_core_inst/bds_data_out\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1197\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[17]\);
    
    \data_in_ibuf[11]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[11]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[11]/YIN\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3\ : 
        CFG3
      generic map(INIT => x"4D")

      port map(A => \can_top_level_inst/txtb_prorities_7[2]\, B
         => \can_top_level_inst/txtb_prorities_6[2]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_1_2\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_i1_mux\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_38_1_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_38_1_0_y0[0]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1160\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1162\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[10]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[10]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[10]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_19\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_19_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[15]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[26]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[26]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIG7IO[0]_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO751, CO => CI_TO_CO752, P(0) => 
        NET_CC_CONFIG801, P(1) => NET_CC_CONFIG805, P(2) => 
        NET_CC_CONFIG809, P(3) => NET_CC_CONFIG813, P(4) => 
        NET_CC_CONFIG817, P(5) => NET_CC_CONFIG821, P(6) => 
        NET_CC_CONFIG825, P(7) => NET_CC_CONFIG829, P(8) => 
        NET_CC_CONFIG833, P(9) => NET_CC_CONFIG837, P(10) => 
        NET_CC_CONFIG841, P(11) => NET_CC_CONFIG845, Y3(0) => 
        NET_CC_CONFIG802, Y3(1) => NET_CC_CONFIG806, Y3(2) => 
        NET_CC_CONFIG810, Y3(3) => NET_CC_CONFIG814, Y3(4) => 
        NET_CC_CONFIG818, Y3(5) => NET_CC_CONFIG822, Y3(6) => 
        NET_CC_CONFIG826, Y3(7) => NET_CC_CONFIG830, Y3(8) => 
        NET_CC_CONFIG834, Y3(9) => NET_CC_CONFIG838, Y3(10) => 
        NET_CC_CONFIG842, Y3(11) => NET_CC_CONFIG846, Y3A(0) => 
        NET_CC_CONFIG803, Y3A(1) => NET_CC_CONFIG807, Y3A(2) => 
        NET_CC_CONFIG811, Y3A(3) => NET_CC_CONFIG815, Y3A(4) => 
        NET_CC_CONFIG819, Y3A(5) => NET_CC_CONFIG823, Y3A(6) => 
        NET_CC_CONFIG827, Y3A(7) => NET_CC_CONFIG831, Y3A(8) => 
        NET_CC_CONFIG835, Y3A(9) => NET_CC_CONFIG839, Y3A(10) => 
        NET_CC_CONFIG843, Y3A(11) => NET_CC_CONFIG847, CC(0) => 
        NET_CC_CONFIG804, CC(1) => NET_CC_CONFIG808, CC(2) => 
        NET_CC_CONFIG812, CC(3) => NET_CC_CONFIG816, CC(4) => 
        NET_CC_CONFIG820, CC(5) => NET_CC_CONFIG824, CC(6) => 
        NET_CC_CONFIG828, CC(7) => NET_CC_CONFIG832, CC(8) => 
        NET_CC_CONFIG836, CC(9) => NET_CC_CONFIG840, CC(10) => 
        NET_CC_CONFIG844, CC(11) => NET_CC_CONFIG848);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[31]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[31]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[31]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa12\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe12\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[30]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[30]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[30]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram99__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_350\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe99\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/crc_spec_enable_i_o2\ : 
        CFG2
      generic map(INIT => x"D")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_952\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_996\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram6__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_353\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe6\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_3_1[35]\ : 
        CFG4
      generic map(INIT => x"04F4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_952\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_939\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_3_1_Z[35]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[185]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIBGHO99[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[62]\, B => 
        \can_top_level_inst/tran_word[30]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_29\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIBGHO99[10]_FCNET1\, 
        CC => NET_CC_CONFIG1047, P => NET_CC_CONFIG1044, Y3 => 
        NET_CC_CONFIG1045, Y3A => NET_CC_CONFIG1046);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_1_1[12]\ : 
        CFG3
      generic map(INIT => x"37")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => \can_top_level_inst/drv_bus[28]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_1_1_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[20]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_RNIDDKS2[8]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[8]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_7\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_8\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_8\, 
        CC => NET_CC_CONFIG284, P => NET_CC_CONFIG281, Y3 => 
        NET_CC_CONFIG282, Y3A => NET_CC_CONFIG283);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[98]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[7]\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => \can_top_level_inst/txtb_parity_error_valid[6]\, C
         => \can_top_level_inst/txtb_allow_bb[6]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_168\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_26[9]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => \can_top_level_inst/int_mask[9]\, C => 
        \can_top_level_inst/drv_bus[235]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1494\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram56_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe56\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram56_0\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q_RNIUFKG4[1]\ : 
        ARI1_CC
      generic map(INIT => x"59966")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[1]\, 
        B => \can_top_level_inst/stat_bus[81]\, C => 
        \can_top_level_inst/stat_bus[82]\, D => ADLIB_GND, FCI
         => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_0\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_1\, 
        CC => NET_CC_CONFIG91, P => NET_CC_CONFIG88, Y3 => 
        NET_CC_CONFIG89, Y3A => NET_CC_CONFIG90);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNIU00R87[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_port_b_address[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/N_4118\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/N_4114\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[30]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[16]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_0[1]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[0]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa16\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe16\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa18\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe18\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[413]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry[5]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[4]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[5]\, 
        CC => NET_CC_CONFIG2366, P => NET_CC_CONFIG2363, Y3 => 
        NET_CC_CONFIG2364, Y3A => NET_CC_CONFIG2365);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[6]\);
    
    \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_0\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/stuff_length[0]\, B => 
        \can_top_level_inst/can_core_inst/destuffed\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/g0_i_a4_0_Z\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm_0[6]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[6]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_16\ : 
        CFG3
      generic map(INIT => x"96")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_16_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_29\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_20_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_19_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_18_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_17_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_29_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.arbl\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/stat_bus[6]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/arbl_0\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[41]\ : 
        SLE
      port map(D => \timestamp_c[41]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[41]\);
    
    \can_top_level_inst/memory_registers_inst/status_comb_4[2]\ : 
        CFG4
      generic map(INIT => x"FF8F")

      port map(A => \can_top_level_inst/txtb_state_3[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_159\, 
        C => 
        \can_top_level_inst/memory_registers_inst/status_comb_4_1_Z[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/status_comb_4_1_0_Z[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/status_comb_4_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[2]\);
    
    \timestamp_ibuf[53]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(53), Y => 
        \timestamp_ibuf[53]/YIN\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ssp_reset_i_4\ : 
        CFG4
      generic map(INIT => x"BB80")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/frame_type\, 
        B => \can_top_level_inst/stat_bus[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ssp_reset_i_4_Z\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/err_warning_limit_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un1_err_warning_limit_d_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/err_warning_limit_q_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m16_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_38_1_wmux_RNO_0[20]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_2_0_Z[20]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_2_Z[20]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_926\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNIAL314[7]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[7]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[6]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[7]\, 
        CC => NET_CC_CONFIG1706, P => NET_CC_CONFIG1703, Y3 => 
        NET_CC_CONFIG1704, Y3A => NET_CC_CONFIG1705);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI8QAP[7]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[7]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[7]\, 
        Y => \can_top_level_inst/tran_word[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/un5_transient_state_i_o2\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        Y => \can_top_level_inst/txtb_allow_bb[4]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_53\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[83]\, B => 
        \can_top_level_inst/rec_ident[2]\, C => 
        \can_top_level_inst/drv_bus[116]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_32\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[19]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[162]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1089\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1092\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_0_y0[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry[3]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[2]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[3]\, 
        CC => NET_CC_CONFIG2358, P => NET_CC_CONFIG2355, Y3 => 
        NET_CC_CONFIG2356, Y3A => NET_CC_CONFIG2357);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe52_0_a4\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_349\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2[1]\ : 
        CFG4
      generic map(INIT => x"FFEC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_0_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[28]\);
    
    
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/g0_2\ : 
        CFG4
      generic map(INIT => x"A808")

      port map(A => \can_top_level_inst/prescaler_inst/g0_0_3\, B
         => \can_top_level_inst/prescaler_inst_tq_edge_dbt\, C
         => \can_top_level_inst/prescaler_inst_un2_tq_edge\, D
         => \can_top_level_inst/prescaler_inst_tq_edge_nbt\, Y
         => \can_top_level_inst/prescaler_inst/resync_edge_valid\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_4\ : 
        CFG4
      generic map(INIT => x"FFFB")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[8]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_3906\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[6]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[7]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_4_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[82]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_0_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO208, CO => OPEN, P(0) => 
        NET_CC_CONFIG245, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, 
        P(3) => ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_VCC, 
        P(6) => ADLIB_VCC, P(7) => ADLIB_VCC, P(8) => ADLIB_VCC, 
        P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11) => ADLIB_VCC, 
        Y3(0) => NET_CC_CONFIG246, Y3(1) => ADLIB_VCC, Y3(2) => 
        ADLIB_VCC, Y3(3) => ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5)
         => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7) => ADLIB_VCC, 
        Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, Y3(10) => 
        ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG247, Y3A(1) => ADLIB_VCC, Y3A(2) => 
        ADLIB_VCC, Y3A(3) => ADLIB_VCC, Y3A(4) => ADLIB_VCC, 
        Y3A(5) => ADLIB_VCC, Y3A(6) => ADLIB_VCC, Y3A(7) => 
        ADLIB_VCC, Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, 
        Y3A(10) => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG248, CC(1) => nc328, CC(2) => nc122, CC(3)
         => nc266, CC(4) => nc35, CC(5) => nc324, CC(6) => nc4, 
        CC(7) => nc227, CC(8) => nc92, CC(9) => nc101, CC(10) => 
        nc346, CC(11) => nc330);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_10[25]\ : 
        CFG4
      generic map(INIT => x"30A0")

      port map(A => \can_top_level_inst/drv_bus[263]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_148\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1096\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNINUGF3_S[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[277]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_0_1\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => \can_top_level_inst/drv_bus[52]\, B => 
        \can_top_level_inst/drv_bus[55]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_0_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[22]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[22]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_9[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => ADLIB_VCC, B => ADLIB_VCC, C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[4]\, 
        D => ADLIB_VCC, Y => OPEN);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[20]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[52]\, 
        C => \can_top_level_inst/rec_ident[20]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2265\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_2[1]\ : 
        CFG4
      generic map(INIT => x"FF20")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[7]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1151_2\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_1_Z[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_2_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_3[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y3[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y1[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y9[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_0_y0[11]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1554\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1557\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_31_2_0_y1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[22]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[22]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[22]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_RNIQ8RR1_0\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/CO2\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_s1\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/compl_ctr_q[8]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_15[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram120_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram121_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_6[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[22]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_21\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_21_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram15_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe15\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_0[7]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[6]\, CLK
         => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[11]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8_2[7]\ : 
        CFG4
      generic map(INIT => x"A820")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => \can_top_level_inst/rx_mem_free[7]\, D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_s_7_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram115__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_343\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe115\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/tx_sr_output\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[2]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/destuff_enable\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[304]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_2_3_1_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_2_3_1_0_y0[1]\, 
        B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_dest[1]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_2[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_0_1[13]\ : 
        CFG3
      generic map(INIT => x"27")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \can_top_level_inst/drv_bus[218]\, C => 
        \can_top_level_inst/drv_bus[189]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_0_1_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[14]\ : 
        CFG4
      generic map(INIT => x"030A")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[14]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a4_0_0_Z[23]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => \can_top_level_inst/can_core_inst/pc_rx_trigger\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_816_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[13]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segm_end_req_capture.1.un2_segm_end_req_capt_ce\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un8_exit_segm_regular_tseg1\, 
        B => 
        \can_top_level_inst/prescaler_inst/exit_segm_req_nbt_1\, 
        C => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_req_capt_ce\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\);
    
    \data_in_ibuf[5]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[5]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[5]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI7L9M[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO2564, P(0) => 
        NET_CC_CONFIG2565, P(1) => NET_CC_CONFIG2569, P(2) => 
        NET_CC_CONFIG2573, P(3) => NET_CC_CONFIG2577, P(4) => 
        NET_CC_CONFIG2581, P(5) => NET_CC_CONFIG2585, P(6) => 
        NET_CC_CONFIG2589, P(7) => NET_CC_CONFIG2593, P(8) => 
        NET_CC_CONFIG2597, P(9) => NET_CC_CONFIG2601, P(10) => 
        NET_CC_CONFIG2605, P(11) => NET_CC_CONFIG2609, Y3(0) => 
        NET_CC_CONFIG2566, Y3(1) => NET_CC_CONFIG2570, Y3(2) => 
        NET_CC_CONFIG2574, Y3(3) => NET_CC_CONFIG2578, Y3(4) => 
        NET_CC_CONFIG2582, Y3(5) => NET_CC_CONFIG2586, Y3(6) => 
        NET_CC_CONFIG2590, Y3(7) => NET_CC_CONFIG2594, Y3(8) => 
        NET_CC_CONFIG2598, Y3(9) => NET_CC_CONFIG2602, Y3(10) => 
        NET_CC_CONFIG2606, Y3(11) => NET_CC_CONFIG2610, Y3A(0)
         => NET_CC_CONFIG2567, Y3A(1) => NET_CC_CONFIG2571, 
        Y3A(2) => NET_CC_CONFIG2575, Y3A(3) => NET_CC_CONFIG2579, 
        Y3A(4) => NET_CC_CONFIG2583, Y3A(5) => NET_CC_CONFIG2587, 
        Y3A(6) => NET_CC_CONFIG2591, Y3A(7) => NET_CC_CONFIG2595, 
        Y3A(8) => NET_CC_CONFIG2599, Y3A(9) => NET_CC_CONFIG2603, 
        Y3A(10) => NET_CC_CONFIG2607, Y3A(11) => 
        NET_CC_CONFIG2611, CC(0) => NET_CC_CONFIG2568, CC(1) => 
        NET_CC_CONFIG2572, CC(2) => NET_CC_CONFIG2576, CC(3) => 
        NET_CC_CONFIG2580, CC(4) => NET_CC_CONFIG2584, CC(5) => 
        NET_CC_CONFIG2588, CC(6) => NET_CC_CONFIG2592, CC(7) => 
        NET_CC_CONFIG2596, CC(8) => NET_CC_CONFIG2600, CC(9) => 
        NET_CC_CONFIG2604, CC(10) => NET_CC_CONFIG2608, CC(11)
         => NET_CC_CONFIG2612);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_109_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_ena_0_a4_3\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1239\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1234\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1215_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram13_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe13\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram13_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_dec_proc.3.l1_prio_39\ : 
        CFG4
      generic map(INIT => x"135F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst_priority_decoder_inst_un1_m2_0_a2_0_1\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_39\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNICCIS\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[16]\);
    
    \data_out_obuf[3]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[3]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[3]/DOUT\, EOUT => \data_out_obuf[3]/EOUT\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lto3_0_0_1\ : 
        CFG3
      generic map(INIT => x"13")

      port map(A => \can_top_level_inst/rec_dlc[2]\, B => 
        \can_top_level_inst/rec_dlc[1]\, C => 
        \can_top_level_inst/rec_dlc[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21lto3_0_0_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[5]\ : 
        SLE
      port map(D => \timestamp_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_bit_err_enable_i_0\ : 
        CFG3
      generic map(INIT => x"F7")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un2_bit_err_enable_i_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm_0[2]\ : 
        CFG4
      generic map(INIT => x"FCAC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_length_i_m\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[2]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_0_iv_2[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[410]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_RNIBM7A[0]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_GND, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_0\, 
        CC => NET_CC_CONFIG2179, P => NET_CC_CONFIG2176, Y3 => 
        NET_CC_CONFIG2177, Y3A => NET_CC_CONFIG2178);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[9]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[3]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[11]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[11]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un12_crc_use_17_tz\ : 
        CFG4
      generic map(INIT => x"CE0A")

      port map(A => \can_top_level_inst/stat_bus[156]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/frame_type\, 
        C => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        D => \can_top_level_inst/stat_bus[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[9]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNIVL115[9]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[9]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[8]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[9]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[9]\, 
        CC => NET_CC_CONFIG1714, P => NET_CC_CONFIG1711, Y3 => 
        NET_CC_CONFIG1712, Y3A => NET_CC_CONFIG1713);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_2[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_1_0[14]\ : 
        CFG4
      generic map(INIT => x"50FC")

      port map(A => \can_top_level_inst/txtb_prorities_3[2]\, B
         => \can_top_level_inst/drv_bus[47]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_1_0_Z[14]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test_dbl_buf.tprm[3]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[11]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/load_ffmt_w_addr\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[4]\, 
        B => \can_top_level_inst/drv_bus[55]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_3_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_4_Z\, 
        CC => NET_CC_CONFIG2394, P => NET_CC_CONFIG2391, Y3 => 
        NET_CC_CONFIG2392, Y3A => NET_CC_CONFIG2393);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_0_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_0_FCNET1\, 
        CO => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_8\, 
        CC => NET_CC_CONFIG1204, P => NET_CC_CONFIG1201, Y3 => 
        NET_CC_CONFIG1202, Y3A => NET_CC_CONFIG1203);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate_3_0_.m23\ : 
        CFG4
      generic map(INIT => x"33A0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21lt4\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_22\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[5]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_24\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_21_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/drv_bus[19]\, D => 
        \can_top_level_inst/stat_bus[260]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_21_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[24]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_wmux_0\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_0_y0\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[1]\, 
        C => \can_top_level_inst/N_3904_i\, D => 
        \can_top_level_inst/N_3903_i\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_txtb_parity_mismatch_vld_m_3_2_1_0_y1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_185\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_183\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_184\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[31]\ : 
        CFG4
      generic map(INIT => x"AA80")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[24]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst_txnf_calc_proc_7_un7_txtb_state_2\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_7_Z[31]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram100_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe100\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram100_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_RNICAKR[0]\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_13\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[27]\ : 
        SLE
      port map(D => \data_in_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[203]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_stuff_count_Z[1]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[0]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_stuff_count\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_stuff_count[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_raw_8_2[1]\ : 
        CFG4
      generic map(INIT => x"A820")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => \can_top_level_inst/rx_mem_free[1]\, D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_1_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_2[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry[6]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[5]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[6]\, 
        CC => NET_CC_CONFIG1079, P => NET_CC_CONFIG1076, Y3 => 
        NET_CC_CONFIG1077, Y3A => NET_CC_CONFIG1078);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0MKOK[30]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[218]\, D => 
        \can_top_level_inst/stat_bus[250]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_29\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0MKOK_S[30]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_30\, 
        CC => NET_CC_CONFIG876, P => NET_CC_CONFIG873, Y3 => 
        NET_CC_CONFIG874, Y3A => NET_CC_CONFIG875);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[1]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[11]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[11]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNIR4OQ[6]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_2_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[16]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_meas_progress_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_rst_d_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un10_trv_meas_progress_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_meas_progress_q_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[27]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNI5LRK7[7]\ : 
        ARI1_CC
      generic map(INIT => x"58DD8")

      port map(A => \can_top_level_inst/stat_bus[78]\, B => 
        \can_top_level_inst/drv_bus[425]\, C => 
        \can_top_level_inst/drv_bus[423]\, D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[4]\, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_6\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_7\, 
        CC => NET_CC_CONFIG2518, P => NET_CC_CONFIG2515, Y3 => 
        NET_CC_CONFIG2516, Y3A => NET_CC_CONFIG2517);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[35]\ : 
        CFG4
      generic map(INIT => x"000B")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_2_Z[2]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1116\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_774_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_21_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_22_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_29_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_28_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/N_3897\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[19]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s_2642_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG2343, P(1) => NET_CC_CONFIG2347, P(2) => 
        NET_CC_CONFIG2351, P(3) => NET_CC_CONFIG2355, P(4) => 
        NET_CC_CONFIG2359, P(5) => NET_CC_CONFIG2363, P(6) => 
        NET_CC_CONFIG2367, P(7) => NET_CC_CONFIG2371, P(8) => 
        ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11)
         => ADLIB_VCC, Y3(0) => NET_CC_CONFIG2344, Y3(1) => 
        NET_CC_CONFIG2348, Y3(2) => NET_CC_CONFIG2352, Y3(3) => 
        NET_CC_CONFIG2356, Y3(4) => NET_CC_CONFIG2360, Y3(5) => 
        NET_CC_CONFIG2364, Y3(6) => NET_CC_CONFIG2368, Y3(7) => 
        NET_CC_CONFIG2372, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG2345, Y3A(1) => NET_CC_CONFIG2349, Y3A(2)
         => NET_CC_CONFIG2353, Y3A(3) => NET_CC_CONFIG2357, 
        Y3A(4) => NET_CC_CONFIG2361, Y3A(5) => NET_CC_CONFIG2365, 
        Y3A(6) => NET_CC_CONFIG2369, Y3A(7) => NET_CC_CONFIG2373, 
        Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => 
        ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG2346, CC(1) => NET_CC_CONFIG2350, CC(2) => 
        NET_CC_CONFIG2354, CC(3) => NET_CC_CONFIG2358, CC(4) => 
        NET_CC_CONFIG2362, CC(5) => NET_CC_CONFIG2366, CC(6) => 
        NET_CC_CONFIG2370, CC(7) => NET_CC_CONFIG2374, CC(8) => 
        nc184, CC(9) => nc200, CC(10) => nc190, CC(11) => nc166);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_1[0]\ : 
        CFG4
      generic map(INIT => x"FEFC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1421\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1414\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1372\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_1_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3[7]\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => \can_top_level_inst/txtb_parity_error_valid[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_177\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram127_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe127\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram127_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[16]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o2[29]\ : 
        CFG2
      generic map(INIT => x"D")

      port map(A => \can_top_level_inst/stat_bus[109]\, B => 
        \can_top_level_inst/drv_bus[513]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1051\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[18]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[18]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[30]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[5]\ : 
        CFG4
      generic map(INIT => x"FEFA")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_172\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_173\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_38_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1331\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1346\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_38_1_0_y0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o4[31]\ : 
        CFG3
      generic map(INIT => x"BA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_arb_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1033\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[11]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[11]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[11]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2635\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_RNITTFP41[5]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/tx_arb_parity_check_valid_1\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_m3_0_a3_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_wmux_3[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_0_y3[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_0_y1[9]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_wmux_3_Y[9]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_33\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[26]\, B => 
        \can_top_level_inst/drv_bus[169]\, C => 
        \can_top_level_inst/drv_bus[202]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_46\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[12]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[13]\, 
        CC => NET_CC_CONFIG742, P => NET_CC_CONFIG739, Y3 => 
        NET_CC_CONFIG740, Y3A => NET_CC_CONFIG741);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/destuff_enable\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[303]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[20]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[20]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[21]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[21]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[21]\);
    
    \can_top_level_inst/frame_filters_inst/un1_filter_b_enable\ : 
        CFG4
      generic map(INIT => x"3F50")

      port map(A => \can_top_level_inst/drv_bus[175]\, B => 
        \can_top_level_inst/drv_bus[173]\, C => 
        \can_top_level_inst/stat_bus[155]\, D => 
        \can_top_level_inst/frame_filters_inst/un1_filter_b_enable_1_0_Z\, 
        Y => 
        \can_top_level_inst/frame_filters_inst/un1_filter_b_enable_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[7]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => \can_top_level_inst/txtb_allow_bb[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[13]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram2_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram2_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[275]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_1_1[5]\ : 
        CFG4
      generic map(INIT => x"00B3")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/int_mask[5]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_53_0_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_30_1_1_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[7]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[3]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe20\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[6]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_1_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO459, CO => OPEN, P(0) => 
        NET_CC_CONFIG484, P(1) => NET_CC_CONFIG488, P(2) => 
        NET_CC_CONFIG492, P(3) => NET_CC_CONFIG496, P(4) => 
        NET_CC_CONFIG500, P(5) => NET_CC_CONFIG504, P(6) => 
        NET_CC_CONFIG508, P(7) => NET_CC_CONFIG512, P(8) => 
        NET_CC_CONFIG516, P(9) => NET_CC_CONFIG520, P(10) => 
        ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG485, 
        Y3(1) => NET_CC_CONFIG489, Y3(2) => NET_CC_CONFIG493, 
        Y3(3) => NET_CC_CONFIG497, Y3(4) => NET_CC_CONFIG501, 
        Y3(5) => NET_CC_CONFIG505, Y3(6) => NET_CC_CONFIG509, 
        Y3(7) => NET_CC_CONFIG513, Y3(8) => NET_CC_CONFIG517, 
        Y3(9) => NET_CC_CONFIG521, Y3(10) => ADLIB_VCC, Y3(11)
         => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG486, Y3A(1) => 
        NET_CC_CONFIG490, Y3A(2) => NET_CC_CONFIG494, Y3A(3) => 
        NET_CC_CONFIG498, Y3A(4) => NET_CC_CONFIG502, Y3A(5) => 
        NET_CC_CONFIG506, Y3A(6) => NET_CC_CONFIG510, Y3A(7) => 
        NET_CC_CONFIG514, Y3A(8) => NET_CC_CONFIG518, Y3A(9) => 
        NET_CC_CONFIG522, Y3A(10) => ADLIB_VCC, Y3A(11) => 
        ADLIB_VCC, CC(0) => NET_CC_CONFIG487, CC(1) => 
        NET_CC_CONFIG491, CC(2) => NET_CC_CONFIG495, CC(3) => 
        NET_CC_CONFIG499, CC(4) => NET_CC_CONFIG503, CC(5) => 
        NET_CC_CONFIG507, CC(6) => NET_CC_CONFIG511, CC(7) => 
        NET_CC_CONFIG515, CC(8) => NET_CC_CONFIG519, CC(9) => 
        NET_CC_CONFIG523, CC(10) => nc355, CC(11) => nc338);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_38_1_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_38_1_0_y0[17]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_816\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_818\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2s2\ : 
        CFG2
      generic map(INIT => x"B")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_877\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\);
    
    \can_top_level_inst/tx_arbitrator_inst/validated_buffer_2\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[1]\, B
         => \can_top_level_inst/txtb_hw_cmd_index[0]\, C => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r[8]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[511]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[7]\ : 
        CFG4
      generic map(INIT => x"0E02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[7]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[7]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry[4]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[3]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[4]\, 
        CC => NET_CC_CONFIG1469, P => NET_CC_CONFIG1466, Y3 => 
        NET_CC_CONFIG1467, Y3A => NET_CC_CONFIG1468);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0RIT8_S[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[199]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[173]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[30]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[16]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2261\, B
         => \can_top_level_inst/rx_buffer_inst/N_4820\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[29]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1[24]\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[24]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un7_txtb_parity_error_valid_i\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \can_top_level_inst/drv_bus[476]\, B => 
        \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un7_txtb_parity_error_valid_i_1_Z\, 
        D => \can_top_level_inst/txtb_parity_check_valid\, Y => 
        \can_top_level_inst/txtb_parity_error_valid[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2\ : 
        CFG4
      generic map(INIT => x"4CCC")

      port map(A => 
        \can_top_level_inst/txtb_parity_error_valid_1[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_0_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_m1_e_1\, 
        D => \can_top_level_inst/txtb_parity_check_valid\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_182\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram98_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram99_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe7\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram7_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/bit_gen.9.reg_present_gen.reg_autoclear_gen.reg_value_r_64[9]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[1]\, 
        B => \data_in_c[9]\, Y => 
        \can_top_level_inst/memory_registers_inst/ctr_pres[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[10]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[10]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_0_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO208, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_GND, P(3) => 
        NET_CC_CONFIG209, P(4) => NET_CC_CONFIG213, P(5) => 
        NET_CC_CONFIG217, P(6) => NET_CC_CONFIG221, P(7) => 
        NET_CC_CONFIG225, P(8) => NET_CC_CONFIG229, P(9) => 
        NET_CC_CONFIG233, P(10) => NET_CC_CONFIG237, P(11) => 
        NET_CC_CONFIG241, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => ADLIB_VCC, Y3(3) => NET_CC_CONFIG210, Y3(4) => 
        NET_CC_CONFIG214, Y3(5) => NET_CC_CONFIG218, Y3(6) => 
        NET_CC_CONFIG222, Y3(7) => NET_CC_CONFIG226, Y3(8) => 
        NET_CC_CONFIG230, Y3(9) => NET_CC_CONFIG234, Y3(10) => 
        NET_CC_CONFIG238, Y3(11) => NET_CC_CONFIG242, Y3A(0) => 
        ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => ADLIB_VCC, 
        Y3A(3) => NET_CC_CONFIG211, Y3A(4) => NET_CC_CONFIG215, 
        Y3A(5) => NET_CC_CONFIG219, Y3A(6) => NET_CC_CONFIG223, 
        Y3A(7) => NET_CC_CONFIG227, Y3A(8) => NET_CC_CONFIG231, 
        Y3A(9) => NET_CC_CONFIG235, Y3A(10) => NET_CC_CONFIG239, 
        Y3A(11) => NET_CC_CONFIG243, CC(0) => nc326, CC(1) => 
        nc132, CC(2) => nc334, CC(3) => NET_CC_CONFIG212, CC(4)
         => NET_CC_CONFIG216, CC(5) => NET_CC_CONFIG220, CC(6)
         => NET_CC_CONFIG224, CC(7) => NET_CC_CONFIG228, CC(8)
         => NET_CC_CONFIG232, CC(9) => NET_CC_CONFIG236, CC(10)
         => NET_CC_CONFIG240, CC(11) => NET_CC_CONFIG244);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[22]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[22]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[22]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_37_1[12]\ : 
        CFG3
      generic map(INIT => x"A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \can_top_level_inst/stat_bus[200]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_37_1_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[20]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_shadowed[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/trv_delay[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[30]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[14]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[14]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_s_1_2643\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_VCC, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_s_1_2643_FCO\, 
        CC => NET_CC_CONFIG2219, P => NET_CC_CONFIG2216, Y3 => 
        NET_CC_CONFIG2217, Y3A => NET_CC_CONFIG2218);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_87\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[29]\, B => 
        \timestamp_c[60]\, C => \timestamp_c[61]\, D => 
        \can_top_level_inst/tran_word[28]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[13]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_87_FCNET1\, 
        CC => NET_CC_CONFIG2309, P => NET_CC_CONFIG2306, Y3 => 
        NET_CC_CONFIG2307, Y3A => NET_CC_CONFIG2308);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/un7_index_sat[3]\ : 
        CFG3
      generic map(INIT => x"70")

      port map(A => \adress_c[7]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[5]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\);
    
    \can_top_level_inst/frame_filters_inst/ident_valid_q\ : SLE
      port map(D => 
        \can_top_level_inst/frame_filters_inst/ident_valid_d_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/frame_filters_inst_ident_valid_q\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[4]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_RNIQAIP\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_stat_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => \sbe_c[0]\, Y => \can_top_level_inst/N_138\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/int_ena_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/un1_int_ena_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_ena[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/op_eq.un6_reinteg_ctr_expired_4\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[8]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[6]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un6_reinteg_ctr_expired_4\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[14]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[13]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[14]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_22\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[293]\, B => 
        \can_top_level_inst/rec_ident[22]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_21\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_22\, 
        CC => NET_CC_CONFIG1921, P => NET_CC_CONFIG1918, Y3 => 
        NET_CC_CONFIG1919, Y3A => NET_CC_CONFIG1920);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_12\ : 
        ARI1_CC
      generic map(INIT => x"56633")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[12]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[12]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_11_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_12_Z\, 
        CC => NET_CC_CONFIG446, P => NET_CC_CONFIG443, Y3 => 
        NET_CC_CONFIG444, Y3A => NET_CC_CONFIG445);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select[2]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6_1\, 
        B => \sbe_c[2]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_2_0\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[22]\ : 
        CFG4
      generic map(INIT => x"F0EE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[22]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_1_Z[22]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[21]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_7_i\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1304\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_736_i\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm_0[5]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[5]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_lm[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[22]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[22]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m17_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_2_0_1[22]\ : 
        CFG3
      generic map(INIT => x"35")

      port map(A => \can_top_level_inst/stat_bus[279]\, B => 
        \can_top_level_inst/drv_bus[293]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_2_0_1_Z[22]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm_0[5]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[5]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[5]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rst_reg_inst/rx_shift_res_reg_inst/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rst_reg_inst/rx_shift_res_reg_inst/un1_res_err_ctrs_d_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_23\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[294]\, B => 
        \can_top_level_inst/rec_ident[23]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_22\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_23\, 
        CC => NET_CC_CONFIG1925, P => NET_CC_CONFIG1922, Y3 => 
        NET_CC_CONFIG1923, Y3A => NET_CC_CONFIG1924);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[30]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2556\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2488\, Y
         => \can_top_level_inst/memory_registers_inst/N_2686\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_3_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[3]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_28\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_23_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_16_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un36_parity_mismatch_28_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[6]\ : 
        CFG4
      generic map(INIT => x"000E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_9\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_1_Z[31]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[31]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_832_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[27]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_22\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[22]\, B => 
        \can_top_level_inst/drv_bus[322]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_21\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_22\, 
        CC => NET_CC_CONFIG617, P => NET_CC_CONFIG614, Y3 => 
        NET_CC_CONFIG615, Y3A => NET_CC_CONFIG616);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[4]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[25]\);
    
    \timestamp_ibuf[13]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[13]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[13]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_20\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_20_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4_2[4]\ : 
        CFG4
      generic map(INIT => x"C808")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_4_S\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, D
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_4_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_arb_1\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/bds_data_out\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_arb_1_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un4_txtb_port_b_address[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_ptr[4]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, Y
         => \can_top_level_inst/txtb_port_b_address[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_23\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[23]\, B => 
        \can_top_level_inst/drv_bus[323]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_22\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_23\, 
        CC => NET_CC_CONFIG621, P => NET_CC_CONFIG618, Y3 => 
        NET_CC_CONFIG619, Y3A => NET_CC_CONFIG620);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/int_ena_proc.un1_int_ena_clear\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[4]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.4.int_module_inst/un1_int_ena_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[21]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe32_0_a4\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_350\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[18]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNICA923[3]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[191]\, D => 
        \can_top_level_inst/stat_bus[223]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_2\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNICA923_S[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_3\, 
        CC => NET_CC_CONFIG768, P => NET_CC_CONFIG765, Y3 => 
        NET_CC_CONFIG766, Y3A => NET_CC_CONFIG767);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[16]\);
    
    \data_out_obuf[1]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[1]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[1]/DOUT\, EOUT => \data_out_obuf[1]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[17]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[29]\ : 
        CFG4
      generic map(INIT => x"3120")

      port map(A => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_975\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[30]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[29]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_786_i\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_2_0[9]\ : 
        CFG4
      generic map(INIT => x"DDA0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1495\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1494\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_0_Z[9]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_2_0_Z[9]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l3_valid\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_19\, B => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_09\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_l3_valid\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[26]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[26]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[26]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[10]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[8]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[10]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[10]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[13]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[12]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_6[27]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac22_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac19_Z\, 
        C => \can_top_level_inst/drv_bus[232]\, D => 
        \can_top_level_inst/drv_bus[141]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_6_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m103_2\ : 
        CFG4
      generic map(INIT => x"C4C7")

      port map(A => \can_top_level_inst/drv_bus[148]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m103_2_1_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => \can_top_level_inst/drv_bus[276]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_104\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_1_1[2]\ : 
        CFG4
      generic map(INIT => x"FFF1")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[6]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[5]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_155\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_121\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_1_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c[3]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_data_length_c[3]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[3]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[17]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[17]\);
    
    \data_in_ibuf[2]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(2), Y => \data_in_ibuf[2]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_10[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y21[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y9[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[4]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_wmux_10_Y[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count_RNI62SU1[3]\ : 
        ARI1_CC
      generic map(INIT => x"599CC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un10_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, C
         => \can_top_level_inst/rx_frame_count[3]\, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_2\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI62SU1_S[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_3\, 
        CC => NET_CC_CONFIG2644, P => NET_CC_CONFIG2641, Y3 => 
        NET_CC_CONFIG2642, Y3A => NET_CC_CONFIG2643);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNISGAN[1]\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[1]\, 
        B => \can_top_level_inst/drv_bus[376]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_0\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNISGAN_S[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_1\, 
        CC => NET_CC_CONFIG1573, P => NET_CC_CONFIG1570, Y3 => 
        NET_CC_CONFIG1571, Y3A => NET_CC_CONFIG1572);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_s_1_2644_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => ADLIB_VCC, 
        P(1) => ADLIB_VCC, P(2) => ADLIB_GND, P(3) => 
        NET_CC_CONFIG2415, P(4) => NET_CC_CONFIG2419, P(5) => 
        NET_CC_CONFIG2423, P(6) => NET_CC_CONFIG2427, P(7) => 
        NET_CC_CONFIG2431, P(8) => NET_CC_CONFIG2435, P(9) => 
        NET_CC_CONFIG2439, P(10) => NET_CC_CONFIG2443, P(11) => 
        ADLIB_VCC, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, Y3(2)
         => ADLIB_GND, Y3(3) => NET_CC_CONFIG2416, Y3(4) => 
        NET_CC_CONFIG2420, Y3(5) => NET_CC_CONFIG2424, Y3(6) => 
        NET_CC_CONFIG2428, Y3(7) => NET_CC_CONFIG2432, Y3(8) => 
        NET_CC_CONFIG2436, Y3(9) => NET_CC_CONFIG2440, Y3(10) => 
        NET_CC_CONFIG2444, Y3(11) => ADLIB_VCC, Y3A(0) => 
        ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => ADLIB_GND, 
        Y3A(3) => NET_CC_CONFIG2417, Y3A(4) => NET_CC_CONFIG2421, 
        Y3A(5) => NET_CC_CONFIG2425, Y3A(6) => NET_CC_CONFIG2429, 
        Y3A(7) => NET_CC_CONFIG2433, Y3A(8) => NET_CC_CONFIG2437, 
        Y3A(9) => NET_CC_CONFIG2441, Y3A(10) => NET_CC_CONFIG2445, 
        Y3A(11) => ADLIB_VCC, CC(0) => nc21, CC(1) => nc237, 
        CC(2) => nc93, CC(3) => NET_CC_CONFIG2418, CC(4) => 
        NET_CC_CONFIG2422, CC(5) => NET_CC_CONFIG2426, CC(6) => 
        NET_CC_CONFIG2430, CC(7) => NET_CC_CONFIG2434, CC(8) => 
        NET_CC_CONFIG2438, CC(9) => NET_CC_CONFIG2442, CC(10) => 
        NET_CC_CONFIG2446, CC(11) => nc262);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[2]\ : 
        CFG4
      generic map(INIT => x"F0F2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_155\, 
        D => \can_top_level_inst/txtb_parity_error_valid[6]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[29]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[29]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[29]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[17]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un6_same_bits_rst_trig\ : 
        CFG3
      generic map(INIT => x"37")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/fixed_reg_q\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un9_stuff_lvl_reached_NE\, 
        C => \can_top_level_inst/can_core_inst/stuff_length[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un6_same_bits_rst_trig_i\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_gate_mem_read.AXB[4]\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[4]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/AXB[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a2[28]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reinteg_ctr_expired\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1269\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[23]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2549\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.is_control_2_i_a2_3_RNIJQBI\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1462_3\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => \can_top_level_inst/stat_bus[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[21]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8\ : 
        CFG4
      generic map(INIT => x"A8AA")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_nbt_tq_active\, 
        B => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_10_N_2L1_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/un8_exit_segm_regular_tseg1\, 
        D => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_1_0\, 
        Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_0_1_N_5L8_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[6]\);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_Z[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_ns[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_185\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_183\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_184\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_shift_ena\ : 
        CFG4
      generic map(INIT => x"1300")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1238\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1304\, 
        D => \can_top_level_inst/stat_bus[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_ena\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.is_control_2_i_a2_2_0\ : 
        CFG4
      generic map(INIT => x"0010")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/is_control_2_i_a2_1\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/is_control_2_i_a2_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe1\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[13]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[13]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[13]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_4[2]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_26_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[18]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_1_0\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => \can_top_level_inst/drv_bus[57]\, B => 
        \can_top_level_inst/drv_bus[58]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[25]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[25]\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_frame_type_com\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_type_dbl_buf_Z\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[67]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry[3]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[2]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_cry_Z[3]\, 
        CC => NET_CC_CONFIG1465, P => NET_CC_CONFIG1462, Y3 => 
        NET_CC_CONFIG1463, Y3A => NET_CC_CONFIG1464);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[4]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0[0]\ : 
        CFG3
      generic map(INIT => x"70")

      port map(A => \adress_c[7]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \sbe_c[0]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram101_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe101\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram101_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[31]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g1_2\ : 
        CFG4
      generic map(INIT => x"BFFF")

      port map(A => \can_top_level_inst/rx_data_wbs\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_sjw_mt_zero_0_0\, 
        C => \can_top_level_inst/prescaler_inst/g0_0_0\, D => 
        \can_top_level_inst/prescaler_inst/g0_0_2\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g1_2_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[25]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[25]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[25]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2649\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8_RNIMF8RO5\ : 
        ARI1_CC
      generic map(INIT => x"49CCC")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[6]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_5\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_6\, 
        CC => NET_CC_CONFIG2724, P => NET_CC_CONFIG2721, Y3 => 
        NET_CC_CONFIG2722, Y3A => NET_CC_CONFIG2723);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un11_crc_use_21\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_N_32\, 
        B => \can_top_level_inst/stat_bus[156]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un15_crc_use_21_tz\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_data_len_can_2_0_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un11_crc_use_21_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        CLK => NN_1, EN => ADLIB_VCC, ALn => ADLIB_VCC, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/un4_tst_ena\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \can_top_level_inst/reg_value_r_rep[17]\, B
         => \can_top_level_inst/test_registers_out.tst_dest[19]\, 
        C => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena_2\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_5[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[6]\);
    
    \timestamp_ibuf[41]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[41]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[41]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa16\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8_2\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa16\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[9]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[8]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[9]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[9]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_126_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_ctr_proc.same_bits_q_3_iv_i_RNO[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/tx_trigger\, B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/CO0_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_1[2]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[9]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un9_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram86_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe86\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram86_0\);
    
    \can_top_level_inst/rx_buffer_inst/rx_read_buff[13]\ : CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        B => \can_top_level_inst/rx_buffer_inst_RAM_data_out[13]\, 
        Y => \can_top_level_inst/rx_read_buff[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m34_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[11]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[11]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[192]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[92]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_13[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram52_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram53_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_5[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[33]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_778_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[33]\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count_RNI305F1[2]\ : 
        ARI1_CC
      generic map(INIT => x"599CC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un10_read_increment\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, C
         => \can_top_level_inst/rx_frame_count[2]\, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_1\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI305F1_S[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_2\, 
        CC => NET_CC_CONFIG2640, P => NET_CC_CONFIG2637, Y3 => 
        NET_CC_CONFIG2638, Y3A => NET_CC_CONFIG2639);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram68__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_353\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe68\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_20[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_8[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y3_0[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y1_0[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y21[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[28]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa14\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe14\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[30]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[30]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[30]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[3]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_12\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[283]\, B => 
        \can_top_level_inst/rec_ident[12]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_11\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_12\, 
        CC => NET_CC_CONFIG1881, P => NET_CC_CONFIG1878, Y3 => 
        NET_CC_CONFIG1879, Y3A => NET_CC_CONFIG1880);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[30]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[30]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram66_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram67_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_write_pointer_ts_ce\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, B => 
        \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\, Y
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_write_pointer_ts_ce_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[24]\ : 
        CFG4
      generic map(INIT => x"EC00")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[24]\, B
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[24]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[26]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d[7]\ : 
        CFG4
      generic map(INIT => x"D888")

      port map(A => \can_top_level_inst/drv_bus[426]\, B => 
        \can_top_level_inst/drv_bus[423]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_m0_Z[7]\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[287]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[1]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_13\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[284]\, B => 
        \can_top_level_inst/rec_ident[13]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_12\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_13\, 
        CC => NET_CC_CONFIG1885, P => NET_CC_CONFIG1882, Y3 => 
        NET_CC_CONFIG1883, Y3A => NET_CC_CONFIG1884);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[11]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[11]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select[3]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_1\, 
        D => \adress_c[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\);
    
    \can_top_level_inst/rx_buffer_inst/is_free_word_1\ : CFG2
      generic map(INIT => x"6")

      port map(A => \can_top_level_inst/rx_read_pointer[1]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[1]\, 
        Y => \can_top_level_inst/rx_buffer_inst/is_free_word_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        D => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/N_4134\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[13]\ : 
        CFG4
      generic map(INIT => x"ACA0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[12]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_388\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[13]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_3[10]\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[124]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1509\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_16_1\ : 
        CFG3
      generic map(INIT => x"A2")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_26\, 
        C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_3_1_RNIMVGD1_Y\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_16_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m18_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa_0_a2_0\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_185\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_183\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_184\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram106__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_333\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_344\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe106\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/prev_val_d\ : 
        CFG3
      generic map(INIT => x"AB")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_in\, B => 
        \can_top_level_inst/can_core_inst/stuff_length[0]\, C => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/fixed_prev_q\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/prev_val_d_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI8UQ54_S[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[278]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[20]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[20]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[20]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_d.lock_i_o4_RNI3U6U\ : 
        CFG4
      generic map(INIT => x"2220")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_970\, 
        C => \can_top_level_inst/can_core_inst/bds_data_out\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_933\, 
        Y => 
        \can_top_level_inst/can_core_inst/operation_control_inst_set_transmitter\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[27]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_sw_cmd_index[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[9]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[24]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[295]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[9]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[9]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[9]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m16_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[6]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[0]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2624\, Y
         => \can_top_level_inst/memory_registers_inst/N_2689\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_38_1_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_38_1_0_y0[10]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1540\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1542\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[4]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/int_mask_load\ : 
        CFG4
      generic map(INIT => x"A800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        D => \data_in_c[10]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/int_mask_load_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_10_2[24]\ : 
        CFG4
      generic map(INIT => x"2A08")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_147\, 
        D => \can_top_level_inst/stat_bus[79]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1058_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[17]\);
    
    \can_top_level_inst/int_manager_inst/un1_int_i_1\ : CFG4
      generic map(INIT => x"135F")

      port map(A => \can_top_level_inst/int_vector[9]\, B => 
        \can_top_level_inst/int_vector[8]\, C => 
        \can_top_level_inst/int_ena[9]\, D => 
        \can_top_level_inst/int_ena[8]\, Y => 
        \can_top_level_inst/int_manager_inst/un1_int_i_1_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_4[27]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac20_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac16_Z\, 
        C => \can_top_level_inst/drv_bus[327]\, D => 
        \can_top_level_inst/drv_bus[203]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_4_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[16]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m[3]\ : 
        CFG4
      generic map(INIT => x"3331")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_1_Z[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_913_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1038\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_3_Z[3]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_m_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_20[0]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => \can_top_level_inst/drv_bus[400]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/drv_bus[81]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1146\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[8]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_3[2]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[25]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un25_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_0_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => ADLIB_VCC, 
        P(1) => ADLIB_GND, P(2) => NET_CC_CONFIG2375, P(3) => 
        NET_CC_CONFIG2379, P(4) => NET_CC_CONFIG2383, P(5) => 
        NET_CC_CONFIG2387, P(6) => NET_CC_CONFIG2391, P(7) => 
        NET_CC_CONFIG2395, P(8) => NET_CC_CONFIG2399, P(9) => 
        NET_CC_CONFIG2403, P(10) => NET_CC_CONFIG2407, P(11) => 
        NET_CC_CONFIG2411, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => NET_CC_CONFIG2376, Y3(3) => NET_CC_CONFIG2380, 
        Y3(4) => NET_CC_CONFIG2384, Y3(5) => NET_CC_CONFIG2388, 
        Y3(6) => NET_CC_CONFIG2392, Y3(7) => NET_CC_CONFIG2396, 
        Y3(8) => NET_CC_CONFIG2400, Y3(9) => NET_CC_CONFIG2404, 
        Y3(10) => NET_CC_CONFIG2408, Y3(11) => NET_CC_CONFIG2412, 
        Y3A(0) => ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => 
        NET_CC_CONFIG2377, Y3A(3) => NET_CC_CONFIG2381, Y3A(4)
         => NET_CC_CONFIG2385, Y3A(5) => NET_CC_CONFIG2389, 
        Y3A(6) => NET_CC_CONFIG2393, Y3A(7) => NET_CC_CONFIG2397, 
        Y3A(8) => NET_CC_CONFIG2401, Y3A(9) => NET_CC_CONFIG2405, 
        Y3A(10) => NET_CC_CONFIG2409, Y3A(11) => 
        NET_CC_CONFIG2413, CC(0) => nc69, CC(1) => nc206, CC(2)
         => NET_CC_CONFIG2378, CC(3) => NET_CC_CONFIG2382, CC(4)
         => NET_CC_CONFIG2386, CC(5) => NET_CC_CONFIG2390, CC(6)
         => NET_CC_CONFIG2394, CC(7) => NET_CC_CONFIG2398, CC(8)
         => NET_CC_CONFIG2402, CC(9) => NET_CC_CONFIG2406, CC(10)
         => NET_CC_CONFIG2410, CC(11) => NET_CC_CONFIG2414);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0[0]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[0]\, 
        C => \can_top_level_inst/drv_bus[56]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_49\);
    
    \timestamp_ibuf[55]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(55), Y => 
        \timestamp_ibuf[55]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[513]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[404]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[29]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/int_status\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/un2_int_status_set\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.10.int_module_inst/un1_int_status_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_vector[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[5]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[5]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_30\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_21_Z\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_22_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_24_Z\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_23_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/parity_mismatch_1_30\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m85\ : 
        CFG3
      generic map(INIT => x"1D")

      port map(A => \can_top_level_inst/drv_bus[407]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/drv_bus[88]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i77_mux\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[28]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[18]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d[0]\ : 
        CFG3
      generic map(INIT => x"48")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un4_dst_ctr_d_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/reg_q_0\, 
        C => \can_top_level_inst/stat_bus[303]\, Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[0]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[4]\, B
         => \can_top_level_inst/txtb_sw_cmd.set_rdy\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_141\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un18_crc_use_21\ : 
        CFG4
      generic map(INIT => x"F8F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/frame_type\, 
        B => \can_top_level_inst/stat_bus[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un11_crc_use_21_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_crc_use_21\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_RNO_3[28]\ : 
        CFG4
      generic map(INIT => x"C090")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_o4_2\, 
        B => \can_top_level_inst/stat_bus[300]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_o4_3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_a5_2_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[20]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[20]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[20]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4823\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.7.int_module_inst/int_status\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst_int_module_gen_7_int_module_inst_int_stat_proc_un2_int_status_set\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/N_141_mux_i\, ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_vector[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[1]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2690\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2657\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_ena_set_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\, 
        D => \adress_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_3_0\ : 
        ARI1_CC
      generic map(INIT => x"5CC66")

      port map(A => \can_top_level_inst/drv_bus[32]\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_fast\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[3]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_2\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_3\, 
        CC => NET_CC_CONFIG900, P => NET_CC_CONFIG897, Y3 => 
        NET_CC_CONFIG898, Y3A => NET_CC_CONFIG899);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1103_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_4_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_976\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_913_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[21]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[21]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[21]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_2\ : 
        CFG3
      generic map(INIT => x"4D")

      port map(A => \can_top_level_inst/txtb_prorities_5[2]\, B
         => \can_top_level_inst/txtb_prorities_4[2]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_m3_2_1_1_Z\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_i1_mux_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_20\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_20_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[21]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO[18]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_37_2_0_Z[18]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_37_1_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_857\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_0\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \adress_c[3]\, B => \adress_c[5]\, C => 
        \adress_c[7]\, D => swr_c, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/NN_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_1\ : 
        CFG4
      generic map(INIT => x"4F00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_cnst[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1324\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_877\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_1_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[18]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[18]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[18]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[12]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[12]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[12]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[9]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[9]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa10\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10_2\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_sx\ : 
        CFG4
      generic map(INIT => x"5553")

      port map(A => 
        \can_top_level_inst/prescaler_inst_tq_edge_dbt\, B => 
        \can_top_level_inst/prescaler_inst_tq_edge_nbt\, C => 
        \can_top_level_inst/stat_bus[10]\, D => 
        \can_top_level_inst/stat_bus[11]\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_sx_Z\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_frame_test.fstc\ : 
        SLE
      port map(D => \can_top_level_inst/tx_arbitrator_inst/fstc\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_frame_test.fstc\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[19]\);
    
    \can_top_level_inst/rx_buffer_inst/commit_overrun_abort\ : 
        SLE
      port map(D => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/commit_overrun_abort_0_sqmuxa_i_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/commit_overrun_abort_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[37]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_771_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[1]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[0]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[17]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_ac0_11_RNI57T721\ : 
        ARI1_CC
      generic map(INIT => x"56699")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[7]\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c7\, 
        C => \can_top_level_inst/stat_bus[88]\, D => ADLIB_GND, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_6\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_7\, 
        CC => NET_CC_CONFIG115, P => NET_CC_CONFIG112, Y3 => 
        NET_CC_CONFIG113, Y3A => NET_CC_CONFIG114);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_3\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_3_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/command_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_0\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \adress_c[5]\, B => \adress_c[6]\, C => 
        \adress_c[7]\, D => swr_c, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0\);
    
    \can_top_level_inst/rx_buffer_inst/frame_form_w_9_iv[12]\ : 
        CFG3
      generic map(INIT => x"CD")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un40_frame_form_w_i\, 
        B => \can_top_level_inst/rec_is_rtr\, C => 
        \can_top_level_inst/rx_buffer_inst/N_10\, Y => 
        \can_top_level_inst/rx_buffer_inst/frame_form_w_9[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[17]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2475\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram37__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_341\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe37\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_7_2_0_1[3]\ : 
        CFG3
      generic map(INIT => x"27")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/int_ena[3]\, C => 
        \can_top_level_inst/drv_bus[329]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_7_2_0_1_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un4_tst_ena\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena_3\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[17]\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q_RNO[1]\ : 
        CFG4
      generic map(INIT => x"E22E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[12]\, B
         => \can_top_level_inst/rx_mof\, C => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[0]\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_3925_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[5]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[28]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[28]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[28]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[1]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[14]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac17\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac1_0_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac17_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m34_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[19]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNIHM4H3[6]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[6]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[5]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[6]\, 
        CC => NET_CC_CONFIG1702, P => NET_CC_CONFIG1699, Y3 => 
        NET_CC_CONFIG1700, Y3A => NET_CC_CONFIG1701);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[2]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[2]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[2]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa2\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[8]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[8]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[8]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_38_1_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_38_1_0_y0[6]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_4991\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1390\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI46KGJ_S[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[216]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[19]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[19]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[14]\ : 
        CFG4
      generic map(INIT => x"F808")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2207_2\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_0_0_Z[14]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        D => \can_top_level_inst/rx_buffer_inst/N_2292\, Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[21]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[30]\ : 
        CFG4
      generic map(INIT => x"F0E0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_7_Z[30]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[894]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[24]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[926]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\);
    
    \can_rx_ibuf/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => can_rx_c, E => ADLIB_GND, YIN => 
        \can_rx_ibuf/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_17\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_17_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa12\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe12\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[23]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[199]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[16]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO\ : 
        ARI1_CC
      generic map(INIT => x"45500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[89]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_erp_cry_7\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_FCNET1\, 
        CC => NET_CC_CONFIG203, P => NET_CC_CONFIG200, Y3 => 
        NET_CC_CONFIG201, Y3A => NET_CC_CONFIG202);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe10\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[3]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[3]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[3]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_10_1[6]\ : 
        CFG4
      generic map(INIT => x"3120")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[182]\, D => 
        \can_top_level_inst/memory_registers_inst/N_4605\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1364_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/N_4177\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram115_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe115\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram115_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0\ : 
        CFG4
      generic map(INIT => x"F070")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0_1_Z\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0_0_Z\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_i1_mux_0\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_16_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[20]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[1]\, CLK
         => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[10]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[10]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[8]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[14]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[14]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[14]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_RNO[1]\ : 
        CFG4
      generic map(INIT => x"5515")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_1_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1370\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1369_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[23]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_cry_9_RNID67MD\ : 
        CFG4
      generic map(INIT => x"2022")

      port map(A => \can_top_level_inst/prescaler_inst/is_tseg1\, 
        B => 
        \can_top_level_inst/prescaler_inst/un7_exit_segm_regular_cry_9_0\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_4_1_sx_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_4_1_0_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/un8_exit_segm_regular_tseg1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1[2]\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \can_top_level_inst/txtb_hw_cmd.failed\, B
         => \can_top_level_inst/tran_valid\, C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_193\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_clk_en_d_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/txtb_ptr[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_816_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m16_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/un3_tst_ena\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[16]\, B
         => \can_top_level_inst/reg_value_r_rep[17]\, C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena_2\, 
        D => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un7_rx_err_ctr_declto8_i_a2_0_a2_4\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => \can_top_level_inst/stat_bus[88]\, B => 
        \can_top_level_inst/stat_bus[81]\, C => 
        \can_top_level_inst/stat_bus[84]\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_4\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[22]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[165]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[6]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI4MAP[5]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[5]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[5]\, 
        Y => \can_top_level_inst/tran_word[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[16]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_RNIJ1OO7[5]\ : 
        ARI1_CC
      generic map(INIT => x"48800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy_Y[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[5]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[4]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[5]\, 
        CC => NET_CC_CONFIG673, P => NET_CC_CONFIG670, Y3 => 
        NET_CC_CONFIG671, Y3A => NET_CC_CONFIG672);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[2]\ : 
        CFG4
      generic map(INIT => x"27FF")

      port map(A => \can_top_level_inst/drv_bus_475_rep2\, B => 
        \can_top_level_inst/txtb_sw_cmd_index[2]\, C => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[11]\, 
        D => \can_top_level_inst/txtb_sw_cmd.set_abt_rep1\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_141\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/g0\ : 
        CFG4
      generic map(INIT => x"5D55")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_ena\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/g0_1_Z\, 
        D => \can_top_level_inst/tx_trigger\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe20\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa9\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe9\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s_2638_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG362, P(1) => NET_CC_CONFIG366, P(2) => 
        NET_CC_CONFIG370, P(3) => NET_CC_CONFIG374, P(4) => 
        NET_CC_CONFIG378, P(5) => NET_CC_CONFIG382, P(6) => 
        NET_CC_CONFIG386, P(7) => NET_CC_CONFIG390, P(8) => 
        ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11)
         => ADLIB_VCC, Y3(0) => NET_CC_CONFIG363, Y3(1) => 
        NET_CC_CONFIG367, Y3(2) => NET_CC_CONFIG371, Y3(3) => 
        NET_CC_CONFIG375, Y3(4) => NET_CC_CONFIG379, Y3(5) => 
        NET_CC_CONFIG383, Y3(6) => NET_CC_CONFIG387, Y3(7) => 
        NET_CC_CONFIG391, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG364, Y3A(1) => NET_CC_CONFIG368, Y3A(2) => 
        NET_CC_CONFIG372, Y3A(3) => NET_CC_CONFIG376, Y3A(4) => 
        NET_CC_CONFIG380, Y3A(5) => NET_CC_CONFIG384, Y3A(6) => 
        NET_CC_CONFIG388, Y3A(7) => NET_CC_CONFIG392, Y3A(8) => 
        ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => ADLIB_VCC, 
        Y3A(11) => ADLIB_VCC, CC(0) => NET_CC_CONFIG365, CC(1)
         => NET_CC_CONFIG369, CC(2) => NET_CC_CONFIG373, CC(3)
         => NET_CC_CONFIG377, CC(4) => NET_CC_CONFIG381, CC(5)
         => NET_CC_CONFIG385, CC(6) => NET_CC_CONFIG389, CC(7)
         => NET_CC_CONFIG393, CC(8) => nc174, CC(9) => nc38, 
        CC(10) => nc113, CC(11) => nc336);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[19]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[19]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[19]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed\ : 
        CFG4
      generic map(INIT => x"8008")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_0_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_1_Z\, 
        C => \can_top_level_inst/drv_bus[466]\, D => 
        \can_top_level_inst/stat_bus[2]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tx_failed_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[26]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[26]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_15\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[15]\, B => 
        \can_top_level_inst/drv_bus[315]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_14\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_15\, 
        CC => NET_CC_CONFIG589, P => NET_CC_CONFIG586, Y3 => 
        NET_CC_CONFIG587, Y3A => NET_CC_CONFIG588);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[18]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[318]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_stuff_count_Z[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_stuff_count\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_stuff_count[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/un5_transient_state_i_o2\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => \can_top_level_inst/txtb_allow_bb[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[182]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[24]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[24]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[24]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, D => 
        \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[25]\);
    
    \data_in_ibuf[4]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[4]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[4]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIKDJDB_S[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[272]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI0C9S\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_0_y3[18]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_0_y1[18]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_851\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0UM0H[24]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[212]\, D => 
        \can_top_level_inst/stat_bus[244]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_23\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0UM0H_S[24]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_24\, 
        CC => NET_CC_CONFIG852, P => NET_CC_CONFIG849, Y3 => 
        NET_CC_CONFIG850, Y3A => NET_CC_CONFIG851);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_RNI189A[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_GND, CO => OPEN, P(0) => 
        NET_CC_CONFIG249, P(1) => NET_CC_CONFIG253, P(2) => 
        NET_CC_CONFIG257, P(3) => NET_CC_CONFIG261, P(4) => 
        NET_CC_CONFIG265, P(5) => NET_CC_CONFIG269, P(6) => 
        NET_CC_CONFIG273, P(7) => NET_CC_CONFIG277, P(8) => 
        NET_CC_CONFIG281, P(9) => NET_CC_CONFIG285, P(10) => 
        ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG250, 
        Y3(1) => NET_CC_CONFIG254, Y3(2) => NET_CC_CONFIG258, 
        Y3(3) => NET_CC_CONFIG262, Y3(4) => NET_CC_CONFIG266, 
        Y3(5) => NET_CC_CONFIG270, Y3(6) => NET_CC_CONFIG274, 
        Y3(7) => NET_CC_CONFIG278, Y3(8) => NET_CC_CONFIG282, 
        Y3(9) => NET_CC_CONFIG286, Y3(10) => ADLIB_VCC, Y3(11)
         => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG251, Y3A(1) => 
        NET_CC_CONFIG255, Y3A(2) => NET_CC_CONFIG259, Y3A(3) => 
        NET_CC_CONFIG263, Y3A(4) => NET_CC_CONFIG267, Y3A(5) => 
        NET_CC_CONFIG271, Y3A(6) => NET_CC_CONFIG275, Y3A(7) => 
        NET_CC_CONFIG279, Y3A(8) => NET_CC_CONFIG283, Y3A(9) => 
        NET_CC_CONFIG287, Y3A(10) => ADLIB_VCC, Y3A(11) => 
        ADLIB_VCC, CC(0) => NET_CC_CONFIG252, CC(1) => 
        NET_CC_CONFIG256, CC(2) => NET_CC_CONFIG260, CC(3) => 
        NET_CC_CONFIG264, CC(4) => NET_CC_CONFIG268, CC(5) => 
        NET_CC_CONFIG272, CC(6) => NET_CC_CONFIG276, CC(7) => 
        NET_CC_CONFIG280, CC(8) => NET_CC_CONFIG284, CC(9) => 
        NET_CC_CONFIG288, CC(10) => nc218, CC(11) => nc342);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[28]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[5]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2250\, B
         => \can_top_level_inst/rx_buffer_inst/N_4265\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2283\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_0_3\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => \can_top_level_inst/prescaler_inst/sync_flag\, 
        B => 
        \can_top_level_inst/bus_sampling_inst_data_edge_detector_inst_rx_data_sync_prev\, 
        C => \can_top_level_inst/stat_bus[15]\, D => 
        \can_top_level_inst/bus_sampling_inst_prev_Sample\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_0_3_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[1]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.un5_sync_edge_i_a2\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \can_top_level_inst/drv_bus[460]\, B => 
        \can_top_level_inst/drv_bus[511]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1314\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[23]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[10]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0_0_0\ : 
        CFG3
      generic map(INIT => x"2A")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[2]\, C => 
        \can_top_level_inst/txtb_sw_cmd.set_abt_rep1\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0_0_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram94_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe94\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram94_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram44__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_354\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe44\);
    
    
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/un6_sync_flag_ce\ : 
        CFG3
      generic map(INIT => x"EA")

      port map(A => 
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/un1_h_or_re_sync_edge_Z\, 
        B => \can_top_level_inst/prescaler_inst/N_5058\, C => 
        \can_top_level_inst/prescaler_inst/is_tseg1\, Y => 
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/un6_sync_flag_ce_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[1]\);
    
    
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_d_1\ : 
        CFG4
      generic map(INIT => x"09F9")

      port map(A => can_tx_c, B => 
        \can_top_level_inst/rx_data_wbs\, C => 
        \can_top_level_inst/bus_sampling_inst/un1_ssp_enable_Z\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/un6_bit_err_ssp_valid_Z\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_d_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[17]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_25[24]\ : 
        CFG4
      generic map(INIT => x"C0A0")

      port map(A => \can_top_level_inst/drv_bus[324]\, B => 
        \can_top_level_inst/txtb_prorities_6[0]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1073\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => \adress_c[10]\, B => 
        \can_top_level_inst/txtb_allow_bb[6]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_1_Z\, 
        D => \adress_c[11]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_28_2[3]\ : 
        CFG4
      generic map(INIT => x"C480")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/stat_bus[103]\, D => 
        \can_top_level_inst/rx_read_buff[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1268_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[15]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[6]\ : 
        CFG4
      generic map(INIT => x"F808")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        B => \can_top_level_inst/stat_bus[155]\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        D => \can_top_level_inst/rx_buffer_inst/N_2284\, Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => \can_top_level_inst/drv_bus[37]\, B => 
        \can_top_level_inst/drv_bus[43]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_1_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_2_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_2_Z\, 
        CC => NET_CC_CONFIG1746, P => NET_CC_CONFIG1743, Y3 => 
        NET_CC_CONFIG1744, Y3A => NET_CC_CONFIG1745);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[25]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO_0[13]\ : 
        CFG4
      generic map(INIT => x"2755")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[13]\, C => 
        \can_top_level_inst/stat_bus[201]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_37_2_1_1_0[13]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_37_2_1_1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[2]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_36[21]\ : 
        CFG4
      generic map(INIT => x"88D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[53]\, C => 
        \can_top_level_inst/stat_bus[241]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_970\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[403]\, B => 
        \can_top_level_inst/stat_bus[84]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_2_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_3_Z\, 
        CC => NET_CC_CONFIG1541, P => NET_CC_CONFIG1538, Y3 => 
        NET_CC_CONFIG1539, Y3A => NET_CC_CONFIG1540);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[13]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[13]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[13]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[13]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[13]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[13]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2[0]\ : 
        CFG4
      generic map(INIT => x"5708")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_1[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1_Z[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_18[0]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_12_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_4\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[27]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_18_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_0_1[10]\ : 
        CFG3
      generic map(INIT => x"47")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_0_1_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[18]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[1]\);
    
    
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_d\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_d_1_Z\, 
        B => \can_top_level_inst/rx_triggers[1]\, C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/bit_err_detector_inst/bit_err_d_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[9]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[9]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[9]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_28_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_28_FCNET1\, 
        CO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid\, 
        CC => NET_CC_CONFIG645, P => NET_CC_CONFIG642, Y3 => 
        NET_CC_CONFIG643, Y3A => NET_CC_CONFIG644);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_13\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[401]\, B => 
        \can_top_level_inst/stat_bus[72]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_0\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_1\, 
        CC => NET_CC_CONFIG1989, P => NET_CC_CONFIG1986, Y3 => 
        NET_CC_CONFIG1987, Y3A => NET_CC_CONFIG1988);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r_10_rep2\ : 
        SLE
      port map(D => \data_in_c[10]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus_475_rep2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un2_select_index_changed_NE\ : 
        CFG4
      generic map(INIT => x"FFDE")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[1]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/un2_select_index_changed_2_Z\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/un2_select_index_changed_0\, 
        Y => \can_top_level_inst/tx_arbitrator_inst/N_109\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI8J6I[0]_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO1271, CO => CI_TO_CO1272, P(0) => 
        NET_CC_CONFIG1298, P(1) => NET_CC_CONFIG1302, P(2) => 
        NET_CC_CONFIG1306, P(3) => NET_CC_CONFIG1310, P(4) => 
        NET_CC_CONFIG1314, P(5) => NET_CC_CONFIG1318, P(6) => 
        NET_CC_CONFIG1322, P(7) => NET_CC_CONFIG1326, P(8) => 
        NET_CC_CONFIG1330, P(9) => NET_CC_CONFIG1334, P(10) => 
        NET_CC_CONFIG1338, P(11) => NET_CC_CONFIG1342, Y3(0) => 
        NET_CC_CONFIG1299, Y3(1) => NET_CC_CONFIG1303, Y3(2) => 
        NET_CC_CONFIG1307, Y3(3) => NET_CC_CONFIG1311, Y3(4) => 
        NET_CC_CONFIG1315, Y3(5) => NET_CC_CONFIG1319, Y3(6) => 
        NET_CC_CONFIG1323, Y3(7) => NET_CC_CONFIG1327, Y3(8) => 
        NET_CC_CONFIG1331, Y3(9) => NET_CC_CONFIG1335, Y3(10) => 
        NET_CC_CONFIG1339, Y3(11) => NET_CC_CONFIG1343, Y3A(0)
         => NET_CC_CONFIG1300, Y3A(1) => NET_CC_CONFIG1304, 
        Y3A(2) => NET_CC_CONFIG1308, Y3A(3) => NET_CC_CONFIG1312, 
        Y3A(4) => NET_CC_CONFIG1316, Y3A(5) => NET_CC_CONFIG1320, 
        Y3A(6) => NET_CC_CONFIG1324, Y3A(7) => NET_CC_CONFIG1328, 
        Y3A(8) => NET_CC_CONFIG1332, Y3A(9) => NET_CC_CONFIG1336, 
        Y3A(10) => NET_CC_CONFIG1340, Y3A(11) => 
        NET_CC_CONFIG1344, CC(0) => NET_CC_CONFIG1301, CC(1) => 
        NET_CC_CONFIG1305, CC(2) => NET_CC_CONFIG1309, CC(3) => 
        NET_CC_CONFIG1313, CC(4) => NET_CC_CONFIG1317, CC(5) => 
        NET_CC_CONFIG1321, CC(6) => NET_CC_CONFIG1325, CC(7) => 
        NET_CC_CONFIG1329, CC(8) => NET_CC_CONFIG1333, CC(9) => 
        NET_CC_CONFIG1337, CC(10) => NET_CC_CONFIG1341, CC(11)
         => NET_CC_CONFIG1345);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[5]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/read_pointer_i_RNO[0]\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => \can_top_level_inst/rx_read_pointer[0]\, Y
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_read_pointer_i[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[15]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2639\, Y
         => \can_top_level_inst/memory_registers_inst/N_2704\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[31]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[31]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[26]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[26]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram2_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[11]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/fsm_wait_state_q_RNO\ : 
        CFG4
      generic map(INIT => x"3133")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/fsm_wait_state_d_i_1_Z\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_100_i_1\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_100_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\);
    
    \data_out_obuf[9]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(9), D => \data_out_obuf[9]/DOUT\, 
        E => \data_out_obuf[9]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[28]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_0_a3[13]\ : 
        CFG4
      generic map(INIT => x"0903")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1206\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_909\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[28]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_dest[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[29]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry[5]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[4]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[5]\, 
        CC => NET_CC_CONFIG312, P => NET_CC_CONFIG309, Y3 => 
        NET_CC_CONFIG310, Y3A => NET_CC_CONFIG311);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[5]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[5]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4[5]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry[3]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[3]\, 
        D => \can_top_level_inst/rx_write_pointer[3]\, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[2]\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[3]\, 
        CC => NET_CC_CONFIG381, P => NET_CC_CONFIG378, Y3 => 
        NET_CC_CONFIG379, Y3A => NET_CC_CONFIG380);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[8]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[16]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_2_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[2]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[2]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_shadowed[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/trv_delay[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[27]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[27]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_2\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        B => \can_top_level_inst/can_core_inst/act_err_ovr_flag\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_2_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe8\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[5]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[5]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[27]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[27]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[27]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c[1]\ : 
        CFG4
      generic map(INIT => x"CE02")

      port map(A => \can_top_level_inst/store_data_word[0]\, B
         => \can_top_level_inst/stat_bus[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_data_len_can_2_0_i_0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_1\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[1]\, B => 
        \timestamp_c[32]\, C => \timestamp_c[33]\, D => 
        \can_top_level_inst/tran_word[0]\, FCI => ADLIB_GND, S
         => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[0]\, 
        CC => NET_CC_CONFIG2253, P => NET_CC_CONFIG2250, Y3 => 
        NET_CC_CONFIG2251, Y3A => NET_CC_CONFIG2252);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_2[0]\ : 
        CFG4
      generic map(INIT => x"FFCD")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_971\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_drv_rom_ena_m\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_854_i\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_0_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_0_iv_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[13]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[45]\, 
        C => \can_top_level_inst/rec_ident[13]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2258\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q_RNIBTKG4[5]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/stat_bus[86]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_4\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_cry_5\, 
        CC => NET_CC_CONFIG349, P => NET_CC_CONFIG346, Y3 => 
        NET_CC_CONFIG347, Y3A => NET_CC_CONFIG348);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[16]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[16]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[16]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[17]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[17]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[12]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[13]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[13]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[13]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_8_RNIGLV7F6\ : 
        ARI1_CC
      generic map(INIT => x"4D2F0")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[7]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_6\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_7\, 
        CC => NET_CC_CONFIG1155, P => NET_CC_CONFIG1152, Y3 => 
        NET_CC_CONFIG1153, Y3A => NET_CC_CONFIG1154);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/N_4197\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_7[30]\ : 
        CFG4
      generic map(INIT => x"4800")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_77_mux\, 
        C => \can_top_level_inst/tran_frame_test.tprm[1]\, D => 
        \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_83_mux\);
    
    \timestamp_ibuf[37]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(37), Y => 
        \timestamp_ibuf[37]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/sel_data_1[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_45\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[14]\, 
        B => ADLIB_VCC, C => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_I_50\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_12_1\ : 
        CFG3
      generic map(INIT => x"A2")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_26\, 
        C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[4]\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_12_1_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[12]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/commit_m1\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[0]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un2_select_index_changed_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un7_rx_err_ctr_declto8_i_a2_0_a2_RNITG6E\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/stat_bus[81]\, B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_GND, S => 
        OPEN, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNITG6E_Y\, 
        FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_0\, 
        CC => NET_CC_CONFIG2318, P => NET_CC_CONFIG2315, Y3 => 
        NET_CC_CONFIG2316, Y3A => NET_CC_CONFIG2317);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_13_2[9]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/drv_bus[409]\, B => 
        \can_top_level_inst/rx_read_buff[9]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1481_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[3]\ : 
        CFG4
      generic map(INIT => x"2232")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_37_2_0[18]\ : 
        CFG4
      generic map(INIT => x"FD0D")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \timestamp_c[18]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => \timestamp_c[50]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_37_2_0_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[21]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2[26]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/tran_identifier[12]\, B
         => \can_top_level_inst/tran_identifier[23]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/crc_spec_enable_i\ : 
        CFG4
      generic map(INIT => x"AAAE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_996\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_N_865\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_2_2[6]\ : 
        CFG4
      generic map(INIT => x"551B")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => \can_top_level_inst/int_ena[6]\, C => 
        \can_top_level_inst/drv_bus[277]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1_Z[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_2_1[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[11]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[11]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[4]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_161_2\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_170_1\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        D => \can_top_level_inst/txtb_parity_error_valid[7]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_170\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_write_address[6]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[6]\, 
        C => \can_top_level_inst/rx_buffer_inst/N_48\, Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/set_idle_i_15_sqmuxa_0_a4\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        Y => \can_top_level_inst/stat_bus[372]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31[9]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_31_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_22\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_mismatch_1_22_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[20]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_31_RNO[12]\ : 
        CFG4
      generic map(INIT => x"F252")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_2_0_1_Z[12]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_2_1_1[12]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_617\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_622\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[18]\);
    
    
        \can_top_level_inst/prescaler_inst/synchronisation_checker_inst/g0_0\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \can_top_level_inst/prescaler_inst/is_tseg2\, 
        B => \can_top_level_inst/prescaler_inst/current_state[1]\, 
        Y => \can_top_level_inst/prescaler_inst/g0_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_a4_0[1]\ : 
        CFG4
      generic map(INIT => x"F020")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_937\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_973\, 
        C => \can_top_level_inst/drv_bus[513]\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1113\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_4_0\ : 
        ARI1_CC
      generic map(INIT => x"5CC66")

      port map(A => \can_top_level_inst/drv_bus[50]\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[4]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_3\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_cry_4\, 
        CC => NET_CC_CONFIG2676, P => NET_CC_CONFIG2673, Y3 => 
        NET_CC_CONFIG2674, Y3A => NET_CC_CONFIG2675);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[27]\ : 
        SLE
      port map(D => \data_in_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[298]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[31]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[2]\, D => 
        \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m19_1_0_wmux\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i69_mux\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/i71_mux\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m19_1_0_y0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[19]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm_0[0]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, B => 
        \can_top_level_inst/prescaler_inst/segment_end\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[0]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[31]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2655\, Y
         => \can_top_level_inst/memory_registers_inst/N_2720\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI0VMF[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => ADLIB_VCC, 
        P(1) => ADLIB_VCC, P(2) => ADLIB_GND, P(3) => 
        NET_CC_CONFIG2629, P(4) => NET_CC_CONFIG2633, P(5) => 
        NET_CC_CONFIG2637, P(6) => NET_CC_CONFIG2641, P(7) => 
        NET_CC_CONFIG2645, P(8) => NET_CC_CONFIG2649, P(9) => 
        ADLIB_VCC, P(10) => ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0)
         => ADLIB_VCC, Y3(1) => ADLIB_VCC, Y3(2) => ADLIB_VCC, 
        Y3(3) => NET_CC_CONFIG2630, Y3(4) => NET_CC_CONFIG2634, 
        Y3(5) => NET_CC_CONFIG2638, Y3(6) => NET_CC_CONFIG2642, 
        Y3(7) => NET_CC_CONFIG2646, Y3(8) => NET_CC_CONFIG2650, 
        Y3(9) => ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => 
        ADLIB_VCC, Y3A(0) => ADLIB_VCC, Y3A(1) => ADLIB_VCC, 
        Y3A(2) => ADLIB_VCC, Y3A(3) => NET_CC_CONFIG2631, Y3A(4)
         => NET_CC_CONFIG2635, Y3A(5) => NET_CC_CONFIG2639, 
        Y3A(6) => NET_CC_CONFIG2643, Y3A(7) => NET_CC_CONFIG2647, 
        Y3A(8) => NET_CC_CONFIG2651, Y3A(9) => ADLIB_VCC, Y3A(10)
         => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => nc106, 
        CC(1) => nc261, CC(2) => nc25, CC(3) => NET_CC_CONFIG2632, 
        CC(4) => NET_CC_CONFIG2636, CC(5) => NET_CC_CONFIG2640, 
        CC(6) => NET_CC_CONFIG2644, CC(7) => NET_CC_CONFIG2648, 
        CC(8) => NET_CC_CONFIG2652, CC(9) => nc1, CC(10) => nc322, 
        CC(11) => nc299);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[7]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[16]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[17]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[9]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[16]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[16]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[16]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4820\);
    
    \timestamp_ibuf[1]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[1]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[1]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_3[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/shift_reg_in_6[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un25_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[6]\ : 
        CFG4
      generic map(INIT => x"0E00")

      port map(A => \can_top_level_inst/txtb_hw_cmd.arbl\, B => 
        \can_top_level_inst/txtb_hw_cmd.err\, C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_136\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m35_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[31]\);
    
    \adress_ibuf[6]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \adress_c[6]\, E => ADLIB_GND, YIN => 
        \adress_ibuf[6]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_11[0]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_11_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[9]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIK7JE8_S[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[268]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[21]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[21]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[21]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[15]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_0_o2_1[14]\ : 
        CFG4
      generic map(INIT => x"FFF9")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[1]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1194\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1196\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/curr_state_ns_0[1]\ : 
        CFG4
      generic map(INIT => x"A0E4")

      port map(A => \can_top_level_inst/stat_bus[108]\, B => 
        \can_top_level_inst/is_bus_off\, C => 
        \can_top_level_inst/stat_bus[79]\, D => 
        \can_top_level_inst/can_core_inst/set_err_active\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/curr_state_ns[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/current_state_s0_0_a2\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \can_top_level_inst/prescaler_inst/is_tseg2\, 
        B => \can_top_level_inst/prescaler_inst/current_state[1]\, 
        Y => \can_top_level_inst/prescaler_inst/is_tseg1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[25]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[168]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[22]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_1\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_0_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_1_Z\, 
        CC => NET_CC_CONFIG1489, P => NET_CC_CONFIG1486, Y3 => 
        NET_CC_CONFIG1487, Y3A => NET_CC_CONFIG1488);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[12]\ : 
        CFG4
      generic map(INIT => x"F088")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/frame_form_w_9[12]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        C => \can_top_level_inst/rx_buffer_inst/N_2290\, D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m19_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[13]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[13]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[13]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d\ : 
        CFG4
      generic map(INIT => x"1300")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_FCO\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_Z\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un7_rx_err_ctr_declto8_i_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => \can_top_level_inst/stat_bus[87]\, B => 
        \can_top_level_inst/stat_bus[89]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_4\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_5\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => \can_top_level_inst/drv_bus[21]\, B => 
        \can_top_level_inst/drv_bus[28]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_4_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_5_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_nbt_d_cry_5_Z\, 
        CC => NET_CC_CONFIG2167, P => NET_CC_CONFIG2164, Y3 => 
        NET_CC_CONFIG2165, Y3A => NET_CC_CONFIG2166);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d[1]\ : 
        CFG4
      generic map(INIT => x"7080")

      port map(A => \can_top_level_inst/stat_bus[300]\, B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un4_bst_ctr_d_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/reg_q\, 
        D => \can_top_level_inst/stat_bus[301]\, Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/bst_ctr_d_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[0]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[24]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIG3FI5_S[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[264]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_0_y0[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[1]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram2_0\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram3_0\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_0_y1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[26]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[26]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[26]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[4]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[4]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2628\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_15\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[5]\, B => 
        \timestamp_c[36]\, C => \timestamp_c[37]\, D => 
        \can_top_level_inst/tran_word[4]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[1]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[2]\, 
        CC => NET_CC_CONFIG2261, P => NET_CC_CONFIG2258, Y3 => 
        NET_CC_CONFIG2259, Y3A => NET_CC_CONFIG2260);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[6]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[6]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\);
    
    \res_n_ibuf/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => res_n_c, E => ADLIB_GND, YIN => 
        \res_n_ibuf/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[24]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[24]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[24]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_27\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[15]\, B => 
        \timestamp_c[46]\, C => \timestamp_c[47]\, D => 
        \can_top_level_inst/tran_word[14]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[6]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[7]\, 
        CC => NET_CC_CONFIG2281, P => NET_CC_CONFIG2278, Y3 => 
        NET_CC_CONFIG2279, Y3A => NET_CC_CONFIG2280);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOLJ46_S[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[227]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_2_1[1]\ : 
        CFG3
      generic map(INIT => x"7F")

      port map(A => \can_top_level_inst/drv_bus[475]\, B => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[2]\, 
        C => \can_top_level_inst/txtb_allow_bb[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_2_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_6\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[405]\, B => 
        \can_top_level_inst/stat_bus[86]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_4\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un27_status_comb_cry_5\, 
        CC => NET_CC_CONFIG1188, P => NET_CC_CONFIG1185, Y3 => 
        NET_CC_CONFIG1186, Y3A => NET_CC_CONFIG1187);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[24]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_6[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[20]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[31]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[16]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[16]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_RNO_0[27]\ : 
        CFG4
      generic map(INIT => x"7FFF")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_20_0\, 
        C => \can_top_level_inst/tran_frame_test.fcrc\, D => 
        \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/m42_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram30_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe30\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram30_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[2]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[2]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_126_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_y0_0[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1152\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1153\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_0_y3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/current_state_RNO[0]\ : 
        CFG4
      generic map(INIT => x"0220")

      port map(A => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        B => \can_top_level_inst/prescaler_inst/current_state[1]\, 
        C => \can_top_level_inst/prescaler_inst/segment_end\, D
         => \can_top_level_inst/prescaler_inst/is_tseg2\, Y => 
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/N_33_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe19\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[21]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[24]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[24]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO_1[15]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_37_2_1_1_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_21[15]\ : 
        CFG4
      generic map(INIT => x"F4F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_727_2\, 
        D => \can_top_level_inst/drv_bus[220]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_727\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[15]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[15]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_8\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_true_gen.parity_calculator_write_inst/parity_8_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_2[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[4]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[6]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[5]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[3]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_146\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[3]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[3]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram8_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe8\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram8_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_1[6]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[13]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un9_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\);
    
    \timestamp_ibuf[54]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(54), Y => 
        \timestamp_ibuf[54]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o3[1]\ : 
        CFG3
      generic map(INIT => x"F2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_138\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa16\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe16\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[22]\);
    
    AFLSDF_INV_10 : INV_BA
      port map(A => \AFLSDF_INV_10\, Y => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_control_reg_comp/wr_select_4[0]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/un3_test_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/addr_dec_3[1]\, 
        D => \adress_c[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_control_reg_comp/wr_select_4_Z[0]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/un2_crc_ena_17_21\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_N_865\, 
        B => \can_top_level_inst/can_core_inst/crc_data_rx_wbs\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/un2_crc_ena_17_21_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[19]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q_RNO[12]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_15[11]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram107_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe107\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram107_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[0]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[0]\);
    
    \scs_ibuf/U_IOPAD\ : IOPAD_IN
      port map(PAD => scs, Y => \scs_ibuf/YIN\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIU4MKH_S[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[245]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_0[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_2[1]\ : 
        CFG3
      generic map(INIT => x"8C")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_183\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_195\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_1_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO1763, CO => OPEN, P(0) => 
        NET_CC_CONFIG1788, P(1) => NET_CC_CONFIG1792, P(2) => 
        NET_CC_CONFIG1796, P(3) => NET_CC_CONFIG1800, P(4) => 
        NET_CC_CONFIG1804, P(5) => NET_CC_CONFIG1808, P(6) => 
        NET_CC_CONFIG1812, P(7) => NET_CC_CONFIG1816, P(8) => 
        NET_CC_CONFIG1820, P(9) => NET_CC_CONFIG1824, P(10) => 
        ADLIB_VCC, P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1789, 
        Y3(1) => NET_CC_CONFIG1793, Y3(2) => NET_CC_CONFIG1797, 
        Y3(3) => NET_CC_CONFIG1801, Y3(4) => NET_CC_CONFIG1805, 
        Y3(5) => NET_CC_CONFIG1809, Y3(6) => NET_CC_CONFIG1813, 
        Y3(7) => NET_CC_CONFIG1817, Y3(8) => NET_CC_CONFIG1821, 
        Y3(9) => NET_CC_CONFIG1825, Y3(10) => ADLIB_VCC, Y3(11)
         => ADLIB_VCC, Y3A(0) => NET_CC_CONFIG1790, Y3A(1) => 
        NET_CC_CONFIG1794, Y3A(2) => NET_CC_CONFIG1798, Y3A(3)
         => NET_CC_CONFIG1802, Y3A(4) => NET_CC_CONFIG1806, 
        Y3A(5) => NET_CC_CONFIG1810, Y3A(6) => NET_CC_CONFIG1814, 
        Y3A(7) => NET_CC_CONFIG1818, Y3A(8) => NET_CC_CONFIG1822, 
        Y3A(9) => NET_CC_CONFIG1826, Y3A(10) => ADLIB_VCC, 
        Y3A(11) => ADLIB_VCC, CC(0) => NET_CC_CONFIG1791, CC(1)
         => NET_CC_CONFIG1795, CC(2) => NET_CC_CONFIG1799, CC(3)
         => NET_CC_CONFIG1803, CC(4) => NET_CC_CONFIG1807, CC(5)
         => NET_CC_CONFIG1811, CC(6) => NET_CC_CONFIG1815, CC(7)
         => NET_CC_CONFIG1819, CC(8) => NET_CC_CONFIG1823, CC(9)
         => NET_CC_CONFIG1827, CC(10) => nc37, CC(11) => nc202);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[31]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[13]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[13]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[12]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_10[28]\ : 
        CFG4
      generic map(INIT => x"FFF8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac23_Z\, 
        B => \can_top_level_inst/drv_bus[109]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_6_Z[28]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[668]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_10_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_RNI5RBQ3[5]\ : 
        ARI1_CC
      generic map(INIT => x"58DD8")

      port map(A => \can_top_level_inst/stat_bus[74]\, B => 
        \can_top_level_inst/drv_bus[425]\, C => 
        \can_top_level_inst/drv_bus[419]\, D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[5]\, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_2\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_3\, 
        CC => NET_CC_CONFIG2502, P => NET_CC_CONFIG2499, Y3 => 
        NET_CC_CONFIG2500, Y3A => NET_CC_CONFIG2501);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\);
    
    \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_4\ : 
        CFG4
      generic map(INIT => x"0090")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[6]\, 
        B => \can_top_level_inst/rx_read_pointer[6]\, C => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_2_Z\, 
        D => \can_top_level_inst/rx_buffer_inst/is_free_word_1_Z\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/un1_overrun_condition_4_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_o2_0[3]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[10]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/drv_bus_off_reset_q_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_966\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2[12]\ : 
        CFG3
      generic map(INIT => x"5C")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_1[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[12]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[18]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[7]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3_1[29]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[61]\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2307_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram18_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe18\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un4_tst_ena_3\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_control[0]\, B
         => \can_top_level_inst/reg_value_r_rep[17]\, C => 
        \can_top_level_inst/test_registers_out.tst_dest[19]\, D
         => \can_top_level_inst/test_registers_out.tst_dest[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_o2_0_RNIPRVNV[3]\ : 
        CFG4
      generic map(INIT => x"0041")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[0]\, 
        D => \can_top_level_inst/tx_arbitrator_inst/N_104\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/commit_m5_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_38_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_683\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_698\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_38_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[13]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram31__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_338\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_351\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe31\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_2_0[1]\ : 
        CFG4
      generic map(INIT => x"F588")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1175\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1176\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_0_Z[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_2_0_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[18]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram18_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram19_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/N_4795\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_0[6]\ : 
        CFG4
      generic map(INIT => x"7FFF")

      port map(A => \can_top_level_inst/txtb_hw_cmd_index[0]\, B
         => \can_top_level_inst/stat_bus[68]\, C => 
        \can_top_level_inst/txtb_hw_cmd_index[1]\, D => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_146\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_0_iv[1]\ : 
        CFG3
      generic map(INIT => x"AE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_switch_to_ssp_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q[1]\, 
        C => \can_top_level_inst/br_shifted\, Y => 
        \can_top_level_inst/stat_bus[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[23]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_y0_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[26]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fcs_changed_1_NE\ : 
        CFG4
      generic map(INIT => x"B3EC")

      port map(A => \can_top_level_inst/is_bus_off\, B => 
        \can_top_level_inst/stat_bus[108]\, C => 
        \can_top_level_inst/can_core_inst/set_err_active\, D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/N_27_i\, 
        Y => \can_top_level_inst/fcs_changed\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/un2_data_len_can_2_0_1.CO3\ : 
        CFG4
      generic map(INIT => x"AAA8")

      port map(A => \can_top_level_inst/tran_dlc[3]\, B => 
        \can_top_level_inst/tran_dlc[2]\, C => 
        \can_top_level_inst/tran_dlc[1]\, D => 
        \can_top_level_inst/tran_dlc[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/un2_data_len_can_2_0_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y3[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[25]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_60_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[25]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[25]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_87_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_87_FCNET1\, 
        CO => 
        \can_top_level_inst/frame_filters_inst/valid_0_I_87_FCO\, 
        CC => NET_CC_CONFIG1827, P => NET_CC_CONFIG1824, Y3 => 
        NET_CC_CONFIG1825, Y3A => NET_CC_CONFIG1826);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_4[1]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[1]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_26_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[13]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[31]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_s_0_2645_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO1205, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => 
        ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_VCC, P(6) => 
        ADLIB_VCC, P(7) => ADLIB_VCC, P(8) => ADLIB_VCC, P(9) => 
        ADLIB_VCC, P(10) => ADLIB_GND, P(11) => NET_CC_CONFIG1207, 
        Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, Y3(2) => 
        ADLIB_VCC, Y3(3) => ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5)
         => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7) => ADLIB_VCC, 
        Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, Y3(10) => 
        ADLIB_GND, Y3(11) => NET_CC_CONFIG1208, Y3A(0) => 
        ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => ADLIB_VCC, 
        Y3A(3) => ADLIB_VCC, Y3A(4) => ADLIB_VCC, Y3A(5) => 
        ADLIB_VCC, Y3A(6) => ADLIB_VCC, Y3A(7) => ADLIB_VCC, 
        Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => 
        ADLIB_GND, Y3A(11) => NET_CC_CONFIG1209, CC(0) => nc144, 
        CC(1) => nc153, CC(2) => nc46, CC(3) => nc258, CC(4) => 
        nc343, CC(5) => nc71, CC(6) => nc124, CC(7) => nc332, 
        CC(8) => nc81, CC(9) => nc201, CC(10) => nc168, CC(11)
         => NET_CC_CONFIG1210);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe24_0_a4\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_340\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[268]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_m3[1]\ : 
        CFG4
      generic map(INIT => x"EFE0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => \can_top_level_inst/txtb_parity_error_valid[7]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_148\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_149\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_150\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[14]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[28]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[28]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select[2]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0_1\, 
        B => \sbe_c[2]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s_2642\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_VCC, S => 
        OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s_2642_FCO\, 
        CC => NET_CC_CONFIG2346, P => NET_CC_CONFIG2343, Y3 => 
        NET_CC_CONFIG2344, Y3A => NET_CC_CONFIG2345);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIPLUQ1[1]\ : 
        ARI1_CC
      generic map(INIT => x"5D827")

      port map(A => \timestamp_c[33]\, B => 
        \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[1]\, 
        FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_0\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_1\, 
        CC => NET_CC_CONFIG931, P => NET_CC_CONFIG928, Y3 => 
        NET_CC_CONFIG929, Y3A => NET_CC_CONFIG930);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_18\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_18_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram46__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_354\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe46\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[25]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[25]\);
    
    \timestamp_ibuf[8]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(8), Y => \timestamp_ibuf[8]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3\ : 
        RGB
      port map(A => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_Y\, 
        EN => ADLIB_VCC, Y => \AFLSDF_INV_16\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[8]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.7.int_module_inst/int_ena_proc.un1_int_ena_clear\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[7]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.7.int_module_inst/un1_int_ena_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram31_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe31\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram31_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[28]\);
    
    I_1 : GB
      port map(A => clk_sys_ibuf_Z, EN => ADLIB_VCC, Y => 
        \I_1/U0_Y\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_3_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[31]\);
    
    \data_out_obuf[0]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(0), D => \data_out_obuf[0]/DOUT\, 
        E => \data_out_obuf[0]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_9[30]\ : 
        CFG4
      generic map(INIT => x"FDFF")

      port map(A => \can_top_level_inst/drv_bus[477]\, B => 
        \can_top_level_inst/tran_frame_test.tprm[1]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_12_0_o4_0_3\, 
        D => \can_top_level_inst/tran_frame_test.tprm[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_8_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_o3[1]\ : 
        CFG3
      generic map(INIT => x"F2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_138\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[219]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_15_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1357\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1360\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_15_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_22\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_22_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[23]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm_0[4]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[4]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[422]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[21]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_4_1_0\ : 
        CFG4
      generic map(INIT => x"5554")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g1_2_Z\, 
        B => \can_top_level_inst/prescaler_inst_tq_edge_nbt\, C
         => \can_top_level_inst/stat_bus[10]\, D => 
        \can_top_level_inst/stat_bus[11]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/g0_4_1_0_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNISK1R2[4]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[4]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_3\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_4\, 
        CC => NET_CC_CONFIG1293, P => NET_CC_CONFIG1290, Y3 => 
        NET_CC_CONFIG1291, Y3A => NET_CC_CONFIG1292);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[5]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[6]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_RNO[2]\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un3_same_bits_rst_i_0\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/N_10\, 
        C => \can_top_level_inst/can_core_inst/tx_frame_no_sof\, 
        D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[31]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[28]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIO8KOB[31]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[31]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[31]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_30\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIO8KOB[31]_FCNET1\, 
        CC => NET_CC_CONFIG1401, P => NET_CC_CONFIG1398, Y3 => 
        NET_CC_CONFIG1399, Y3A => NET_CC_CONFIG1400);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_15[0]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_9_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_928_i\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[4]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[30]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_15_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un19_timestamp_valid_axb_31_i_RNIJ9GCA9\ : 
        CFG3
      generic map(INIT => x"1D")

      port map(A => \timestamp_c[63]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_axb_31_i\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_30\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i_1_0\);
    
    \timestamp_ibuf[58]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(58), Y => 
        \timestamp_ibuf[58]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[3]\ : 
        CFG3
      generic map(INIT => x"AE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_168\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[17]\);
    
    \timestamp_ibuf[60]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(60), Y => 
        \timestamp_ibuf[60]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/dff_ena_reg/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_stuff_enable\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/reg_q\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[12]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[12]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_0_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO1828, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_GND, P(3) => 
        NET_CC_CONFIG1830, P(4) => NET_CC_CONFIG1834, P(5) => 
        NET_CC_CONFIG1838, P(6) => NET_CC_CONFIG1842, P(7) => 
        NET_CC_CONFIG1846, P(8) => NET_CC_CONFIG1850, P(9) => 
        NET_CC_CONFIG1854, P(10) => NET_CC_CONFIG1858, P(11) => 
        NET_CC_CONFIG1862, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => ADLIB_VCC, Y3(3) => NET_CC_CONFIG1831, Y3(4) => 
        NET_CC_CONFIG1835, Y3(5) => NET_CC_CONFIG1839, Y3(6) => 
        NET_CC_CONFIG1843, Y3(7) => NET_CC_CONFIG1847, Y3(8) => 
        NET_CC_CONFIG1851, Y3(9) => NET_CC_CONFIG1855, Y3(10) => 
        NET_CC_CONFIG1859, Y3(11) => NET_CC_CONFIG1863, Y3A(0)
         => ADLIB_VCC, Y3A(1) => ADLIB_VCC, Y3A(2) => ADLIB_VCC, 
        Y3A(3) => NET_CC_CONFIG1832, Y3A(4) => NET_CC_CONFIG1836, 
        Y3A(5) => NET_CC_CONFIG1840, Y3A(6) => NET_CC_CONFIG1844, 
        Y3A(7) => NET_CC_CONFIG1848, Y3A(8) => NET_CC_CONFIG1852, 
        Y3A(9) => NET_CC_CONFIG1856, Y3A(10) => NET_CC_CONFIG1860, 
        Y3A(11) => NET_CC_CONFIG1864, CC(0) => nc323, CC(1) => 
        nc34, CC(2) => nc28, CC(3) => NET_CC_CONFIG1833, CC(4)
         => NET_CC_CONFIG1837, CC(5) => NET_CC_CONFIG1841, CC(6)
         => NET_CC_CONFIG1845, CC(7) => NET_CC_CONFIG1849, CC(8)
         => NET_CC_CONFIG1853, CC(9) => NET_CC_CONFIG1857, CC(10)
         => NET_CC_CONFIG1861, CC(11) => NET_CC_CONFIG1865);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[17]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm_0[2]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[2]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[24]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram75_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe75\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram75_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[30]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[30]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[30]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[3]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[13]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[13]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_0\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[0]\, 
        B => \can_top_level_inst/drv_bus[51]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => OPEN, 
        FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un2_phase_err_mt_sjw_cry_0_Z\, 
        CC => NET_CC_CONFIG2378, P => NET_CC_CONFIG2375, Y3 => 
        NET_CC_CONFIG2376, Y3A => NET_CC_CONFIG2377);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_4[29]\ : 
        CFG4
      generic map(INIT => x"ECFF")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac8_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_0_Z[29]\, 
        C => \can_top_level_inst/txtb_prorities_7[1]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_4_1_Z[29]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_4_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram97_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe97\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram97_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac23\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_0_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac23_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_fsm_inst_N_159\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_y0_0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y3[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_34_2[4]\ : 
        CFG4
      generic map(INIT => x"D800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[36]\, C => 
        \can_top_level_inst/stat_bus[224]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1312_2\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_4_1[29]\ : 
        CFG4
      generic map(INIT => x"135F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5_Z\, 
        B => \timestamp_c[29]\, C => 
        \can_top_level_inst/stat_bus[217]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac1_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_4_1_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_o4_0\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[14]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1012\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE\ : 
        CFG4
      generic map(INIT => x"FEFF")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_2_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_4_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_3_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un6_phase_err_sjw_by_one_NE_1_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/un6_phase_err_sjw_by_one_NE\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[212]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m22_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m30_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[2]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_804_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/destuff_ena_reg_proc.un21_ctrl_signal_upd_i_o2\ : 
        CFG3
      generic map(INIT => x"F1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_952\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_go_to_suspend_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_941\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[7]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[242]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[1]\);
    
    \timestamp_ibuf[5]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(5), Y => \timestamp_ibuf[5]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un8_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_2_0[22]\ : 
        CFG4
      generic map(INIT => x"3708")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_2_0_1_Z[22]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_1_0_Z[22]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_2_0_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[16]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_8_x\ : 
        CFG3
      generic map(INIT => x"07")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\, 
        B => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        C => \can_top_level_inst/stat_bus[11]\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_8_x_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_0\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[19]\, B
         => \can_top_level_inst/can_core_inst/crc_21[18]\, C => 
        \can_top_level_inst/can_core_inst/crc_21[19]\, D => 
        \can_top_level_inst/store_data_word[18]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[8]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[9]\, 
        CC => NET_CC_CONFIG38, P => NET_CC_CONFIG35, Y3 => 
        NET_CC_CONFIG36, Y3A => NET_CC_CONFIG37);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[59]\ : 
        SLE
      port map(D => \timestamp_c[59]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[59]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[25]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[25]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2649\, Y
         => \can_top_level_inst/memory_registers_inst/N_2714\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.0.l2_prio_2_1.CO2\ : 
        CFG3
      generic map(INIT => x"B2")

      port map(A => \can_top_level_inst/txtb_prorities_2[2]\, B
         => \can_top_level_inst/txtb_prorities_0[2]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_0\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_023\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0_1\, 
        B => \sbe_c[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_3\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => \can_top_level_inst/rx_data_wbs\, B => 
        \can_top_level_inst/prescaler_inst/sync_flag\, C => 
        \can_top_level_inst/stat_bus[14]\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_3_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[2]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_0[5]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[4]\, CLK
         => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[18]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[18]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[18]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_1_1[1]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \adress_c[3]\, B => \adress_c[4]\, C => 
        \adress_c[6]\, D => swr_c, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_1_1_Z[1]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[9]\ : 
        CFG4
      generic map(INIT => x"F088")

      port map(A => \can_top_level_inst/rec_brs\, B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        C => \can_top_level_inst/rx_buffer_inst/N_2287\, D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[9]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[12]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[12]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[4]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_1\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[407]\, B => 
        \can_top_level_inst/stat_bus[78]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_6\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/memory_registers_inst/un20_status_comb_cry_7\, 
        CC => NET_CC_CONFIG2013, P => NET_CC_CONFIG2010, Y3 => 
        NET_CC_CONFIG2011, Y3A => NET_CC_CONFIG2012);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[8]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_22\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_22_Z\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_segm_counter_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_2_1_0_y0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[18]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_4[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNI131G3[0]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/txtb_port_b_address[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/N_4117\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/N_4118\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_17[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram124_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram125_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[5]\ : 
        CFG4
      generic map(INIT => x"080A")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_4\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[5]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1035\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_834_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[20]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[20]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_2[28]\ : 
        CFG4
      generic map(INIT => x"FEFC")

      port map(A => \timestamp_c[60]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_0_Z[28]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7_Z\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_2_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe20\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram20_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_read_address[2]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => \can_top_level_inst/rx_read_pointer[2]\, C => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[20]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNO[6]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_q_cry_5\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNO_S[6]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG1669, P => 
        NET_CC_CONFIG1666, Y3 => NET_CC_CONFIG1667, Y3A => 
        NET_CC_CONFIG1668);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[22]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[22]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_dlc[1]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[0]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_dlc\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_dlc[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_10[10]\ : 
        CFG4
      generic map(INIT => x"F2F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1516_1\, 
        D => \can_top_level_inst/drv_bus[44]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1516\);
    
    
        \can_top_level_inst/can_core_inst/trigger_mux_inst/un3_pc_rx_trigger\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => \can_top_level_inst/stat_bus[184]\, B => 
        \can_top_level_inst/can_core_inst/destuffed\, Y => 
        \can_top_level_inst/can_core_inst/pc_rx_trigger\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[22]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[136]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[16]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[20]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[15]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNICB062[5]\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[5]\, 
        B => \can_top_level_inst/drv_bus[380]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_4\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNICB062_S[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_5\, 
        CC => NET_CC_CONFIG1589, P => NET_CC_CONFIG1586, Y3 => 
        NET_CC_CONFIG1587, Y3A => NET_CC_CONFIG1588);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_38_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1293\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1308\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_38_1_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[19]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[24]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[24]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[24]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[15]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2704\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2671\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[15]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_d[1]\ : 
        CFG3
      generic map(INIT => x"14")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un17_same_bits_erase_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_q_Z[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/CO0_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/same_bits_d_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[1]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_3[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_5\ : 
        CFG4
      generic map(INIT => x"8C00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/frame_type\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_is_transmitter\, 
        C => \can_top_level_inst/tran_is_rtr\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[32]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_5_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[21]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_1_i\, 
        SD => ADLIB_VCC, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_2[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram46_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram47_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m36_1_0_wmux_0\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m36_1_0_y0\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_26_0\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_35_0\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/N_37_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/next_state_17_sqmuxa_0_a4_RNI2TRJ\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/next_state_17_sqmuxa\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_is_fd_frame_m\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_2_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[2]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test.tprm[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[2]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_frame_test.tprm[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[23]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_63\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[20]\, B => 
        \can_top_level_inst/drv_bus[101]\, C => 
        \can_top_level_inst/drv_bus[134]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_21\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[9]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[10]\, 
        CC => NET_CC_CONFIG1807, P => NET_CC_CONFIG1804, Y3 => 
        NET_CC_CONFIG1805, Y3A => NET_CC_CONFIG1806);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_5[1]\ : 
        CFG4
      generic map(INIT => x"F8F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1236\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_2_3_Z[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_4_Z[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_978\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_5_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_38_1_wmux_RNO_0[23]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_0_Z[23]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_30_2_Z[23]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1040\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[8]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[8]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"553AC")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_i[2]\, 
        B => \can_top_level_inst/bus_sampling_inst/ssp_delay[2]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[2]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_1_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_2_Z\, 
        CC => NET_CC_CONFIG406, P => NET_CC_CONFIG403, Y3 => 
        NET_CC_CONFIG404, Y3A => NET_CC_CONFIG405);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe10\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm_0[0]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un10_tq_counter_d_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[0]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[0]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/int_status\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/un2_int_status_set\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.1.int_module_inst/un1_int_status_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_vector[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_FCNET1\, 
        CO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_1_RNO_FCO\, 
        CC => NET_CC_CONFIG82, P => NET_CC_CONFIG79, Y3 => 
        NET_CC_CONFIG80, Y3A => NET_CC_CONFIG81);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d_RNO[2]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => 
        \can_top_level_inst/can_core_inst/stuff_length[0]\, B => 
        \can_top_level_inst/rx_triggers[1]\, C => 
        \can_top_level_inst/stat_bus[303]\, D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_stuff_lvl_reached_NE_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/CO0_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_6_1[9]\ : 
        CFG4
      generic map(INIT => x"3210")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/memory_registers_inst/mode[9]\, 
        D => \can_top_level_inst/drv_bus[152]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1474_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[6]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI0PKSI_S[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[247]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[7]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/int_ena_proc.un1_int_ena_clear\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[0]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.0.int_module_inst/un1_int_ena_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_0[3]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[25]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI8J6I[0]_CC_3\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO1273, CO => OPEN, P(0) => 
        NET_CC_CONFIG1394, P(1) => NET_CC_CONFIG1398, P(2) => 
        NET_CC_CONFIG1402, P(3) => ADLIB_VCC, P(4) => ADLIB_VCC, 
        P(5) => ADLIB_VCC, P(6) => ADLIB_VCC, P(7) => ADLIB_VCC, 
        P(8) => ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, 
        P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1395, Y3(1) => 
        NET_CC_CONFIG1399, Y3(2) => NET_CC_CONFIG1403, Y3(3) => 
        ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5) => ADLIB_VCC, Y3(6)
         => ADLIB_VCC, Y3(7) => ADLIB_VCC, Y3(8) => ADLIB_VCC, 
        Y3(9) => ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => 
        ADLIB_VCC, Y3A(0) => NET_CC_CONFIG1396, Y3A(1) => 
        NET_CC_CONFIG1400, Y3A(2) => NET_CC_CONFIG1404, Y3A(3)
         => ADLIB_VCC, Y3A(4) => ADLIB_VCC, Y3A(5) => ADLIB_VCC, 
        Y3A(6) => ADLIB_VCC, Y3A(7) => ADLIB_VCC, Y3A(8) => 
        ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => ADLIB_VCC, 
        Y3A(11) => ADLIB_VCC, CC(0) => NET_CC_CONFIG1397, CC(1)
         => NET_CC_CONFIG1401, CC(2) => NET_CC_CONFIG1405, CC(3)
         => nc115, CC(4) => nc264, CC(5) => nc192, CC(6) => nc319, 
        CC(7) => nc134, CC(8) => nc32, CC(9) => nc40, CC(10) => 
        nc297, CC(11) => nc99);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[0]\, 
        D => \can_top_level_inst/drv_bus[46]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[30]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOLJ46_S[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[195]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[20]\);
    
    \data_out_obuf[4]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[4]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[4]/DOUT\, EOUT => \data_out_obuf[4]/EOUT\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_0_y3[25]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_0_y1[25]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1117\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_5_sqmuxa_i\ : 
        CFG3
      generic map(INIT => x"DF")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_957\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_879_i\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_q.unlock\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/unlock_2\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_hw_cmd.unlock\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_2_2[1]\ : 
        CFG4
      generic map(INIT => x"2755")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[82]\, C => 
        \can_top_level_inst/drv_bus[401]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_1_Z[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_30_2_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[30]\ : 
        SLE
      port map(D => \data_in_c[30]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_7[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_RNO_9[28]\ : 
        CFG4
      generic map(INIT => x"FFF7")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[1]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[0]\, C => 
        \can_top_level_inst/tran_frame_test.tprm[2]\, D => 
        \can_top_level_inst/tran_frame_test.tprm[4]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_o4_3_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_5\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[405]\, B => 
        \can_top_level_inst/stat_bus[86]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_4_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_5_Z\, 
        CC => NET_CC_CONFIG1549, P => NET_CC_CONFIG1546, Y3 => 
        NET_CC_CONFIG1547, Y3A => NET_CC_CONFIG1548);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[16]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[16]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[16]\);
    
    \can_top_level_inst/tx_arbitrator_inst/int_txtb_index[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_hw_cmd_index[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r_rep[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/reg_value_r_rep[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_30_2[8]\ : 
        CFG4
      generic map(INIT => x"C480")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1458\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1455\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1460_2\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.un4_rx_shift_ena_1_0\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[1]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[3]\ : 
        CFG4
      generic map(INIT => x"F088")

      port map(A => \can_top_level_inst/rec_dlc[3]\, B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        C => \can_top_level_inst/rx_buffer_inst/N_2281\, D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[17]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2706\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2673\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[17]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/load_ts_lw_addr_i_o2\ : 
        CFG4
      generic map(INIT => x"EFEE")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, B
         => \can_top_level_inst/tx_arbitrator_inst/un1_l3_valid\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[7]\, D
         => \can_top_level_inst/stat_bus[68]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/N_120\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_10[30]\ : 
        CFG3
      generic map(INIT => x"53")

      port map(A => \can_top_level_inst/tran_identifier[27]\, B
         => \can_top_level_inst/tran_identifier[16]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_11_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4[20]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un1_tx_load_base_id_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4_1[20]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[19]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[3]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2627\, Y
         => \can_top_level_inst/memory_registers_inst/N_2692\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_0[0]\ : 
        CFG4
      generic map(INIT => x"44CE")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_1_0_Z[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_147\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_0_1_Z[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_15_2_0_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO[30]\ : 
        CFG4
      generic map(INIT => x"FF20")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_19_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_14_0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_12_0_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI9BRM7[10]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[283]\, C => 
        \can_top_level_inst/stat_bus[267]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_9\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI9BRM7_S[10]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_10\, 
        CC => NET_CC_CONFIG2608, P => NET_CC_CONFIG2605, Y3 => 
        NET_CC_CONFIG2606, Y3A => NET_CC_CONFIG2607);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m22_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[17]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_4[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa1\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_data_mux_test_registers_comp_un2_index_sat_1\, 
        B => \adress_c[6]\, C => \adress_c[3]\, D => 
        \adress_c[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa1\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac11\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac5_2_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac11_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_51\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[7]\, B => 
        \timestamp_c[38]\, C => \timestamp_c[39]\, D => 
        \can_top_level_inst/tran_word[6]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[2]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[3]\, 
        CC => NET_CC_CONFIG2265, P => NET_CC_CONFIG2262, Y3 => 
        NET_CC_CONFIG2263, Y3A => NET_CC_CONFIG2264);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m19_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[25]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI2V31C_S[16]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[236]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_o2_4[1]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[5]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state[6]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state_0_Z[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_953\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c2_RNIT3KP6\ : 
        CFG4
      generic map(INIT => x"FF7F")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/N_3920\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/N_3918\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/N_3921\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_NE_5_2\, 
        Y => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE_5\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[11]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_38_1_wmux_RNO_0[14]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_0_Z[14]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_2_Z[14]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_698\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/alc_Z[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/stat_bus[6]\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[292]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac8\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac7_2_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac8_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram65__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_346\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe65\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[18]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[18]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[18]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_o2_2[0]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[17]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[16]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m29_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[27]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[27]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_y0_0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_2[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[9]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIG3FI5[7]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[280]\, C => 
        \can_top_level_inst/stat_bus[264]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_6\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIG3FI5_S[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_7\, 
        CC => NET_CC_CONFIG2596, P => NET_CC_CONFIG2593, Y3 => 
        NET_CC_CONFIG2594, Y3A => NET_CC_CONFIG2595);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_1_1[5]\ : 
        CFG3
      generic map(INIT => x"46")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/int_vector[5]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_1_1_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_y0_0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/alc_Z[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/stat_bus[6]\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[291]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_38_1_0_wmux_RNO[14]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_0_Z[14]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_15_2_Z[14]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_683\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => \can_top_level_inst/is_bus_off\, B => 
        \can_top_level_inst/stat_bus[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[23]\, 
        D => \can_top_level_inst/stat_bus[108]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_6[1]\ : 
        CFG4
      generic map(INIT => x"0040")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_data_nbs_prev_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[7]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_6_1_Z[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_953\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1202\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[21]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[10]\);
    
    
        \can_top_level_inst/can_core_inst/trigger_mux_inst/crc_data_rx_wbs_reg/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_in\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_triggers[1]\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_data_rx_wbs\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_1_CC_2\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO2249, CO => OPEN, P(0) => 
        NET_CC_CONFIG2310, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, 
        P(3) => ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_VCC, 
        P(6) => ADLIB_VCC, P(7) => ADLIB_VCC, P(8) => ADLIB_VCC, 
        P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11) => ADLIB_VCC, 
        Y3(0) => NET_CC_CONFIG2311, Y3(1) => ADLIB_VCC, Y3(2) => 
        ADLIB_VCC, Y3(3) => ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5)
         => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7) => ADLIB_VCC, 
        Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, Y3(10) => 
        ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG2312, Y3A(1) => ADLIB_VCC, Y3A(2) => 
        ADLIB_VCC, Y3A(3) => ADLIB_VCC, Y3A(4) => ADLIB_VCC, 
        Y3A(5) => ADLIB_VCC, Y3A(6) => ADLIB_VCC, Y3A(7) => 
        ADLIB_VCC, Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, 
        Y3A(10) => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG2313, CC(1) => nc75, CC(2) => nc183, CC(3)
         => nc345, CC(4) => nc333, CC(5) => nc288, CC(6) => nc85, 
        CC(7) => nc27, CC(8) => nc108, CC(9) => nc325, CC(10) => 
        nc16, CC(11) => nc155);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_22\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_22_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa14\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe14\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIDSV21[2]\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[2]\, 
        B => \can_top_level_inst/drv_bus[377]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_1\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIDSV21_S[2]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_2\, 
        CC => NET_CC_CONFIG1577, P => NET_CC_CONFIG1574, Y3 => 
        NET_CC_CONFIG1575, Y3A => NET_CC_CONFIG1576);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_3_Y[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[28]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_38_1_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_38_1_0_y0[16]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_778\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_780\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[3]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_err_frm_req_3_0_a4\ : 
        CFG4
      generic map(INIT => x"1105")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_930\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_952\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1103_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_a3[2]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => \can_top_level_inst/rec_abort_f\, B => 
        \can_top_level_inst/rx_buffer_inst/data_selector[0]\, Y
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2[23]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/tran_identifier[9]\, B
         => \can_top_level_inst/tran_identifier[20]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[23]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_28\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_23_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_16_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_28_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[8]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[28]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[28]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[29]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[24]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[24]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[175]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[29]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[29]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[29]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_shift_cmd_gen.3.un25_rx_shift_cmd\ : 
        CFG4
      generic map(INIT => x"C404")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1077\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[3]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un25_rx_shift_cmd\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/reg_value_r[6]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/erp_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[414]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[29]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0[5]\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_143\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[5]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[0]\ : 
        SLE
      port map(D => \timestamp_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_6[1]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_7_Z[1]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_36_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[11]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[8]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[11]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[11]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[25]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[25]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un9_inc_eight_i_1\ : 
        CFG4
      generic map(INIT => x"002A")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/stat_bus[108]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_ack_err\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_detector_inst_un10_err_ctrs_unchanged\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un9_inc_eight_i_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1[4]\ : 
        CFG4
      generic map(INIT => x"F805")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1_1_Z[4]\, 
        B => \can_top_level_inst/stat_bus[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_2[4]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[19]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un17_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[16]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[3]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[36]\ : 
        SLE
      port map(D => \timestamp_c[36]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[36]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[19]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[19]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[19]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_0_1[13]\ : 
        CFG4
      generic map(INIT => x"0AC0")

      port map(A => \can_top_level_inst/drv_bus[94]\, B => 
        \can_top_level_inst/txtb_prorities_3[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_30_2_0_1_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.is_control_2_i_a2_3\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[27]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1462_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_2_0[21]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_962\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_0_Z[21]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_2_0_Z[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un3_reinteg_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2[22]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/tran_identifier[8]\, B
         => \can_top_level_inst/tran_identifier[19]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram11__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa11\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe11\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_command_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[11]\);
    
    \data_in_ibuf[3]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(3), Y => \data_in_ibuf[3]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_87_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_0_5\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \can_top_level_inst/prescaler_inst/is_tseg2\, 
        B => \can_top_level_inst/stat_bus[14]\, C => 
        \can_top_level_inst/rx_data_wbs\, D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/g0_0_3_0\, 
        Y => \can_top_level_inst/prescaler_inst/g0_0_5\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_2[5]\ : 
        CFG3
      generic map(INIT => x"35")

      port map(A => \can_top_level_inst/drv_bus[40]\, B => 
        \can_top_level_inst/stat_bus[86]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_1[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[16]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/sel_value_i[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[220]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_794_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_o2_0[3]\ : 
        CFG3
      generic map(INIT => x"EA")

      port map(A => \can_top_level_inst/stat_bus[68]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_09\, C => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_19\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/N_104\);
    
    \can_top_level_inst/frame_filters_inst/un1_rec_valid_f\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/frame_filters_inst_ident_valid_q\, B
         => \can_top_level_inst/rec_valid\, Y => 
        \can_top_level_inst/rec_valid_f\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1[2]\ : 
        CFG4
      generic map(INIT => x"45CF")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_141\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_147\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_49_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[4]\ : 
        SLE
      port map(D => \data_in_c[4]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[172]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram83__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_355\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe83\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI0H3V6_S[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[266]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[5]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[19]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[19]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[27]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[27]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_33_RNIGSF41[25]\ : 
        CFG4
      generic map(INIT => x"434F")

      port map(A => \timestamp_c[57]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1119\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_37_2_1_1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[31]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/int_ena_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.9.int_module_inst/un1_int_ena_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_ena[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un4_bst_ctr_d\ : 
        CFG3
      generic map(INIT => x"08")

      port map(A => 
        \can_top_level_inst/can_core_inst/stuff_length[0]\, B => 
        \can_top_level_inst/tx_trigger\, C => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un9_stuff_lvl_reached_NE\, 
        Y => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un4_bst_ctr_d_Z\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIB9FT[0]_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO921, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => 
        ADLIB_VCC, P(4) => ADLIB_VCC, P(5) => ADLIB_VCC, P(6) => 
        ADLIB_VCC, P(7) => ADLIB_VCC, P(8) => ADLIB_GND, P(9) => 
        NET_CC_CONFIG924, P(10) => NET_CC_CONFIG928, P(11) => 
        NET_CC_CONFIG932, Y3(0) => ADLIB_VCC, Y3(1) => ADLIB_VCC, 
        Y3(2) => ADLIB_VCC, Y3(3) => ADLIB_VCC, Y3(4) => 
        ADLIB_VCC, Y3(5) => ADLIB_VCC, Y3(6) => ADLIB_VCC, Y3(7)
         => ADLIB_VCC, Y3(8) => ADLIB_VCC, Y3(9) => 
        NET_CC_CONFIG925, Y3(10) => NET_CC_CONFIG929, Y3(11) => 
        NET_CC_CONFIG933, Y3A(0) => ADLIB_VCC, Y3A(1) => 
        ADLIB_VCC, Y3A(2) => ADLIB_VCC, Y3A(3) => ADLIB_VCC, 
        Y3A(4) => ADLIB_VCC, Y3A(5) => ADLIB_VCC, Y3A(6) => 
        ADLIB_VCC, Y3A(7) => ADLIB_VCC, Y3A(8) => ADLIB_VCC, 
        Y3A(9) => NET_CC_CONFIG926, Y3A(10) => NET_CC_CONFIG930, 
        Y3A(11) => NET_CC_CONFIG934, CC(0) => nc51, CC(1) => 
        nc301, CC(2) => nc33, CC(3) => nc204, CC(4) => nc173, 
        CC(5) => nc278, CC(6) => nc169, CC(7) => nc78, CC(8) => 
        nc263, CC(9) => NET_CC_CONFIG927, CC(10) => 
        NET_CC_CONFIG931, CC(11) => NET_CC_CONFIG935);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram40__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_344\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe40\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac4\ : 
        CFG4
      generic map(INIT => x"0400")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => \adress_c[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac4_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[8]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[8]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[8]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_s_1_2643_FCO\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_1_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_1_Z\, 
        CC => NET_CC_CONFIG2223, P => NET_CC_CONFIG2220, Y3 => 
        NET_CC_CONFIG2221, Y3A => NET_CC_CONFIG2222);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y3[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[4]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[0]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/test_registers_out.tst_dest[4]\, D
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.2.next_bytes_gen.shift_reg_in_4[2]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/stat_bus[372]\, B => 
        \can_top_level_inst/store_data_word[15]\, C => 
        \can_top_level_inst/can_core_inst/bds_data_out\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/shift_reg_in_4[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[19]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry[7]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[7]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[6]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[7]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[7]\, 
        CC => NET_CC_CONFIG1083, P => NET_CC_CONFIG1080, Y3 => 
        NET_CC_CONFIG1081, Y3A => NET_CC_CONFIG1082);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[24]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[123]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m19_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[3]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[3]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_s_0_2645_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO1205, CO => CI_TO_CO1206, P(0) => 
        NET_CC_CONFIG1211, P(1) => NET_CC_CONFIG1215, P(2) => 
        NET_CC_CONFIG1219, P(3) => NET_CC_CONFIG1223, P(4) => 
        NET_CC_CONFIG1227, P(5) => NET_CC_CONFIG1231, P(6) => 
        NET_CC_CONFIG1235, P(7) => NET_CC_CONFIG1239, P(8) => 
        NET_CC_CONFIG1243, P(9) => NET_CC_CONFIG1247, P(10) => 
        NET_CC_CONFIG1251, P(11) => NET_CC_CONFIG1255, Y3(0) => 
        NET_CC_CONFIG1212, Y3(1) => NET_CC_CONFIG1216, Y3(2) => 
        NET_CC_CONFIG1220, Y3(3) => NET_CC_CONFIG1224, Y3(4) => 
        NET_CC_CONFIG1228, Y3(5) => NET_CC_CONFIG1232, Y3(6) => 
        NET_CC_CONFIG1236, Y3(7) => NET_CC_CONFIG1240, Y3(8) => 
        NET_CC_CONFIG1244, Y3(9) => NET_CC_CONFIG1248, Y3(10) => 
        NET_CC_CONFIG1252, Y3(11) => NET_CC_CONFIG1256, Y3A(0)
         => NET_CC_CONFIG1213, Y3A(1) => NET_CC_CONFIG1217, 
        Y3A(2) => NET_CC_CONFIG1221, Y3A(3) => NET_CC_CONFIG1225, 
        Y3A(4) => NET_CC_CONFIG1229, Y3A(5) => NET_CC_CONFIG1233, 
        Y3A(6) => NET_CC_CONFIG1237, Y3A(7) => NET_CC_CONFIG1241, 
        Y3A(8) => NET_CC_CONFIG1245, Y3A(9) => NET_CC_CONFIG1249, 
        Y3A(10) => NET_CC_CONFIG1253, Y3A(11) => 
        NET_CC_CONFIG1257, CC(0) => NET_CC_CONFIG1214, CC(1) => 
        NET_CC_CONFIG1218, CC(2) => NET_CC_CONFIG1222, CC(3) => 
        NET_CC_CONFIG1226, CC(4) => NET_CC_CONFIG1230, CC(5) => 
        NET_CC_CONFIG1234, CC(6) => NET_CC_CONFIG1238, CC(7) => 
        NET_CC_CONFIG1242, CC(8) => NET_CC_CONFIG1246, CC(9) => 
        NET_CC_CONFIG1250, CC(10) => NET_CC_CONFIG1254, CC(11)
         => NET_CC_CONFIG1258);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[10]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[10]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_4_0\ : 
        ARI1_CC
      generic map(INIT => x"5CC66")

      port map(A => \can_top_level_inst/drv_bus[33]\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_fast\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[4]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_3\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_4\, 
        CC => NET_CC_CONFIG904, P => NET_CC_CONFIG901, Y3 => 
        NET_CC_CONFIG902, Y3A => NET_CC_CONFIG903);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[15]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[15]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_4[30]\ : 
        CFG4
      generic map(INIT => x"BF80")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[2]\, 
        B => \can_top_level_inst/drv_bus[477]\, C => 
        \can_top_level_inst/tran_frame_test.sdlc\, D => 
        \can_top_level_inst/tran_dlc[2]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_12_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[29]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_10[31]\ : 
        CFG4
      generic map(INIT => x"0503")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[16]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[14]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_1225_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask[18]\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[4]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[3]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1_Z[18]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_Z[18]\);
    
    \data_out_obuf[25]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[25]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[25]/DOUT\, EOUT => 
        \data_out_obuf[25]/EOUT\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_54_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[1]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[1]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2625\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2[0]\ : 
        CFG3
      generic map(INIT => x"42")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[2]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[1]\, C => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_y0_0[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/read_pointer_q_Z[1]\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram6_0\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ram7_0\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_ramout_7_2_0_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIU4MKH[25]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[213]\, D => 
        \can_top_level_inst/stat_bus[245]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_24\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIU4MKH_S[25]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_25\, 
        CC => NET_CC_CONFIG856, P => NET_CC_CONFIG853, Y3 => 
        NET_CC_CONFIG854, Y3A => NET_CC_CONFIG855);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[8]\);
    
    \timestamp_ibuf[7]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[7]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[7]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[3]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_3\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_2_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_cry_3_Z\, 
        CC => NET_CC_CONFIG1497, P => NET_CC_CONFIG1494, Y3 => 
        NET_CC_CONFIG1495, Y3A => NET_CC_CONFIG1496);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[17]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[17]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[17]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_28\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_23_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_16_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_28_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[25]\ : 
        SLE
      port map(D => \timestamp_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/next_state_0_sqmuxa_0_a2\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end\, B => 
        \can_top_level_inst/prescaler_inst/is_tseg1\, Y => 
        \can_top_level_inst/rx_triggers[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1[3]\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[3]\, 
        C => \can_top_level_inst/txtb_parity_error_valid[1]\, D
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_168\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[57]\ : 
        SLE
      port map(D => \timestamp_c[57]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[57]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[7]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108[20]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[20]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[9]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[8]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_ext_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[2]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2460\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_y0_0[18]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_845\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_848\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_31_2_0_y3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_2[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram15_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/un7_index_sat[1]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \adress_c[7]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_0[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_06_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_29\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_20_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_19_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_18_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_17_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_29_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa10_2\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \adress_c[2]\, B => \adress_c[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10_2\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q_RNO[2]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_17[1]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m0_0_x2[14]\ : 
        CFG4
      generic map(INIT => x"95C0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1196\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        C => \can_top_level_inst/can_core_inst/crc_21[3]\, D => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_46_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[25]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI5VLH2[6]\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[6]\, 
        B => \can_top_level_inst/drv_bus[381]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_5\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNI5VLH2_S[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un3_trv_delay_sum_cry_6\, 
        CC => NET_CC_CONFIG1593, P => NET_CC_CONFIG1590, Y3 => 
        NET_CC_CONFIG1591, Y3A => NET_CC_CONFIG1592);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI1295B4[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[51]\, B => 
        \can_top_level_inst/tran_word[19]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_18\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_19\, 
        CC => NET_CC_CONFIG1003, P => NET_CC_CONFIG1000, Y3 => 
        NET_CC_CONFIG1001, Y3A => NET_CC_CONFIG1002);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0_RNIJV60J3[3]\ : 
        ARI1_CC
      generic map(INIT => x"5936C")

      port map(A => \can_top_level_inst/drv_bus[49]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_52\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_2\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_3\, 
        CC => NET_CC_CONFIG2712, P => NET_CC_CONFIG2709, Y3 => 
        NET_CC_CONFIG2710, Y3A => NET_CC_CONFIG2711);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_shadowed[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/trv_delay[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[15]\);
    
    \data_out_obuf[16]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[16]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[16]/DOUT\, EOUT => 
        \data_out_obuf[16]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa4\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/wa4_2\, 
        B => \adress_c[6]\, C => \adress_c[4]\, D => 
        \adress_c[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa4\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIUBEJ4[5]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[193]\, D => 
        \can_top_level_inst/stat_bus[225]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_4\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIUBEJ4_S[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_5\, 
        CC => NET_CC_CONFIG776, P => NET_CC_CONFIG773, Y3 => 
        NET_CC_CONFIG774, Y3A => NET_CC_CONFIG775);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.un1_valid\ : 
        CFG3
      generic map(INIT => x"07")

      port map(A => 
        \can_top_level_inst/frame_filters_inst/un1_filter_c_enable_1_Z\, 
        B => 
        \can_top_level_inst/frame_filters_inst/un1_filter_c_enable_0_Z\, 
        C => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_I_87_FCO_1\, 
        Y => 
        \can_top_level_inst/frame_filters_inst/int_filter_C_valid\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[19]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[18]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[18]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[18]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_1[1]\ : 
        CFG4
      generic map(INIT => x"FFEC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_223\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1415\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/sp_control_q_i_ns_i_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_38_1_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_38_1_0_y0[20]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_930\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_932\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \adress_c[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa16\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe16\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[17]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIILP4F_S[21]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[241]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe6\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa0\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => \adress_c[4]\, B => \adress_c[5]\, C => 
        \adress_c[6]\, D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\);
    
    \timestamp_ibuf[52]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(52), Y => 
        \timestamp_ibuf[52]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[24]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un1_tq_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/data_length[2]\ : 
        CFG4
      generic map(INIT => x"2C0C")

      port map(A => \can_top_level_inst/tran_dlc[0]\, B => 
        \can_top_level_inst/tran_dlc[2]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_tx_inst/un2_data_len_can_2_0_i\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/frame_type\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_shift_ena_cnst_i_a4[0]\ : 
        CFG4
      generic map(INIT => x"0010")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[19]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1238\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1077\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_18[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_7[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram62_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram63_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y7_0[0]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIRV4S4_S[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[263]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_2_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1225\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1226\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_2_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_3[4]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[27]\, CLK
         => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un25_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_ac0_11\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => \can_top_level_inst/stat_bus[87]\, B => 
        \can_top_level_inst/stat_bus[86]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c5\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c7\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac28\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac20_1_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac28_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI5RDS\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_RNISD182[6]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_5\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_6\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_6\, 
        CC => NET_CC_CONFIG276, P => NET_CC_CONFIG273, Y3 => 
        NET_CC_CONFIG274, Y3A => NET_CC_CONFIG275);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_17[6]\ : 
        CFG4
      generic map(INIT => x"FA3A")

      port map(A => \can_top_level_inst/stat_bus[87]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_190\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1371\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_3_1\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/can_core_inst/act_err_ovr_flag\, B
         => \can_top_level_inst/drv_bus[513]\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc_axb_0_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_5[20]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/drv_bus[1]\, B => 
        \can_top_level_inst/rx_read_pointer[4]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_901\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/un5_transient_state_0_o3\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[0]\ : 
        CFG4
      generic map(INIT => x"8F88")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_141\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        C => \can_top_level_inst/txtb_allow_bb[6]\, D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[11]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3_1[31]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[63]\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2309_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[11]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c4_a0_RNI2G5B1\ : 
        CFG4
      generic map(INIT => x"6999")

      port map(A => \can_top_level_inst/drv_bus[13]\, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[5]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c4_i\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[4]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/N_3920\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[19]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[100]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[14]\);
    
    \can_top_level_inst/rx_buffer_inst/un3_is_free_wordlto5_3\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => \can_top_level_inst/rx_frame_count[5]\, B => 
        \can_top_level_inst/rx_frame_count[4]\, C => 
        \can_top_level_inst/rx_frame_count[3]\, D => 
        \can_top_level_inst/rx_frame_count[2]\, Y => 
        \can_top_level_inst/rx_buffer_inst/un3_is_free_wordlto5_3_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_2[23]\ : 
        CFG4
      generic map(INIT => x"A820")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1464_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram96_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe96\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram96_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_6\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[3]\, B
         => \can_top_level_inst/can_core_inst/crc_17[2]\, C => 
        \can_top_level_inst/can_core_inst/crc_17[3]\, D => 
        \can_top_level_inst/store_data_word[2]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[0]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[1]\, 
        CC => NET_CC_CONFIG2531, P => NET_CC_CONFIG2528, Y3 => 
        NET_CC_CONFIG2529, Y3A => NET_CC_CONFIG2530);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe8\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_rtr_i_a2_RNIBU18\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1234\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_715_i\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOOLCL_S[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[251]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_0[10]\ : 
        CFG3
      generic map(INIT => x"5C")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_0_1_Z[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[10]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_2_0_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[6]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un7_rx_err_ctr_declto8_i_a2_0_a2_RNIQLJA1\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/stat_bus[83]\, B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_1\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNIQLJA1_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_2\, 
        CC => NET_CC_CONFIG2326, P => NET_CC_CONFIG2323, Y3 => 
        NET_CC_CONFIG2324, Y3A => NET_CC_CONFIG2325);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE_4\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[7]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[8]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_1_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_2_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un6_phase_err_sjw_by_one_NE_4_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_34_1[16]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/stat_bus[204]\, B => 
        \can_top_level_inst/drv_bus[375]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_778_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exp_seg_length_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\);
    
    \I_1/U0_RGB1_RGB0\ : RGB
      port map(A => \I_1/U0_Y\, EN => ADLIB_VCC, Y => 
        \AFLSDF_INV_17\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[20]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y1[22]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un7_txtb_parity_error_valid_i_1\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/N_3899\, 
        B => \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/drv_bus[476]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/un7_txtb_parity_error_valid_i_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[90]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI8N9PA[14]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[202]\, D => 
        \can_top_level_inst/stat_bus[234]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_13\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI8N9PA_S[14]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_14\, 
        CC => NET_CC_CONFIG812, P => NET_CC_CONFIG809, Y3 => 
        NET_CC_CONFIG810, Y3A => NET_CC_CONFIG811);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[28]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[53]\ : 
        SLE
      port map(D => \timestamp_c[53]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[53]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram48_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe48\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram48_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m32_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[24]\ : 
        SLE
      port map(D => \timestamp_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_0[3]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[4]\, D => 
        \can_top_level_inst/stat_bus[68]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[3]\);
    
    \res_n_ibuf/U_IOPAD\ : IOPAD_IN
      port map(PAD => res_n, Y => \res_n_ibuf/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/int_ena_proc.un1_int_ena_clear\ : 
        CFG4
      generic map(INIT => x"E000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => \sbe_c[0]\, D => \data_in_c[3]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/un1_int_ena_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[2]\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[6]\, 
        B => \can_top_level_inst/txtb_hw_cmd.unlock\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_134\);
    
    \can_top_level_inst/rx_buffer_inst/read_counter_q[2]\ : SLE
      port map(D => \can_top_level_inst/rx_buffer_inst/N_3926_i\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram8__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe8\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_RNO_1[25]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[1]\, 
        B => \can_top_level_inst/tran_frame_test.fcrc\, C => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, D => 
        \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/m50_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa9\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe9\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[5]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2629\, Y
         => \can_top_level_inst/memory_registers_inst/N_2694\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        C => \can_top_level_inst/drv_bus[50]\, D => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[4]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[28]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[28]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[28]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un4_write_pointer_ts_ce_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[23]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[23]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[23]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"555AA")

      port map(A => \can_top_level_inst/drv_bus[38]\, B => 
        \can_top_level_inst/drv_bus[44]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_2_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_3_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_3_Z\, 
        CC => NET_CC_CONFIG1750, P => NET_CC_CONFIG1747, Y3 => 
        NET_CC_CONFIG1748, Y3A => NET_CC_CONFIG1749);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_3\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_2_Z\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_3_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_3_Z\, 
        CC => NET_CC_CONFIG1226, P => NET_CC_CONFIG1223, Y3 => 
        NET_CC_CONFIG1224, Y3A => NET_CC_CONFIG1225);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNINUGF3[4]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[277]\, C => 
        \can_top_level_inst/stat_bus[261]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_3\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNINUGF3_S[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_4\, 
        CC => NET_CC_CONFIG2584, P => NET_CC_CONFIG2581, Y3 => 
        NET_CC_CONFIG2582, Y3A => NET_CC_CONFIG2583);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[23]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[23]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y3[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y1[30]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_3_Y[30]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_shorten_tseg1_after_tseg2_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/un18_sel_tseg1_2230\, 
        C => \can_top_level_inst/prescaler_inst/start_edge\, D
         => \can_top_level_inst/prescaler_inst/h_sync_valid\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\);
    
    \data_in_ibuf[13]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(13), Y => \data_in_ibuf[13]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_failed_bof\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[12]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram20__RNISDM1V7[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_port_b_address[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/N_4258\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/N_4254\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[29]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[25]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3_2[31]\ : 
        CFG4
      generic map(INIT => x"A0C0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[31]\, 
        B => \can_top_level_inst/store_data_word[31]\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_2309_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/dlc_decoder_rx_inst/un2_data_len_can_2_0_1.CO3\ : 
        CFG4
      generic map(INIT => x"AAA8")

      port map(A => \can_top_level_inst/rec_dlc[3]\, B => 
        \can_top_level_inst/rec_dlc[2]\, C => 
        \can_top_level_inst/rec_dlc[1]\, D => 
        \can_top_level_inst/rec_dlc[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_data_len_can_2_0_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_RNI7E6S2[8]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[8]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_7\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_8\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_8\, 
        CC => NET_CC_CONFIG2211, P => NET_CC_CONFIG2208, Y3 => 
        NET_CC_CONFIG2209, Y3A => NET_CC_CONFIG2210);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[24]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[13]\, 
        B => \can_top_level_inst/tran_word[0]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[10]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry[5]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[5]\, 
        D => \can_top_level_inst/rx_write_pointer[5]\, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[4]\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[5]\, 
        CC => NET_CC_CONFIG389, P => NET_CC_CONFIG386, Y3 => 
        NET_CC_CONFIG387, Y3A => NET_CC_CONFIG388);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_3[1]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[24]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un25_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[407]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/un4_txtb_port_b_address[3]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, B
         => \can_top_level_inst/txtb_ptr[3]\, Y => 
        \can_top_level_inst/txtb_port_b_address[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_32_0_iv_7_RNO[31]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[31]\, B
         => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac11_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[895]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_8[26]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac28_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac29_Z\, 
        C => \can_top_level_inst/drv_bus[15]\, D => 
        \can_top_level_inst/drv_bus[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_8_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_6_1[24]\ : 
        CFG4
      generic map(INIT => x"5410")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => \can_top_level_inst/drv_bus[511]\, D => 
        \can_top_level_inst/drv_bus[167]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1054_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_85_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_13[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram20_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram21_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_5[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/settings_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[510]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_21[24]\ : 
        CFG4
      generic map(INIT => x"E4A0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1068\, 
        D => \can_top_level_inst/drv_bus[229]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1069\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[14]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2472\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0[31]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[31]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_63_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i[5]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[5]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa13\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe13\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_set_err_active_0\ : 
        CFG4
      generic map(INIT => x"3222")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1269\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_950\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        D => \can_top_level_inst/is_bus_off\, Y => 
        \can_top_level_inst/can_core_inst/set_err_active\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[16]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[16]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_30_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_800\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_803\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_30_1_0_y0[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_shift_cmd_gen.1.un9_rx_shift_cmd\ : 
        CFG4
      generic map(INIT => x"C404")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1077\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un9_rx_shift_cmd\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_control_reg_comp/wr_select_0[0]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => \adress_c[3]\, B => \adress_c[4]\, C => 
        \adress_c[2]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_control_reg_comp/wr_select_0_Z[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNI8J6I[0]_CC_2\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO1272, CO => CI_TO_CO1273, P(0) => 
        NET_CC_CONFIG1346, P(1) => NET_CC_CONFIG1350, P(2) => 
        NET_CC_CONFIG1354, P(3) => NET_CC_CONFIG1358, P(4) => 
        NET_CC_CONFIG1362, P(5) => NET_CC_CONFIG1366, P(6) => 
        NET_CC_CONFIG1370, P(7) => NET_CC_CONFIG1374, P(8) => 
        NET_CC_CONFIG1378, P(9) => NET_CC_CONFIG1382, P(10) => 
        NET_CC_CONFIG1386, P(11) => NET_CC_CONFIG1390, Y3(0) => 
        NET_CC_CONFIG1347, Y3(1) => NET_CC_CONFIG1351, Y3(2) => 
        NET_CC_CONFIG1355, Y3(3) => NET_CC_CONFIG1359, Y3(4) => 
        NET_CC_CONFIG1363, Y3(5) => NET_CC_CONFIG1367, Y3(6) => 
        NET_CC_CONFIG1371, Y3(7) => NET_CC_CONFIG1375, Y3(8) => 
        NET_CC_CONFIG1379, Y3(9) => NET_CC_CONFIG1383, Y3(10) => 
        NET_CC_CONFIG1387, Y3(11) => NET_CC_CONFIG1391, Y3A(0)
         => NET_CC_CONFIG1348, Y3A(1) => NET_CC_CONFIG1352, 
        Y3A(2) => NET_CC_CONFIG1356, Y3A(3) => NET_CC_CONFIG1360, 
        Y3A(4) => NET_CC_CONFIG1364, Y3A(5) => NET_CC_CONFIG1368, 
        Y3A(6) => NET_CC_CONFIG1372, Y3A(7) => NET_CC_CONFIG1376, 
        Y3A(8) => NET_CC_CONFIG1380, Y3A(9) => NET_CC_CONFIG1384, 
        Y3A(10) => NET_CC_CONFIG1388, Y3A(11) => 
        NET_CC_CONFIG1392, CC(0) => NET_CC_CONFIG1349, CC(1) => 
        NET_CC_CONFIG1353, CC(2) => NET_CC_CONFIG1357, CC(3) => 
        NET_CC_CONFIG1361, CC(4) => NET_CC_CONFIG1365, CC(5) => 
        NET_CC_CONFIG1369, CC(6) => NET_CC_CONFIG1373, CC(7) => 
        NET_CC_CONFIG1377, CC(8) => NET_CC_CONFIG1381, CC(9) => 
        NET_CC_CONFIG1385, CC(10) => NET_CC_CONFIG1389, CC(11)
         => NET_CC_CONFIG1393);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[1]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[13]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8_FCNET1\, 
        CO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        CC => NET_CC_CONFIG2486, P => NET_CC_CONFIG2483, Y3 => 
        NET_CC_CONFIG2484, Y3A => NET_CC_CONFIG2485);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa5\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe5\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[27]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/int_status\ : 
        SLE
      port map(D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/un2_int_status_set\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/un1_int_status_clear\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_vector[5]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_87_FCINST1\ : 
        FCEND_BUFF_CC
      port map(FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_87_FCNET1\, 
        CO => 
        \can_top_level_inst/frame_filters_inst/valid_0_I_87_FCO_0\, 
        CC => NET_CC_CONFIG750, P => NET_CC_CONFIG747, Y3 => 
        NET_CC_CONFIG748, Y3A => NET_CC_CONFIG749);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[4]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[4]\, 
        C => \can_top_level_inst/txtb_port_b_address[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_rst_d\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => can_tx_c, B => 
        \can_top_level_inst/bus_sampling_inst/tx_data_prev\, C
         => \can_top_level_inst/stat_bus[13]\, Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_rst_d_Z\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm_0[4]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un10_tq_counter_d_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[4]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIFKKC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram49_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe49\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram49_0\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIG63U9_S[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[270]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[22]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_s_1_2644\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_mem_free[0]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => ADLIB_VCC, S => OPEN, Y => OPEN, 
        FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_s_1_2644_FCO\, 
        CC => NET_CC_CONFIG2418, P => NET_CC_CONFIG2415, Y3 => 
        NET_CC_CONFIG2416, Y3A => NET_CC_CONFIG2417);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[16]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[401]\, B => 
        \can_top_level_inst/stat_bus[82]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_0_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_1_Z\, 
        CC => NET_CC_CONFIG1533, P => NET_CC_CONFIG1530, Y3 => 
        NET_CC_CONFIG1531, Y3A => NET_CC_CONFIG1532);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_load_stuff_count_i_o4\ : 
        CFG4
      generic map(INIT => x"CCDF")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_no_data_field_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_971\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1004\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.un1_is_receiver\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_match\, 
        C => 
        \can_top_level_inst/can_core_inst/operation_control_inst_curr_state[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state_proc_un1_is_receiver\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/is_arbitration\ : 
        CFG4
      generic map(INIT => x"4501")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_9[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ide_is_arbitration_i\, 
        Y => \can_top_level_inst/stat_bus[370]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m24_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/sel_value_i[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[188]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[21]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNO[8]\ : 
        ARI1_CC
      generic map(INIT => x"42EE2")

      port map(A => \can_top_level_inst/stat_bus[79]\, B => 
        \can_top_level_inst/drv_bus[424]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/drv_bus_i[425]\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[4]\, 
        FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_7\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[8]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG2522, P => 
        NET_CC_CONFIG2519, Y3 => NET_CC_CONFIG2520, Y3A => 
        NET_CC_CONFIG2521);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_read_address[5]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => \can_top_level_inst/rx_read_pointer[5]\, C => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_8[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[240]\, C => 
        \can_top_level_inst/drv_bus[116]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1210\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_1_0[5]\ : 
        CFG4
      generic map(INIT => x"084C")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => \can_top_level_inst/txtb_prorities_1[1]\, D => 
        \can_top_level_inst/rx_frame_count[1]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_1_0_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa14_1\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \adress_c[3]\, B => \adress_c[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[15]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[115]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_2_y0[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1228\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1229\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_30_1_wmux_0_Y[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[11]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_39\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[13]\, B => 
        \can_top_level_inst/drv_bus[156]\, C => 
        \can_top_level_inst/drv_bus[189]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_42\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[5]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_data_tmp[6]\, 
        CC => NET_CC_CONFIG714, P => NET_CC_CONFIG711, Y3 => 
        NET_CC_CONFIG712, Y3A => NET_CC_CONFIG713);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g1_1\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => 
        \can_top_level_inst/can_core_inst/trigger_mux_inst_tx_trigger_q\, 
        B => \can_top_level_inst/stat_bus[184]\, C => 
        \can_top_level_inst/can_core_inst/crc_calc_from_rx_0\, D
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g1_0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g1_1_Z\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_1_i\, 
        SD => ADLIB_VCC, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[10]\);
    
    AFLSDF_INV_17 : INV_BA
      port map(A => \AFLSDF_INV_17\, Y => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_go_to_stuff_count_i_o4\ : 
        CFG2
      generic map(INIT => x"D")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src_tz[0]\, 
        B => \can_top_level_inst/drv_bus[510]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_971\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[31]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[313]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un1_tq_counter_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[21]\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count[1]\ : SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI1VDV_S[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_frame_count[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram2_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m23_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_4_RNO[30]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/stat_bus[287]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac25_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[446]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0\ : 
        CFG4
      generic map(INIT => x"FFEA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_7_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1300\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i_0_a4_2_3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1213\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[13]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1454_2\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_qe\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[15]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[15]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[15]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[26]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[140]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_38_1_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_38_1_0_y0[9]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1502\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1504\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_16\ : 
        CFG3
      generic map(INIT => x"96")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/un6_parity_mismatch_16_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_1[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram106_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram107_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_0_y5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram69__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_353\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe69\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0[1]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/un2_phase_err_mt_sjw_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[1]\, 
        C => \can_top_level_inst/drv_bus[52]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_48\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[9]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[9]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[9]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_1[16]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_0_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[0]\);
    
    \sbe_ibuf[2]/U_IOPAD\ : IOPAD_IN
      port map(PAD => sbe(2), Y => \sbe_ibuf[2]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un3_reinteg_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[12]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/current_state_0_rep1\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_fsm_inst/N_33_i_rep1\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_36[20]\ : 
        CFG4
      generic map(INIT => x"88D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[52]\, C => 
        \can_top_level_inst/stat_bus[240]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_932\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[4]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_RNI2IM72[6]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_q_Z[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_5\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un7_exit_segm_regular_6\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un5_exit_segm_regular_cry_6\, 
        CC => NET_CC_CONFIG2203, P => NET_CC_CONFIG2200, Y3 => 
        NET_CC_CONFIG2201, Y3A => NET_CC_CONFIG2202);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/tseg1_dbt_Z[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un6_tseg1_dbt_d_cry_4_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_cfg_capture_inst/un1_capture_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/tseg1_dbt[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_12[27]\ : 
        CFG4
      generic map(INIT => x"FFF8")

      port map(A => \can_top_level_inst/drv_bus[170]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac21_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_8_Z[27]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_2_Z[27]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_28_1_iv_12_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_2_0[17]\ : 
        CFG4
      generic map(INIT => x"F388")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_793\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_794\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_0_Z[17]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_2_0_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[1]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[9]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[9]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[9]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[31]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2557\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_13[18]\ : 
        CFG4
      generic map(INIT => x"F4F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_833_2\, 
        D => \can_top_level_inst/drv_bus[223]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_833\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rst_reg_inst/rx_shift_res_reg_inst/reg_q_RNO\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/set_err_active\, B => 
        \can_top_level_inst/memory_registers_inst_global_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rst_reg_inst/rx_shift_res_reg_inst/un1_res_err_ctrs_d_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[19]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[19]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[19]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_18[17]\ : 
        CFG4
      generic map(INIT => x"F4F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_800_2\, 
        D => \can_top_level_inst/drv_bus[255]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_800\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_1_0[3]\ : 
        CFG4
      generic map(INIT => x"AAD8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[3]\, C => 
        \can_top_level_inst/stat_bus[191]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_1_0_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa3\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_shift_cmd_gen.0.un1_rx_shift_cmd\ : 
        CFG4
      generic map(INIT => x"C404")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1077\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_19[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y7_0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y5_0[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_8[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\);
    
    
        \can_top_level_inst/frame_filters_inst/un1_filter_range_enable_0\ : 
        CFG4
      generic map(INIT => x"3F5F")

      port map(A => \can_top_level_inst/drv_bus[269]\, B => 
        \can_top_level_inst/drv_bus[270]\, C => 
        \can_top_level_inst/stat_bus[156]\, D => 
        \can_top_level_inst/stat_bus[155]\, Y => 
        \can_top_level_inst/frame_filters_inst/un1_filter_range_enable_0_Z\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q_RNO[2]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_15[1]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[29]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_81_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[30]\);
    
    \can_top_level_inst/frame_filters_inst/ident_valid_d\ : CFG4
      generic map(INIT => x"7F33")

      port map(A => \can_top_level_inst/drv_bus[330]\, B => 
        \can_top_level_inst/drv_bus[329]\, C => 
        \can_top_level_inst/rec_is_rtr\, D => 
        \can_top_level_inst/frame_filters_inst/un6_filter_result\, 
        Y => 
        \can_top_level_inst/frame_filters_inst/ident_valid_d_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_28\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_23_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_16_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_28_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_wmux_3_Y[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_1_0_y0[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[22]\);
    
    \timestamp_ibuf[61]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(61), Y => 
        \timestamp_ibuf[61]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[6]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[12]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[12]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[21]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m16_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_7[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y7[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y5[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_3[0]\);
    
    \sbe_ibuf[2]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \sbe_c[2]\, E => ADLIB_GND, YIN => 
        \sbe_ibuf[2]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_7[31]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_20_0\, 
        B => \can_top_level_inst/drv_bus[477]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_82_mux\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[5]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[5]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[13]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[2]\ : 
        CFG4
      generic map(INIT => x"47FF")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[6]\, B
         => \can_top_level_inst/drv_bus[475]\, C => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[15]\, 
        D => \can_top_level_inst/txtb_sw_cmd.set_abt_rep1\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_N_141\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select[2]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa20_1\, 
        B => \sbe_c[2]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2[6]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/NN_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram53__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_349\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe53\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_RNIO1SK1[5]\ : 
        CFG4
      generic map(INIT => x"7FFF")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[5]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[4]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[2]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[1]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/g1_4_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_2[2]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[17]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un17_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[12]\ : 
        SLE
      port map(D => \data_in_c[12]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[155]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_0[21]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_0_1_Z[21]\, 
        B => \can_top_level_inst/txtb_prorities_5[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_30_1_0_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[29]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNIK7JE8_S[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[284]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[20]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[20]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[9]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_o2[2]\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_9\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_932\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1043\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_2_1_0_y0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/frame_count_ctr_proc.un10_read_increment\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un10_read_increment_1\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/read_counter_q_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/un10_read_increment\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[28]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI79P86[8]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[281]\, C => 
        \can_top_level_inst/stat_bus[265]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_7\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI79P86_S[8]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_8\, 
        CC => NET_CC_CONFIG2600, P => NET_CC_CONFIG2597, Y3 => 
        NET_CC_CONFIG2598, Y3A => NET_CC_CONFIG2599);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[22]\ : 
        SLE
      port map(D => \timestamp_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2_0_0[1]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1320\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_953\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a2_0_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_2\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[402]\, B => 
        \can_top_level_inst/stat_bus[83]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_1_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_2_Z\, 
        CC => NET_CC_CONFIG1537, P => NET_CC_CONFIG1534, Y3 => 
        NET_CC_CONFIG1535, Y3A => NET_CC_CONFIG1536);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[7]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[4]\);
    
    \data_out_obuf[16]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(16), D => \data_out_obuf[16]/DOUT\, 
        E => \data_out_obuf[16]/EOUT\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_20[24]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[24]\, B
         => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1068\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[31]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[12]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa17\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe17\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[3]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un18_sel_tseg1_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length1[4]\, 
        D => \can_top_level_inst/prescaler_inst/tseg1_nbt[4]\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/exp_seg_length_d[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_107_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_i[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_write_pointer[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[7]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[4]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[4]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_1_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/read_pointer_i[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_read_pointer[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8[6]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_3[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y3[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y1[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_8_Y[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[19]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[6]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry[4]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[4]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[3]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_cry_Z[4]\, 
        CC => NET_CC_CONFIG1071, P => NET_CC_CONFIG1068, Y3 => 
        NET_CC_CONFIG1069, Y3A => NET_CC_CONFIG1070);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry[4]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw_i_s_2638_Y\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[4]\, 
        D => \can_top_level_inst/rx_write_pointer[4]\, FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[3]\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_cry_Z[4]\, 
        CC => NET_CC_CONFIG385, P => NET_CC_CONFIG382, Y3 => 
        NET_CC_CONFIG383, Y3A => NET_CC_CONFIG384);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[8]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI0H3V6[9]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[282]\, C => 
        \can_top_level_inst/stat_bus[266]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_8\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI0H3V6_S[9]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_9\, 
        CC => NET_CC_CONFIG2604, P => NET_CC_CONFIG2601, Y3 => 
        NET_CC_CONFIG2602, Y3A => NET_CC_CONFIG2603);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[6]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[6]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[15]\ : 
        CFG4
      generic map(INIT => x"00CA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[15]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[16]\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_814_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_o2_3_0[1]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_926\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_o2_3_0_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m24_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[6]\ : 
        CFG4
      generic map(INIT => x"CDCC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_140\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[6]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2[1]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_1_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[377]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram5_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe5\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram5_0\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/br_shifted_u_1_1\ : 
        CFG4
      generic map(INIT => x"55CF")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_223\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/br_shifted_u_1_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_1[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram42_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram43_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y5[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIAAIS\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_22\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_22_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[26]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa12\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe12\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_RNIP3CO2[4]\ : 
        ARI1_CC
      generic map(INIT => x"4D800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\, C
         => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_ts[4]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[3]\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_s[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_ts_i_cry[4]\, 
        CC => NET_CC_CONFIG1111, P => NET_CC_CONFIG1108, Y3 => 
        NET_CC_CONFIG1109, Y3A => NET_CC_CONFIG1110);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram124__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_347\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe124\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram63_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe63\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram63_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[1]\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_153\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_152\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_m2[21]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[16]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1024\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m23_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_20[4]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => \can_top_level_inst/drv_bus[404]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/drv_bus[85]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1298\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa14\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe14\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_1[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram74_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram75_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y5[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_0[19]\ : 
        CFG4
      generic map(INIT => x"D0FA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/stat_bus[276]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_0_Z[19]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_0_1_Z[19]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_2_0_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2[4]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[1]\ : 
        CFG4
      generic map(INIT => x"00AC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state_0_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1033\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_842_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[20]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22[20]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[2]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI46KGJ_S[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[248]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI3OU69[9]\ : 
        ARI1_CC
      generic map(INIT => x"5D827")

      port map(A => \timestamp_c[41]\, B => 
        \can_top_level_inst/txtb_index_muxed[0]\, C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[9]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[9]\, 
        FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_8\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_9\, 
        CC => NET_CC_CONFIG963, P => NET_CC_CONFIG960, Y3 => 
        NET_CC_CONFIG961, Y3A => NET_CC_CONFIG962);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62[2]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_62_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/read_pointer_i[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_read_pointer[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_2[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_5_sqmuxa_i_o2\ : 
        CFG3
      generic map(INIT => x"BA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        B => \can_top_level_inst/drv_bus[460]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[28]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_957\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[2]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2[4]\ : 
        CFG4
      generic map(INIT => x"5708")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_1[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_1_Z[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_15_2_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[16]\);
    
    AFLSDF_INV_15 : INV_BA
      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1317_i\, 
        Y => \AFLSDF_INV_15\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select[3]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6_1\, 
        B => \sbe_c[3]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_2_0\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/addr_dec_2[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[0]\);
    
    \data_in_ibuf[5]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(5), Y => \data_in_ibuf[5]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[31]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[31]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[9]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[30]\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf[1]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[1]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/buffer_md_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[30]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[30]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_13[25]\ : 
        CFG4
      generic map(INIT => x"0AC0")

      port map(A => \can_top_level_inst/stat_bus[282]\, B => 
        \can_top_level_inst/drv_bus[296]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1099\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_RNIIEBG1[3]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[0]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[2]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_s[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/compl_ctr_q_cry[3]\, 
        CC => NET_CC_CONFIG1617, P => NET_CC_CONFIG1614, Y3 => 
        NET_CC_CONFIG1615, Y3A => NET_CC_CONFIG1616);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1142\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1143\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_1_30_2_0_y0[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_cZ[2]\ : 
        CFG4
      generic map(INIT => x"BAAA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[34]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1451\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_0_a2_7[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_pos[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_1_0[21]\ : 
        CFG4
      generic map(INIT => x"D9C8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_943\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_942\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_15_1_0_Z[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_81\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[27]\, B => 
        \timestamp_c[58]\, C => \timestamp_c[59]\, D => 
        \can_top_level_inst/tran_word[26]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[12]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[13]\, 
        CC => NET_CC_CONFIG2305, P => NET_CC_CONFIG2302, Y3 => 
        NET_CC_CONFIG2303, Y3A => NET_CC_CONFIG2304);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2[1]\ : 
        CFG4
      generic map(INIT => x"FFEC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_0_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_138\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_2_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_116_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_1[17]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[131]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_1_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un2_err_frm_req_i\ : 
        CFG4
      generic map(INIT => x"7040")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/bit_err_disable_1_1\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1310\, 
        C => \can_top_level_inst/bit_err\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un2_bit_err_enable_i_0\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_1250\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\);
    
    \timestamp_ibuf[40]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(40), Y => 
        \timestamp_ibuf[40]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_108_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[2]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_trigger_reg_inst/reg_q\ : 
        SLE
      port map(D => \can_top_level_inst/tx_trigger\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_trigger_q\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c2_RNIA16T1\ : 
        CFG4
      generic map(INIT => x"F6F9")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_Z[2]\, 
        B => \can_top_level_inst/drv_bus[10]\, C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_NE_5_0\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_a_4_c2_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_tq_counter_d_NE_5_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram42__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_344\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe42\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.un4_rx_shift_ena_3\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_counted_byte_index[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[84]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_5[29]\ : 
        CFG2
      generic map(INIT => x"6")

      port map(A => \can_top_level_inst/stat_bus[300]\, B => 
        \can_top_level_inst/stat_bus[301]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_17_i_x4_0\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_34[2]\ : 
        CFG4
      generic map(INIT => x"CCFE")

      port map(A => \can_top_level_inst/trv_delay[2]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1236_2\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1236\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_a4[7]\ : 
        CFG3
      generic map(INIT => x"07")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_arb_1_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[30]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1124\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_18_2[16]\ : 
        CFG4
      generic map(INIT => x"C480")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[351]\, D => 
        \can_top_level_inst/drv_bus[49]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_762_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[13]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[13]\);
    
    \data_in_ibuf[30]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[30]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[30]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[13]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un7_retr_ctr_clear_i\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/failed_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_retr_ctr_clear_i_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/retr_ctr_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI0HBC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram119__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_335\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_349\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe119\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[23]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2549\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2481\, Y
         => \can_top_level_inst/memory_registers_inst/N_2679\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[14]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[14]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[30]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[30]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[30]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r[13]\ : 
        SLE
      port map(D => \data_in_c[13]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_124_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4_2[6]\ : 
        CFG4
      generic map(INIT => x"C808")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_i_inc_1_cry_6_S\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, D
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_6_S\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[4]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_133\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_buf_cmds_proc.rec_valid_2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[13]\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_one\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid_d_1_sqmuxa_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rec_valid_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_111_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[11]\);
    
    \scs_ibuf/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => scs_c, E => ADLIB_GND, YIN => \scs_ibuf/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[28]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry[1]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[1]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s_2642_FCO\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[1]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[1]\, 
        CC => NET_CC_CONFIG2350, P => NET_CC_CONFIG2347, Y3 => 
        NET_CC_CONFIG2348, Y3A => NET_CC_CONFIG2349);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[6]\);
    
    \timestamp_ibuf[11]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[11]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[11]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[7]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[7]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m34_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram8__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_334\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_348\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe8\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_0[0]\ : 
        CFG3
      generic map(INIT => x"AB")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/next_state_16_sqmuxa\, 
        B => \can_top_level_inst/can_core_inst/bds_data_out\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_890_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_0_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_3_Y[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[30]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[17]\ : 
        CFG4
      generic map(INIT => x"C055")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/N_1186\, 
        B => \can_top_level_inst/tran_word[9]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_load_data_word\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1[20]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_1_Z[20]\, 
        B => \can_top_level_inst/rx_mem_free[4]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_15_1_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[28]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[4]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[4]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4813\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_0\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err[0]\, 
        B => \can_top_level_inst/drv_bus[56]\, C => ADLIB_GND, D
         => ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => OPEN, 
        FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_0_Z\, 
        CC => NET_CC_CONFIG2450, P => NET_CC_CONFIG2447, Y3 => 
        NET_CC_CONFIG2448, Y3A => NET_CC_CONFIG2449);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_5_24[4]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[209]\, C => 
        \can_top_level_inst/drv_bus[172]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1302\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_21\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/un24_parity_mismatch_21_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_y0[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.perform_hsync_2_i_a2\ : 
        CFG4
      generic map(INIT => x"002A")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/perform_hsync_2_i_a2_2\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un109_ctrl_ctr_zero\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_3_Z[3]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1457\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[27]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_RNIH1071\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_RNIH1071_Z\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_1\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[1]\, B => 
        \can_top_level_inst/drv_bus[301]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_0\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_1\, 
        CC => NET_CC_CONFIG533, P => NET_CC_CONFIG530, Y3 => 
        NET_CC_CONFIG531, Y3A => NET_CC_CONFIG532);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[26]\);
    
    AFLSDF_INV_3 : INV_BA
      port map(A => \AFLSDF_INV_3\, Y => 
        \I_1/U0_RGB1_RGB6_rgb_net_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[3]\);
    
    \data_out_obuf[8]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(8), D => \data_out_obuf[8]/DOUT\, 
        E => \data_out_obuf[8]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[15]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[12]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[12]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram112__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_343\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe112\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[158]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_3_RNI3DCL2\ : 
        ARI1_CC
      generic map(INIT => x"5E44E")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_sn_N_2_i\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_cry_3_S\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_read_increment_1_Z[0]\, 
        FCI => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_2\, 
        S => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/N_2375\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_8_1_cry_3\, 
        CC => NET_CC_CONFIG1965, P => NET_CC_CONFIG1962, Y3 => 
        NET_CC_CONFIG1963, Y3A => NET_CC_CONFIG1964);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa3\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_1[7]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[14]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un9_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[15]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIQ5P6B[29]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[29]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[29]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_28\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_29\, 
        CC => NET_CC_CONFIG1393, P => NET_CC_CONFIG1390, Y3 => 
        NET_CC_CONFIG1391, Y3A => NET_CC_CONFIG1392);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry[6]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[6]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[5]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[6]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_cry_Z[6]\, 
        CC => NET_CC_CONFIG2370, P => NET_CC_CONFIG2367, Y3 => 
        NET_CC_CONFIG2368, Y3A => NET_CC_CONFIG2369);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_RNO[1]\ : 
        CFG3
      generic map(INIT => x"78")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[1]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_3[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_RNO[29]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2685\, D
         => \can_top_level_inst/memory_registers_inst/N_2718\, Y
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_shift_cmd_gen.2.un17_rx_shift_cmd\ : 
        CFG4
      generic map(INIT => x"C404")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1077\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un1_rx_shift_cmd_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un17_rx_shift_cmd\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_9[31]\ : 
        CFG3
      generic map(INIT => x"53")

      port map(A => \can_top_level_inst/tran_identifier[28]\, B
         => \can_top_level_inst/tran_identifier[17]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_11\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/int_stat_proc.un1_int_status_clear\ : 
        CFG4
      generic map(INIT => x"8F88")

      port map(A => \can_top_level_inst/N_138\, B => 
        \data_in_c[5]\, C => \can_top_level_inst/int_mask[5]\, D
         => \can_top_level_inst/stat_bus[6]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.5.int_module_inst/un1_int_status_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[18]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_0_1[9]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[309]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_30_1_0_1_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_76_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[25]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[5]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[5]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram9__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_348\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe9\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/load_ts_lw_addr_i_o2_0\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/l2_prio_19\, B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arb_locked\, C
         => \can_top_level_inst/tx_arbitrator_inst/l2_prio_09\, Y
         => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_114\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/un4_tst_ena_1\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => \can_top_level_inst/reg_value_r_rep[17]\, Y => 
        \can_top_level_inst/memory_registers_inst_test_registers_in_tst_rdata_sn_N_8\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[24]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[24]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[24]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2648\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_2_2[1]\ : 
        CFG4
      generic map(INIT => x"07C7")

      port map(A => \can_top_level_inst/rx_data_overrun\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_1_Z[1]\, 
        D => \can_top_level_inst/drv_bus[36]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_15_2_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_110_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOD6T6_S[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[228]\);
    
    \data_in_ibuf[12]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(12), Y => \data_in_ibuf[12]/YIN\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[19]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[19]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_RNI98V81[6]\ : 
        CFG3
      generic map(INIT => x"4C")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_abt\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[6]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_fsm_inst_N_129_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_a3_2[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/data_selector[1]\, 
        C => \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\, 
        D => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_a3_2_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_15[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram88_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram89_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_6[0]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[26]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[26]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_1[1]\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => \can_top_level_inst/txtb_hw_cmd.arbl\, B => 
        \can_top_level_inst/txtb_hw_cmd.err\, C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_fsm_inst/N_193\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_1[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[11]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_50_RNIDSB8C\ : 
        CFG2
      generic map(INIT => x"7")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_I_50\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i_1_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_18_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_18_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNIGS19[8]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[8]\, 
        Y => \can_top_level_inst/can_core_inst/act_err_ovr_flag\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[13]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2539\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2471\, Y
         => \can_top_level_inst/memory_registers_inst/N_2669\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_1[3]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNIERIU[0]\ : 
        ARI1_CC
      generic map(INIT => x"58DD8")

      port map(A => \can_top_level_inst/stat_bus[71]\, B => 
        \can_top_level_inst/drv_bus[425]\, C => 
        \can_top_level_inst/drv_bus[416]\, D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_tx_err_ctr_dec_Z[4]\, 
        FCI => ADLIB_GND, S => OPEN, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q_RNIERIU_Y[0]\, 
        FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d_cry_0\, 
        CC => NET_CC_CONFIG2490, P => NET_CC_CONFIG2487, Y3 => 
        NET_CC_CONFIG2488, Y3A => NET_CC_CONFIG2489);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI8N9PA_S[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[202]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[13]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2539\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[241]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, B => 
        \can_top_level_inst/prescaler_inst/segment_end\, C => 
        \can_top_level_inst/prescaler_inst/start_edge\, D => 
        \can_top_level_inst/prescaler_inst/resync_edge_valid\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/stat_bus[0]\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d_0_sqmuxa_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[23]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[19]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2643\, Y
         => \can_top_level_inst/memory_registers_inst/N_2708\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30[6]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_30_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram10_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram11_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y5[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/g0_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_21[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[16]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un3_segment_end_i_0_1_N_2L1\ : 
        CFG4
      generic map(INIT => x"135F")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segm_end_req_capt_q_Z[2]\, 
        B => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/segm_end_req_capt_q_Z[1]\, 
        C => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_nbt_tq_active\, 
        Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un3_segment_end_i_0_1_N_2L1_Z\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_s_2639_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG1052, P(1) => NET_CC_CONFIG1056, P(2) => 
        NET_CC_CONFIG1060, P(3) => NET_CC_CONFIG1064, P(4) => 
        NET_CC_CONFIG1068, P(5) => NET_CC_CONFIG1072, P(6) => 
        NET_CC_CONFIG1076, P(7) => NET_CC_CONFIG1080, P(8) => 
        NET_CC_CONFIG1084, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, 
        P(11) => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1053, Y3(1) => 
        NET_CC_CONFIG1057, Y3(2) => NET_CC_CONFIG1061, Y3(3) => 
        NET_CC_CONFIG1065, Y3(4) => NET_CC_CONFIG1069, Y3(5) => 
        NET_CC_CONFIG1073, Y3(6) => NET_CC_CONFIG1077, Y3(7) => 
        NET_CC_CONFIG1081, Y3(8) => NET_CC_CONFIG1085, Y3(9) => 
        ADLIB_VCC, Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, 
        Y3A(0) => NET_CC_CONFIG1054, Y3A(1) => NET_CC_CONFIG1058, 
        Y3A(2) => NET_CC_CONFIG1062, Y3A(3) => NET_CC_CONFIG1066, 
        Y3A(4) => NET_CC_CONFIG1070, Y3A(5) => NET_CC_CONFIG1074, 
        Y3A(6) => NET_CC_CONFIG1078, Y3A(7) => NET_CC_CONFIG1082, 
        Y3A(8) => NET_CC_CONFIG1086, Y3A(9) => ADLIB_VCC, Y3A(10)
         => ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG1055, CC(1) => NET_CC_CONFIG1059, CC(2) => 
        NET_CC_CONFIG1063, CC(3) => NET_CC_CONFIG1067, CC(4) => 
        NET_CC_CONFIG1071, CC(5) => NET_CC_CONFIG1075, CC(6) => 
        NET_CC_CONFIG1079, CC(7) => NET_CC_CONFIG1083, CC(8) => 
        NET_CC_CONFIG1087, CC(9) => nc335, CC(10) => nc24, CC(11)
         => nc88);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[29]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_7_sqmuxa_i_o2\ : 
        CFG2
      generic map(INIT => x"B")

      port map(A => \can_top_level_inst/drv_bus[460]\, B => 
        \can_top_level_inst/drv_bus[511]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_949\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram25_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe25\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram25_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_17\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_17_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[13]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/segm_extension_0[4]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[4]\, 
        C => \can_top_level_inst/drv_bus[60]\, Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/N_53\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIB9FT[0]_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO921, CO => CI_TO_CO922, P(0) => 
        NET_CC_CONFIG936, P(1) => NET_CC_CONFIG940, P(2) => 
        NET_CC_CONFIG944, P(3) => NET_CC_CONFIG948, P(4) => 
        NET_CC_CONFIG952, P(5) => NET_CC_CONFIG956, P(6) => 
        NET_CC_CONFIG960, P(7) => NET_CC_CONFIG964, P(8) => 
        NET_CC_CONFIG968, P(9) => NET_CC_CONFIG972, P(10) => 
        NET_CC_CONFIG976, P(11) => NET_CC_CONFIG980, Y3(0) => 
        NET_CC_CONFIG937, Y3(1) => NET_CC_CONFIG941, Y3(2) => 
        NET_CC_CONFIG945, Y3(3) => NET_CC_CONFIG949, Y3(4) => 
        NET_CC_CONFIG953, Y3(5) => NET_CC_CONFIG957, Y3(6) => 
        NET_CC_CONFIG961, Y3(7) => NET_CC_CONFIG965, Y3(8) => 
        NET_CC_CONFIG969, Y3(9) => NET_CC_CONFIG973, Y3(10) => 
        NET_CC_CONFIG977, Y3(11) => NET_CC_CONFIG981, Y3A(0) => 
        NET_CC_CONFIG938, Y3A(1) => NET_CC_CONFIG942, Y3A(2) => 
        NET_CC_CONFIG946, Y3A(3) => NET_CC_CONFIG950, Y3A(4) => 
        NET_CC_CONFIG954, Y3A(5) => NET_CC_CONFIG958, Y3A(6) => 
        NET_CC_CONFIG962, Y3A(7) => NET_CC_CONFIG966, Y3A(8) => 
        NET_CC_CONFIG970, Y3A(9) => NET_CC_CONFIG974, Y3A(10) => 
        NET_CC_CONFIG978, Y3A(11) => NET_CC_CONFIG982, CC(0) => 
        NET_CC_CONFIG939, CC(1) => NET_CC_CONFIG943, CC(2) => 
        NET_CC_CONFIG947, CC(3) => NET_CC_CONFIG951, CC(4) => 
        NET_CC_CONFIG955, CC(5) => NET_CC_CONFIG959, CC(6) => 
        NET_CC_CONFIG963, CC(7) => NET_CC_CONFIG967, CC(8) => 
        NET_CC_CONFIG971, CC(9) => NET_CC_CONFIG975, CC(10) => 
        NET_CC_CONFIG979, CC(11) => NET_CC_CONFIG983);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_10[15]\ : 
        CFG4
      generic map(INIT => x"C0A0")

      port map(A => \can_top_level_inst/drv_bus[253]\, B => 
        \can_top_level_inst/txtb_state_3[3]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_716\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_ac0_3_RNI62DBH\ : 
        ARI1_CC
      generic map(INIT => x"5D22D")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_inc[4]\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_4_c3\, 
        C => \can_top_level_inst/stat_bus[84]\, D => 
        \can_top_level_inst/stat_bus[85]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_3\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_sat_cry_4\, 
        CC => NET_CC_CONFIG103, P => NET_CC_CONFIG100, Y3 => 
        NET_CC_CONFIG101, Y3A => NET_CC_CONFIG102);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8143\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8139\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[6]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[6]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_RNO[1]\ : 
        CFG4
      generic map(INIT => x"0C0E")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[3]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[1]\, C
         => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_72\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_122\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_89_i\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un1_dec_one\ : 
        CFG4
      generic map(INIT => x"F8F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_decrement_rec_0_0\, 
        B => \can_top_level_inst/can_core_inst/pc_rx_trigger\, C
         => \can_top_level_inst/tran_valid\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state_proc_un1_is_receiver\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/dec_one\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_wdata[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[24]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNI0IAP[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/txtb_index_muxed[0]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[3]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[3]\, 
        Y => \can_top_level_inst/tran_word[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[17]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[17]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[17]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[17]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2641\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[1]\ : 
        CFG4
      generic map(INIT => x"2A00")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \adress_c[7]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe6\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m127_2\ : 
        CFG4
      generic map(INIT => x"4C7C")

      port map(A => \can_top_level_inst/drv_bus[181]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m127_2_1_0_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => \can_top_level_inst/drv_bus[305]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/N_128\);
    
    
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_tx_data_cache_inst.tx_cache_ram3__RNO[0]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[0]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/un1_tx_trigger_ssp_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[2]\, 
        D => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/write_pointer_q_Z[1]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/tx_data_cache_inst/tx_cache_0_awe3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_14[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_5[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram54_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram55_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y3_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[9]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[9]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_0_sqmuxa_5_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_5_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_4_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_next_state164_25_sn\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram13__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_332\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_342\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe13\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[21]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[53]\, 
        C => \can_top_level_inst/rec_ident[21]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2266\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_rx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[88]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[31]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/err_warning_limit_1\ : 
        CFG3
      generic map(INIT => x"93")

      port map(A => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_8_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/err_warning_limit_q_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_rx_err_ctr_mt_ewl_cry_8_Z\, 
        Y => \can_top_level_inst/stat_bus[253]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[26]\);
    
    \can_top_level_inst/can_core_inst/trigger_mux_inst/g0_1\ : 
        CFG3
      generic map(INIT => x"47")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un6_same_bits_rst_trig_i\, 
        B => \can_top_level_inst/tx_trigger\, C => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_data_halt_q\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst_crc_17_21_trigger_1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[12]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[8]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[8]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[8]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4815\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[16]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_rx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[84]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[28]\ : 
        SLE
      port map(D => \data_in_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[142]\);
    
    \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f[0]\ : CFG3
      generic map(INIT => x"E4")

      port map(A => \can_top_level_inst/drv_bus[351]\, B => 
        \can_top_level_inst/rec_valid_f\, C => 
        \can_top_level_inst/sof_pulse\, Y => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m17_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_RNII9KD5[3]\ : 
        ARI1_CC
      generic map(INIT => x"48800")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry_cy_Y[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[3]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[2]\, 
        S => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_s[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_cry[3]\, 
        CC => NET_CC_CONFIG665, P => NET_CC_CONFIG662, Y3 => 
        NET_CC_CONFIG663, Y3A => NET_CC_CONFIG664);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60[9]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_60_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_6[1]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2527\, B => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        C => \can_top_level_inst/memory_registers_inst/N_2459\, Y
         => \can_top_level_inst/memory_registers_inst/N_2657\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[24]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[30]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIUBEJ4_S[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[225]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_34_1[23]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/stat_bus[211]\, B => 
        \can_top_level_inst/drv_bus[382]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1044_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[1]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[31]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[20]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_53_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_17[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram28_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram29_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_7[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_0_y0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select[2]\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_1_0[2]\, 
        B => \sbe_c[2]\, C => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_1_0_Z[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[16]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio\ : 
        CFG4
      generic map(INIT => x"F070")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_1_Z\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_0_Z\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_i1_mux_2\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_26_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[18]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[19]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[19]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[19]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_22_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_y0[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1[19]\ : 
        CFG4
      generic map(INIT => x"8B8A")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_1_Z[19]\, 
        D => \can_top_level_inst/drv_bus[133]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_15_1_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[27]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[27]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[27]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y3[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y1[30]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/txtb_sw_cmd.set_abt\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/tx_command[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_sw_cmd.set_abt\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/write_ts_i_a3\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_48\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un7_rx_err_ctr_declto8_i_a2_0_a2_RNIRU072\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/stat_bus[85]\, B => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_3\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_RNIRU072_S\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_rx_err_ctr_q_cry_4\, 
        CC => NET_CC_CONFIG2334, P => NET_CC_CONFIG2331, Y3 => 
        NET_CC_CONFIG2332, Y3A => NET_CC_CONFIG2333);
    
    \adress_ibuf[9]/U_IOPAD\ : IOPAD_IN
      port map(PAD => adress(9), Y => \adress_ibuf[9]/YIN\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[11]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2469\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_no_data_transmitter\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => \can_top_level_inst/tran_dlc[3]\, B => 
        \can_top_level_inst/tran_dlc[2]\, C => 
        \can_top_level_inst/tran_dlc[1]\, D => 
        \can_top_level_inst/tran_dlc[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_no_data_transmitter_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/segm_extension_0_RNI70REA4[4]\ : 
        ARI1_CC
      generic map(INIT => x"58778")

      port map(A => \can_top_level_inst/drv_bus[33]\, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/N_51\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_h_sync_valid_2_1_Z\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_1_cry_0_cy_Y_0\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_3\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length0[4]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/un1_basic_segm_length_0_cry_4\, 
        CC => NET_CC_CONFIG1143, P => NET_CC_CONFIG1140, Y3 => 
        NET_CC_CONFIG1141, Y3A => NET_CC_CONFIG1142);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[29]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram55_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe55\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram55_0\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_17\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[17]\, B => 
        \can_top_level_inst/drv_bus[317]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_16\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_17\, 
        CC => NET_CC_CONFIG597, P => NET_CC_CONFIG594, Y3 => 
        NET_CC_CONFIG595, Y3A => NET_CC_CONFIG596);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[16]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[16]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[16]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[18]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_shift_res_reg_inst/rx_shift_res_reg_inst/reg_q_RNO\ : 
        CFG3
      generic map(INIT => x"A2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_global_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ack_err_0\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1003\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rx_shift_res_reg_inst/rx_shift_res_reg_inst/N_1251_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[21]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.ack_err_flag_clr_1_u\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_protocol_control_fsm_inst_curr_state[6]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un3_ack_err_flag\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[7]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_flag_clr_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[23]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_2[4]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => \can_top_level_inst/txtb_hw_cmd.failed\, B
         => \can_top_level_inst/tran_valid\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_161_2\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un1_inc_eight\ : 
        CFG4
      generic map(INIT => x"3230")

      port map(A => 
        \can_top_level_inst/can_core_inst/act_err_ovr_flag\, B
         => \can_top_level_inst/drv_bus[513]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_rules_inst/un23_inc_eight_i_2_Z\, 
        D => \can_top_level_inst/err_detected\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/inc_eight\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m42_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[21]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[9]\ : 
        CFG4
      generic map(INIT => x"3210")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un18_sel_tseg1_i\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un8_use_basic_segm_length_Z\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[9]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length1[9]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[23]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_2\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \adress_c[10]\, B => \adress_c[11]\, Y => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_2_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[13]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/txtb_cmd_index_gen.1.txtb_priority_odd_gen.txtb_sw_cmd_index_2[1]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[0]\, B
         => \can_top_level_inst/drv_bus[475]\, C => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[9]\, 
        Y => \can_top_level_inst/txtb_sw_cmd_index[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8145\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\);
    
    
        \can_top_level_inst/bus_sampling_inst/shift_regs_rst_reg_inst/rx_shift_res_reg_inst/reg_q_RNO\ : 
        CFG1
      generic map(INIT => "01")

      port map(A => \can_top_level_inst/ssp_reset\, Y => 
        \can_top_level_inst/bus_sampling_inst/shift_regs_rst_reg_inst/rx_shift_res_reg_inst/ssp_reset_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_36_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa_0_a2_0\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_185\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_183\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_184\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_y0[3]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[3]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[3]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_1_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[7]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[4]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/N_133\);
    
    \timestamp_ibuf[39]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(39), Y => 
        \timestamp_ibuf[39]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe16\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram16_0\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[19]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2264\, B
         => \can_top_level_inst/rx_buffer_inst/N_4279\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_4[17]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => \can_top_level_inst/drv_bus[466]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => \can_top_level_inst/drv_bus[160]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_786\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_q[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/un6_crc_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_15[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/g0_1\ : 
        CFG4
      generic map(INIT => x"00B8")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un6_same_bits_rst_trig_i\, 
        B => \can_top_level_inst/tx_trigger\, C => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_data_halt_q\, 
        D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un1_data_halt_d_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/g0_1_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[0]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[16]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[16]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[16]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNIGCJC1_S[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/N_13_i_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[274]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q_RNO[10]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_17[9]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_8[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_3[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y3[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y1[0]\, 
        D => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/N_4234\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_1[1]\ : 
        CFG4
      generic map(INIT => x"EAFA")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_163\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[5]\, 
        D => \can_top_level_inst/N_4606\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_1_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_type_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un24_err_type_d_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/err_detected\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[107]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[2]\);
    
    
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/un9_tx_trig_req_flag_d\ : 
        CFG4
      generic map(INIT => x"FFB3")

      port map(A => \can_top_level_inst/prescaler_inst/is_tseg2\, 
        B => \can_top_level_inst/stat_bus[184]\, C => 
        \can_top_level_inst/prescaler_inst/segment_end\, D => 
        \can_top_level_inst/prescaler_inst/N_27\, Y => 
        \can_top_level_inst/prescaler_inst/trigger_generator_inst/un9_tx_trig_req_flag_d_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_1_Z[0]\, 
        B => \sbe_c[1]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_ident_type_dbl_buf\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[6]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/buffer_md_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tran_ident_type_dbl_buf_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_36[2]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => \can_top_level_inst/stat_bus[302]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1238_1\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1238\);
    
    \can_top_level_inst/tx_arbitrator_inst/last_txtb_index[0]\ : 
        SLE
      port map(D => \can_top_level_inst/txtb_hw_cmd_index[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_last_txtb_index\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/last_txtb_index_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_43_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\);
    
    \data_out_obuf[14]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(14), D => \data_out_obuf[14]/DOUT\, 
        E => \data_out_obuf[14]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29[10]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux_0_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_20_36[19]\ : 
        CFG4
      generic map(INIT => x"88D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[51]\, C => 
        \can_top_level_inst/stat_bus[239]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_894\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[18]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[223]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_i_3\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_i_3_tz_Z\, 
        B => \can_top_level_inst/drv_bus[471]\, C => 
        \can_top_level_inst/stat_bus[0]\, D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ack_err_i_3_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns[3]\ : 
        CFG4
      generic map(INIT => x"F0F2")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_tz_Z[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_158\, 
        D => \can_top_level_inst/txtb_parity_error_valid[6]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a2[1]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_fsm_inst_N_190\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[31]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_RNIHESH1[1]\ : 
        CFG4
      generic map(INIT => x"E7DB")

      port map(A => \can_top_level_inst/drv_bus[1]\, B => 
        \can_top_level_inst/drv_bus[0]\, C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[0]\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[1]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_tq_counter_d_NE_0\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/int_stat_proc.un1_int_status_clear\ : 
        CFG4
      generic map(INIT => x"FF80")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_stat_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        B => \sbe_c[0]\, C => \data_in_c[3]\, D => 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/un2_int_status_set\, 
        Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.3.int_module_inst/un1_int_status_clear\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[16]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_s_8\ : 
        ARI1_CC
      generic map(INIT => x"46600")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[8]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err_cry_7_Z\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_nbt_inst/phase_err[8]\, 
        Y => OPEN, FCO => OPEN, CC => NET_CC_CONFIG920, P => 
        NET_CC_CONFIG917, Y3 => NET_CC_CONFIG918, Y3A => 
        NET_CC_CONFIG919);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_31_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[15]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_121_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[26]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_78_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/curr_state[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_37_1_0[15]\ : 
        CFG4
      generic map(INIT => x"B9A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/stat_bus[379]\, D => 
        \can_top_level_inst/stat_bus[235]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_37_1_0_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_metadata\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_metadata_2\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/store_metadata\);
    
    \can_top_level_inst/int_manager_inst/un1_int_i_5\ : CFG4
      generic map(INIT => x"135F")

      port map(A => \can_top_level_inst/int_vector[5]\, B => 
        \can_top_level_inst/int_vector[4]\, C => 
        \can_top_level_inst/int_ena[5]\, D => 
        \can_top_level_inst/int_ena[4]\, Y => 
        \can_top_level_inst/int_manager_inst/un1_int_i_5_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[18]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[194]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram14_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe14\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram14_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[21]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[21]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[21]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram121__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_352\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe121\);
    
    
        \can_top_level_inst/rx_buffer_inst/rst_reg_inst/rx_shift_res_reg_inst/reg_q_RNO\ : 
        CFG3
      generic map(INIT => x"4C")

      port map(A => \data_in_c[2]\, B => 
        \can_top_level_inst/memory_registers_inst_global_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_command_reg_comp_wr_select[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rst_reg_inst/rx_shift_res_reg_inst/un1_rx_buf_res_n_d_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[31]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_brs\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bds_data_out\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_brs\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_brs\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_a4_1_0[1]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_a4_1_0_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_6\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[413]\, B => 
        \can_top_level_inst/stat_bus[76]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_4\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_5\, 
        CC => NET_CC_CONFIG2082, P => NET_CC_CONFIG2079, Y3 => 
        NET_CC_CONFIG2080, Y3A => NET_CC_CONFIG2081);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/reg_value_r[2]\ : 
        SLE
      port map(D => \data_in_c[2]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ewl_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[402]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram32_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe32\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram32_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[18]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[18]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[18]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm_0[1]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[1]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[3]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_RNO[20]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_data_mux_control_registers_comp_un2_index_satlt4_1\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2676\, D
         => \can_top_level_inst/memory_registers_inst/N_2709\, Y
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/tst_rdata_m[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_16_3[15]\ : 
        CFG3
      generic map(INIT => x"31")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[129]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_709\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[10]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[10]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[10]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[28]\ : 
        SLE
      port map(D => \data_in_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[57]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_104_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[0]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[0]\, 
        C => \can_top_level_inst/txtb_port_b_address[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[29]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[29]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[29]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2653\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_11[1]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \can_top_level_inst/drv_bus[301]\, C => 
        \can_top_level_inst/drv_bus[177]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1175\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_1[13]\ : 
        CFG3
      generic map(INIT => x"27")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \can_top_level_inst/drv_bus[156]\, C => 
        \can_top_level_inst/drv_bus[127]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_15_1_1_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[5]\ : 
        CFG4
      generic map(INIT => x"F088")

      port map(A => \can_top_level_inst/rec_is_rtr\, B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_12_mux\, 
        C => \can_top_level_inst/rx_buffer_inst/N_2283\, D => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[30]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[31]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_RNO_8[28]\ : 
        CFG4
      generic map(INIT => x"BFEF")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[3]\, 
        B => \can_top_level_inst/tran_frame_test.tprm[1]\, C => 
        \can_top_level_inst/drv_bus[477]\, D => 
        \can_top_level_inst/tran_frame_test.tprm[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_o4_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_12_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[16]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_93\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[17]\, B => 
        \timestamp_c[48]\, C => \timestamp_c[49]\, D => 
        \can_top_level_inst/tran_word[16]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[7]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[8]\, 
        CC => NET_CC_CONFIG2285, P => NET_CC_CONFIG2282, Y3 => 
        NET_CC_CONFIG2283, Y3A => NET_CC_CONFIG2284);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_92_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[16]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[16]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[16]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_6[6]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_2[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y7[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_1_wmux_5[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram12_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram13_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_1_y0_2[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[13]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[300]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_y0_0[25]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1111\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1114\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_31_2_0_y3[25]\);
    
    \timestamp_ibuf[2]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(2), Y => \timestamp_ibuf[2]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8147\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[7]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_2_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/un1_load_ts_lw_addr_1_i\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_pointer_meta_q_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select[1]\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => \adress_c[5]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_2\, 
        C => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        D => \sbe_c[1]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram38__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_336\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_341\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe38\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_96_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[20]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_q[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_rx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[86]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[17]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_4\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[11]\, B
         => \can_top_level_inst/can_core_inst/crc_21[10]\, C => 
        \can_top_level_inst/can_core_inst/crc_21[11]\, D => 
        \can_top_level_inst/store_data_word[10]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[4]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[5]\, 
        CC => NET_CC_CONFIG22, P => NET_CC_CONFIG19, Y3 => 
        NET_CC_CONFIG20, Y3A => NET_CC_CONFIG21);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[17]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[17]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/err_type_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/N_1170_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/err_detected\, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[106]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[18]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[18]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[18]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4822\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_1\ : 
        CFG3
      generic map(INIT => x"04")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[6]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tran_data_length[1]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_tran_data_length_13_1_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[20]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_2_0_Z[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_4_37_2_Z[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1277\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[50]\ : 
        SLE
      port map(D => \timestamp_c[50]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[50]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[17]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[17]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[17]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_41_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[28]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[4]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[4]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[4]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[6]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[6]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_frame_test.tprm[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/tx_arbitrator_inst/tprm_0[3]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_frame_test.tprm[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1[23]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_1_Z[23]\, 
        B => \can_top_level_inst/rx_mem_free[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_24_15_1_Z[23]\);
    
    \data_in_ibuf[26]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(26), Y => \data_in_ibuf[26]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_16\ : 
        CFG3
      generic map(INIT => x"96")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_16_Z\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_2[0]\ : 
        CFG4
      generic map(INIT => x"0004")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_934\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1320\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        D => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1192\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[18]\);
    
    
        \can_top_level_inst/bus_sampling_inst/data_edge_detector_inst/tx_data_prev\ : 
        SLE
      port map(D => can_tx_c, CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, 
        EN => ADLIB_VCC, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/tx_data_prev\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3[10]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y3[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[10]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_3_Y[10]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNIOAFV75[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[53]\, B => 
        \can_top_level_inst/tran_word[21]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_20\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_21\, 
        CC => NET_CC_CONFIG1011, P => NET_CC_CONFIG1008, Y3 => 
        NET_CC_CONFIG1009, Y3A => NET_CC_CONFIG1010);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8144\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_27[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/stat_bus[4]\, B => 
        \can_top_level_inst/drv_bus[91]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1533\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[15]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/op_gt.un7_rx_err_ctr_declto8_i_a2_0_a2_5\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => \can_top_level_inst/stat_bus[86]\, B => 
        \can_top_level_inst/stat_bus[85]\, C => 
        \can_top_level_inst/stat_bus[83]\, D => 
        \can_top_level_inst/stat_bus[82]\, Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un7_rx_err_ctr_declto8_i_a2_0_a2_5\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_3_iv_i[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/same_bits_q_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram12_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe12\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/ram12_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI79P86_S[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un3_nom_err_ctr_ce_0_a2_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[265]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[0]\ : 
        CFG3
      generic map(INIT => x"20")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_ety\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[7]\, 
        C => \can_top_level_inst/txtb_sw_cmd_index[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram105__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_344\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe105\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[26]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[24]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[12]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIOD6T6_S[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[196]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_1[29]\ : 
        CFG4
      generic map(INIT => x"007F")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[29]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac11_Z\, 
        C => 
        \can_top_level_inst/int_manager_inst_int_input_active[10]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_30_0_iv_7_Z[29]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_1_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[13]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_parity_check_valid_0_RNO_0\ : 
        CFG4
      generic map(INIT => x"0802")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_parity_check_valid_0_RNO_2_Z\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index_reg_Z[1]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/un2_select_index_changed_2_Z\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/select_buf_index[1]\, 
        Y => \can_top_level_inst/tx_arbitrator_inst/r_N_5_mux\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf[3]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[3]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/buffer_md_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1383\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_85_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[23]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[12]\, 
        B => \can_top_level_inst/tran_word[15]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1[6]\ : 
        CFG4
      generic map(INIT => x"0F0E")

      port map(A => \can_top_level_inst/stat_bus[89]\, B => 
        \can_top_level_inst/stat_bus[88]\, C => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_modif_rx_ctr_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_RNO_S[6]\, 
        Y => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/rx_err_ctr_d_1_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/txtb_sw_cmd.set_rdy\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/tx_command[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_sw_cmd.set_rdy\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_125_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/reg_value_r[3]\ : 
        SLE
      port map(D => \data_in_c[3]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[419]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_33[8]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/stat_bus[228]\, B => 
        \timestamp_c[40]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1463\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[31]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_0\ : 
        CFG3
      generic map(INIT => x"13")

      port map(A => \data_in_c[9]\, B => 
        \can_top_level_inst/txtb_parity_error_valid[6]\, C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_command_reg_comp_wr_select[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tx_parity_error_1_sqmuxa_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[20]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[20]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[20]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txtb_port_b_data[2]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[4]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        Y => \can_top_level_inst/txtb_port_b_data_4[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_16[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_6[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram90_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram91_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y5_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_a3[3]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        B => \can_top_level_inst/txtb_allow_bb[0]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        Y => \can_top_level_inst/txtb_state_0[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y0[9]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_3[6]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2251\, B
         => \can_top_level_inst/rx_buffer_inst/N_4814\, C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_7\, 
        Y => \can_top_level_inst/rx_buffer_inst/N_2284\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ctrl_ctr_pload_val_3_sqmuxa_1_i_o4\ : 
        CFG4
      generic map(INIT => x"EAC0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_949\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_956\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_976\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[1]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un2_commit_rx_frame\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_mem_free[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_27_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_27_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m42_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_RNI91HNJ1\ : 
        CFG4
      generic map(INIT => x"B0A0")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_12_1_sx\, 
        B => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_12_1_0_Z\, 
        C => \can_top_level_inst/prescaler_inst/is_tseg2\, D => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/un2_segm_end_dbt_valid_0_Z\, 
        Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_y0_0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_109_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_select[1]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select\, 
        B => \sbe_c[1]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_select_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[10]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO_0[30]\ : 
        CFG4
      generic map(INIT => x"F0E0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac10_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_in_m[926]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_126_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[4]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.un8_ide_is_arbitration\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un5_arbitration_lost_condition_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ide_is_arbitration_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_ide_is_arbitration\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[17]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[17]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[17]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[19]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry[3]\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/segm_counter_nbt[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[2]\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[3]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_cry_Z[3]\, 
        CC => NET_CC_CONFIG304, P => NET_CC_CONFIG301, Y3 => 
        NET_CC_CONFIG302, Y3A => NET_CC_CONFIG303);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_down_proc.shift_regs_4_RNO_0[28]\ : 
        CFG4
      generic map(INIT => x"000E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_a5_2_0\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/G_20_0_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[12]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_RNO_4[31]\ : 
        CFG4
      generic map(INIT => x"BF80")

      port map(A => \can_top_level_inst/tran_frame_test.tprm[3]\, 
        B => \can_top_level_inst/drv_bus[477]\, C => 
        \can_top_level_inst/tran_frame_test.sdlc\, D => 
        \can_top_level_inst/tran_dlc[3]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/N_12\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[9]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[9]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[26]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2552\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_2_2[22]\ : 
        CFG4
      generic map(INIT => x"4F7F")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[22]\, B
         => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_Z[22]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => \can_top_level_inst/drv_bus[227]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_2_1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_21\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[3]\, B
         => \can_top_level_inst/rx_buffer_inst_RAM_data_out[2]\, 
        C => \can_top_level_inst/rx_buffer_inst_RAM_data_out[1]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_21_Z\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_25[17]\ : 
        CFG4
      generic map(INIT => x"C0A0")

      port map(A => \can_top_level_inst/drv_bus[317]\, B => 
        \can_top_level_inst/txtb_prorities_4[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_807\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/un73_txtb_hw_cmd_int_0_a3_0\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_136\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_180\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_load_crc_i_0\ : 
        CFG4
      generic map(INIT => x"FF0D")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_971\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_955\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[20]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_950\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_909\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[18]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2476\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[28]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/int_mask_i\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_mask_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/int_mask_load_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/int_mask[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[29]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[29]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2653\, Y
         => \can_top_level_inst/memory_registers_inst/N_2718\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.ramout_15_1_0_wmux_3[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_port_b_address[0]\, 
        B => \can_top_level_inst/txtb_port_b_address[1]\, C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram8_0\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ramout_15_1_0_y0_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8_RNI1IKF15\ : 
        ARI1_CC
      generic map(INIT => x"49CCC")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[5]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_4\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[5]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_5\, 
        CC => NET_CC_CONFIG2720, P => NET_CC_CONFIG2717, Y3 => 
        NET_CC_CONFIG2718, Y3A => NET_CC_CONFIG2719);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_58_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIK3PC1\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[21]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2_0[4]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => \can_top_level_inst/txtb_sw_cmd_index[4]\, B
         => \can_top_level_inst/txtb_sw_cmd.set_ety\, C => 
        \can_top_level_inst/txtb_sw_cmd.set_rdy\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_136\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[24]\ : 
        SLE
      port map(D => \data_in_c[24]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram10__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa10\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe10\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i[1]\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_5_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1197\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1203\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_900\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[1]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIQ7PL7[9]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[197]\, D => 
        \can_top_level_inst/stat_bus[229]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_8\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIQ7PL7_S[9]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_9\, 
        CC => NET_CC_CONFIG792, P => NET_CC_CONFIG789, Y3 => 
        NET_CC_CONFIG790, Y3A => NET_CC_CONFIG791);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIUBEJ4_S[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[193]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[4]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram125_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe125\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram125_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[28]\);
    
    \timestamp_ibuf[27]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(27), Y => 
        \timestamp_ibuf[27]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_14_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/un14_crc_match_c_RNO_7\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[1]\, B
         => \can_top_level_inst/can_core_inst/crc_17[0]\, C => 
        \can_top_level_inst/can_core_inst/crc_17[1]\, D => 
        \can_top_level_inst/store_data_word[0]\, FCI => ADLIB_GND, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_17_ok_0_data_tmp[0]\, 
        CC => NET_CC_CONFIG2527, P => NET_CC_CONFIG2524, Y3 => 
        NET_CC_CONFIG2525, Y3A => NET_CC_CONFIG2526);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_34[14]\ : 
        CFG4
      generic map(INIT => x"D800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[46]\, C => 
        \can_top_level_inst/stat_bus[234]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_702\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_0_a2_0_a2_0\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_24_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_24_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101[22]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1[18]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => \can_top_level_inst/drv_bus[477]\, B => 
        \can_top_level_inst/tran_frame_test.tprm[1]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/un7_flip_mask_axbxc2\, 
        D => \can_top_level_inst/tran_frame_test.tprm[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_1_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[29]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[29]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un2_ssp_shadow_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_1_i\, 
        SD => ADLIB_VCC, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_delay[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/op_eq.un6_reinteg_ctr_expired_5\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[5]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[4]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[3]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/reinteg_ctr_q_Z[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/reintegration_counter_inst/un6_reinteg_ctr_expired_5\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_0_1.CO2_1\ : 
        CFG4
      generic map(INIT => x"7150")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_Z[1]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_1_Z[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3_Z[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_3_Z[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO2_1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_11[28]\ : 
        CFG3
      generic map(INIT => x"EA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_7_Z[28]\, 
        B => \can_top_level_inst/drv_bus[52]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac29_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_29_1_iv_11_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_hw_cmd_q.valid\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_retr_ctr_clear_i_Z\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/tran_valid\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm_0[3]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[3]\, 
        B => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        C => \can_top_level_inst/prescaler_inst/h_sync_valid\, D
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[7]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_22\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[150]\, B => 
        \can_top_level_inst/rec_ident[7]\, C => 
        \can_top_level_inst/drv_bus[183]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/valid_0_N_56\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_101_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[22]\ : 
        SLE
      port map(D => \data_in_c[22]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[227]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/data_out[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_2[16]\ : 
        CFG4
      generic map(INIT => x"AFC0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_749\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_748\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_1_Z[16]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_15_2_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_38_1_0_wmux_RNO[5]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_0_Z[5]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_6_15_2_Z[5]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1331\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[22]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[22]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[22]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un6_timestamp_valid_0_I_21\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/tran_word[23]\, B => 
        \timestamp_c[54]\, C => \timestamp_c[55]\, D => 
        \can_top_level_inst/tran_word[22]\, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[10]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un6_timestamp_valid_0_data_tmp[11]\, 
        CC => NET_CC_CONFIG2297, P => NET_CC_CONFIG2294, Y3 => 
        NET_CC_CONFIG2295, Y3A => NET_CC_CONFIG2296);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[27]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[12]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[12]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[12]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_RNIH4A85[13]\ : 
        ARI1_CC
      generic map(INIT => x"44400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/btmc_reset\, C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[13]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[12]\, 
        S => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_s[13]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_cry[13]\, 
        CC => NET_CC_CONFIG1730, P => NET_CC_CONFIG1727, Y3 => 
        NET_CC_CONFIG1728, Y3A => NET_CC_CONFIG1729);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_77_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1_0[6]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1_0_1_Z[6]\, 
        B => \can_top_level_inst/txtb_prorities_1[2]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1_0_Z[6]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_3[3]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[26]\, CLK
         => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un25_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIGR327[14]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[14]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[14]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_13\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_14\, 
        CC => NET_CC_CONFIG1333, P => NET_CC_CONFIG1330, Y3 => 
        NET_CC_CONFIG1331, Y3A => NET_CC_CONFIG1332);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a2[4]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \can_top_level_inst/txtb_sw_cmd.set_ety\, B
         => \can_top_level_inst/txtb_sw_cmd.set_rdy\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_176\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[9]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[9]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[6]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/less_than.un19_timestamp_valid_axb_31_i\ : 
        CFG2
      generic map(INIT => x"9")

      port map(A => \can_top_level_inst/tran_word[31]\, B => 
        \timestamp_c[63]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_axb_31_i\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_29_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[18]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[18]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[18]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_17_1[26]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_2_Z[26]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_3_Z[26]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_10_Z[26]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_11_Z[26]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_27_1_iv_17_1_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_6\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[277]\, B => 
        \can_top_level_inst/rec_ident[6]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_5\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_6\, 
        CC => NET_CC_CONFIG1857, P => NET_CC_CONFIG1854, Y3 => 
        NET_CC_CONFIG1855, Y3A => NET_CC_CONFIG1856);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[21]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[259]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_1[12]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_1[12]\, D => 
        \can_top_level_inst/txtb_port_b_data_5[12]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_y0_0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select[3]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_1_Z[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_1_0_Z[1]\, 
        D => \can_top_level_inst/drv_bus[477]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_wdata_reg_comp/wr_select_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/shift_reg_byte_inst/byte_shift_reg_gen.0.shift_reg_proc.shift_reg_q_3[5]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[28]\, CLK
         => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/un25_rx_shift_cmd\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/store_data_word[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_0_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4PNCG_S[23]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[243]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_35_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[30]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2488\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3[3]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y1[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[3]\);
    
    \data_in_ibuf[21]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[21]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[21]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \adress_c[8]\, B => 
        \can_top_level_inst/txtb_allow_bb[2]\, C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_un1_ram_write_0\, 
        D => \adress_c[9]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[21]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[21]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[21]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4281\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_11_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_63_1_0_wmux_20[0]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y0_8[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y3_0[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_y1_0[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_63_1_0_0_y21[0]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIUDL8I[26]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[214]\, D => 
        \can_top_level_inst/stat_bus[246]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_25\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIUDL8I_S[26]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_26\, 
        CC => NET_CC_CONFIG860, P => NET_CC_CONFIG857, Y3 => 
        NET_CC_CONFIG858, Y3A => NET_CC_CONFIG859);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_1_0_wmux_0_Y[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_63_2_0_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[24]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_segm_counter_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data_d_0_sqmuxa\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un4_rx_shift_ena[3]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[21]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un4_ctrl_counted_byte_1\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_data_d_0_sqmuxa_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[14]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5\ : 
        RGB
      port map(A => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_Y\, 
        EN => ADLIB_VCC, Y => \AFLSDF_INV_18\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[3]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[3]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_72_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[14]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/nom_err_ctr_q_RNI7L9M[0]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[273]\, C => 
        \can_top_level_inst/stat_bus[257]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => ADLIB_VCC, S
         => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_0\, 
        CC => NET_CC_CONFIG2568, P => NET_CC_CONFIG2565, Y3 => 
        NET_CC_CONFIG2566, Y3A => NET_CC_CONFIG2567);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[26]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/un5_transient_state_i_o2\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => \can_top_level_inst/txtb_allow_bb[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram85_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe85\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram85_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[12]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[12]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[12]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_76\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[214]\, B => 
        \can_top_level_inst/rec_ident[9]\, C => 
        \can_top_level_inst/drv_bus[247]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_11\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_4[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y5[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_91_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_91_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_2[22]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2548\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[4]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[6]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[4]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y3[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_0_y1[11]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_3_Y[11]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_dec_proc.1.l2_prio_1_1.CO2\ : 
        CFG3
      generic map(INIT => x"B2")

      port map(A => \can_top_level_inst/txtb_prorities_6[2]\, B
         => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_5_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO1_2\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l2_prio_122\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_54_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_53_2_1_y0_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[15]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[15]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[15]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d[14]\ : 
        CFG3
      generic map(INIT => x"12")

      port map(A => \can_top_level_inst/can_core_inst/crc_15[13]\, 
        B => \can_top_level_inst/can_core_inst/load_init_vect\, C
         => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_nxt_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_15_inst/crc_d_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_0[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[9]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y1[9]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI16B69[12]\ : 
        ARI1_CC
      generic map(INIT => x"4AAAC")

      port map(A => \can_top_level_inst/stat_bus[11]\, B => 
        \can_top_level_inst/stat_bus[285]\, C => 
        \can_top_level_inst/stat_bus[269]\, D => 
        \can_top_level_inst/stat_bus[10]\, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_11\, 
        S => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/data_err_ctr_q_RNI16B69_S[12]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un1_nom_dat_sel_ctr_add_cry_12\, 
        CC => NET_CC_CONFIG2616, P => NET_CC_CONFIG2613, Y3 => 
        NET_CC_CONFIG2614, Y3A => NET_CC_CONFIG2615);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_match_d_RNO_2\ : 
        ARI1_CC
      generic map(INIT => x"68421")

      port map(A => \can_top_level_inst/store_data_word[15]\, B
         => \can_top_level_inst/can_core_inst/crc_21[14]\, C => 
        \can_top_level_inst/can_core_inst/crc_21[15]\, D => 
        \can_top_level_inst/store_data_word[14]\, FCI => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[6]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/err_detector_inst/crc_21_ok_0_data_tmp[7]\, 
        CC => NET_CC_CONFIG30, P => NET_CC_CONFIG27, Y3 => 
        NET_CC_CONFIG28, Y3A => NET_CC_CONFIG29);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[19]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[19]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m19_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[19]\ : 
        SLE
      port map(D => \data_in_c[19]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[133]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_0[4]\ : 
        CFG4
      generic map(INIT => x"F9F0")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[0]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un8_is_fd_frame_m\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/next_state_16_sqmuxa\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_0_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[15]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o2_1[2]\ : 
        CFG3
      generic map(INIT => x"FE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[9]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[36]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_968\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac21\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac37_1\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac21_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a2[2]\ : 
        CFG4
      generic map(INIT => x"070F")

      port map(A => \can_top_level_inst/txtb_parity_check_valid\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_curr_m1_e_1\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        D => \can_top_level_inst/txtb_parity_error_valid_1[0]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_175\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[5]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[14]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[14]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[14]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4274\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q\ : 
        SLE
      port map(D => \can_top_level_inst/rec_valid\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_3_Y[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[18]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[16]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, D
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[18]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[18]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_gate_mem_read.ALTB_1[2]\ : 
        CFG4
      generic map(INIT => x"2302")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_ptr_d[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/AXB[2]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ALTB[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ALTB_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3_a2_2_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m42_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123[11]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[11]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux_0_Y[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_Z[11]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIC0M98_S[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q_0\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[230]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa9\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe9\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_4\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[404]\, B => 
        \can_top_level_inst/stat_bus[75]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_3_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_ewl_cry_4_Z\, 
        CC => NET_CC_CONFIG228, P => NET_CC_CONFIG225, Y3 => 
        NET_CC_CONFIG226, Y3A => NET_CC_CONFIG227);
    
    \data_in_ibuf[19]/U_IOPAD\ : IOPAD_IN
      port map(PAD => data_in(19), Y => \data_in_ibuf[19]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[8]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_5[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select[3]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_0_0\, 
        D => \adress_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_2_2[12]\ : 
        CFG4
      generic map(INIT => x"07C7")

      port map(A => \can_top_level_inst/stat_bus[269]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_1_Z[12]\, 
        D => \can_top_level_inst/drv_bus[93]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_30_2_1[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_load_dlc_i_a2\ : 
        CFG3
      generic map(INIT => x"23")

      port map(A => 
        \can_top_level_inst/can_core_inst/bds_data_out\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_934\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_1324\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_3[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_36_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m31_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_identifier_com[11]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[11]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_identifier[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_cry_8_RNI3EGI77\ : 
        ARI1_CC
      generic map(INIT => x"49CCC")

      port map(A => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[8]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un2_phase_err_mt_sjw_i\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_1_cry_0_cy_Y\, 
        D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un4_use_basic_segm_length_i_1\, 
        FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_7\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length0[8]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_8\, 
        CC => NET_CC_CONFIG2732, P => NET_CC_CONFIG2729, Y3 => 
        NET_CC_CONFIG2730, Y3A => NET_CC_CONFIG2731);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_25_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_25_Z[17]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_stuff_err_d\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => 
        \can_top_level_inst/can_core_inst/destuff_enable\, B => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un2_stuff_rule_violate_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_discard_stuff_bit_Z\, 
        D => \can_top_level_inst/rx_triggers[1]\, Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un1_stuff_err_d_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[17]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[12]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[7]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[7]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[7]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\);
    
    \int_obuf/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => int_c, E => ADLIB_VCC, DOUT => 
        \int_obuf/DOUT\, EOUT => \int_obuf/EOUT\);
    
    \data_out_obuf[19]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[19]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[19]/DOUT\, EOUT => 
        \data_out_obuf[19]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[3]\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst_priority_decoder_inst_un1_m2_0_a2_0_1\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_Z\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/curr_m4_0_a2_2_0\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_fsm_inst/N_158\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[24]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[24]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/txtb_state_i_a3[1]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_148\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[22]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[22]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_dominant_36_sqmuxa_0_a4\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => \can_top_level_inst/stat_bus[380]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[22]\);
    
    \data_out_obuf[22]/U_IOTRI\ : IOTRI_OB_EB
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(D => \data_out_c[22]\, E => ADLIB_VCC, DOUT => 
        \data_out_obuf[22]/DOUT\, EOUT => 
        \data_out_obuf[22]/EOUT\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_RNII8N1G7[10]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[58]\, B => 
        \can_top_level_inst/tran_word[26]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_25\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid_cry_26\, 
        CC => NET_CC_CONFIG1031, P => NET_CC_CONFIG1028, Y3 => 
        NET_CC_CONFIG1029, Y3A => NET_CC_CONFIG1030);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[0]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_95_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_95_Z[17]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[23]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[166]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[6]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_4[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/reg_value_r[20]\ : 
        SLE
      port map(D => \data_in_c[20]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_VAL_present_gen_t.filter_a_val_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[134]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[27]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[14]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[14]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[14]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[23]\);
    
    \timestamp_ibuf[63]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(63), Y => 
        \timestamp_ibuf[63]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_2_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m37_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_0[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_VAL_present_gen_t.filter_c_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[249]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_1[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_111_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1[6]\ : 
        CFG4
      generic map(INIT => x"139B")

      port map(A => \adress_c[3]\, B => \adress_c[2]\, C => 
        \can_top_level_inst/test_registers_out.tst_wdata[6]\, D
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[6]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_1[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_y0_0[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_5[30]\ : 
        CFG3
      generic map(INIT => x"EA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_3_Z[30]\, 
        B => \can_top_level_inst/drv_bus[54]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac29_Z\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_31_0_iv_5_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_2[30]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_y0_0[30]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[23]\ : 
        SLE
      port map(D => \data_in_c[23]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[12]\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dff_fixed_stuff_reg/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/fixed_prev_d\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/destuff_enable\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/fixed_prev_q\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_2_1_0_y1[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNIT89S\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa_0_a2_0\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_185\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_write_i_Z\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_183\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_184\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8149\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0[16]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[16]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2[26]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[15]\, 
        B => \can_top_level_inst/tran_word[2]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm0\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss0_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_m2_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_114_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[14]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/tx_err_ctr_d[8]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/un6_tx_err_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/err_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[79]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[28]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[2]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[2]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[2]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2626\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[21]\ : 
        SLE
      port map(D => \data_in_c[21]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[321]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8140\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_69_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[15]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[15]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[15]\);
    
    
        \can_top_level_inst/can_core_inst/bit_stuffing_inst/un1_data_halt_d_RNIHSAI\ : 
        CFG4
      generic map(INIT => x"00B8")

      port map(A => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un6_same_bits_rst_trig_i\, 
        B => \can_top_level_inst/tx_trigger\, C => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_data_halt_q\, 
        D => 
        \can_top_level_inst/can_core_inst/bit_stuffing_inst_un1_data_halt_d_i\, 
        Y => \can_top_level_inst/can_core_inst/data_halt\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_82_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[8]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_1_0[1]\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_store_edl_i_1\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_953\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_1_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[12]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[12]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[12]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_33_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_33_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[24]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[24]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/data_out_Z[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_y0[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_122_1_0_wmux_0_Y[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata[4]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/N_2693\, B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_9\, 
        C => \can_top_level_inst/memory_registers_inst/N_2660\, Y
         => 
        \can_top_level_inst/memory_registers_inst/tst_rdata[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[5]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[8]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[8]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y3[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[26]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[26]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[26]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[26]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped[10]\ : 
        CFG4
      generic map(INIT => x"6CCC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_2_0[8]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[10]\, 
        C => \can_top_level_inst/tran_frame_test.fcrc\, D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/flip_mask_0_Z[10]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_flipped_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram17_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe17\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram17_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_i_a2_3[1]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \can_top_level_inst/drv_bus[475]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_183_1\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        D => \can_top_level_inst/txtb_allow_bb[4]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/N_183\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_100_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_100_Z[4]\);
    
    \timestamp_ibuf[41]/U_IOPAD\ : IOPAD_IN
      port map(PAD => timestamp(41), Y => 
        \timestamp_ibuf[41]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_y0[23]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_56_1_0_wmux_0_Y[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[20]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[7]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[18]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/gen_filt_pos.valid_0_I_69\ : 
        ARI1_CC
      generic map(INIT => x"60D07")

      port map(A => \can_top_level_inst/rec_ident[22]\, B => 
        \can_top_level_inst/drv_bus[103]\, C => 
        \can_top_level_inst/drv_bus[136]\, D => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_N_16\, 
        FCI => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[10]\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/bit_filter_A_inst/valid_0_data_tmp[11]\, 
        CC => NET_CC_CONFIG1811, P => NET_CC_CONFIG1808, Y3 => 
        NET_CC_CONFIG1809, Y3A => NET_CC_CONFIG1810);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[26]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q_RNO[15]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_17[14]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[26]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0[24]\ : 
        CFG4
      generic map(INIT => x"0002")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_data_mux_test_registers_comp_un2_index_sat_1\, 
        C => \adress_c[7]\, D => \adress_c[6]\, Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[4]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_rst_reg_inst/rx_shift_res_reg_inst/reg_q\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/un1_trv_delay_ctr_rst_d_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/reg_q\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data[24]\ : 
        CFG4
      generic map(INIT => x"A280")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[24]\, 
        B => \adress_c[7]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1085\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1079\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o2[27]\ : 
        CFG3
      generic map(INIT => x"7F")

      port map(A => \can_top_level_inst/is_bus_off\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_933\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1045\);
    
    \can_top_level_inst/rx_buffer_inst/timestamp_capture[28]\ : 
        SLE
      port map(D => \timestamp_c[28]\, CLK => 
        \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/un1_rec_valid_f_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_119_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[6]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[6]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[6]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[6]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[24]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[24]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[24]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[18]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.2.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa7\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/awe7\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_106_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_18_1_0_y0[30]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_13_31[12]\ : 
        CFG4
      generic map(INIT => x"E4A0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_622\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_606\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_623\);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_frame_test.fcrc\ : 
        SLE
      port map(D => \can_top_level_inst/tx_arbitrator_inst/fcrc\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/commit_dbl_bufs\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tran_frame_test.fcrc\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_1[14]\ : 
        CFG4
      generic map(INIT => x"8B89")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_1_1_Z[14]\, 
        D => \can_top_level_inst/drv_bus[285]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_15_30_1_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[30]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[29]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_3_Y[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[16]\ : 
        SLE
      port map(D => \data_in_c[16]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[221]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_46_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_46_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_0[23]\ : 
        CFG4
      generic map(INIT => x"ECA0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_un3_tst_ena\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2481\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[4]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[4]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[2]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm_0[6]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un10_tq_counter_d_Z\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_s[6]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/tq_counter_q_lm[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0[3]\ : 
        CFG3
      generic map(INIT => x"10")

      port map(A => \can_top_level_inst/txtb_hw_cmd.arbl\, B => 
        \can_top_level_inst/txtb_hw_cmd.err\, C => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_1_0_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_proc.err_pos_12_0_a2_9_RNIEREL[1]\ : 
        CFG3
      generic map(INIT => x"D1")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_12_9[1]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_ide_is_arbitration_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/g2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_o2_3[1]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1407_9\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_o2_3_0_Z[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_939\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1046\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1[17]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_1_Z[17]\, 
        B => \can_top_level_inst/rx_mem_free[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_18_15_1_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/reg_value_r[18]\ : 
        SLE
      port map(D => \data_in_c[18]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_LOW_present_gen_t.filter_ran_low_reg_comp/wr_select_Z[2]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[289]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_86_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_86_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_45_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_45_Z[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_17[10]\ : 
        CFG4
      generic map(INIT => x"F0E0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1523\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_0[0]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1320\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_984\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1236\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_0_2_Z[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1190\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_18[18]\ : 
        CFG4
      generic map(INIT => x"F4F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_838_2\, 
        D => \can_top_level_inst/drv_bus[256]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_838\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_18\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_RAM_data_out[23]\, B
         => \can_top_level_inst/rx_buffer_inst_RAM_data_out[22]\, 
        C => \can_top_level_inst/rx_buffer_inst_RAM_data_out[21]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_mismatch_1_18_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y3[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_0_y1[14]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_2_1_wmux_3_Y[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[23]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_62_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_6[27]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data[15]\ : 
        CFG4
      generic map(INIT => x"F808")

      port map(A => \can_top_level_inst/rx_buffer_inst/N_2207_2\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_0_2_Z[15]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_sn_N_11\, 
        D => \can_top_level_inst/rx_buffer_inst/N_2293\, Y => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_120_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_120_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[10]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[10]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un2_tq_counter_allowlto7\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => \can_top_level_inst/drv_bus[1]\, B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un2_tq_counter_allowlto7_4_Z\, 
        C => \can_top_level_inst/drv_bus[3]\, D => 
        \can_top_level_inst/drv_bus[2]\, Y => 
        \can_top_level_inst/prescaler_inst/un2_tq_counter_allowlto7\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[9]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[9]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2633\, Y
         => \can_top_level_inst/memory_registers_inst/N_2698\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_22_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_22_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116[12]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[12]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram17_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe17\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram17_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[12]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[12]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[12]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_RNO_0[24]\ : 
        CFG4
      generic map(INIT => x"530F")

      port map(A => \can_top_level_inst/stat_bus[212]\, B => 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_37_2_1_1[24]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_37_2_1_2[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_5\ : 
        CFG4
      generic map(INIT => x"4000")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_962_i\, 
        B => \can_top_level_inst/can_core_inst/pc_rx_trigger\, C
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_2_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un1_ctrl_ctr_ce_5_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1_RNI3PDS\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/reg_value_r[25]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_C_MASK_present_gen_t.filter_c_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[230]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[9]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[9]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[9]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[31]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45[8]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[8]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[8]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38[19]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[19]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[19]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1[11]\ : 
        CFG4
      generic map(INIT => x"0A0C")

      port map(A => \can_top_level_inst/can_core_inst/crc_17[7]\, 
        B => \can_top_level_inst/can_core_inst/crc_15[5]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[0]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[30]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[30]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[30]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[15]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_MASK_present_gen_t.filter_b_mask_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[150]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, CLK
         => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/stored_ts\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[1]\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count[5]\ : SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNO_S[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => \can_top_level_inst/rx_frame_count[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[5]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[5]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram40_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe40\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram40_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[18]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[18]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[18]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un10_l3_winner_3_1.CO2_1\ : 
        CFG4
      generic map(INIT => x"0A8E")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2_Z[1]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_2_Z[0]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0_Z[1]\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/l1_prio_0_Z[0]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/CO2_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram100_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram101_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_0[0]\);
    
    \I_1/U0_IOBA\ : ICB_CLKINT
      port map(A => \clk_sys_ibuf/YIN\, Y => clk_sys_ibuf_Z);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92[19]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_92_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_control_reg_comp/reg_value_r[0]\ : 
        SLE
      port map(D => \data_in_c[0]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_tst_control_reg_comp_wr_select[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/test_registers_out.tst_control[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_76_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_108_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_35_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m42_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[21]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[21]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[21]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_8\ : 
        ARI1_CC
      generic map(INIT => x"56633")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/btmc_q_Z[8]\, 
        B => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/first_ssp_q_Z\, 
        C => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[8]\, 
        D => ADLIB_GND, FCI => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_7_Z\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un2_sspc_expired_cry_8_Z\, 
        CC => NET_CC_CONFIG430, P => NET_CC_CONFIG427, Y3 => 
        NET_CC_CONFIG428, Y3A => NET_CC_CONFIG429);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[26]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/reg_value_r[7]\ : 
        SLE
      port map(D => \data_in_c[7]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ctr_pres_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[423]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/br_shifted_u\ : 
        CFG4
      generic map(INIT => x"4474")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_223\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, D
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/br_shifted_u_1_1_Z\, 
        Y => \can_top_level_inst/br_shifted\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m35_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_2\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1_1[0]\ : 
        CFG4
      generic map(INIT => x"2A7F")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        D => \can_top_level_inst/rx_buffer_inst_RAM_data_out[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram9_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe9\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram9_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[23]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[16]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[48]\, 
        C => \can_top_level_inst/rec_ident[16]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2261\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_RNO[24]\ : 
        CFG4
      generic map(INIT => x"070F")

      port map(A => \can_top_level_inst/drv_bus[511]\, B => 
        \can_top_level_inst/drv_bus[460]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_0_Z[13]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_arb_1_Z\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_796_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_d[4]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exp_seg_length_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i[6]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[6]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[17]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[27]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_RNIKHKK_S[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/trv_delay_ctr_q_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8150\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_o2[2]\ : 
        CFG4
      generic map(INIT => x"35FF")

      port map(A => 
        \can_top_level_inst/memory_registers_inst_control_registers_out_tx_command[13]\, 
        B => \can_top_level_inst/txtb_sw_cmd_index[4]\, C => 
        \can_top_level_inst/drv_bus[475]\, D => 
        \can_top_level_inst/txtb_sw_cmd.set_abt\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_fsm_inst_N_141\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\);
    
    \can_top_level_inst/rx_buffer_inst/frame_count_RNI0VMF[0]\ : 
        ARI1_CC
      generic map(INIT => x"59966")

      port map(A => \can_top_level_inst/rx_frame_count[0]\, B => 
        \can_top_level_inst/rx_buffer_inst/commit_rx_frame_Z\, C
         => 
        \can_top_level_inst/rx_buffer_inst/un10_read_increment\, 
        D => ADLIB_GND, FCI => ADLIB_GND, S => OPEN, Y => 
        \can_top_level_inst/rx_buffer_inst/frame_count_RNI0VMF_Y[0]\, 
        FCO => 
        \can_top_level_inst/rx_buffer_inst/un1_frame_count_cry_0\, 
        CC => NET_CC_CONFIG2632, P => NET_CC_CONFIG2629, Y3 => 
        NET_CC_CONFIG2630, Y3A => NET_CC_CONFIG2631);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_30_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[16]\);
    
    
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/un1_sspc_q_cry_0_S\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_GND, SLn => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/N_1_i\, 
        SD => ADLIB_VCC, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/bus_sampling_inst/ssp_generator_inst/sspc_q_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1_0[10]\ : 
        CFG4
      generic map(INIT => x"F045")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1_0_1_Z[10]\, 
        B => \can_top_level_inst/txtb_prorities_2[2]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_11_30_1_0_Z[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_13_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[25]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[25]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[26]\ : 
        SLE
      port map(D => \data_in_c[26]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[202]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_71_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[22]\);
    
    
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO[2]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/N_38\, 
        B => 
        \can_top_level_inst/bus_sampling_inst_trv_delay_measurement_inst_ssp_delay_select[0]\, 
        Y => 
        \can_top_level_inst/bus_sampling_inst/trv_delay_measurement_inst/ssp_delay_shadowed_RNO_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_102_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[28]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[28]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_6[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_iv\ : 
        CFG4
      generic map(INIT => x"ECAA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_i\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_unaliged_Z\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1317_i\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txtb_port_b_data[8]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[6]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        Y => \can_top_level_inst/txtb_port_b_data_6[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_1\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_67_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y3[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_0_y1[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_3_Y[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[9]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_10_6[9]\ : 
        CFG4
      generic map(INIT => x"F2F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1474_1\, 
        D => \can_top_level_inst/drv_bus[25]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1474\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[13]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_2[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/store_ts_wr_ptr\, CLK
         => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_Z[3]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[35]\, 
        C => \can_top_level_inst/rec_ident[3]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2248\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[13]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/test_registers_out.tst_wdata[13]\, C
         => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_78_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[7]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_31[8]\ : 
        CFG4
      generic map(INIT => x"E5C5")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_9_31_1_Z[8]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1460_2\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1452\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1461\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_wmux[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_101_Z[22]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_Z[22]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txtb_port_b_data[2]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        Y => \can_top_level_inst/txtb_port_b_data_2[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un2_txtb_gate_mem_read.ALTB[2]\ : 
        CFG3
      generic map(INIT => x"BA")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ALTB_1[2]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/txtb_num_words_gate[2]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/AXB[2]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ALTB[2]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/tst_dest_reg_comp/reg_value_r_RNI7M8I[16]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[16]\, B
         => \can_top_level_inst/test_registers_out.tst_dest[19]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/rx_buf_cmds_proc.store_metadata_2\ : 
        CFG3
      generic map(INIT => x"40")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_980\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un26_is_receiver\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[22]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/store_metadata_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram16__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa16\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe16\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[9]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[15]\ : 
        SLE
      port map(D => \data_in_c[15]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[315]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_37_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_25[13]\ : 
        CFG4
      generic map(INIT => x"F1F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_655_2\, 
        D => \can_top_level_inst/drv_bus[46]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_655\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/rec_ident_Z[23]\ : 
        SLE
      port map(D => \can_top_level_inst/store_data_word[4]\, CLK
         => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_store_base_id\, 
        ALn => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/rx_shift_reg_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/rec_ident[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o4[10]\ : 
        CFG2
      generic map(INIT => x"E")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/bit_err_arb_1_Z\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1035\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[22]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_3_sqmuxa_i_RNI941B\ : 
        CFG3
      generic map(INIT => x"45")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_886_i\, 
        B => \can_top_level_inst/sync_edge\, C => 
        \can_top_level_inst/can_core_inst/bds_data_out\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un22_rx_data_nbs_m\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m22_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[31]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_48_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_48_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[15]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_31_Z[15]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_119_1_0_y0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[4]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[2]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/NN_1\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[2]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => \can_top_level_inst/txtb_prorities_0[0]\, B
         => \can_top_level_inst/txtb_prorities_1[0]\, C => 
        \can_top_level_inst/drv_bus_fast[475]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/txtb_prorities_txbbm_1_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_wmux_1[28]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_106_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_y0_0[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3_a2_3_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_wmux_1[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[30]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[30]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_2_1_y0_0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[30]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[30]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[30]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_94_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"1000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8146\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8133\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_89_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.4.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa13\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/awe13\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m44_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_26_25[25]\ : 
        CFG4
      generic map(INIT => x"C0A0")

      port map(A => \can_top_level_inst/drv_bus[325]\, B => 
        \can_top_level_inst/txtb_prorities_6[1]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1111\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_0_a2_0_a2_1\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_3_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_67_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_97_1_0_y0[17]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI4Q6DB_S[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[203]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2[6]\ : 
        CFG4
      generic map(INIT => x"5D0A")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[18]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[6]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[6]\, 
        Y => \can_top_level_inst/memory_registers_inst/N_2630\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[32]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_780_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[32]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[3]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_RNIA6591[3]\ : 
        ARI1_CC
      generic map(INIT => x"65500")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/exp_seg_length_q_Z[3]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_2\, 
        S => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un7_exit_segm_regular_3\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un5_exit_segm_regular_cry_3\, 
        CC => NET_CC_CONFIG264, P => NET_CC_CONFIG261, Y3 => 
        NET_CC_CONFIG262, Y3A => NET_CC_CONFIG263);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_3[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[22]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45[5]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_Z[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_20_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0_0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[7]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[11]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[11]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac1_0\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/_decfrac1_0_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[24]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[17]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[17]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[17]\);
    
    \data_in_ibuf[2]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \data_in_c[2]\, E => ADLIB_GND, YIN => 
        \data_in_ibuf[2]/YIN\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_3\ : 
        CFG4
      generic map(INIT => x"AC00")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[29]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[25]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/frame_type\, 
        D => \can_top_level_inst/stat_bus[0]\, Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_curr_state_40_3_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[20]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[20]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m43_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[31]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[31]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[31]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_21\ : 
        CFG4
      generic map(INIT => x"6996")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/un12_parity_mismatch_21_Z\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[25]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[25]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_127_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[0]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux_0_Y[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_112_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_3[3]\ : 
        CFG4
      generic map(INIT => x"FFDC")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/data_length_c_Z[0]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_0_Z[3]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/next_state_16_sqmuxa\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_0_iv_2_Z[3]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload_val_0_iv_3[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_2[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_55_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[16]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_2\);
    
    \timestamp_ibuf[0]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[0]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[0]/YIN\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_83_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_66_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_107_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc[11]\ : 
        CFG3
      generic map(INIT => x"F8")

      port map(A => \can_top_level_inst/can_core_inst/crc_21[11]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/crc_src[1]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_1_Z[11]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_crc_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_113_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[21]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_32_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_32_Z[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[18]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_Z[18]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram19__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa19\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe19\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_RNO[5]\ : 
        CFG4
      generic map(INIT => x"003B")

      port map(A => \can_top_level_inst/drv_bus[472]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_0_Z[2]\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/un19_timestamp_valid\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_1_2_Z[2]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_81_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_wmux[4]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[4]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[4]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_15_1_0_y0[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm_0[3]\ : 
        CFG3
      generic map(INIT => x"02")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_s[3]\, 
        B => \can_top_level_inst/prescaler_inst/h_sync_valid\, C
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/segm_counter_q_lm[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[21]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[21]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/wr_sel_gen.0.wr_sel_no_lock_gen.wr_select_RNITN121\ : 
        CFG4
      generic map(INIT => x"7F00")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select\, 
        B => \sbe_c[0]\, C => \data_in_c[0]\, D => 
        \can_top_level_inst/prescaler_inst_bit_time_fsm_inst_drv_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/un2_res_core_d_n_i\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_0[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[27]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_2[27]\, D => 
        \can_top_level_inst/txtb_port_b_data_6[27]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[27]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m34_1_0\ : 
        CFG4
      generic map(INIT => x"2604")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/txtb_state_1[3]\, D => 
        \can_top_level_inst/rx_frame_count[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/int_stat_reg_comp/m34_1_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[27]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i[17]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_wdata[17]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/memory_write_data_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_70_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_70_Z[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_65_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.6.txt_buffer_inst.txt_buffer_ram_inst.parity_ram6_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/awe6\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/ram6_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[4]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram41_[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_3894_i\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe41\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram41_0\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[20]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_ns_i_a2_0[2]\ : 
        CFG2
      generic map(INIT => x"2")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/fsm_wait_state_q\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/curr_state_Z[5]\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/tx_arbitrator_fsm_inst/N_155\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[16]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_0[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa8\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8_2\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa8\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_98_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[9]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_78_Z[9]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_110_Z[9]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_7_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_71_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_0_y0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_796_i\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[29]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txtb_port_b_data[4]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_port_b_data_i[4]\, 
        Y => \can_top_level_inst/txtb_port_b_data_5[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[29]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a2[2]\ : 
        CFG4
      generic map(INIT => x"1333")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_m1_e_1_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        C => \can_top_level_inst/txtb_parity_check_valid\, D => 
        \can_top_level_inst/txtb_parity_error_valid_1[0]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_175\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[16]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[6]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/un6_ctrl_ctr_ce_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1[4]\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \can_top_level_inst/txtb_hw_cmd.unlock\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_133\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_161_2\, 
        D => 
        \can_top_level_inst/tx_arbitrator_inst_validated_buffer[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_2[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_y0_0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_63_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y3[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_25_10[24]\ : 
        CFG4
      generic map(INIT => x"F4F0")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1058_2\, 
        D => \can_top_level_inst/drv_bus[262]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1058\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[14]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_81_Z[14]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_113_Z[14]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txtb_port_b_data[19]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_1_txt_buffer_inst_txtb_port_b_data_i[19]\, 
        Y => \can_top_level_inst/txtb_port_b_data_1[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram7__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/un1_ram_write_Z\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa7\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe7\);
    
    
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d[1]\ : 
        CFG4
      generic map(INIT => x"4C80")

      port map(A => \can_top_level_inst/stat_bus[303]\, B => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/reg_q_0\, 
        C => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/un4_dst_ctr_d_Z\, 
        D => \can_top_level_inst/stat_bus[304]\, Y => 
        \can_top_level_inst/can_core_inst/bit_destuffing_inst/dst_ctr_d_Z[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[5]\);
    
    \can_top_level_inst/rst_sync_inst/rst\ : SLE
      port map(D => \can_top_level_inst/rst_sync_inst/rff_Z\, CLK
         => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => res_n_c, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => res_n_out_c);
    
    
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_RNIEI8D3[5]\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \timestamp_c[5]\, B => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[5]\, 
        C => ADLIB_GND, D => ADLIB_GND, FCI => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_4\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/tx_arbitrator_inst/un13_timestamp_valid_cry_5\, 
        CC => NET_CC_CONFIG1297, P => NET_CC_CONFIG1294, Y3 => 
        NET_CC_CONFIG1295, Y3A => NET_CC_CONFIG1296);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3[28]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y3[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_0_y1[28]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        D => ADLIB_VCC, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_7_2_1_wmux_3_Y[28]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/reg_value_r[25]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/tx_priority_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txtb_prorities_6[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_1_Z[0]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_65_Z[0]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_y0[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_3[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i[3]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[3]\, 
        C => \can_top_level_inst/txtb_port_b_address[3]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.wa14\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa6_1\, 
        B => \adress_c[6]\, C => \adress_c[5]\, D => 
        \adress_c[2]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_ram_inst_parity_0_wa14\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/reg_value_r[5]\ : 
        SLE
      port map(D => \data_in_c[5]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_59_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_59_Z[30]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data[3]\ : 
        CFG4
      generic map(INIT => x"00A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_0_Z[0]\, 
        B => \adress_c[3]\, C => 
        \can_top_level_inst/test_registers_out.tst_dest[3]\, D
         => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_1_Z[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/test_registers_gen_true.test_registers_reg_map_comp/data_mux_test_registers_comp/masked_data_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[1]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[0]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[1]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[1]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_1[9]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \can_top_level_inst/store_data_word[9]\, B
         => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[9]\, 
        C => \can_top_level_inst/rx_buffer_inst/inc_ts_wr_ptr\, Y
         => \can_top_level_inst/rx_buffer_inst/N_4816\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[24]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[24]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[13]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_20[2]\ : 
        CFG3
      generic map(INIT => x"E2")

      port map(A => \can_top_level_inst/drv_bus[402]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/drv_bus[83]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1222\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_10_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_10_Z[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_1_wmux_1[0]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram68_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram69_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_1_y0_0[0]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_RNIEQGV[6]\ : 
        CFG3
      generic map(INIT => x"80")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[6]\, 
        B => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[5]\, 
        C => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_Z[4]\, 
        Y => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/un6_m1_0_a2_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_123_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_123_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_37_1_0[13]\ : 
        CFG4
      generic map(INIT => x"B9A8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => \can_top_level_inst/stat_bus[377]\, D => 
        \can_top_level_inst/stat_bus[233]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_14_37_1_0_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_load_stuff_count_i\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_971\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_955\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_922\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/N_877\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_72_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_0_y0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[0]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[0]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[0]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_52_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_52_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[14]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[14]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/tx_frame_no_sof_d_i_o4\ : 
        CFG4
      generic map(INIT => x"ABAF")

      port map(A => \can_top_level_inst/is_bus_off\, B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_zero\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[12]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_929\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_control[1]\, B
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/un1_ram_write_Z\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_write_i_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[28]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[28]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77[28]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux_0_Y[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[28]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_1_0[4]\ : 
        CFG4
      generic map(INIT => x"FFFE")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[10]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[11]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[35]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_1_0_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[0]\);
    
    
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_10_N_3L3\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => \can_top_level_inst/prescaler_inst/N_7\, B
         => \can_top_level_inst/prescaler_inst/N_8\, Y => 
        \can_top_level_inst/prescaler_inst/segment_end_detector_inst/g0_10_N_3L3_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[23]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[23]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[1]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[1]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_50_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram0_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram0_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_88_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_88_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_77_Z[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux_0_Y[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0[24]\ : 
        CFG3
      generic map(INIT => x"70")

      port map(A => \adress_c[7]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un2_index_satlt5\, 
        C => \sbe_c[3]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_0_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[7]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[7]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_0[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \adress_c[10]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_2_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txtb_allow_bb\, 
        D => \adress_c[9]\, Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/un1_ram_write_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[17]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[17]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[17]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_6[17]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2[24]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/tran_identifier[10]\, B
         => \can_top_level_inst/tran_identifier[21]\, C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_ss3_Z\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_sm3\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_sr_pload_val_2_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[27]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[27]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_1[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_3[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_37_2_0[11]\ : 
        CFG4
      generic map(INIT => x"FD0D")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => \timestamp_c[11]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        D => \timestamp_c[43]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_37_2_0_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[13]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_16\ : 
        CFG3
      generic map(INIT => x"96")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/parity_read_exp_Z\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txtb_port_b_data_i[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/un18_parity_mismatch_16_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[18]\);
    
    \can_top_level_inst/rx_buffer_inst/rwcnt_com_4_0_.m4\ : CFG4
      generic map(INIT => x"0B1A")

      port map(A => \can_top_level_inst/rec_dlc[0]\, B => 
        \can_top_level_inst/rec_dlc[3]\, C => 
        \can_top_level_inst/rec_dlc[2]\, D => 
        \can_top_level_inst/rec_dlc[1]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_5\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_28\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[299]\, B => 
        \can_top_level_inst/rec_ident[28]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_27\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_28_FCNET1\, 
        CC => NET_CC_CONFIG1945, P => NET_CC_CONFIG1942, Y3 => 
        NET_CC_CONFIG1943, Y3A => NET_CC_CONFIG1944);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[11]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[11]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_79_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[14]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[14]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_93_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_93_Z[5]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_17_34[16]\ : 
        CFG4
      generic map(INIT => x"FF8C")

      port map(A => \timestamp_c[16]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_778_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_778\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/segm_counter_q_lm[3]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_dbt_inst/un6_segm_counter_ce_0\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/prescaler_inst/segm_counter_dbt[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.3.txt_buffer_inst.txt_buffer_ram_inst.parity_ram13_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/awe13\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/ram13_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_y0_0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_116_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_29_2_1_0_y3[31]\);
    
    \can_top_level_inst/rx_buffer_inst/RAM_read_address[1]\ : 
        CFG3
      generic map(INIT => x"E4")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => \can_top_level_inst/rx_read_pointer[1]\, C => 
        \can_top_level_inst/rx_buffer_inst/read_pointer_inc_1[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_data_access_signaller_comp/access_active[0]\ : 
        CFG4
      generic map(INIT => x"0080")

      port map(A => srd_c, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/access_active_1[0]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/rx_data_access_signaller_comp/access_active_0_Z[0]\, 
        D => \adress_c[7]\, Y => 
        \can_top_level_inst/memory_registers_inst/rx_data_read\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[28]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[28]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[22]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[11]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[11]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[11]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_50_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_34_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[21]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_8_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_8_Z[26]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe4_0_a4\ : 
        CFG4
      generic map(INIT => x"0100")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_353\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_57_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_4[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_4[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_99_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_99_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_12[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[16]\);
    
    
        \can_top_level_inst/can_core_inst/operation_control_inst/curr_state_ns_1_0_.m3_e\ : 
        CFG3
      generic map(INIT => x"01")

      port map(A => 
        \can_top_level_inst/can_core_inst/operation_control_inst_set_receiver\, 
        B => 
        \can_top_level_inst/can_core_inst/operation_control_inst/un2_go_to_off_i_0\, 
        C => \can_top_level_inst/stat_bus[6]\, Y => 
        \can_top_level_inst/can_core_inst/operation_control_inst/N_20\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_28\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/rec_ident[28]\, B => 
        \can_top_level_inst/drv_bus[328]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un2_valid_cry_27\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un2_valid_cry_28_FCNET1\, 
        CC => NET_CC_CONFIG641, P => NET_CC_CONFIG638, Y3 => 
        NET_CC_CONFIG639, Y3A => NET_CC_CONFIG640);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/reg_value_r[27]\ : 
        SLE
      port map(D => \data_in_c[27]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_A_MASK_present_gen_t.filter_a_mask_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[108]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[6]\ : 
        CFG4
      generic map(INIT => x"0010")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_N_140\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[6]\, 
        D => \can_top_level_inst/txtb_parity_error_valid[1]\, Y
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_fsm_inst/N_175\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txtb_port_b_data[20]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => \can_top_level_inst/txtb_allow_bb[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[20]\, 
        Y => \can_top_level_inst/txtb_port_b_data_0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_5[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_5[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[12]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_2[16]\ : 
        CFG4
      generic map(INIT => x"C840")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_44_1_0_wmux_0_Y[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1457_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_17_Z[1]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_49_Z[1]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_y0[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_wmux[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_49_1_0_wmux_0_Y[13]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_52_1_0_wmux_0_Y[13]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_61_1_0_y0[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46[16]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_1457_2\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_38_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_46_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[25]\ : 
        SLE
      port map(D => \data_in_c[25]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_0[3]\ : 
        CFG2
      generic map(INIT => x"1")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/N_137\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_fsm_inst_N_131\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0_Z[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txtb_port_b_data_i[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[24]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[24]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[24]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_Z[24]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.ramout_126_1_0_wmux_16[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y0_6[0]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[1]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram122_0\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ram123_0\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/ramout_126_1_0_y5_0[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/un31_status_comb_RNO_16_CC_1\ : 
        CC_CONFIG
      port map(CI => CI_TO_CO1164, CO => OPEN, P(0) => 
        NET_CC_CONFIG1173, P(1) => NET_CC_CONFIG1177, P(2) => 
        NET_CC_CONFIG1181, P(3) => NET_CC_CONFIG1185, P(4) => 
        NET_CC_CONFIG1189, P(5) => NET_CC_CONFIG1193, P(6) => 
        NET_CC_CONFIG1197, P(7) => NET_CC_CONFIG1201, P(8) => 
        ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11)
         => ADLIB_VCC, Y3(0) => NET_CC_CONFIG1174, Y3(1) => 
        NET_CC_CONFIG1178, Y3(2) => NET_CC_CONFIG1182, Y3(3) => 
        NET_CC_CONFIG1186, Y3(4) => NET_CC_CONFIG1190, Y3(5) => 
        NET_CC_CONFIG1194, Y3(6) => NET_CC_CONFIG1198, Y3(7) => 
        NET_CC_CONFIG1202, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG1175, Y3A(1) => NET_CC_CONFIG1179, Y3A(2)
         => NET_CC_CONFIG1183, Y3A(3) => NET_CC_CONFIG1187, 
        Y3A(4) => NET_CC_CONFIG1191, Y3A(5) => NET_CC_CONFIG1195, 
        Y3A(6) => NET_CC_CONFIG1199, Y3A(7) => NET_CC_CONFIG1203, 
        Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => 
        ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG1176, CC(1) => NET_CC_CONFIG1180, CC(2) => 
        NET_CC_CONFIG1184, CC(3) => NET_CC_CONFIG1188, CC(4) => 
        NET_CC_CONFIG1192, CC(5) => NET_CC_CONFIG1196, CC(6) => 
        NET_CC_CONFIG1200, CC(7) => NET_CC_CONFIG1204, CC(8) => 
        nc111, CC(9) => nc55, CC(10) => nc10, CC(11) => nc22);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_19_Z[20]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[20]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.1.txt_buffer_inst.txt_buffer_ram_inst.parity_ram1_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/awe1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/ram1_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.7.txt_buffer_inst.txt_buffer_ram_inst.parity_ram9_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => NN_1, 
        EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/awe9\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/ram9_0\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[15]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[15]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[15]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[12]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => \can_top_level_inst/txtb_index_muxed[2]\, B
         => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_0[23]\, D => 
        \can_top_level_inst/txtb_port_b_data_4[23]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y0[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"0200")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[14]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_5[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/data_out[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/masked_data_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/read_mux_ena_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_61_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[0]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[0]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_4[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_wmux_3_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_0_wmux_0_Y[31]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[28]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[28]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[28]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_64_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_64_Z[11]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85[31]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_85_Z[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_2_1[31]\ : 
        CFG4
      generic map(INIT => x"535F")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[31]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_2_1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i[3]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \adress_c[5]\, B => 
        \can_top_level_inst/test_registers_out.tst_dest[3]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_6_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.awe40_0_a4\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[3]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/RAM_write_address_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_344\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/un1_crc_check_0_sqmuxa_i_a4\ : 
        CFG3
      generic map(INIT => x"15")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[14]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[18]\, 
        C => \can_top_level_inst/can_core_inst/pc_rx_trigger\, Y
         => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_1085_i\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[13]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_1[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_1[13]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1366\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/reg_value_r[30]\ : 
        SLE
      port map(D => \data_in_c[30]\, CLK => 
        \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/btr_fd_reg_comp/wr_select_Z[3]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[59]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_117_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[6]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_12_13_2[11]\ : 
        CFG4
      generic map(INIT => x"CA00")

      port map(A => \can_top_level_inst/drv_bus[411]\, B => 
        \can_top_level_inst/rx_read_buff[11]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1557_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0[18]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[18]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_89_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_121_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux_0_Y[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[3]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[3]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_75_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_75_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_77_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_126_1_1_y0[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[10]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_q_RNO[9]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/can_core_inst/load_init_vect\, B => 
        \can_top_level_inst/can_core_inst/crc_21[8]\, Y => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_21_rx_inst/crc_d[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[12]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[12]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[12]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1[6]\ : 
        CFG4
      generic map(INIT => x"D9C8")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1371\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1370\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_7_30_1_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[0]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[0]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[0]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/mode_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[1]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[470]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_62_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_126_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[23]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\);
    
    
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/int_ena_proc.un1_int_ena_clear\ : 
        CFG4
      generic map(INIT => x"A800")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_set_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        C => 
        \can_top_level_inst/memory_registers_inst_control_registers_reg_map_comp_int_ena_clr_reg_comp_wr_sel_gen_0_wr_sel_no_lock_gen_wr_select\, 
        D => \data_in_c[11]\, Y => 
        \can_top_level_inst/int_manager_inst/int_module_gen.11.int_module_inst/un1_int_ena_clear\);
    
    \timestamp_ibuf[36]/U_IOIN\ : IOIN_IB_E
      generic map(TX_MODE => "000" & x"0", RX_MODE => x"0",
         TX_OE_MODE => "000", INPUT_DELAY_SEL => "00",
         DELAY_LINE_MODE => "00", RX_DELAY_VAL => "000" & x"0",
         RX_DELAY_VAL_X2 => "0", TX_DELAY_VAL => "000" & x"0")

      port map(Y => \timestamp_c[36]\, E => ADLIB_GND, YIN => 
        \timestamp_ibuf[36]/YIN\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_6[7]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[7]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[7]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[7]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un1_rx_mem_free_raw_inc_1_s_1_2643_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => OPEN, P(0) => 
        NET_CC_CONFIG2216, P(1) => NET_CC_CONFIG2220, P(2) => 
        NET_CC_CONFIG2224, P(3) => NET_CC_CONFIG2228, P(4) => 
        NET_CC_CONFIG2232, P(5) => NET_CC_CONFIG2236, P(6) => 
        NET_CC_CONFIG2240, P(7) => NET_CC_CONFIG2244, P(8) => 
        ADLIB_VCC, P(9) => ADLIB_VCC, P(10) => ADLIB_VCC, P(11)
         => ADLIB_VCC, Y3(0) => NET_CC_CONFIG2217, Y3(1) => 
        NET_CC_CONFIG2221, Y3(2) => NET_CC_CONFIG2225, Y3(3) => 
        NET_CC_CONFIG2229, Y3(4) => NET_CC_CONFIG2233, Y3(5) => 
        NET_CC_CONFIG2237, Y3(6) => NET_CC_CONFIG2241, Y3(7) => 
        NET_CC_CONFIG2245, Y3(8) => ADLIB_VCC, Y3(9) => ADLIB_VCC, 
        Y3(10) => ADLIB_VCC, Y3(11) => ADLIB_VCC, Y3A(0) => 
        NET_CC_CONFIG2218, Y3A(1) => NET_CC_CONFIG2222, Y3A(2)
         => NET_CC_CONFIG2226, Y3A(3) => NET_CC_CONFIG2230, 
        Y3A(4) => NET_CC_CONFIG2234, Y3A(5) => NET_CC_CONFIG2238, 
        Y3A(6) => NET_CC_CONFIG2242, Y3A(7) => NET_CC_CONFIG2246, 
        Y3A(8) => ADLIB_VCC, Y3A(9) => ADLIB_VCC, Y3A(10) => 
        ADLIB_VCC, Y3A(11) => ADLIB_VCC, CC(0) => 
        NET_CC_CONFIG2219, CC(1) => NET_CC_CONFIG2223, CC(2) => 
        NET_CC_CONFIG2227, CC(3) => NET_CC_CONFIG2231, CC(4) => 
        NET_CC_CONFIG2235, CC(5) => NET_CC_CONFIG2239, CC(6) => 
        NET_CC_CONFIG2243, CC(7) => NET_CC_CONFIG2247, CC(8) => 
        nc210, CC(9) => nc185, CC(10) => nc143, CC(11) => nc248);
    
    \data_out_obuf[4]/U_IOPAD\ : IOPAD_TRI
      port map(PAD => data_out(4), D => \data_out_obuf[4]/DOUT\, 
        E => \data_out_obuf[4]/EOUT\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[1]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[1]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[1]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_wmux_3[19]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y3[19]\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_2_1_0_y1[19]\, 
        C => \can_top_level_inst/txtb_index_muxed[0]\, D => 
        ADLIB_VCC, Y => \can_top_level_inst/tran_word[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_30_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_94_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_59_1_0_y0[23]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/reg_value_r[9]\ : 
        SLE
      port map(D => \data_in_c[9]\, CLK => 
        \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_RAN_HIGH_present_gen_t.filter_ran_high_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[309]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[14]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[14]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[14]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[14]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[3]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[3]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[3]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[10]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_37_Z[10]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_101_Z[10]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux_0_Y[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[4]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[4]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[16]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[16]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[16]\);
    
    \can_top_level_inst/memory_registers_inst/data_out_cZ[8]\ : 
        CFG4
      generic map(INIT => x"B888")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/test_registers_rdata[8]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/test_registers_cs_reg_Z\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_rdata[8]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_cs_reg_Z\, 
        Y => \data_out_c[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_6[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_6_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_6_Z[1]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_5_1[24]\ : 
        CFG4
      generic map(INIT => x"88BA")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/tst_dest[17]\, 
        B => \can_top_level_inst/test_registers_out.tst_dest[18]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        D => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1_1[24]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_5_1[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_47_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_47_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[19]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[19]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103[14]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[14]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_103_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[14]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_2_34_2[1]\ : 
        CFG4
      generic map(INIT => x"D800")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        B => \timestamp_c[33]\, C => 
        \can_top_level_inst/stat_bus[221]\, D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1198_2\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_0[7]\ : 
        CFG2
      generic map(INIT => x"4")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txt_buffer_fsm_inst_un5_go_to_failed\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[23]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m45_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[1]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m42_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm_0[8]\ : 
        CFG4
      generic map(INIT => x"B830")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/next_state_16_sqmuxa\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/ctrl_ctr_pload\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_s[8]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/un3_data_length_sub_c_axbxc5\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/control_counter_inst/ctrl_ctr_q_lm[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23[11]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[11]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_23_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_23_Z[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_39_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[10]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[13]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[13]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[13]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[13]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_51_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_51_Z[16]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_22_34[21]\ : 
        CFG4
      generic map(INIT => x"FF8C")

      port map(A => \timestamp_c[21]\, B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[2]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[0]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_968_1\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_968\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_0[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_1_0[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/reg_value_r[14]\ : 
        SLE
      port map(D => \data_in_c[14]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/FILTER_B_VAL_present_gen_t.filter_b_val_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[190]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_82_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_21_1_0_y0[18]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_1[22]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[198]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_30_1_1_Z[22]\);
    
    
        \can_top_level_inst/memory_registers_inst/test_registers_in.tst_rdata_7[6]\ : 
        CFG4
      generic map(INIT => x"EC20")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txtb_port_b_data_i[6]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/tst_rdata_sn_N_6\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_2_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        D => \can_top_level_inst/memory_registers_inst/N_2630\, Y
         => \can_top_level_inst/memory_registers_inst/N_2695\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[28]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[28]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[28]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0[17]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_y0[17]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_98_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_114_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_37_1_0_wmux_0_Y[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[27]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[27]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[27]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[27]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_4[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/un1_enable\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/tx_shift_reg_inst/tx_shift_reg_inst/shift_regs_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_44_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_44_Z[5]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/gen_filt_pos.valid_0_I_82\ : 
        CFG3
      generic map(INIT => x"28")

      port map(A => \can_top_level_inst/drv_bus[222]\, B => 
        \can_top_level_inst/rec_ident[17]\, C => 
        \can_top_level_inst/drv_bus[255]\, Y => 
        \can_top_level_inst/frame_filters_inst/bit_filter_C_inst/valid_0_N_6\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[14]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[14]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[14]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[14]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[14]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_105_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_105_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_118_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_118_Z[7]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[28]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[28]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_6[28]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[28]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[28]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.0.txt_buffer_inst.txt_buffer_ram_inst.parity_ram3_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/awe3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/ram3_0\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_8[1]\ : 
        CFG4A
      generic map(INIT => x"aad8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[2]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[2]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[22]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_15_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_79_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_123_2_1_0_y0[25]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_0_1[20]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[320]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_21_30_1_0_1_Z[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_56_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_56_Z[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[22]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[22]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_wmux_1[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_96_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_14_2_1_y0_0[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_122_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_122_Z[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i[23]\ : 
        CFG3
      generic map(INIT => x"CA")

      port map(A => \data_in_c[23]\, B => 
        \can_top_level_inst/test_registers_out.tst_wdata[23]\, C
         => 
        \can_top_level_inst/txt_buf_comp_gen_5_txt_buffer_inst_txt_buffer_ram_inst_un4_tst_ena\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[1]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_wmux_1[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[2]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[2]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_116_2_1_y0_0[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[30]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/rx_ctr_i[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNI2V31C_S[16]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_rx_ctr_q_Z\, 
        ALn => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[204]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[1]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[1]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[26]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[26]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[16]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[16]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[16]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2_a2_1_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[3]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1368\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa_2\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[7]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[39]\, 
        C => \can_top_level_inst/rec_ident[7]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2252\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_9_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_73_1_0_y0[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i[3]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/test_registers_out.tst_dest[3]\, B
         => 
        \can_top_level_inst/rx_buffer_inst_rx_buffer_ram_inst_un4_tst_ena\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/RAM_read_address_Z[3]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d[8]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/reg_value_r[1]\ : 
        SLE
      port map(D => \data_in_c[17]\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_Z[0]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_GND, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[376]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[15]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_2[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_2[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_0_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1369\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[21]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[21]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[21]\);
    
    
        \can_top_level_inst/frame_filters_inst/bit_filter_B_inst/gen_filt_pos.valid_0_I_1_CC_0\ : 
        CC_CONFIG
      port map(CI => ADLIB_VCC, CO => CI_TO_CO686, P(0) => 
        ADLIB_VCC, P(1) => ADLIB_VCC, P(2) => ADLIB_VCC, P(3) => 
        ADLIB_VCC, P(4) => ADLIB_GND, P(5) => NET_CC_CONFIG687, 
        P(6) => NET_CC_CONFIG691, P(7) => NET_CC_CONFIG695, P(8)
         => NET_CC_CONFIG699, P(9) => NET_CC_CONFIG703, P(10) => 
        NET_CC_CONFIG707, P(11) => NET_CC_CONFIG711, Y3(0) => 
        ADLIB_VCC, Y3(1) => ADLIB_VCC, Y3(2) => ADLIB_VCC, Y3(3)
         => ADLIB_VCC, Y3(4) => ADLIB_VCC, Y3(5) => 
        NET_CC_CONFIG688, Y3(6) => NET_CC_CONFIG692, Y3(7) => 
        NET_CC_CONFIG696, Y3(8) => NET_CC_CONFIG700, Y3(9) => 
        NET_CC_CONFIG704, Y3(10) => NET_CC_CONFIG708, Y3(11) => 
        NET_CC_CONFIG712, Y3A(0) => ADLIB_VCC, Y3A(1) => 
        ADLIB_VCC, Y3A(2) => ADLIB_VCC, Y3A(3) => ADLIB_VCC, 
        Y3A(4) => ADLIB_VCC, Y3A(5) => NET_CC_CONFIG689, Y3A(6)
         => NET_CC_CONFIG693, Y3A(7) => NET_CC_CONFIG697, Y3A(8)
         => NET_CC_CONFIG701, Y3A(9) => NET_CC_CONFIG705, Y3A(10)
         => NET_CC_CONFIG709, Y3A(11) => NET_CC_CONFIG713, CC(0)
         => nc77, CC(1) => nc6, CC(2) => nc109, CC(3) => nc87, 
        CC(4) => nc123, CC(5) => NET_CC_CONFIG690, CC(6) => 
        NET_CC_CONFIG694, CC(7) => NET_CC_CONFIG698, CC(8) => 
        NET_CC_CONFIG702, CC(9) => NET_CC_CONFIG706, CC(10) => 
        NET_CC_CONFIG710, CC(11) => NET_CC_CONFIG714);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[31]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[31]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_5[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m44_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4[9]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[9]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[9]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/parity_0.txt_buf_comp_gen.5.txt_buffer_inst.txt_buffer_ram_inst.parity_ram14_[0]\ : 
        SLE
      port map(D => \can_top_level_inst/N_3896_i\, CLK => 
        \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/awe14\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/ram14_0\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_104_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_104_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[15]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[15]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_20[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_20[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txtb_port_b_data[8]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txtb_port_b_data_i[8]\, 
        Y => \can_top_level_inst/txtb_port_b_data_7[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_58_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_41_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIMNFH9[12]\ : 
        ARI1_CC
      generic map(INIT => x"4E400")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/inc_tx_ctr_q_Z\, 
        C => \can_top_level_inst/stat_bus[200]\, D => 
        \can_top_level_inst/stat_bus[232]\, FCI => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_11\, 
        S => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/tx_ctr_i_RNIMNFH9_S[12]\, 
        Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/bus_traffic_ctrs_gen.bus_traffic_counters_inst/un1_inc_value_cry_12\, 
        CC => NET_CC_CONFIG804, P => NET_CC_CONFIG801, Y3 => 
        NET_CC_CONFIG802, Y3A => NET_CC_CONFIG803);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/reg_value_r[11]\ : 
        SLE
      port map(D => \data_in_c[11]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/filter_control_reg_comp/wr_select_Z[1]\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst_soft_rst_rst_reg_inst_rx_shift_res_reg_inst_reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/drv_bus[237]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_1[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram120__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_339\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_352\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe120\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/write_pointer_raw_i_s[3]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/un8_read_increment_i\, 
        ALn => \can_top_level_inst/rx_buffer_inst/reg_q\, ADn => 
        ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => 
        ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/write_pointer_raw[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[19]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[30]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_68_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_68_Z[18]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_115_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_wmux_1[5]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_43_Z[5]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_107_Z[5]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_108_2_1_y0_0[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_73_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_73_Z[9]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[20]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[20]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_2[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[20]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_4[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_40_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_40_Z[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_0[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[31]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_19[22]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[22]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_wmux[23]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_13_Z[23]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_29_Z[23]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_88_1_0_y0[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_87_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_87_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[2]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[2]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[8]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[8]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[8]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[8]\);
    
    
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_q[3]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/crc_d_Z[3]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/can_core_inst/can_crc_inst/crc_calc_17_rx_inst/g0_Z\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/crc_17[3]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_2[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_5[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_2[16]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2_a2_0_a2\ : 
        CFG4
      generic map(INIT => x"0800")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_2\);
    
    AFLSDF_INV_8 : INV_BA
      port map(A => \AFLSDF_INV_8\, Y => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_14[23]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[23]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[23]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_wmux[25]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[3]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_66_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_45_2_1_0_y0[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_wmux[7]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_41_Z[7]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_57_Z[7]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_76_1_0_y0[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[29]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[29]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[29]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_6[20]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[20]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[21]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[21]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[21]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[24]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_812_i\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[16]\);
    
    
        \can_top_level_inst/frame_filters_inst/range_filter_inst/gen_filt_pos.un7_valid_cry_18\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[289]\, B => 
        \can_top_level_inst/rec_ident[18]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_17\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/frame_filters_inst/range_filter_inst/un7_valid_cry_18\, 
        CC => NET_CC_CONFIG1905, P => NET_CC_CONFIG1902, Y3 => 
        NET_CC_CONFIG1903, Y3A => NET_CC_CONFIG1904);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_3\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[12]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[12]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[12]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_16[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[31]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[31]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[31]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_38_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_38_Z[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[2]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[2]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_Z[2]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_1[20]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[20]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_3[20]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[20]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0[26]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_y0[26]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_92_Z[26]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_124_Z[26]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_28_1_0_wmux_0_Y[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m43_0_a2\ : 
        CFG4
      generic map(INIT => x"2000")

      port map(A => \sbe_c[2]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1372\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_18_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_18_Z[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_42_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_42_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_90_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_90_Z[13]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_21_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[8]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112[19]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[19]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_112_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_112_Z[19]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_1_1[22]\ : 
        CFG3
      generic map(INIT => x"57")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => \can_top_level_inst/drv_bus[136]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_15_1_1_Z[22]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/fc_fsm_next_state_proc.un12_tx_err_ctr_mt_erp_RNO_4\ : 
        ARI1_CC
      generic map(INIT => x"5AA55")

      port map(A => \can_top_level_inst/drv_bus[414]\, B => 
        \can_top_level_inst/stat_bus[77]\, C => ADLIB_GND, D => 
        ADLIB_GND, FCI => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_5\, 
        S => OPEN, Y => OPEN, FCO => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/un2_tx_err_ctr_mt_erp_cry_6\, 
        CC => NET_CC_CONFIG2086, P => NET_CC_CONFIG2083, Y3 => 
        NET_CC_CONFIG2084, Y3A => NET_CC_CONFIG2085);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_188\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_187\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8148\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/N_8134\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_102_1_sqmuxa\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[8]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[10]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[10]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[10]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[10]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_13[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_13[5]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[11]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[11]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_Z[11]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[11]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[11]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[29]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[29]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\);
    
    
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/curr_state[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/N_27_i\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/can_core_inst/fault_confinement_inst/fault_confinement_fsm_inst/reg_q\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => \can_top_level_inst/stat_bus[108]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_7[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[27]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_RNIU99F7\ : 
        CFG3
      generic map(INIT => x"53")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_26_i\, 
        B => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_36_i\, 
        C => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_i\, 
        Y => 
        \can_top_level_inst/tx_arbitrator_inst/priority_decoder_inst/un1_l1_prio_39_RNIU99F7_Z\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[10]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[10]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84[18]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[18]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_84_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_84_Z[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_10[9]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[9]\, 
        CLK => NN_1, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10_1_sqmuxa_2\, 
        ALn => \can_top_level_inst/reg_q_arst\, ADn => ADLIB_VCC, 
        SLn => ADLIB_VCC, SD => ADLIB_GND, LAT => ADLIB_GND, Q
         => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[9]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_28_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_16_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_16_Z[4]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_34_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_34_Z[0]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux[18]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_39_Z[18]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_55_Z[18]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[18]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_5[5]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[5]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[5]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0[29]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_y0[29]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_103_Z[29]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_119_Z[29]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_115_1_0_wmux_0_Y[29]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux[17]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_21_Z[17]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_53_Z[17]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[17]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_15[4]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[4]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_15[4]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[16]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[16]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[16]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[7]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0[7]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[7]\, 
        CLK => \I_1/U0_RGB1_RGB1_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_0_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB1_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_0_Z[7]\);
    
    \can_top_level_inst/rx_buffer_inst/memory_write_data_2[22]\ : 
        CFG3
      generic map(INIT => x"D8")

      port map(A => \can_top_level_inst/rx_buffer_inst/stored_ts\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/timestamp_capture_Z[54]\, 
        C => \can_top_level_inst/rec_ident[22]\, Y => 
        \can_top_level_inst/rx_buffer_inst/N_2267\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1[1]\ : 
        CFG3
      generic map(INIT => x"EC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_a3_0_1_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_0_Z[1]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/N_182\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_ns_1_Z[1]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns[0]\ : 
        CFG4
      generic map(INIT => x"FCEC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/N_54\, 
        B => \can_top_level_inst/rx_buffer_inst/stored_ts\, C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_a3_2_Z[0]\, 
        D => \can_top_level_inst/rec_abort_f\, Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_fsm_inst/curr_state_ns_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_3[1]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[1]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[1]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_10[1]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/data_out[24]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data[24]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => ADLIB_VCC, ALn
         => ADLIB_VCC, ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => 
        ADLIB_GND, LAT => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_0_txt_buffer_inst_txtb_port_b_data_i[24]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/parity_0.rx_buffer_inst.rx_buffer_ram_inst.parity_ram73__RNO[0]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_337\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/N_348\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/awe73\);
    
    \can_top_level_inst/tx_arbitrator_inst/ts_low_internal[8]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[8]\, CLK => 
        \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/store_ts_l_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/ts_low_internal_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_3[0]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[0]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txtb_port_b_data[22]\ : 
        CFG2
      generic map(INIT => x"8")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_allow_bb\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen_3_txt_buffer_inst_txtb_port_b_data_i[22]\, 
        Y => \can_top_level_inst/txtb_port_b_data_3[22]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/curr_state_Z[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_fsm_inst/N_126_i\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => ADLIB_VCC, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen_7_txt_buffer_inst_txt_buffer_fsm_inst_curr_state[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70[6]\ : 
        CFG3
      generic map(INIT => x"B8")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_69_1_0_wmux_0_Y[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_66_1_0_wmux_0_Y[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_70_Z[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/mem_free_proc.rx_mem_free_i_i_4[6]\ : 
        CFG3
      generic map(INIT => x"DC")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/un1_read_increment_i\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4_2[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_raw_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_pointers_inst/rx_mem_free_i_i_4[6]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0[12]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_y0[12]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_85_Z[12]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_117_Z[12]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_84_1_0_wmux_0_Y[12]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_1_1_wmux[2]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[1]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1210\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1211\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_3_15_1_1_y0[2]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_ns_i_o4[3]\ : 
        CFG2
      generic map(INIT => x"D")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_973\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst_err_frm_req\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/N_960\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_7[29]\ : 
        CFG4A
      generic map(INIT => x"acf0")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y7[29]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[29]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[0]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[3]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_3[29]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_19_18_2[18]\ : 
        CFG4
      generic map(INIT => x"2A08")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        B => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen_4_txt_buffer_inst_txt_buffer_fsm_inst_N_181\, 
        D => \can_top_level_inst/stat_bus[73]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_838_2\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0[19]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_y0[19]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_61_Z[19]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_125_Z[19]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_91_1_0_wmux_0_Y[19]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97[13]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[13]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_97_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_97_Z[13]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[22]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[22]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[22]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[22]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[22]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_127_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_127_Z[17]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_s_0_2636\ : 
        ARI1_CC
      generic map(INIT => x"4AA00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/is_tseg2_rep1\, C => 
        ADLIB_GND, D => ADLIB_GND, FCI => ADLIB_VCC, S => OPEN, Y
         => OPEN, FCO => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/phase_err_s_0_2636_FCO\, 
        CC => NET_CC_CONFIG2656, P => NET_CC_CONFIG2653, Y3 => 
        NET_CC_CONFIG2654, Y3A => NET_CC_CONFIG2655);
    
    AFLSDF_INV_16 : INV_BA
      port map(A => \AFLSDF_INV_16\, Y => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[27]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[27]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[27]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[27]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[27]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[16]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[16]\, 
        CLK => \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[16]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[4]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_28_Z[25]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_60_Z[25]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_25_1_0_wmux_0_Y[25]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm_0[1]\ : 
        CFG4
      generic map(INIT => x"0008")

      port map(A => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_s[1]\, 
        B => 
        \can_top_level_inst/prescaler_inst/un6_tq_counter_d_NE\, 
        C => \can_top_level_inst/prescaler_inst/h_sync_valid\, D
         => \can_top_level_inst/prescaler_inst/segment_end\, Y
         => 
        \can_top_level_inst/prescaler_inst/bit_time_counters_nbt_inst/tq_counter_q_lm[1]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/m18_0_a2\ : 
        CFG4
      generic map(INIT => x"8000")

      port map(A => \sbe_c[0]\, B => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[3]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/port_a_address_i_Z[2]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/N_1371\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.2.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14_1_sqmuxa_1\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[15]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[15]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_1[15]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[15]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[15]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_0[25]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0[25]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_4_4[25]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_6[25]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y1[25]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2[8]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_18[8]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_2[8]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_2_1[8]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_1_2_0[0]\ : 
        CFG4
      generic map(INIT => x"0020")

      port map(A => 
        \can_top_level_inst/memory_registers_inst/un3_control_registers_cs_Z\, 
        B => 
        \can_top_level_inst/memory_registers_inst_test_registers_gen_true_test_registers_reg_map_comp_data_mux_test_registers_comp_un2_index_sat_1\, 
        C => \adress_c[7]\, D => \adress_c[6]\, Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/ssp_cfg_reg_comp/wr_select_1_2[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_2[21]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_0[21]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_5[21]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_7[21]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.6.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y3[21]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_wmux_4[5]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y0_1[5]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[2]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_12[5]\, 
        D => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_14[5]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.0.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_20_1_0_y5[5]\);
    
    
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_0_cy\ : 
        ARI1_CC
      generic map(INIT => x"4EE00")

      port map(A => ADLIB_VCC, B => 
        \can_top_level_inst/prescaler_inst/h_sync_valid\, C => 
        \can_top_level_inst/prescaler_inst/is_tseg2\, D => 
        ADLIB_GND, FCI => ADLIB_VCC, S => OPEN, Y => OPEN, FCO
         => 
        \can_top_level_inst/prescaler_inst/bit_segment_meter_dbt_inst/un1_basic_segm_length_0_cry_0_cy_Z\, 
        CC => NET_CC_CONFIG2696, P => NET_CC_CONFIG2693, Y3 => 
        NET_CC_CONFIG2694, Y3A => NET_CC_CONFIG2695);
    
    \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf[0]\ : 
        SLE
      port map(D => \can_top_level_inst/tran_word[0]\, CLK => 
        \I_1/U0_RGB1_RGB4_rgb_net_1\, EN => 
        \can_top_level_inst/tx_arbitrator_inst/buffer_md_w\, ALn
         => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB4_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/tx_arbitrator_inst/tran_dlc_dbl_buf_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8[26]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_19[26]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_3[26]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.5.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_8_Z[26]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_11[25]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[25]\, 
        CLK => \I_1/U0_RGB1_RGB2_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB2_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.1.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_11[25]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_wmux[6]\ : 
        CFG4A
      generic map(INIT => x"ba98")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[5]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_5_Z[6]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_69_Z[6]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_81_1_0_y0[6]\);
    
    
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0[0]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_0_y0[0]\, 
        B => \can_top_level_inst/txtb_index_muxed[1]\, C => 
        \can_top_level_inst/txtb_port_b_data_3[0]\, D => 
        \can_top_level_inst/txtb_port_b_data_7[0]\, Y => 
        \can_top_level_inst/tx_arbitrator_inst/tran_word_m_3_1_0_wmux_0_Y[0]\);
    
    
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_0_2[0]\ : 
        CFG4
      generic map(INIT => x"0001")

      port map(A => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/curr_state_Z[26]\, 
        B => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[37]\, 
        C => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/curr_state[31]\, 
        D => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/err_pos_2_Z[4]\, 
        Y => 
        \can_top_level_inst/can_core_inst/protocol_control_inst/protocol_control_fsm_inst/ctrl_ctr_pload_val_cnst_i_a4_0_2_Z[0]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_17[6]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[6]\, 
        CLK => \I_1/U0_RGB1_RGB6_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB6_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.3.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[6]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7[10]\ : 
        CFG3
      generic map(INIT => x"AC")

      port map(A => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_17[10]\, 
        B => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_1[10]\, 
        C => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_b_address_i_Z[4]\, 
        Y => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/int_read_data_7_3[10]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_9[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9_1_sqmuxa_1\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.7.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_9[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80[17]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[17]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_80_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_80_Z[17]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74[8]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[8]\, 
        CLK => \I_1/U0_RGB1_RGB3_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_74_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB3_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_74_Z[8]\);
    
    
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_rst_true_gen.ram_write_process.ram_memory_8[27]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/port_a_data_in_i_Z[27]\, 
        CLK => \I_1/U0_RGB1_RGB0_rgb_net_1\, EN => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8_1_sqmuxa_2\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB0_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/txt_buf_comp_gen.4.txt_buffer_inst/txt_buffer_ram_inst/txt_buf_ram_inst/ram_memory_8[27]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4[31]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[31]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_4_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_4_Z[31]\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0[31]\ : 
        CFG4A
      generic map(INIT => x"ea62")

      port map(A => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_y0[31]\, 
        B => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_b_address_i_Z[6]\, 
        C => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_83_Z[31]\, 
        D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_115_Z[31]\, 
        Y => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/int_read_data_100_1_0_wmux_0_Y[31]\);
    
    
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/sel_data_23_28[22]\ : 
        CFG4
      generic map(INIT => x"AC00")

      port map(A => \can_top_level_inst/stat_bus[295]\, B => 
        \can_top_level_inst/drv_bus[103]\, C => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[4]\, 
        D => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/un7_index_sat_i[3]\, 
        Y => 
        \can_top_level_inst/memory_registers_inst/control_registers_reg_map_comp/data_mux_control_registers_comp/N_1000\);
    
    
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26[30]\ : 
        SLE
      port map(D => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/port_a_data_in_i_Z[30]\, 
        CLK => \I_1/U0_RGB1_RGB5_rgb_net_1\, EN => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_memory_26_1_sqmuxa\, 
        ALn => 
        \can_top_level_inst/memory_registers_inst/global_rst_rst_reg_inst/rx_shift_res_reg_inst/reg_q_rep_RNIGSV3/U0_RGB1_RGB5_rgb_net_1\, 
        ADn => ADLIB_VCC, SLn => ADLIB_VCC, SD => ADLIB_GND, LAT
         => ADLIB_GND, Q => 
        \can_top_level_inst/rx_buffer_inst/rx_buffer_ram_inst/rx_buf_RAM_inst/ram_rst_true_gen.ram_write_process.ram_memory_26_Z[30]\);
    
    GND_power_inst1 : GND
      port map( Y => GND_power_net1);

    VCC_power_inst1 : VCC
      port map( Y => VCC_power_net1);


end DEF_ARCH; 
